//	UHandlerBehavior.cp
//	Copyright  1992-1996 Nick Nallick.  All rights reserved.
//
//	Changes:
//
//		Use permanent memory.
//			Add MacApp 3.3 compatibility.
//			Add MacApp 3.1 compatibility.
//			Remove the HScratch class from the header to make it more opaque.
//			Substitute HLock(fHandlerData) for Lock() member.
//			Fix bug reading multiple version 2 handlers by padding handlers to an even word.
//			Add version 2 to resource file format to facilitate Rez and DeRez.
//			Save dynamic data in a handle (fHandlerData) to be compatible with MacApp 3.1.
//			Fix bug when using Window (by ID) target mode (thanks Viktor).
//		 Add Depends On & Doesn't Depend On actions.
//		 Add Add Dependent & Remove Dependent actions.
//		 Check with AppleEvent manager before posing a modal dialog.
//		 Fix bug that occured when the handler behavior was deleted before the
//		  initialize command has had a chance to execute by defering the delete.
//

#include "UHandlerBehavior.h"

#pragma segment HandlerBehavior

const short		kVersionNum =	2;

typedef enum
{
	kAppleCommandMask	= 0x1,
	kCommandKeyMask		= 0x2,
	kEventMask				= 0x4,
	kIdleMask					= 0x8,
	kInitializeMask		= 0x10,
	kKeyDownMask			= 0x20,
	kKeyUpMask				= 0x40,
	kMenuCommandMask	= 0x80,
	kMouseDownMask		= 0x100,
	kMouseUpMask			= 0x200,
	kSetupMenuMask		= 0x400,
	kUpdateMask				= 0x800,
	kTriedToFreeMask	= 0x8000
} messageMask;


////////////////////////////////// HScratch //////////////////////////////////
//
//	A private data class used while parsing messages.
//

struct HScratch
{
	HScratch(TBehavior* theBehavior);

	TBehavior*	behavior;
	long				dataOffset;
	BOOLEAN			behaviorFreed;
};

HScratch::HScratch(TBehavior* theBehavior)
{
	behavior = theBehavior;
	dataOffset = 0;
	behaviorFreed = FALSE;
}


///////////////////////////// THandlerInitMsgCmd /////////////////////////////
//
//	A command to deal with initialize messages to handlers.
//	This is done with a command to allow everything a chance to stabilize
//	before we start sending messages around the system.
//

class THandlerInitMsgCmd : public TCommand
{
	public:
		VIRTUAL void				DoIt();

#if qMacAppVersion <= 33
						void				IHandlerInitMsgCmd(THandlerBehavior* handler);
#else
												THandlerInitMsgCmd(THandlerBehavior* handler);
		virtual void				DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace);
#endif

	private:
		THandlerBehavior*		fHandler;
};


//	Overrides TCommand.
//	Handle the initialize message for the handler.
//
MACAPP_METHOD
void THandlerInitMsgCmd::DoIt()
{
#if qMacAppVersion <= 33
	fHandler->fHandlers &= ~kInitializeMask;		// clear the init bit so it can be freed
	if (fHandler->fHandlers & kTriedToFreeMask)	// somebody tried to free it
		fHandler->Free();
	else
#else
#endif
	{
		if (fHandler && fHandler->IsEnabled())
		{
			// there's nothing to forward to so just loop until we're done
			HScratch scratch(fHandler);
			while (fHandler && fHandler->handleMessage(kInitializeMsg, 0, &scratch))
				;
		}
	}
}

#if qMacAppVersion <= 33

//	Initialize the command.
//
void THandlerInitMsgCmd::IHandlerInitMsgCmd(THandlerBehavior* handler)
{
	ICommand(0, NULL, FALSE, FALSE, NULL);
	fHandler = handler;
}

#else

//	Initialize the command.
//
THandlerInitMsgCmd::THandlerInitMsgCmd(THandlerBehavior* handler) :
	TCommand(0, NULL, false, false, NULL),
	fHandler(handler)
{
	AddDependent(handler);	// we want to be notified when this object is deleted
}

#if qMacAppVersion > 33

void THandlerInitMsgCmd::DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace)
{
	if (theChange == mClosed && changedObject == fHandler)
	{
		fHandler = NULL;
	}
	else
		TCommand::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}





#endif

#endif

////////////////////////////// THandlerBehavior //////////////////////////////
//
//	A general purpose behavior to handle MacApp messages in user defined ways.
//

#if qMacAppVersion > 30
MA_DEFINE_CLASS_M1(THandlerBehavior, TBehavior);
#endif

typedef struct
{
	char			message,
						action,
						target,
						anyMessage;
	long			messageParam,
						actionParam,
						targetParam1,
						targetParam2;
	// comment string (pascal style) goes here
} handlerData;


THandlerBehavior::THandlerBehavior()
#if qMacAppVersion > 33
	: TBehavior(kHandlerBehaviorID)
#endif
{
#ifndef HANDLE_BASED_OBJECTS
	fHandlerData = NULL;
	fHandlers = 0;
	fDataSize = 0;
	fFreeFlagPtr = NULL;
#endif
}


#if qMacAppVersion <= 33

//	Initialize a behavior created manually.
//
MACAPP_METHOD
void THandlerBehavior::IHandlerBehavior()
{
	IBehavior(kHandlerBehaviorID);
}

//	Overrides TObject.
//	Update the free flag in case we're processing a message.
//
MACAPP_METHOD
void THandlerBehavior::Free()
{
	if (fHandlers & kInitializeMask)		// the THandlerInitMsgCmd is still around so
		fHandlers |= kTriedToFreeMask;		// we'll let it do the freeing
	else
	{
		if (fFreeFlagPtr)
			*fFreeFlagPtr = TRUE;
	
		DisposeIfHandle(fHandlerData);

		TBehavior::Free();
	}
}

#else

THandlerBehavior::~THandlerBehavior()
{

	if (fHandlers & kInitializeMask)		// the THandlerInitMsgCmd is still around so
	{
		fHandlers |= kTriedToFreeMask;		// we'll let it do the freeing
		Changed(mClosed, this);					// tell it we're going away.
	}
	//else
	{
		if (fFreeFlagPtr)
			*fFreeFlagPtr = TRUE;
	
		if (fHandlerData)
			DisposeHandle(fHandlerData);
		fHandlerData = NULL;
	}

}

#endif


#ifdef HANDLE_BASED_OBJECTS

//	Overrides TObject.
//	Initialize the behavior's private fields.
//
MACAPP_METHOD
void THandlerBehavior::Initialize()
{
	fHandlerData = NULL;
	fHandlers = 0;
	fDataSize = 0;
	fFreeFlagPtr = NULL;

	TBehavior::Initialize();
}

#endif


#if qMacAppVersion == 30 || qMacAppVersion == 31

//	Overrides TBehavior.
//	Handle AppleCommand messages.
//
MACAPP_METHOD
void THandlerBehavior::DoAppleCommand(CommandNumber aCommandNumber, const AppleEvent& message, const AppleEvent& reply)
{
	if (fHandlers & kAppleCommandMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while (target = handleMessage(kAppleCommandMsg, aCommandNumber, &scratch))
			target->HandleAppleCommand(aCommandNumber, message, reply);
	}
	else
		TBehavior::DoAppleCommand(aCommandNumber, message, reply);
}

#else

//	Overrides TBehavior.
//	Handle AppleCommand messages.
//
MACAPP_METHOD
BOOLEAN THandlerBehavior::DoScriptCommand(CommandNumber aCommandNumber, TAppleEvent* message, TAppleEvent* reply)
{
	if (fHandlers & kAppleCommandMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kAppleCommandMsg, aCommandNumber, &scratch)) != 0)
		{
			TBehavior* targetBehavior = target->GetFirstEnabledBehavior();
			if (targetBehavior)
				targetBehavior->DoScriptCommand(aCommandNumber, message, reply);
		}
		return TRUE;
	}
	else
		return TBehavior::DoScriptCommand(aCommandNumber, message, reply);
}

#endif


//	Overrides TBehavior.
//	Handle Update messages.
//
MACAPP_METHOD
#if qMacAppVersion <= 33
void THandlerBehavior::DoBehaviorUpdate(ChangeID theChange, TObject* changedObject, TObject* changedBy, TDependencySpace* dependencySpace)
#else
void THandlerBehavior::DoUpdate(ChangeID_AC theChange, MDependable_AC* changedObject, void* changeData, CDependencySpace_AC* dependencySpace)
#endif
{
	if (fHandlers & kUpdateMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kUpdateMsg, theChange, &scratch)) != 0)
			target->DoUpdate(theChange, changedObject, changeData, dependencySpace);
	}
	else
		MDependable_AC::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}


//	Overrides TBehavior.
//	Handle CommandKey messages.
//
static EventRecord TToolboxEvent_GetEventRecord(TToolboxEvent* event)
{
#if qMacAppVersion > 33
	return event->GetEventRecord(1);
#else
	return event->fEventRecord;
#endif
}

//	Overrides TBehavior.
//	Handle CommandKey messages.
//
MACAPP_METHOD
void THandlerBehavior::DoCommandKeyEvent(TToolboxEvent* event)
{
	if ((fHandlers & kCommandKeyMask) && !event->IsAutoKeyEvent())
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kCommandKeyMsg, TToolboxEvent_GetEventRecord(event).message & charCodeMask, &scratch)) != 0)
			target->HandleCommandKey(event);
	}
	else
		TBehavior::DoCommandKeyEvent(event);
}


//	Overrides TBehavior.
//	Handle Event messages.
//
MACAPP_METHOD
void THandlerBehavior::DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event)
{
	if (fHandlers & kEventMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kEventMsg, eventNumber, &scratch)) != 0)
			target->HandleEvent(eventNumber, source, event);
	}
	else
		TBehavior::DoEvent(eventNumber, source, event);
}


//	Overrides TBehavior.
//	Handle Idle messages.
//
MACAPP_METHOD
BOOLEAN THandlerBehavior::DoIdle(IdlePhase phase)
{
	if (fHandlers & kIdleMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kIdleMsg, phase, &scratch)) != 0)
			target->HandleIdle(phase);
		return scratch.behaviorFreed;
	}
	else
		return TBehavior::DoIdle(phase);
}

//	Overrides TBehavior.
//	Handle KeyDown messages.
//
MACAPP_METHOD
void THandlerBehavior::DoKeyEvent(TToolboxEvent* event)
{
	if (fHandlers & kKeyDownMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kKeyDownMsg, TToolboxEvent_GetCharacter(event), &scratch)) != 0)
			target->HandleKeyCommand(event);
	}
	else
		TBehavior::DoKeyEvent(event);
}


//	Overrides TBehavior.
//	Handle KeyUp messages.
//
MACAPP_METHOD
void THandlerBehavior::DoKeyUp(TToolboxEvent* event)
{
	if (fHandlers & kKeyUpMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kKeyUpMsg, TToolboxEvent_GetCharacter(event), &scratch)) != 0)
			target->HandleKeyUp(event);
	}
	else
		TBehavior::DoKeyUp(event);
}


//	Overrides TBehavior.
//	Handle MenuCommand messages.
//
MACAPP_METHOD
void THandlerBehavior::DoMenuCommand(CommandNumber aCommandNumber)
{
	if (fHandlers & kMenuCommandMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kMenuCommandMsg, aCommandNumber, &scratch)) != 0)
			target->HandleMenuCommand(aCommandNumber);
	}
	else
		TBehavior::DoMenuCommand(aCommandNumber);
}


//	Overrides TBehavior.
//	Handle MouseDown messages.
//
MACAPP_METHOD
BOOLEAN THandlerBehavior::DoMouseCommand(CVIEWPOINT& theMouse, TToolboxEvent* event, CPOINT hysteresis)
{
	if (fHandlers & kMouseDownMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kMouseDownMsg, 0, &scratch)) != 0)
			((TView*) target)->HandleMouseDown(theMouse, event, hysteresis);
		return TRUE;
	}
	else
		return TBehavior::DoMouseCommand(theMouse, event, hysteresis);
}


//	Overrides TBehavior.
//	Handle MouseUp messages.
//
MACAPP_METHOD
BOOLEAN THandlerBehavior::DoMouseUp(CVIEWPOINT& theMouse, TToolboxEvent* event, CPOINT hysteresis)
{
	if (fHandlers & kMouseUpMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kMouseUpMsg, 0, &scratch)) != 0)
			((TView*) target)->HandleMouseUp(theMouse, event, hysteresis);
		return TRUE;
	}
	else
		return TBehavior::DoMouseUp(theMouse, event, hysteresis);
}


//	Overrides TBehavior.
//	Handle SetupMenu messages.
//
MACAPP_METHOD
void THandlerBehavior::DoSetupMenus()
{
	if (fHandlers & kSetupMenuMask)
	{
		TEventHandler* target;
		HScratch scratch(this);
		while ((target = handleMessage(kSetupMenuMsg, 0, &scratch)) != 0)
			target->HandleSetupMenus();
	}
	else
		TBehavior::DoSetupMenus();
}


//	Handle user defined messages.
//	This must be overridden to handle custom messages.
//
MACAPP_METHOD
void THandlerBehavior::DoUserAction(HandlerMessage, long, TEventHandler*)
{
}


//	Overrides TObject.
//	Return the standard signature.
//	This way if the user overrides the class, we'll still know what it is.
//
MACAPP_METHOD
IDType THandlerBehavior::GetStandardSignature() CONST
{
	return kHandlerBehaviorID;
}


//	Overrides TObject.
//	Read the behavior data.
//
MACAPP_METHOD
void THandlerBehavior::ReadFrom(TStream* aStream)
{
	TBehavior::ReadFrom(aStream);

	long size = aStream->ReadLong();
	short version = aStream->ReadInteger();
	if (version <= kVersionNum)
	{
		if (version > 1)
			size = 0;
		if (fHandlerData)
			SetPermHandleSize(fHandlerData, size);
		else
		{
			Handle tmpHandle = NewPermHandle(size);
			fHandlerData = tmpHandle;
		}

		if (version == 1)
		{ // the file contains padded handlers
			fDataSize = size;
			HLock(fHandlerData);
			aStream->ReadBytes(*fHandlerData, size);
			HUnlock(fHandlerData);
		}
		else
		{ // the handlers aren't padded - we have to read them individually
			unsigned char buffer[sizeof(handlerData) + 256];
			short numHandlers = aStream->ReadInteger();
			fDataSize = 0;
			while (numHandlers-- > 0)
			{
				aStream->ReadBytes((Ptr) buffer, sizeof(handlerData) + 1);
				aStream->ReadBytes((Ptr) (buffer + sizeof(handlerData) + 1), buffer[sizeof(handlerData)]);
				size = sizeof(handlerData) + buffer[sizeof(handlerData)] + 1;
				if (size & 1)
					++size;		// round up to an even word boundary
				SetPermHandleSize(fHandlerData, fDataSize + size);
				BlockMove((Ptr) buffer, *fHandlerData + fDataSize, size);
				fDataSize += size;
			}
		}
		initHandlers();
		if (fHandlers & kInitializeMask)
		{
#if qMacAppVersion <= 33
			THandlerInitMsgCmd* initMsgCmd = new THandlerInitMsgCmd;
			initMsgCmd->IHandlerInitMsgCmd(this);
			gApplication->PostCommand(initMsgCmd);
#else
			gApplication->PostCommand(TH_new THandlerInitMsgCmd(this));
#endif
		}
	}
	else
	{
#if qMacAppVersion <= 33
		aStream->SetPosition(aStream->GetPosition() + size);
#else
		aStream->SetReadPosition(aStream->GetReadPosition() + size);
#endif
		if (qDebug)
			PROGRAMBREAK("In THandlerBehavior: unknown version");
	}
}


//	Overrides TObject.
//	Write the behavior data.
//
MACAPP_METHOD
void THandlerBehavior::WriteTo(TStream* aStream) CONST
{
	TBehavior::WriteTo(aStream);

	// count the number of handlers and the non-padded data size
	long strLength;
	long size = 0;
	short numHandlers = 0;
	long dataOffset = 0;
	if (fHandlerData)
	{
#if qMacAppVersion <= 33
		fDataSize = GetHandleSize(fHandlerData);
#else
		((THandlerBehavior*) this)->fDataSize = GetHandleSize(fHandlerData); // violates const-ness
#endif
		while (dataOffset < fDataSize)
		{
			dataOffset += sizeof(handlerData);
			strLength = (*(unsigned char**) fHandlerData)[dataOffset];
			dataOffset += strLength + 1;
			if (strLength % 2 == 0)
				++dataOffset;

			size += sizeof(handlerData) + strLength + 1;
			++numHandlers;
		}
	}

	// write the handler data to the stream
	aStream->WriteLong(size + sizeof(numHandlers));
	short version = kVersionNum;
	aStream->WriteInteger(version);
	aStream->WriteInteger(numHandlers);
	HLock(fHandlerData);

	dataOffset = 0;
	while (numHandlers-- > 0)
	{
		strLength = (*(unsigned char**) fHandlerData)[dataOffset + sizeof(handlerData)];
		aStream->WriteBytes(*fHandlerData + dataOffset, sizeof(handlerData) + strLength + 1);
		dataOffset += sizeof(handlerData) + strLength + 1;
		if (strLength % 2 == 0)
			++dataOffset;
	}

	HUnlock(fHandlerData);
}


//	Get the dynamic data handle.
//	This function is used by Ad Lib to set the handlers data.
//
//	RETURNS:	a valid handle
//
Handle THandlerBehavior::GetHandlerData()
{
	if (!fHandlerData)
	{
		Handle tmpHandle = NewPermHandle(0);
		fHandlerData = tmpHandle;
	}

	return fHandlerData;
}


///////////////////////// THandlerBehavior (private) /////////////////////////

typedef enum
{
	kNullAction,
	kCloseAction,
	kCloseAndFreeAction,
	kEnableMenuAction,
	kForwardAction,
	kOpenWindowAction,
	kPoseModalAction,
	kScriptAction,
	kEventAction,
	kMenuCommandAction,
	kUpdateAction,
	kSetUserValueAction,
	kUserDefinedAction,
	kAddDependentAction,
	kRmvDependentAction,
	kDependsOnAction,
	kNotDependsOnAction
} actionType;

typedef enum
{
	kNullTarget,
	kApplicationTarget,
	kDocumentTarget,
	kSelfTarget,
	kSubviewTarget,
	kSuperviewTarget,
	kTargetChainTarget,
	kViewTarget,
	kViewWindowTarget,
	kWindowTarget,
	kWindowIDTarget
} targetType;


//	Process all messages matching the parameters.
//	Searches a behaviors dynamic data for messages that match the parameters.
//	When it finds a matching message, it finds the target and performs the action.
//	A special case action is kForwardAction which is returned to the calling method to perform.
//	This function is static in case the behavior is freed while processing a message.
//
//	message:		the message type
//	msgParam:		the message parameter
//	scratch:		scratch data to maintain state for reentrant calls (must be HScratch*)
//
//	RETURNS:		a target to forward to or NULL when done
//
TEventHandler* THandlerBehavior::handleMessage(HandlerMessage message, long msgParam, void* scratch)
{
	THandlerBehavior* behavior = (THandlerBehavior*) ((HScratch*) scratch)->behavior;
	if (((HScratch*) scratch)->dataOffset == 0)
	{
		// this is the first time through
		behavior->fFreeFlagPtr = &((HScratch*) scratch)->behaviorFreed;
		behavior->SetEnabled(FALSE);	// to avoid recursion
	}
	else
		if (((HScratch*) scratch)->behaviorFreed)
			return NULL;		// the behavior was freed while forwarding the message

	handlerData* data = (handlerData*) (*(behavior->fHandlerData) + ((HScratch*) scratch)->dataOffset);
	for (;;)
	{
		// check for end of data
		if (((HScratch*) scratch)->dataOffset >= behavior->fDataSize)
		{
			behavior->fFreeFlagPtr = NULL;
			behavior->SetEnabled(TRUE);
			return NULL;
		}

		// find the beginning of the next handler
		long handlerSize = sizeof(handlerData) + *((unsigned char*) (data + 1)) + 1;
		if (handlerSize & 1)
			++handlerSize;		// round up to an even word boundary
		((HScratch*) scratch)->dataOffset += handlerSize;

		// check to current handler for a match
		if (data->message == message && (data->anyMessage || data->messageParam == msgParam))
		{
			// found a match - find the target
			TEventHandler* target = behavior->getTarget(data);
			if (target)
			{
				// found a target - now do the action
				if (data->action == kForwardAction)
					return target;		// special handling for forward message action
				behavior->performAction(target, data);
				if (((HScratch*) scratch)->behaviorFreed)
					return NULL;		// the behavior was freed performing the action
				data = (handlerData*) (*(behavior->fHandlerData) + ((HScratch*) scratch)->dataOffset);		// the object may have moved
				continue;		// do the next handler
			}
		}

		// no match found - go to the next handler
		data = (handlerData*) (((char*) data) + handlerSize);
	}
}

inline TWindow* WMgrToWindow(WindowPtr wMgrWindow)
{
#if qMacAppVersion <= 33
	return gApplication->WMgrToWindow(wMgrWindow);
#else
	return TWindow::WMgrToWindow(wMgrWindow);
#endif
}

//	Get the target object.
//
//	data:			the handler data
//
//	RETURNS:	the target (or NULL if not found)
//
TEventHandler* THandlerBehavior::getTarget(void* data)
{
	TEventHandler* target = NULL;
	switch (((handlerData*) data)->target)
	{
		case kApplicationTarget:		target = gApplication;
																break;
		case kDocumentTarget:				if (fOwner)
																{
#if qMacAppVersion <= 33
																	target = ((TView*) fOwner)->fDocument;
#else
																	target = ((TView*) fOwner)->GetDocument();
#endif
																	if (!target)
																	{ // try the window
																		TWindow* window = ((TView*) fOwner)->GetWindow();
																		if (window)
#if qMacAppVersion <= 33
																		target = window->fDocument;
#else
																		target = window->GetDocument();
#endif
																	}
																}
																break;
		case kSelfTarget:						target = fOwner;
																break;
		case kSubviewTarget:				if (fOwner)
																	target = ((TView*) fOwner)->FindSubView(((handlerData*) data)->targetParam1);
																break;
		case kSuperviewTarget:			if (fOwner)
#if qMacAppVersion <= 33
																	target = ((TView*) fOwner)->fSuperView;
#else
																	target = ((TView*) fOwner)->GetSuperView();
#endif
																break;
		case kTargetChainTarget:		target = gApplication->GetTarget();
																break;
		case kViewTarget:						if (fOwner)
																{
																	TWindow* window = ((TView*) fOwner)->GetWindow();
																	if (window)
																		target = window->FindSubView(((handlerData*) data)->targetParam1);
																}
																break;
		case kViewWindowTarget:			{
																	CWMgrIterator iter;
																	for (WindowPtr wndwPtr = iter.FirstWMgrWindow(); iter.ITERCURRENT(); wndwPtr = iter.NextWMgrWindow())
																	{ // find a "shown" window with the right ID
																		TWindow* window = WMgrToWindow(wndwPtr);
																		if (window && window->IsShown() && window->GetIdentifier() == ((handlerData*) data)->targetParam2)
																		{
																			target = window->FindSubView(((handlerData*) data)->targetParam1);
																			break;
																		}
																	}
																}
																break;
		case kWindowTarget:					if (fOwner)
																	target = ((TView*) fOwner)->GetWindow();
																break;
		case kWindowIDTarget:				{
																	CWMgrIterator iter;
																	for (WindowPtr wndwPtr = iter.FirstWMgrWindow(); iter.ITERCURRENT(); wndwPtr = iter.NextWMgrWindow())
																	{ // find a "shown" window with the right ID
																		TWindow* window = WMgrToWindow(wndwPtr);
																		if (window && window->IsShown() && window->GetIdentifier() == ((handlerData*) data)->targetParam1)
																		{
																			target = (TEventHandler*) window;
																			break;
																		}
																	}
																}
																break;
	}
	return target;
}


//	Perform the desired action on an object.
//
//	target:		the target object
//	data:			the handler data
//
void THandlerBehavior::performAction(TEventHandler* target, void* data)
{
	switch (((handlerData*) data)->action)
	{

		case kCloseAction:				

#ifdef HANDLE_BASED_OBJECTS
															if (qDebug && !IsMemberClassID(target, GetClassIDFromSignature(kStdView)))
#else
															if (qDebug && !MA_MEMBER(target, TView))
#endif
																PROGRAMBREAK("In THandlerBehavior: attempt to close non TView object");
															((TView*) target)->Close();
															break;

		case kCloseAndFreeAction:

#ifdef HANDLE_BASED_OBJECTS
															if (IsMemberClassID(target, GetClassIDFromSignature(kStdWindow)))
																((TWindow*) target)->CloseAndFree();
															else
																if (qDebug && !IsMemberClassID(target, GetClassIDFromName("TDocument")))
#else
															if (MA_MEMBER(target, TWindow))
																((TWindow*) target)->CloseAndFree();
															else
																if (qDebug && MA_MEMBER(target, TDocument))
#endif
																	PROGRAMBREAK("In THandlerBehavior: CloseAndFree needs a window or a document");
																else
																	((TDocument*) target)->CloseAndFree();
															break;

		case kEnableMenuAction:		Enable(((handlerData*) data)->actionParam, TRUE);
															break;

		case kOpenWindowAction:		target = (TEventHandler*) gViewServer->NewTemplateWindow((short) ((handlerData*) data)->actionParam, NULL);
															THROWIFNULL(target);
															((TWindow*) target)->Open();
															break;

		case kPoseModalAction:		THROWIFOSERR(MAInteractWithUser(kNoTimeOut, gNotificationPtr, gMacAppAppleEventIdleProc));
															target = (TEventHandler*) gViewServer->NewTemplateWindow((short) ((handlerData*) data)->actionParam, NULL);
															THROWIFNULL(target);
															((TWindow*) target)->Open();
															((TWindow*) target)->PoseModally();
															((TWindow*) target)->CloseAndFree();
															break;

		case kEventAction:				target->HandleEvent(((handlerData*) data)->actionParam, fOwner, NULL);
															break;

		case kMenuCommandAction:	target->HandleMenuCommand(((handlerData*) data)->actionParam);
															break;

		case kUpdateAction:				target->Changed(((handlerData*) data)->actionParam, fOwner);
															break;

		case kSetUserValueAction:

#ifdef HANDLE_BASED_OBJECTS
															if (qDebug && !IsMemberClassID(target, GetClassIDFromSignature(kStdView)))
#else
															if (qDebug && MA_MEMBER(target, TView))
#endif
																PROGRAMBREAK("In THandlerBehavior: attempt to set user value of non TView object");
#if qMacAppVersion > 33
															((TView*) target)->SetUserArea(((handlerData*) data)->actionParam);
#else
															((TView*) target)->fUserArea = ((handlerData*) data)->actionParam;
#endif
															break;

		case kUserDefinedAction:	DoUserAction((HandlerMessage) ((handlerData*) data)->message, ((handlerData*) data)->actionParam, target);
															break;

		case kAddDependentAction:	if (fOwner)
																fOwner->AddDependent(target);
															break;

		case kRmvDependentAction:	if (fOwner)
																fOwner->RemoveDependent(target);
															break;

		case kDependsOnAction:		if (fOwner)
																target->AddDependent(fOwner);
															break;

		case kNotDependsOnAction:	if (fOwner)
																target->RemoveDependent(fOwner);
															break;
	}
}


//	Go through the handler data and set up the fHandlers field.
//
void THandlerBehavior::initHandlers()
{
	handlerData* data = (handlerData*) *fHandlerData;
	long dataOffset = 0;
	while (dataOffset < fDataSize)
	{
		fHandlers |= (1 << (data->message - 1));
		long handlerSize = sizeof(handlerData) + *((unsigned char*) (data + 1)) + 1;
		if (handlerSize & 1)
			++handlerSize;		// round up to an even word boundary
		data = (handlerData*) (((char*) data) + handlerSize);
		dataOffset += handlerSize;
	}
}


//////////////////////////////////////////////////////////////////////////////

#pragma segment AInit

void RegisterTHandlerBehavior()
{
#if qMacAppVersion == 30
	new THandlerBehavior;
#else
	MA_REGISTER_SIGNATURE(THandlerBehavior, kHandlerBehaviorID);
#endif
}