// UGridView.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UGRIDVIEW__
#define __UGRIDVIEW__

// ACS

#ifndef __CIterator__
#include "CIterator_AC.h"
#endif

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

// Toolbox

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TDocument;
class TDragItem;
class TToolboxEvent;

//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------
typedef CViewPoint GridCell;
typedef CViewRect GridRect;
typedef ViewCoordinate GridCoordinate;

enum GridViewPart { badChoice, inCell, inRow, inColumn, inVertex };


//----------------------------------------------------------------------------------------
// bool for Adornment
//----------------------------------------------------------------------------------------

enum
{
	kAdorn = true,
	kDontAdorn = false
};

//----------------------------------------------------------------------------------------
// bools for SetSelection
//----------------------------------------------------------------------------------------

enum
{
	kExtend = true,
	kDontExtend = false,

	kHighlight = true,
	kDontHighlight = false,

	kSelect = true,
	kDeSelect = false
};

//----------------------------------------------------------------------------------------
// Boundaries of the cell system
//----------------------------------------------------------------------------------------

const GridCoordinate kMinCell = 1;
const GridCoordinate kMaxCell = SHRT_MAX;

//----------------------------------------------------------------------------------------
// TRunArray: The run array class is used to maintain "runs" of items with the same value.
// Entries in the array are values for a given item, where the items are indexed from one.
// fNoOfItems indicates the number of items (and values) in the array. The values are
// maintained in "chunks" which lump together consecutive items with the same value. A
// variable length array, fChunks, is used to store the chunks. It is indexed from zero.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// RunArrayChunk
//----------------------------------------------------------------------------------------
struct RunArrayChunk
{
	long count;									// Number of consecutive items with this value
	long value;									// The value represented by this chunk
};

typedef RunArrayChunk *ChunkArrayPtr;

//----------------------------------------------------------------------------------------
// TRunArray
//----------------------------------------------------------------------------------------
class EXPORT_MA TRunArray : public CDynamicArray_AC
{
public:
	TRunArray(ArrayIndex_AC initialSize = 0);
		// Constructor
		
	virtual ~TRunArray();
		// Frees the fChunks field which is a handle to the chunks, before calling
        // Inherited::Free.

	virtual void* Clone() const;
		// Clone

	void InsertItems(ArrayIndex_AC firstItem,
									ArrayIndex_AC noOfItems,
									long value);
		// Inserts the indicated items into the run array, all of which have the given
        // value.

	void DeleteItems(ArrayIndex_AC firstItem,
									ArrayIndex_AC noOfItems);
		// Deletes the indicated items from the run array.

	bool FindChunk(ArrayIndex_AC inItem,
							 ArrayIndex_AC& outChunk,
							 ArrayIndex_AC& outIndexInChunk,
							 long& outTheTotal) const;
		// Returns information about a given item in the run array. chunk indicates the
        // chunk in which the item is located, where zero is the first chunk in the run
        // array. indexInChunk is the location of the item withinin the chunk. Note that
        // the chunk indexes are one-based--the first item in the chunk is index 1.
        // theTotal is the sum of values up to, but not including, the chunk in which the
        // given item is located. FindChunk returns false if the given item is outside the
        // range of items in the run-array (i.e. item < 1 or item > fNoOfItems), or the
        // given item is not represented in a chunk (i.e. a bug).
		//
	  	// Example: Item	Value	Chunk	indexInChunk	theTotal
	  	// 			1		05		0			1			000
	  	// 			2		05		0			2			000
	  	// 			3		20		1			1			010
	  	// 			4		35		2			1			030
	  	// 			5		35		2			2			030
	  	// 			6		10		3			1			100

	ArrayIndex_AC FindItem(long theTotal) const;
		// Returns the item number for which the sum of the values exceeds theTotal, or
        // zero if theTotal is outside the sum of values represented by the run array.

	long GetValue(ArrayIndex_AC item) const;
		// Returns the value for the given item.

	long SumValues(ArrayIndex_AC firstItem, ArrayIndex_AC noOfItems) const;
		// Returns the sum of the values over the given items.

	inline RunArrayChunk& ChunkAt(ArrayIndex_AC chunkIndex) const { return *(RunArrayChunk*)(*this)[chunkIndex]; }
		// returns a reference to the indexed chunk

	inline long GetTotal() const { return fTotal; }

	inline void SetTotal(long newTotal) { fTotal = newTotal; }

	inline ArrayIndex_AC GetNoOfChunks() const { return GetSize(); }
	// left in for compatibility. However, If you were calling this or using
	// fNoOfChunks just use GetSize() instead.
	
	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	long fTotal;								// The sum of the values in the run array

	long fLastTotal;							// cache the last total calculated
	long fLastItem;								// cache the last item found
	long fLastChunk;							// cache the last chunk found
	ArrayIndex_AC fLastIndex;						// cache the last index used

	ArrayIndex_AC fNoOfItems;						// Number of items (values) in this
                                                // run-array
};


//----------------------------------------------------------------------------------------
// TGridViewDesignator: represents an arbitrary selection of gridview cells.
//----------------------------------------------------------------------------------------
class EXPORT_MA TGridViewDesignator : public TDesignator
{
public:
	TGridViewDesignator();
	// Creates an empty designation

	TGridViewDesignator(const GridRect& fromRect);
	// Creates a designation whose contents is the rectangle

	TGridViewDesignator(const TGridViewDesignator& inGridViewDesignator);
	// Copy constructor

	virtual ~TGridViewDesignator();
	// Destructor
	
	virtual void* Clone() const;
		// Clone

	void Set();	
	// Set the designation's contents to be empty

	void Set(const TGridViewDesignator* fromSelection);
	// Set the designation's contents to the same as another designation

	void Set(const GridRect& fromRect);
	// Set the designation's contents to be a rectangle

	bool Contains(const GridCell& itsPoint) const;
	// Return true if the point is contained within the designation

	GridRect GetBounds() const;
	// Return the smallest enclosing rectangle that contains the designation
	
	void Append(const TGridViewDesignator* fromSelection);
	// Append fromSelection to the designation 

	void Intersect(const TGridViewDesignator* fromSelection);
	// Intersection the designation with fromSelection

	void Remove(const TGridViewDesignator* fromSelection);
	// Remove the intersection with fromSelection from the designation
	
	// Overrides
	virtual Size GetSize() const;
	// Returns size of the designator's data
	
	virtual bool IsEmpty() const;
	// Returns true if designation is empty
	
	virtual ContainmentType IsContained(TDesignator* aDesignator);
	// Returns a value indicating whether this designator is contained by aDesignator

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFrom(CStream_AC* aStream);
	virtual void WriteTo(CStream_AC* aStream) const;

protected:
	void Reduce();
	// Call to reduce the list to simplest form

	CDynamicArray_AC fSelection;					// list of rectangles enclosing selected cells
};


//----------------------------------------------------------------------------------------
// TGridView: TGridView forms the base class for a building block that allows the creation
// of views that can contain and manage cells, similar to a spreadsheet.
//----------------------------------------------------------------------------------------

class EXPORT_MA TGridView : public TView
{
public:
	TGridView(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,	// Size determiners 
								GridCoordinate numOfRows = 0,		// Number of rows initially 
								GridCoordinate numOfCols = 0,		// Number of columns initially 
								ViewCoordinate rowHeight = 0,		// Height of initial rows 
								ViewCoordinate colWidth = 0,			// Height of initial columns 
								bool adornRows = false,			// Adornment for Rows? 
								bool adornCols = false,			// Adornment for Columns? 
								ViewCoordinate rowInset = 0,			// horizontal space between cells 
								ViewCoordinate colInset = 0,			// vertical space between cells 
								bool singleSelection = true);		// single cell selection? 
		// Initialize the gridview. If numOfRows or numOfCols is non-zero then the
        // gridview is initialized to the specified size using rowHeight and colWidth if
        // the sizedeterminer is sizeVariable. If either adorn is kDontAdorn, then that
        // adorn will not be called.

	TGridView(const TGridView& inGridView);
		// Copy Constructor

	virtual ~TGridView();
		// Free the TGridView object

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Inherited Methods(OVERRIDE)
	//------------------------------------------------------------------------------------

	virtual void DoMenuCommand(CommandNumber aCommandNumber); // Override
		// Handles the Select All menu command

	virtual CViewRect CalcMinFrame();
		// Sets the extent of the view.

	virtual void DoHighlightSelection(HLState fromHL, HLState toHL);
		// Do highlighting

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Handle mouse commands

	virtual void Draw(const CViewRect& area);
		// Calls DrawRangeOfCells and AdornRow/ Col as appropriate.


	//------------------------------------------------------------------------------------
	// UGridView Methods to OVERRIDE
	//------------------------------------------------------------------------------------

	virtual void AdornCol(GridCoordinate aCol, const CViewRect& area);
		// If fAdornCol is true, this method is called to draw the column adornments.

	virtual void AdornRow(GridCoordinate aRow, const CViewRect& area);
		// If fAdornRow is true, this method is called to draw the rowadornments.

	virtual bool CanSelectCell(const GridCell& aCell);
		// This method checks to see if a cell can be seleced. By default, this method
        // always returns true.

	virtual void HighlightCells(const TGridViewDesignator* theCells, HLState fromHL, HLState toHL);
		// Highlights the cells in theCells according to the given highlight state.

	virtual void DrawRangeOfCells(const GridCell& startCell,
										 const GridCell& stopCell,
										 const CViewRect& aRect);
		// This method calls DrawCell for each cell in need of re-drawing.

	virtual void DrawCell(const GridCell& aCell, const CViewRect& aRect);
		// This method draws each individual cell.IT MUST BE OVERRIDDEN!


	//------------------------------------------------------------------------------------
	// General UGridView Methods
	//------------------------------------------------------------------------------------

	CViewRect CellToVRect(const GridCell& aCell) const;
	inline void CellToVRect(const GridCell& aCell, CViewRect& theVRect) const
	{ theVRect = CellToVRect(aCell); }
		// Get the rectangle bounding a given cell. This includes the row and column
        // insets.

	CViewRect ColToVRect(GridCoordinate aCol, GridCoordinate numOfCols) const;
	inline void ColToVRect(GridCoordinate aCol, GridCoordinate numOfCols, CViewRect& theVRect) const
	{ theVRect = ColToVRect(aCol, numOfCols); }
		// Get the rectangle bounding the given columns. This includes the cells in the
        // row or column and the row or column insets.

	CViewRect RowToVRect(GridCoordinate aRow, GridCoordinate numOfRows) const;
	inline void RowToVRect(GridCoordinate aRow, GridCoordinate numOfRows, CViewRect& theVRect) const
	{ theVRect = RowToVRect(aRow, numOfRows); }
		// Get the rectangle bounding the given rows. This includes the cells in the row
        // or column and the row or column insets.

	void CellsToPixels(const TGridViewDesignator* theCells, RgnHandle thePixels);
		// Returns in thePixels a region that contains all of the cells in theCells

	virtual void DelColAt(GridCoordinate aCol, GridCoordinate numOfCols);
		// Delete the specified columns, starting at 'aCol' with the number to be deleted
        // 'numOfCols'.This causes a redraw of only the necessary cells.

	virtual void DelRowAt(GridCoordinate aRow, GridCoordinate numOfRows);
		// Delete the specified rows, starting at 'aRow' with the number of to be deleted
        // 'numOfRows'.This causes a redraw of only the necessary cells.

	virtual void DelColFirst(GridCoordinate numOfCols);
		// Delete the specified number of columns from the beginning of the list.This
        // causes a redraw of only the necessary cells.

	virtual void DelRowFirst(GridCoordinate numOfRows);
		// Delete the specified number of rows from the beginning of the list.This causes
        // a redraw of only the necessary cells.

	virtual void DelColLast(GridCoordinate numOfCols);
		// Delete the specified number of columns from the last cell of the list.This
        // causes a redraw of only the necessary cells.

	virtual void DelRowLast(GridCoordinate numOfRows);
		// Delete the specified number of rows from the last cell of the list.This causes
        // a redraw of only the necessary cells.

	virtual GridCell FirstSelectedCell();
		// Returns the top left cell in the selection range, if any

	ViewCoordinate GetColWidth(GridCoordinate aCol) const;
		// Return the width for the specifiedcolumn.

	ViewCoordinate GetRowHeight(GridCoordinate aRow) const;
		// Return the height for the specified row .

	virtual GridViewPart IdentifyPoint(const CViewPoint& thePoint,
											  GridCell& aCell);
		// Return the GridViewPart (inCell, inColumn, inRow, inVertex) in which the
        // specified CViewPoint lies. The "vertex" is the area where a row and column meet.

	bool InsColBefore(GridCoordinate aCol, GridCoordinate numOfCols, ViewCoordinate aWidth);
		// Insert the specified columns before the column given.The widths of the columns
        // are all set to the specified aWidth.

	bool InsRowBefore(GridCoordinate aRow, GridCoordinate numOfRows, ViewCoordinate aHeight);
		// Insert the specified rows before the row given.The heights of the rows are all
        // set to the specified aHeight.

	virtual bool InsColFirst(GridCoordinate numOfCols, ViewCoordinate aWidth);
		// Insert the specified number of columns at the beginning of the list.The widths
        // of the columns are all set to the specified aWidth.

	virtual bool InsRowFirst(GridCoordinate numOfRows, ViewCoordinate aHeight);
		// Insert the specified number of rows at the beginning of the list.The heights of
        // the rows are all set to the specified aHeight.

	virtual bool InsColLast(GridCoordinate numOfCols, ViewCoordinate aWidth);
		// Insert the specified number columns at the end of the list.The widths of the
        // columns are all set to the specified aWidth.

	virtual bool InsRowLast(GridCoordinate numOfRows, ViewCoordinate aHeight);
		// Insert the specified number of rows at the end of the list.The heights of the
        // rows are all set to the specified aHeight.

	virtual void InvalidateCell(const GridCell& aCell);
		// Cause a cell to be marked invalid (in need of re-drawing).

	virtual void InvalidateSelection();
		// Cause the rectangle bounding the selections to be marked invalid (in need of
        // re-drawing).

	virtual bool IsCellSelected(const GridCell& aCell);
		// Check if the specified cell is currently selected.

	virtual bool IsAnyCellSelected();
		// Returns true if any cell is currently selected.

	virtual GridCell LastSelectedCell();
		// Returns the bottom right cell in the selection range, if any

	virtual void ScrollSelectionIntoView(bool redraw);	// Override
		// Scroll the selection into view.

	virtual void SetColWidth(GridCoordinate aCol, GridCoordinate numOfCols, ViewCoordinate aWidth);
		// Set the width of the specified columns to that given.

	virtual void SetRowHeight(GridCoordinate aRow, GridCoordinate numOfRows, ViewCoordinate aHeight);
		// Set the height of the specified rows to that given.

	virtual void SelectCell(const GridCell& theCell,
								   bool extendSelection,
								   bool highlight,
								   bool select);
		// Set the current selection to the specified cell. Sets up a region and then
        // calls SetSelection.

	virtual void SetEmptySelection(bool highlight);
		// Set the current selection to be empty or nothing. If highlight is kHighlight
        // then the appropriate highlighting is performed.

	virtual void SetSelection(const TGridViewDesignator* cellsToSelect,
									 bool extendSelection,
									 bool highlight,
									 bool select);
		// Set the current selections to the specified region. If extend is kExtend then
        // cellsToSelect is "added" to that already selected, if highlight is kHighlight
        // then cellsToSelect is highlighted as well. IF select is kSelect then
        // cellsToSelect is selected, if kDeSelect then de-selected

	virtual void SetSelectionRect(const CViewRect& selectionRect,
										 bool extendSelection,
										 bool highlight,
										 bool select);
		// Set the current selections to the specified rectangle. and call SetSelection.

	virtual void SetSingleSelection(bool theSetting);
		// If true then only one item/ cell can be selected at a time

	GridCell VPointToCell(const CViewPoint& aPoint) const;
		// Determine the cell in which a given CViewPoint lies, or (0, 0) if the given CViewPoint
        // doesn't lie within any cell.

	GridCell VPointToLastCell(const CViewPoint& aPoint) const;
		// Returns the cell in which the given CViewPoint lies, or the last cell of the row/
        // column if the horizontal/ vertical coordinate lies beyond the last cell of the
        // row/ column.

	//------------------------------------------------------------------------------------
	// Accessors
	//------------------------------------------------------------------------------------

	inline bool GetSingleSelection() const { return fSingleSelection; }

	inline GridCoordinate GetNumOfRows() const { return fNumOfRows; }

	inline GridCoordinate GetNumOfCols() const { return fNumOfCols; }

	inline TGridViewDesignator* GetSelections() const { return &const_cast<TGridView*>(this)->fSelections; }

	inline TGridViewDesignator* GetHLRegion() const { return &const_cast<TGridView*>(this)->fHLRegion; }

	inline GridCoordinate GetRowInset() const { return fRowInset; }
	
	inline GridCoordinate GetColInset() const { return fColInset; }
	
	inline void SetRowInset(GridCoordinate newRowInset) { fRowInset = newRowInset; }
	
	inline void SetColInset(GridCoordinate newColInset) { fColInset = newColInset; }
	
	inline bool IsAdornRows() const { return fAdornRows; }
	
	inline bool IsAdornCols() const { return fAdornCols; }
	
	inline void SetAdornRows(bool adornRows) { fAdornRows = adornRows; }
	
	inline void SetAdornCols(bool adornCols) { fAdornCols = adornCols; }
	
	
#if qDrag
	//------------------------------------------------------------------------------------
	// Drag and Drop Methods
	//------------------------------------------------------------------------------------

	virtual RgnHandle DoMakeDragCursorRegion(); // Override
	
	virtual bool WillDrag(const CViewPoint& localMouse, const RgnHandle dragCursorRegion); // Override
	
	virtual void DoDragEnter(); // Override
	
	virtual void DoDragLeave(); // Override
	
	virtual void DoDragWithin(const CViewPoint& localMouse); // Override
	
	virtual void DrawCellCaret(const GridCell& targetCell);
	
	
#endif // qDrag
	
protected:
	virtual void AddStrip(RgnHandle thePixels,
					 			 VHSelect direction,
								 ViewCoordinate& startOfStrip,
								 ViewCoordinate endOfStrip,
								 CViewRect& stripRect,
								 GridCoordinate row,
								 GridCoordinate col,
								 CViewRect& pixels,
								 CViewRect& previousPixels,
								 CViewRect& prevStripRect);
	// Called by CellsToPixels to add a strip of cells to thePixels.

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	TRunArray fColWidths;						// bag containing col widths
	TRunArray fRowHeights;						// bag containing row heights

	TGridViewDesignator fSelections;			// Region of currently selected cells
	TGridViewDesignator fHLRegion;				// Region of cells to be highlighted. This
                                                // will be different from fSelections
                                                // while selection with the mouse is
                                                // taking place.

	GridCoordinate fNumOfRows;					// number of rows
	GridCoordinate fNumOfCols;					// number of columns
	ViewCoordinate fRowInset;						// Number of pixels between cell rows
	ViewCoordinate fColInset;						// Number of pixels between cell cols
	bool fAdornRows;							// Draw adornment for rows?
	bool fAdornCols;							// Draw adornment for columns?
	bool fSingleSelection;						// only one cell selected at a time?

// Drawing order
	bool fDrawRowsForward;
	bool fDrawColsForward;
	bool fDrawRowMajor;

#if qDrag
protected:
	GridCell		fLastTargetCell;
	unsigned long	fLastCaretTime;
	bool			fCaretIsShown;
#endif //qDrag
};


//----------------------------------------------------------------------------------------
// TTextGridView: A sub-class of TGridView that can handle the display of text in its
// cells.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTextGridView : public TGridView
{
public:
	//------------------------------------------------------------------------------------
	// Initialization and Free Methods
	//------------------------------------------------------------------------------------

	TTextGridView(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,	// Size determiners 
								GridCoordinate numOfRows = 0,		// Number of rows initially 
								GridCoordinate numOfCols = 0,		// Number of columns initially 
								ViewCoordinate rowHeight = 0,		// Height of initial rows 
								ViewCoordinate colWidth = 0,			// Height of initial columns 
								bool adornRows = false,			// Adornment for Rows? 
								bool adornCols = false,			// Adornment for Columns? 
								ViewCoordinate rowInset = 0,			// horizontal space between cells 
								ViewCoordinate colInset = 0,			// vertical space between cells 
								bool singleSelection = true,
								const CTextStyle_AC& itsTextStyle = gSystemStyle);
		// Initialize the TextGridView. If numOfRows or numOfCols is non-zero then the
        // gridview is initialized to the specified size. The row and height of the cells
        // is determine by the font size, style etc. The default size may be changed with
        // calls to the the SetRowHeight and SetColWidth methods. If either adorn is
        // kDontAdorn, then that adorn will not be called. The "Insets" allow space
        // between cells and may be used to draw row and column adornments.

	virtual ~TTextGridView();
		// Destructor
		
	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Methods To OVERRIDE
	//------------------------------------------------------------------------------------

	virtual CStr255_AC GetText(const GridCell&			) const;
		// This routine gets the text to be draw in a given cell. IT MUST BE OVERRIDDEN!

#if qDrag
	//------------------------------------------------------------------------------------
	// Drag and Drop Methods
	//------------------------------------------------------------------------------------
	
	virtual void DoAddDragContent();
	
	virtual void DoFulfillPromise(TDragItem* promisedItem);
	
#endif // qDrag
	//------------------------------------------------------------------------------------
	// General Methods
	//------------------------------------------------------------------------------------

	virtual void DrawCell(const GridCell& aCell, const CViewRect& aRect);
		// Calls GetText and then draws the text

	virtual bool Focus();
		// Calls Inherited::Focus and calls SetUpFont

	virtual void SetUpFont();
		// Sets up the font for this view.

	virtual void SetPen();
		// Sets the font characteristics of the current port. Called at the beginning of
        // the Draw method.

	//----------------------------------------------------------------------------------------
	// data accessors/manipulators
	//----------------------------------------------------------------------------------------

	inline CTextStyle_AC GetTextStyle() const { return fTextStyle; }
	inline short GetTextStyleRsrcID() const { return fTextStyleRsrcID; }
	inline short GetJustification() const { return fJustification; }
	inline bool PreferOutline() const { return fPreferOutline; }
	
	inline void SetTextStyle(const CTextStyle_AC& newTextStyle) { fTextStyle = newTextStyle; }
	inline void SetJustification(short newJustification) { fJustification = newJustification; }
	inline void SetTextStyleRsrcID(short newTextStyleRsrcID) { fTextStyleRsrcID = newTextStyleRsrcID; }
	inline void SetPreferOutline(bool preferOutline) { fPreferOutline = preferOutline; }
	
	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CTextStyle_AC fTextStyle;						// The text style (color, size, etc. )

	short fTextStyleRsrcID;						// Rsrc ID of 'TxSt' resource which has
												// TextStyle information.

	short fJustification;						// Justification of text cells

	short fLineHeight;							// height of each item including leading

	short fLineAscent;							// pos. of baseline relative to top of
                                                // line

	bool fPreferOutline;						// prefer outline fonts?

};


//----------------------------------------------------------------------------------------
// TTextListView: This subclass of TTextGridView handles lists in a manner similar to the
// List Manager in the toolbox. These lists typically consist of single columns and the
// case of this subclass handle the placeme { Initialization and Free Methods
//----------------------------------------------------------------------------------------

class EXPORT_MA TTextListView : public TTextGridView
{
public:
	inline TTextListView(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,	// Size determiners 
								GridCoordinate numOfItems = 0,
								ViewCoordinate rowHeight = 0,		// Height of initial rows 
								ViewCoordinate colWidth = 0,			// Height of initial columns 
								bool adornRows = false,			// Adornment for Rows? 
								bool adornCols = false,			// Adornment for Columns? 
								ViewCoordinate rowInset = 0,			// horizontal space between cells 
								ViewCoordinate colInset = 0,			// vertical space between cells 
								bool singleSelection = true,
								const CTextStyle_AC& itsTextStyle = gSystemStyle) :
	TTextGridView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, numOfItems, 1, rowHeight, colWidth, adornRows, adornCols, rowInset, colInset, singleSelection, itsTextStyle) { }
		// Initialize the TextListView. If numOfItems is non-zero then the listview is
        // initialized to the specified size. The width and height of the items are
        // determined by the font size, style etc. The default size may be changed with
        // calls to the the SetItemHeight and SetItemWidth methods. The "Insets" allow
        // space between items.

	virtual ~TTextListView();
		// Destructor
		
	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Methods To OVERRIDE
	//------------------------------------------------------------------------------------

	virtual CStr255_AC GetItemText(GridCoordinate) const;
		// Get the text to be drawn for a given item. THIS METHOD MUST BE OVERRIDDEN !!

	virtual GridCoordinate GetItemIndexOrdered(GridCoordinate nthItem);
		// Override this method if you are using the key selection behavior and the items
        // returned by GetItemText are not in alpha order. Return the position in the text
		// list of the 'nthItem' in alpha order.
		
	virtual bool CanSelectItem(GridCoordinate anItem);
		// Determine if the item is selectable


	//------------------------------------------------------------------------------------
	// General Methods
	//------------------------------------------------------------------------------------

	virtual bool CanSelectCell(const GridCell& aCell);
		// This method checks to see if a cell can be selected. This method simply calls
        // CanSelectItem

	virtual void DelItemAt(GridCoordinate anItem, GridCoordinate numOfItems);
		// Delete the specified items.This causes a redraw of only the necessary cells.

	virtual void DelItemFirst(GridCoordinate numOfItems);
		// Delete the specified items.This causes a redraw of only the necessary cells.

	virtual void DelItemLast(GridCoordinate numOfItems);
		// Delete the specified items.This causes a redraw of only the necessary cells.

	virtual void DoKeySelection(const CStr255_AC& selectionString);
		// Selection the proper element in this list view based on the partial CPascalStr_AC
        // 'selectionString'. This will be called by the key selection behavior if one is
        // added to this view.

	virtual ViewCoordinate GetItemHeight(GridCoordinate anItem);
		// Return the height of the specified item.

	virtual ViewCoordinate GetItemWidth();
		// Return the width of the view.

	virtual CStr255_AC GetText(const GridCell& aCell) const;
		// Calls GetItemText with an item number.

	virtual bool InsItemBefore(GridCoordinate anItem, GridCoordinate numOfItems);
		// Insert numOfItems before anItem in the list.

	virtual bool InsItemFirst(GridCoordinate numOfItems);
		// Insert numOfItems at the beginning of the list.

	virtual bool InsItemLast(GridCoordinate numOfItems);
		// Insert numOfItems at the end of the list.

	virtual bool IsItemSelected(GridCoordinate anItem);
		// Check if the specified item is currently selected.

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
		// Resize the view.

	virtual void SelectCell(const GridCell& theCell,
								   bool extendSelection,
								   bool highlight,
								   bool select);
		// Calls SelectItem with the appropriate item number

	virtual void SelectItem(GridCoordinate anItem,
								   bool extendSelection,
								   bool highlight,
								   bool select);
		// Select the given item. If extend is true then the item is added to the current
        // selection. Otherwise the current selection is deselected. If highlight is true
        // then the selected item is highlighted.

	virtual void SetItemHeight(GridCoordinate anItem,
									  GridCoordinate numOfItems,
									  ViewCoordinate aHeight);
		// Set the height of an item to be different from the default.This changes the
        // height of only the specified items.

	virtual void SetItemWidth(ViewCoordinate aWidth);
		// Set the width of the view to aWidth. (This changes the width of all the items.

	virtual GridCoordinate FirstSelectedItem();
		// Returns the first item selected.

	virtual GridCoordinate LastSelectedItem();
		// Returns the last item selected.

	virtual void InvalidateItem(GridCoordinate anItem);
		// Invalidates the given item, causing it to be redrawn

};


//----------------------------------------------------------------------------------------
// TCellSelectCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TCellSelectCommand : public TTracker
{
public:

	TCellSelectCommand(TGridView* itsView = NULL,
										   const CViewPoint& itsMouse = gZeroVPt,
										   bool theShiftKey = false,
										   bool theCommandKey = false);
		// Constructor

	virtual ~TCellSelectCommand();
		// If the region fPreviousSelection and fDifference are not NULL then dispose of them
        // before calling Inherited::Free.

	virtual void TrackFeedback(TrackPhase aTrackPhase,
										const CViewPoint& anchorPoint,
										const CViewPoint& previousPoint,
										const CViewPoint& nextPoint,
										bool mouseDidMove,
										bool turnItOn);
		// Override this method to give feedback while the mouse button is down. The
        // default does nothing.

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										CViewPoint& anchorPoint,
										CViewPoint& previousPoint,
										CViewPoint& nextPoint,
										bool mouseDidMove);
		// Track the mouse when the button is down.

	virtual void DoIt();
		// The method that will do the actual task to be performed by the command.MUST BE
        // OVERRIDDEN.

	virtual void ComputeAnchorCell(const GridCell& clickedCell);
		// Computes the cell that the mouse is first clicked in when making a selection.

	virtual void ComputeNewSelection(const GridCell& clickedCell);
		// Calculate what the new selection should be.

	virtual void HighlightNewSelection();
		// Highlight the new selection.

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
public:
	TGridViewDesignator fPreviousSelection;

	TGridView* fGridView;						// The associated GridView
	TGridViewDesignator* fThisSelection;		// pointer to the view's selection
	GridCell fAnchorCell;
	GridCell fPreviousCell;
	bool fShiftKey;								// Shift Key down?
	bool fCommandKey;							// Command Key down?
	bool fDeselecting;
};


//----------------------------------------------------------------------------------------
// TRCSelectCommand: An abstract superclass for row and column selection.
//----------------------------------------------------------------------------------------

class EXPORT_MA TRCSelectCommand : public TCellSelectCommand
{
public:
	inline TRCSelectCommand(TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false) :
	TCellSelectCommand(itsView, theMouse, theShiftKey, theCommandKey) { }
		// constructor

	virtual ~TRCSelectCommand();
		// Destructor
		
	virtual void ComputeNewSelection(const GridCell& clickedCell);
		// Computes the new selection.

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										CViewPoint& anchorPoint,
										CViewPoint& previousPoint,
										CViewPoint& nextPoint,
										bool mouseDidMove);
		// Track the mouse during selection.
};


//----------------------------------------------------------------------------------------
// TRowSelectCommand: A subclass that performs selections on rows
//----------------------------------------------------------------------------------------

class EXPORT_MA TRowSelectCommand : public TRCSelectCommand
{
public:
	inline TRowSelectCommand(TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false) :
	TRCSelectCommand(itsView, theMouse, theShiftKey, theCommandKey) { }
		// constructor
	
	virtual ~TRowSelectCommand();
		// Destructor
		
	virtual void ComputeAnchorCell(const GridCell& clickedCell);
		// Override of method to perform computations specific to row selection.

	virtual void ComputeNewSelection(const GridCell& clickedCell);
		// Override of method to perfom computation of new selection specific to row
        // selection.
};


//----------------------------------------------------------------------------------------
// TColumnSelectCommand: A subclass that handles the selection of columns in a GridView.
//----------------------------------------------------------------------------------------

class EXPORT_MA TColumnSelectCommand : public TRCSelectCommand
{
public:
	inline TColumnSelectCommand(TGridView* itsView = NULL,
									const CViewPoint& theMouse = gZeroVPt,
									bool theShiftKey = false,
									bool theCommandKey = false) :
	TRCSelectCommand(itsView, theMouse, theShiftKey, theCommandKey) { }
		// constructor
	
	virtual ~TColumnSelectCommand();
		// Destructor
		
	virtual void ComputeAnchorCell(const GridCell& clickedCell);
		// Override of method to perform computations specific to column selection.

	virtual void ComputeNewSelection(const GridCell& clickedCell);
		// Override of method to perform computation of new selection specific to column
        // selection.
};


//----------------------------------------------------------------------------------------
// CRowIterator: an iterator class designed to replace the EachRowDo method of
// TGridView.  APW 6-13
//----------------------------------------------------------------------------------------

class EXPORT_MA CRowIterator : public CIterator_AC
{
public:
	enum
	{
		kNullRow = 0 
	};

	inline CRowIterator(const TGridView*	itsGridView,
							GridCoordinate		startRow,
							GridCoordinate		stopRow,
							bool		itsForward)
	{ IRowIterator(itsGridView, startRow, stopRow, itsForward); }
							
	CRowIterator(const TGridView* itsGridView, bool itsForward);
	CRowIterator(const TGridView* itsGridView);

	//
	// Required functions 
	//
	GridCoordinate First();
	inline GridCoordinate Current() const { return fCurrent; }
	GridCoordinate Next();

	//
	// Convenience functions for C++meisters
	//
	inline GridCoordinate operator++() { return Next(); }
	inline GridCoordinate operator++(int) { GridCoordinate returnVal = Current(); Next(); return returnVal; }
	inline operator GridCoordinate() const { return Current(); }

	//
	// Compatibility functions
	//
	inline GridCoordinate FirstRow() { return First(); }
	inline GridCoordinate CurrentRow() const { return Current(); }
	inline GridCoordinate NextRow() { return Next(); }

private:
	void IRowIterator(const TGridView*	itsGridView,	//called exclusively by the
								GridCoordinate		startRow,		//constructors of this class.
								GridCoordinate		stopRow,
								bool		itsForward); 

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	GridCoordinate 	fCurrent;
	GridCoordinate	fFirstRow;
	GridCoordinate	fLastRow;
	bool	fIterateForward;
};

//----------------------------------------------------------------------------------------
// CColumnIterator: an iterator class designed to replace the EachColumnDo
// method of TGridView.  APW 6-13
//----------------------------------------------------------------------------------------

class EXPORT_MA CColumnIterator : public CIterator_AC
{
public:
	enum
	{
		kNullColumn = 0
	}; 

	inline CColumnIterator(const TGridView*	itsGridView,
							GridCoordinate		startColumn,
							GridCoordinate		stopColumn,
							bool		itsForward)
	{ IColumnIterator(itsGridView, startColumn, stopColumn, itsForward); }
							
	CColumnIterator(const TGridView* itsGridView, bool itsForward);
	CColumnIterator(const TGridView* itsGridView);

	//
	// Required functions 
	//
	GridCoordinate First();
	inline GridCoordinate Current() const { return fCurrent; }
	GridCoordinate Next();

	//
	// Convenience functions for C++meisters
	//
	inline GridCoordinate operator++() { return Next(); }
	inline GridCoordinate operator++(int) { GridCoordinate returnVal = Current(); Next(); return returnVal; }
	inline operator GridCoordinate() const { return Current(); }

	//
	// Compatibility functions
	//
	inline GridCoordinate FirstColumn() { return First(); }
	inline GridCoordinate CurrentColumn() const { return Current(); }
	inline GridCoordinate NextColumn() { return Next(); }

protected:

	void IColumnIterator(const TGridView*	itsGridView,	//to be called exclusively
									GridCoordinate		startColumn,	//by the constructors of
									GridCoordinate		stopColumn,		//this class.
									bool		itsForward); 

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	GridCoordinate fCurrent;
	GridCoordinate fFirstColumn;
	GridCoordinate fLastColumn;
	bool	fIterateForward;
};

//----------------------------------------------------------------------------------------
// CCellIterator: an iterator class designed to replace the EachCellDo method
// of TGridView.  APW 6-13
//----------------------------------------------------------------------------------------

class EXPORT_MA CCellIterator : public CIterator_AC
{
public:
	static const GridCell kNullCell; 
	static const bool kIterateRowMajor;
	
	CCellIterator(const TGridView*	itsGridView,
							const GridCell&	startCell,
							const GridCell&	stopCell,
							bool		itsRowForward,
							bool		itsColumnForward,
							bool		itsRowMajor);
							
	CCellIterator(const TGridView*	itsGridView,
							bool		itsRowForward,
							bool		itsColumnForward,
							bool		itsRowMajor);

	CCellIterator(const TGridView* itsGridView);

	//
	// Required functions 
	//
	GridCell First();
	inline GridCell Current() const { return fCurrent; }
	GridCell Next();

	//
	// Convenience functions for C++meisters
	//
	inline GridCell operator++() { return Next(); }
	inline GridCell operator++(int) { GridCell returnVal = Current(); Next(); return returnVal; }
	inline operator GridCell() const { return Current(); }
	inline operator const long() const { return *reinterpret_cast<const long*>(&fCurrent); }

	//
	// Compatibility functions
	//
	inline GridCell FirstCell() { return First(); }
	inline GridCell CurrentCell() const { return Current(); }
	inline GridCell NextCell() { return Next(); }

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	GridCell 		fCurrent;
	CRowIterator	fRowIterator;
	CColumnIterator	fColumnIterator;
	bool			fIterateRowMajor;
};


//----------------------------------------------------------------------------------------
// CCellInRegionIterator: an iterator class designed to replace the EachInRegion method
// of TGridView.  APW 7-15
//----------------------------------------------------------------------------------------

class EXPORT_MA CCellInRegionIterator : public CCellIterator
{
public:
	CCellInRegionIterator(const TGridView*	itsGridView,
							const TGridViewDesignator*	aRegion,
							bool		itsRowForward,
							bool		itsColumnForward,
							bool		itsRowMajor);

	CCellInRegionIterator(const TGridView*	itsGridView,
							const TGridViewDesignator*	aRegion);

	//
	// Required functions 
	//
	GridCell First();
	inline GridCell Current() const { return CCellIterator::Current(); }
	GridCell Next();

	//
	// Convenience functions for C++meisters
	//
	inline GridCell operator++() { return Next(); }
	inline GridCell operator++(int) { GridCell returnVal = Current(); Next(); return returnVal; }
	inline operator GridCell() const { return Current(); }
	inline operator const long() const { return *reinterpret_cast<const long*>(&fCurrent); }

	//
	// Compatibility functions
	//
	inline GridCell FirstCell() { return First(); }
	inline GridCell CurrentCell() const { return Current(); }
	inline GridCell NextCell() { return Next(); }

protected:
	
	bool CellIsInRegion();

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	const TGridViewDesignator* fRegion;
};

//----------------------------------------------------------------------------------------
// CSelectedCellIterator: an iterator class designed to replace the EachSelectedCellDo method
// of TGridView.  APW 7-15
//----------------------------------------------------------------------------------------

class EXPORT_MA CSelectedCellIterator : public CCellInRegionIterator
{
public:
	CSelectedCellIterator(const TGridView*	itsGridView,
							bool		itsRowForward,
							bool		itsColumnForward,
							bool		itsRowMajor);

	CSelectedCellIterator(const TGridView*	itsGridView);
};


//----------------------------------------------------------------------------------------
// Global functions declarations.
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUGridView();				
	// Creates all the regions used by the GridView class and then sets the global flag
    // 'gUGridViewInitialized'.


#endif
