// UView.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UVIEW__
#define __UVIEW__

// ACS

#ifndef __CIterator__
#include "CIterator_AC.h"
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#if qDrag
#	ifndef __UDRAGDROP__
#	include "UDragDrop.h"
#	endif
#endif

#ifndef __UDRAWINGENVIRONMENT__
#include "UDrawingEnvironment.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#if qGXViews || qGXPrinting	
#	ifndef __GXTYPES__
#	include <GXTypes.h>
#	endif
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class CAEDesc;
class TAdorner;
class TAdornerList;
class TDesignator;
class TDocument;
class TDragItem;
class TDrawingEnvironment;
class TPrintHandler;
class TScroller;
class TToolboxEvent;
class TWindow;

class EXPORT_MA CAutoPtr_AC<TDrawingEnvironment>;
class EXPORT_MA CAutoPtr_AC<TAdornerList>;

//----------------------------------------------------------------------------------------
// MouseDownType
//----------------------------------------------------------------------------------------

enum EMouseDownType 
{
	//kDragOnly,					// (never used)
	kDragOrClick,				// !isFirstClick
	kFirstClick_dragOnly,		// isFirstClick		!windowDoesFirstClick
	kFirstClick_dragOrClick		// isFirstClick		windowDoesFirstClick
};

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------
const unsigned char kFrame = adnLineTop | adnLineLeft | adnLineBottom | adnLineRight;
												// Defines that a view be completely framed.

const bool kImmediateSuperView = true;			// Used with GetScroller to return only the
												// scroller of the immediate superview
const bool kAnySuperView = false;				// Used with GetScroller to return scroller
												// in the super view chain
const bool kInsertSubViewFirst = false;			// Use to insert subviews at the front of
												// the list rather than the end (default). 

//----------------------------------------------------------------------------------------
// SizeDeterminer: Tells how a view's size is to be determined; specified separately in
// each dimension.
//----------------------------------------------------------------------------------------

enum SizeDeterminer
{
	sizeSuperView,								// View is the same size as its superview

	sizeRelSuperView,							// View size is relative to the
                                                // superview's size

	sizePage,									// View to be the size of one page

	sizeFillPages,								// View to grow upward to fill an exact
                                                // number of pages

	sizeVariable,								// View size fluctuates according to
                                                // app-specific criteria

	sizeFixed
};

//----------------------------------------------------------------------------------------
// FrameInvalidateDeterminer: Tells how a view's frame is to be invalidated in SetFrame.
//----------------------------------------------------------------------------------------

enum FrameInvalidateDeterminer
{
	frameInvalidateDifference,					// Always invalidate frame difference
		
	frameInvalidateWholeFrame					// Always invalidate whole frame, content is proportional
};

//----------------------------------------------------------------------------------------
// ViewPortInfo: Set by GetFocus; used by SetFocus. GetFocus/ SetFocus are used to save and
// restore the focus behind MacApp's back.
//----------------------------------------------------------------------------------------

struct ViewPortInfo
{
	CPoint_AC org;
	GrafPtr port;
	RgnHandle clip;
	bool isValid;							// true if FocusRec is valid
};


//----------------------------------------------------------------------------------------
// FocusRec
//----------------------------------------------------------------------------------------

struct FocusRec
{
	ViewPortInfo itsViewPortInfo;
	bool printing;
	bool drawingPictScrap;
	TView* drawingPictScrapView;
	TView* focusedView;
};


//----------------------------------------------------------------------------------------
// TView: Renders an image of a document or other data, and identifies mouse-clicked
// objects.
//----------------------------------------------------------------------------------------

class EXPORT_MA TView : public TCommandHandler
{
  public:
	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------
	TView(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable);
		// Constructor

	TView(const TView& inView);
		// Copy Constructor

	virtual ~TView();
		// Frees its subviews, then if its superview is not NULL removes itself from the
        // superviews list of subviews and if its document is not NULL removes itself from
        // the documents list of views. If it has a print handler then it frees it, then
        // calls Inherited::Free. Frees its fAdorners list.

	virtual void* Clone() const;
		// return a copy of the view by using the copy constructor
	
	virtual void DoPostCreate(TDocument* itsDocument);
		// Called in a post-order traversal of a newly-created view hierarchy
		
	virtual void FreeFromClipboard();
		// If its document is not NULL then calls the document's FreeFromClipboard method,
        // otherwise calls Free.


	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFrom(CStream_AC* aStream);
	virtual void ReadFields(CStream_AC* aStream);
	virtual void ReadSubViews(CStream_AC* aStream);
	
	virtual void WriteTo(CStream_AC* aStream) const;
	virtual void WriteFields(CStream_AC* aStream) const;
	virtual void WriteSubViews(CStream_AC* aStream) const;


	//------------------------------------------------------------------------------------
	// Coordinate Conversion Methods
	//------------------------------------------------------------------------------------

	CViewPoint QDToView(CPoint_AC qdPoint);
	CViewRect QDToView(const CRect_AC& qdRect);
		// Convert a QuickDraw measurement to a view measurement.

	CPoint_AC ViewToQD(const CViewPoint& viewPt);
	CRect_AC ViewToQD(const CViewRect& viewRect);
		// Convert a view measurement to a QuickDraw measurement.

	CViewPoint GetLocalOffsetInSuper() const;
		// Returns the location of the origin for this view coordinate systemin the
        // superview's coordinate system

	virtual void SetLocalOrigin(const CViewPoint& newOrigin, bool redraw);
		// Sets the origin of the view's local coordinate system. Redraws if requested.

    CPoint_AC LocalToGlobal(CPoint_AC thePoint) const;
    CViewPoint LocalToGlobal(const CViewPoint& thePoint) const;
    CViewRect LocalToGlobal(const CViewRect& viewRect) const;
    void LocalToGlobal(RgnHandle theRgn);
    	// Given a measurement in local view coordinates, returns the same measurement in
    	// global screen coordinates.
        
	CViewPoint LocalToSuperView(const CViewPoint& thePoint) const;
	CViewRect LocalToSuperView(const CViewRect& viewRect) const;
	void LocalToSuperView(RgnHandle theRgn);
		// Given a measurement in local view coordinates, returns the same measurement in
		// its superview's coordinate system.

	CViewPoint SuperViewToLocal(const CViewPoint& thePoint) const;
	CViewRect SuperViewToLocal(const CViewRect& viewRect) const;
	void SuperViewToLocal(RgnHandle theRgn);
		// Given a measurement in superview coordinates, returns the same measurement in
		// the view's own coordinate system.

	CViewPoint LocalToRootView(const CViewPoint& thePoint) const;
    CViewRect LocalToRootView(const CViewRect& viewRect) const;
    void LocalToRootView(RgnHandle theRgn);
		// Given a measurement in local view coordinates, returns the same measurement in
		// its most super superview's (usually window) coordinate system.

	CViewPoint RootViewToLocal(const CViewPoint& thePoint) const;
	CViewRect RootViewToLocal(const CViewRect& viewRect) const;
	void RootViewToLocal(RgnHandle theRgn);
		// Given a point in window (or most super superview) coordinates, returns the same
        // point in the view's own coordinate system.

	//------------------------------------------------------------------------------------
	// Dispatching
	//------------------------------------------------------------------------------------
	virtual bool HandleMouseDown(const CViewPoint& theMouse,
									TToolboxEvent* event,
									CPoint_AC 	hysteresis,
									EMouseDownType mouseDownType = kDragOrClick);
		// The method responsible for handling a mouse down in the view, calls
        // DoMouseUp to handle it.

	bool HandleMouseUp(const CViewPoint& theMouse,
								  TToolboxEvent* event,
								  CPoint_AC hysteresis);
		// The method responsible for handling a mouse up in the view, calls
        // DoMouseUp to handle it.

	void HandlePostCreate(TDocument* itsDocument);
		// Post-order traversal calling DoPostCreate
	
	void HandleDraw(const CViewRect& area);
		// Causes the area of the views contents to be drawn and that of any of its subviews.

	void HandleHelp(const CViewPoint& theMouse, RgnHandle helpRegion);
		// Try to find a subview that contains the mouse and wants to handle the help. 
		// Otherwise, call our own DoShowHelp method to actually show the help balloon. 
	
	void HandleCursor(const CViewPoint& theMouse, RgnHandle cursorRegion);
		// Essentially mouse-enter
		// Try to find a subview that contains the mouse and wants to handle the cursor. 
		// Otherwise, call our behaviors' DoSetCursor method and then our own DoSetCursor
		// method to actually set the cursor. 

	void HandleUnCursor();
		// Essentially mouse-exit
		// Called against the last view that set the cursor when the cursor is set by
		// the application or another view.
		// This method finds a view to call DoUnsetCursor against. 

#if qDrag
	void HandleDragCursor(const CViewPoint& theMouse, RgnHandle cursorRegion);
#endif

	//------------------------------------------------------------------------------------
	// View Navigation Methods
	//------------------------------------------------------------------------------------

	TView* FindPeerView(IDType itsID);
		// Locate a sibling view by its ID.
		
	TView* FindParentView(IDType itsID);
		// Locate a containing view by its ID.

	//------------------------------------------------------------------------------------
	// SubView Management Methods
	//------------------------------------------------------------------------------------

	void InsertSubView(TView* theSubView, bool last = true);
		// Add 'theSubView' to our list of subviews. By default adds to end of list
		// (in drawing order). To add to the front of the list, pass kInsertSubViewFirst. 
		// Just does list manipulation doesn't do any sub/superview notification

	void InsertSubView(TView* theSubView, TView* beforeView);
		// Add 'theSubView' to our list of subviews.  Adds before beforeView if it is in
		// the list, otherwise adds to the end of the list.
		// Just does list manipulation doesn't do any sub/superview notification

	void DeleteSubView(TView* theSubView);
		// Remove 'theSubView' from our list of subviews.
		// Just does list manipulation doesn't do any sub/superview notification

	void AddSubView(TView* theSubView);
		// Add 'theSubView' to our list of subviews.

	void RemoveSubView(TView* theSubView);
		// Remove 'theSubView' from our list of subviews.

	void MakeFirstSubView(TView* theSubView);
		// Make 'theSubView ' the first subview in our list.

	void MakeLastSubView(TView* theSubView);
		// Make 'theSubView' the last subview in our list.

	short CountSubViews() const;
		// Return the number of subviews in our view.

	inline bool HasSubViews() const { return fSubView != NULL; }
		// True if this view has subviews.

	inline TView* FirstSubView() const { return fSubView; }
		// Return the first subview in our list (NULL if none).

	TView* FindSubView(IDType itsIdentifier) const;
		// Find subview with identifier 'itsIdentifier' in our list of subviews.

	bool ContainsSubView(TView* aView) const;
		// Returns true if aView is in the view hierarchy with this view as root

		
	//------------------------------------------------------------------------------------
	// Open/ Close/Activate Methods
	//------------------------------------------------------------------------------------

	virtual void Activate(bool entering);
		// Focuses and then highlights any current selection before calling all its
        // subviews to activate themselves.

	virtual void Open();
		// Calls each of its subviews to Open themselves.

	virtual void Close();
		// Commits the last command before calling each of its subviews to Close
        // themselves.

	virtual void BeInDocument(TDocument* itsDocument);
		// If 'itsDocument' is not NULL then sets its fDocument to itsDocument and
		// add itself to the documents viewlist.

	virtual void RemoveFromDocument();
		// If 'fDocument' is not NULL then removes itself from the document's viewlist
		// and sets its fDocument to NULL.

	virtual void BeInScroller(TScroller* itsScroller);
		// If 'itsScroller' is not NULL then sets its scroll limits to its fSize.

	virtual void BeInPort(GrafPtr itsPort);
		// If its print handler is not NULL, calls DoCheckPrinter. Then calls each of its
        // subviews to BeInPort.

	virtual void ShowReverted();
		// First calls AdjustSize before forcing a redraw of the view, then call
        // ShowReverted for each of its subviews.


	//------------------------------------------------------------------------------------
	// Frame (Location and Size) Methods
	//------------------------------------------------------------------------------------

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);
		//  sets our frame to newFrame, invalidating if 'invalidate' is true. 

	virtual void InvalidateFrameDifference(const CViewRect& oldFrame, const CViewRect& newFrame);
		// Invalidates the difference between the old and new frames, unless the location has
		// changed, in which case the union of the two frames is invalidated.
		
	virtual void NotifyAdornersOfFrameChange(const CViewRect& oldFrame,
											 const CViewRect& newFrame,
											 bool invalidate);
		// Iterates over this view's adorners, calling ViewChangedFrame. Called by TView::SetFrame

	virtual void NotifySubViewsOfFrameChange(const CViewRect& oldFrame,
											 const CViewRect& newFrame,
											 bool invalidate);
		// Iterates over this view's subviews, calling SuperViewChangedFrame. Called by TView::SetFrame

	virtual void Locate(const CViewPoint& newLoc, bool invalidate);
		//  Convenience routine. Sets the frame to newLoc. Don't override it since
		//  MacApp usually calls SetFrame instead. 

	virtual void Resize(const CViewPoint& newSize, bool invalidate);
		//  Convenience routine. Sizes the frame to newSize. Don't override it since
		// MacApp usually calls SetFrame instead. 

	virtual void AdjustFrame();
		// Calls ComputeFrame, if the size has changed resizes itself and then calls
        // DoPagination so that any page breaks can be recalculated.

	virtual CViewRect CalcMinFrame();
		// Calculate the minimum size of this.

	virtual void ComputeFrame(CViewRect& newFrame);
		// Called by MacAppto get the size of the view computed.

	virtual void SuperViewChangedFrame(const CViewRect& oldFrame,
									   const CViewRect& newFrame,
									   bool invalidate);
		// Called when our superview changes frame.

	virtual void SubViewChangedFrame(TView* theSubView,
									 const CViewRect& oldFrame,
									 const CViewRect& newFrame,
									 bool invalidate);
		// Called when one of our subviews changes frame.


	//------------------------------------------------------------------------------------
	// Focusing Methods
	//------------------------------------------------------------------------------------

	virtual bool Focus();
		// Attempts to set up the drawing environment for this view. That consists of
        // port, origin, gLongOffset and clipping. Returns false if not able to focus. One
        // reason focus could fail is if no superview is able to supply a port (ie not in
        // a window and not printing or drawing pict scrap). Focus will return true,
        // however, if the drawing environment can be setup but none of the view is
        // currently visible due to clipping or scrolling by a superview. This lets you
        // assemble view hierarchies and manipulate views without showing them to the
        // user, but requires that you test IsVisible if you MUST know if the view is in
        // fact visible. (MacApp will never ask a view to draw unless it is visible)

	virtual bool FocusOnSuperView();
		// Focuses on our superview.

	inline void InvalidateFocus() { gFocusedView = NULL; }
		// Indicates that the view's focus is no longer valid and must be recomputed.

	inline bool IsFocused() const { return gFocusedView == this; }
		// Check to see if we are currently focused.

	void ClipFurtherTo(RgnHandle rRgn, CPoint_AC deltaOrg);
		// Set clipping to: (<current clipping> INTERSECT r) OFFSET-BY deltaOrg.

	void OffsetAndClipFurtherTo(RgnHandle rRgn, CPoint_AC deltaOrg);
		// Set clipping to: OFFSET-BY deltaOrg and (<current clipping>
        // INTERSECT r).

#if qDebug
	void AssumeFocused() const;
		// Causes a ProgramBreak_AC if the focused view isn't this.
#endif

	void InvalidateCoordinates();
		// Indicates that the view's computed offsets are no longer valid and must be
        // recomputed.  Calls InvalidateCoordinates for all subviews.

	virtual void UpdateCoordinates();
		// Called If a coordinate related field must be retrieved and the coordinates
		// are invalid.

	CViewRect GetVisibleExtent();
		// Retrieves the fVisibleExtent, calls UpdateCoordinates if necessary.

	CPoint_AC GetQDOrigin();
		// Retrieves the fQDOrigin, calls UpdateCoordinates if necessary.

	CViewPoint GetViewToQDOffset();
		// Retrieves the fViewToQDOffset, calls UpdateCoordinates if necessary.

	bool IsOpaque() const;
		// Returns true if the view is opaque or any of its adorners are opaque

	bool ClipSubViews(RgnHandle rRgn);
		// Removes any opaque subviews from the clip when focusing
		// returns true if the region was modified

	bool ClipSiblingViews(RgnHandle rRgn);
		// Removes any opaque sibling views from the clip when focusing
		// returns true if the region was modified

	//------------------------------------------------------------------------------------
	// Adornment Methods
	//------------------------------------------------------------------------------------

	virtual TAdornerList* MakeAdornerList();
		// Creates and returns an adorner list.
		
	void DeleteAllAdorners();
		// Deletes adorners.
		
	void AddAdorner(TAdorner* anAdorner,
					AdornPriority itsPriority,
					bool invalidate);
		// adds an adorner to the view; invalidates the adorner if "invalidate" is true

	void DeleteAdorner(TAdorner* anAdorner, bool invalidate);
		// removes an adorner from the view; invalidates the adorner if "invalidate" is
        // true

	void DeleteAdornerByID(IDType itsID, bool invalidate);
		// removes an adorner from the view; invalidates the adorner if "invalidate" is
        // true

	TAdorner* AdornerAt(ArrayIndex_AC i) const;
		// Returns the i'th adorner in the list

	TAdorner* AdornerWithID(IDType itsID) const;
		// Returns the first adorner with the specified ID
		
	virtual void Dim();
		// dims the current image in the view

	virtual CViewRect GetAdornExtent();
		// returns the adorn extent for adorning this view

	virtual void Hilite();
		// hilites the current image in the view by inverting its extent

	//------------------------------------------------------------------------------------
	// Drawing Methods
	//------------------------------------------------------------------------------------

	virtual TDrawingEnvironment* DoMakeNewDrawingEnvironment();
		// Create a new drawing environment

	void CreateDrawingEnvironment();
		// Creates a new drawing environment, if none exists.
		
	void SetDrawingEnvironment(TDrawingEnvironment* itsDrawingEnvironment,
							   bool invalidate);
		// Frees the old drawing environment and stores the new one.
		
	inline TDrawingEnvironment* GetDrawingEnvironment() const { return fDrawingEnvironment; }
		// Returns this view's drawing environment object, which may be NULL.
	
	void PrepareForDrawing();
		// A possible "hook" for doing offscreen drawing. 
		// If the view has an fDrawingEnvironment, calls its Prepare method.
		// Calls SetupDrawingEnvironment.

	void SetupDrawingEnvironment();
		// If the view has an fDrawingEnvironment, calls its Setup method.
		// Otherwise calls PenNormal.

	void CompleteDrawing();
		// If the view has an fDrawingEnvironment, calls its Complete method.
		// A possible "hook" for doing offscreen drawing

	virtual void UpdateSelection(const CViewPoint& theMouse, 
								 TToolboxEvent* event);
		// Called prior to DoMouseDown and WillDrag. Provides an opportunity
		// to update selection before being asked to drag or handle a mouse
		// down.
		
	virtual void DoHighlightSelection(HLState fromHL, HLState toHL);
		// Currently does nothing. Must be overridden.

	HLState GetCurrentHL();
		// Returns fActiveHL if the view is active, otherwise fInActiveHL.

	virtual void Draw(const CViewRect& area);
		// Currently does nothing. Must be overridden.

	bool HandleDrawFailed(const CViewRect& area, const CException_AC& theException);
		// Called after catching an exception from drawing. Calls NotifyDrawFailed
		// and then DrawFailed.
		// Returns true if drawing should continue.

	virtual void NotifyDrawFailed(const CViewRect& area, const CException_AC& theException);
		// Called when a drawing failure occurs. Default to showing the error if window
		// updating is occuring.

	virtual void DrawFailed(const CViewRect& area, const CException_AC& theException);
		// Called when a drawing failure occurs. Default fills in the view
		// with ltGray and shows its outline in debug mode.

	void DrawContents();
		// Causes the views contents to be drawn and that of any of its subviews.

	void DrawAdorners(const CViewRect& area);
		//	Asks each adorner in the list of adorners to draw

	void HighlightAdorners(const CViewRect& area,
						   HLState fromHL,
						   HLState toHL);
		// Calls each adorner's DoHighlightSelection method
		
	virtual TDesignator* GetUserSelection();
		// Returns the designator for the user selection (if any).Defaults to returning
        // the user selection of the document (if any)

	virtual void SetUserSelection(TDesignator* newSelection);
		// Sets the designator for the user selection (if any). Defaults to setting the
        // user selection of the document (if any)

	virtual void UserSelectionChanged(TView* changedView);
		// Call this whenever the user's selection changes. Defaults to calling the
        // UserSelectionChanged method of the document (if any).

	virtual void ScrollSelectionIntoView(bool redraw);	
		// Scroll the selection into view.

	virtual bool HasPendingUpdate() const;
		// Returns true if there is a pending update event for this view, false if not.

	virtual void Update();
		// If its window is not NULL then does an update by calling DrawContents.
	

	//------------------------------------------------------------------------------------
	// Validation/ Invalidation Methods
	//------------------------------------------------------------------------------------

	virtual void ForceRedraw();
		// Invalidates the entire view forcing a redraw.

	void Invalidate(const CViewRect& badVRect);
	void Invalidate(const CRect_AC& badRect);
	void Invalidate(const RgnHandle badRgn);
		// Invalidates an area of the view.

	virtual void DoInvalidateRegion(const RgnHandle badRgn);
		// Invalidates a Region by passing it to the superview
		// Intersects badRgn with the visible extent then calls DoInvalidateRegion with the
        // result. We can only call InvalidRgn on a grafPort belonging to a window, if
        // your code calls InvalidRgn on a non-window grafPort, it will die miserably. So
        // views forward up their superview chain until they reach a view which overrides
        // DoInvalidateRegion (usually a window) which then makes the actual InvalidRgn call

	void Validate(const CViewRect& goodVRect);
	void Validate(const CRect_AC& goodRect);
	void Validate(const RgnHandle goodRgn);
		// Validates an area of the view.

	virtual void DoValidateRegion(const RgnHandle goodRgn);
		// Validates a Region by passing it to the superview
		// Intersects goodRgn with the visible extent then calls DoValidateRegion with the
        // result. We can only call ValidRgn on a grafPort belonging to a window, if your
        // code calls ValidRgn on a non-window grafPort, it will die miserably. So views
        // forward up their superview chain until they reach a view which overrides
        // DoValidateRegion (usually a window) which then makes the actual ValidRgn call

	virtual void RevealRect(const CViewRect& rectToReveal,
							const CViewPoint& minToSee,
							bool redraw);
		// Scrolls a rect into view, with 'minToSee' being the point which will end up at
        // the bottom of the view, i.e.the minimum amount to be seen and 'rectToReveal'
        // the rect being scrolled.

	void RevealTop(bool redraw);
		// Scroll the top of a rect into view.

	void RevealBottom(bool redraw);
		// Scroll the bottom of a rect into view.

	virtual void RevealUndoRedo(TCommand* command); // override
		// Make this context visible after undo/redo. 


	//------------------------------------------------------------------------------------
	// Mouse Handling Methods
	//------------------------------------------------------------------------------------

	virtual EventNumber GetEventNumber();
		// Returns cNoEvent. Overridden by TControl.

	virtual bool ContainsMouse(const CViewPoint& theMouse);
		// Is the view point 'theMouse' currently in our view.

	virtual void DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis);
		// Posts a simple tracker which by default calls back tracking methods of this
        // view. There are two possibilities here: 1) Override DoMouseCommand to post
        // commands that represent more complex, even undoable actions.That command may
        // either let its tracking call back the view's tracking methods (the default)
        // which must be overridden to do anything useful or it may simply do all the
        // tracking itself. This mostly depends on whether different commands returned
        // from the view would have different tracking behavior.For example this is the
        // approach taken by DrawShapes. 2) Override the view's tracking methods and let
        // the default tracker them.For example this is the approach taken by TControl.

	virtual void DoMouseUp(CViewPoint& theMouse,
						   TToolboxEvent* event,
						   CPoint_AC hysteresis);
		// Called for the most deeply nested view that contains the mouse.

	//------------------------------------------------------------------------------------
	// Cursor and Help Handling Methods
	//------------------------------------------------------------------------------------

	void RestrictRegionToVisible(RgnHandle aRegion);
		// Restricts the region to the visible extent of this view
		
	virtual bool HandlesCursor() const;
		// Returns true, if this view tracks the mouse so that it can claim the cursor.
        // The default returns true. If false, subviews may still get a chance to set the
        // cursor, as determined by TView::SubViewsHandleCursor.
				
	virtual bool LetsSubViewsHandleCursor() const;
		// Returns true if this view should allow subviews a chance to set the cursor. If
        // true, the subviews themselves may still return false for HandlesCursor. By
        // setting both values false, cursor tracking can be turned off for an entire
        // window, such as the clipboard window.

	virtual bool HandlesHelp() const;
		// Returns true, if this view tracks the mouse so that it can show help. The
        // default returns true. If false, subviews may still get a chance to show help,
        // as determined by TView::SubViewsHandleHelp.

	virtual bool LetsSubViewsHandleHelp() const;
		// Returns true if this view should allow subviews a chance to show help. The
        // subviews themselves may still return false for HandlesHelp. By setting both
        // values false, help can be turned off for an entire view hierarchy
	
#if qWantsContextualMenus
	
	virtual void GetContextualMenu(MenuRef& theMenuRef, CAEDesc& theSelection);
		// Return a contextual menu and AppleEvent descriptor for the selection

#endif
		
	virtual void DoSetCursor(const CViewPoint& localPoint, 
							 RgnHandle cursorRegion);
		// Can be overridden to set the cursor and compute a cursor region. The default
        // sets the cursor to an arrow and uses TView::GetDefaultCursorRegion to compute the
        // largest appropriate cursor region

	virtual void DoUnsetCursor();
		// Called against the last view that set the cursor when the cursor is set by
		// the application or another view. 

	virtual ResNumber GetHelpID();
		// Returns fHelpID.
		
	virtual short GetHelpIndex();
		// Returns fHelpIndex.
		
	virtual short GetHelpState();
		// Returns kHMEnabledItem, kHMDisabledItem, etc.
		
	virtual ResNumber GetCursorID();
		// Returns fCursorID.
		
	void GetHelpParameters(const CViewPoint& localPoint,
						   RgnHandle helpRegion,
						   HMMessageRecord& helpMessage,
						   CPoint_AC& localQDTip,
						   CRect_AC& localQDRect,
						   short& balloonVariant);
		// Determine the parameters needed for showing a help balloon for this view. The
		// default calls the TApplication method of the same name which gets the information
		// from a resource of type 'hdlg'. The fields fHelpID and fHelpIndex specify the
		// resource and component within the resource containing the information for this
		// view.

	void DoShowBalloon(const CViewPoint& localPoint,
					   RgnHandle helpRegion,
					   const HMMessageRecord& helpMessage,
					   CPoint_AC localQDTip,
					   const CRect_AC& localQDRect,
					   short balloonVariant);
		// Show the help balloon. 

	void DoShowHelp(const CViewPoint& localPoint, RgnHandle helpRegion);
		// Gets the help parameters and shows the help balloon. 

	virtual void GetDefaultCursorRegion(const CViewPoint& localPoint, 
										RgnHandle cursorRegion);
		// Returns the largest appropriate cursor region, called by DoSetCursor.

	virtual void GetDefaultHelpRegion(const CViewPoint& localPoint, RgnHandle helpRegion);
		// Returns the largest appropriate help region, called by DoShowHelp.


	//------------------------------------------------------------------------------------
	// Miscellaneous Methods
	//------------------------------------------------------------------------------------

	virtual TCommandHandler* GetContext(CommandNumber aCommandNumber); // Override
		// Returns the view's document, if any, or the view itself.

	virtual void PostCommand(TCommand* command);
		// If no fNextHandler then try posting to document

	virtual GrafPtr GetGrafPort() const;
		// Returns a pointer to our Graf Port.

	virtual TScroller* GetScroller(bool immediateSuperView);
		// Returns a reference to our scroller.

	virtual TView* GetRootView();
		// Returns a reference to the root view.

	virtual TWindow* GetWindow() const;
		// Returns a reference to the window that we are in, if any.
		
	CViewRect GetExtent() const;
		// Returns a view CRect_AC the size of our extent in 'itsExtent'.

	virtual void GetExtentRegion(RgnHandle itsExtentRgn);
		// Shapes the supplied region to the view's extent. Override for non-rectangular
		// views to return a region for the drawable (qdExtent) portion of the view. 

	CRect_AC GetQDExtent();
		// Returns a QuickDraw CRect_AC the size of our extent in 'qdExtent'.

	CViewRect GetFrame() const;
		// Returns in'itsFrame' a CViewRect that is the boundary of our view in superview
        // coordinates. This is calculated by adding its size to its location.

	CViewRect GetVisibleRect();
	CRect_AC GetVisibleQDRect();
	void GetVisibleRegion(RgnHandle visibleRgn);
		// Returns area enclosing the visible portion of the view in view coordinates.

	CViewRect GetDrawableRect();
	CRect_AC GetDrawableQDRect();
	void GetDrawableRegion(RgnHandle drawRgn);
		// Returns area enclosing the drawable portion of the view in view coordinates.

	virtual short GetPartCode(const CViewPoint& theMouse) const;
		// tells which part of a view, if any, the mouse button was pressed in. Returns:
        // inContent = 3, inDrag = 4, inGrow = 5, inGoAway = 6, or inDesk = 0 (ie. none of
        // the above),depending on where theMouse is in the view

	void ViewEnable(bool state, bool redraw);
		// Enables & Redraws the view based on 'state' & 'redraw'.

	virtual void Show(bool state, bool redraw);
		// If 'state' is not fShown then set fShown to true and call ForceRedraw if
        // 'redraw' is true. Otherwise invalidate our focus and set fShown to 'state'.

	virtual bool IsShown() const;
		// Checks to see if our view is currently shown.

	bool IsVisible();
		// Returns true if any portion of the view is currently visible when focused

	bool IsDrawable();
		// Returns true if any portion of the view is currently drawable when focused

	virtual bool IsActive() const;
		// Returns true if the view is in an "active" superview (typically a window) 

	
	//------------------------------------------------------------------------------------
	// Target Management and Validation
	//------------------------------------------------------------------------------------

	virtual bool WantsToBeTarget();	// Override
		// Returns the value of fWantsToBeTarget

	virtual long GetValidationError();
		// Returns an error value "noErr". Overridden by TEditText and TNumberText.
		// Formerly known as "Validate"
		
	bool IsValid();
		// Returns true if this view is in a valid state. Calls ValidationFailed
		// if it is not. 

	bool IsHierarchyValid();
		// Returns true if this view and all its subviews are in a valid state. 
		// User may be alerted via ValidationFailed method.

	virtual void ValidationFailed(long reason);
		// Can be overridden to alert the user, as is done by TEditText.

	TView* FindTarget();
		// Find a subview which wants to be the target

	virtual void SetTargetSelection(bool redraw);	// Override
		// Called when this object becomes the target, except as a result of a mouse click
		// Overridden to reveal the view if it is scrolled out of view


	//------------------------------------------------------------------------------------
	// Clipboard Handling
	//------------------------------------------------------------------------------------

	virtual PicHandle AsPict();
		// Returns a PicHandle containing a rendering of the view and its subviews.

	virtual bool ContainsClipType(ResType aType);
		// Checks to see if the clipboard contains the type 'aType'.

	virtual long GivePasteData(Handle aDataHandle, ResType dataType);
		// Gets the scrap data of the type 'dataType', returns any errors as function
        // result.

	virtual void WriteToDeskScrap();
		// Writes to desk scrap.


	//------------------------------------------------------------------------------------
	// PrintingMethods
	//------------------------------------------------------------------------------------

	TPrintHandler* GetPrintHandler();
		// return the associated print handler, if any.

	void AttachPrintHandler(TPrintHandler* itsPrintHandler);
		// Called by the PrintHandler during its initialization, NOT directly by you

	void DetachPrintHandler(TPrintHandler* itsPrintHandler);
		// Called by the PrintHandler when freed, NOT directly by you

	virtual ViewCoordinate DoBreakFollowing(VHSelect vhs,
										 ViewCoordinate previousBreak,
										 bool& automatic);
		// Returns the location of the page break which follows the page break located at
        // 'previousBreak', in direction vhs; returns automatic = true if the page-break is
        // thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note
        // that page-breaks in dimension 'v' are drawn as vertical lines, those in
        // dimension 'h' as horizontal lines

	virtual CViewPoint DoCalcPageStrips();
		// Computes the number of horizontal and vertical page strips.

	virtual CViewPoint DoCalcViewPerPage();
		// Determine how much of the view normally goes into each printed page, in each
        // dimension.

	virtual void DoCheckPrinter();
		// Check whether printer (if relevant) has changed, and if so, take appropriate
        // action

	virtual void DoDrawPrintFeedback(const CViewRect& area);
		// Draw page-breaks, page-numbers, view-borders, rulers, etc.

	virtual void DoDrawPageBreak(VHSelect vhs,
								 short whichBreak,
								 ViewCoordinate loc,
								 bool automatic);
		// Draw one page break.

	virtual void DoPagination();
		// Recompute the dividing lines between areas of the view which will be mapped
        // into different printed pages

	virtual void DoPrinterChanged();
		// The metrics relating to printer use have changed; absorb the information and
        // react

	virtual void DoSetPageOffset(const CViewPoint& coord);
		// Only reimplement in very unusual circumstances involving Tall-UnAdjusted text
        // printing etc.

	virtual CViewRect GetPrintExtent();
		// Returns the part of the view that is to be printed. The default is to return
        // the view's extent.

	virtual void PageInteriorChanged(const CViewRect& newInterior);
		// Allows a view which is printable to react to the changing of the page-interior
        // rectangle


	//------------------------------------------------------------------------------------
	// View Tracking Methods
	//------------------------------------------------------------------------------------

	virtual void GoAwayByUser(const CViewPoint& theMouse);
		// Provides tracking for the GoAway box (or other indicator) and calls CloseByUser
        // if go away was indicated.

	virtual void MoveByUser(const CViewPoint& theMouse);
		// Provides tracking for the Title bar (or other drag area) and calls Locate if
        // the location changed.

	virtual void ResizeByUser(const CViewPoint& theMouse);
		// Provides tracking for the Resize box (or other indicator) and calls Resize if
        // the size changed.

	virtual void ZoomByUser(const CViewPoint& theMouse, short partCode);
		// Provides tracking for the Zoom box (or other indicator) and calls Zoom if zoom
        // was indicated.


	//------------------------------------------------------------------------------------
	// Tracker Tracking Methods
	//------------------------------------------------------------------------------------

	virtual bool IsDoneTracking();
		// Indicates whether the Tracker is through tracking.(the command is also through
        // tracking if you return NULL from TrackMouse). Defaults to NOT StillDown. Most
        // programs won't have to mess with this. NOTE you still have to deal with queued
        // events if you change the criteria for true.

	virtual void TrackConstrain(TrackPhase aTrackPhase,
								const CViewPoint& anchorPoint,
								const CViewPoint& previousPoint,
								CViewPoint& nextPoint,
								bool mouseDidMove);
		// Override this if your Tracker calls back to the view (the Default) and you want
        // to constrain the mouse point to a grid, force drawing a square, etc. This is
        // called only if fConstrainsMouse is true.

	virtual void TrackFeedback(TrackPhase aTrackPhase,
							   const CViewPoint& anchorPoint,
							   const CViewPoint& previousPoint,
							   const CViewPoint& nextPoint,
							   bool mouseDidMove,
							   bool turnItOn);
		// Override this if your Tracker calls back to the view (the Default) and you want
        // to display any tracking feedback

	virtual void TrackMouse(TrackPhase aTrackPhase,
							CViewPoint& anchorPoint,
							CViewPoint& previousPoint,
							CViewPoint& nextPoint,
							bool mouseDidMove);
		// Override this if your Tracker calls back to the view (the Default) an you want
        // to display any tracking feedback


	//------------------------------------------------------------------------------------
	// View hierarchy management
	//------------------------------------------------------------------------------------
	
	virtual void AddedASubView(TView* newSubView);
		// Called by AddSubView to notify this view that a subview was added. Calls
        // fSuperView->AddedASubView so that the notification is passed on to all
        // superviews

	virtual void RemovedASubView(TView* theSubView);
		// Called by RemoveSubView to notify this view that a subview was removed. Calls
        // fSuperView->RemovedASubView so that the notification is passed on to all
        // superviews

	virtual void BeInSuperView(TView* itsSuperView);
		// Called to notify this view that it has become a subview of theSuperView.
        // theSuperView can be NULL. Called by TView::AddSubView and TView::RemoveSubView

	virtual void SetHLPenState(HLState fromHL, HLState toHL);
		// Set the pen state for highlighting by XOR, given the highlight transition. You
        // can use this if you use Paint<X> or Frame<X> or line drawing for your
        // highlighting.


	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	
	inline TView* GetNextView() const { return fNextView; }
	inline TView* GetSubView() const { return fSubView; }
	inline TView* GetSuperView() const { return fSuperView; }
	inline void SetNextView(TView* nextView) { fNextView = nextView; }
	inline void SetSubView(TView* subView) { fSubView = subView; }
	inline void SetSuperView(TView* superView) { fSuperView = superView; }

	inline CViewPoint GetSize() const { return fSize; }
	inline CViewPoint GetLocation() const { return fLocation; }
	inline CViewPoint GetTranslation() const { return fTranslation; }
	inline void SetTranslation(const CViewPoint& inTranslation ) { fTranslation = inTranslation; }

	inline TDocument* GetDocument() const { return fDocument; }
	inline void SetDocument(TDocument* inDocument) { fDocument = inDocument; }

	inline bool GetShown() const { return fShown; }
	inline TAdornerList* GetAdorners() const { return fAdorners; }
	inline SizeDeterminer GetSizeDeterminer(VHSelect vhs) const { return fSizeDeterminer[vhs]; }
	inline void SetHandlesCursor(bool handlesCursor) { fHandlesCursor = handlesCursor; }
	inline void SetLetsSubViewsHandleCursor(bool handlesCursor) { fLetsSubViewsHandleCursor = handlesCursor; }
	inline void SetHelpID(ResNumber inHelpID) { fHelpID = inHelpID; }
	inline void SetHelpIndex(short inHelpIndex) { fHelpIndex = inHelpIndex; }
	inline void SetCursorID(ResNumber inCursorID) { fCursorID = inCursorID; }
	inline void SetSizeDeterminer(VHSelect vhs, SizeDeterminer inSizeDeterminer) { fSizeDeterminer[vhs] = inSizeDeterminer; }

	inline void SetHandlesHelp(bool handlesHelp) { fHandlesHelp = handlesHelp; }
	inline void SetLetsSubViewsHandleHelp(bool subViewsHandleHelp) { fLetsSubViewsHandleHelp = subViewsHandleHelp; }
	
	inline long GetUserArea() const { return fUserArea; }
	inline void SetUserArea(long userArea) { fUserArea = userArea; }
	
	inline void SetAdornerList(TAdornerList* newAdornerList) { fAdorners = newAdornerList; }
	
	inline FrameInvalidateDeterminer GetFrameInvalidateDeterminer() const { return fFrameInvalidateDeterminer; }
	inline void SetFrameInvalidateDeterminer( FrameInvalidateDeterminer newFrameInvalidateDeterminer) { fFrameInvalidateDeterminer = newFrameInvalidateDeterminer; }

	inline void SetWantsToBeTarget(bool wantsToBeTarget) { fWantsToBeTarget = wantsToBeTarget; }

	inline bool GetOpacity() const { return fIsOpaque; }
	inline void SetOpacity(bool inOpacity) { fIsOpaque = inOpacity; }

	//------------------------------------------------------------------------------------
	// Drag and Drop Methods
	//------------------------------------------------------------------------------------
	
#if qDrag
	virtual bool DoSetDragCursor(const CViewPoint& localPoint, 
									RgnHandle cursorRegion,
									RgnHandle& dragCursorRegion);
		// Can be overridden to set the drag cursor and compute a drag cursor region. 
		// The default calls DoMakeDragCursorRegion and calls WillDrag, and sets the
		// cursor to the drag "open hand" if a mouse down would cause a drag. 
		// dragCursorRegion should be NULL on entry, and must be disposed after calling. 

	virtual short GetWillDragCursorID();
		// Get the resource id of the cursor that indicates that the mouse is
		// over draggable content.
		
	virtual short GetIsDraggingCursorID();
		// Get the resource id of the cursor that indicates that the user is dragging.
	
	virtual RgnHandle DoMakeDragCursorRegion();
		// Create and return a region in local coordinates that can be used
		// to test whether the mouse is in a location that would initiate a
		// drag.  This region is also used by DoSetCursor to create a cursor
		// sleep region
		
	virtual bool WillDrag(const CViewPoint& localMouse, const RgnHandle dragCursorRegion);
		// Returns true if the view wants to drag based on the mouse location
	
	virtual TView* DoGetDragProxy();
		// Dragging view may provide a proxy to manage the drag operation. Not
		// necessary in most cases.
		
	void HandleDrag(TToolboxEvent* event);
		// Initiate a drag operation

	virtual void DoAddDragContent();
		// Add drag items, flavors, and promises to the current drag operation

	virtual RgnHandle DoMakeDragOutlineRegion();
		// Create and return a region in local coordinates that can be drawn
		// to represent data being dragged

	virtual bool WillAcceptDrop(CDragItemIterator& dragItemIterator);
		// Return true if the view will accept the drop. 
		
	virtual void DoDragEnter();
		// Become the drop target. Set up private data necessary for tracking
		// the drag within the view.
		
	virtual RgnHandle DoMakeDropHiliteRegion();
		// Create and return a region in local coordinates that can be drawn
		// to represent that the view is droppable. 

	virtual void DoDragWithin(const CViewPoint& localMouse);
		// update visual drag tracking indication based on the mouse coords.
									
	virtual void DoDragLeave();
		// Clean up and remove any view-managed hiliting.
	
	virtual bool DoMakeDropLocationDescriptor(CAEDesc& dropLocationDesc);
		// return an AppleEvent descriptor describing the drop location.

	virtual bool WillDragMove(TView* destinationView);
		// If the drag initiated in this application, return true if data should
		// be moved instead of copied.

	virtual TCommand* DoMakeDragDropCommand(CommandNumber itsCommandNumber,
											CDragItemIterator& dragItemIterator);
		// Create a drag drop command that will be posted by the drag session.
	
	virtual void DoFulfillPromise(TDragItem* promisedItem);
		// Provide data for promised flavors
		
	//
	// If qDrag is false, these accessors will still be around but they won't do
	// anything. 
	//
	inline bool GetDraggable() const { return fDraggable; }

	inline void SetDraggable(bool isDraggable) { fDraggable = isDraggable; }
	
	inline bool GetDroppable() const { return fDroppable; }

	void SetDroppable(bool isDroppable);

	inline short GetDragMoveDeterminer() const { return fDragMoveDeterminer; }
	
	inline void SetDragMoveDeterminer(short dragMoveDeterminer) { fDragMoveDeterminer = dragMoveDeterminer; }
	
	inline IDType GetDragMoveFamily() const { return fDragMoveFamily; }
	
	inline void SetDragMoveFamily(IDType dragMoveFamily) { fDragMoveFamily = dragMoveFamily; }
#else
	inline bool GetDraggable() const { return false; }

	inline void SetDraggable(bool /*isDraggable*/) {  }
	
	inline bool GetDroppable() const { return false; }

	inline void SetDroppable(bool /*isDroppable*/) { }

	inline short GetDragMoveDeterminer() const { return kNeverMove; }
	
	inline void SetDragMoveDeterminer(short /*dragMoveDeterminer*/) { }
	
	inline IDType GetDragMoveFamily() const { return kNoIdentifier; }
	
	inline void SetDragMoveFamily(IDType /*dragMoveFamily*/) { }
#endif // qDrag


#if qBackwardCompatible
//----------------------------------------------------------------------------------------
// left in for compatibility
//----------------------------------------------------------------------------------------
	inline void QDToViewPt(CPoint_AC qdPoint, CViewPoint& viewPt) { viewPt = QDToView(qdPoint); }
	inline void QDToViewRect(const CRect_AC& qdRect, CViewRect& viewRect) { viewRect = QDToView(qdRect); }
	inline void ViewToQDRect(const CViewRect& viewRect, CRect_AC& qdRect) { qdRect = ViewToQD(viewRect); }
	inline CViewPoint QDToViewPt(CPoint_AC qdPoint) { return QDToView(qdPoint); }
	inline CViewRect QDToViewRect(const CRect_AC& qdRect) { return QDToView(qdRect); }
	inline CPoint_AC ViewToQDPt(const CViewPoint& viewPt) { return ViewToQD(viewPt); }
	inline CRect_AC ViewToQDRect(const CViewRect& viewRect) { return ViewToQD(viewRect); }

	inline void GetLocalOffsetInSuper(CViewPoint& localOffsetInSuper) const { localOffsetInSuper = GetLocalOffsetInSuper(); }

	inline void LocalToSuper(CViewPoint& viewPoint) const { viewPoint = LocalToSuperView(viewPoint); }
	inline void LocalToSuperVRect(CViewRect& viewRect) const { viewRect = LocalToSuperView(viewRect); }
	inline void LocalToSuperRegion(RgnHandle theRgn) { LocalToSuperView(theRgn); }
	inline void SuperToLocal(CViewPoint& viewPoint) const { viewPoint = SuperViewToLocal(viewPoint); }
	inline void SuperToLocalVRect(CViewRect& viewRect) const { viewRect = SuperViewToLocal(viewRect); }
	inline void SuperToLocalRegion(RgnHandle theRgn) { SuperViewToLocal(theRgn); }

    inline void LocalToGlobalRegion(RgnHandle theRgn) { LocalToGlobal(theRgn); }

	inline void LocalToWindow(CViewPoint& thePoint) const { thePoint = LocalToRootView(thePoint); }
    inline void LocalToWindowVRect(CViewRect& viewRect) const { viewRect = LocalToRootView(viewRect); }
    inline void LocalToWindowRegion(RgnHandle theRgn) { LocalToRootView(theRgn); }
	inline void WindowToLocal(CViewPoint& thePoint) const { thePoint = RootViewToLocal(thePoint); }
	inline void WindowToLocalVRect(CViewRect& viewRect) const { viewRect = RootViewToLocal(viewRect); }
	inline void WindowToLocalRegion(RgnHandle theRgn) { RootViewToLocal(theRgn); }

	inline void GetExtent(CViewRect& itsExtent) const { itsExtent = GetExtent(); }
	inline void GetQDExtent(CRect_AC& qdExtent) { qdExtent = GetQDExtent(); }
	inline void GetFrame(CViewRect& itsFrame) const { itsFrame = GetFrame(); }

	inline void InvalidateVRect(const CViewRect& badVRect) { Invalidate(badVRect); }
	inline void InvalidateRect(const CRect_AC& badRect) { Invalidate(badRect); }
	inline void InvalidateRegion(const RgnHandle badRgn) { Invalidate(badRgn); }
	inline void ValidateVRect(const CViewRect& goodVRect) { Validate(goodVRect); }
	inline void ValidateRect(const CRect_AC& goodRect) { Validate(goodRect); }
	inline void ValidateRegion(const RgnHandle goodRgn) { Validate(goodRgn); }

	inline void GetVisibleRect(CViewRect& visRect) { visRect = GetVisibleRect(); }
	inline void GetVisibleQDRect(CRect_AC& visQDRect) { visQDRect = GetVisibleQDRect(); }
	inline void GetDrawableRect(CViewRect& drawRect) { drawRect = GetDrawableRect(); }
	inline void GetDrawableQDRect(CRect_AC& drawQDRect) { drawQDRect = GetDrawableQDRect(); }
#endif

//----------------------------------------------------------------------------------------
// static functions
//----------------------------------------------------------------------------------------
public:

	static void InitUView(bool initGX = true);
		// Initialize the view system.
		// if initGX is TRUE then QuickDraw GX will be initialized

	static void TerminateUView();
		// Terminate the view system

	static void GetFocus(FocusRec& theFocusRec);
		// gets the current focus into a focusrec. The theFocusRec.itsPortInfo.clipRgn must be
		// a valid rgn.Useful for saving and restoring the focus behind MacApp's back.
	
	static void SetFocus(const FocusRec& theFocusRec);
		// sets the current focus from a focus rec
	
	static void GetViewPortInfo(ViewPortInfo& theViewPortInfo);
		// gets the current PortInfo into a ViewPortInfo. theViewPortInfo.clipRgn must be a
		// valid rgn.
	
	static void SetViewPortInfo(const ViewPortInfo& theViewPortInfo);
		// sets the current PortInfo from a ViewPortInfo
	
	static bool IsRectVisible(const CRect_AC& r);
		// Determine if a CRect_AC is visible in the current grafport.
	
	static bool IsRectDrawable(const CRect_AC& r);
		// Determine if any portion of a CRect_AC intersects the ports visRgn AND clipRgn.
	
	static void VisibleRect(CRect_AC& rectToRestrict);
		// trims the supplied CRect_AC down to its intersection with the port's visRgn
	
	static void DrawableRect(CRect_AC& rectToRestrict);
		// trims the supplied CRect_AC down to its intersection with the port's visRgn AND
		// clipRgn.
	
	static void VisibleRgn(RgnHandle rgn);
		// trims the supplied rgn down to its intersection with the port's visRgn
	
	static void DrawableRgn(RgnHandle rgn);
		// trims the supplied rgn down to its intersection with the port's visRgn AND
		// clipRgn.
	
	static bool IsPtVisible(CPoint_AC pt);
		// Determine if a pt intersects the ports visRgn
	
	static bool IsPtDrawable(CPoint_AC pt);
		// Determine if a pt intersects the ports visRgn AND clipRgn.
	
	inline static void PtAndRgn(CPoint_AC aPoint, RgnHandle aRegion) { CRegion_AC(aRegion) |= aPoint; }
		// Adds the given CPoint_AC to the given region
	
	inline static void VRectAndRgn(const CViewRect& aVRect, RgnHandle aRegion) { CRegion_AC(aRegion) |= aVRect; }
		// Call this procedure to add theVRect to theRgn.
	
#if qDebug
	static void WriteFocus();
		// Writes information about the current focus to the output
#endif

	static RgnHandle GetClipRegion(GrafPtr theGrafPtr);
		// Return the clipRgn

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CViewPoint fLocation;						// the location in superview coordinates

	CViewPoint fSize;							// the size of this view

	CViewPoint fTranslation;					// translation of coordinates required to
                                                // account for the origin of the view's
                                                // local coordinate system.

protected:
	CViewPoint fViewToQDOffset;					// the computed translation of coordinates
                                                // necessary to account for adjustment of
                                                // large views down to QD space.
												// Access only through GetViewToQDOffset!

	CViewRect fVisibleExtent;					// The extent of the view that could be
                                                // "visible" through all its superviews
                                                // (not taking clipping into account
												// Access only through GetVisibleExtent!

	CPoint_AC fQDOrigin;						// the computed QD origin for the port to
                                                // account for the view's location.
												// Access only through GetQDOrigin!

#if qBackwardCompatible
public:
#else
protected:
#endif
	TView* fSuperView;							// the current superview

	TView* fSubView;							// first of subviews of this view (linked list)

	TView* fNextView;							// first of siblings of this view (linked list)

	TDocument* fDocument;						// the associated document if any.

	CAutoPtr_AC<TDrawingEnvironment> fDrawingEnvironment;	// PenSize etc.

	CAutoPtr_AC<TAdornerList> fAdorners;			// The list of adorners

	long	fUserArea;							// assigned from view template for use
												// by client

	short	fTemplateVersion;					// version number of the template used to
												// instantiate this view

	short	  fHelpIndex;

	ResNumber fHelpID;

	ResNumber fCursorID;

	SizeDeterminer fSizeDeterminer[2];			// how this view's size is to be
                                                // determined

	HLState fActiveHL;							// the current highlighting state for this 
                                                // view in an active window. Typically hlOn.
	HLState fInactiveHL;						// the current highlighting state for this 
                                                // view in an inactive window. Typically 
												// hlOff or hlDim
	
	bool	fHandlesCursor;						// true if the view tracks the mouse for
                                                // cursor setting

	bool	fLetsSubViewsHandleCursor;			// true if subviews should be given a
                                                // chance to set the cursor

	bool	fHandlesHelp;						// true if the view tracks the mouse for
                                                // showing help balloons
                                                
    bool fHandlesFirstClicks;					// true if the view handles first clicks

	bool	fLetsSubViewsHandleHelp;			// true if subviews should be given a
                                                // chance to display help

	bool fShown;								// is this view to be shown

	bool fWantsToBeTarget;						// If true, then clicking in this view or tabbing
												// makes it the target.

	bool fCoordinatesAreInvalid;				// Set true by InvalidateCoordinates
												// Set false by UpdateCoordinates


	bool fIsOpaque;								// true if this view should be considered "opaque"
												// and none of the bits behind it will be visible
#if qDrag
  protected:
	bool	fDraggable;							// can the view be dragged 
	bool	fDroppable;							// can the view receive drops
	short	fDragMoveDeterminer;				// defines context for drag move
	IDType	fDragMoveFamily;					// defines dragmove family within specified context
#endif

#if qWriteTemplateViews
  protected:
	CTempHandle_AC	fTemplateUnknownData;		// extra data found after the end of ReadFields
#endif

	FrameInvalidateDeterminer	fFrameInvalidateDeterminer;
	
//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
public:

	static TView* gFocusedView;						
		// the view that is currently focused
		
	static TView* fgSetCursorView;						
		// the view that was responsible for the last setting of the cursor
		
	static bool gPrinting;						
		// True if currently Printing
	
	static bool gDrawingPictScrap;				
		// True if a view's Draw routine is being called in order to create PICT data in the
		// Desk Scrap; your View.Draw can check this routine, and insert PictComments as
		// appropriate if it wishes to have them in the Picture in the Desk Scrap
	  
	static TView* gDrawingPictScrapView;			
		// the view being currently drawn in the PICT scrap ??? this stuff is a good candidate
		// for a small change to the view architecture to more easily allow providing
		// alternate drawing environments

#if qGXViews || qGXPrinting	
	static gxGraphicsClient gGXGraphicsClient;
		// The main GX Graphic client (when used).
#endif
};

//----------------------------------------------------------------------------------------
// CSubViewIterator: A simple iterator for subviews.
//----------------------------------------------------------------------------------------

class EXPORT_MA CSubViewIterator : public CIterator_AC
{
public:
	CSubViewIterator(const TView* theSuperView, bool itsForward = true);

	//
	// Required functions 
	//
	TView* First();
	inline TView* Current() const { return fCurrent; }
	TView* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TView* operator++() { return Next(); }
	inline TView* operator++(int) { TView* returnVal = Current(); Next(); return returnVal; }
	inline operator TView*() const { return Current(); }
	inline TView* operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline TView* FirstSubView() { return First(); }
	inline TView* CurrentSubView() const { return Current(); }
	inline TView* NextSubView() { return Next(); }

protected:
	inline TView* NextView(const TView* aView) const { return aView->GetNextView(); }
		// returns the next subview in the subview list
		
	TView* PreviousView(const TView* aView) const;
		// returns the previous subview in the subview list

	TView* LastView() const;
		// returns the last subview in the subview list

private:
	TView* fCurrent;
	const TView* fFirstView;

protected:
	bool fIterateForward;
	
};

//----------------------------------------------------------------------------------------
// TIncludeView: specialized view which includes another view resource.
//----------------------------------------------------------------------------------------

class EXPORT_MA TIncludeView : public TView
{
public:
	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

	TIncludeView(short includeRsrcID = kNoResource,
					  const CViewPoint& itsSubViewOffset = gZeroVPt,
					  const CStr255_AC& resourceName = gEmptyString,
					  short resourceAttributes = resPurgeable);
		// Constructor

	virtual ~TIncludeView();
		// Destructor
		
	virtual void* Clone() const;
		// return a copy of the view by using the copy constructor
	
	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void DoPostCreate(TDocument* itsDocument);
	
	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	virtual void WriteSubViews(CStream_AC* aStream) const;
		// don't write any subviews

	//------------------------------------------------------------------------------------
	// Frame (Location and Size) Methods
	//------------------------------------------------------------------------------------

	virtual void SubViewChangedFrame(TView* theSubView,
									 const CViewRect& oldFrame,
									 const CViewRect& newFrame,
									 bool invalidate);
		// Called when one of our subviews changes frame.

	//------------------------------------------------------------------------------------
	// General accessors/manipulators
	//------------------------------------------------------------------------------------

	inline short GetIncludeRsrcID() const { return fIncludeRsrcID; }
	inline CViewPoint GetSubViewOffset() const { return fItsSubViewOffset; }
	
	inline void SetIncludeRsrcID(short newRsrcID) { fIncludeRsrcID = newRsrcID; }
	inline void SetSubViewOffset(const CViewPoint& subViewOffset) { fItsSubViewOffset = subViewOffset; }
	
//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CStr255_AC	fResourceName;
	
	CViewPoint	fItsSubViewOffset;				// offset of view in superview

	short	fIncludeRsrcID;						// the included view's rsrc id

	short	fResourceAttributes;
};

#endif
