//----------------------------------------------------------------------------------------
// UScheduleBehavior.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __USCHEDULEBEHAVIOR__
#include "UScheduleBehavior.h"
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __USCHEDULETRACKER__
#include "UScheduleTracker.h"
#endif

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __USECTION__
#include <USection.h>
#endif

#ifndef __UCLIPBOARDMGR__
#include <UClipboardMgr.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

//----------------------------------------------------------------------------------------
#pragma segment AInit

void InitUScheduleBehavior()
{
	// So the linker doesn't dead strip class info
	macroDontDeadStrip(TAppointmentSelector);
	macroDontDeadStrip(TAppointmentDragLauncher);
	macroDontDeadStrip(TAppointmentStretchLauncher);
	macroDontDeadStrip(TAppointmentSketchLauncher);
	macroDontDeadStrip(TSubscriberSketchLauncher);
	macroDontDeadStrip(TAppointmentChangeLauncher);

	macroDontDeadStrip(TScheduleEditLauncher);
	macroDontDeadStrip(TSubscriberEditLauncher);
}

//----------------------------------------------------------------------------------------
// CLASS TScheduleViewBehavior
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TBehavior
DefineClass(TScheduleViewBehavior, Inherited);

TScheduleViewBehavior::TScheduleViewBehavior()
{
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TScheduleView* TScheduleViewBehavior::GetScheduleView()
{
	return (TScheduleView*)fOwner;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSelector
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TAppointmentSelector, Inherited);

bool TAppointmentSelector::DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint hysteresis)	// OVERRIDE
{
	// Handles clicking on an appointment to select it.
	// Works on the top-most appointment under the mouse point.
	// Shift-clicking inverts the selection state. Can extend selection.
	// Non-shift-clicking will deselect all other appointments, similar to a text selection.
	// When an appointment is selected, control passes along behavior chain.
	// When an appointment is deselected, mouse handling terminates.
	// Appointments are highlighted immediately.

	TScheduleView* itsView = this->GetScheduleView();
	TAppointment* topAppointment = itsView->GetTopAppointment(theMouse);

	if (topAppointment)
	{
		if (event->IsShiftKeyPressed())		// extend the selection
		{
			itsView->HighlightAppointment(topAppointment, topAppointment->InvertSelect());
			if (!topAppointment->IsSelected())
				return true;				// if we deselected it, absorb the click. Otherwise, pass it on.
		}
		else								// single selection
		{
			// Deselect all other appointments
			CSelectedAppointmentIterator otherIter(itsView);
			for (TAppointment* selected = otherIter.FirstAppointment(); otherIter.More();
					selected = otherIter.NextAppointment())
				if (selected != topAppointment)
					itsView->HighlightAppointment(selected, selected->SetSelect(kDeSelect));

			if (!topAppointment->IsSelected())
				itsView->HighlightAppointment(topAppointment, topAppointment->SetSelect(kSelect));
		}
	}

	return Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentDragLauncher
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TAppointmentDragLauncher, Inherited);

bool TAppointmentDragLauncher::DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint hysteresis)	// OVERRIDE
{
	// Finds the top-most appointment under the mouse, and if present, launches an
	// appointment dragger.

	TScheduleView* itsView = this->GetScheduleView();
	TAppointment* topAppointment = itsView->GetTopSelectedAppointment(theMouse);

	if (topAppointment)
	{
		itsView->PostCommand(TH_new TAppointmentDragger(itsView, theMouse, topAppointment));
		return true;
	}

	return Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentStretchLauncher
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TAppointmentStretchLauncher, Inherited);

bool TAppointmentStretchLauncher::DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint hysteresis)	// OVERRIDE
{
	// Finds the top-most appointment whose knob contains the mouse.
	// If one is found, this method launches an appointment stretcher for it.

	TScheduleView* itsView = this->GetScheduleView();
	CSelectedAppointmentIterator iter(itsView);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
	{
		KnobSelector whichKnob;
		if (itsView->MouseInKnob(anAppointment, theMouse, whichKnob))
		{
			itsView->PostCommand(TH_new TAppointmentStretcher(itsView, theMouse, anAppointment, whichKnob));
			return true;
		}
	}
	return Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSketchLauncher
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TAppointmentSketchLauncher, Inherited);

bool TAppointmentSketchLauncher::DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* /* event */,
										  CPoint /* hysteresis */)	// OVERRIDE
{
	// If mouse is not down in an appointment, deselects all appointments
	// and launches an appoinment sketcher.
	// In any event, it terminates mouse-down handling.

	TScheduleView* itsView = this->GetScheduleView();
	TAppointment* topAppointment = itsView->GetTopAppointment(theMouse);

	if (!topAppointment)	// if the mouse was not in any appointment, sketch a new one.
	{
		itsView->SetEmptySelection(kHighlight);		// Deselect all selected appointmnts
		this->LaunchSketcher(itsView, theMouse);
	}
	return true;
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TAppointmentSketchLauncher::LaunchSketcher(TScheduleView* itsView,
												const VPoint theMouse)
{
	// Launches an appoinment sketcher.

	itsView->PostCommand(TH_new TAppointmentSketcher(itsView, theMouse));
}

//----------------------------------------------------------------------------------------
// CLASS TSubscriberSketchLauncher
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TAppointmentSketchLauncher
DefineClass(TSubscriberSketchLauncher, Inherited);

void TSubscriberSketchLauncher::LaunchSketcher(TScheduleView* /* itsView */,
												const VPoint /* theMouse */)	// OVERRIDE
{
	// Can't sketch in a subscriber view.
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentChangeLauncher
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TAppointmentChangeLauncher, Inherited);

void TAppointmentChangeLauncher::DoMenuCommand(CommandNumber aCommandNumber)	// OVERRIDE
{
	// Handles the change appointment command by launching a command changer
	// for the selected appointment.

	switch (aCommandNumber)
	{
	case cChangeAppointment:
		{
			TScheduleView* itsView = this->GetScheduleView();

			itsView->PostCommand(TH_new TAppointmentChanger(itsView, itsView->GetSelectedAppointment()));
			return;
		}
		break;

	default:
		Inherited::DoMenuCommand(aCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

bool TAppointmentChangeLauncher::DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint hysteresis)	// OVERRIDE
{
	// Handles a double-click by determining the top-moust appointment hit
	// and, if present, launches an appointment changer for it.

	if (event->fClickCount == 2)
	{
		TScheduleView* itsView = this->GetScheduleView();

		TAppointment* topAppointment = itsView->GetTopSelectedAppointment(theMouse);

		if (topAppointment)
		{
			itsView->PostCommand(TH_new TAppointmentChanger(itsView, topAppointment));
			return true;
		}
	}
	return Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TAppointmentChangeLauncher::DoSetupMenus()		// OVERRIDE
{
	// Enables the change appointment menu item if any appointment is selected.

	Inherited::DoSetupMenus();

	TScheduleView* itsView = this->GetScheduleView();

	Enable(cChangeAppointment, itsView->HaveASelectedAppointment());

}

//----------------------------------------------------------------------------------------
// CLASS TScheduleEditLauncher
//----------------------------------------------------------------------------------------
#pragma segment MARes

#undef Inherited
#define Inherited TScheduleViewBehavior
DefineClass(TScheduleEditLauncher, Inherited);

void TScheduleEditLauncher::DoKeyEvent(TToolboxEvent* event)
{
	// Launches a clear command if the backspace character is pressed and clear is enabled.
	// MacApp doesn't do this automatically.

	if (event && (event->fCharacter == chBackspace) && CommandEnabled(cClear))
		this->DoMenuCommand(cClear);
	else
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

void TScheduleEditLauncher::DoMenuCommand(CommandNumber aCommandNumber)	// OVERRIDE
{
	// Handles clipboard commands for appointments.

	TScheduleView* itsView = this->GetScheduleView();
	switch (aCommandNumber)
	{
	case cCut:
	case cCopy:
	case cClear:
		{
			itsView->PostCommand(TH_new TAppointmentCutCopyClearCommand(aCommandNumber, itsView));
		}
		break;

	case cPaste:
		{
			itsView->PostCommand(TH_new TAppointmentPasteCommand(aCommandNumber, itsView));
		}
		break;

	case cSelectAll:
		{
			CAppointmentIterator iter(itsView);
			for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
					anAppointment = iter.NextAppointment())
				if (!anAppointment->IsSelected())
					itsView->HighlightAppointment(anAppointment, anAppointment->SetSelect(kSelect));
		}
		break;

	default:
		Inherited::DoMenuCommand(aCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleEditLauncher::DoSetupMenus()		// OVERRIDE
{
	// Calls a method that can be easily overridden by subclasses.

	Inherited::DoSetupMenus();

	this->SetupEditMenu();
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleEditLauncher::SetupEditMenu()
{
	// Enables edit commands for its view's appointments.

	TScheduleView* itsView = this->GetScheduleView();
	bool haveMemory = !MemSpaceIsLow();

	bool anySelected	= itsView->HaveAnySelectedAppointments();

	Enable(cCut, anySelected  && haveMemory);
	Enable(cCopy, anySelected && haveMemory);

	if (haveMemory)
		gClipboardMgr->CanPaste(itsView->GetScheduleDocument()->fScrapType);

	Enable(cClear, anySelected);

	Enable(cSelectAll, itsView->HaveAnyAppointments());

}

//----------------------------------------------------------------------------------------
// CLASS TSubscriberEditLauncher
//----------------------------------------------------------------------------------------
#pragma segment ARes

#undef Inherited
#define Inherited TScheduleEditLauncher
DefineClass(TSubscriberEditLauncher, Inherited);

void TSubscriberEditLauncher::SetupEditMenu()		// OVERRIDE
{
	// Only allows copy and select all clipboard commands. Replaces superclass's version.

	TScheduleView* itsView = this->GetScheduleView();
	bool haveMemory = !MemSpaceIsLow();

	bool anySelected	= (itsView->HaveAnySelectedAppointments());

	Enable(cCopy, anySelected && haveMemory);
	Enable(cSelectAll, itsView->HaveAnyAppointments());

}

