//----------------------------------------------------------------------------------------
// UScheduleCommand.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __USCHEDULEAPPLICATION__
#include "UScheduleApplication.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __UGRIDVIEW__
#include <UGridView.h>
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __PPCTOOLBOX__
#include <PPCToolbox.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __EPPC__
#include <EPPC.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UDIALOGBEHAVIOR__
#include <UDialogBehavior.h>
#endif

#ifndef __UCLIPBOARDMGR__
#include <UClipboardMgr.h>
#endif

#ifndef __UERRORMGR__
#include <UErrorMgr.h>
#endif

#ifndef __UAPPLEEVENTS__
#include <UAppleEvents.h>
#endif

//----------------------------------------------------------------------------------------

// AppleEvent identifiers
const OSType kAEScheduleSuiteID			= 'SKDL';
const OSType kAELookupID				= 'LKUP';

// AppleEvent request parameter identifiers
const OSType kAEDay						= 'DAY ';
const OSType kAETime					= 'TIME';

// Response parameters
const OSType kAEDuration				= 'DUR ';
const OSType kAEDocument				= 'DOC ';

/* Buzz String */

const ResNumber kLookupBuzz				= 1012;
const short kAppointmentAvailable		= 1;
const short kPPCPrompt					= 2;
const short kPPCType					= 3;

/* Custom error result */
const OSErr errAppointmentNotAvailable	= -26001;

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

static pascal Boolean MyPPCBrowserFilter(LocationNamePtr /* theLocation */, PortInfoPtr thePortInfo)
/*
 *	MyPPCBrowserFilter
 *
 *	Thanks to Eric Berhdahl & Eric Soldan and his Kibitz application for the tips
 *	on how to implement this function.
 */
{
	OSType	type;

	if (thePortInfo->name.portKindSelector == ppcByString)
	{
		::BlockMove(thePortInfo->name.u.portTypeStr + 1, Ptr(&type), sizeof(type));
			// The BlockMove is so that we don't get an address error
			// on a 68000-based machine due to referencing a long at
			// an odd-address.
		if (type == kSignature)
			return true;
	}

	return false;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentEditCommand
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TCommand

TAppointmentEditCommand::~TAppointmentEditCommand()					// OVERRIDE
{
	// Frees the clip appointments if appropriate, then frees the appointment list.

	if (fClipAppointments && this->ShouldFreeClipAppointments())
		FREEALL(fClipAppointments, TAppointment*);

	fClipAppointments = (TAppointmentList*)FreeIfObject(fClipAppointments);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentEditCommand::GetClipAppointments()
{
	// Prepares to edit by getting creating and filling
	// the list of clip appointments.

	// It is better to defer allocating the list until DoIt time, because there is
	// more memory available, since segments are unloaded before commands are done.

	// A failure handler is not necessary here, because this is
	// not an initialization method. If a failure occurs, the command handler
	// executing this command will free this object.

	// Subclasses should call Inherited before doing their thing.

	fClipAppointments = NewAppointmentList();

}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentEditCommand::TAppointmentEditCommand(CommandNumber aCommandNumber, TScheduleView* itsView,
				bool causesChange, bool changesClipboard) :
#ifdef MACAPP_R_13
	TCommand(aCommandNumber, fScheduleDocument, kCanUndo, causesChange, fScheduleDocument),
#endif
	fScheduleView(itsView),
	fScheduleDocument(itsView->GetScheduleDocument()),
	fClipAppointments(NULL)
{
#ifndef MACAPP_R_13
	ICommand(aCommandNumber, fScheduleDocument, kCanUndo, causesChange, fScheduleDocument);
#endif

	// This data member must be set here because it is not passed as a parameter
	// to ICommand.
	fChangesClipboard	= changesClipboard;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentEditCommand::RemoveClipAppointments()
{
	// Deletes the clip appointments from the Schedule document.

	CAppointmentIterator iter(fClipAppointments);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		 fScheduleDocument->DeleteAppointment(anAppointment);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentEditCommand::RestoreClipAppointments()
{
	// Adds the clip appointments to the Schedule document. Does the best it can
	// to recover from failures that may occur adding appointments.

	fScheduleView->SetEmptySelection(kHighlight);		// clear current selection before adding appointments.

	TRY
	{
		{
			CAppointmentIterator iter(fClipAppointments);
			for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
					anAppointment = iter.NextAppointment())
			{
				anAppointment->SetSelect(kSelect);		// newly added appointments are always selected.
				fScheduleDocument->AddAppointment(anAppointment);
			}
		}

		fScheduleView->ScrollSelectionIntoView(kRedraw);
	}
	CATCH_ALL
	{
		{
			CAppointmentIterator iter(fClipAppointments);
			for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
					anAppointment = iter.NextAppointment())
				fScheduleDocument->DeleteAppointment(anAppointment);
		}
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

bool TAppointmentEditCommand::ShouldFreeClipAppointments()
{
	// Allows subclasses to indicate whether the clip appointments
	// should be freed.

	SubClassResponsibility();
	return false;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentCutCopyClearCommand
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TAppointmentEditCommand

void TAppointmentCutCopyClearCommand::DoIt()	// OVERRIDE
{
	// Prepares to edit by getting a list of the currently selected appointments,
	// copying them to the clipboard if necessary.
	// Then calls RedoIt to do the action appropriate for the command identifier.

	this->GetClipAppointments();				// Get the selected appointments.

	if (fIdentifier != cClear)					// if not clear,
		this->PutAppointmentsToClip();			// put clones of the selection into the clipboard

	this->RedoIt();								// Do the main action.
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentCutCopyClearCommand::GetClipAppointments()		// OVERRIDE
{
	// Fills clip list with the document's currently selected appointments.

	Inherited::GetClipAppointments();

	CSelectedAppointmentIterator iter(fScheduleView);		// get appointments from view.
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		 fClipAppointments->Insert(anAppointment);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentCutCopyClearCommand::TAppointmentCutCopyClearCommand(CommandNumber aCommandNumber, TScheduleView* itsView) :
		TAppointmentEditCommand(aCommandNumber, itsView,
		aCommandNumber != cCopy /* causesChange */, aCommandNumber != cClear /* changesClipboard */)

{
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentCutCopyClearCommand::PutAppointmentsToClip()
{
	// Creates a clip document fills it with clones of the clip appointments,
	// then creates a clip view to display them, which is passed to
	// MacApp's clip manager.

	MAVolatileInit(TScheduleDocument*, clipDoc, new TScheduleDocument(NULL));		// create a clip document
							// initialize it.

	MAVolatileInit(TAppointment*, clonedAppointment, NULL);

	TRY
	{
		{													// establish context for CAppointmentIterator
			CAppointmentIterator iter(fClipAppointments);
			for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
					anAppointment = iter.NextAppointment())
			{
				clonedAppointment = (TAppointment*)anAppointment->Clone();
				clonedAppointment->SetSelect(kDeSelect);	// Clipboard appointments are never selected.
				clipDoc->AddAppointment(clonedAppointment);
				clonedAppointment = NULL;
			}
		}
		this->ClaimClipboard(clipDoc->MakeClipboardView());	// install the clipboard view.
	}
	CATCH_ALL
	{
		clonedAppointment	= (TAppointment*)FreeIfObject(clonedAppointment);
		clipDoc				= (TScheduleDocument*)FreeIfObject(clipDoc);

		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentCutCopyClearCommand::RedoIt()	// OVERRIDE
{
	// Does the command's action, based on its identifier.

	if (fIdentifier != cCopy)			// if cut or clear,
		this->RemoveClipAppointments();		// remove the appointments from the document.
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

bool TAppointmentCutCopyClearCommand::ShouldFreeClipAppointments()		// OVERRIDE
{
	// Frees the appointments if the cut, or clear command was done.

	return fCommandDone && fIdentifier != cCopy;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentCutCopyClearCommand::UndoIt()				// OVERRIDE
{
	// If undoing a cut or clear, restore the appointments to the document.

	if (fIdentifier != cCopy)
		this->RestoreClipAppointments();
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentPasteCommand
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TAppointmentEditCommand

void TAppointmentPasteCommand::DoIt()		// OVERRIDE
{
	// Prepares to paste, then calls RedoIt to take the action.

	this->GetClipAppointments();					// get appointments to paste
	this->RedoIt();									// do the paste.
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentPasteCommand::GetClipAppointments()			// OVERRIDE
{
	// Fills list of appointments to paste
	// with clones of those in the clipboard.

	Inherited::GetClipAppointments();

	MAVolatileInit(TAppointment*, clonedAppointment, NULL);

	TRY
	{
		{										// establish context for CAppointmentIterator
			CAppointmentIterator iter((TScheduleDocument*)gClipboardMgr->fClipView->fDocument);
			for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
					anAppointment = iter.NextAppointment())
			{
				clonedAppointment = (TAppointment*)anAppointment->Clone();
				fClipAppointments->Insert(clonedAppointment);
				clonedAppointment = NULL;
			}
		}
	}
	CATCH_ALL
	{
		clonedAppointment = (TAppointment*)FreeIfObject(clonedAppointment);
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentPasteCommand::TAppointmentPasteCommand(CommandNumber aCommandNumber, TScheduleView* itsView) :
		TAppointmentEditCommand(aCommandNumber, itsView,
		true /* causesChange */, false /* changesClip */)
{
	// Passes proper pasting command parameters to base class

}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentPasteCommand::RedoIt()		// OVERRIDE
{
	// Adds appointments cloned from the clipboard to the document.

	this->RestoreClipAppointments();
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

bool TAppointmentPasteCommand::ShouldFreeClipAppointments()		// OVERRIDE
{
	// If didn't paste, free the appointments in the list.

	return !fCommandDone;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentPasteCommand::UndoIt()				// OVERRIDE
{
	// Removes pasted appointments.

	this->RemoveClipAppointments();
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentAdder
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TCommand

void TAppointmentAdder::DoIt()		// OVERRIDE
{
	// Prepares to do the addition by creating the new appointment
	// from information in the data members, then calls RedoIt to add the
	// appointment. It is better to allocate memory in the DoIt method
	// than in the IAppointmentAdder method, because there is more free memory
	// around when the command is done than when it is initialized.

	fAppointment = TH_new TAppointment(fDay, fTime, fDuration, fDescription);		// create the new appointment

	
	this->RedoIt();							// add it to the document.
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

TAppointmentAdder::~TAppointmentAdder()		// OVERRIDE
{
	// Frees the appointment, if necessary.

	// !!! Test if fAppointment belongs to fScheduleDocument, if not, we still own it.
	// Multiple-level undo causes us to leak here at app quit time.
	if (!fCommandDone)
		fAppointment = (TAppointment*)FreeIfObject(fAppointment);	// if the command was not done, we still own the appointment.
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentAdder::TAppointmentAdder(CommandNumber aCommandNumber, TScheduleView* itsView,
					DayIndex itsDay, TimeIndex itsTime, TimeIndex itsDuration,
					CStr255& itsDescription) :
#ifdef MACAPP_R_13
	TCommand(aCommandNumber, itsView->GetScheduleDocument(), kCanUndo, kCausesChange, itsView->GetScheduleDocument()),
#endif
	fScheduleView(itsView),
	fScheduleDocument(itsView->GetScheduleDocument()),
	fAppointment(NULL),
	fDay(itsDay),
	fTime(itsTime),
	fDuration(itsDuration),
	fDescription(itsDescription)
{
#ifndef MACAPP_R_13
	ICommand(aCommandNumber, itsView->GetScheduleDocument(), kCanUndo, kCausesChange, itsView->GetScheduleDocument());
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentAdder::RedoIt()		// OVERRIDE
{
	// Actually adds the appointment to the document.

	// Clear existing selection before adding an appointment.
	fScheduleView->SetEmptySelection(kHighlight);

	// Newly added appointments are always selected.
	fAppointment->SetSelect(kSelect);

	fScheduleDocument->AddAppointment(fAppointment);
	fScheduleView->ScrollSelectionIntoView(kRedraw);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentAdder::UndoIt()		// OVERRIDE
{
	// Removes the appointment from the document.

	fScheduleDocument->DeleteAppointment(fAppointment);
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentChanger
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TCommand

void TAppointmentChanger::DoIt()				// OVERRIDE
{
	// Prepares to add an appointment by getting its new description.
	// Then calls RedoIt to make the change.

	// Get the dialog window
	TWindow* aWindow = gViewServer->NewTemplateWindow(phGetDescription, NULL);
	FailNIL(aWindow);

	// Get the edit text item.
	TEditText* anEditText = (TEditText*)(aWindow->FindSubView('DESC'));

	CStr255 oldDescription(fAppointment->GetDescription());	// Get the starting description

	anEditText->SetText(oldDescription, kDontRedraw);

	if (aWindow->PoseModally() == 'OK  ')			// run the dialog
	{
		CStr255 newDescription;						// user accepted dialog.

		anEditText->GetText(newDescription);		// get the appoinment

#ifdef MACAPP_R_13
		aWindow->Close();						// close it.
		delete aWindow;
#else
		aWindow->CloseAndFree();
#endif
		
		fOriginalDescription	= oldDescription;	// fill in data members with old and new
		fNewDescription			= newDescription;
		this->RedoIt();								// do the change.
	}
	else
	{
#ifdef MACAPP_R_13
		aWindow->Close();						// close it.
		delete aWindow;
#else
		aWindow->CloseAndFree();
#endif
		Failure(0,0);								// and fail silently.
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentChanger::TAppointmentChanger(TScheduleView* itsView, TAppointment* anAppointment) :
#ifdef MACAPP_R_13
	TCommand(cChangeAppointment, itsView->GetScheduleDocument(), kCanUndo, kCausesChange, itsView->GetScheduleDocument()),
#endif
	fAppointment(anAppointment)
{
#ifndef MACAPP_R_13
	ICommand(cChangeAppointment, itsView->GetScheduleDocument(), kCanUndo, kCausesChange, itsView->GetScheduleDocument());
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentChanger::RedoIt()		// OVERRIDE
{
	// Changes the appointment's description to the new version.

	fAppointment->SetDescription(fNewDescription);
	fAppointment->Changed(fIdentifier, this);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentChanger::UndoIt()		// OVERRIDE
{
	// Changes the appointment's description back to the original version.

	fAppointment->SetDescription(fOriginalDescription);
	fAppointment->Changed(fIdentifier, this);
}

//----------------------------------------------------------------------------------------
// CLASS TLookupLauncher
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TCommand

void TLookupLauncher::DoIt()	// OVERRIDE
{
	// This command runs the PPCBrowser dialog. If not canceled by the user,
	// it opens the modeless Lookup dialog.

	PortInfoRec		thePortInfo;
	TargetID		theTargetID;
	TLookupDialog*	theDialog;
	PPCFilterUPP	pfUPP = NULL;
	CStr255			thePrompt;
	CStr255			theType;

	::GetIndString(thePrompt, kLookupBuzz, kPPCPrompt);
	::GetIndString(theType, kLookupBuzz, kPPCType);


	// use PPCBrowser to locate the schedule to use.
	
	{ // scoped because there would otherwise be more than one try block in the same scope
		TRY
		{
			pfUPP = NewPPCFilterProc(MyPPCBrowserFilter);
			FailOSErr(::PPCBrowser(thePrompt, theType, false, &theTargetID.location, &thePortInfo,
								   pfUPP, gEmptyString));
			DisposeRoutineDescriptor(pfUPP);
		}
		CATCH_ALL
		{
			DisposeRoutineDescriptor(pfUPP);
			if (fi.error == userCanceledErr)	// if user cancelled,
				fi.error = noErr;				// fail silently.
			THROW;
		}
		ENDTRY; 
	}
	theTargetID.name = thePortInfo.name;

	// got it, so create the (modeless) lookup window. (It will free itself on closing.)

	MAVolatileInit(TWindow*, theWindow, gViewServer->NewTemplateWindow(phLookupDialog, NULL));
	FailNIL(theWindow);
	theDialog = (TLookupDialog*)(theWindow->FindSubView('DLOG'));

	{ // scoped because there would otherwise be more than one try block in the same scope
		TRY
		{
			theDialog->HaveAddress(theTargetID);		// register the target

			theWindow->SetTitleForDoc(thePortInfo.name.name);

			theWindow->Open();
			theWindow->Select();
		}
		CATCH_ALL
		{
			FreeIfObject(theWindow);
			THROW;
		}
		ENDTRY;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TLookupLauncher::TLookupLauncher(CommandNumber aCommandNumber)
#ifdef MACAPP_R_13
	: TCommand(aCommandNumber, NULL, kCantUndo, kDoesNotCauseChange, NULL)
#endif
{
#ifndef MACAPP_R_13
	ICommand(aCommandNumber, NULL, kCantUndo, kDoesNotCauseChange, NULL);
#endif
}

//----------------------------------------------------------------------------------------
// CLASS TScheduleClientCommand
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TClientCommand

void TScheduleClientCommand::DoIt()	// OVERRIDE
{
	if (fDuration)
	{
		CStr255 aString(GetDay(fDay));
		MAParamText("^LookupDay", aString);

		aString = GetTime(fTime);
		MAParamText("^LookupTime", aString);

		aString = GetDuration(fDuration);
		MAParamText("^LookupDuration", aString);

		MAParamText("^LookupDocument", fDocumentTitle);

		::GetIndString(aString, kLookupBuzz, 1);
		MAReplaceText(aString);

		((TLookupDialog*)fContext)->HaveResponse(aString);	// show the result in the window
	}
	else
	{
		((TLookupDialog*)fContext)->HaveResponse(gEmptyString);		// clear the response
		Failure(errAppointmentNotAvailable, messageLookup + errAppointmentNotAvailable);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TScheduleClientCommand::TScheduleClientCommand(CommandNumber itsCommandNumber,
													TLookupDialog* itsView,
													const DayIndex theDay,
													const TimeIndex theTime,
													const AEAddressDesc& itsAddress) :
#ifdef MACAPP_R_13
	TClientCommand(itsCommandNumber, itsView, kCantUndo, kDoesNotCauseChange, NULL),
#endif
	fDay(theDay),
	fTime(theTime),
	fDuration(0),
	fDocumentTitle(gEmptyString)				
{
	// Prepares the TAppleEvent for transmission to the server.
#ifndef MACAPP_R_13
	IClientCommand(itsCommandNumber, itsView, kCantUndo, kDoesNotCauseChange, NULL);
	TAppleEvent* aMessage = TH_new TAppleEvent;
	aMessage->IAppleEvent(kAEScheduleSuiteID, kAELookupID, itsAddress, kAEQueueReply);
#else
	TAppleEvent* aMessage = TH_new TAppleEvent(kAEScheduleSuiteID, kAELookupID, itsAddress, kAEQueueReply);
#endif
	fMessage = aMessage;

	fMessage->WriteShort(kAEDay, theDay);
	fMessage->WriteShort(kAETime, theTime);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TScheduleClientCommand::ProcessReply(TAppleEvent* theReply)		// OVERRIDE
{
	// Processes the reply from the server. Loads members with results of lookup.

	Inherited::ProcessReply(theReply);

	TRY
	{
		FailOSErr(theReply->ReadShort(keyErrorNumber));
		ENDTRY;
	}
	CATCH_ALL
	{
		if (fi.error != errAEDescNotFound) {
			THROW;
		}
	}

	TimeIndex theDuration = theReply->ReadShort(kAEDuration);
	fDuration = theDuration;

	CStr255 documentTitle;
	theReply->ReadString(kAEDocument, documentTitle);
	fDocumentTitle = documentTitle;
}

//----------------------------------------------------------------------------------------
// CLASS TScheduleServerCommand
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TServerCommand

TScheduleServerCommand::TScheduleServerCommand(CommandNumber itsCommandNumber,
						APPLEEVENTPTR itsMessage,
						APPLEEVENTPTR itsReply)
#ifdef MACAPP_R_13
		: TServerCommand(itsCommandNumber, NULL, kCantUndo, kDoesNotCauseChange, NULL, itsMessage, itsReply) 
#endif
{
#ifndef MACAPP_R_13
	InitializeFromAppleEvent(itsCommandNumber, NULL, kCantUndo, kDoesNotCauseChange, NULL, itsMessage, itsReply);
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TScheduleServerCommand::DoIt()	//	OVERRIDE
{
	// In response to a lookup request from the client, finds the document
	// that can accommodate the appointment, if any. Prepares the reply to
	// return to the client.

	DayIndex	theDay	= fMessage->ReadShort(kAEDay);
	TimeIndex	theTime	= fMessage->ReadShort(kAETime);

	TimeIndex	theDuration;
	CStr255		documentTitle;

	MA_DYNAMIC_CAST(TScheduleApplication, fContext)->IsAppointmentAvailable(theDay, theTime, theDuration, documentTitle);

	// have the application look it up

	fReply->WriteShort(kAEDuration,		theDuration);
	fReply->WriteString(kAEDocument, 	documentTitle);
}
