//----------------------------------------------------------------------------------------
// UScheduleDocument.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UAPPLICATION__
#include <UApplication.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __USCHEDULEPRINTING__
#include "USchedulePrinting.h"
#endif

#ifndef __USCHEDULEVIEW__
#include "UScheduleView.h"
#endif

#ifndef __UTEMPORARYOBJECTS__
#include "UTemporaryObjects.h"
#endif

#ifndef __USECTION__
#include <USection.h>
#endif

#ifndef __USYNCHSCROLLER__
#include <USynchScroller.h>
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __LIMITS__
#include <Limits.h>
#endif

#ifndef __UDEPENDENCIES__
#include <UDependencies.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __UCLIPBOARDMGR__
#include <UClipboardMgr.h>
#endif


//----------------------------------------------------------------------------------------

// Document State resource

const OSType	kDocStateType	= 'INFO';
const ResNumber	kDocStateID		= 1001;
const CStr255	kDocStateName	= "Document State";

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

#undef Inherited
#define Inherited TEditionDocument

void TScheduleDocument::AddAppointment(TAppointment* anAppointment)
{
	// Inserts an appointment into the document and establishes relevant dependencies.

	Assertion(fAppointments->GetIdentityItemNo(anAppointment) == kEmptyIndex, "Appointment already in fAppointments");

	fAppointments->Insert(anAppointment);

	anAppointment->AddDependent(fAppointments);				// the list depends on the appointment.
	anAppointment->Changed(cAddAppointment, this);			// inserting the appointment changes it.

	// if this appointment being added to a publisher, the publisher is dependent on it.
	CSectionIterator iter(this);
	for (TSection * aSection = iter.FirstSection(); iter.More(); aSection = iter.NextSection())
	{
		if (aSection->GetSectionType() == stPublisher &&
			GetDayDesignator(aSection)->Contains(anAppointment))
				anAppointment->AddDependent(aSection);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::AddSection(TSection* aSection)			// OVERRIDE
{
	// Establishes relevant dependencies between new publishers and the appointments in them.

	Inherited::AddSection(aSection);

	// When a publisher is added, it depends on all published appointments.
	if (aSection->GetSectionType() == stPublisher)
	{
		CAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
			if (GetDayDesignator(aSection)->Contains(anAppointment))
				anAppointment->AddDependent(aSection);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleDocument::AddSubscribedAppointment(TAppointment* anAppointment)
{
	// Adds a subscribed appointment to the document and establishes relevant dependencies.

	Assertion(fSubscribedAppointments->GetIdentityItemNo(anAppointment) == kEmptyIndex,
			  "Appointment already in fSubscribedAppointments");

	fSubscribedAppointments->Insert(anAppointment);

	anAppointment->AddDependent(fSubscribedAppointments);	// the list depends on the appointment.
	anAppointment->Changed(cAddAppointment, this);			// inserting the appointment changes it.

}

//----------------------------------------------------------------------------------------
#pragma segment MADocumentRes

bool TScheduleDocument::CanPublishSelection()		// OVERRIDE
{
	// User must have a day selected from the fDayView to be able to create a publisher.

	return (gApplication->GetTarget() == fDayView) && Inherited::CanPublishSelection();
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

bool TScheduleDocument::CanSubscribe()		// OVERRIDE
{
	// Enables subscribing if the target is the subscriber day
	// and there is not a subscriber in the document already.

	return gApplication->GetTarget()->fIdentifier == kSubscriberDayID && !this->GetTheSubscriber();
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleDocument::DeleteAppointment(TAppointment* anAppointment)
{
	// Deletes an appointment from the document, handling changing and dependencies.

	Assertion(fAppointments->GetIdentityItemNo(anAppointment) != kEmptyIndex, "Appointment is not in fAppointments");

	anAppointment->Changed(cDeleteAppointment, this);			// deleting changes the item.
	anAppointment->RemoveDependent(fAppointments);				// list no longer depends on this appointment

	fAppointments->Delete(anAppointment);

	// if the appointment being removed is in a publisher, remove the dependency.
	CSectionIterator iter(this);
	for (TSection * aSection = iter.FirstSection(); iter.More(); aSection = iter.NextSection())
	{
		if (aSection->GetSectionType() == stPublisher &&
			GetDayDesignator(aSection)->Contains(anAppointment))
				anAppointment->RemoveDependent(aSection);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleDocument::DeleteSubscribedAppointment(TAppointment* anAppointment)
{
	// Deletes a subscribed appointment from the document, handling changes and dependencies.

	Assertion(fSubscribedAppointments->GetIdentityItemNo(anAppointment) != kEmptyIndex,
			  "Appointment is not in fSubscribedAppointments");

	// Note: When the document closes, it notifies all dependents. Undoable commands
	// are dependent on their contexts. When such a command is notified that its context
	// is closing, it tells its context to commit its last command.
	//
	// TSectionCancelCommands are undoable commands whose context is the TEditionDocument.
	// TSectionCancelCommand::Commit, deletes a subscriber. This operation causes all
	// subscribed appointments to be deleted, by eventually calling this method.
	//
	// But Changed cannot be called during the update process. So the
	// following if statement keeps that from happening. It keeps badness from happening
	// when this method is called when the document is closing.

	if (!((TStandardDependencySpace*)this->GetDependencySpace())->fUpdating)
		anAppointment->Changed(cDeleteAppointment, this);		// deleting changes the item.
	anAppointment->RemoveDependent(fSubscribedAppointments);	// list no longer depends on this appointment

	fSubscribedAppointments->Delete(anAppointment);
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

void TScheduleDocument::DeleteSubscribedAppointments()
{
	// Deletes all subscribed appointments.

	CSubscribedAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
			this->DeleteSubscribedAppointment(anAppointment);
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::DoAddBorder(TSection* aSection)			// OVERRIDE
{
	// Adds the appropriate border to a section.

	Inherited::DoAddBorder(aSection);
	// create the appropriate adorner.
	if (fScheduleView)
	{
		if (aSection->GetSectionType() == stPublisher)
		{
			MAVolatileInit(RgnHandle, itsBorderRegion, NULL);
			MAVolatileInit(TPublisherAdorner*, aPubAdorner, NULL);

			TRY
			{
				itsBorderRegion	= MakeNewRgn();
#ifdef MACAPP_R_13
				aPubAdorner = TH_new TPublisherAdorner(kPublisherAdornerID, (TPublisher*)aSection, itsBorderRegion);
#else
				aPubAdorner = TH_new TPublisherAdorner;
				aPubAdorner->IPublisherAdorner(kPublisherAdornerID, (TPublisher*)aSection, itsBorderRegion);
#endif
				itsBorderRegion = NULL;		// the adorner owns the region now.

				fScheduleView->ComputeBorderRegion(aPubAdorner);
				fScheduleView->AddAdorner(aPubAdorner, kAdornAfter, this->IsBorderShown(aSection));
			}
			CATCH_ALL
			{
				itsBorderRegion	= DisposeIfRgnHandle(itsBorderRegion);
				aPubAdorner		= (TPublisherAdorner*)FreeIfObject(aPubAdorner);
				THROW;
			}
			ENDTRY;
		}
		else if (aSection->GetSectionType() == stSubscriber)
		{
			MAVolatileInit(RgnHandle, itsBorderRegion, NULL);
			MAVolatileInit(TSubscriberAdorner*, aSubAdorner, NULL);

			TRY
			{
				itsBorderRegion	= MakeNewRgn();
#ifdef MACAPP_R_13
				aSubAdorner = TH_new TSubscriberAdorner(kSubscriberAdornerID, (TSubscriber*)aSection, itsBorderRegion);
#else
				aSubAdorner = TH_new TSubscriberAdorner;
				aSubAdorner->ISubscriberAdorner(kSubscriberAdornerID, (TSubscriber*)aSection, itsBorderRegion);
#endif
				itsBorderRegion = NULL;		// the adorner owns the region now.

				fSubscriberView->ComputeBorderRegion(aSubAdorner);
				fSubscriberView->AddAdorner(aSubAdorner, kAdornAfter, this->IsBorderShown(aSection));

				fSubscriberView->ForceRedraw();			// redraw to show subscribed appointments.
				fSubscriberDayView->ForceRedraw();		// make sure title gets redrawn, too.
			}
			CATCH_ALL
			{
				itsBorderRegion	= DisposeIfRgnHandle(itsBorderRegion);
				aSubAdorner		= (TSubscriberAdorner*)FreeIfObject(aSubAdorner);
				THROW;
			}
			ENDTRY;
		}
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::DoDeleteBorder(TSection* aSection)		// OVERRIDE
{
	// Removes the appropriate border from a section.

	Inherited::DoDeleteBorder(aSection);
	if (aSection->GetSectionType() == stPublisher)
	{
		CAdornerIterator iter(fScheduleView);
		for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
		{
			if (anAdorner->fIdentifier == kPublisherAdornerID && ((TSectionAdorner*)anAdorner)->fSection == aSection)
			{
				fScheduleView->DeleteAdorner(anAdorner, kInvalidate);
				return;
			}
		}
	}
	else if (aSection->GetSectionType() == stSubscriber)
	{
		CAdornerIterator iter(fSubscriberView);
		for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
		{
			if (anAdorner->fIdentifier == kSubscriberAdornerID && ((TSectionAdorner*)anAdorner)->fSection == aSection)
			{
				fSubscriberView->DeleteAdorner(anAdorner, kInvalidate);
				fSubscriberView->ForceRedraw();			// the subscribed appointments go as well.
				fSubscriberDayView->ForceRedraw();		// make sure title gets redrawn, too.
				return;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TScheduleDocument::DoInitialState() // OVERRIDE
{
	// Called for 'New' && 'Revert' [to blank] commands && for default open tool icon

	Inherited::DoInitialState();

	fSubscriberName			= gEmptyString;

	fReopening				= false;
	fSavedWindowPosition	= gZeroVRect;
	fSavedScrollPosition	= gZeroVPt;
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TScheduleDocument::DoMakeViews(bool /*forPrinting*/) // OVERRIDE
{
	// Creates document window and fills in data member references.
	// Makes the new schedule view dependent on appointments already in document.

	TWindow* aWindow	= gViewServer->NewTemplateWindow(kScheduleWindowID, this);
	FailNIL(aWindow);

	fScheduleWindow		= aWindow;

	fScheduleView		= (TScheduleView*) fScheduleWindow->FindSubView(kScheduleID);
	fAppointments->AddDependent(fScheduleView);					// The view depends on the list.

	fSubscriberView		= (TScheduleView*) fScheduleWindow->FindSubView(kSubscriberID);
	fSubscribedAppointments->AddDependent(fSubscriberView);		// The view depends on the list.

	fDayView			= (TDayView*)fScheduleWindow->FindSubView('DAYS');
	fSubscriberDayView	= (TSubscriberDayView*)fScheduleWindow->FindSubView('SUBD');

	// Link the scrollers together.

	fPrimaryScroller	= (TPrimaryScroller*) fScheduleWindow->FindSubView('SCRP');
	if (fPrimaryScroller)
	{
		TSecondaryScroller* aTimeScroller = (TSecondaryScroller*) fScheduleWindow->FindSubView('SCRT');;
		fPrimaryScroller->AddSecondaryScroller(aTimeScroller, kNotHDependent, kVDependent);

		TSecondaryScroller* aDayScroller = (TSecondaryScroller*) fScheduleWindow->FindSubView('SCRD');;
		fPrimaryScroller->AddSecondaryScroller(aDayScroller, kHDependent, kNotVDependent);
	}

	TSchedulePrintHandler* aHandler = TH_new TSchedulePrintHandler(this, gViewServer->DoCreateViews(this, NULL, kSchedulePrintID, gZeroVPt));

	this->SetupTheWindow();		// restore window state or set initial scroll offset.
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TScheduleDocument::DoNeedDiskSpace(TFile* aFile,
										long& dataForkBytes,
										long& rsrcForkBytes)		// OVERRIDE
{
	// Uses temporary streams to calculate required bytes.

	Inherited::DoNeedDiskSpace(aFile, dataForkBytes, rsrcForkBytes);

	{
		CTemporaryCountingStream aStream;
		this->WriteDocumentData(aStream);
		dataForkBytes += aStream.GetBytesNeeded();
	}

	{
		CTemporaryCountingStream aStream;
		this->WriteDocumentState(aStream);
		rsrcForkBytes += aStream.GetBytesNeeded() + kRsrcTypeOverhead + kRsrcOverhead
							+ kDocStateName.Length() + kLengthByte;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TScheduleDocument::DoRead(TFile* aFile, bool forPrinting)	// OVERRIDE
{
	// Reads the document data and state

	Inherited::DoRead(aFile, forPrinting);

	CTemporaryFileStream aStream(aFile);
	this->ReadDocumentData(aStream);

	Handle docState = ::GetResource(kDocStateType, kDocStateID);
	if (docState)
	{
		CTemporaryHandleStream aStream(docState);
		this->ReadDocumentState(aStream);
		fReopening = true;
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::DoReadData(const OSType /* aScrapType */,
						TDesignator* aDesignator,
						TStream* aStream, long /*length*/ )			// OVERRIDE
{
	// Reads subscriber data. Replaces existing subscribed appointments with new ones,
	// if any. Sets the designator to indicate the new subscriber's appointments.

	this->DeleteSubscribedAppointments();			// remove existing subscribed appointments.

	CStr255 subscriberName;
	aStream->ReadString(subscriberName, sizeof(CStr255));
	this->SetSubscriberName(subscriberName);

	// Setup the subscriber's designator
	((TDayDesignator*)aDesignator)->SetDay(aStream->ReadInteger());	// Day number

	MAVolatileInit(TAppointment*, anAppointment, NULL);

	// read in new subscribed appointments
	TRY
	{
		long numAppointments = aStream->ReadLong();
		// bpa - validate this change while (!aStream->AtEnd())
		for (long iter = 0; iter < numAppointments; ++iter)
		{
			
			if (ReadAppointment(aStream, anAppointment))
			{
				this->AddSubscribedAppointment(anAppointment);
				anAppointment = NULL;
			}
			else
				Assertion(false, "Wierd stream object");
		}
	}
	CATCH_ALL
	{
		anAppointment = (TAppointment*)FreeIfObject(anAppointment);		// couldn't add it, so free remnant.
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TScheduleDocument::DoWrite(TFile* aFile, bool makingCopy)	// OVERRIDE
{
	// Writes document data and state

	Inherited::DoWrite(aFile, makingCopy);

	{
		CTemporaryFileStream aStream(aFile);
		this->WriteDocumentData(aStream);
	}
	{
		CTemporaryHandleStream aStream;
		this->WriteDocumentState(aStream);

		aStream.AdjustHandleSize();				// trim handle to stream size.

		::AddResource(aStream, kDocStateType, kDocStateID, kDocStateName);	// note cooersion of aStream to Handle.
		FailResError();
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::DoWriteData(const OSType aScrapType,
					TDesignator* aDesignator,  TStream* aStream)			// OVERRIDE
{
	// Writes designated (published) appointments, in the specified scrap type.

	if (aScrapType == fScrapType)
	{
		aStream->WriteString(GetUserName());											// user name

		aStream->WriteInteger(((TDayDesignator*)aDesignator)->GetDay());		// the day number
	}

	aStream->WriteLong(fAppointments->GetSize());
	CAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
		if (((TDayDesignator*)aDesignator)->Contains(anAppointment))
			if (aScrapType == fScrapType)
				aStream->WriteStreamObject(anAppointment);
			else if (aScrapType == kTextScrapType)								// making a preview
				anAppointment->WriteAsTextTo(aStream);
}

//----------------------------------------------------------------------------------------
#pragma segment AClose

TScheduleDocument::~TScheduleDocument()		// OVERRIDE
{
	// Free all data, then free the lists.

	fAppointments			= (TAppointmentList*)FreeListIfObject(fAppointments);
	fSubscribedAppointments = (TAppointmentList*)FreeListIfObject(fSubscribedAppointments);
}

//----------------------------------------------------------------------------------------
#pragma segment AClose

void TScheduleDocument::FreeData()	// OVERRIDE
{
	// Frees all appointments.

	FREEALL(fAppointments, TAppointment*);
	FREEALL(fSubscribedAppointments, TAppointment*);

	fSubscriberName			= gEmptyString;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TAppointmentList* TScheduleDocument::GetAppointmentList(bool isSubscriber)
{
	// Accessor function.
	// If the view is a subscriber, it returns the subscriber appointment list.

	return (isSubscriber ? fSubscribedAppointments : fAppointments);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TScheduleView* TScheduleDocument::GetScheduleView()
{
	// Accessor function

	return fScheduleView;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

TSection* TScheduleDocument::GetSelectedSection()		// OVERRIDE
{
	// If the subscriber day view is the target, return the (only) subscriber, if any.
	// Else if the day view is the target, return the publisher that contains the current selection.
	// Else (if any other view is the target), return NULL.

	TEventHandler* theTarget = gApplication->GetTarget();
	if (theTarget == fSubscriberDayView)
		return this->GetTheSubscriber();
	else if (theTarget == fDayView)
	{		// only check publishers.
		TSection* selectedSection = NULL;
		TDesignator* userSelection = this->GetUserSelection();

		if (userSelection)
		{
			TSection* aSection;
			CSectionIterator iter(this);

			for (aSection = iter.FirstSection(); iter.More(); aSection = iter.NextSection())
				if (!aSection->IsCanceled() && aSection->GetSectionType() == stPublisher)
					if (userSelection->IsContained(aSection->fDesignator) != kNotContained)
						if (!selectedSection)
							selectedSection = aSection;
						else
						{
							// in this case, >1 sections are in the user selection, so return NULL
							selectedSection = NULL;
							break;
						}
		}

		return selectedSection;

	}
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleDocument::GetSubscriberName(CStr255& itsName)
{
	// Accessor function
	itsName = fSubscriberName;
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TSubscriber* TScheduleDocument::GetTheSubscriber()
{
	// returns one-and-only (non-canceled) subscriber, if any.

	CSectionIterator iter(this);
	for (TSection * aSection = iter.FirstSection(); iter.More(); aSection = iter.NextSection())
		if (aSection->GetSectionType() == stSubscriber && !aSection->IsCanceled())
			return (TSubscriber*) aSection;
	return NULL;
}

//----------------------------------------------------------------------------------------
#pragma segment AAppointmentRes

bool TScheduleDocument::IsAppointmentAvailable(DayIndex theDay, TimeIndex theTime, TimeIndex& theDuration)
{
	// returns true if the specified appointment is available. Also returns the duration until
	// next appointment, or until end of the day.

	CAppointmentIterator iter(this);
	for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
			anAppointment = iter.NextAppointment())
	{
		DayIndex	itsDay;
		TimeIndex	itsTime;
		TimeIndex	itsDuration;
		anAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

		if (itsDay < theDay)
			continue;
		else if (itsDay > theDay)
			break;
		else if (itsTime < theTime)
		{
			if (itsTime + itsDuration > theTime)
				return false;
		}
		else if (itsTime == theTime)
			return false;
		else
		{
			theDuration = itsTime - theTime;
			return true;
		}
	}
	theDuration = fScheduleView->fNumOfRows - theTime + 1;
	return true;
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

TScheduleDocument::TScheduleDocument(TFile* itsFile) :
#ifdef MACAPP_R_13
	TEditionDocument(itsFile, kAppointmentScrapType, kSignature),
#endif
	// Puts document in "safe" state.
	fAppointments(NULL),
	fSubscribedAppointments(NULL),

	fScheduleView(NULL),
	fSubscriberView(NULL),

	fScheduleWindow(NULL),
	fPrimaryScroller(NULL),
	fReopening(false),
	fSavedWindowPosition(gZeroVRect),
	fSavedScrollPosition(gZeroVPt),

	fDayView(NULL),
	fSubscriberDayView(NULL),

	fSubscriberName(gEmptyString)
{
#ifndef MACAPP_R_13
	IEditionDocument(itsFile, kAppointmentScrapType, kSignature);
#endif
	// Creates empty appointment lists and current selection designator.

	fSaveUserSelection = false;

	fAppointments 			= NewAppointmentList();		// Create the appointment list
	fSubscribedAppointments = NewAppointmentList();		// Create the subscriber list

	// Create the user selection
	this->SetUserSelection(TH_new TDayDesignator);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TView* TScheduleDocument::MakeClipboardView()
{
	// Returns the clipboard view for display of custom scrap.
	// This method should be called _after_ the document has read its data.
	// Then, the view will automatically synch itself up with the document
	// to have the correct number of rows.

	return 	gViewServer->DoCreateViews(this, NULL, kScheduleClipViewID, gZeroVPt);
}

//----------------------------------------------------------------------------------------
#pragma segment AClipboard

void TScheduleDocument::ReadClipboardData(TStream* aStream)
{
	// Reads private scrap and creates appointments from it.
	// Used when this document is a clipboard document.

	MAVolatileInit(TAppointment*, anAppointment, NULL);

	TRY
	{
		long count = aStream->ReadLong();
		for (long index = 1; index <= count; index++)
		{
			if (ReadAppointment(aStream, anAppointment))
			{
				this->AddAppointment(anAppointment);
				anAppointment = NULL;
			}
			else
				Assertion(false, "Wierd stream object");
		}
	}
	CATCH_ALL
	{
		anAppointment = (TAppointment*)FreeIfObject(anAppointment);		// couldn't add it, so free remnant.
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

bool TScheduleDocument::ReadAppointment(TStream* aStream, TAppointment*& anAppointment)
{
#ifdef MACAPP_R_13
	bool wasValid;
	anAppointment = dynamic_cast<TAppointment*> (aStream->ReadStreamObject(wasValid));
	return wasValid;
#else
	return aStream->ReadStreamObject((TObject*&) anAppointment);
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TScheduleDocument::ReadDocumentData(TStream* aStream)
{
	// Reads document data from a stream.

	MAVolatileInit(TAppointment*, anAppointment, NULL);

	TRY
	{
		long count = aStream->ReadLong();
		for (long index = 1; index <= count; index++)
		{
			if (ReadAppointment(aStream, anAppointment))
			{
				this->AddAppointment(anAppointment);
				anAppointment = NULL;
			}
			else
				Assertion(false, "Wierd stream object");
		}
		CStr255 subscriberName;
		aStream->ReadString(subscriberName, sizeof(CStr255));
		this->SetSubscriberName(subscriberName);

		{
			long count = aStream->ReadLong();
			for (long index = 1; index <= count; index++)
			{
				if (ReadAppointment(aStream, anAppointment))
				{
					this->AddSubscribedAppointment(anAppointment);
					anAppointment = NULL;
				}
				else
					Assertion(false, "Wierd stream object");
			}
		}
	}
	CATCH_ALL
	{
		anAppointment = (TAppointment*)FreeIfObject(anAppointment);		// couldn't add it, so free remnant.
		THROW;
	}
	ENDTRY;
}

//----------------------------------------------------------------------------------------
#pragma segment AReadFile

void TScheduleDocument::ReadDocumentState(TStream* aStream)
{
	// Reads document state information from a stream.

	*aStream >> fSavedWindowPosition;
	*aStream >> fSavedScrollPosition;
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::RemoveSection(TSection* aSection)		// OVERRIDE
{
	// When a subscriber is removed, all subscribed appointments go with it.
	if (aSection->GetSectionType() == stSubscriber && !this->GetTheSubscriber())
		this->DeleteSubscribedAppointments();

	Inherited::RemoveSection(aSection);
}

//----------------------------------------------------------------------------------------
#pragma segment ANonRes

void TScheduleDocument::RevealSelection(TDesignator* aSelection)
{
	// Used to show the publisher.

	Inherited::RevealSelection(aSelection);
	if (aSelection && aSelection->GetSize())
		fScheduleView->RevealSelection(aSelection);
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleDocument::SetSubscriberName(ConstCStr255Param itsName)
{
	// Accessor function
	fSubscriberName = itsName;
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TScheduleDocument::SetupTheWindow()
{
	// Restores stored document state, or sets up initial state.

	if (fReopening)
	{
		// restore window position and scroll offset.
		fScheduleWindow->SetFrame(fSavedWindowPosition, kInvalidate);
		fScheduleWindow->fStaggered = true;	// make window think it's already staggered.

		fPrimaryScroller->SetLocalOrigin(fSavedScrollPosition, kInvalidate);
	}
	else
	{
		// set scroller to initial offset
		VRect initialCell;
		fScheduleView->CellToVRect(CPoint(kInitialDay, kInitialTime), initialCell);
		fPrimaryScroller->SetLocalOrigin(initialCell[topLeft], kInvalidate);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AOpen

void TScheduleDocument::ShowReverted()
{
	// Restores window state after reverting a document.
	Inherited::ShowReverted();
	this->SetupTheWindow();
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

void TScheduleDocument::UserSelectionChanged(TView* changedView)
{
	// Called to update user selection

	if (changedView == fDayView)		// only fDayView changes the user selection.
		GetDayUserSelection()->SelectDay(fDayView);

	Inherited::UserSelectionChanged(changedView);
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TScheduleDocument::WriteDocumentData(TStream* aStream)
{
	// Writes document data to a stream.

	aStream->WriteLong(fAppointments->GetSize());
	{
		CAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
			aStream->WriteStreamObject(anAppointment);
	}

	CStr255 subscriberName;
	this->GetSubscriberName(subscriberName);
	aStream->WriteString(subscriberName);

	aStream->WriteLong(fSubscribedAppointments->GetSize());
	{
		CSubscribedAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
			aStream->WriteStreamObject(anAppointment);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment AClipboard

void TScheduleDocument::WriteDocumentScrap()
{
	// Writes private data scrap. Used when this document is a clipboard document.

	CTemporaryHandleStream scrap;

	scrap.GetStream()->WriteLong(fAppointments->GetSize());

	{												// need a context for the iterator.
		CAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
				scrap.GetStream()->WriteStreamObject(anAppointment);
	}

	scrap.AdjustHandleSize();
	FailOSErr(gClipboardMgr->PutDeskScrapData(fScrapType, scrap));
}

//----------------------------------------------------------------------------------------
#pragma segment AWriteFile

void TScheduleDocument::WriteDocumentState(TStream* aStream)
{
	// Writes document state to a stream.

	VRect windowPosition;
	fScheduleWindow->GetFrame(windowPosition);
	*aStream << windowPosition;

	*aStream << fPrimaryScroller->fTranslation;
}

//----------------------------------------------------------------------------------------
#pragma segment AClipboard

void TScheduleDocument::WriteTextScrap()
{
	// Writes private scrap as text. Used when this document is a clipboard document.

	CTemporaryHandleStream scrap(kStr255Len + kLengthByte);

	{												// need a context for the iterator.
		CAppointmentIterator iter(this);
		for (TAppointment* anAppointment = iter.FirstAppointment(); iter.More();
				anAppointment = iter.NextAppointment())
			anAppointment->WriteAsTextTo(scrap);
	}

	scrap.AdjustHandleSize();						// make trim the handle to the correct size

	FailOSErr(gClipboardMgr->PutDeskScrapData(kTextScrapType, scrap));
}



//----------------------------------------------------------------------------------------
#pragma segment AClipboard

void TScheduleDocument::WriteToDeskScrap()
{
	// Puts the private document scrap, then a text version to the desk scrap.

	this->WriteDocumentScrap();							// my preferred scrap
	this->WriteTextScrap();								// the required type
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TDayDesignator* TScheduleDocument::GetDayUserSelection()
{
	// In R13, fUserSelection is an AutoPtr, it's simpler to call Get than to add .get()
	return dynamic_cast<TDayDesignator*> (GetUserSelection());
}

//----------------------------------------------------------------------------------------
#pragma segment ARes

TDayDesignator* GetDayDesignator(TSection* aSection)
{
#ifdef MACAPP_R_13
	return dynamic_cast<TDayDesignator*> (aSection->GetDesignator());
#else
	return dynamic_cast<TDayDesignator*> (aSection->fDesignator);
#endif
}
