//----------------------------------------------------------------------------------------
// UScheduleTracker.cp
// Copyright 1991-1995 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __USCHEDULETRACKER__
#include "UScheduleTracker.h"
#endif

#ifndef __USCHEDULEOBJECTS__
#include "UScheduleObjects.h"
#endif

#ifndef __USCHEDULEDOCUMENT__
#include "UScheduleDocument.h"
#endif

#ifndef __USCHEDULECOMMAND__
#include "UScheduleCommand.h"
#endif

#ifndef __UDIALOG__
#include <UDialog.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __UVIEWSERVER__
#include <UViewServer.h>
#endif

#ifndef __UDIALOGBEHAVIOR__
#include <UDialogBehavior.h>
#endif

#ifndef __UDOCUMENT__
#include <UDocument.h>
#endif

//----------------------------------------------------------------------------------------
// CLASS TAppointmentSketcher
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TRCSelectCommand

void TAppointmentSketcher::ComputeNewSelection(CONST GridCell& clickedCell)	// OVERRIDE
{
	// Overridden to only sketch vertically. The clicked cell's horizontal
	// coordinate is always taken from the anchor cell.
	GridCell anchoredCell(fAnchorCell.h, clickedCell.v);
	Inherited::ComputeNewSelection(anchoredCell);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentSketcher::DoIt() 						// OVERRIDE
{
	// Obtains the description of the appointment to add.
	// If accepted, launches an appointment adder.

	GridRect	cells(GRIDVIEWDESIGNATOR_GETBOUNDS(fThisSelection));					// note selection for later
	fGridView->HighlightCells(fThisSelection, hlOn, hlOff);		// turn off the highlighting before opening window
	GRIDVIEWDESIGNATOR_SETEMPTY(fThisSelection);								// clear the selection of cells.

	TWindow* aWindow = gViewServer->NewTemplateWindow(phGetDescription, NULL);
	FailNIL(aWindow);

	OSType dismisser = aWindow->PoseModally();	// run the dialog to get the description
	CStr255 itsDescription;

	if (dismisser == 'OK  ')					// if accepted, gather results before closing window
	{
		TEditText* anEditText = (TEditText*)(aWindow->FindSubView('DESC'));
		anEditText->GetText(itsDescription);
	}
#ifdef MACAPP_R_13
	aWindow->Close();						// close it.
	delete aWindow;
#else
	aWindow->CloseAndFree();
#endif
	if (dismisser == 'OK  ')					// now that window is safely closed, try to allocate the command.
	{
		TScheduleView* scheduleView = MA_DYNAMIC_CAST(TScheduleView, fGridView);
		fGridView->PostCommand(TH_new TAppointmentAdder(cAddAppointment, scheduleView,
			cells.left, cells.top, cells.GetLength(vSel), itsDescription));
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentSketcher::TAppointmentSketcher(TScheduleView* itsView, const VPoint& itsMouse)
#ifdef MACAPP_R_13
	: TRCSelectCommand(itsView, itsMouse, true /*theShiftKey*/, false /*theCommandKey*/)
#endif
{
#ifndef MACAPP_R_13
	IRCSelectCommand(itsView, itsMouse, true /*theShiftKey*/, false /*theCommandKey*/);
#endif
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

TTracker* TAppointmentSketcher::TrackMouse(TrackPhase aTrackPhase,
											  VPoint& /* anchorPoint */,
											  VPoint& /* previousPoint */,
											  VPoint& nextPoint,
											  bool mouseDidMove)		// OVERRIDE
{
	if (mouseDidMove)
	{
		GridCell clickedCell;
		VRect viewExtent;
		VPoint clippedPoint;

		fGridView->GetExtent(viewExtent);
		clippedPoint = nextPoint;
		clippedPoint.ConstrainTo(viewExtent);
		clickedCell = fGridView->VPointToCell(clippedPoint);

		switch (aTrackPhase)
		{
		case trackBegin:
			this->ComputeAnchorCell(clickedCell);
			fPreviousCell = clickedCell;
			break;

		case trackContinue:
			if (clickedCell != fPreviousCell)
			{
				this->ComputeNewSelection(clickedCell);
				this->HighlightNewSelection();

				GRIDVIEWDESIGNATOR_SET(fThisSelection, fPreviousSelection);
				fPreviousCell = clickedCell;
			}
			break;

		case trackEnd:
			if (GRIDVIEWDESIGNATOR_ISEMPTY(fThisSelection)) 	// clicks without drags don't sketch new appointments.
				return NULL;
			break;
		}
	}

	return this;
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentTracker
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

#undef Inherited
#define Inherited TTracker


//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::ComputeNewAppointment(const GridCell /* nextCell */)
{
	// subclasses override this.
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::DoIt()					// OVERRIDE
{
	// Pass this call to the method that really does the action.
	// No preparation is necessary.

	this->RedoIt();
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

TAppointmentTracker::~TAppointmentTracker()					// OVERRIDE
{
	fOldAppointment = (TAppointment*)FreeIfObject(fOldAppointment);
	fNewAppointment = (TAppointment*)FreeIfObject(fNewAppointment);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentTracker::TAppointmentTracker(CommandNumber itsCommandNumber,
			TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment) :
#ifdef MACAPP_R_13
	TTracker(itsCommandNumber, itsView->GetScheduleDocument(), kCanUndo, kCausesChange,
		itsView->GetScheduleDocument(), itsView, itsView->GetScroller(kAnySuperView), theMouse),
#endif
	fAppointment(theAppointment),
	fOldAppointment(reinterpret_cast<TAppointment*> (theAppointment->Clone())),
	fNewAppointment(reinterpret_cast<TAppointment*> (theAppointment->Clone())),

	fAnchorCell(gZeroPt),

	fScheduleView(itsView)
{
#ifndef MACAPP_R_13
	ITracker(itsCommandNumber, itsView->GetScheduleDocument(), kCanUndo, kCausesChange,
		itsView->GetScheduleDocument(), itsView, itsView->GetScroller(kAnySuperView), theMouse);
#endif
	// Preares for tracking by cloning theAppointment into the new and old versions.
	// Fills in appropriate data members.

	fConstrainsMouse	= true;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::RedoIt()					// OVERRIDE
{
	// Changes the appointment to the new version.

	fAppointment->Changed(cPreTrack, this);
	fAppointment->ChangeTo(fNewAppointment);
	fAppointment->Changed(cPostTrack, this);
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::TrackConstrain(TrackPhase aTrackPhase,
										const VPoint& /* anchorPoint */,
									  	const VPoint& previousPoint,
									    VPoint& nextPoint,
										bool mouseDidMove)			// OVERRIDE
{
	// Constrains mouse movement to cell-by-cell jumps.

	if (mouseDidMove && aTrackPhase == trackContinue)
	{
		GridCell previousCell	= fScheduleView->VPointToCell(previousPoint);
		GridCell nextCell		= fScheduleView->VPointToCell(nextPoint);

		if (previousCell == nextCell)
			nextPoint = previousPoint;			// cell didn't change, so mouse didn't move.
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::TrackFeedback(TrackPhase		/* aTrackPhase */,
								 const VPoint& /* anchorPoint */,
								 const VPoint& /* previousPoint */,
								 const VPoint& /* nextPoint */,
								 bool mouseDidMove,
								 bool		/* turnItOn */)	// OVERRIDE

{
	// If the mouse moved, it frames the new appointment's extent. Note that pen mode
	// is already set to patXOR before this method is executed.

	if (mouseDidMove)
	{
		VRect itsExtent;
		fScheduleView->GetAppointmentExtent(fNewAppointment, itsExtent);

		CRect qdExtent;
		fScheduleView->ViewToQDRect(itsExtent, qdExtent);
		::PenPat(&qd.gray);
		::PenSize(2,2);
		::FrameRect(qdExtent);
	}
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

TTracker* TAppointmentTracker::TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										bool mouseDidMove)	// OVERRIDE
{
	// If the track is beginning, it calculates the anchor cell.
	// If the track is continuing, it calculates the new appointment if the mouse moved.
	// If the track is ending, it checks to see if any change was made. If not, the command is aborted.

	TTracker* aTracker = Inherited::TrackMouse(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove);

	switch (aTrackPhase)
	{
	case trackBegin:
		fAnchorCell = fScheduleView->VPointToCell(anchorPoint);
		break;

	case trackContinue:
		if (mouseDidMove)
			{
				GridCell nextCell		= fScheduleView->VPointToCell(nextPoint);
				this->ComputeNewAppointment(nextCell);
			}
		break;

	case trackEnd:
		if (fOldAppointment->CompareTo(fNewAppointment) == kItem1EqualItem2)
			return NULL;
		break;
	}
	return aTracker;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

void TAppointmentTracker::UndoIt()				// OVERRIDE
{
	// Changes the appointment back to the old version.

	fAppointment->Changed(cPreTrack, this);
	fAppointment->ChangeTo(fOldAppointment);
	fAppointment->Changed(cPostTrack, this);
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentDragger
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TAppointmentTracker

void TAppointmentDragger::ComputeNewAppointment(const GridCell nextCell)	// OVERRIDE

{
	// Changes fNewAppointment to be located at the nextCell.
	// The calculations are based on fOldAppointment
	// and the offset to nextCell from fAnchorCell.
	// Carefully limits the result to the view's available rows and columns.

	DayIndex	itsDay;
	TimeIndex	itsTime;
	TimeIndex	itsDuration;
	fOldAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

	itsDay	= (short)MinMax(1, itsDay + (nextCell.h - fAnchorCell.h),
								fScheduleView->fNumOfCols);

	itsTime	= (short)MinMax(1, itsTime + (nextCell.v - fAnchorCell.v),
								fScheduleView->fNumOfRows - itsDuration + 1);

	fNewAppointment->SetAppointmentInfo(itsDay, itsTime, itsDuration);
}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentDragger::TAppointmentDragger(TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment) :
	TAppointmentTracker(cDragAppointment, itsView, theMouse, theAppointment)
{
}

//----------------------------------------------------------------------------------------
// CLASS TAppointmentStretcher
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand

#undef Inherited
#define Inherited TAppointmentTracker

void TAppointmentStretcher::ComputeNewAppointment(const GridCell nextCell)	// OVERRIDE

{
	// Stretches the appointment via the knob that was clicked in.
	// The validate call allows full range of motion in the vertical direction.

	CRect		newExtent;
	DayIndex	itsDay;
	TimeIndex	itsTime;
	TimeIndex	itsDuration;

	fOldAppointment->GetAppointmentInfo(itsDay, itsTime, itsDuration);

	switch (fWhichKnob)
	{
	case inUpperKnob:
		newExtent = CRect(nextCell.h, nextCell.v, nextCell.h + 1, itsTime + itsDuration);
		break;

	case inLowerKnob:
		newExtent = CRect(nextCell.h, nextCell.v + 1, nextCell.h + 1, itsTime);
		break;
	}
	newExtent.Validate();

	fNewAppointment->SetAppointmentInfo(itsDay, newExtent.top, newExtent.GetLength(vSel));

}

//----------------------------------------------------------------------------------------
#pragma segment ASelCommand

TAppointmentStretcher::TAppointmentStretcher(TScheduleView* itsView, const VPoint& theMouse,
			TAppointment* theAppointment, const KnobSelector whichKnob) :
	TAppointmentTracker(cStretchAppointment, itsView, theMouse, theAppointment),
	fWhichKnob(whichKnob)

{
	// Fills in data member.

}
