//-----------------------------------------------------------------------------------
// ULiveScrollerScrollBars.cpp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
// Contains: TScrollbar subclass that does live scrolling if the user drags the thumb
// Based on UMouserScrollBars by Mary Boetcher
//-----------------------------------------------------------------------------------

/*
	Change History:
*/

#ifndef	__ULIVESCROLLERSCROLLBAR__
#include "ULiveScrollerScrollBars.h"
#endif

#ifndef	__USCROLLER__
#include "UScroller.h"
#endif


//-----------------------------------------------------------------------------------

const short kScrollSlop	= 10;	// distance mouse can be outside scrollbar and still scroll 
const long	kLargeScale = 2000; // kludge to prevent overflow when scrolling large views

//-----------------------------------------------------------------------------------

void InitULiveScrollerScrollBars()
{
	// Suppress Linker dead-stripping of these classes 
	MA_REGISTER_CLASS(TLiveScrollerScrollBar);

	// Instead of normal scrollbars, use TLiveScrollerScrollBars 
	// Call MA_REGISTER_CLASS directly inst
	MA_REGISTER_SIGNATURE(TLiveScrollerScrollBar, kStdScrollerScrollBar);
}

//===================================================================================
//	CLASS TLiveScrollerScrollBar
//===================================================================================

#undef Inherited
#define Inherited TScrollerScrollBar

MA_DEFINE_CLASS(TLiveScrollerScrollBar);

//----------------------------------------------------------------------------------------
// TLiveScrollerScrollBar
//----------------------------------------------------------------------------------------

TLiveScrollerScrollBar::TLiveScrollerScrollBar(const CViewPoint& itsLocation,
								const CViewPoint& itsSize,
								SizeDeterminer itsHSizeDet,
								SizeDeterminer itsVSizeDet,
								VHSelect itsDirection,
								long itsMax,
								TScroller* itsScroller) :
	TScrollerScrollBar(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsDirection, itsMax, itsScroller),
	fOriginalValue(0)
{
}

//----------------------------------------------------------------------------------------
// DoMouseCommand
//----------------------------------------------------------------------------------------

void TLiveScrollerScrollBar::DoMouseCommand(CViewPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint_AC hysteresis)
{
	if (TestControl(fCMgrControl, ViewToQD(theMouse)) == kControlIndicatorPart)
	{
		PostCommand(TH_new TControlTracker(this, theMouse));
	}
	else
	{
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
	}
}

//----------------------------------------------------------------------------------------
// TrackMouse
//----------------------------------------------------------------------------------------

void TLiveScrollerScrollBar::TrackMouse(TrackPhase aTrackPhase,
								  CViewPoint& /*anchorPoint*/,
								  CViewPoint& previousPoint,
								  CViewPoint& nextPoint,
								  bool mouseDidMove)
{	
	long			sBarDelta;
	ViewCoordinate	newVal;

	switch (aTrackPhase)
	{
		case trackBegin:
			fOriginalValue = fLongVal;				// Save original thumb setting 
			if (fDirection == vSel)					// left, top, right, bottom 
				fThumbRect.Set(0, kSBarSize, fSize.h, fSize.v-kSBarSize);
			else
				fThumbRect.Set(kSBarSize, 0, fSize.h-kSBarSize, fSize.v);
			break;
			
		case trackEnd:
			if ( !IsScrollPoint(nextPoint) )
			{			// If mouse button is released outside of scroll area, 
				SetLongVal(fOriginalValue, kRedraw);	// snap back to original thumb position 

				for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
					newVal = iter->ScrollRelative(fDirection, fLongVal);
			}
			break;
			
		case trackContinue: 
			if ( mouseDidMove && IsScrollPoint(nextPoint) )
			{
				long scrollPixels = fThumbRect.GetLength(fDirection) - kSBarSize;
				if (scrollPixels > 0)
				{
					if (fDirection == vSel)
						sBarDelta = nextPoint.v - fThumbRect.top;
					else	
						sBarDelta = nextPoint.h - fThumbRect.left;

					// newVal = (sBarDelta * fLongMax) / scrollPixels;
					// The multiplication sBarDelta * fLongMax can overflow for large
					// values of sBarDelta or fLongMax.  We test for this overflow, and
					// if present, divide first to avoid it.
					
					if (labs(sBarDelta) * labs (fLongMax) < 0)
						newVal = sBarDelta * (fLongMax/scrollPixels);
					else
						newVal = (sBarDelta * fLongMax) / scrollPixels;
						
					
					if (newVal > fLongVal)
					{
						if (fDirection == vSel && nextPoint.v < previousPoint.v) 
							return;					// avoid scrolling forward when mouse moved backward 
						else if (fDirection == hSel && nextPoint.h < previousPoint.h) 
							return;					// avoid scrolling forward when mouse moved backward 
					}
					
					SetLongVal(newVal, kRedraw);	// SetLongVal does range checking, etc. on newVal 

					for (TemplateIterator_AC<TScroller*> iter(&fScrollers); iter; ++iter)
					{
						newVal = iter->ScrollRelative(fDirection, fLongVal);
						iter->Update();
					}
					
				}
			}
			break;
	}
}

//----------------------------------------------------------------------------------------
// IsScrollPoint
//----------------------------------------------------------------------------------------

bool TLiveScrollerScrollBar::IsScrollPoint(CViewPoint& thePoint)
{
	bool isScrollPoint = true;
	VHSelect oppositeDirection;
	
	if (! ContainsMouse(thePoint))
	{
		if (fDirection == vSel) 
			oppositeDirection = hSel;
		else 
			oppositeDirection = vSel;
			
		if (oppositeDirection == vSel)
		{
			if (thePoint.v < -kScrollSlop || thePoint.v > fSize.v + kScrollSlop) 
				isScrollPoint = false;
		}
		else
		{
			if (thePoint.h < -kScrollSlop || thePoint.h > fSize.h + kScrollSlop) 
				isScrollPoint = false;
		}
	}
	
	return isScrollPoint;
}
