//----------------------------------------------------------------------------------------
// USizer.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

/*
	Change History:
		Added TBaseSizer::ReadFrom.  Changed TInitSizerCommand to use
		TBaseSizer rather than TBaseSpecSizer.  Added TSizeToParent.
		Changed TStickToParentSizer changed to TParentRelatedSizer.
*/

#ifndef __USizer__
#define __USizer__

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif


class CStream_AC;
class TView;

//----------------------------------------------------------------------------------------
// TBaseSizer - A pure virtual base class for all other resizing behaviors.
//	This implements basic association to the target view and command initialization.
//----------------------------------------------------------------------------------------

class EXPORT_MA TBaseSizer : public TBehavior, public MDependable_AC
{
		MA_DECLARE_CLASS;
public:
	TBaseSizer();
	virtual	~TBaseSizer() { }
	virtual	void	InitializeSizer();
			void	DoUpdate(ChangeID_AC			theChange,
							 MDependable_AC*		changedObject,
							 void*					changeData,
							 CDependencySpace_AC*	dependencySpace);
	virtual	void	ReadFrom(CStream_AC* aStream);

protected:
	virtual	void	RespondToFrameAdjustment(CViewRect&  /* newFrame */,
											 CViewRect&	 /* deltaFrame */,
											 TView*		 /* changedView */) { } // = 0;
	virtual	void	RespondToOtherViewChange(TView*		 /* changedView */) { } // = 0;

	TView*			fTargetView;		// Actual target view.
	CViewRect		fOldFrame;			// Frame of target view last time we checked.
	bool			fInitialized;		// Prevents double initialization.
};

//----------------------------------------------------------------------------------------
// TBaseSpecSizer - A pure virtual base class for all resizing behaviors which read
//  their targets from the resource specification.
//----------------------------------------------------------------------------------------

class EXPORT_MA TBaseSpecSizer : public TBaseSizer
{
		MA_DECLARE_CLASS;
public:
	TBaseSpecSizer();
	virtual	~TBaseSpecSizer() { }
	virtual	void InitializeSizer();
	virtual	void ReadFrom(CStream_AC* aStream);
	virtual void WriteTo(CStream_AC* aStream) const;

protected:
	// An enum which specifies where to look for the target view.
	enum AssociationMode
	{
		eChildAssociation			= 0,
		ePeerAssociation			= 1,
		eParentAssociation			= 2
	};
	
	virtual	void					RespondToFrameAdjustment(CViewRect& /* newFrame */,
															CViewRect&		/* deltaFrame */,
															TView*		/* changedView */) { } // = 0;
	virtual	void					RespondToOtherViewChange(TView* /* changedView */) { } // = 0;

#if 1
	// This change was required in order to accomodate a problem
	// in the CodeWarrior compiler which found it difficult to
	// deal with enums.  See also TMultiSizer.
	short					fChildPeerOrParent;
#else
	AssociationMode			fChildPeerOrParent;
#endif
	IDType					fTargetViewID;	// ID of view who notifies us of changed frame.
};

//----------------------------------------------------------------------------------------
// TMultiSizer - Use this sizer in order to change multiple aspects of the view to
//	which it is attached in response to a change in the target.
//----------------------------------------------------------------------------------------

class EXPORT_MA TMultiSizer : public TBaseSpecSizer
{
		MA_DECLARE_CLASS;
public:
	TMultiSizer();
	virtual	~TMultiSizer() { }
	virtual	void ReadFrom(CStream_AC* aStream);
	virtual void WriteTo(CStream_AC* aStream) const;

protected:
	// An enum which specifies how to react in relation
	// to the target view's frame change.
	enum MultiAdjustMode
	{
		eDontAdjust					= 0,
		eAdjPosByTopOrLeft			= 1,
		eAdjNegByTopOrLeft			= -1,
		eAdjPosByBotOrRight			= 2,
		eAdjNegByBotOrRight			= -2
	};
	
	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*		changedView);
#if 1
	// This change was required in order to accomodate a problem
	// in the CodeWarrior compiler which found it difficult to
	// deal with enums.
	short							fTopMode;
	short							fLeftMode;
	short							fHeightMode;
	short							fWidthMode;
#else
	MultiAdjustMode					fTopMode;
	MultiAdjustMode					fLeftMode;
	MultiAdjustMode					fHeightMode;
	MultiAdjustMode					fWidthMode;
#endif
};

//----------------------------------------------------------------------------------------
// TSizer - This version of TBaseSpecSizer reads the mode parameters from the
//	behavior's resource.  This is probably the most generally useful sizer.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSizer : public TBaseSpecSizer
{
		MA_DECLARE_CLASS;
public:
	TSizer();
	virtual	~TSizer() { }
	virtual void InitializeSizer();
	virtual	void ReadFrom(CStream_AC* aStream);
	virtual void WriteTo(CStream_AC* aStream) const;

protected:
	// An enum which specifies how to respond to any change we see.
	enum ChangeTypeMode
	{
		eNoMode								= 0,
		eTopMode							= 1,
		eLeftMode							= 2,
		eHeightMode							= 3,
		eWidthMode							= 4
	};
	
	// An enum which specifies how to react in relation to the target view's frame change.
	enum AdjustMode
	{
		eDontAdjust							= 0,
		eAdjPosByTopOrLeft			= 1,
		eAdjNegByTopOrLeft			= -1,
		eAdjPosByBotOrRight			= 2,
		eAdjNegByBotOrRight			= -2,
		
		// Adjust proportional to the height/width.  Top/Left can be proportional to 
		// any parent's width and Height/Width can be proportional to any other width.
		eAdjProportional				= 3
	};
	
	virtual	void	RespondToFrameAdjustment(	CViewRect&		newFrame,
												CViewRect&		deltaFrame,
												TView*			changedView);
	virtual	void	SetGeneralMode(	ChangeTypeMode		theChangeType,
									AdjustMode			theAdjustMode);

	ChangeTypeMode			fChangeTypeMode;
	AdjustMode				fAdjustMode;
	double					fProportion;		// Proportion if eAdjProportional
};

//----------------------------------------------------------------------------------------
// TRightSizer - This version of TBaseSpecSizer sets up a link with the target view
//	in such as way as the width of our view is always set to be a constant distance
//	from the target views right edge.
//----------------------------------------------------------------------------------------

class EXPORT_MA TRightSizer : public TBaseSpecSizer
{
		MA_DECLARE_CLASS;
public:
	TRightSizer()
	 : fInProcess(false) { }
	virtual	~TRightSizer() { }
	virtual void	InitializeSizer();

protected:
	CViewRect			fOldSelfFrame;			// Frame of ourself last time we checked.
	bool				fInProcess;				// To protect against recursion.

	virtual	void	RespondToFrameAdjustment(	CViewRect&		newFrame,
												CViewRect&		deltaFrame,
												TView*		changedView);
	virtual	void	RespondToOtherViewChange(TView* changedView);
};

//----------------------------------------------------------------------------------------
// TBottomSizer - This version of TBaseSpecSizer sets up a link with the target view
//	in such as way as the height of our view is always set to be a constant distance
//	from the target views bottom edge.
//----------------------------------------------------------------------------------------

class EXPORT_MA TBottomSizer : public TBaseSpecSizer
{
		MA_DECLARE_CLASS;
public:
	TBottomSizer()
	 : fInProcess(false) { }
	virtual	~TBottomSizer() { }
	virtual void	InitializeSizer();

protected:
	CViewRect		fOldSelfFrame;			// Frame of ourself last time we checked.
	bool			fInProcess;					// To protect against recursion.

	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*		changedView);
	virtual	void	RespondToOtherViewChange(TView* changedView);
};

//----------------------------------------------------------------------------------------
// TParentRelatedSizer - This abstract version of TBaseSizer sets up a link with the 
// immediate parent view.
//----------------------------------------------------------------------------------------

class EXPORT_MA TParentRelatedSizer : public TBaseSizer
{
		MA_DECLARE_CLASS;
public:
	TParentRelatedSizer()
	 : fInProcess(false) { }
	virtual	~TParentRelatedSizer() { }
	virtual void	InitializeSizer();

protected:
	CViewRect		fOldSelfFrame;			// Frame of ourself last time we checked.
	bool			fInProcess;					// To protect against recursion.

	virtual	void	RespondToFrameAdjustment(CViewRect&		/* newFrame */,
											 CViewRect&		/* deltaFrame */,
											 TView*		/* changedView */) { } // = 0;
};

//----------------------------------------------------------------------------------------
// TStickToBottomOfParent - This version of TParentRelatedSizer insures that our view stays
// a constant distance from the target views bottom edge.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStickToBottomOfParent : public TParentRelatedSizer
{
		MA_DECLARE_CLASS;
public:
	TStickToBottomOfParent() { }
	virtual	~TStickToBottomOfParent() { }

protected:
	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*			changedView);
	virtual	void	RespondToOtherViewChange(TView* changedView);
};

//----------------------------------------------------------------------------------------
// TStickToRightOfParent - This version of TParentRelatedSizer insures that our view stays
// a constant distance from the target views left edge.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStickToRightOfParent : public TParentRelatedSizer
{
		MA_DECLARE_CLASS;
public:
	TStickToRightOfParent() { }
	virtual	~TStickToRightOfParent() { }

protected:
	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*			changedView);
	virtual	void	RespondToOtherViewChange(TView* changedView);
};

//----------------------------------------------------------------------------------------
// TStickToBottomRightOfParent - This version of TParentRelatedSizer insures that our view
// stays a constant distance from the target views bottom and right edges.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStickToBottomRightOfParent : public TParentRelatedSizer
{
		MA_DECLARE_CLASS;
public:
	TStickToBottomRightOfParent() { }
	virtual	~TStickToBottomRightOfParent() { }

protected:
	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*		changedView);
	virtual	void	RespondToOtherViewChange(TView* changedView);
};

//----------------------------------------------------------------------------------------
// TSizeToParent - This T...Sizer maintains the size of our view in accordance to changes
// in the size of the parent view.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSizeToParent : public TParentRelatedSizer
{
		MA_DECLARE_CLASS;
public:
	TSizeToParent() { }
	virtual	~TSizeToParent() { }

protected:
	virtual	void	RespondToFrameAdjustment(CViewRect&		newFrame,
											 CViewRect&		deltaFrame,
											 TView*			changedView);
};

//----------------------------------------------------------------------------------------
// TInitSizerCommand - Special class for initializing a
// behavior.
//----------------------------------------------------------------------------------------

class EXPORT_MA TInitSizerCommand : public TCommand
{
public:
	TInitSizerCommand(TBaseSizer* aBehavior = NULL);		

	virtual	~TInitSizerCommand() { }

	virtual	void	DoIt();

protected:
	TBaseSizer*		fBehavior;
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUSizer();

#endif
