//----------------------------------------------------------------------------------------
// CODSession_AC
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODSession_AC__
#define __CODSession_AC__

// ALOE

#ifndef __ALOE_AC__
#include "ALOE_AC.h"
#endif

// Core

#ifndef __CAssertionFailure_AC__
#include "CAssertionFailure_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

// ACS Imaging

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Forward declarations
//----------------------------------------------------------------------------------------

class CODAdaptorMgr_AC;
class CPoint_AC;

//----------------------------------------------------------------------------------------
// CODSession_AC is a singleton that initiates and terminates the ALOE session.
//----------------------------------------------------------------------------------------

class EXPORT_AC CODSession_AC {

	// --------------------------------------------------------------------------------
	// Initialization and termination
	// --------------------------------------------------------------------------------

protected:

	CODSession_AC();

public:

	virtual ~CODSession_AC();

	// Copy constructor and assignment are disallowed.
private:

	CODSession_AC(const CODSession_AC& that); // not implemented
	CODSession_AC& operator = (const CODSession_AC& that); // not implemented

	// --------------------------------------------------------------------------------
	// These are for establishing and querying for contact with ALOE and OpenDoc.
	// --------------------------------------------------------------------------------

public:

	static CODSession_AC* Instance();
		// This returns a singleton instance, the syntax is: SODession::Instance().

	void Connect();
		// This starts a session with ALOE.

	void Disconnect();
		// This terminates a session with ALOE.
					
	inline bool IsConnected() const
		{ return fIsConnected; }
		// Returns whether we have started ALOE.

	inline bool CanConnect() const
		{ return fCanConnect; }
		// If false, it means we have determined that OpenDoc >= 1.2 or ALOE is not installed.

	static inline bool ApplicationHasFocus()
		{ if (fInstance) return Instance()->fApplicationHasFocus; else return true; }
		// Returns whether the application has the focus.  Returns true if the instance 
		// hasn't been initialized yet.

protected:

	inline void SetConnection(bool connected)
		{ fIsConnected = connected; }
		
	// --------------------------------------------------------------------------------
	// These are factory methods for creating adaptor managers.
	// --------------------------------------------------------------------------------
			
public:
	
	virtual CODAdaptorMgr_AC* NewAdaptorMgr();
	// This factory method creates an adaptor manager for a new document.

	virtual CODAdaptorMgr_AC* NewAdaptorMgrFromFile(long dataRefNum);
	// This factory method creates an adaptor manager for a saved document whose
	// file is open for reading and specified with dataRefNum.

	// --------------------------------------------------------------------------------
	// This is debugging assistance code for checking when a developer 
	// forgets to set the base menu bar.
	// --------------------------------------------------------------------------------

protected:

#if qCoreDebug
	static inline void DebugMenuBarIsSet() 
		{ Instance()->fIsMenuBarSet = true; }	
	static void DebugCheckMenuBar();
#else
	static inline void DebugMenuBarIsSet() {}
	static inline void DebugCheckMenuBar() {}
#endif // qCoreDebug

	// --------------------------------------------------------------------------------
	// ALOE global functions are scoped as static and inlined where possible
	// so they have no state and may be called without using the Instance().
	// --------------------------------------------------------------------------------

public:	
	
	static inline void SelectWindow(WindowPtr window)
		{ ThrowIfError_AC(::ALOESelectWindow(window)); }
	
	static inline void SetBaseMenuBar(short mbarResource)
		{ ThrowIfError_AC(::ALOESetBaseMenuBar(mbarResource)); DebugMenuBarIsSet(); }

	static inline void SetBaseMenuBarHandle(Handle mbarHandle)
		{ ThrowIfError_AC(::ALOESetBaseMenuBarHandle(mbarHandle)); DebugMenuBarIsSet(); }

	static inline void RegisterMenuCommand(short menuID, short menuItem, long commandID)
		{ ThrowIfError_AC(::ALOERegisterMenuCommand(menuID, menuItem, commandID)); }
	
	static bool HandleEvent(EventRecord* event);
		
	static inline bool IsUnhandledMenuEvent(const EventRecord* event)
		{ WindowPtr aWMgrWindow; return (event->what == mouseDown && ::FindWindow(event->where, &aWMgrWindow) == inMenuBar); }

	static inline long GetUnhandledMenuEvent(const EventRecord* event)
		{ Assertion_AC(IsUnhandledMenuEvent(event), NULL); return event->message; }
	
	static inline bool IsWindowContentEvent(const EventRecord* event)
		{ WindowPtr	window; return (event->what == mouseDown && ::FindWindow(event->where, &window) == inContent); }
	
	static inline bool IsSuspendResumeEvent(const EventRecord* event)
		{ return (event->what == osEvt && ((event->message & osEvtMessageMask) >> 24) == suspendResumeMessage); }

	static inline bool Idle()
		{ DebugCheckMenuBar(); return ::ALOEIdle(); }
		
	static inline RgnHandle GetSleepRegion(RgnHandle appSleepRegion)
		{ return ::ALOEGetSleepRegion(appSleepRegion); }
	
	static inline EventMask GetEventMask(EventMask appMask)
		{ return ::ALOEGetEventMask(appMask); }
	
	static inline UInt32 GetSleepTime(UInt32 appSleepTime)
		{ return ::ALOEGetSleepTime(appSleepTime); }
		
	static inline void ApplicationActivated()
		{ if (Instance()->IsConnected()) ThrowIfError_AC(::ALOEApplicationActive()); }
	
	static inline bool IsUndoClear()
		{ return ::ALOEIsUndoClear(); }
	
protected:	
		
	virtual void SetCallbacks();
		// This sets all callbacks. Override, call inherited, and 
		// add any additional callbacks your app needs.
		
	void DisposeCallbacks();
		// This disposes all installed callbacks.

	void SetAdaptorImageUpdatedCallback();
		// This sets the ALOE callback to call DoAdaptorImageUpdated.
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoAdaptorImageUpdated.
	
	void SetRevealAdaptorCallback();
		// This sets the ALOE callback to call DoRevealAdaptor.
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoRevealAdaptor.
		
	void SetAdaptorChangedCallback();
		// This sets the ALOE callback to call DoAdaptorChanged.
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoAdaptorChanged.

	virtual void SetAdjustActiveBorderCallback();
		// This sets the ALOE callback to call DoAdjustActiveBorder.
		
	virtual void SetActivateApplicationCallback();
		// This sets the ALOE callback to call DoActivateApp.

	void SetWindowActionCallback();
		// This sets the ALOE callback to call DoWindowAction.  
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoWindowAction.

	virtual void SetAdaptorShapeRequestCallback();
		// This sets the ALOE callback to call DoAdaptorSizeRequest.
		
	virtual void SetResetMenuBarCallback();
		// This sets the ALOE callback to call DoResetMenubar.

	void SetCreateAdaptorObjectSpecifierCallback();
		// This sets the ALOE callback to call DoCreateAdaptorObjectSpecifier.
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoCreateAdaptorObjectSpecifier.
		
	void SetWindowActivationCallback();
		// This sets the ALOE callback to call DoWindowActivate.
		// Note: this is not called in the base class, a client must call this
		// in an override of SetCallbacks and override DoWindowActivate.
		
	// --------------------------------------------------------------------------------
	// These are ALOE Callback methods that a subclass may want to re-implement.
	// --------------------------------------------------------------------------------
	
public:	

	virtual void DoAdaptorImageUpdated(ALOEAdaptorView adaptorView);
		// This implements a recipe for image updating.
	
	virtual void DoRevealAdaptor(ALOEAdaptor theAdaptor);
		// This does nothing by default.  A subclass may want to implement
		// a recipe for this.
		
	virtual void DoAdaptorChanged(ALOEAdaptorView adaptorView);
		// This does nothing by default.  A subclass may want to implement
		// a recipe for this.
		
	virtual void DoAdjustActiveBorder(ALOEAdaptorView adaptorView, RgnHandle shape);
		// This implements a recipe for adjusting the active border.

	virtual void DoActivateApp(bool activate);
		// This implements a recipe for adaptor de-activation by default.

	virtual void DoWindowAction(WindowPtr window, ALOEWindowAction action, bool param);
		// This is not called by default.  SetWindowActionCallback must be called
		// by a subclass that also overrides this callback in order to let ALOE use
		// a client implementation of floating window support.

	virtual void DoAdaptorShapeRequest(ALOEAdaptor theAdaptor, RgnHandle theRequestedRegion);
		// This implements a recipe for setting a default shape.

	virtual void DoResetMenuBar(WindowPtr window);
		// This does nothing by default.  A subclass may want to implement
		// a menu bar swapping recipe.
		
	virtual void DoCreateAdaptorObjectSpecifier(ALOEAdaptorMgr theMgr, ALOEAdaptor theAdaptor, 
												WindowPtr currentResolutionContext, 
												AEDesc& objectSpec, bool& specIsWindowOnly);
		// This does nothing by default.  A subclass may want to implement
		// a menu bar swapping recipe.
	
	virtual void DoWindowActivate(WindowPtr window, bool activate);
		// This implements a recipe for activating a window.  A subclass may want
		// to use custom window activation code.
	
	
	virtual void FocusOnApplication();
		// This is called by the default recipe for DoActivateApp,
		// and sets the clip region of the frontmost window by default.

	virtual void DefaultAdaptorShapeRequest(RgnHandle theRequestedRegion);
		// This is a the default implementation for setting a default shape
				
	// --------------------------------------------------------------------------------
	// The following are methods required to be overridden by a client's subclass.
	// --------------------------------------------------------------------------------

protected:	

	virtual void DoSetBaseMenuBar() = 0;
		// Set the base menu bar.
		// The client must implement this because ALOE can't function without a menubar.
		// The client must call either SetBaseMenuBar() or SetBaseMenuBarHandle().
		// If the client implements this function but doesn't call either function, 
		// an exception will be thrown (when debugging is turned on).
	
	virtual CODAdaptorMgr_AC* GetFrontAdaptorMgr() = 0;
		// Return the frontmost window's corresponding adaptor mgr
		// Client must implement this because we don't know how the ODAdaptorMgrs 
		// will be managed.

	virtual WindowPtr GetFrontNonFloatingWindow() const;
		// Return the frontmost non-floating window
		
	// --------------------------------------------------------------------------------
	// The following utilities are provided for Drag and Drop frame negotiation.
	// --------------------------------------------------------------------------------

public:
	inline void SetDropLocation(CPoint_AC& theDropLocation)
		{ fDropLocation = theDropLocation; }
	
	inline CPoint_AC GetDropLocation()
		{ return fDropLocation; }
					
	// --------------------------------------------------------------------------------
	// Data members.
	// --------------------------------------------------------------------------------

private:

	static CODSession_AC*				fInstance;
	
	// To avoid polluting the name space and to ensure that a session instance is 
	// managing the state of these globals, they are scoped and non-static.
	
	bool								fCanConnect;
	bool								fIsConnected;
	bool								fApplicationHasFocus;

	ALOEAdaptorImageUpdatedUPP			fAdaptorImageUpdatedUPP;
	ALOERevealAdaptorUPP				fRevealAdaptorUPP;
	ALOEAdaptorChangedUPP				fAdaptorChangedUPP;
	ALOEAdjustActiveBorderShapeUPP		fAdjustActiveBorderUPP;
	ALOEActivateApplicationUPP			fActivateAppUPP;
	ALOEWindowCallbackUPP				fWindowActionUPP;
	ALOEAdaptorShapeRequestUPP			fAdaptorShapeRequestUPP;
	ALOEResetMenuBarUPP					fResetMenuBarUPP;
	ALOECreateAdaptorObjectSpecifierUPP	fCreateAdaptorObjectSpecifierUPP;
	ALOEWindowActivationCallbackUPP		fWindowActivateUPP;
	
	CPoint_AC							fDropLocation;
	
#if qCoreDebug		
	// This flag is used by the menu bar debugging assistance code
	bool								fIsMenuBarSet;	
#endif
};

#endif // __CODSession_AC__