//----------------------------------------------------------------------------------------
// CODAdaptorMgr_AC.cp
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

// ALOE

#ifndef __CODAdaptor_AC__
#include "CODAdaptor_AC.h"
#endif

#ifndef __CODAdaptorView_AC__
#include "CODAdaptorView_AC.h"
#endif

// STL

#ifndef LIST_H
#include <list.h>
#endif

//========================================================================================
// Base live object adaptor manager defines shared behavior for subclasses
//========================================================================================

template class list<CODAdaptor_AC*, allocator<CODAdaptor_AC*> >;

//----------------------------------------------------------------------------------------
// Construct ALOE manager
//----------------------------------------------------------------------------------------

CODAdaptorMgr_AC::CODAdaptorMgr_AC() :
		fManager(NULL),
		fApplicationHasFocus(true),
		fSelectedAdaptorView(NULL)
{
	ALOEAdaptorMgr manager;
	ThrowIfError_AC(::ALOENewAdaptorMgr(&manager));
	fManager = manager;
}
	
//----------------------------------------------------------------------------------------
// Construct ALOE manager from file
//----------------------------------------------------------------------------------------

CODAdaptorMgr_AC::CODAdaptorMgr_AC(long dataRefNum) :
	fManager(NULL),
	fSelectedAdaptorView(NULL)
{
	ALOEAdaptorMgr manager;
	ThrowIfError_AC(::ALOENewAdaptorMgrFromFile(dataRefNum, &manager));
	fManager = manager;
}

//----------------------------------------------------------------------------------------
// Close adaptor manager
//----------------------------------------------------------------------------------------

CODAdaptorMgr_AC::~CODAdaptorMgr_AC()
{
	DeleteAdaptors();
	
	if (fManager != NULL) {
		ThrowIfError_AC(::ALOEDisposeAdaptorMgr(fManager));
		fManager = NULL;
	}
}

//----------------------------------------------------------------------------------------
// returns whether manager has changes and whether storage size is different
//----------------------------------------------------------------------------------------
		
bool CODAdaptorMgr_AC::HasChanged(bool& isSameSize)
{
	Boolean isDirtyBoolean;
	// !!! ALOE will always claim it is not initialized here due to a bug
#if 1
	isSameSize = true;
	isDirtyBoolean = false;	
#else
	Boolean isSameSizeBoolean;
	ThrowIfError_AC(::ALOEIsAdaptorMgrDirty(fManager, &isDirtyBoolean, &isSameSizeBoolean));
	isSameSize = isSameSizeBoolean;
#endif
	return isDirtyBoolean;
}

//----------------------------------------------------------------------------------------
// Create an adaptor
//----------------------------------------------------------------------------------------

CODAdaptor_AC* CODAdaptorMgr_AC::NewAdaptor(
						WindowPtr		window,
						ALOEISOString	kind,
						RgnHandle		shape,
						Ptr				dataPtr,
						long			dataSize)
{
	return TH_new CODAdaptor_AC(this, window, kind, shape, dataPtr, dataSize);
}

//----------------------------------------------------------------------------------------
// Create an adaptor from OD storage
//----------------------------------------------------------------------------------------

CODAdaptor_AC* CODAdaptorMgr_AC::NewAdaptorFromStorage(
						ALOEPersistentID id,
						WindowPtr		window)
{
	// a new adaptor from storage may already be read in, in which case,
	// we would much rather return the read one than create a new wrapper
	CODAdaptor_AC* adaptor = FindAdaptor(id);
	if (adaptor == NULL)
	{
		adaptor = new CODAdaptor_AC(this, id, window);
	}
	return adaptor;
}

//----------------------------------------------------------------------------------------
// Create an adaptor from a drag
//----------------------------------------------------------------------------------------

CODAdaptor_AC* CODAdaptorMgr_AC::NewAdaptorFromDrag(
					 WindowPtr			window,
					 DragReference 		theDragRef, 
					 long				index)
{
	return new CODAdaptor_AC(this, window, theDragRef, index);
}

//----------------------------------------------------------------------------------------
// DoActivateApp
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::DoActivateApp(bool activate)
{
	fApplicationHasFocus = activate;
}

//----------------------------------------------------------------------------------------
// toggle selected view state
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::ToggleAdaptorViewSelectState(CODAdaptorView_AC* adaptorView)
{
	Boolean shouldSelect = ! adaptorView->GetSelectState();
	adaptorView->SetSelectState(shouldSelect);
	
	if (shouldSelect)
	{
		fSelectedAdaptorView = adaptorView;
		// Put DrawSelection before to avoid flicker
		DrawSelection(shouldSelect);
		::ALOEApplicationActive();
	}
	else
	{
		// Deselecting, so clear selection
		DrawSelection(shouldSelect);
		fSelectedAdaptorView = NULL;
	}
}

//----------------------------------------------------------------------------------------
// DrawSelection
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::DrawSelection(bool isSelected)
{
	RGBColor  originalColor;
	
	::GetForeColor(&originalColor); // save off original
	RgnHandle theBoundsRgn = ::NewRgn();
	
	try 
	{
		//!!! tbd: check TESample change here
		//fSelectedAdaptorView->GetBounds(theBoundsRgn);

		// ??? Focus?  globalPoint should be focused view?
		Point globalPoint = { 0, 0 };

		RgnHandle tempRgn = ::NewRgn();
		::CopyRgn(theBoundsRgn, tempRgn);
		
		::LocalToGlobal(&globalPoint);
		::InsetRgn(tempRgn, 3, 3);
		::DiffRgn(theBoundsRgn, tempRgn, tempRgn);
		::OffsetRgn(tempRgn, globalPoint.h, globalPoint.v);
		
		::PenSize(2,2);
		
		if (isSelected)
			::ForeColor(cyanColor);
		else
		{	
			RGBColor  backColor;
			::GetBackColor(&backColor);
			::RGBForeColor(&backColor);
		}
				
		::FrameRect(&(*theBoundsRgn)->rgnBBox);
		
		::PenNormal();		
		::RGBForeColor(&originalColor); //return it to the way it was
		::DisposeRgn(tempRgn);
	}
	catch (CException_AC& theException)
	{
		DebugStr("\pCODAdaptorView_AC::GetBounds returned an error in CODAdaptorMgr_AC::DrawSelection");
		// let it go, don't rethrow
	}

	// Clean up	
	::DisposeRgn(theBoundsRgn);
}

//----------------------------------------------------------------------------------------
// add adaptor to my list
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::AddAdaptor(CODAdaptor_AC* adaptor)
{
	GetAdaptorList()->push_front(adaptor);
}

//----------------------------------------------------------------------------------------
// remove adaptor from my list
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::RemoveAdaptor(CODAdaptor_AC* adaptor)
{
	GetAdaptorList()->remove(adaptor);
}

//----------------------------------------------------------------------------------------
// return adaptor list, initializing it if not created
//----------------------------------------------------------------------------------------

CODAdaptorList_AC* CODAdaptorMgr_AC::GetAdaptorList()
{
	if (!fAdaptors)
	{
		fAdaptors = TH_new CODAdaptorList_AC;
	}
	return fAdaptors;
}

//----------------------------------------------------------------------------------------
// delete list of adaptors
//----------------------------------------------------------------------------------------

void CODAdaptorMgr_AC::DeleteAdaptors()
{
	for (CODAdaptorList_AC::iterator iter = GetAdaptorList()->begin(); iter != GetAdaptorList()->end(); ++iter)
		delete *iter;
}

//----------------------------------------------------------------------------------------
// find adaptor that has this persistent ID
//----------------------------------------------------------------------------------------

CODAdaptor_AC* CODAdaptorMgr_AC::FindAdaptor(ALOEPersistentID id)
{
	CODAdaptor_AC* returnVal = NULL;

	if (fAdaptors)
		for (CODAdaptorList_AC::iterator iter = GetAdaptorList()->begin(); iter != GetAdaptorList()->end(); ++iter)
			if ((*iter)->IsSamePersistentID(id))
			{
				returnVal = *iter;
				break;
			}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// find adaptor that has this persistent ID
//----------------------------------------------------------------------------------------

CODAdaptor_AC* CODAdaptorMgr_AC::FindAdaptor(ALOEAdaptor adaptor)
{
	CODAdaptor_AC* returnVal = NULL;

	if (fAdaptors)
		for (CODAdaptorList_AC::iterator iter = GetAdaptorList()->begin(); iter != GetAdaptorList()->end(); ++iter)
			if ((*iter)->IsSameAdaptor(adaptor))
			{
				returnVal = *iter;
				break;
			}

	return returnVal;
}

