//----------------------------------------------------------------------------------------
// CODSession_AC.cp
// Copyright  1996 by Apple Computer. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CODSession_AC__
#include "CODSession_AC.h"
#endif

// ALOE

#ifndef __CODAdaptorMgr_AC__
#include "CODAdaptorMgr_AC.h"
#endif

#ifndef __CODAdaptor_AC__
#include "CODAdaptor_AC.h"
#endif

// Window support

#ifndef __CODWindow_AC__
#include "CODWindow_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CODSession_AC is a singleton that initiates and terminates the session.
//----------------------------------------------------------------------------------------

CODSession_AC* CODSession_AC::fInstance = NULL;

//----------------------------------------------------------------------------------------
// The following are callback functions installed with ALOE that
// in turn call virtual functions of the ODSession instace.
// It would be real bad if we threw an exception and didn't catch it in our callback,
// because ALOE, OpenDoc, SOM, and everything else might get really confuzzled.
//----------------------------------------------------------------------------------------

static void pascal AdaptorImageUpdatedProc(ALOEAdaptorView view, void* /*contextPtr*/)
{
	try
	{
		CODSession_AC::Instance()->DoAdaptorImageUpdated(view);
	}
	catch (CException_AC& exception)
	{
		// Report an error
	}
}

//----------------------------------------------------------------------------------------

static void pascal RevealAdaptorProc(ALOEAdaptor adaptor, void* /*contextPtr*/)
{
	try
	{
		CODSession_AC::Instance()->DoRevealAdaptor(adaptor);
	}
	catch (CException_AC& exception)
	{
		// Report an error
	}
}

//----------------------------------------------------------------------------------------

static void pascal AdaptorChangedProc(ALOEAdaptorView view, void* /*contextPtr*/)
{
	try
	{
		CODSession_AC::Instance()->DoAdaptorChanged(view);
	}
	catch (CException_AC& exception)
	{
		// Report an error
	}
}

//----------------------------------------------------------------------------------------

static void pascal AdjustActiveBorderProc(ALOEAdaptorView adaptorView, RgnHandle shape, void* /*contextPtr*/) 
{
	try
	{
		CODSession_AC::Instance()->DoAdjustActiveBorder(adaptorView, shape);
	}
	catch (CException_AC& exception)
	{
		// If an exception gets thrown, the debugger will have informed us.
	}
}

//----------------------------------------------------------------------------------------

static void pascal ActivateAppProc(Boolean activate, void* /*contextPtr*/) 
{
	try
	{
		CODSession_AC::Instance()->DoActivateApp(activate);
	}
	catch (CException_AC& exception)
	{
		// Report an error
	}
}

//----------------------------------------------------------------------------------------

static void pascal WindowActionProc(WindowPtr window, ALOEWindowAction action, Boolean param, void* /*contextPtr*/) 
{
	try
	{
		CODSession_AC::Instance()->DoWindowAction(window, action, param);
	}
	catch (CException_AC& exception)
	{
		// If an exception gets thrown, the debugger will have informed us.
	}
}

//----------------------------------------------------------------------------------------

static OSErr pascal AdaptorShapeRequestProc(ALOEAdaptor theAdaptor, RgnHandle theRequestedRegion, void* /*contextPtr*/) 
{
	try 
	{
		CODSession_AC::Instance()->DoAdaptorShapeRequest(theAdaptor, theRequestedRegion);
	}
	catch(CException_AC& exception)
	{
		return exception.GetMessage();
	}
	return noErr;
}

//----------------------------------------------------------------------------------------

static void pascal ResetMenuBarProc(WindowPtr window, void* /*contextPtr*/) 
{
	try
	{
		CODSession_AC::Instance()->DoResetMenuBar(window);
	}
	catch (CException_AC& exception)
	{
		// If an exception gets thrown, the debugger will have informed us.
	}
}

//----------------------------------------------------------------------------------------

static void pascal CreateAdaptorObjectSpecifierProc(ALOEAdaptorMgr mgr, ALOEAdaptor adaptor, 
													WindowPtr currentResolutionContext, 
													AEDesc* objectSpec, 
													Boolean* specIsWindowOnly, 
													void* /*contextPtr*/)
{
	try
	{
		AEDesc aeObjectSpec;
		bool bSpecIsWindowOnly;
		CODSession_AC::Instance()->DoCreateAdaptorObjectSpecifier(mgr, adaptor, 
						currentResolutionContext, aeObjectSpec, bSpecIsWindowOnly);
		*objectSpec = aeObjectSpec;
		*specIsWindowOnly = bSpecIsWindowOnly;
	}
	catch (CException_AC& exception)
	{
		// If an exception gets thrown, the debugger will have informed us.
	}

}

//----------------------------------------------------------------------------------------

static void pascal WindowActivationProc(WindowPtr window, Boolean active, void* /*contextPtr*/)
{
	try
	{
		CODSession_AC::Instance()->DoWindowActivate(window, active);
	}
	catch (CException_AC& exception)
	{
		// If an exception gets thrown, the debugger will have informed us.
	}
}

//----------------------------------------------------------------------------------------
// The constructor initializes everything and sets the instance.
//----------------------------------------------------------------------------------------

CODSession_AC::CODSession_AC() :
		fCanConnect(true),  // true until we learn otherwise
		fIsConnected(false),
		fApplicationHasFocus(true),
		fAdaptorImageUpdatedUPP(NULL),
		fRevealAdaptorUPP(NULL),
		fAdaptorChangedUPP(NULL),
		fAdjustActiveBorderUPP(NULL),
		fActivateAppUPP(NULL),
		fWindowActionUPP(NULL),
		fAdaptorShapeRequestUPP(NULL),
		fResetMenuBarUPP(NULL),
		fCreateAdaptorObjectSpecifierUPP(NULL),
		fWindowActivateUPP(NULL),
		fDropLocation(CPoint_AC::kZeroPt)
#if qCoreDebug
		,
		fIsMenuBarSet(false)
#endif // qCoreDebug
{ 
	// ??? Are there any other configuration tests to make here,
	// such as existence of ALOE library or ALOE ProxyPart?
	
	// ALOE only supports one global session, so we enforce this.
	Assertion_AC(fInstance == NULL, NULL);
	fInstance = this;
}

//----------------------------------------------------------------------------------------
// The destructor ensures that the session is terminated and resets fInstance.
//----------------------------------------------------------------------------------------

CODSession_AC::~CODSession_AC()
{ 
	Disconnect();
	fInstance = NULL;
}

//----------------------------------------------------------------------------------------
// Establish the session with ALOE
//----------------------------------------------------------------------------------------

void CODSession_AC::Connect()
{
	if (CanConnect() && !IsConnected())
	{
		try 
		{
			OSErr err = ::StartALOE();
			fCanConnect = (err == noErr);

#if qCoreDebug
			// Debugging aids to help understand the most common errors returned
			if (err == kALOENewerOpenDocRequiredErr)
			{
				DebugStr("\pALOE cannot be loaded because OpenDoc is an old version (< 1.2).");
			}
			else if (err == kALOEOpenDocNotInstalledErr)
			{
				DebugStr("\pALOE cannot be loaded because OpenDoc is not installed.");
			}
			else if (err != noErr)
			{
				DebugStr("\pThe ALOE Shared Library was not loaded");
			}
#endif // qCoreDebug
			
			// Failure to load ALOE shared library means we can't make ALOE calls.
			// If caller wants to cope with this, caller must catch the exception,
			// and always check CanConnect() before making any other ALOE call.
			ThrowIfError_AC(err);
			
			// Install callbacks
			SetCallbacks();

			// Set up the ALOE menu bar		
			DoSetBaseMenuBar();

			SetConnection(true);

		}
		catch (CException_AC& exception)
		{
			Disconnect();
			
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// Terminate an ALOE session
//----------------------------------------------------------------------------------------
void CODSession_AC::Disconnect()
{
	if (IsConnected())
	{
		ThrowIfError_AC(::TerminateALOE());
		DisposeCallbacks();
		
		SetConnection(false);
	}
}
	
//----------------------------------------------------------------------------------------
// Instance ensures that only one instance exists
//----------------------------------------------------------------------------------------

CODSession_AC* CODSession_AC::Instance()
{
#if qCoreDebug
	if (fInstance == NULL)
	{
		::DebugStr("\pAn instance of a subclass of CODSession_AC must be created before accessing Instance().");
	}
#endif
	return fInstance;
}

//----------------------------------------------------------------------------------------
// Handle an event, being careful about how ALOE munges the event
//----------------------------------------------------------------------------------------

bool CODSession_AC::HandleEvent(EventRecord* event)
{ 
	DebugCheckMenuBar();
	EventRecord eventCopy = *event;
	bool handledEvent;
	
#if 1
	// ??? Mask out osEvts that suspend/resume, b/c OpenDoc activates the wrong windows
	if (IsSuspendResumeEvent(event))
		handledEvent = false;
	else
#endif
		handledEvent = ::ALOEHandleEvent(&eventCopy); 
		
	if ((!handledEvent) && (IsUnhandledMenuEvent(event))) {
		// Save modified event.message for GetUnhandledMenuEvent responses
		event->message = eventCopy.message;
	}
	
#if 1 // ??? ALOE does not really handle activate events
	if (handledEvent && (event->what == activateEvt))
		handledEvent = false;
#endif

#if 1 // ??? ALOE does not handle certain window drag/resize events
	if (!handledEvent && (event->what == mouseDown))
		handledEvent = CODWindow_AC::HandleEvent(event);
#endif


// the mouse click is in another window: MacApp beeps in complaint and the window isn't activated.
#if 0 // If ALOE handled the event and it was of a mousedown inContent 
		// event, then we need to let the app know about this so it can keep
		// track of selections, targeting etc.....
	if (handledEvent && (IsWindowContentEvent(event)))
		handledEvent = false;
#endif

	return handledEvent;
}

//----------------------------------------------------------------------------------------
// Create an AdaptorMgr
//----------------------------------------------------------------------------------------
	
CODAdaptorMgr_AC* CODSession_AC::NewAdaptorMgr()
{
	// if we aren't connected, caller should have called Connect() first
	Assertion_AC(IsConnected(), NULL);
		
	return TH_new CODAdaptorMgr_AC();
}

//----------------------------------------------------------------------------------------
// Create an AdaptorMgr from persistent storage
//----------------------------------------------------------------------------------------
	
CODAdaptorMgr_AC* CODSession_AC::NewAdaptorMgrFromFile(long dataRefNum)
{
	// if we aren't connected, caller should have called Connect() first
	Assertion_AC(IsConnected(), NULL);
	
	return TH_new CODAdaptorMgr_AC(dataRefNum);
}

//----------------------------------------------------------------------------------------
// Set AdaptorImageUpdated callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetAdaptorImageUpdatedCallback()
{
	fAdaptorImageUpdatedUPP = NewALOEAdaptorImageUpdatedProc(&AdaptorImageUpdatedProc);
	ThrowIfNULL_AC(fAdaptorImageUpdatedUPP);
	ThrowIfError_AC(::ALOESetAdaptorImageUpdatedCallback(fAdaptorImageUpdatedUPP, NULL));
}

//----------------------------------------------------------------------------------------
// Set RevealAdaptor callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetRevealAdaptorCallback()
{
	fRevealAdaptorUPP = NewALOERevealAdaptorProc(&RevealAdaptorProc);
	ThrowIfNULL_AC(fRevealAdaptorUPP);
	ThrowIfError_AC(::ALOESetRevealAdaptorCallback(fRevealAdaptorUPP, NULL));
}

//----------------------------------------------------------------------------------------
// Set AdaptorChanged callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetAdaptorChangedCallback()
{
	fAdaptorChangedUPP = NewALOEAdaptorChangedProc(&AdaptorChangedProc);
	ThrowIfNULL_AC(fAdaptorChangedUPP);
	ThrowIfError_AC(::ALOESetAdaptorChangedCallback(fAdaptorChangedUPP, NULL));
}

//----------------------------------------------------------------------------------------
// Set AdjustActiveBorder callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetAdjustActiveBorderCallback()
{
	fAdjustActiveBorderUPP = NewALOEAdjustActiveBorderShapeProc(&AdjustActiveBorderProc);
	ThrowIfNULL_AC(fAdjustActiveBorderUPP);
	ThrowIfError_AC(::ALOESetAdjustActiveBorderShapeCallback(fAdjustActiveBorderUPP, NULL));		
}

//----------------------------------------------------------------------------------------
// Set ActivateApp callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetActivateApplicationCallback()
{
	fActivateAppUPP = NewALOEActivateApplicationProc(&ActivateAppProc);
	ThrowIfNULL_AC(fActivateAppUPP);
	ThrowIfError_AC(::ALOESetActivateApplicationCallback(fActivateAppUPP, NULL));
}

//----------------------------------------------------------------------------------------
// Set WindowAction callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetWindowActionCallback()
{
	fWindowActionUPP = NewALOEWindowCallbackProc(&WindowActionProc);
	ThrowIfNULL_AC(fWindowActionUPP);
	ThrowIfError_AC(::ALOESetWindowCallback(fWindowActionUPP, NULL));		
}

//----------------------------------------------------------------------------------------
// Set AdaptorShapeRequest callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetAdaptorShapeRequestCallback()
{
	fAdaptorShapeRequestUPP = NewALOEAdaptorShapeRequestProc(&AdaptorShapeRequestProc);
	ThrowIfNULL_AC(fAdaptorShapeRequestUPP);
	ThrowIfError_AC(::ALOESetAdaptorShapeRequestCallback(fAdaptorShapeRequestUPP, NULL));		
}


//----------------------------------------------------------------------------------------
// Set ResetMenuBar callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetResetMenuBarCallback()
{
	fResetMenuBarUPP = NewALOEResetMenuBarProc(&ResetMenuBarProc);
	ThrowIfNULL_AC(fResetMenuBarUPP);
	ThrowIfError_AC(::ALOESetResetMenuBarCallback(fResetMenuBarUPP, NULL));		
}

//----------------------------------------------------------------------------------------
// Set CreateAdaptorObjectSpecifier callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetCreateAdaptorObjectSpecifierCallback()
{
	fCreateAdaptorObjectSpecifierUPP = NewALOECreateAdaptorObjectSpecifierProc(&CreateAdaptorObjectSpecifierProc);
	ThrowIfNULL_AC(fCreateAdaptorObjectSpecifierUPP);
	ThrowIfError_AC(::ALOESetCreateAdaptorObjectSpecifierCallback(fCreateAdaptorObjectSpecifierUPP, NULL));		
}
	
//----------------------------------------------------------------------------------------
// Set ResetMenuBar callback
//----------------------------------------------------------------------------------------

void CODSession_AC::SetWindowActivationCallback()
{
	fWindowActivateUPP = NewALOEWindowActivationCallbackProc(&WindowActivationProc);
	ThrowIfNULL_AC(fWindowActivateUPP);
	ThrowIfError_AC(::ALOESetWindowActivationCallback(fWindowActivateUPP, NULL));		
}


//----------------------------------------------------------------------------------------
// set all callbacks
//----------------------------------------------------------------------------------------

void CODSession_AC::SetCallbacks()
{
	try {
		SetAdjustActiveBorderCallback();
		SetActivateApplicationCallback();
		SetAdaptorShapeRequestCallback();
		SetResetMenuBarCallback();
		
		// ALOE doesn't require setting the following callbacks by default.
		// You may call any of the following functions in your override of 
		// SetCallbacks, after calling inherited::SetCallbacks.
		// They will automatically get installed and uninstalled at the appropriate times.
		
		// SetAdaptorImageUpdatedCallback();
		// SetRevealAdaptorCallback();
		// SetAdaptorChangedCallback();
		// SetWindowActionCallback();
		// SetAdaptorShapeChangedCallback();
		// SetResetMenuBarCallback();
		// SetCreateAdaptorObjectSpecifierCallback();
		// SetWindowActivateCallback();
	}
	catch (CException_AC& exception) {
		DisposeCallbacks();
		
		throw;
	}
}		
		
//----------------------------------------------------------------------------------------
// dispose all callbacks
//----------------------------------------------------------------------------------------

void CODSession_AC::DisposeCallbacks()
{		
	if (fAdaptorImageUpdatedUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fAdaptorImageUpdatedUPP);
		fAdaptorImageUpdatedUPP = NULL;
	}
	if (fRevealAdaptorUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fRevealAdaptorUPP);
		fRevealAdaptorUPP = NULL;
	}
	if (fAdaptorChangedUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fAdaptorChangedUPP);
		fAdaptorChangedUPP = NULL;
	}
	if (fAdjustActiveBorderUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fAdjustActiveBorderUPP);
		fAdjustActiveBorderUPP = NULL;
	}
	if (fActivateAppUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fActivateAppUPP);
		fActivateAppUPP = NULL;
	}
	if (fWindowActionUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fWindowActionUPP);
		fWindowActionUPP = NULL;
	}
	if (fAdaptorShapeRequestUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fAdaptorShapeRequestUPP);
		fAdaptorShapeRequestUPP = NULL;
	}
	if (fResetMenuBarUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fResetMenuBarUPP);
		fResetMenuBarUPP = NULL;
	}
	if (fCreateAdaptorObjectSpecifierUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fCreateAdaptorObjectSpecifierUPP);
		fCreateAdaptorObjectSpecifierUPP = NULL;
	}
	if (fWindowActivateUPP != NULL) {
		::ALOEDisposeRoutineDescriptor(fWindowActivateUPP);
		fWindowActivateUPP = NULL;
	}
}

//----------------------------------------------------------------------------------------
// FocusToWindow: support code for DoActivateApp 
//----------------------------------------------------------------------------------------

void CODSession_AC::FocusOnApplication()
{	
	// ??? This is what TESample FocusToWindow does,
	// ??? Do we need this behavior?  Is a check needed?  It may bomb if there is no window

	// Focus to whole window.  If an embedded part did any drawing, the 
	// clip will be too small.
	WindowPtr window = GetFrontNonFloatingWindow();
	if (window != NULL)
	{
		::SetOrigin(0, 0);
		RgnHandle tempRgn = ((GrafPtr) window)->visRgn;
		::SetClip(tempRgn);
	}
}

//----------------------------------------------------------------------------------------
// Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoAdaptorImageUpdated(ALOEAdaptorView /*adaptorView*/)
{
}
	
//----------------------------------------------------------------------------------------
// Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoRevealAdaptor(ALOEAdaptor /*theAdaptor*/)
{
}
		
//----------------------------------------------------------------------------------------
// Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoAdaptorChanged(ALOEAdaptorView /*adaptorView*/)
{
}

//----------------------------------------------------------------------------------------
// AdjustActiveBorder Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoAdjustActiveBorder(ALOEAdaptorView /*adaptorView*/, RgnHandle /*shape*/)
{
// ??? what is the recipe?  Call into CODAdaptorView_AC first?
}

//----------------------------------------------------------------------------------------
// ActivateApp Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoActivateApp(bool activate)
{
	bool hadFocus = fApplicationHasFocus;
	fApplicationHasFocus = activate;

	CODAdaptorMgr_AC* adaptorMgr = GetFrontAdaptorMgr();
	if (adaptorMgr)
	{
		adaptorMgr->DoActivateApp(activate);
	}
	
	if (activate && !hadFocus)
	{
		FocusOnApplication();
	}
	else if (!activate && hadFocus && adaptorMgr)
	{
		// We should not have an "application" selection, turn whatever we have off
		if (adaptorMgr->HasSelection())
		{		
			// get selected adaptor, get its state, toggle state to turn it off
			adaptorMgr->ToggleAdaptorViewSelectState(adaptorMgr->GetSelectedAdaptorView());
		}
	}
}

//----------------------------------------------------------------------------------------
// The default implementation of DoWindowAction throws because it shouldn't be 
// installed except when a subclass implements it.   Applications that use floating windows 
// may want to call back to their window management code.
//----------------------------------------------------------------------------------------

void CODSession_AC::DoWindowAction(WindowPtr /*window*/, ALOEWindowAction /*action*/, bool /*param*/)
{
#if qCoreDebug
	DebugStr("\pYou called SetWindowActionCallback(), but you haven't implemented DoWindowAction().");
#endif

	ThrowError_AC(0);
}

//----------------------------------------------------------------------------------------
// AdaptorShapeRequest Callback implementation
//----------------------------------------------------------------------------------------

void CODSession_AC::DoAdaptorShapeRequest(ALOEAdaptor theAdaptor, RgnHandle theRequestedRegion)
{
	// default delegates to CODAdaptor_AC
	CODAdaptorMgr_AC* adaptorMgr = GetFrontAdaptorMgr();
	if (adaptorMgr)
	{
		CODAdaptor_AC* sAdaptor = adaptorMgr->FindAdaptor(theAdaptor);
		if (sAdaptor)
		{
			sAdaptor->DoAdaptorShapeRequest(theRequestedRegion);
		}
		else
			; // Implement a default behavior

	}
#if qCoreDebug
	else
		DebugStr("\pAdaptorShapeRequest was called but there is no frontmost adaptor manager.");
#endif

}


//----------------------------------------------------------------------------------------
// The default implementation of DoResetMenubar does nothing.  Your subclass may 
// implement menu bar swapping if you have a need for it.
//----------------------------------------------------------------------------------------

void CODSession_AC::DoResetMenuBar(WindowPtr /* window */)
{	
}

//----------------------------------------------------------------------------------------
// DoCreateAdaptorObjectSpecifier
//----------------------------------------------------------------------------------------

void CODSession_AC::DoCreateAdaptorObjectSpecifier(ALOEAdaptorMgr /*theMgr*/, ALOEAdaptor /*theAdaptor*/, 
												WindowPtr /*currentResolutionContext*/, 
												AEDesc& /*objectSpec*/, bool& /*specIsWindowOnly*/)
{
}
	
//----------------------------------------------------------------------------------------
// DoWindowActivate
//----------------------------------------------------------------------------------------

void CODSession_AC::DoWindowActivate(WindowPtr /*window*/, bool /*activate*/)
{
}
		
//----------------------------------------------------------------------------------------
// This is a the default implementation for setting a default shape
//----------------------------------------------------------------------------------------

void CODSession_AC::DefaultAdaptorShapeRequest(RgnHandle theRequestedRegion)
{
	// creates a default shape
	// ??? Do we really want the shape to be relative to the drop location?
	CPoint_AC itsDefaultSize(100, 100);
	CPoint_AC itsLocation = GetDropLocation();	
	if(::EmptyRgn(theRequestedRegion))
	{
		// Set to a default size relative to drop location
		// ??? will this work instead?
		//::SetRectRgn(theRequestedRegion, fDropLocation.h, fDropLocation.v, 
		//							  fDropLocation.h + itsDefaultSize.h, 
		//							  fDropLocation.v + itsDefaultSize.v);
		
		(*theRequestedRegion)->rgnBBox.left 	= itsLocation.h;
		(*theRequestedRegion)->rgnBBox.top 		= itsLocation.v;
		(*theRequestedRegion)->rgnBBox.right 	= (*theRequestedRegion)->rgnBBox.left +  itsDefaultSize.h;
		(*theRequestedRegion)->rgnBBox.bottom 	= (*theRequestedRegion)->rgnBBox.top  +  itsDefaultSize.v;
	}
	else
	{
		// Offset relative to the drop location
		::OffsetRgn(theRequestedRegion, itsLocation.h, itsLocation.v);
	}

}

//----------------------------------------------------------------------------------------
// GetFrontNonFloatingWindow
//----------------------------------------------------------------------------------------

WindowPtr CODSession_AC::GetFrontNonFloatingWindow() const
{
	return CODWindow_AC::GetFrontNonFloatingWindow();
}

//----------------------------------------------------------------------------------------
// DebugCheckMenuBar
//----------------------------------------------------------------------------------------

#if qCoreDebug
void CODSession_AC::DebugCheckMenuBar()
{
	if (Instance()->IsConnected() && !Instance()->fIsMenuBarSet)
	{ 
		DebugStr("\pALOE can't handle events or idle until it has been given a menu bar.");
	}
}
#endif // qCoreDebug
