// CDialog_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

// This file defines a set of helper functions and classes that wrap the
// standard MacOS Dialog Manager functionality, and interface with the core
// Gray Council code. To give a standard dialog the AGA look, you must
// instantiate an CDialog_AC object to interface with Dialog Manager for
// that dialog.
//
// There are several ways that you normally access the Dialog Manager's
// alerts and dialogs, and this determines how you use the interfaces
// provided here.
//
// To wrap simple alert calls, you just replace the standard calls with
// the corresponding wrappers shown here. The parameters and function
// result are exactly the same. However, if you access dialog items from
// inside your filterProc, see the notes below for how you must also wrap
// those accesses.
//
// - Alert() becomes Alert_AC()
// - StopAlert() becomes StopAlert_AC()
// - NoteAlert() becomes NoteAlert_AC()
// - CautionAlert() becomes CautionAlert_AC()
//
// Dialogs are more complicated than Alerts, with or without the AGA. This
// is because you often set up various dialog items before displaying the
// dialog, and then run a more complicated filter that plays around with
// the dialog items as the user clicks and types. Fortunately, The Gray
// Council makes wrapping all of this easy -- you just have to be thorough
// about wrapping your accesses to dialog items.
//
// Running a simple modal dialog normally looks something like this:
//
/*
	SInt16		itemHit = -1;
	DialogPtr	aDialog = ::GetNewDialog(kDialogID, NULL, (WindowPtr) -1);
	
	::ShowWindow(aDialog);
	
	while (itemHit != ok)
		::ModalDialog(myFilterUPP, &itemHit);
	
	::DisposeDialog(aDialog);
*/
//
// Wrapping this with The Gray Council looks like this:
//
/*
	CDialog_AC	CDialog_AC;
	SInt16		itemHit = -1;
	DialogPtr	aDialog = CDialog_AC.GetNewDialog(kDialogID, NULL, (WindowPtr) -1);
	
	::ShowWindow(aDialog);
	
	while (itemHit != ok)
		CDialog_AC.ModalDialog(myFilterUPP, &itemHit);
	
	CDialog_AC.DisposeDialog(aDialog);
*/
//
// That's the basics. You create an CDialog_AC object, and let it wrap the
// GetNewDialog, ModalDialog, and DisposeDialog calls rather than calling
// those Dialog Manager functions directly.
//
// But you almost certainly will need to access dialog items either before
// you show the dialog or inside your filter proc. When you do this, you
// need to wrap the Dialog Manager or Control Manager call so that The Gray
// Council can do its work on the AGA object where appropriate. There are
// functions provided for all necessary calls:
//
// Dialog Manager wrappers:
// - CDialog_AC::SetDialogItem()
// - CDialog_AC::HideDialogItem()
// - CDialog_AC::ShowDialogItem()
// - CDialog_AC::FindDialogItem()
// - CDialog_AC::AppendDITL()
// - CDialog_AC::ShortenDITL()
// - CDialog_AC::CountDITL()
// - CDialog_AC::SetDialogDefaultItem()
// - CDialog_AC::SetDialogCancelItem()
// - CDialog_AC::SetDialogItemText()
// - CDialog_AC::SelectDialogItemText()
//
// Control Manager wrappers:
// - CDialog_AC::SetControlValue()
// - CDialog_AC::SetControlMinimum()
// - CDialog_AC::SetControlMaximum()
// - CDialog_AC::SetControlTitle()
// - CDialog_AC::HideControl()
// - CDialog_AC::MoveControl()
// - CDialog_AC::SizeControl()
// - CDialog_AC::HiliteControl()
//
// If you call one of these Toolbox functions directly, avoiding the wrapper,
// the desired operation may fail to appear on screen, or in some cases may cause
// all hell to break loose (mainly with the xxxDITL calls). So just remember
// that pretty much all Dialog Manager and Control Manager calls should be
// made via wrapper functions of the CDialog_AC class. Some are not strictly
// necessary but are provided for completeness. The ones list above are the
// ones that are required.
//
// Please also read the comments below for the other individual functions
// that you may need to use.

#pragma once

#ifndef __CDialog_AC__
#define __CDialog_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __StandardControls_AC__
#include "StandardControls_AC.h"
#endif

// ACS

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

EXPORT_AC SInt16 Alert_AC(SInt16 alertID, ModalFilterUPP modalFilter);
EXPORT_AC SInt16 StopAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter);
EXPORT_AC SInt16 NoteAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter);
EXPORT_AC SInt16 CautionAlert_AC(SInt16 alertID, ModalFilterUPP modalFilter);

//----------------------------------------------------------------------------------------
// CustomObjectInfo
//----------------------------------------------------------------------------------------

typedef struct
{
	CControl_AC*	fAGAControl;
	bool			fVisible;
	bool			fTracksMouse;
} CustomObjectInfo, *CustomObjectInfoPtr, **CustomObjectInfoHandle;

//----------------------------------------------------------------------------------------
// CDialog_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CDialog_AC
{
public:

	CDialog_AC();
	virtual ~CDialog_AC();
	
	DialogPtr	GetNewDialog(SInt16 dialogID, Ptr dStorage, WindowPtr behind);
	SInt16		Alert(SInt16 alertID, ModalFilterUPP modalFilter, SInt16 iconID);

	//		
	// Dialog Manager wrapper functions. In order to properly wrap a
	// dialog with the AGA look and feel, you must use these methods
	// instead of their Dialog Manager counterparts when dealing with
	// an CDialog_AC object. If you call the Dialog Manager calls
	// directly, we can't do the right thing with the replacement
	// AGA objects.
	//

	void	CloseDialog();
	void	DisposeDialog();
	void	GetDialogItem(SInt16 itemNo, SInt16* itemType, Handle* item, Rect* box);	// wrapper not strictly needed
	void	SetDialogItem(SInt16 itemNo, SInt16 itemType, Handle item, Rect* box);
	void	HideDialogItem(SInt16 itemNo);
	void	ShowDialogItem(SInt16 itemNo);
	SInt16	FindDialogItem(Point thePt);
	void	AppendDITL(Handle theDITL, DITLMethod theMethod);
	void	ShortenDITL(SInt16 numberOfItems);
	SInt16	CountDITL();
	OSErr	SetDialogDefaultItem(SInt16 newItem);
	OSErr	SetDialogCancelItem(SInt16 newItem);
	void	GetDialogItemText(Handle item, CStr255_AC& text);	// wrapper not strictly needed
	void	SetDialogItemText(Handle item, const CStr255_AC& text);
	void	SelectDialogItemText(SInt16 itemNo, SInt16 strtSel, SInt16 endSel);
	void	ModalDialog(ModalFilterUPP filterProc, SInt16* itemHit);
	bool	DialogSelect(EventRecord* event, SInt16* itemHit);
	void	DrawDialog();
	void	UpdateDialog(RgnHandle updateRgn);
	
	//
	// Control Manager wrapper functions. In order to properly wrap a
	// dialog with AGA look and feel, you must also use these methods
	// instead of their Control Manager counterparts when dealing a
	// control that is part of an CDialog_AC. If you call the Control
	// Manager calls directly, we can't do the right thing with the
	// replacement AGA objects.
	//
	
	void	SetControlValue(ControlHandle theControl, SInt16 theValue);
	void	SetControlMinimum(ControlHandle theControl, SInt16 minValue);
	void	SetControlMaximum(ControlHandle theControl, SInt16 maxValue);
	void	SetControlTitle(ControlHandle theControl, const CStr255_AC& title);
	void	HideControl(ControlHandle theControl);
	void	MoveControl(ControlHandle theControl, SInt16 h, SInt16 v);
	void	SizeControl(ControlHandle theControl, SInt16 w, SInt16 h);
	void	HiliteControl(ControlHandle theControl, SInt16 hiliteState);

	// This is the actual UPP-based UserItem static function.
	static pascal void UserItem(WindowPtr theWindow, SInt16 itemNo);
	// This is our UserItem callback called by the static function.
	void	HandleUserItem(SInt16 itemNo);

	// This is the actual UPP-based ModalFilter static function.
	static pascal bool ModalFilter(DialogPtr theDialog, EventRecord* theEvent, SInt16* itemHit);
	// This is our ModalFilter callback called by the static function.
	bool	HandleModalFilter(EventRecord* theEvent, SInt16* itemHit);
	
	//
	// Useful utilities.
	//

	CControl_AC*			GetAGAControl(SInt16 itemIndex);
	
	// InstallAGAUserItem will take the supplied AGA object and use it
	// as the userItem for the specified itemNo. To make things easier,
	// you can construct the object with any bounds; this function will
	// set the bounds to match the item. Note that it will actually
	// reside at the (itemNo + mNumAGAItems) item. Pass true for objects
	// that should eat mouse clicks. (e.g., false for group boxes)
	void	InstallAGAUserItem(CControl_AC* anObject, SInt16 itemNo, bool tracksMouse);
	enum { kUserItemTracksMouse = true, kUserItemDoesNotTrackMouse = false };
	
	// HintPopupControl is necessary to get AGA popup menu CTRL items
	// in a dialog. This is because the popup menu CDEF is such a hack
	// that it is too unwieldy to track the necessary information to
	// replace them automatically. Anyway, this method will create
	// the CPopup_AC object and install it the same way it does for the
	// other controls that can be installed through the normal route.
	void	HintPopupControl(SInt16 itemNo, SInt16 cntlRsrcID);
	
	// FlashButton will flash a pushbutton object. If the specified itemNo
	// has been turned into an CButton_AC, or if it is a Control Mgr control,
	// it will work.
	void	FlashButton(SInt16 itemNo);
	
	// MarkDefaultOutlineUserItem is what you should use to inform the
	// AGA dialog object which item (if any) is your userItem proc for
	// drawing the default button outline. It needs this so that in
	// AGA mode it can remove the proc (since the AGA pushbutton
	// object draws its own AGA outline), and in AGA-suppressed mode
	// it will do nothing (so the userItem proc will draw the outline).
	void	MarkDefaultOutlineUserItem(SInt16 itemNo);

	// ShowAGAControl is a placeholder for informing an AGA control to show/hide itself.
	// A workaround is implemented for userItems that may continue to draw themselves.
	void	ShowAGAControl(SInt16 itemNo);
	
	// If you get an activate or deactivate event for an AGADialog, call this
	// function to let the dialog perform inactivate dimming of the controls.
	// This is automatically handled where possible inside the AGADialog
	// DialogSelect and filter methods, but you may also see the activate/deactivate
	// in your main event loop code.
	// Depending on whether your app's SIZE resource is set to accept suspend/resume
	// events, this will either be actual activate/deactivate events, or
	// suspend/resume events. For activate/deactivate events, which are per
	// window, use AGADialog::ObjectFromDialogPtr to get the pointer to the
	// appropriate dialog, if any, and then call its HandleActivateEvent method.
	// For suspend/resume events, you'll be keeping track of which window to do.
	void	HandleActivateEvent(bool becomeActive);
	
	// ObjectFromDialogPtr is a static function that looks at the
	// supplied dialog's refcon, and if it appears to be owned by
	// an CDialog_AC, the CDialog_AC* is returned. Otherwise, NULL
	// is returned, indicating that the dialog is not wrapped by us.
	static CDialog_AC*	ObjectFromDialogPtr(WindowPtr theWindow);

	enum { kAGADialogMarker = 'GCdm'} ;
	OSType	fMarker;	// a sanity check so we can see if a dialog's refcon really points to an CDialog_AC

protected:

	//
	// After constructing this object and giving it its DialogPtr (either
	// in the constructor or via SetDialog), we call BeginAGADialog
	// to let the object prepare all of its custom AGA stuff. Basically,
	// the rule is that this must be called before the dialog is made visible.
	//

	void	BeginAGADialog();

	//
	// Data usage:
	// When we install ourself into the dialog, suppose there are n items
	// in the item list.
	// We make room for n AGA objects in our array.
	// We append n userItems to the item list, one for each AGA object.
	// If an item is one we can replace, we hide the original, make
	// the appropriate AGA object take its coordinate position, and
	// install the user item callback for the AGA object.
	// If an item is not one we can replace, we store NULL in our array
	// and hide the useritem.
	// We always keep the original dialog items at the front of the DITL,
	// so that existing user code can continue to use the same item numbers
	// that it used to.
	//
	
	void				AdjustAlertPosition(AlertTHndl alrtHandle, const Rect* origBounds);
	void				AdjustDialogPosition(DialogTHndl dlogHandle, const Rect* origBounds);
	void				AdjustWindowPosition(SInt16 positionSpecifier, const Rect* origBounds);
	void				CreateAllCustomObjects();
	void				DestroyAllCustomObjects();
	void				CreateCustomObject(SInt16 itemIndex, Ptr* ditlItemPtr);
	void				DestroyCustomObject(SInt16 itemIndex);
	bool				IsCustomObject(SInt16 itemIndex);
	bool 				IsCustomObjectVisible(SInt16 itemIndex);
	void				SetCustomObjectVisibility(SInt16 itemIndex, bool isVisible);
	CustomObjectInfoPtr	GetCustomObjectInfoPtr(SInt16 itemIndex);
	CControl_AC*		GetAGAControlFromHandle(Handle itemHandle);
	CControl_AC*		GetAGAControlFromControlHandle(ControlHandle theControl);
	SInt16				GetDITLItemNoFromHandle(Handle itemHandle);
	
	CControl_AC*		BuildCustomObject(SInt16 itemIndex, bool* hideOriginal, Ptr* ditlItemPtr);
	
	bool				FilterMouseDown(EventRecord* theEvent, SInt16* itemHit);
	bool				FilterKeyDown(EventRecord* theEvent, SInt16* itemHit);

	void				GetNextDITLItemPtr(SInt16 itemType, Ptr* ditlItemPtr);
	bool				IsDITLItemEnabled(SInt16 itemNo);
	SInt16				GetDITLItemType(SInt16 itemNo);
	CButton_AC*			GetVerifiedPushButton(SInt16 itemNo);
	void				DoParamTextSubstitution(CStr255_AC& dialogString);

	bool					fIsAlert;		// true if using Alert(), not ModalDialog()
	DialogPtr				fDialogPtr;		// the Dialog Manager object we are wrapping
	SInt16					fNumAGAControls;// num elements in our array 
	CustomObjectInfoHandle	fAGAControls;	// handle to array of CustomObjectInfo
	ModalFilterUPP			fUserFilter;	// the filter passed by client code to our ModalDialog member function
	SInt16					fDefaultItemNo;	// itemNo of default button -- gets a button outline
	SInt16					fCancelItemNo;	// itemNo of cancel button -- cmd-. and esc keys
	CButton_AC*				fDefaultButton;	// null if default is not a push button
	CButton_AC*				fCancelButton;	// null if cancel is not a push button
};

//----------------------------------------------------------------------------------------
// CTempPort_AC
// CTempPort_AC is a stack-based object that sets the port, and
// restores it upon destruction.
//----------------------------------------------------------------------------------------

class EXPORT_AC CTempPort_AC
{
public:

	CTempPort_AC(GrafPtr itsPort);
	virtual ~CTempPort_AC();
	
	void	Save();
	void	Restore();

protected:

	GrafPtr fSavedPort;
};

//----------------------------------------------------------------------------------------
// CDialogEditTextFrame_AC
// CDialogEditTextFrame_AC draws a frame around an editText item in a dialog box.
//----------------------------------------------------------------------------------------

class EXPORT_AC CDialogEditTextFrame_AC : public CControl_AC
{
public:

	CDialogEditTextFrame_AC(const CRect_AC& bounds);
	virtual ~CDialogEditTextFrame_AC();
	
	virtual void	DrawObject();
};

//----------------------------------------------------------------------------------------
// CDialogIcon_AC
// CDialogIcon_AC draws a DITL Icon object (a 1-bit 'ICON').
// If an 'icl8' color icon family resource is available, it
// is used instead of the black & white ICON.
//----------------------------------------------------------------------------------------

class EXPORT_AC CDialogIcon_AC : public CControl_AC
{
public:

	CDialogIcon_AC(const CRect_AC& bounds, SInt16 iconID);
	virtual ~CDialogIcon_AC();
	
	virtual void	DrawObject();

protected:

	SInt16	fIconID;
};

#endif // __CDialog_AC__
