// CIconButton_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CIconButton_AC__
#define __CIconButton_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

// Mix in icons for these controls

#ifndef __CButton_AC__
#include "CButton_AC.h"
#endif

#ifndef __CCheckBox_AC__
#include "CCheckBox_AC.h"
#endif

#ifndef __CRadioButton_AC__
#include "CRadioButton_AC.h"
#endif

#ifndef __CPopup_AC__
#include "CPopup_AC.h"
#endif

// ACS

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

// Toolbox

#ifndef __ICONS__
#include <Icons.h>
#endif

#pragma mark MIcon_AC

//----------------------------------------------------------------------------------------
// MIcon_AC
// MIcon_AC is a mixin class for the classes that have an
// icon button appearance. There are two different basic visual
// attributes for a button: the frame type and the image type.
//
// The frame type determines what the button edges look like. There
// are 3 different frame types specified by the AGA. The size of
// the button determines which frame type is used, but you can
// force a given type to be used.
// - 40x40 or larger: kLargeFrame (4 pixels shading on each side)
//		This is room for a 32x32 'iclx' plus the shading.
// - 22x22 or larger: kSmallFrame (3 pixels shading on each side)
//		This is room for a 16x16 'icsx' plus the shading.
// - 20x16 or larger: kMiniFrame (2 pixels shading on each side)
//		This is room for a 16x12 'icmx' plus the shading.
// The constant kAutoFrame is used by default. This sets the frame type
// automatically based on the actual button bounds. The dimensions shown
// above are the ideal dimensions shown in the AGA spec.
//
// The image type determines how the actual picture image inside the
// button is drawn. By default the image type is an icon family of size
// based on the button size (according to the frame sizes listed above),
// but you can explicitly set the type, including non-icon-family types.
// In all cases, the image is centered inside the button frame.
// - kAutoIconFamily: 32x32, 16x16, or 12x12 icon rect, derived from
//		the button bounds, according to the ideal sizes list above,
//		using an icon family.
// - kLargeIcon: 32x32 icon rect ('iclx' size) using an icon family.
// - kSmallIcon: 16x16 icon rect ('icsx' size) using an icon family.
// - kMiniIcon: 16x12 icon rect ('icmx' size) using an icon family.
// - kColorIcon: arbitrary icon rect derived from / using a 'cicn' resource.
// - kPictureIcon: arbitrary icon rect derived from / using a 'PICT' resource.
//
// An icon button has two image resource IDs. Usually these are the same,
// but for some button types you can make them different. For example, if
// you want different icons for on/off when the button is used as a radio
// button or check box, you can set the IDs to different values.
//
// Note that mini icons are a rather rare resource type, and ResEdit 2.x
// doesn't have an editor for them. I think Resorcerer does.
//----------------------------------------------------------------------------------------

class EXPORT_AC MIcon_AC
{
public:

	enum { kNoIconID = -1 };
	enum ButtonImageType { kAutoIconFamily, kLargeIcon, kSmallIcon, kMiniIcon, kColorIcon, kPictureIcon };

	MIcon_AC(SInt16 offIconID = kNoIconID, SInt16 onIconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily);
	virtual ~MIcon_AC();
	
	virtual void	SetIconIDs(SInt16 offIconID, SInt16 onIconID);
	virtual void	SetFrameType(ButtonFrameType frameType);
	virtual void	SetImageType(ButtonImageType imageType);
	
	virtual void	DrawIconButton(const CRect_AC& bounds,
									bool isOn,
									bool isPressed,
									bool isEnabled,
									bool deep);

	static ButtonImageType	GetDefaultImageType(SInt16 buttonWidth);

protected:

	virtual PicHandle	GetPictureImage(CPoint_AC& imageSize, SInt16 pictureResourceID);
	virtual CIconHandle	GetColorIconImage(CPoint_AC& imageSize, SInt16 cicnResourceID);

	SInt16			fOffIconID;
	SInt16			fOnIconID;
	ButtonFrameType	fFrameType;
	ButtonImageType	fImageType;
};

#pragma mark CIconButton_AC

//----------------------------------------------------------------------------------------
// CIconButton_AC
// CIconButton_AC implements the standard AGA icon button, with
// pushbutton behavior; that is to say, it has no state. This is
// done by subclassing CButton_AC for tracking, and mixing in
// MIcon_AC for drawing.
//----------------------------------------------------------------------------------------

class EXPORT_AC CIconButton_AC : public CButton_AC, public MIcon_AC
{
public:

	CIconButton_AC(const CRect_AC& bounds, SInt16 iconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily);
	virtual ~CIconButton_AC();
	
protected:

	virtual void	DrawButtonNormal(bool deep);
	virtual void	DrawButtonPressed(bool deep);
	virtual void	DrawButtonDisabled(bool deep);
};

#pragma mark CIconCheckBox_AC

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC
// CIconCheckBox_AC implements the standard AGA icon button, with
// check box behavior; that is to say, it has an on or off state
// that toggles when hit. This is done by subclassing CCheckBox_AC
// for tracking and state, and mixing in MIcon_AC for drawing.
//----------------------------------------------------------------------------------------

class EXPORT_AC CIconCheckBox_AC : public CCheckBox_AC, public MIcon_AC
{
public:

	CIconCheckBox_AC(const CRect_AC& bounds, bool automaticState, SInt16 offIconID = kNoIconID, SInt16 onIconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily);
	virtual ~CIconCheckBox_AC();
	
protected:

	virtual void	DrawButtonNormal(bool deep);
	virtual void	DrawButtonPressed(bool deep);
	virtual void	DrawButtonDisabled(bool deep);
};

#pragma mark CIconRadioButton_AC

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC

// CIconRadioButton_AC implements the standard AGA icon button, with
// radio button behavior; that is to say, it has an on or off state
// that can be automatically made mutually exclusive with other
// buttons in its group. This is done by subclassing CRadioButton_AC
// for tracking and state, and mixing in MIcon_AC for drawing.
//----------------------------------------------------------------------------------------

class EXPORT_AC CIconRadioButton_AC : public CRadioButton_AC, public MIcon_AC
{
public:

	CIconRadioButton_AC(const CRect_AC& bounds, UInt32 groupIDPart1, UInt32 groupIDPart2, bool automaticState, SInt16 offIconID = kNoIconID, SInt16 onIconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily);
	virtual ~CIconRadioButton_AC();
	
protected:

	virtual void	DrawButtonNormal(bool deep);
	virtual void	DrawButtonPressed(bool deep);
	virtual void	DrawButtonDisabled(bool deep);
};

#pragma mark CIconPopup_AC

//----------------------------------------------------------------------------------------
// CIconPopup_AC

// CIconPopup_AC implements the standard AGA icon button, with
// popup menu behavior. This is done by subclassing CRadioButton_AC
// for tracking and state, and mixing in MIcon_AC for drawing.
//----------------------------------------------------------------------------------------

class EXPORT_AC CIconPopup_AC : public CPopup_AC, public MIcon_AC
{
public:

	CIconPopup_AC(const CRect_AC& bounds, MenuRef itsMenuRef = NULL, const CTextStyle_AC& itsTextStyle = CTextStyle_AC::kSystemStyle, DisposalKind menuRefDisposalKind = kDontDispose, RefHandling refHandling = kNothing, SInt16 offIconID = kNoIconID, SInt16 onIconID = kNoIconID, ButtonFrameType frameType = kAutoFrame, ButtonImageType imageType = kAutoIconFamily);
	virtual ~CIconPopup_AC();
	
protected:

	virtual void	DrawButtonNormal(bool deep);
	virtual void	DrawButtonPressed(bool deep);
	virtual void	DrawButtonDisabled(bool deep);
	
	virtual void	DrawMiniArrow(bool deep);
};




#endif // __CIconButton_AC__

