// CTabbedView_AC.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#pragma once

#ifndef __CTabbedView_AC__
#define __CTabbedView_AC__

// Appearance

#ifndef __Appearance_AC__
#include "Appearance_AC.h"
#endif

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __CSimpleOffscreenImage_AC__
#include "CSimpleOffscreenImage_AC.h"
#endif

// ACS

#ifndef __CTextStyle_AC__
#include "CTextStyle_AC.h"
#endif


#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif


//----------------------------------------------------------------------------------------
// CTabbedView_AC
// CTabbedView_AC implements an AGA tab panels control.
//
// You can specify the location of the tabs (top or bottom, left or right).
// (Only "top" tabs are currently implemented!!! Other values do NOT work yet.)
// You can specify whether the whole tab panel frame is drawn, or whether just
// the tabs themselves are drawn, which allows you to attach tabs to something
// that you are drawing yourself. The simple constructor decides this based on
// the object bounds: if it's too small, there's no frame.
//----------------------------------------------------------------------------------------

typedef struct
	{
	StringHandle	fLabel;
	SInt32			fTabWidth;
	bool			fDisabled;
	void*			fUserData;	// used by adapter classes for pane/view pointer
	} TabInfo, *TabInfoPtr, **TabInfoHandle;

class EXPORT_AC CTabbedView_AC : public CControl_AC
	{
	public:

		enum TabSize { kSmallTabs, kLargeTabs };
		enum TabsLocation { kTopTabs, kBottomTabs, kLeftTabs, kRightTabs };
		enum TabWidthKind { kVariableTabWidth, kCommonTabWidth };	// AGA spec is kVariableTabWidth
		enum { kDontShrink = false, kShrinkToFit = true };			// for SetNumTabs shrinkToFit

		CTabbedView_AC(const CRect_AC& bounds, SInt32 initialTabIndex, const CTextStyle_AC& textStyle, SInt16 labelsID, TabWidthKind widthKind = kVariableTabWidth, TabsLocation location = kTopTabs, bool drawFrame = true);
		virtual ~CTabbedView_AC();
		
		virtual void	DrawObject();
		virtual bool	TrackMouse(const CPoint_AC& mouseLocation);
		virtual void	SetBounds(const CRect_AC& bounds, bool redraw);
		
		virtual SInt32	GetCurrentTab();
		virtual void	SetCurrentTab(SInt32 tabIndex, bool redraw);
		virtual SInt32	GetLastTabClicked();
		
		virtual void	SetTabSize(TabSize size);							// specify large or small tabs
		virtual void	SetTabsLocation(TabsLocation location);				// specify tabs location
		virtual void	SetTabsWidth(TabWidthKind widthKind);				// specify tab vs. label width
		virtual SInt16	GetTabHeight();										// get "height" of the tabs
		virtual bool	HasFrame();											// true if we will draw a frame
		virtual void	SetLabelsFromResource(SInt16 stringListResourceID);	// creates 1 tab per string in the 'STR#'
		virtual void	SetNumTabs(SInt32 numLabels, bool shrinkToFit);	// ensures space for n tabs, optionally removes excess
		virtual SInt32	GetNumTabs();										// get number of tabs
		virtual void	RemoveTab(SInt32 tabIndex);							// removes specified tab from list
		virtual void	SetTabLabel(SInt32 tabIndex, const CStr255_AC& itsLabel);	// set one tab's label
		virtual void	GetTabLabel(SInt32 tabIndex, CStr255_AC& itsLabel);	// get one tab's label
		virtual void	SetTabEnable(SInt32 tabIndex, bool enable);		// enable/disable one tab
		virtual bool	IsTabEnabled(SInt32 tabIndex);						// get enable/disable state of one tab
		virtual void	SetTabUserData(SInt32 tabIndex, void* userData);	// set extra data value for one tab; used by adapter classes for panel ptr
		virtual void*	GetTabUserData(SInt32 tabIndex);					// get extra data value for one tab; used by adapter classes for panel ptr
		virtual void	CalculateTabWidths();								// recalc all tab widths using current labels
		virtual void	SetLabelsStyle(const CTextStyle_AC& textStyle);		// set the text style for the labels

	protected:

		enum { kSmallTabImageHeight = 19, kSmallTabImageWidth = 12 };
		enum { kLargeTabImageHeight = 24, kLargeTabImageWidth = 12 };
		enum { kPanelEndMargin = 6 };

		// CControl_AC mouse tracking overrides.
		virtual bool	HitTest(const CPoint_AC& mouseLocation);
		virtual void	SetTrackingState(bool isIn);

		virtual void	DrawBackground();
		virtual void	DrawFrame();
		virtual void	DrawTabs();
		
		virtual void	DrawFrameBox();
		virtual void	DrawFrameTop();
		virtual void	DrawOneTab(SInt32 tabIndex, bool pressed);
		virtual void	GetTabBounds(SInt32 tabIndex, CRect_AC& tabBounds);
		virtual void	CalculateTabWidthsByKind(TabWidthKind widthKind);
		
		virtual void	CalculateTabSizeInfo();
		
		virtual void	AllocateTabInfo(SInt32 numTabs, bool shrinkToFit);
		virtual void	DisposeTabInfo();
		
		virtual void	CopyTabImage(UInt32 imageIndex, const CPoint_AC& imageDestination, bool deep);
		
		SInt32			fNumTabs;
		SInt32			fCurrentTabIndex;
		SInt32			fLastTabClicked;

		TabInfoHandle	fTabInfo;				// handle to array of TabInfo
		TabSize			fTabSize;				// small or large
		TabsLocation	fTabsLocation;			// top, bottom, left, or right
		TabWidthKind	fTabWidthKind;			// variable or same
		CPoint_AC		fTabImageSize;			// depends on fTabSize & fTabsLocation
		bool			fDrawFrame;
		CTextStyle_AC	fTextStyle;

		// Drawing the tab contours manually is too tedious.
		// So we use offscreen PixMap structures and blast them onto the screen.
		// Current implementation doesn't use clip, so must be on an r2 background.

		enum { kSmallTabImages, kLargeTabImages, kNumImageTypes };
		enum { kTabNormalDeselectedL, kTabNormalDeselectedR,
				kTabNormalSelectedL, kTabNormalSelectedR,
				kTabPressedL, kTabPressedR,
				kTabDisabledDeselectedL, kTabDisabledDeselectedR,
				kTabDisabledSelectedL, kTabDisabledSelectedR,
				kNumTabImages };

public:
		static bool fgImagesAllocated;
		static void AllocateTabImages();
		static void DisposeTabImages();

protected:		
		static SInt8	kSmallTabImageValues [kNumTabImages] [kSmallTabImageHeight] [kSmallTabImageWidth];
		static SInt8	kLargeTabImageValues [kNumTabImages] [kLargeTabImageHeight] [kLargeTabImageWidth];

		static SInt8	k1BitSmallTabImageValues [kNumTabImages] [kSmallTabImageHeight] [kSmallTabImageWidth];
		static SInt8	k1BitLargeTabImageValues [kNumTabImages] [kLargeTabImageHeight] [kLargeTabImageWidth];

		static CAutoPtr_AC<CSimpleOffscreenImage_AC>	fgOffscreenTabImages [kNumTabImages] [kNumImageTypes];
		static CAutoPtr_AC<CSimpleOffscreenImage_AC>	fg1BitOffscreenTabImages [kNumTabImages] [kNumImageTypes];

	public:		
		static CTextStyle_AC kSmallTabPanelStyle;
	};

#endif // __CTabbedView_AC__

