// CDisclosureTriangle_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1996 by Trygve Isaacson.

#ifndef __CDisclosureTriangle_AC__
#include "CDisclosureTriangle_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

//========================================================================================
// CDisclosureTriangle_AC
// This class implements a disclosure triangle.
//========================================================================================

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::CDisclosureTriangle_AC
//----------------------------------------------------------------------------------------

CDisclosureTriangle_AC::CDisclosureTriangle_AC(const CRect_AC& bounds, bool automaticState) : 
	CControl_AC(bounds),
	fAutomaticState(automaticState),
	fIsDisclosed(kClosedState)
{
	// Construct as specified.

}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::~CDisclosureTriangle_AC
//----------------------------------------------------------------------------------------

CDisclosureTriangle_AC::~CDisclosureTriangle_AC()
{
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::DrawObject
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::DrawObject()
{
	// Draw the triangle in the current state.

	this->DrawTriangle(fIsDisclosed ? kDTDisclosed : kDTClosed);
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CDisclosureTriangle_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	// Let superclass handle tracking, and change our state
	// if we have automatic state change turned on.

	bool	wasItHit = CControl_AC::TrackMouse(mouseLocation);

	if (wasItHit && fAutomaticState)
		this->SetStateAnimate(! fIsDisclosed);

	return wasItHit;
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::GetState
//----------------------------------------------------------------------------------------

bool CDisclosureTriangle_AC::GetState()
{
	// Return the current state.

	return fIsDisclosed;
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::SetState
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::SetState(bool isDisclosed, bool redraw)
{
	// Set the state and optionally redraw.

	if (isDisclosed != fIsDisclosed)
	{
		fIsDisclosed = isDisclosed;
		
		if (redraw)
			this->DrawTriangle(fIsDisclosed ? kDTDisclosed : kDTClosed);
	}
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::SetStateAnimate
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::SetStateAnimate(bool isDisclosed)
{
	// Set the state, but animate the triangle as it changes
	// to the new state.

	const SInt32 kAnimationDelay = 4;

	if (isDisclosed)
	{
		this->DrawTriangle(kDTPressedClosed);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
		this->DrawTriangle(kDTIntermediate);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
		this->DrawTriangle(kDTPressedDisclosed);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
	}
	else
	{
		this->DrawTriangle(kDTPressedDisclosed);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
		this->DrawTriangle(kDTIntermediate);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
		this->DrawTriangle(kDTPressedClosed);
		DelayFuture_AC(kAnimationDelay);

		this->EraseBackground();
	}
	
	this->SetState(isDisclosed, kRedraw);
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::SetTrackingState
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::SetTrackingState(bool isIn)
{
	// Draw the triangle pressed or unpressed as specified.

	DTAnimationState	state;
	
	if (isIn)
		state = fIsDisclosed ? kDTPressedDisclosed : kDTPressedClosed;
	else
		state = fIsDisclosed ? kDTDisclosed : kDTClosed;

	this->DrawTriangle(state);
}

//----------------------------------------------------------------------------------------
// CDisclosureTriangle_AC::DrawTriangle
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::DrawTriangle(DTAnimationState state)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the triangle at the specified animation frame.

	CPoint_AC	fulcrum;
	CPoint_AC	corners[3];
	bool		drawEnabled = fEnabled && fActive;
	
	fulcrum.Set(GetBounds().left + 5, GetBounds().top + 5);

	switch (state)
	{
		case kDTClosed:
		case kDTPressedClosed:
			corners[0].Set(fulcrum.h - 2, fulcrum.v - 5);
			corners[1].Set(fulcrum.h + 3, fulcrum.v);
			corners[2].Set(fulcrum.h - 2, fulcrum.v + 5);
			break;
		case kDTIntermediate:
			corners[0].Set(fulcrum.h + 3, fulcrum.v - 5);
			corners[1].Set(fulcrum.h + 3, fulcrum.v + 3);
			corners[2].Set(fulcrum.h - 5, fulcrum.v + 3);
			break;
		case kDTPressedDisclosed:
		case kDTDisclosed:
			corners[0].Set(fulcrum.h + 5, fulcrum.v - 2);
			corners[1].Set(fulcrum.h, fulcrum.v + 3);
			corners[2].Set(fulcrum.h - 5, fulcrum.v - 2);
			break;
	}
	
	// Little trick: tweak state so that disabled state (gray
	// fill) is handled by pressed case code below. We already set
	// the color so they only set pressed fill color if enabled.
	DTAnimationState	deepState = state;
	
	if (! drawEnabled)
	{
		if (deepState == kDTClosed)
			deepState = kDTPressedClosed;
		else
			deepState = kDTPressedDisclosed;
	}
			
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			if (drawEnabled)
				kRGBColorRamp[rB].SetForeColor();
			else
				kRGBColorRamp[r5].SetForeColor();

			::MoveTo(corners[0].h, corners[0].v);
			::LineTo(corners[1].h, corners[1].v);
			::LineTo(corners[2].h, corners[2].v);
			::LineTo(corners[0].h, corners[0].v);
			
			switch (deepState)
			{
				case kDTClosed:
					kRGBColorRamp[rP1].SetForeColor();
					::MoveTo(corners[0].h + 1, corners[0].v + 2);
					::Line(0, 6);
					kRGBColorRamp[rP2].SetForeColor();
					::Line(1, -1);
					::Line(0, -4);
					::Line(1, 1);
					::Line(0, 1);
					::Move(1, 0);
					kRGBColorRamp[rP3].SetForeColor();
					::Line(-2, 2);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(corners[2].h + 1, corners[2].v);
					::Line(4, -4);
					kRGBColorRamp[r4].SetForeColor();
					::Move(1, 0);
					::Line(-5, 5);
					break;
				case kDTPressedClosed:
					if (drawEnabled)
						kRGBColorRamp[rP4].SetForeColor();

					::MoveTo(corners[0].h + 1, corners[0].v + 2);
					::Line(0, 6);
					::Move(1, -1);
					::Line(0, -4);
					::Move(1, 1);
					::Line(0, 2);
					::Line(1, -1);

					kRGBColorRamp[r2].SetForeColor();
					::MoveTo(corners[2].h + 1, corners[2].v);
					::Line(4, -4);
					::Move(1, 0);
					::Line(-5, 5);
					break;
				case kDTIntermediate:
					kRGBColorRamp[rP4].SetForeColor();
					::MoveTo(corners[0].h - 1, corners[0].v + 2);
					::Line(-5, 5);
					::Move(1, 0);
					::Line(4, -4);
					::Move(0, 1);
					::Line(-3, 3);
					::Move(1, 0);
					::Line(2, -2);
					::Move(0, 1);
					::Line(-1, 1);
					::Line(1, 0);
					break;
				case kDTPressedDisclosed:
					if (drawEnabled)
						kRGBColorRamp[rP4].SetForeColor();

					::MoveTo(corners[2].h + 2, corners[2].v + 1);
					::Line(6, 0);
					::Move(-1, 1);
					::Line(-4, 0);
					::Move(1, 1);
					::Line(2, 0);
					::Line(-1, 1);

					kRGBColorRamp[r2].SetForeColor();
					::MoveTo(corners[0].h, corners[0].v + 1);
					::Line(-4, 4);
					::Move(0, 1);
					::Line(5, -5);
					break;
				case kDTDisclosed:
					kRGBColorRamp[rP1].SetForeColor();
					::MoveTo(corners[2].h + 2, corners[2].v + 1);
					::Line(6, 0);
					kRGBColorRamp[rP2].SetForeColor();
					::Line(-1, 1);
					::Line(-4, 0);
					::Line(1, 1);
					::Line(1, 0);
					::Move(0, 1);
					kRGBColorRamp[rP3].SetForeColor();
					::Line(2, -2);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(corners[0].h, corners[0].v + 1);
					::Line(-4, 4);
					kRGBColorRamp[r4].SetForeColor();
					::Move(0, 1);
					::Line(5, -5);
					break;
			}
		}
		else	// 1-bit
		{
			PolyHandle	theTriangle = ::OpenPoly();

			::MoveTo(corners[0].h, corners[0].v);
			::LineTo(corners[1].h, corners[1].v);
			::LineTo(corners[2].h, corners[2].v);
			::LineTo(corners[0].h, corners[0].v);
			
			::ClosePoly();
			
			switch (state)
			{
				case kDTClosed:
				case kDTDisclosed:
					kRGBColorRamp[rW].SetForeColor();
					::PaintPoly(theTriangle);
					kRGBColorRamp[rB].SetForeColor();
					::FramePoly(theTriangle);

					if (! drawEnabled)
					{
						PenPatGray();
						::PenMode(patBic);
						::PaintRect(GetBounds());

						::PenNormal();
					}
					break;
				case kDTPressedClosed:
				case kDTIntermediate:
				case kDTPressedDisclosed:
					kRGBColorRamp[rB].SetForeColor();
					::PaintPoly(theTriangle);
					break;
			}
			
			::KillPoly(theTriangle);
			::PenNormal();
		}
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void CDisclosureTriangle_AC::EraseBackground()
{
	CTempDrawingEnvironment_AC	env;

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			this->ApplyBackgroundColor();
		else
			kRGBColorRamp[rW].SetForeColor();

		::PaintRect(GetBounds());
	}
}


