//----------------------------------------------------------------------------------------
// CGroupsContainer_AC.cp 
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __CGroupsContainer_AC__
#include "CGroupsContainer_AC.h"
#endif

// ACS

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

#if 0
#ifndef __Exceptions_AC__
#include "Exceptions_AC.h"
#endif
#endif

//========================================================================================
// Global data
//========================================================================================

CAutoPtr_AC<CGroupsContainer_AC> CGroupsContainer_AC::fgInstance;

const UInt32 MExclusiveObject_AC::kNoGroupID = 0xFFFFFFFF;

//========================================================================================
// CLASS MExclusiveObject_AC
// This class is a mixin class for objects that can be included in a
// group of mutually exclusively selected objects, i.e. radio buttons
// or icon buttons. When any group member is selected, the others in
// the group are sent the TurnOff message.
//========================================================================================

//----------------------------------------------------------------------------------------
// MExclusiveObject_AC
//----------------------------------------------------------------------------------------

MExclusiveObject_AC::MExclusiveObject_AC(UInt32 groupIDPart1, UInt32 groupIDPart2) :
	fGroupIDPart1(groupIDPart1),
	fGroupIDPart2(groupIDPart2)
{
	// Construct by setting the ID pair and adding to
	// the group.
	CGroupsContainer_AC::Instance()->AddGroupMember(this);
}

//----------------------------------------------------------------------------------------
// MExclusiveObject_AC
//----------------------------------------------------------------------------------------

MExclusiveObject_AC::~MExclusiveObject_AC()
{
	// Destruct by removing from the group.

	CGroupsContainer_AC::Instance()->RemoveGroupMember(this);
}

//----------------------------------------------------------------------------------------
// MExclusiveObject_AC
//----------------------------------------------------------------------------------------

void MExclusiveObject_AC::SetGroupID(UInt32 groupIDPart1, UInt32 groupIDPart2)
{
	CGroupsContainer_AC::Instance()->RemoveGroupMember(this);

	fGroupIDPart1 = groupIDPart1;
	fGroupIDPart2 = groupIDPart2;

	CGroupsContainer_AC::Instance()->AddGroupMember(this);
}

//----------------------------------------------------------------------------------------
// MExclusiveObject_AC
//----------------------------------------------------------------------------------------

void MExclusiveObject_AC::TurnOff()
{
	// Override as needed.
}


//========================================================================================
// CLASS CGroupsContainer_AC
// This class is used internally to maintain the MExclusiveObject_AC
// behavior that allows radio buttons to automatically maintain
// standard radio button group XOR behavior.
//========================================================================================

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

CGroupsContainer_AC::CGroupsContainer_AC() :
	fGroupsArraySize(0),
	fGroups((SGroupMember_AC**) ::NewHandle(0))
{
	::ThrowIfNULL_AC(fGroups);
	this->ExpandArray();
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

CGroupsContainer_AC::~CGroupsContainer_AC()
{
	::DisposeIfHandle_AC(reinterpret_cast<Handle>(fGroups));
	fGroups = NULL;
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

void CGroupsContainer_AC::AddGroupMember(MExclusiveObject_AC* groupMember)
{
	this->AddGroupMember(groupMember, true);
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

void CGroupsContainer_AC::RemoveGroupMember(MExclusiveObject_AC* groupMember)
{
	// If the supplied object has an ID pair, search for it in the
	// members array and zero it out.

	if ((fGroups != NULL) && (groupMember->ValidGroupIDs()))
	{
		SGroupMember_AC*	memberPtr;

		CTempHandleLock_AC tempHandleLock(reinterpret_cast<Handle>(fGroups));

		memberPtr = *fGroups;

		for (UInt32 i = 0; i < fGroupsArraySize; ++i)
		{
			if ((memberPtr->sGroupIDPart1 == groupMember->fGroupIDPart1) &&
				(memberPtr->sGroupIDPart2 == groupMember->fGroupIDPart2) &&
				(memberPtr->sGroupMember == groupMember))
			{
				memberPtr->sGroupIDPart1 = MExclusiveObject_AC::kNoGroupID;
				memberPtr->sGroupIDPart2 = MExclusiveObject_AC::kNoGroupID;
				memberPtr->sGroupMember = NULL;
			}
				
			++memberPtr;
		}
	}
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

void CGroupsContainer_AC::HitGroupMember(MExclusiveObject_AC* groupMember)
{
	// If the supplied object has an ID pair, send every other
	// member of its group the TurnOff message.

	if ((fGroups != NULL) && (groupMember->ValidGroupIDs()))
	{
		SGroupMember_AC*	memberPtr;

		CTempHandleLock_AC temphandleLock(reinterpret_cast<Handle>(fGroups));

		memberPtr = *fGroups;

		for (UInt32 i = 0; i < fGroupsArraySize; ++i)
		{
			if ((memberPtr->sGroupIDPart1 == groupMember->fGroupIDPart1) &&
				(memberPtr->sGroupIDPart2 == groupMember->fGroupIDPart2) &&
				(memberPtr->sGroupMember != groupMember))
				memberPtr->sGroupMember->TurnOff();
				
			++memberPtr;
		}
	}
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

void CGroupsContainer_AC::AddGroupMember(MExclusiveObject_AC* groupMember, bool allowExpand)
{
	// If the supplied object has an ID pair, add it to the
	// members array. Try to re-use a zeroed-out entry; if none
	// is left, expand the handle by another chunk.

	if ((fGroups != NULL) && (groupMember->ValidGroupIDs()))
	{
		SGroupMember_AC*	memberPtr;
		bool				done = false;

		{ // scope the handle lock before handle expansion
			CTempHandleLock_AC tempHandleLock(reinterpret_cast<Handle>(fGroups));

			memberPtr = *fGroups;

			for (UInt32 i = 0; (i < fGroupsArraySize) && ! done; ++i)
				{
				if ((memberPtr->sGroupIDPart1 == MExclusiveObject_AC::kNoGroupID) &&
					(memberPtr->sGroupIDPart2 == MExclusiveObject_AC::kNoGroupID))
					{
					memberPtr->sGroupIDPart1 = groupMember->fGroupIDPart1;
					memberPtr->sGroupIDPart2 = groupMember->fGroupIDPart2;
					memberPtr->sGroupMember = groupMember;
					done = true;
					}
					
				++memberPtr;
			}
		}
		if (allowExpand && ! done)
		{
			this->ExpandArray();
			this->AddGroupMember(groupMember, false);	// false = recur once only!
		}
	}
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

void CGroupsContainer_AC::ExpandArray()
{
	if (fGroups != NULL)
	{
		::SetHandleSize((Handle) fGroups, (fGroupsArraySize + kGroupArrayChunkSize) * sizeof(SGroupMember_AC));
		
		if (::MemError() == noErr)
		{
			fGroupsArraySize += kGroupArrayChunkSize;
			
			for (UInt32 i = fGroupsArraySize - kGroupArrayChunkSize; i < fGroupsArraySize; ++i)
			{
				(*fGroups)[i].sGroupIDPart1 = MExclusiveObject_AC::kNoGroupID;
				(*fGroups)[i].sGroupIDPart2 = MExclusiveObject_AC::kNoGroupID;
				(*fGroups)[i].sGroupMember = NULL;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CGroupsContainer_AC
//----------------------------------------------------------------------------------------

CGroupsContainer_AC* CGroupsContainer_AC::Instance()
{
	if (fgInstance.IsNULL())
	{
		// Allocate the group container.
		fgInstance = TH_new CGroupsContainer_AC;
	}
	return fgInstance;
}