// CIconButton_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CIconButton_AC__
#include "CIconButton_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif


//========================================================================================
// MIcon_AC
// This mixin class handles button drawing for the different types
// of icon button classes. This class draws the button, and subclasses
// for push/radio/checkbox that mix it in handle the behavioral aspects.
//
// Ideal button sizes, to match AGA specification:
// - Large icon button: 40x40 (32x32 iclx plus 4 pixel border on each side)
// - Small icon button: 22x22 (16x16 icsx plus 3 pixel border on each side)
// - Mini icon button:  20x16 (16x12 icmx plus 2 pixel border on each side)
// Note that mini icons are a rather rare resource that ResEdit 2.x doesn't grok.
//========================================================================================

//----------------------------------------------------------------------------------------
// MIcon_AC::MIcon_AC
//----------------------------------------------------------------------------------------


MIcon_AC::MIcon_AC(SInt16 offIconID, SInt16 onIconID, ButtonFrameType frameType, ButtonImageType imageType) :
	fOffIconID(offIconID),
	fOnIconID(onIconID),
	fFrameType(frameType),
	fImageType(imageType)
{
	// Construct with specified icon IDs.
}

//----------------------------------------------------------------------------------------
// MIcon_AC::~MIcon_AC
//----------------------------------------------------------------------------------------

MIcon_AC::~MIcon_AC()
{
}

//----------------------------------------------------------------------------------------
// MIcon_AC::SetIconIDs
//----------------------------------------------------------------------------------------

void MIcon_AC::SetIconIDs(SInt16 offIconID, SInt16 onIconID)
{
	// Set icon IDs as specified.

	fOffIconID = offIconID;
	fOnIconID = onIconID;
}

//----------------------------------------------------------------------------------------
// MIcon_AC::SetFrameType
//----------------------------------------------------------------------------------------

void MIcon_AC::SetFrameType(ButtonFrameType frameType)
{
	// Set button frame type as specified.

	fFrameType = frameType;
}

//----------------------------------------------------------------------------------------
// MIcon_AC::SetImageType
//----------------------------------------------------------------------------------------

void MIcon_AC::SetImageType(ButtonImageType imageType)
{
	// Set buttom image type as specified.

	fImageType = imageType;
}

//----------------------------------------------------------------------------------------
// MIcon_AC::DrawIconButton
//----------------------------------------------------------------------------------------

void MIcon_AC::DrawIconButton(const CRect_AC& bounds,
										bool isOn,
										bool isPressed,
										bool isEnabled,
										bool deep)
{
	// Draw the icon button in the appropriate state.

	CRect_AC	r(bounds);
	
	//
	// Draw the blank button with the appropriate frame type.
	//
	
	AGADrawBevelButtonAtDepth(deep, bounds, fFrameType, isOn, isPressed, isEnabled);

	//
	// Determine the target area into which the image must
	// be drawn.
	//

	ButtonImageType	imageType = fImageType;
	CPoint_AC		imageSize;
	CPoint_AC		imageCenter;
	SInt16			imageResourceID = isOn ? fOnIconID : fOffIconID;
	PicHandle		pictureHandle;
	CIconHandle		colorIconHandle;
	
	imageCenter.h = r.left + ((r.right - r.left) / 2);
	imageCenter.v = r.top + ((r.bottom - r.top) / 2);
	
	if (imageType == kAutoIconFamily)
		imageType = MIcon_AC::GetDefaultImageType(r.right - r.left);

	switch (imageType)
	{
		case kPictureIcon:
			pictureHandle = this->GetPictureImage(imageSize, imageResourceID);
			break;

		case kColorIcon:
			colorIconHandle = this->GetColorIconImage(imageSize, imageResourceID);
			break;

		case kMiniIcon:
			imageSize.Set(16, 12);	// 16x12 icm#
			break;

		case kSmallIcon:
			imageSize.Set(16, 16);	// 16x16 ics#
			break;

		case kLargeIcon:
		default:
			imageSize.Set(32, 32);	// 32x32 icl#
			break;
	}
	
	r.left = imageCenter.h - (imageSize.h / 2);
	r.right = r.left + imageSize.h;
	r.top = imageCenter.v - (imageSize.v / 2);
	r.bottom = r.top + imageSize.v;

	//
	// Draw the image onto the blank button.
	//

	IconTransformType	aTransform = isPressed ? ttSelected : (isEnabled ? ttNone : ttDisabled);

	if (imageResourceID != kNoIconID)
	{
		kRGBColorRamp[rB].SetForeColor();

		switch (imageType)
		{
			case kPictureIcon:
				if (pictureHandle != NULL)
				{
					::DrawPicture(pictureHandle, &r);
					// The PicHandle is not explicitly released. Just let it get purged as needed.
				}
				break;

			case kColorIcon:
				if (colorIconHandle != NULL)
				{
					(void) ::PlotCIconHandle(&r, atNone, aTransform, colorIconHandle);
					::DisposeCIcon(colorIconHandle);
				}
				break;

			case kMiniIcon:
			case kSmallIcon:
			case kLargeIcon:
			default:
				(void) ::PlotIconID(&r, atNone, aTransform, imageResourceID);
				break;
		}
	}
}

//----------------------------------------------------------------------------------------
// MIcon_AC::GetPictureImage
//----------------------------------------------------------------------------------------

PicHandle MIcon_AC::GetPictureImage(CPoint_AC& imageSize, SInt16 pictureResourceID)
{
	PicHandle	pictureHandle = ::GetPicture(pictureResourceID);
					
	if (pictureHandle != NULL)
	{
		CRect_AC	frame((**pictureHandle).picFrame);

		imageSize.Set(frame.right - frame.left, frame.bottom - frame.top);
	}

	return pictureHandle;
}

//----------------------------------------------------------------------------------------
// MIcon_AC::GetColorIconImage
//----------------------------------------------------------------------------------------

CIconHandle MIcon_AC::GetColorIconImage(CPoint_AC& imageSize, SInt16 cicnResourceID)
{
	CIconHandle	colorIconHandle = ::GetCIcon(cicnResourceID);
					
	if (colorIconHandle != NULL)
	{
		CRect_AC	frame((**colorIconHandle).iconBMap.bounds);

		imageSize.Set(frame.right - frame.left, frame.bottom - frame.top);
	}

	return colorIconHandle;
}

//----------------------------------------------------------------------------------------
// MIcon_AC::GetDefaultImageType
//----------------------------------------------------------------------------------------

MIcon_AC::ButtonImageType MIcon_AC::GetDefaultImageType(SInt16 buttonWidth)
{
	if (buttonWidth > 39)
		return kLargeIcon;
	else if (buttonWidth > 21)
		return kSmallIcon;
	else
		return kMiniIcon;
}


//========================================================================================
// CIconButton_AC
// This class implements an icon-based pushbutton object, such as
// a toolbar icon command button.
//========================================================================================

//----------------------------------------------------------------------------------------
// CIconButton_AC::CIconButton_AC
//----------------------------------------------------------------------------------------

CIconButton_AC::CIconButton_AC(const CRect_AC& bounds, SInt16 iconID, ButtonFrameType frameType, ButtonImageType imageType)
: CButton_AC(bounds, CTextStyle_AC::kSystemStyle), 
  MIcon_AC(iconID, iconID, frameType, imageType)
{
	// Just call through to mixed in constructors.
}

//----------------------------------------------------------------------------------------
// CIconButton_AC::~CIconButton_AC
//----------------------------------------------------------------------------------------

CIconButton_AC::~CIconButton_AC()
{
}

//----------------------------------------------------------------------------------------
// CIconButton_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CIconButton_AC::DrawButtonNormal(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kNotPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconButton_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CIconButton_AC::DrawButtonPressed(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconButton_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CIconButton_AC::DrawButtonDisabled(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kNotPressed, kDisabled, deep);
}

//========================================================================================
// CIconCheckBox_AC
// This class implements an icon-based checkbox object, such as
// a toolbar icon on/off button.
//========================================================================================

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC::CIconCheckBox_AC
//----------------------------------------------------------------------------------------

CIconCheckBox_AC::CIconCheckBox_AC(const CRect_AC& bounds, bool automaticState, SInt16 offIconID, SInt16 onIconID, ButtonFrameType frameType, ButtonImageType imageType)
: CCheckBox_AC(bounds, CTextStyle_AC::kSystemStyle, kEmptyString_AC, automaticState), 
  MIcon_AC(offIconID, onIconID, frameType, imageType)
{
	// Just call through to mixed in constructors.
}

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC::~CIconCheckBox_AC
//----------------------------------------------------------------------------------------

CIconCheckBox_AC::~CIconCheckBox_AC()
{
}

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CIconCheckBox_AC::DrawButtonNormal(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kCheckBoxOff, kNotPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CIconCheckBox_AC::DrawButtonPressed(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kCheckBoxOff, kPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconCheckBox_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CIconCheckBox_AC::DrawButtonDisabled(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kCheckBoxOff, kNotPressed, kDisabled, deep);
}

//========================================================================================
// CIconRadioButton_AC
// This class implements an icon-based pushbutton object, such as
// a toolbar icon XOR group member button.
//========================================================================================

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC::CIconRadioButton_AC
//----------------------------------------------------------------------------------------

CIconRadioButton_AC::CIconRadioButton_AC(const CRect_AC& bounds, UInt32 groupIDPart1, UInt32 groupIDPart2, bool automaticState, SInt16 offIconID, SInt16 onIconID, ButtonFrameType frameType, ButtonImageType imageType)
: CRadioButton_AC(bounds, CTextStyle_AC::kSystemStyle, kEmptyString_AC, groupIDPart1, groupIDPart2, automaticState), 
  MIcon_AC(offIconID, onIconID, frameType, imageType)
{
	// Just call through to mixed in constructors.
}

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC::~CIconRadioButton_AC
//----------------------------------------------------------------------------------------

CIconRadioButton_AC::~CIconRadioButton_AC()
{
}

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CIconRadioButton_AC::DrawButtonNormal(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kRadioButtonOff, kNotPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CIconRadioButton_AC::DrawButtonPressed(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kRadioButtonOff, kPressed, kEnabled, deep);
}

//----------------------------------------------------------------------------------------
// CIconRadioButton_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CIconRadioButton_AC::DrawButtonDisabled(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), fValue != kRadioButtonOff, kNotPressed, kDisabled, deep);
}

//========================================================================================
// CIconPopup_AC
// This class implements an icon-based popup object.
//========================================================================================

//----------------------------------------------------------------------------------------
// CIconPopup_AC::CIconPopup_AC
//----------------------------------------------------------------------------------------

CIconPopup_AC::CIconPopup_AC(const CRect_AC& bounds, MenuRef itsMenuRef, const CTextStyle_AC& itsTextStyle, DisposalKind menuRefDisposalKind, RefHandling refHandling, SInt16 offIconID, SInt16 onIconID, ButtonFrameType frameType, ButtonImageType imageType)
: CPopup_AC(bounds, 0, kEmptyString_AC, teJustLeft, CTextStyle_AC::kSystemStyle, itsTextStyle, kFixedWidth, itsMenuRef, menuRefDisposalKind, refHandling), 
  MIcon_AC(offIconID, onIconID, frameType, imageType)
{
	// Just call through to mixed in constructors.
}

//----------------------------------------------------------------------------------------
// CIconPopup_AC::~CIconPopup_AC
//----------------------------------------------------------------------------------------

CIconPopup_AC::~CIconPopup_AC()
{
}

//----------------------------------------------------------------------------------------
// CIconPopup_AC::DrawButtonNormal
//----------------------------------------------------------------------------------------

void CIconPopup_AC::DrawButtonNormal(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kNotPressed, kEnabled, deep);
	DrawMiniArrow(deep);
}

//----------------------------------------------------------------------------------------
// CIconPopup_AC::DrawButtonPressed
//----------------------------------------------------------------------------------------

void CIconPopup_AC::DrawButtonPressed(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kPressed, kEnabled, deep);
	DrawMiniArrow(deep);
}

//----------------------------------------------------------------------------------------
// CIconPopup_AC::DrawButtonDisabled
//----------------------------------------------------------------------------------------

void CIconPopup_AC::DrawButtonDisabled(bool deep)
{
	// Call icon button parent class with appropriate parameters.

	MIcon_AC::DrawIconButton(GetBounds(), false, kNotPressed, kDisabled, deep);
	DrawMiniArrow(deep);
}

//----------------------------------------------------------------------------------------
// CIconPopup_AC::DrawMiniArrow
//----------------------------------------------------------------------------------------

void CIconPopup_AC::DrawMiniArrow(bool /*deep*/)
{
	// Add arrow.  Arrow size is 5x3 pixels near the bottom right corner:
	//
	//											     X
	//										XXXXX    X
	//										 XXX     X
	//										  X      X
	//											     X
	//											     X
	//											     X
	//									XXXXXXXXXXXXX
	
	CPoint_AC arrowPt(GetBounds()[botRight]);
	arrowPt.v -= 7;
	arrowPt.h -= 9;
	
	if (IsEnabled() && IsActive())
		kRGBColorRamp[rB].SetForeColor();
	else
		kRGBColorRamp[r5].SetForeColor();
	
	::MoveTo(arrowPt.h, arrowPt.v);
	::LineTo(arrowPt.h + 4, arrowPt.v);
	::Move(-1, 1);
	::LineTo(arrowPt.h + 1, arrowPt.v + 1);
	::Move(1, 1);
	::Line(0, 0);
}