//----------------------------------------------------------------------------------------
// CIndicator_AC.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __CIndicator_AC__
#include "CIndicator_AC.h"
#endif


//========================================================================================
// CIndicator_AC
// This is the abstract superclass for scroll bars and sliders.
// They all have a range, and a value shown by a draggable indicator.
//========================================================================================

CIndicator_AC::CIndicator_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, SInt32 pageSize, bool liveTracking, bool proportional)
	: CControl_AC(bounds),
	fMinimum(minimum),
	fMaximum(maximum),
	fValue(initialValue),
	fPageSize(pageSize),
	fGhostValue(fValue),
	fLiveTracking(liveTracking),
	fIsProportional(proportional),
	fIsHorizontal((bounds.right - bounds.left) > (bounds.bottom - bounds.top)),
	fIsPressed(false),
	
	fNotificationRoutine(NULL)
{
// Construct with specified options.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

CIndicator_AC::~CIndicator_AC()
{
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::InstallNotificationRoutine(CIndicatorCallback_AC notificationRoutine, void* userData)
{
	// Install the supplied function pointer and user data to
	// be called during notification.

	fNotificationRoutine = notificationRoutine;
	fUserData = userData;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetAttributes(bool isHorizontal, bool liveTracking, bool isProportional)
{
	// Set options as specified.

	fIsHorizontal = isHorizontal;
	fLiveTracking = liveTracking;
	fIsProportional = isProportional;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetLiveTracking(bool liveTracking)
{
	// Turn live tracking on or off.

	fLiveTracking = liveTracking;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawObject()
{
	// Call component drawing functions. These are all overridden
	// by subclasses.

	this->DrawBackground();
	this->DrawIndicatorTrack();
	this->DrawTrackEnds();
	this->DrawIndicator();
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

bool CIndicator_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	if (! fEnabled)
		return false;

	// Track the mouse and return true if the indicator
	// value was changed.

	return this->TrackPart(mouseLocation, this->TrackTestPart(mouseLocation));
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

bool CIndicator_AC::TrackPart(const CPoint_AC& mouseLocation, SInt32 partHit)
{
	// Track the mouse in the specified part and return true
	// if the indicator value was changed.

	if (partHit != kIndicatorPress)
		return false;

	SInt32		originalValue = fValue;
	CPoint_AC	oldPoint = mouseLocation;
	CPoint_AC	newPoint = mouseLocation;
	SInt32		oldValue = originalValue;
	SInt32		newValue = originalValue;
	bool		isButtonDown = true;	// ensure at least one time through
	
	fGhostValue = fValue;
	fIsPressed = true;
	this->DrawIndicator();	// will draw it pressed
	
	while (isButtonDown)
	{
		::GetMouse(&newPoint);	// gives local coordinates
		
		if (this->IsValidIndicatorTrackMouse(newPoint) &&
			! ::EqualPt(newPoint, oldPoint))
		{
			newValue = this->GetValueFromMouseDelta(oldValue, mouseLocation, newPoint);
		
			if (fLiveTracking)
			{
				SInt32	oldValue = fValue;

				this->SetRawValue(newValue, kRedraw);	// will draw new indicator
				
				if (fValue != oldValue)
					this->NotifyValue();
			}
			else
				this->SetGhostValue(newValue, kRedraw);
			
			oldPoint = newPoint;
		}
		
		isButtonDown = ::StillDown();
	}
	
	fIsPressed = false;
	
	if (fLiveTracking)
		this->DrawIndicator();	// unpressed
	else
	{
		this->RemoveGhost(newValue);
		this->DrawIndicator();	// unpressed

		if (newValue != fValue)	// don't notify if no change
		{
			this->SetRawValue(newValue, kRedraw);	// will draw new indicator
			this->NotifyValue();
		}
	}
	
	return (fValue != originalValue);
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::GetValue() const
{
	// Return the current indicator value.

	return fValue;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetGhostValue(SInt32 newValue, bool redraw)
{
	// Set the ghost indicator value, and optionally redraw it
	// at its new position.

	if (newValue > fMaximum)
		newValue = fMaximum;
	else if (newValue < fMinimum)
		newValue = fMinimum;

	if (newValue != fGhostValue)
		{
		if (redraw)
			this->RemoveGhost(newValue);

		fGhostValue = newValue;
		
		if (redraw)
			this->DrawGhost();
	}
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetValue(SInt32 newValue, bool redraw)
{
	this->SetRawValue(newValue, redraw);
}

void CIndicator_AC::SetRawValue(SInt32 newValue, bool redraw)
{
	// Set the indicator value, and optionally redraw it
	// at its new position.

	if (newValue > fMaximum)
		newValue = fMaximum;
	else if (newValue < fMinimum)
		newValue = fMinimum;

	if (newValue != fValue)
	{
		if (redraw)
			this->RemoveIndicator(newValue);

		fValue = newValue;
		
		if (redraw)
			this->DrawIndicator();
	}
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::GetMinimum() const
{
	return fMinimum;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::GetMaximum() const
{
	return fMaximum;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw)
{
	// Set the min/max range, and optionally redraw.

	if ((newMinimum != fMinimum) || (newMaximum != fMaximum))
	{
		if (redraw)
			this->RemoveIndicator(fValue);

		fMinimum = newMinimum;
		fMaximum = newMaximum;
		
		fValue = MaxSInt32(fMinimum, MinSInt32(fValue, fMaximum));

		if (redraw)
			this->DrawIndicator();
	}
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::GetPageSize()
{
	// Return the current indicator page size.

	return fPageSize;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::SetPageSize(SInt32 newPageSize, bool redraw)
{
	// Set the indicator page size. Redraw is not
	// necessary if the indicator is not proportional.

	if (newPageSize != fPageSize)
	{
		if (redraw && fIsProportional)
			this->RemoveIndicator(fValue);

		fPageSize = newPageSize;
		
		if (redraw && fIsProportional)
			this->DrawIndicator();
	}
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::TrackTestPart(const CPoint_AC& /*mouseLocation*/)
{
	// Must be overridden. Return the part value that the
	// specified mouse position would hit.

	return kNoTrackPress;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

bool CIndicator_AC::IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation)
{
	// Usually overridden. Return true if the specified mouse
	// position should be considered "in" for tracking purposes
	// after the initial hit has succeeded. Return false if the
	// specified mouse position is too far away from the control
	// to use it for the next momentary tracking position.

	return ::PtInRect(mouseLocation, &fBounds);
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

SInt32 CIndicator_AC::GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& /*originalMouseLocation*/, const CPoint_AC& /*newMouseLocation*/)
{
	// Must be overridden. Return the indicator value that should be
	// applied assuming the supplied starting value and mouse hit point,
	// and the supplied current mouse tracking location.
	return originalValue;
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawBackground()
{
	// Override to paint the background area so that the
	// indicator drawn at its old position will be wiped out.
	// Clipping will be already set up to avoid excess flicker.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawIndicatorTrack()
{
	// Override to draw the empty indicator track. Must include
	// all possible indicator coverage.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawTrackEnds()
{
	// Override to draw the track ends outside the possible
	// indicator coverage if not drawn in DrawIndicatorTrack.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawIndicator()
{
	// Override to draw the indicator at its current value/state.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::RemoveIndicator(SInt32 /*newValue*/)
{
	// Override to draw/erase whatever is needed to remove the
	// current indicator in preparation for being drawn at
	// the specified new value. The new value is supplied so
	// that erasing can be kept to a minimal area.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::DrawGhost()
{
	// Override to draw the indicator in ghost mode at
	// the current ghost value.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::RemoveGhost(SInt32 /*newValue*/)
{
	// Override to draw/erase whatever is needed to remove the
	// ghost indicator in preparation for being drawn at
	// the specified new value. The new value is supplied so
	// that erasing can be kept to a minimal area.
}

//----------------------------------------------------------------------------------------
// CIndicator_AC
//----------------------------------------------------------------------------------------

void CIndicator_AC::NotifyValue()
{
	// Call the installed notification routine, if any, with
	// the installed user data and our new value.

	if (fNotificationRoutine != NULL)
		(*(fNotificationRoutine))(this, GetValue(), fUserData);
}

