// CScrollBar_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CScrollBar_AC__
#include "CScrollBar_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

//========================================================================================
// CScrollBar_AC
// This class implements a standard AGA scroll bar, with the additional
// ability to perform live tracking and a proportional thumb.
//========================================================================================


// These regions are used momentarily while tracking the scroll
// bar indicator to clip and optimize redrawing.
RgnHandle CScrollBar_AC::fgSavedIndicatorClip = ::NewRgn();
RgnHandle CScrollBar_AC::fgOldIndicatorClip = ::NewRgn();
RgnHandle CScrollBar_AC::fgNewIndicatorClip = ::NewRgn();

//----------------------------------------------------------------------------------------
// CScrollBar_AC::CScrollBar_AC
//----------------------------------------------------------------------------------------

CScrollBar_AC::CScrollBar_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue) :
	CIndicator_AC(bounds, minimum, maximum, initialValue),
	fSingleStepSize(1),
	fPageStepSize(1)
{
	// Construct with default options.
	SetLiveTracking(GetAGAFeatureFlag_AC(kAGALiveScrolling_AC));
	SetIsProportional(GetAGAFeatureFlag_AC(kAGAProportionalScrolling_AC));
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::CScrollBar_AC
//----------------------------------------------------------------------------------------

CScrollBar_AC::CScrollBar_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, SInt32 singleStepSize, SInt32 pageStepSize, SInt32 pageSize, bool liveTracking, bool proportional) : 
	CIndicator_AC(bounds, minimum, maximum, initialValue, pageSize, liveTracking, proportional),
	fSingleStepSize(singleStepSize),
	fPageStepSize(pageStepSize)
{
	// Construct with specified options.
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::~CScrollBar_AC
//----------------------------------------------------------------------------------------

CScrollBar_AC::~CScrollBar_AC()
{
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::SetRange
//----------------------------------------------------------------------------------------

void CScrollBar_AC::SetRange(SInt32 newMinimum, SInt32 newMaximum, bool redraw)
{
	// For scroll bars, min==max means the scroll bar is drawn disabled.
	// So we need to catch a range change that changes this implicit state
	// that is peculiar to this subclass. When this happens, we don't want
	// the normal indicator redraw to happen; rather, we want to redraw the
	// whole object in its new state.
	
	bool	wasEnabled = (fMaximum > fMinimum);
	bool	isEnabled = (newMaximum > newMinimum);
	bool	changedState = (isEnabled != wasEnabled);
	
	CIndicator_AC::SetRange(newMinimum, newMaximum, redraw && ! changedState);

	if (redraw && changedState)
		this->DrawObject();
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::SetStepSizes
//----------------------------------------------------------------------------------------

void CScrollBar_AC::SetStepSizes(SInt32 singleStepSize, SInt32 pageStepSize)
{
	// Set the scrolling step delta amounts.

	fSingleStepSize = singleStepSize;
	fPageStepSize = pageStepSize;
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::TrackPart
//----------------------------------------------------------------------------------------

bool CScrollBar_AC::TrackPart(const CPoint_AC& mouseLocation, SInt32 partHit)
{
	// Track the mouse and return true if tracking changed the
	// indicator value. Thumb tracking is handled by superclass.
	// Scroll bar just handles arrow and page tracking.

	if ((partHit != kArrowMinusPress) && (partHit != kArrowPlusPress) &&
		(partHit != kPageMinusPress) && (partHit != kPagePlusPress))
		return CIndicator_AC::TrackPart(mouseLocation, partHit);

	if (fMaximum <= fMinimum)	// implicitly disabled (no range)
		return false;

	bool		wasIn = false;
	bool		isIn = true;
	bool		isButtonDown = true;	// ensure at least one time through
	CPoint_AC	newMouseLocation;
	UInt32		accelerationDelay = 18;	// will divide by 2 each time
	
	while (isButtonDown)
	{
		if (isIn)
			this->TrackDelta(partHit);

		::GetMouse(&newMouseLocation);	// gives local coordinates
		
		isIn = (this->TrackTestPart(newMouseLocation) == partHit);
		
		if (isIn != wasIn)
			this->DrawArrow(partHit, isIn);
		
		wasIn = isIn;

		if (isButtonDown && (accelerationDelay != 0))
		{
			DelayFutureWhileStillDown_AC(accelerationDelay);
			accelerationDelay = accelerationDelay >> 1;	// divide by 2
		}

		isButtonDown = ::StillDown();
	}
	
	if (isIn)	// if finished IN, restore normal appearance
		this->DrawArrow(partHit, kNotPressed);
	
	return true;
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::TrackTestPart
//----------------------------------------------------------------------------------------

SInt32 CScrollBar_AC::TrackTestPart(const CPoint_AC& mouseLocation)
{
	// Return the part value that the specified mouse
	// position would hit.

	CRect_AC	arrowMinusRect(GetBounds());
	CRect_AC	arrowPlusRect(GetBounds());
	CRect_AC	pageMinusRect(GetBounds());
	CRect_AC	pagePlusRect(GetBounds());
	CRect_AC	indicatorRect;
	
	this->GetIndicatorBox(GetValue(), indicatorRect);
	
	if (IsHorizontal())
	{
		arrowPlusRect.left = arrowPlusRect.right - kArrowSize;
		arrowMinusRect.right = arrowMinusRect.left + kArrowSize;
		
		pageMinusRect.left = arrowMinusRect.right;
		pageMinusRect.right = indicatorRect.left;
		
		pagePlusRect.left = indicatorRect.right;
		pagePlusRect.right = arrowPlusRect.left;
	}
	else
	{
		arrowPlusRect.top = arrowPlusRect.bottom - kArrowSize;
		arrowMinusRect.bottom = arrowMinusRect.top + kArrowSize;
		
		pageMinusRect.top = arrowMinusRect.bottom;
		pageMinusRect.bottom = indicatorRect.top;
		
		pagePlusRect.top = indicatorRect.bottom;
		pagePlusRect.bottom = arrowPlusRect.top;
	}
	
	if (arrowMinusRect.Contains(mouseLocation))
		return kArrowMinusPress;
	else if (arrowPlusRect.Contains(mouseLocation))
		return kArrowPlusPress;
	else if (pageMinusRect.Contains(mouseLocation))
		return kPageMinusPress;
	else if (pagePlusRect.Contains(mouseLocation))
		return kPagePlusPress;
	else if (indicatorRect.Contains(mouseLocation))
		return kIndicatorPress;
	else
		return kNoTrackPress;
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::IsValidIndicatorTrackMouse
//----------------------------------------------------------------------------------------

bool CScrollBar_AC::IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation)
{
	// Return true if the mouse is reasonably near
	// the track.

	CRect_AC	okRect(GetBounds());
	
	okRect.Inset(-32, -32);	// slop for mouse tracking

	return okRect.Contains(mouseLocation);
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::GetValueFromMouseDelta
//----------------------------------------------------------------------------------------

SInt32 CScrollBar_AC::GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& originalMouseLocation, const CPoint_AC& newMouseLocation)
{
	// Return the new indicator value based on the mouse delta
	// from the original mouse location and indicator value.

	SInt32	dontCare;
	SInt32	valueRange = fMaximum - fMinimum;
	SInt32	pixelRange = this->GetIndicatorPixelRange(&dontCare);
	SInt32	pixelDelta;
	float	pixelFraction;
	
	if (IsHorizontal())
		pixelDelta = newMouseLocation.h - originalMouseLocation.h;
	else
		pixelDelta = newMouseLocation.v - originalMouseLocation.v;

	pixelFraction = ((float) pixelDelta) / (float) pixelRange;
	
	float delta = pixelFraction * ((float) valueRange);
	delta += (float) 0.5;	// round to nearest integer, don't just truncate fraction
	
	return originalValue + delta;
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::DrawIndicatorTrack
//----------------------------------------------------------------------------------------

void CScrollBar_AC::DrawIndicatorTrack()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the empty scroll bar track.

	CGDIterator_AC	iter;
	bool			deep;

	bool	enabled = IsEnabled() && (fMaximum > fMinimum);

	while (iter.NextDevice(deep))
		{
		if (deep)
			{
			CRect_AC	r(GetBounds());

			if (fActive)
				kRGBColorRamp[rB].SetForeColor();
			else
				kRGBColorRamp[r10].SetForeColor();

			::FrameRect(&r);
			
			if (IsHorizontal())
				r.Inset(kArrowSize - 1, 0);
			else
				r.Inset(0, kArrowSize - 1);
			
			if ((! fActive) || (! enabled))
				{
				kRGBColorRamp[r1].SetForeColor();
				r.Inset(1, 1);
				::PaintRect(&r);
				r.Inset(-1, -1);
				
				if (fActive)
					kRGBColorRamp[r8].SetForeColor();
				
				if (IsHorizontal())
				{
					::MoveTo(r.left, r.top + 1);
					::LineTo(r.left, r.bottom - 2);
					::MoveTo(r.right - 1, r.top + 1);
					::LineTo(r.right - 1, r.bottom - 2);
				}
				else
				{
					::MoveTo(r.left + 1, r.top);
					::LineTo(r.right - 2, r.top);
					::MoveTo(r.left + 1, r.bottom - 1);
					::LineTo(r.right - 2, r.bottom - 1);
				}
			}
			else	// enabled
			{
				kRGBColorRamp[r5].SetForeColor();
				r.Inset(1, 1);
				::PaintRect(&r);
				r.Inset(-1, -1);
				
				kRGBColorRamp[rB].SetForeColor();
				
				if (IsHorizontal())
				{
					::MoveTo(r.left, r.top + 1);
					::LineTo(r.left, r.bottom - 2);
					::MoveTo(r.right - 1, r.top + 1);
					::LineTo(r.right - 1, r.bottom - 2);
					
					// Draw the shadowing.

					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left + 1, r.bottom - 3);
					::LineTo(r.left + 1, r.top + 1);
					::LineTo(r.right - 2, r.top + 1);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 2, r.bottom - 4);
					::LineTo(r.left + 2, r.top + 2);
					::LineTo(r.right - 3, r.top + 2);

					kRGBColorRamp[r3].SetForeColor();
					::MoveTo(r.left + 2, r.bottom - 2);
					::LineTo(r.right - 2, r.bottom - 2);

					kRGBColorRamp[r4].SetForeColor();
					::MoveTo(r.left + 3, r.bottom - 3);
					::LineTo(r.right - 2, r.bottom - 3);
				}
				else
				{
					::MoveTo(r.left + 1, r.top);
					::LineTo(r.right - 2, r.top);
					::MoveTo(r.left + 1, r.bottom - 1);
					::LineTo(r.right - 2, r.bottom - 1);
					
					// Draw the shadowing.

					kRGBColorRamp[r8].SetForeColor();
					::MoveTo(r.left + 1, r.bottom - 2);
					::LineTo(r.left + 1, r.top + 1);
					::LineTo(r.right - 3, r.top + 1);

					kRGBColorRamp[r7].SetForeColor();
					::MoveTo(r.left + 2, r.bottom - 2);
					::LineTo(r.left + 2, r.top + 2);
					::LineTo(r.right - 4, r.top + 2);

					kRGBColorRamp[r3].SetForeColor();
					::MoveTo(r.right - 2, r.top + 2);
					::LineTo(r.right - 2, r.bottom - 2);

					kRGBColorRamp[r4].SetForeColor();
					::MoveTo(r.right - 3, r.top + 3);
					::LineTo(r.right - 3, r.bottom - 2);
				}
			}
		}
		else	// 1-bit
		{
			CRect_AC	r(GetBounds());

			r.Inset(1, 1);
			kRGBColorRamp[rW].SetForeColor();
			::PaintRect(&r);
			r.Inset(-1, -1);

			kRGBColorRamp[rB].SetForeColor();
			::FrameRect(GetBounds());

			if (enabled && fActive)
			{
				if (IsHorizontal())
				{
					r.Inset(kArrowSize - 1, 0);
					::MoveTo(r.left, r.top + 1);
					::LineTo(r.left, r.bottom - 2);
					::MoveTo(r.right - 1, r.top + 1);
					::LineTo(r.right - 1, r.bottom - 2);
				}
				else
				{
					r.Inset(0, kArrowSize - 1);
					::MoveTo(r.left + 1, r.top);
					::LineTo(r.right - 2, r.top);
					::MoveTo(r.left + 1, r.bottom - 1);
					::LineTo(r.right - 2, r.bottom - 1);
				}
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::DrawTrackEnds
//----------------------------------------------------------------------------------------

void CScrollBar_AC::DrawTrackEnds()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the scroll arrows.

	this->DrawArrow(kArrowMinusPress, kNotPressed);
	this->DrawArrow(kArrowPlusPress, kNotPressed);
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::DrawIndicator
//----------------------------------------------------------------------------------------

void CScrollBar_AC::DrawIndicator()
{
	bool	enabled = IsEnabled() && (fMaximum > fMinimum);

	if ((! enabled) || (! fActive))
		return;

	CTempDrawingEnvironment_AC	env;

	// Draw the thumb indicator.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		enum { Fill, Light, TL, BR, Dark, kIndicatorColors };

		UInt8	colorIndexes[kIndicatorColors];
		CRect_AC	r(GetBounds());
		SInt32		startPt;
		SInt32		valuePt;
		SInt32		stopPt;
		CPoint_AC	drawDirection;
		CPoint_AC	moveDirection;
		SInt16		lineLength;
		
		if (deep)
		{
			if (IsPressed())
			{
				colorIndexes[Fill] = r8;
				colorIndexes[Light] = r3;
				colorIndexes[TL] = r5;
				colorIndexes[BR] = r10;
				colorIndexes[Dark] = r12;
			}
			else
			{
				colorIndexes[Fill] = r5;
				colorIndexes[Light] = r1;
				colorIndexes[TL] = r3;
				colorIndexes[BR] = r8;
				colorIndexes[Dark] = r10;
			}
		}
		else
		{
			if (IsPressed())
			{
				colorIndexes[Fill] = rB;
				colorIndexes[Light] = rB;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rB;
				colorIndexes[Dark] = rB;
			}
			else
			{
				colorIndexes[Fill] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rB;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
			}
		}
		
		this->GetIndicatorSpan(GetValue(), &startPt, &valuePt, &stopPt);
		this->GetIndicatorBox(GetValue(), r);
		
		if (IsHorizontal())
		{
			drawDirection.Set(0, 1);
			moveDirection.Set(1, 0);
			lineLength = r.bottom - r.top - 11;
		}
		else
		{
			drawDirection.Set(1, 0);
			moveDirection.Set(0, 1);
			lineLength = r.right - r.left - 11;
		}

		kRGBColorRamp[rB].SetForeColor();
		::FrameRect(&r);
		
		kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
		r.Inset(1, 1);
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		if (deep)
		{
			kRGBColorRamp[colorIndexes[TL]].SetForeColor();
			::MoveTo(r.left + 1, r.bottom - 3);
			::LineTo(r.left + 1, r.top + 2);
			::Move(1, -1);
			::LineTo(r.right - 3, r.top + 1);
			
			kRGBColorRamp[colorIndexes[BR]].SetForeColor();
			::MoveTo(r.right - 2, r.top + 2);
			::LineTo(r.right - 2, r.bottom - 2);
			::LineTo(r.left + 2, r.bottom - 2);

			kRGBColorRamp[colorIndexes[Light]].SetForeColor();
			::MoveTo(r.left + 1, r.top + 1);
			::Line(0, 0);
		}
		
		::MoveTo(r.left + 4 + (moveDirection.h * (((r.right - r.left) / 2) - 8)),
				r.top + 4 + (moveDirection.v * (((r.bottom - r.top) / 2) - 8)));

		if (! deep)
			::Move(moveDirection.h, moveDirection.v);

		for (SInt32 i = 0; i < 4; ++i)
		{
			kRGBColorRamp[colorIndexes[Light]].SetForeColor();
			::Line(0, 0);
			
			::Move(drawDirection.h, drawDirection.v);
			kRGBColorRamp[colorIndexes[TL]].SetForeColor();
			::Line(drawDirection.h * lineLength, drawDirection.v * lineLength);
			
			::Move(moveDirection.h, moveDirection.v);
			::Move(drawDirection.h, drawDirection.v);
			kRGBColorRamp[colorIndexes[Dark]].SetForeColor();
			::Line(- (drawDirection.h * (lineLength+1)), - (drawDirection.v * (lineLength+1)));

			::Move(moveDirection.h, moveDirection.v);
			::Move(- drawDirection.h, - drawDirection.v);
		}
	}
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::RemoveIndicator
//----------------------------------------------------------------------------------------

void CScrollBar_AC::RemoveIndicator(SInt32 newValue)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the track/background to remove the current
	// indicator.

	//
	// Set clipping so we only draw the track to
	// wipe out the old indicator, but not where
	// the new indicator will get drawn anyway.
	// This reduces flicker.
	//

	::GetClip(fgSavedIndicatorClip);

	if (newValue != GetValue())
	{
		CRect_AC	oldIndicatorBox;
		CRect_AC	newIndicatorBox;

		this->GetIndicatorBox(GetValue(), oldIndicatorBox);
		this->GetIndicatorBox(newValue, newIndicatorBox);
		
		::RectRgn(fgOldIndicatorClip, &oldIndicatorBox);
		::RectRgn(fgNewIndicatorClip, &newIndicatorBox);
		::DiffRgn(fgOldIndicatorClip, fgNewIndicatorClip, fgOldIndicatorClip);

		::SetClip(fgOldIndicatorClip);
	}

	this->DrawIndicatorTrack();

	::SetClip(fgSavedIndicatorClip);
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::DrawGhost
//----------------------------------------------------------------------------------------

void CScrollBar_AC::DrawGhost()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the indicator as a ghost at the current mGhostValue
	// position.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		CRect_AC	r(GetBounds());
		SInt32		startPt;
		SInt32		valuePt;
		SInt32		stopPt;
		CPoint_AC	drawDirection;
		CPoint_AC	moveDirection;
		SInt16		lineLength;
		
		this->GetIndicatorSpan(GetGhostValue(), &startPt, &valuePt, &stopPt);
		this->GetIndicatorBox(GetGhostValue(), r);
		
		if (deep)
		{
			if (IsHorizontal())
			{
				::SetPt(&drawDirection, 0, 1);
				::SetPt(&moveDirection, 1, 0);
				lineLength = r.bottom - r.top - 11;
				
				kRGBColorRamp[r4].SetForeColor();
				r.Inset(1, 3);
				::PaintRect(&r);
				r.Inset(-1, -3);
				
				kRGBColorRamp[r11].SetForeColor();
				::MoveTo(r.left, r.top + 1);
				::Line(0, 0);
				::MoveTo(r.right - 1, r.top + 1);
				::Line(0, 0);
				
				kRGBColorRamp[r10].SetForeColor();
				::MoveTo(r.left, r.top + 2);
				::Line(0, 0);
				::MoveTo(r.right - 1, r.top + 2);
				::Line(0, 0);
				
				kRGBColorRamp[r8].SetForeColor();
				::MoveTo(r.left, r.top + 3);
				::LineTo(r.left, r.bottom - 4);
				::MoveTo(r.right - 1, r.top + 3);
				::LineTo(r.right - 1, r.bottom - 4);
				
				kRGBColorRamp[r6].SetForeColor();
				::MoveTo(r.left, r.bottom - 3);
				::Line(0, 0);
				::MoveTo(r.right - 1, r.bottom - 3);
				::Line(0, 0);
				
				kRGBColorRamp[r4].SetForeColor();
				::MoveTo(r.left, r.bottom - 2);
				::Line(0, 0);
				::MoveTo(r.right - 1, r.bottom - 2);
				::Line(0, 0);
				
				kRGBColorRamp[r6].SetForeColor();
				::MoveTo(r.left + 1, r.top + 1);
				::LineTo(r.right - 2, r.top + 1);
				
				kRGBColorRamp[r5].SetForeColor();
				::MoveTo(r.left + 1, r.top + 2);
				::LineTo(r.right - 2, r.top + 2);
				
				kRGBColorRamp[r3].SetForeColor();
				::MoveTo(r.left + 1, r.bottom - 3);
				::LineTo(r.right - 2, r.bottom - 3);
				
				kRGBColorRamp[r2].SetForeColor();
				::MoveTo(r.left + 1, r.bottom - 2);
				::LineTo(r.right - 2, r.bottom - 2);
			}
			else
			{
				::SetPt(&drawDirection, 1, 0);
				::SetPt(&moveDirection, 0, 1);
				lineLength = r.right - r.left - 11;
				
				kRGBColorRamp[r4].SetForeColor();
				r.Inset(3, 1);
				::PaintRect(&r);
				r.Inset(-3, -1);
				
				kRGBColorRamp[r11].SetForeColor();
				::MoveTo(r.left + 1, r.top);
				::Line(0, 0);
				::MoveTo(r.left + 1, r.bottom - 1);
				::Line(0, 0);
				
				kRGBColorRamp[r10].SetForeColor();
				::MoveTo(r.left + 2, r.top);
				::Line(0, 0);
				::MoveTo(r.left + 2, r.bottom - 1);
				::Line(0, 0);
				
				kRGBColorRamp[r8].SetForeColor();
				::MoveTo(r.left + 3, r.top);
				::LineTo(r.right - 4, r.top);
				::MoveTo(r.left + 3, r.bottom - 1);
				::LineTo(r.right - 4, r.bottom - 1);
				
				kRGBColorRamp[r6].SetForeColor();
				::MoveTo(r.right - 3, r.top);
				::Line(0, 0);
				::MoveTo(r.right - 3, r.bottom - 1);
				::Line(0, 0);
				
				kRGBColorRamp[r4].SetForeColor();
				::MoveTo(r.right - 2, r.top);
				::Line(0, 0);
				::MoveTo(r.right - 2, r.bottom - 1);
				::Line(0, 0);
				
				kRGBColorRamp[r6].SetForeColor();
				::MoveTo(r.left + 1, r.top + 1);
				::LineTo(r.left + 1, r.bottom - 2);
				
				kRGBColorRamp[r5].SetForeColor();
				::MoveTo(r.left + 2, r.top + 1);
				::LineTo(r.left + 2, r.bottom - 2);
				
				kRGBColorRamp[r3].SetForeColor();
				::MoveTo(r.right - 3, r.top + 1);
				::LineTo(r.right - 3, r.bottom - 2);
				
				kRGBColorRamp[r2].SetForeColor();
				::MoveTo(r.right - 2, r.top + 1);
				::LineTo(r.right - 2, r.bottom - 2);
			}

			::MoveTo(r.left + 4 + (moveDirection.h * (((r.right - r.left) / 2) - 8)),
					r.top + 4 + (moveDirection.v * (((r.bottom - r.top) / 2) - 8)));

			for (SInt32 i = 0; i < 4; ++i)
			{
				kRGBColorRamp[r2].SetForeColor();
				::Line(drawDirection.h * (lineLength+1), drawDirection.v * (lineLength+1));
				
				::Move(moveDirection.h, moveDirection.v);
				::Move(drawDirection.h, drawDirection.v);
				kRGBColorRamp[r6].SetForeColor();
				::Line(- (drawDirection.h * (lineLength+1)), - (drawDirection.v * (lineLength+1)));

				::Move(moveDirection.h, moveDirection.v);
				::Move(- drawDirection.h, - drawDirection.v);
			}
		}
		else
		{
			kRGBColorRamp[rB].SetForeColor();
			::FrameRect(&r);
		}
	}
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::RemoveGhost
//----------------------------------------------------------------------------------------

void CScrollBar_AC::RemoveGhost(SInt32 /*newValue*/)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the track/background to remove the current
	// ghost indicator.

	CRect_AC	r;
	RgnHandle	savedClip = ::NewRgn();
	
	::GetClip(savedClip);

	this->GetIndicatorBox(GetGhostValue(), r);
	::ClipRect(&r);
	this->DrawIndicatorTrack();
	this->DrawIndicator();
	
	::SetClip(savedClip);
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::TrackDelta
//----------------------------------------------------------------------------------------

void CScrollBar_AC::TrackDelta(SInt32 partHit)
	{
	// Change our value based on the delta amount
	// for the part that was hit. Do notification
	// of the value change.

	SInt32	oldValue = GetValue();
	SInt32	newValue = GetValue();
	
	switch (partHit)
	{
		case kPageMinusPress:
			newValue -= fPageStepSize;
			break;
		case kArrowMinusPress:
			newValue -= fSingleStepSize;
			break;
		case kArrowPlusPress:
			newValue += fSingleStepSize;
			break;
		case kPagePlusPress:
			newValue += fPageStepSize;
			break;
	}
	
	this->SetValue(newValue, kRedraw);
	
	if (GetValue() != oldValue)
		this->NotifyValue();
}

//----------------------------------------------------------------------------------------
// CScrollBar_AC::DrawArrow
//----------------------------------------------------------------------------------------

void CScrollBar_AC::DrawArrow(SInt32 arrowPart, bool pressed)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the specified arrow in the specified mode.

	if ((arrowPart != kArrowMinusPress) &&
		(arrowPart != kArrowPlusPress))
		return;

	enum { Fill, TL, BR, Arrow, kNumArrowColors };

	bool	enabled = IsEnabled() && (fMaximum > fMinimum);
	UInt8	colorIndexes[kNumArrowColors];
	CRect_AC	r(GetBounds());
	
	if (IsHorizontal())
	{
		if (arrowPart == kArrowPlusPress)
			r.left = r.right - kArrowSize;
		else
			r.right = r.left + kArrowSize;
	}
	else
	{
		if (arrowPart == kArrowPlusPress)
			r.top = r.bottom - kArrowSize;
		else
			r.bottom = r.top + kArrowSize;
	}
	
	// Note: we don't draw the outer frame.
	// That's already been drawn.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			if (! fActive)
			{
				colorIndexes[Fill] = r1;
				colorIndexes[TL] = r1;
				colorIndexes[BR] = r1;
				colorIndexes[Arrow] = r1;
			}
			else if (pressed)
			{
				colorIndexes[Fill] = r8;
				colorIndexes[TL] = r10;
				colorIndexes[BR] = r6;
				colorIndexes[Arrow] = rB;
			}
			else if (enabled)
			{
				colorIndexes[Fill] = r2;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = r5;
				colorIndexes[Arrow] = rB;
			}
			else	// disabled
			{
				colorIndexes[Fill] = r2;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = r4;
				colorIndexes[Arrow] = r8;
			}
		}
		else
		{
			if ((! fActive) || ! enabled)
			{
				colorIndexes[Fill] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Arrow] = rW;
			}
			else if (pressed)
			{
				colorIndexes[Fill] = rB;
				colorIndexes[TL] = rB;
				colorIndexes[BR] = rB;
				colorIndexes[Arrow] = rW;
			}
			else // unpressed + enabled
			{
				colorIndexes[Fill] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Arrow] = rB;
			}
		}

		kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
		r.Inset(1, 1);
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		kRGBColorRamp[colorIndexes[TL]].SetForeColor();
		::MoveTo(r.left + 1, r.bottom - 3);
		::LineTo(r.left + 1, r.top + 1);
		::LineTo(r.right - 3, r.top + 1);

		kRGBColorRamp[colorIndexes[BR]].SetForeColor();
		::MoveTo(r.right - 2, r.top + 2);
		::LineTo(r.right - 2, r.bottom - 2);
		::LineTo(r.left + 2, r.bottom - 2);

		kRGBColorRamp[colorIndexes[Arrow]].SetForeColor();
		SInt16	direction = (arrowPart == kArrowPlusPress) ? 1 : -1;
		SInt16	offset = (arrowPart == kArrowPlusPress) ? 6 : 9;
		
		if (IsHorizontal())
		{
			::MoveTo(r.left + offset, ((r.top + r.bottom) / 2) - 4);
			::Line(0, 7);
			::Move(direction, -6);
			::Line(0, 5);
			::Move(direction, -4);
			::Line(0, 3);
			::Move(direction, -2);
			::Line(0, 1);
		}
		else
		{
			::MoveTo(((r.left + r.right) / 2) - 4, r.top + offset);
			::Line(7, 0);
			::Move(-6, direction);
			::Line(5, 0);
			::Move(-4, direction);
			::Line(3, 0);
			::Move(-2, direction);
			::Line(1, 0);
		}
	}
}

void CScrollBar_AC::GetIndicatorBox(SInt32 indicatorValue, CRect_AC& indicatorBox)
{
	// Return the bounds of the indicator box assuming that
	// it has specified indicator value.

	CRect_AC	r(GetBounds());
	SInt32		startPt;
	SInt32		valuePt;
	SInt32		stopPt;

	this->GetIndicatorSpan(indicatorValue, &startPt, &valuePt, &stopPt);
	
	if (IsHorizontal())
	{
		r.left = startPt;
		r.right = stopPt;
	}
	else
	{
		r.top = startPt;
		r.bottom = stopPt;
	}
	
	indicatorBox = r;
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

SInt32 CScrollBar_AC::GetIndicatorPixelRange(SInt32* proportionalIndicatorPixels)
{
	// Return the pixel range that the center point of the indicator
	// can travel along the track, and also return the amount of
	// range reduction that is caused by proportional indicator growth.
	// Non-proportional indicators use the full track range, minus
	// the size of the indicator itself.

	SInt32	range;
	
	*proportionalIndicatorPixels = 0;

	if (IsHorizontal())
		range = GetBounds().right - GetBounds().left - (2 * (kArrowSize + kIndicatorPixelInset)) + 1;
	else
		range = GetBounds().bottom - GetBounds().top - (2 * (kArrowSize + kIndicatorPixelInset)) + 1;

	if (IsProportional())
		{
		SInt32	pagePixels;
		
		pagePixels =
			(range * (((float) GetPageSize()) / ((float) (fMaximum - fMinimum))));
		
		if (pagePixels > 0)
		{
			range -= pagePixels;
			*proportionalIndicatorPixels = pagePixels;
		}
	}
	
	return range;
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void CScrollBar_AC::GetIndicatorPixelEnds(SInt32* startPt, SInt32* stopPt)
{
	// Return the start and end pixel locations of the indicator
	// track traveled by the center point of the indicator.

	SInt32	dontCare;
	SInt32	range = this->GetIndicatorPixelRange(&dontCare);

	if (IsHorizontal())
	{
		*startPt = ((GetBounds().left + GetBounds().right) / 2) - (range / 2) - 1;
		
		// Bump by 1 if width is odd but range isn't.
		if ((((GetBounds().right - GetBounds().left) & 1) != 0) &&
			((range & 1) == 0))
			(*startPt)++;
	}
	else
	{
		*startPt = ((GetBounds().top + GetBounds().bottom) / 2) - (range / 2) - 1;
		
		// Bump by 1 if height is odd but range isn't.
		if ((((GetBounds().bottom - GetBounds().top) & 1) != 0) &&
			((range & 1) == 0))
			(*startPt)++;
	}

	*stopPt = *startPt + range;	// avoid rounding of 1/2 pixel twice
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void CScrollBar_AC::GetIndicatorSpan(SInt32 value, SInt32* startPt, SInt32* valuePt, SInt32* stopPt)
{
	// Return the pixel locations of the indicator's ends and value.

	SInt32	proportionalIndicatorPixels;
	SInt32	trackStart;
	SInt32	trackStop;
	SInt32	trackRange = this->GetIndicatorPixelRange(&proportionalIndicatorPixels);
	
	this->GetIndicatorPixelEnds(&trackStart, &trackStop);
	
	*valuePt = trackStart +
		(trackRange * (((float) (value - fMinimum)) / ((float) (fMaximum - fMinimum))));

	*startPt = *valuePt - kIndicatorPixelInset - (proportionalIndicatorPixels / 2);
	*stopPt = *startPt + (2 * kIndicatorPixelInset) + proportionalIndicatorPixels + 1;
}

