//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC.cp 
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved.
// Portions Copyright  1996 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __CSimpleOffscreenImage_AC__
#include "CSimpleOffscreenImage_AC.h"
#endif

// ACS

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

//========================================================================================
// CLASS CSimpleOffscreenImage_AC
// CSimpleOffscreenImage_AC holds the data needed to blast a clipped region
// of pixel values onto the current port.
//========================================================================================

//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC
//----------------------------------------------------------------------------------------

CSimpleOffscreenImage_AC::CSimpleOffscreenImage_AC() :
	fImagePixMap(NULL),
	fClippingRegion(NULL)
{
}

//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC
//----------------------------------------------------------------------------------------

CSimpleOffscreenImage_AC::~CSimpleOffscreenImage_AC()
{
	if (fImagePixMap != NULL)
		::DisposePixMap(fImagePixMap);

	if (fClippingRegion != NULL)
		::DisposeRgn(fClippingRegion);
}

//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC
//----------------------------------------------------------------------------------------

void CSimpleOffscreenImage_AC::CreateImageData(const CPoint_AC& imageSize,
										SInt8* imageColorIndexes,
										RgnHandle clippingRegion,
										const CRGBColor_AC& outsideColor)
{
	// Allocate an offscreen PixMap to contain the RGB values
	// specified.

	const UInt32 kImageDepth = 4;
	
	fImageSourceRect.Set(0, 0, imageSize.h, imageSize.v);

	if (clippingRegion != NULL)
	{
		// Copy the supplied clipping region.

		fClippingRegion = CRegion_AC::MakeNewRgn();
		::CopyRgn(clippingRegion, fClippingRegion);
		::ThrowIfError_AC(::MemError());
	}

	//
	// Allocate and set up the PixMap structure.
	//

	Ptr	imagePtr = ::NewPtr(kImageDepth * imageSize.h * imageSize.v);
	ThrowIfNULL_AC(imagePtr);

	fImagePixMap = ::NewPixMap();
	ThrowIfNULL_AC(fImagePixMap);
	::HLockHi((Handle) fImagePixMap);	
	
	PixMapPtr	pixMapPtr = *fImagePixMap;
	
	pixMapPtr->baseAddr = imagePtr;
	pixMapPtr->rowBytes = (kImageDepth * imageSize.h) | 0x8000;
	pixMapPtr->bounds = fImageSourceRect;
	pixMapPtr->pixelType = RGBDirect;
	pixMapPtr->pixelSize = 32;	// direct 32-bit color values
	pixMapPtr->cmpCount = 3;	// 3 color components (RGB)
	pixMapPtr->cmpSize = 8;		// 8 bits per component
	pixMapPtr->planeBytes = 0;

	if (pixMapPtr->pmTable != NULL)
	{
		// We're using direct color, so toss the table.
		::DisposeCTable(pixMapPtr->pmTable);
		pixMapPtr->pmTable = NULL;
	}
	
	pixMapPtr->pmReserved = 0;
	
	//
	// Finally, set the pixel RGB color values.
	//
	
	UInt32*	pixelPtr = (UInt32*) imagePtr;
	UInt32	numPixels = imageSize.h * imageSize.v;
	SInt8*	indexPtr = imageColorIndexes;
	
	for (UInt32 pixelIndex = 0; pixelIndex < numPixels; ++pixelIndex)
	{
		SInt8			rampIndex = *indexPtr++;
		CRGBColor_AC	rampColor;
		
		if (rampIndex == OUT)
			rampColor = outsideColor;
		else
			rampColor = kRGBColorRamp[rampIndex];
		
		//
		// Construct 32-bit color value.
		// Assuming red is RRrr, blue is BBbb, green is GGgg,
		// we want 0x00RRGGBB, using high byte of each color.
		// This way we toss the low-order (LSB) color data.
		//

		UInt32	pixelValue =
				((((UInt32) rampColor.red) & 0x0000FF00) << 8) |
				(((UInt32) rampColor.green) & 0x0000FF00) |
				((((UInt32) rampColor.blue) & 0x0000FF00) >> 8);

		*pixelPtr++ = pixelValue;
		}
}

//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC
//----------------------------------------------------------------------------------------

void CSimpleOffscreenImage_AC::DrawImage(SInt16 imageHOrigin, SInt16 imageVOrigin)
{
	// Blast the offscreen image data into the current
	// port at the specified coordinate location.

	GrafPtr		currentPort;
	CRect_AC	outputRect = fImageSourceRect;
	
	// Set to black-on-white before doing CopyBits or our colors may
	// be massaged on the way to the screen.
	CTempBackColor_AC tempBackColor();	// temporarily use white background
	CRGBColor_AC::kRGBBlack.SetForeColor();
	
	outputRect.Offset(imageHOrigin, imageVOrigin);
	
	if (fClippingRegion != NULL)
		::OffsetRgn(fClippingRegion, imageHOrigin, imageVOrigin);

	::GetPort(&currentPort);
	::CopyBits((BitMapPtr) *fImagePixMap,
		&(currentPort->portBits),
		&fImageSourceRect,
		&outputRect,
		srcCopy,
		fClippingRegion);
	
	if (fClippingRegion != NULL)
		::OffsetRgn(fClippingRegion, - imageHOrigin, - imageVOrigin);
}

//----------------------------------------------------------------------------------------
// CSimpleOffscreenImage_AC
//----------------------------------------------------------------------------------------

void CSimpleOffscreenImage_AC::DrawPattern(const CRect_AC& imageRect)
	{
	// Blast the offscreen image data into the current
	// port, starting at the specified coordinate location,
	// repeating tiled until the imageRect is filled.

	GrafPtr		currentPort;
	CRect_AC	outputRect;
	
	CTempBackColor_AC tempBackColor();
	
	if (fClippingRegion != NULL)
		::OffsetRgn(fClippingRegion, imageRect.left, imageRect.top);

	::GetPort(&currentPort);
	
	outputRect.top = imageRect.top;
	outputRect.bottom = outputRect.top + fImageSourceRect.bottom;

	while (outputRect.top < imageRect.bottom)
	{
		outputRect.left = imageRect.left;
		outputRect.right = outputRect.left + fImageSourceRect.right;
		
		while (outputRect.left < imageRect.right)
		{
			::CopyBits((BitMapPtr) *fImagePixMap,
				&(currentPort->portBits),
				&fImageSourceRect,
				&outputRect,
				srcCopy,
				fClippingRegion);
			
			outputRect.Offset(fImageSourceRect.right, 0);
		}

		outputRect.Offset(0, fImageSourceRect.bottom);
	}

	if (fClippingRegion != NULL)
		::OffsetRgn(fClippingRegion, - imageRect.left, - imageRect.top);
}
