// CSlider_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CSlider_AC__
#include "CSlider_AC.h"
#endif

// Appearance

#ifndef __AGA_AC__
#include "AGA_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS


#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif


//========================================================================================
// CSlider_AC
// This class implements a standard AGA slider, with the additional
// ability to perform live tracking and a proportional thumb.
//========================================================================================


// These regions are used momentarily while tracking the slider
// indicator to clip and optimize redrawing.
RgnHandle CSlider_AC::fgSavedIndicatorClip = ::NewRgn();
RgnHandle CSlider_AC::fgOldIndicatorClip = ::NewRgn();
RgnHandle CSlider_AC::fgNewIndicatorClip = ::NewRgn();

//----------------------------------------------------------------------------------------
// CSlider_AC::CSlider_AC
//----------------------------------------------------------------------------------------

CSlider_AC::CSlider_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, const CTextStyle_AC& textStyle, SInt16 labelsID) :
	CIndicator_AC(bounds, minimum, maximum, initialValue, 0),
	fLabelStringHandles(NULL),
	fJustification(teFlushDefault),
	fTextStyle(textStyle)
{
	// Construct with default options.

	if (labelsID == 0)
	{
		fSliderKind = kRectSlider;

		SetLiveTracking(GetAGAFeatureFlag_AC(kAGALiveRSliders_AC));
		SetIsProportional(GetAGAFeatureFlag_AC(kAGAProportionalRSliders_AC));
	}
	else
	{
		fSliderKind = kPointerSlider;
		this->SetLabelsFromResource(labelsID);

		SetLiveTracking(GetAGAFeatureFlag_AC(kAGALivePSliders_AC));
		SetIsProportional(GetAGAFeatureFlag_AC(kAGAProportionalPSliders_AC));
	}
		
	fInverseDirection = GetAGAFeatureFlag_AC(fIsHorizontal ? kAGAInverseHSliderValues_AC : kAGAInverseVSliderValues_AC);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::CSlider_AC
//----------------------------------------------------------------------------------------

CSlider_AC::CSlider_AC(const CRect_AC& bounds, SInt32 minimum, SInt32 maximum, SInt32 initialValue, const CTextStyle_AC& textStyle, SInt16 labelsID, SliderKind kind, bool liveTracking, bool proportional) : 
	CIndicator_AC(bounds, minimum, maximum, initialValue, 0, liveTracking, proportional),
	fLabelStringHandles(NULL),
	fJustification(teFlushDefault),
	fSliderKind(kind),
	fTextStyle(textStyle)
{
	// Construct with specified options.

	fInverseDirection = GetAGAFeatureFlag_AC(fIsHorizontal ? kAGAInverseHSliderValues_AC : kAGAInverseVSliderValues_AC);

	this->SetLabelsFromResource(labelsID);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::~CSlider_AC
//----------------------------------------------------------------------------------------

CSlider_AC::~CSlider_AC()
{
	// Destruct by releasing memory allocated for labels.

	this->DisposeLabels();
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetSliderKind
//----------------------------------------------------------------------------------------

void CSlider_AC::SetSliderKind(SliderKind kind)
{
	// Set the slider kind (pointy or rectangular).

	fSliderKind = kind;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetInverseDirection
//----------------------------------------------------------------------------------------

void CSlider_AC::SetInverseDirection(bool inverseDirection)
{
	// Set the inverse value direction indicator.

	fInverseDirection = inverseDirection;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetValue
//----------------------------------------------------------------------------------------

SInt32 CSlider_AC::GetValue() const
{
	// Tweak the return value if using inverse values.

	SInt32	value = CIndicator_AC::GetValue();
	
	if (fInverseDirection)
		value = fMaximum - (value - fMinimum);
	
	return value;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetValue
//----------------------------------------------------------------------------------------

void CSlider_AC::SetValue(SInt32 newValue, bool redraw)
{
	// Tweak the input value if using inverse values.

	SInt32	actualValue = newValue;
	
	if (fInverseDirection)
		actualValue = fMaximum - (newValue - fMinimum);
	
	CIndicator_AC::SetValue(actualValue, redraw);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetLabelsFromResource
//----------------------------------------------------------------------------------------

void CSlider_AC::SetLabelsFromResource(SInt16 stringListResourceID)
{
	// Build the slider labels from the specified 'STR#' resource.
	// A label slot is allocated for each string in the resource;
	// there will be a visual label tick for each of these strings.
	// If the current min/max range is smaller than the number of
	// labels (i.e., it's uninitialized), the min/max range will
	// be set to match the number of labels.

	this->DisposeLabels();
	
	if (stringListResourceID != 0)
	{
		SInt16**	stringListHandle = (SInt16**) ::GetResource('STR#', stringListResourceID);
		SInt16		numLabels = (stringListHandle == NULL) ? 0 : **stringListHandle;
		
		if (numLabels != 0)
		{
			fLabelStringHandles = ::NewHandleClear(numLabels * sizeof(StringHandle));
			
			for (SInt16 i = 0; i < numLabels; ++i)
			{
				Str255			aLabel;
				StringHandle	aStringHandle;
				
				::GetIndString(aLabel, stringListResourceID, i+1);
				aStringHandle = ::NewString(aLabel);
				
				((StringHandle*)(*fLabelStringHandles))[i] = aStringHandle;
			}
		}
	
		if ((fMaximum - fMinimum + 1) < numLabels)
			fMaximum = fMinimum + numLabels - 1;
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetNumLabels
//----------------------------------------------------------------------------------------

void CSlider_AC::SetNumLabels(UInt32 numLabels)
{
	// Allocate slots for the specified number of labels.
	// Any existing labels are thrown away.

	this->DisposeLabels();
	
	fLabelStringHandles = ::NewHandleClear(numLabels * sizeof(StringHandle));

	if ((fMaximum - fMinimum + 1) < numLabels)
		fMaximum = fMinimum + numLabels - 1;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetLabel
//----------------------------------------------------------------------------------------

void CSlider_AC::SetLabel(UInt32 labelIndex, const CStr255_AC& itsLabel)
{
	// Assign the specified string to the specified label slot.

	if (fLabelStringHandles != NULL)
	{
		UInt32	numLabels = ::GetHandleSize(fLabelStringHandles) / sizeof(StringHandle);
		
		if (labelIndex < numLabels)
			((StringHandle*)(*fLabelStringHandles))[labelIndex] = ::NewString(itsLabel);
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetLabel
//----------------------------------------------------------------------------------------

void CSlider_AC::GetLabel(UInt32 labelIndex, CStr255_AC& itsLabel)
{
	// Return the label string of the specified label slot.

	itsLabel.Empty();

	if (fLabelStringHandles != NULL)
	{
		UInt32	numLabels = ::GetHandleSize(fLabelStringHandles) / sizeof(StringHandle);
		
		if (labelIndex < numLabels)
		{
			Ptr	sourceStringPtr = (Ptr) *(((StringHandle*)(*fLabelStringHandles))[labelIndex]);

			::BlockMoveData(sourceStringPtr, itsLabel, 1 + *sourceStringPtr);
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetLabelsStyle
//----------------------------------------------------------------------------------------

void CSlider_AC::SetLabelsStyle(const CTextStyle_AC& textStyle)
{
	// Set the QuickDraw text style for the labels.

	fTextStyle = textStyle;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::SetJustification
//----------------------------------------------------------------------------------------

void CSlider_AC::SetJustification(SInt32 justification)
{
	// Set the justification of the slider. For vertical
	// sliders, default/left means the slider is on the
	// left and the labels are on the right; flush right
	// reverses this. For horizontal sliders, default/left
	// means the slider is above and the labels are below;
	// flush right reverses this.
	// In fact, for "teFlushDefault", the system script
	// direction is checked at runtime, so this is usually
	// what you should use. Use flush right or flush left
	// to force it one way or another.

	fJustification = justification;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::TrackTestPart
//----------------------------------------------------------------------------------------

SInt32 CSlider_AC::TrackTestPart(const CPoint_AC& mouseLocation)
{
	// Return the indicator part value if the mouse location
	// would hit the indicator.

	CRect_AC	indicatorRect;
	
	this->GetIndicatorBox(GetValue(), indicatorRect);
	
	if (indicatorRect.Contains(mouseLocation))
		return kIndicatorPress;
	else
		return kNoTrackPress;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::IsValidIndicatorTrackMouse
//----------------------------------------------------------------------------------------

bool CSlider_AC::IsValidIndicatorTrackMouse(const CPoint_AC& mouseLocation)
{
	// Return true if the mouse location is reasonably close
	// to the indicator track. This will depend on the
	// justification.

	CRect_AC	okRect = GetBounds();
	
	if (IsHorizontal())
	{
		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
			okRect.bottom = okRect.top + kIndicatorWidth;
		else
			okRect.top = okRect.bottom - kIndicatorWidth;
	}
	else
	{
		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
			okRect.right = okRect.left + kIndicatorWidth;
		else
			okRect.left = okRect.right - kIndicatorWidth;
	}
	
	okRect.Inset(-32, -32);	// slop for mouse tracking

	return okRect.Contains(mouseLocation);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetValueFromMouseDelta
//----------------------------------------------------------------------------------------

SInt32 CSlider_AC::GetValueFromMouseDelta(SInt32 originalValue, const CPoint_AC& originalMouseLocation, const CPoint_AC& newMouseLocation)
{
	// Return the new potential indicator value based on the mouse
	// delta from the original click location and the original value.

	SInt32	dontCare;
	SInt32	valueRange = fMaximum - fMinimum;
	SInt32	pixelRange = this->GetIndicatorPixelRange(&dontCare);
	SInt32	pixelDelta;
	float	pixelFraction;
	
	if (IsHorizontal())
		pixelDelta = newMouseLocation.h - originalMouseLocation.h;
	else
		pixelDelta = newMouseLocation.v - originalMouseLocation.v;

	pixelFraction = ((float) pixelDelta) / (float) pixelRange;
	
	float delta = pixelFraction * ((float) valueRange);
	delta += (float) 0.5;	// round to nearest integer, don't just truncate fraction
	
	return originalValue + delta;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawBackground
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawBackground()
{
	CTempDrawingEnvironment_AC	env;

	// Paint the background and labels.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		if (deep)
			this->ApplyBackgroundColor();
		else
			kRGBColorRamp[rW].SetForeColor();

		::PaintRect(GetBounds());
	}
	iter.Cleanup();	// Restore clipping since not yet destructed.

	this->DrawLabels();
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawIndicatorTrack
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawIndicatorTrack()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the empty slider track.

	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			CRect_AC	trackFrame;
			
			this->GetTrackFrame(trackFrame, kIncludeBackground);

			this->ApplyBackgroundColor();
			::PaintRect(&trackFrame);
			
			this->GetTrackFrame(trackFrame, kTrackOnly);

			if (drawEnabled)
				kRGBColorRamp[rB].SetForeColor();
			else
				kRGBColorRamp[r8].SetForeColor();
			
			trackFrame.Inset(1, 1);
			::MoveTo(trackFrame.left + 1, trackFrame.top);
			::LineTo(trackFrame.right - 2, trackFrame.top);
			::Move(1, 1);
			::LineTo(trackFrame.right - 1, trackFrame.bottom - 2);
			::Move(-1, 1);
			::LineTo(trackFrame.left + 1, trackFrame.bottom - 1);
			::Move(-1, -1);
			::LineTo(trackFrame.left, trackFrame.top + 1);

			trackFrame.Inset(1, 1);

			if (drawEnabled)
				kRGBColorRamp[r5].SetForeColor();
			else
				kRGBColorRamp[r4].SetForeColor();
			
			::PaintRect(&trackFrame);

			if (drawEnabled)
				{
				trackFrame.Inset(-2, -2);
				::MoveTo(trackFrame.left, trackFrame.bottom - 3);
				::LineTo(trackFrame.left, trackFrame.top + 1);
				::Line(1, 0);
				::Line(0, -1);
				::LineTo(trackFrame.right - 3, trackFrame.top);
				
				if (IsHorizontal())
					{
					::Move(1, 1);
					::Line(0, 0);
					}
				
				kRGBColorRamp[rW].SetForeColor();
				::MoveTo(trackFrame.left + 1, trackFrame.bottom - 2);
				::Line(0, 0);
				::Move(1, 1);
				::LineTo(trackFrame.right - 2, trackFrame.bottom - 1);
				::Line(0, -1);
				::Line(1, 0);
				::LineTo(trackFrame.right -  1, trackFrame.top + 2);

				if (! IsHorizontal())
				{
					::Move(-1, -1);
					::Line(0, 0);
				}
			}
		}
		else	// 1 -bit
		{
			CRect_AC	trackFrame;
			
			this->GetTrackFrame(trackFrame, kIncludeBackground);

			kRGBColorRamp[rW].SetForeColor();
			::PaintRect(&trackFrame);
			
			this->GetTrackFrame(trackFrame, kTrackOnly);

			kRGBColorRamp[rB].SetForeColor();
			
			if (! drawEnabled)
				PenPatGray();
			
			trackFrame.Inset(1, 1);
			
			::MoveTo(trackFrame.left + 1, trackFrame.top);
			::LineTo(trackFrame.right - 2, trackFrame.top);
			::Move(1, 1);
			::LineTo(trackFrame.right - 1, trackFrame.bottom - 2);
			::Move(-1, 1);
			::LineTo(trackFrame.left + 1, trackFrame.bottom - 1);
			::Move(-1, -1);
			::LineTo(trackFrame.left, trackFrame.top + 1);
			
			if (! drawEnabled)
				PenPatBlack();
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawIndicator
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawIndicator()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the slider indicator.

	CRect_AC	indicatorRect;
	
	this->GetIndicatorBox(GetValue(), indicatorRect);

	if (fSliderKind == kRectSlider)
		this->DrawRectIndicator(indicatorRect, kNormalIndicator);
	else
		this->DrawPointerIndicator(indicatorRect, kNormalIndicator);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawGhost
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawGhost()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the ghost slider indicator.

	CRect_AC	indicatorRect;
	
	this->GetIndicatorBox(GetGhostValue(), indicatorRect);

	if (fSliderKind == kRectSlider)
		this->DrawRectIndicator(indicatorRect, kGhostIndicator);
	else
		this->DrawPointerIndicator(indicatorRect, kGhostIndicator);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawLabels
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawLabels()
{
	CTempDrawingEnvironment_AC	env;

	// Draw the slider labels and ticks.

	UInt32	numHandles = 0;
	bool	drawEnabled = fEnabled && fActive;

	if (fLabelStringHandles != NULL)
		numHandles = ::GetHandleSize(fLabelStringHandles) / sizeof(StringHandle);

	if (numHandles < 2)	// Gotta at least label the ends, man!
		return;
	
	SInt16		tickRange;
	CPoint_AC	tickOrigin;
	CPoint_AC	drawDirection;
	CPoint_AC	moveDirection;
	CRect_AC	trackFrame;
	bool		flushLeft = (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft);
	FontInfo	fontInfo;

	fTextStyle.SetPortTextStyle();
	::GetFontInfo(&fontInfo);
	
	this->GetTrackFrame(trackFrame, kTrackOnly);

	if (IsHorizontal())
	{
		tickRange = trackFrame.right - trackFrame.left - (2 * kLabelTickEndInset) - 1;
		drawDirection.h = 0;
		drawDirection.v = 1;
		moveDirection.h = 1;
		moveDirection.v = 0;
		tickOrigin.h = trackFrame.left + kLabelTickEndInset;
		
		if (flushLeft)
			tickOrigin.v = trackFrame.bottom + kLabelTickOffset;
		else
			tickOrigin.v = trackFrame.top - kLabelTickOffset - 2 - kLabelTickLength;
	}
	else
	{
		tickRange = trackFrame.bottom - trackFrame.top - (2 * kLabelTickEndInset) - 1;
		drawDirection.h = 1;
		drawDirection.v = 0;
		moveDirection.h = 0;
		moveDirection.v = 1;
		tickOrigin.v = trackFrame.top + kLabelTickEndInset;
		
		if (flushLeft)
			tickOrigin.h = trackFrame.right + kLabelTickOffset;
		else
			tickOrigin.h = trackFrame.left - kLabelTickOffset - 2 - kLabelTickLength;
	}
	
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		for (UInt32 i = 0; i < numHandles; ++i)
		{
			CPoint_AC	tickLocation;
			float	fraction = ((float) i) / ((float) (numHandles - 1));
			
			tickLocation.h = tickOrigin.h + (moveDirection.h * tickRange * fraction);
			tickLocation.v = tickOrigin.v + (moveDirection.v * tickRange * fraction);

			::MoveTo(tickLocation.h, tickLocation.v);
			
			kRGBColorRamp[rB].SetForeColor();
			
			if (! drawEnabled)
				{
				if (deep)
					kRGBColorRamp[r7].SetForeColor();
				else
					PenPatGray();
			}

			::Line(drawDirection.h * (kLabelTickLength + 1), drawDirection.v * (kLabelTickLength + 1));

			::PenNormal();

			if (deep)
			{
				// If disabled, we still draw in order to erase the 7/W
				// shading pixels from prior enabled drawing.

				if (drawEnabled)
					kRGBColorRamp[r7].SetForeColor();
				else
					this->ApplyBackgroundColor();

				::Line(moveDirection.h, moveDirection.v);
				::Line( - (kLabelTickLength * drawDirection.h), - (kLabelTickLength * drawDirection.v));
				
				if (drawEnabled)
					kRGBColorRamp[rW].SetForeColor();
				
				::Move(-1, -1);
				::Line(-moveDirection.h, -moveDirection.v);
				::Line(drawDirection.h * kLabelTickLength, drawDirection.v * kLabelTickLength);
			}

			CStr255_AC	aLabel;
			CRect_AC labelBox;
			SInt16	labelHeight = fontInfo.ascent + fontInfo.descent;
			SInt16	labelWidth;
			SInt32	textJustification;

			this->GetLabel(i, aLabel);
			labelWidth = ::StringWidth(aLabel);
			
			if (IsHorizontal())
			{
				textJustification = teCenter;
				labelBox.left = tickLocation.h - (labelWidth / 2);
				labelBox.right = labelBox.left + labelWidth;

				if (flushLeft)
				{
					labelBox.top = tickLocation.v + 9;
					labelBox.bottom = labelBox.top + labelHeight;
				}
				else
				{
					labelBox.bottom = tickLocation.v - 2;
					labelBox.top = labelBox.bottom - labelHeight;
				}
			}
			else
			{
				textJustification = fJustification;
				labelBox.top = tickLocation.v - (labelHeight / 2);
				labelBox.bottom = labelBox.top + labelHeight;

				if (flushLeft)
				{
					labelBox.left = tickLocation.h + 11;
					labelBox.right = labelBox.left + labelWidth;
				}
				else
				{
					labelBox.right = tickLocation.h - 2;
					labelBox.left = labelBox.right - labelWidth;
				}
			}

			//
			// To make view layout simpler, we assume that the labels may
			// extend outside the GetBounds(). So if this happens
			// we need to make sure we aren't being clipped out, but also
			// make sure we don't extend the unclipped area out to other
			// screens during device iteration.
			//
			
			RgnHandle	savedRegion = NULL;

			if ((labelBox.left < GetBounds().left) ||
				(labelBox.right > GetBounds().right))
			{
				savedRegion = ::NewRgn();
				::GetClip(savedRegion);
				labelBox.Inset(-1, -1);
				::ClipRect(&labelBox);
				labelBox.Inset(1, 1);
				
				iter.ClipFurtherToCurrentDevice();
			}

			AGAStringOut(aLabel, labelBox, kNoTruncation, textJustification, drawEnabled ? kNormalOutput : kDisabledOutput, deep, fTextStyle);

			if (savedRegion != NULL)
			{
				::SetClip(savedRegion);
				::DisposeRgn(savedRegion);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawRectIndicator
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawRectIndicator(const CRect_AC& indicatorRect, bool isGhost)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the rectangular slider indicator at the
	// specified location. The location will be sized
	// to account for a proportional indicator.

	enum { Frame, Fill, Corner, Light, TL, BR, Dark, Grip, BG, kNumIndicatorColors };

	UInt8		colorIndexes[kNumIndicatorColors];
	CRect_AC 	r(indicatorRect);
	CPoint_AC	drawDirection;
	CPoint_AC	moveDirection;
	SInt16		lineLength;

	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			colorIndexes[BG] = r2;

			if (isGhost)
			{
				colorIndexes[Frame] = r7;
				colorIndexes[Fill] = r2;
				colorIndexes[Corner] = r1;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = r4;
				colorIndexes[Dark] = r5;
				colorIndexes[Grip] = rW;
			}
			else if (! drawEnabled)
			{
				colorIndexes[Frame] = r8;
				colorIndexes[Fill] = r2;
				colorIndexes[Corner] = r2;
				colorIndexes[Light] = r2;
				colorIndexes[TL] = r2;
				colorIndexes[BR] = r2;
				// n/a colorIndexes[Dark] = r9;
				colorIndexes[Grip] = r2;
			}
			else if (IsPressed())
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = r8;
				colorIndexes[Corner] = r3;
				colorIndexes[Light] = r3;
				colorIndexes[TL] = r5;
				colorIndexes[BR] = r10;
				colorIndexes[Dark] = r12;
				colorIndexes[Grip] = r5;
			}
			else // normal
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = r5;
				colorIndexes[Corner] = r1;
				colorIndexes[Light] = r1;
				colorIndexes[TL] = r3;
				colorIndexes[BR] = r8;
				colorIndexes[Dark] = r10;
				colorIndexes[Grip] = r3;
			}
		}
		else	// 1-bit
		{
			colorIndexes[BG] = rW;

			if (isGhost)
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
				colorIndexes[Grip] = rB;
			}
			else if (! drawEnabled)
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
				colorIndexes[Grip] = rW;
			}
			else if (IsPressed())
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rB;
				colorIndexes[Corner] = rB;
				colorIndexes[Light] = rB;
				colorIndexes[TL] = rB;
				colorIndexes[BR] = rB;
				colorIndexes[Dark] = rB;
				colorIndexes[Grip] = rW;
			}
			else	// normal
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
				colorIndexes[Grip] = rB;
			}
		}
		
		kRGBColorRamp[colorIndexes[Frame]].SetForeColor();
		::FrameRect(&r);
		kRGBColorRamp[colorIndexes[BG]].SetForeColor();
		::MoveTo(r.left, r.top);
		::Line(0, 0);
		::MoveTo(r.right - 1, r.top);
		::Line(0, 0);
		::MoveTo(r.right - 1, r.bottom - 1);
		::Line(0, 0);
		::MoveTo(r.left, r.bottom - 1);
		::Line(0, 0);
		
		kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
		r.Inset(1, 1);
		::PaintRect(&r);
		r.Inset(-1, -1);
		
		kRGBColorRamp[colorIndexes[Corner]].SetForeColor();
		::MoveTo(r.left + 1, r.top + 1);
		::Line(0, 0);
		
		kRGBColorRamp[colorIndexes[TL]].SetForeColor();
		::MoveTo(r.left + 1, r.bottom - 3);
		::LineTo(r.left + 1, r.top + 2);
		::Move(1, -1);
		::LineTo(r.right - 3, r.top + 1);
		
		kRGBColorRamp[colorIndexes[BR]].SetForeColor();
		::Move(1, 1);
		::LineTo(r.right - 2, r.bottom - 2);
		::LineTo(r.left + 2, r.bottom - 2);

		if (IsHorizontal())
		{
			drawDirection.Set(0, 1);
			moveDirection.Set(1, 0);
			lineLength = r.bottom - r.top - 10;
		}
		else
		{
			drawDirection.Set(1, 0);
			moveDirection.Set(0, 1);
			lineLength = r.right - r.left - 10;
		}

		if (drawEnabled)
		{
			::MoveTo(r.left + 3 + drawDirection.h + (moveDirection.h * (((r.right - r.left) / 2) - 6)),
					r.top + 3 + drawDirection.v + (moveDirection.v * (((r.bottom - r.top) / 2) - 6)));

			if (! deep)
				::Move(moveDirection.h, moveDirection.v);

			for (SInt32 i = 0; i < 3; ++i)
			{
				kRGBColorRamp[colorIndexes[Light]].SetForeColor();
				::Line(drawDirection.h, drawDirection.v);
				
				kRGBColorRamp[colorIndexes[Grip]].SetForeColor();
				::Line(drawDirection.h * (lineLength-1), drawDirection.v * (lineLength-1));
				
				::Move(moveDirection.h, moveDirection.v);
				::Move(drawDirection.h, drawDirection.v);
				kRGBColorRamp[colorIndexes[Dark]].SetForeColor();
				::Line(- (drawDirection.h * lineLength), - (drawDirection.v * lineLength));

				::Move(moveDirection.h, moveDirection.v);
				::Move(- drawDirection.h, - drawDirection.v);
			}
		}
		
		if (isGhost && deep)
		{
			// Draw the little see-thru track slides.
			// They run inverse draw direction from the grip lines.
			// To handle proportional indicators, we need to vary
			// the length of the track slides on the flat part.
			SInt32	wholeFlatLength;
			SInt32	firstFlatLength;
			SInt32	remainingFlatLength;
			
			if (IsHorizontal())
				wholeFlatLength = r.right - r.left - 10;
			else
				wholeFlatLength = r.bottom - r.top - 10;

			firstFlatLength = wholeFlatLength / 2;
			remainingFlatLength = wholeFlatLength - firstFlatLength; // don't round down twice

			kRGBColorRamp[r11].SetForeColor();
			::MoveTo(r.left + (drawDirection.h * 5), r.top + (drawDirection.v * 5));
			
			for (UInt32 i = 0; i < 2; ++i)
			{
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r5].SetForeColor();
				::Line(moveDirection.h * firstFlatLength, moveDirection.v * firstFlatLength);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r5].SetForeColor();
				::Line(moveDirection.h * remainingFlatLength, moveDirection.v * remainingFlatLength);
				kRGBColorRamp[r7].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r11].SetForeColor();
				::Line(0, 0);
				
				::MoveTo(r.left + (drawDirection.h * 9), r.top + (drawDirection.v * 9));
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DrawPointerIndicator
//----------------------------------------------------------------------------------------

void CSlider_AC::DrawPointerIndicator(const CRect_AC& indicatorRect, bool isGhost)
{
	CTempDrawingEnvironment_AC	env;

	// Draw the pointy slider indicator at the
	// specified location. The location will be sized
	// to account for a proportional indicator.

	enum { Frame, Fill, Corner, Light, TL, Grip, BR, Dark, kNumIndicatorColors };

	UInt8		colorIndexes[kNumIndicatorColors];
	CRect_AC	r(indicatorRect);
	CPoint_AC	drawDirection;
	CPoint_AC	moveDirection;
	SInt16		lineLength;
	SInt16		propExtra;	// extra pixels due to proportional indicator shape

	CGDIterator_AC	iter;
	bool			deep;
	bool			drawEnabled = fEnabled && fActive;

	while (iter.NextDevice(deep))
	{
		if (deep)
		{
			if (isGhost)
			{
				colorIndexes[Frame] = r7;
				colorIndexes[Fill] = r2;
				colorIndexes[Corner] = r1;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[Grip] = rW;
				colorIndexes[BR] = r4;
				colorIndexes[Dark] = r5;
			}
			else if (! drawEnabled)
			{
				colorIndexes[Frame] = r8;
				colorIndexes[Fill] = r2;
				colorIndexes[Corner] = r2;
				colorIndexes[Light] = r2;
				colorIndexes[TL] = r2;
				colorIndexes[Grip] = r2;
				colorIndexes[BR] = r2;
				// n/a colorIndexes[Dark] = r9;
			}
			else if (IsPressed())
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = r8;
				colorIndexes[Corner] = r3;
				colorIndexes[Light] = r3;
				colorIndexes[TL] = r5;
				colorIndexes[Grip] = r5;
				colorIndexes[BR] = r10;
				colorIndexes[Dark] = r12;
			}
			else // normal
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = r5;
				colorIndexes[Corner] = r1;
				colorIndexes[Light] = r1;
				colorIndexes[TL] = r3;
				colorIndexes[Grip] = r3;
				colorIndexes[BR] = r8;
				colorIndexes[Dark] = r10;
			}
		}
		else	// 1-bit
		{
			if (isGhost)
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[Grip] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
			}
			else if (! drawEnabled)
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[Grip] = rW;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
			}
			else if (IsPressed())
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rB;
				colorIndexes[Corner] = rB;
				colorIndexes[Light] = rB;
				colorIndexes[TL] = rB;
				colorIndexes[Grip] = rW;
				colorIndexes[BR] = rB;
				colorIndexes[Dark] = rB;
			}
			else // normal
			{
				colorIndexes[Frame] = rB;
				colorIndexes[Fill] = rW;
				colorIndexes[Corner] = rW;
				colorIndexes[Light] = rW;
				colorIndexes[TL] = rW;
				colorIndexes[Grip] = rB;
				colorIndexes[BR] = rW;
				colorIndexes[Dark] = rW;
			}
		}

		if (IsHorizontal())
		{
			drawDirection.Set(0, 1);
			moveDirection.Set(1, 0);
			lineLength = r.bottom - r.top - 11;
			propExtra = MaxSInt16(0, r.right - r.left - 15);
		}
		else
		{
			drawDirection.Set(1, 0);
			moveDirection.Set(0, 1);
			lineLength = r.right - r.left - 11;
			propExtra = MaxSInt16(0, r.bottom - r.top - 15);
		}
		
		PolyHandle	pointerPoly = ::OpenPoly();
		
		this->BuildPointerIndicator(indicatorRect);
		
		::ClosePoly();
		
		kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
		::PaintPoly(pointerPoly);
		kRGBColorRamp[colorIndexes[Frame]].SetForeColor();
		::FramePoly(pointerPoly);
		
		::KillPoly(pointerPoly);

		if (IsHorizontal())
		{
			if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
			{
				::MoveTo(r.left + 1, r.top + 1);
				kRGBColorRamp[colorIndexes[Corner]].SetForeColor();
				::Line(0, 0);
				::MoveTo(r.right - 2, r.top + 2);
				kRGBColorRamp[colorIndexes[BR]].SetForeColor();
				::LineTo(r.right - 2, r.top + 9);
				::Line(-5, 5);
				::Line(-2 - propExtra, 0);
				::Line(-5, -5);
				kRGBColorRamp[colorIndexes[TL]].SetForeColor();
				::LineTo(r.left + 1, r.top + 2);
				::Line(1, -1);
				::LineTo(r.right - 3, r.top + 1);
			}
			else
			{
				::MoveTo(r.right - 6, r.top + 2);
				kRGBColorRamp[colorIndexes[BR]].SetForeColor();
				::Line(4, 4);
				::LineTo(r.right - 2, r.bottom - 2);
				::LineTo(r.left + 2, r.bottom - 2);
				::Move(-1, -1);
				kRGBColorRamp[colorIndexes[TL]].SetForeColor();
				::Line(0, -7);
				::Line(5, -5);
				::Line(1 + propExtra, 0);
				::MoveTo(r.left + 6, r.top + 1);
				kRGBColorRamp[colorIndexes[Corner]].SetForeColor();
				::Line(0, 0);
				::Move(-5, 5);
				::Line(0, 0);
			}
		}
		else
		{
			if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
			{
				::MoveTo(r.left + 1, r.top + 1);
				kRGBColorRamp[colorIndexes[Corner]].SetForeColor();
				::Line(0, 0);
				::MoveTo(r.left + 1, r.bottom - 3);
				kRGBColorRamp[colorIndexes[TL]].SetForeColor();
				::LineTo(r.left + 1, r.top + 2);
				::Line(1, -1);
				::Line(7, 0);
				::Move(1, 1);
				kRGBColorRamp[colorIndexes[BR]].SetForeColor();
				::Line(4, 4);
				::Line(0, 2 + propExtra);
				::Line(-5, 5);
				::Line(-7, 0);
			}
			else
			{
				::MoveTo(r.left + 2, r.top + 9 + propExtra);
				kRGBColorRamp[colorIndexes[BR]].SetForeColor();
				::Line(4, 4);
				::LineTo(r.right - 2, r.bottom - 2);
				::LineTo(r.right - 2, r.top + 2);
				::Move(-1, -1);
				kRGBColorRamp[colorIndexes[TL]].SetForeColor();
				::Line(-7, 0);
				::Line(-5, 5);
				::Line(0, 1 + propExtra);
				::Move(0, - (1 + propExtra));
				kRGBColorRamp[colorIndexes[Corner]].SetForeColor();
				::Line(0, 0);
				::Move(5, -5);
				::Line(0, 0);
			}
		}
		
		if (drawEnabled)
		{
			::MoveTo(r.left + 3 + (moveDirection.h * (((r.right - r.left) / 2) - 6)),
					r.top + 3 + (moveDirection.v * (((r.bottom - r.top) / 2) - 6)));

			if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushRight)
				::Move(2 * drawDirection.h, 2 * drawDirection.v);

			if (! deep)
				::Move(moveDirection.h, moveDirection.v);

			for (SInt32 i = 0; i < 3; ++i)
			{
				kRGBColorRamp[colorIndexes[Light]].SetForeColor();
				::Line(drawDirection.h, drawDirection.v);
				
				kRGBColorRamp[colorIndexes[Grip]].SetForeColor();
				::Line(drawDirection.h * (lineLength-1), drawDirection.v * (lineLength-1));
				
				::Move(moveDirection.h, moveDirection.v);
				::Move(drawDirection.h, drawDirection.v);
				kRGBColorRamp[colorIndexes[Dark]].SetForeColor();
				::Line(- (drawDirection.h * lineLength), - (drawDirection.v * lineLength));

				::Move(moveDirection.h, moveDirection.v);
				::Move(- drawDirection.h, - drawDirection.v);
			}
		}
		
		if (isGhost && deep)
		{
			// Draw the little see-thru track slides.
			// They run inverse draw direction from the grip lines.
			// To handle proportional indicators, we need to vary
			// the length of the track slides on the flat part.
			SInt32	wholeFlatLength;
			SInt32	firstFlatLength;
			SInt32	remainingFlatLength;
			
			if (IsHorizontal())
				wholeFlatLength = r.right - r.left - 10;
			else
				wholeFlatLength = r.bottom - r.top - 10;

			firstFlatLength = wholeFlatLength / 2;
			remainingFlatLength = wholeFlatLength - firstFlatLength; // don't round down twice

			kRGBColorRamp[r11].SetForeColor();
			::MoveTo(r.left + (drawDirection.h * 4), r.top + (drawDirection.v * 4));
			
			for (UInt32 i = 0; i < 2; ++i)
			{
				if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushRight)
					::Move(3 * drawDirection.h, 3 * drawDirection.v);

				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r5].SetForeColor();
				::Line(moveDirection.h * firstFlatLength, moveDirection.v * firstFlatLength);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r2].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r9].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r5].SetForeColor();
				::Line(moveDirection.h * remainingFlatLength, moveDirection.v * remainingFlatLength);
				kRGBColorRamp[r7].SetForeColor();
				::Line(moveDirection.h, moveDirection.v);
				kRGBColorRamp[r11].SetForeColor();
				::Line(0, 0);
				
				::MoveTo(r.left + (drawDirection.h * 8), r.top + (drawDirection.v * 8));
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::BuildPointerIndicator
//----------------------------------------------------------------------------------------

void CSlider_AC::BuildPointerIndicator(const CRect_AC& indicatorRect)
{
	// Perform the QuickDraw framing commands to build the
	// pointy indicator. This function is called while wrapped
	// in polygon or region building calls. The caller will
	// use the resulting polygon or region to frame, fill,
	// or clip.

	CRect_AC	r = indicatorRect;
	SInt16	propExtra;

	if (IsHorizontal())
	{
		propExtra = MaxSInt16(0, r.right - r.left - 15);

		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
		{
			::MoveTo(r.right - 1, r.top + 1);
			::LineTo(r.right - 1, r.bottom - 7);
			::Line(-6, 6);
			::Line(-2 - propExtra, 0);
			::Line(-6, -6);
			::LineTo(r.left, r.top + 1);
			::Line(1, -1);
			::LineTo(r.right - 2, r.top);
			::Line(1, 1);
		}
		else
		{
			::MoveTo(r.left, r.bottom - 2);
			::LineTo(r.left, r.top + 6);
			::Line(6, -6);
			::Line(2 + propExtra, 0);
			::Line(6, 6);
			::LineTo(r.right - 1, r.bottom - 2);
			::Line(-1, 1);
			::LineTo(r.left + 1, r.bottom - 1);
			::Line(-1, -1);
		}
	}
	else
	{
		propExtra = MaxSInt16(0, r.bottom - r.top - 15);

		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
		{
			::MoveTo(r.left + 1, r.top);
			::LineTo(r.right - 7, r.top);
			::Line(6, 6);
			::Line(0, 2 + propExtra);
			::Line(-6, 6);
			::LineTo(r.left + 1, r.bottom - 1);
			::Line(-1, -1);
			::LineTo(r.left, r.top + 1);
			::Line(1, -1);
		}
		else
		{
			::MoveTo(r.right - 2, r.bottom - 1);
			::LineTo(r.left + 6, r.bottom - 1);
			::Line(-6, -6);
			::Line(0, -2 - propExtra);
			::Line(6, -6);
			::LineTo(r.right - 2, r.top);
			::Line(1, 1);
			::LineTo(r.right - 1, r.bottom - 2);
			::Line(-1, 1);
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::RemoveIndicator
//----------------------------------------------------------------------------------------

void CSlider_AC::RemoveIndicator(SInt32 newValue)
{
	// Draw the background to remove the indicator
	// at the specified value.

	//
	// Set clipping so we only draw the track to
	// wipe out the old indicator, but not where
	// the new indicator will get drawn anyway.
	// This reduces flicker. Polygon geometry makes
	// tweaking necessary and still imperfect.
	//

	::GetClip(fgSavedIndicatorClip);

	if (newValue != GetValue())
	{
		CRect_AC	oldIndicatorBox;
		CRect_AC	newIndicatorBox;

		this->GetIndicatorBox(GetValue(), oldIndicatorBox);
		this->GetIndicatorBox(newValue, newIndicatorBox);
		
		if (fSliderKind == kRectSlider)
		{
			::RectRgn(fgOldIndicatorClip, &oldIndicatorBox);
			::RectRgn(fgNewIndicatorClip, &newIndicatorBox);
		}
		else
		{
			//
			// For optimal clipping, because of polygon and
			// region geometry, we have to tweak the regions
			// a little bit.
			//
			
			RgnHandle	tempRegion = ::NewRgn();

			::OpenRgn();
			this->BuildPointerIndicator(oldIndicatorBox);
			::CloseRgn(fgOldIndicatorClip);
			::CopyRgn(fgOldIndicatorClip, tempRegion);
			::OffsetRgn(tempRegion, 1, 1);
			::InsetRgn(tempRegion, -1, -1);
			::UnionRgn(fgOldIndicatorClip, tempRegion, fgOldIndicatorClip);

			::OpenRgn();
			this->BuildPointerIndicator(newIndicatorBox);
			::CloseRgn(fgNewIndicatorClip);
			::CopyRgn(fgNewIndicatorClip, tempRegion);
			::UnionRgn(fgNewIndicatorClip, tempRegion, fgNewIndicatorClip);
			
			::DisposeRgn(tempRegion);
		}

		::DiffRgn(fgOldIndicatorClip, fgNewIndicatorClip, fgOldIndicatorClip);

		::SetClip(fgOldIndicatorClip);
	}

	this->DrawIndicatorTrack();

	::SetClip(fgSavedIndicatorClip);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::RemoveGhost
//----------------------------------------------------------------------------------------

void CSlider_AC::RemoveGhost(SInt32 /*newValue*/)
{
	// Draw the background necessary to remove the
	// ghost indicator while not removing the solid
	// value indicator.

	CRect_AC	r;
	RgnHandle	savedClip = ::NewRgn();
	
	::GetClip(savedClip);

	this->GetIndicatorBox(GetGhostValue(), r);
	::ClipRect(&r);
	this->DrawIndicatorTrack();
	this->DrawIndicator();
	
	::SetClip(savedClip);
	::DisposeRgn(savedClip);
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetTrackFrame
//----------------------------------------------------------------------------------------

void CSlider_AC::GetTrackFrame(CRect_AC& trackFrame, bool includeBackground)
{
	// Return the rectangle that bounds just the indicator
	// track, including shading pixels but not the area covered
	// by the indicator itself.

	bool	leftJustify = (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft);

	trackFrame = GetBounds();
	
	if (IsHorizontal())
	{
		if (leftJustify)
		{
			trackFrame.top += 4;
			trackFrame.bottom = trackFrame.top + 7;
		}
		else
		{
			trackFrame.bottom -= 4;
			trackFrame.top = trackFrame.bottom - 7;
		}
		
		if (includeBackground)
		{
			trackFrame.top -= leftJustify ? 4 : 6;
			trackFrame.bottom += leftJustify ? 6 : 4;
		}
	}
	else
	{
		if (leftJustify)
		{
			trackFrame.left += 4;
			trackFrame.right = trackFrame.left + 7;
		}
		else
		{
			trackFrame.right -= 4;
			trackFrame.left = trackFrame.right - 7;
		}
		
		if (includeBackground)
		{
			trackFrame.left -= leftJustify ? 4 : 6;
			trackFrame.right += leftJustify ? 6 : 4;
		}
	}
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetIndicatorBox
//----------------------------------------------------------------------------------------

void CSlider_AC::GetIndicatorBox(SInt32 indicatorValue, CRect_AC& indicatorBox)
{
	// Return the box that defines the indicator's location
	// given the specified indicator value.

	CRect_AC	r(GetBounds());
	SInt32	startPt;
	SInt32	valuePt;
	SInt32	stopPt;

	this->GetIndicatorSpan(indicatorValue, &startPt, &valuePt, &stopPt);
	
	if (IsHorizontal())
	{
		r.left = startPt;
		r.right = stopPt;

		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
		{
			if (fSliderKind == kPointerSlider)
				++r.top;

			r.bottom = r.top + kIndicatorWidth;
		}
		else
		{
			if (fSliderKind == kPointerSlider)
				--r.bottom;

			r.top = r.bottom - kIndicatorWidth;
		}
	}
	else
	{
		r.top = startPt;
		r.bottom = stopPt;

		if (CTextStyle_AC::RuntimeJustify(fJustification) == teFlushLeft)
		{
			if (fSliderKind == kPointerSlider)
				++r.left;

			r.right = r.left + kIndicatorWidth;
		}
		else
		{
			if (fSliderKind == kPointerSlider)
				--r.right;

			r.left = r.right - kIndicatorWidth;
		}
	}
	
	indicatorBox = r;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetIndicatorPixelRange
//----------------------------------------------------------------------------------------

SInt32 CSlider_AC::GetIndicatorPixelRange(SInt32* proportionalIndicatorPixels)
{
	// Return the pixel range that the center point of the indicator
	// can travel along the track, and also return the amount of
	// range reduction that is caused by proportional indicator growth.
	// Non-proportional indicators use the full track range, minus
	// the size of the indicator itself.

	SInt32	range;
	SInt32	indicatorInset = (static_cast<SInt32>(fSliderKind) == kRectSlider)
								? static_cast<SInt32>(kRectIndicatorPixelInset)
								: static_cast<SInt32>(kPointerIndicatorPixelInset);
	
	*proportionalIndicatorPixels = 0;

	if (IsHorizontal())
		range = GetBounds().right - GetBounds().left - (2 * (kEndGapSize + indicatorInset)) + 1;
	else
		range = GetBounds().bottom - GetBounds().top - (2 * (kEndGapSize + indicatorInset)) + 1;

	if (IsProportional())
	{
		SInt32	pagePixels;
		
		pagePixels =
			(range * (((float) GetPageSize()) / ((float) (fMaximum - fMinimum))));
		
		if (pagePixels > 0)
		{
			range -= pagePixels;
			*proportionalIndicatorPixels = pagePixels;
		}
	}
	
	return range;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::GetIndicatorPixelEnds
//----------------------------------------------------------------------------------------

void CSlider_AC::GetIndicatorPixelEnds(SInt32* startPt, SInt32* stopPt)
{
	// Return the start and end pixel locations of the indicator
	// track traveled by the center point of the indicator.

	SInt32	dontCare;
	SInt32	range = this->GetIndicatorPixelRange(&dontCare);

	if (IsHorizontal())
		*startPt = ((GetBounds().left + GetBounds().right) / 2) - (range / 2) - 1;
	else
		*startPt = ((GetBounds().top + GetBounds().bottom) / 2) - (range / 2) - 1;

	*stopPt = *startPt + range;	// avoid rounding of 1/2 pixel twice
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void CSlider_AC::GetIndicatorSpan(SInt32 value, SInt32* startPt, SInt32* valuePt, SInt32* stopPt)
{
	// Return the pixel locations of the indicator's ends and value.

	SInt32	proportionalIndicatorPixels;
	SInt32	trackStart;
	SInt32	trackStop;
	SInt32	trackRange = this->GetIndicatorPixelRange(&proportionalIndicatorPixels);
	SInt32	indicatorInset = (static_cast<SInt32>(fSliderKind) == kRectSlider)
								? static_cast<SInt32>(kRectIndicatorPixelInset)
								: static_cast<SInt32>(kPointerIndicatorPixelInset);
	
	this->GetIndicatorPixelEnds(&trackStart, &trackStop);
	
	*valuePt = trackStart +
		(trackRange * (((float) (value - fMinimum)) / ((float) (fMaximum - fMinimum))));

	*startPt = *valuePt - indicatorInset - (proportionalIndicatorPixels / 2);
	*stopPt = *startPt + (2 * indicatorInset) + proportionalIndicatorPixels + 1;
}

//----------------------------------------------------------------------------------------
// CSlider_AC::DisposeLabels
//----------------------------------------------------------------------------------------

void CSlider_AC::DisposeLabels()
{
	// Release the memory allocated for slider labels.

	if (fLabelStringHandles != NULL)
	{
		UInt32	numHandles = ::GetHandleSize(fLabelStringHandles) / sizeof(StringHandle);
		
		for (UInt32 i = 0; i < numHandles; ++i)
			::DisposeHandle((Handle) ((StringHandle*)(*fLabelStringHandles))[i]);

		::DisposeHandle(fLabelStringHandles);
	}
}

