// CTabbedView_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

#ifndef __CTabbedView_AC__
#include "CTabbedView_AC.h"
#endif

// Appearance

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// ACS

#ifndef __CDrawingEnvironment_AC__
#include "CDrawingEnvironment_AC.h"
#endif

#ifndef __CGDIterator_AC__
#include "CGDIterator_AC.h"
#endif

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

//========================================================================================
// CTabbedView_AC
// CTabbedView_AC implements a folder tabs box, often referred to as a "tabbed
// dialog" interface.
//========================================================================================

CTextStyle_AC CTabbedView_AC::kSmallTabPanelStyle(::GetAppFont(), bold, 10, CRGBColor_AC::kRGBBlack);

//----------------------------------------------------------------------------------------
// CTabbedView_AC::CTabbedView_AC
//----------------------------------------------------------------------------------------

CTabbedView_AC::CTabbedView_AC(const CRect_AC& bounds, SInt32 initialTabIndex, const CTextStyle_AC& textStyle, SInt16 labelsID, TabWidthKind widthKind, TabsLocation location, bool drawFrame) :
	CControl_AC(bounds),
	fNumTabs(0),
	fCurrentTabIndex(initialTabIndex),
	fTabInfo(NULL),
	fTabsLocation(location),
	fTabWidthKind(widthKind),
	fDrawFrame(drawFrame),
	fTextStyle(textStyle)
{
	// Construct with specified options.

	fTabSize = ((textStyle.tsSize > 10) || (textStyle.tsSize == 0)) ? kLargeTabs : kSmallTabs;

	this->CalculateTabSizeInfo();

	this->SetLabelsFromResource(labelsID);
	
	if (! fgImagesAllocated)
		AllocateTabImages();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::~CTabbedView_AC
//----------------------------------------------------------------------------------------

CTabbedView_AC::~CTabbedView_AC()
{
	// Destruct by releasing memory allocated for labels.

	this->DisposeTabInfo();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawObject
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawObject()
{
	this->DrawBackground();
	this->DrawFrame();
	this->DrawTabs();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::TrackMouse
//----------------------------------------------------------------------------------------

bool CTabbedView_AC::TrackMouse(const CPoint_AC& mouseLocation)
{
	bool		wasItHit;
	CRect_AC	tabRect;

	//
	// First we determine which tab, if any, was clicked.
	//

	fLastTabClicked = -1;
	
	for (SInt32 i = 0; i < fNumTabs; ++i)
	{
		this->GetTabBounds(i, tabRect);
		if (tabRect.Contains(mouseLocation) && this->IsTabEnabled(i))
		{
			fLastTabClicked = i;
			break;
		}
	}
	
	//
	// Now we let the standard in/out tracking occur.
	// Our hit test will compare with the fLastTabClicked
	// area.
	//

	wasItHit = CControl_AC::TrackMouse(mouseLocation);
	
	if (! wasItHit)
		fLastTabClicked = -1;
		
	return wasItHit;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetBounds
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetBounds(const CRect_AC& bounds, bool redraw)
{
	CControl_AC::SetBounds(bounds, redraw);

	this->CalculateTabWidths();
	this->CalculateTabSizeInfo();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::HitTest
//----------------------------------------------------------------------------------------

bool CTabbedView_AC::HitTest(const CPoint_AC& mouseLocation)
{
	bool	isIn;

	// Return true if the mouse is in the fLastTabClicked.
	
	if (fLastTabClicked == -1)
		isIn = false;
	else
	{
		CRect_AC	tabRect;

		this->GetTabBounds(fLastTabClicked, tabRect);
		
		isIn = tabRect.Contains(mouseLocation);
	}

	return isIn;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTrackingState
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTrackingState(bool isIn)
{
	// Draw the fLastTabClicked pressed or unpressed as specified.

	if ((fLastTabClicked != -1) && (fLastTabClicked != fCurrentTabIndex))
		this->DrawOneTab(fLastTabClicked, isIn);
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetCurrentTab
//----------------------------------------------------------------------------------------

SInt32 CTabbedView_AC::GetCurrentTab()
{
	return fCurrentTabIndex;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetCurrentTab
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetCurrentTab(SInt32 tabIndex, bool redraw)
{
	if (tabIndex != fCurrentTabIndex)
	{
		fCurrentTabIndex = tabIndex;
		
		if (redraw)
			this->DrawTabs();
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetLastTabClicked
//----------------------------------------------------------------------------------------

SInt32 CTabbedView_AC::GetLastTabClicked()
{
	return fLastTabClicked;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabSize
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabSize(TabSize size)
{
	// Set the tabs size.

	fTabSize = size;
	this->CalculateTabSizeInfo();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabsLocation
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabsLocation(TabsLocation location)
{
	// Set the tabs location.

	fTabsLocation = location;
	this->CalculateTabSizeInfo();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabsWidth
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabsWidth(TabWidthKind widthKind)
	{
	// Set the tabs width style.

	fTabWidthKind = widthKind;
	
	this->CalculateTabWidths();
	}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetTabHeight
//----------------------------------------------------------------------------------------

SInt16 CTabbedView_AC::GetTabHeight()
{
	// Return the current tab image height (or width if oriented vertically).

	if ((fTabsLocation == kTopTabs) || (fTabsLocation == kBottomTabs))
		return fTabImageSize.v;
	else
		return fTabImageSize.h;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::HasFrame
//----------------------------------------------------------------------------------------

bool CTabbedView_AC::HasFrame()
{
	return fDrawFrame;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetLabelsFromResource
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetLabelsFromResource(SInt16 stringListResourceID)
{
	// Build the tab labels from the specified 'STR#' resource.
	// A label slot is allocated for each string in the resource;
	// there will be a tab for each of these strings.

	this->DisposeTabInfo();
	
	if (stringListResourceID != 0)
	{
		SInt16**	stringListHandle = (SInt16**) ::GetResource('STR#', stringListResourceID);
		SInt32		numTabs = (stringListHandle == NULL) ? 0 : **stringListHandle;
		
		if (numTabs != 0)
		{
			this->AllocateTabInfo(numTabs, kShrinkToFit);

			for (SInt16 i = 0; i < fNumTabs; ++i)
			{
				Str255	aLabel;
				
				::GetIndString(aLabel, stringListResourceID, i+1);
				this->SetTabLabel(i, aLabel);
			}
			
			this->CalculateTabWidths();
		}
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetNumTabs
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetNumTabs(SInt32 numTabs, bool shrinkToFit)
{
	// Allocate slots for the specified number of labels.
	
	this->AllocateTabInfo(numTabs, shrinkToFit);
	
	// In case the caller never informs us when the labels have
	// all been set (i.e., never calls CalculateTabWidths), we
	// init the widths to fill the bounds.
	
	this->CalculateTabWidthsByKind(kVariableTabWidth);
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetNumTabs
//----------------------------------------------------------------------------------------

SInt32 CTabbedView_AC::GetNumTabs()
{
	return fNumTabs;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::RemoveTab
//----------------------------------------------------------------------------------------

void CTabbedView_AC::RemoveTab(SInt32 tabIndex)
{
	if ((fTabInfo != NULL) && (fNumTabs > 0))
	{
		// Only the label handle is our responsibility to free.

		::DisposeHandle((Handle) (*fTabInfo)[tabIndex].fLabel);
		
		// Move the tab info struct for all higher-index tabs back
		// by 1, and then shrink the handle by 1 entry.

		Ptr		destPtr = (Ptr) &((*fTabInfo)[tabIndex]);
		Ptr		sourcePtr = (Ptr) &((*fTabInfo)[tabIndex + 1]);
		SInt32	numBytesToMove = (fNumTabs - 1 - tabIndex) * sizeof(TabInfo);

		::BlockMoveData(sourcePtr, destPtr, numBytesToMove);
		
		--fNumTabs;

		// We're shrinking, so no out-of-memory error possible.
		::SetHandleSize((Handle) fTabInfo, fNumTabs * sizeof(TabInfo));
		
		// If we just deleted the current and last tab, need to change
		// the current selection to be the new last tab.
		if ((fCurrentTabIndex == tabIndex) && (fCurrentTabIndex >= fNumTabs))
			fCurrentTabIndex = fNumTabs - 1;

		this->CalculateTabWidthsByKind(kVariableTabWidth);
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::AllocateTabInfo
//----------------------------------------------------------------------------------------

void CTabbedView_AC::AllocateTabInfo(SInt32 numTabs, bool shrinkToFit)
{
	// Make sure the tab info handle is sized to fit the
	// specified number of tabs.
	
	if (fTabInfo == NULL)
	{
		fTabInfo = (TabInfoHandle) ::NewHandleClear(numTabs * sizeof(TabInfo));
		
		if (fTabInfo == NULL)
			fNumTabs = 0;
		else
			fNumTabs = numTabs;
	}
	else if ((fNumTabs < numTabs) || (shrinkToFit && (fNumTabs > numTabs)))
	{
		::SetHandleSize((Handle) fTabInfo, numTabs * sizeof(TabInfo));
		
		if (::MemError() != noErr)
			return;
		
		// Make sure the new tab info elements are zeroed like NewHandleClear.

		for (SInt32 i = fNumTabs; i < numTabs; ++i)
		{
			(*fTabInfo)[i].fLabel = ::NewString(kEmptyString_AC);
			(*fTabInfo)[i].fTabWidth = 0;
			(*fTabInfo)[i].fDisabled = false;
			(*fTabInfo)[i].fUserData = NULL;
		}

		fNumTabs = numTabs;
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabLabel
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabLabel(SInt32 tabIndex, const CStr255_AC& itsLabel)
{
	// Assign the specified string to the specified label slot.

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
		{
			if ((*fTabInfo)[tabIndex].fLabel != NULL)
				::DisposeHandle((Handle) (*fTabInfo)[tabIndex].fLabel);
			
			(*fTabInfo)[tabIndex].fLabel = ::NewString(itsLabel);
		}
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetTabLabel
//----------------------------------------------------------------------------------------

void CTabbedView_AC::GetTabLabel(SInt32 tabIndex, CStr255_AC& itsLabel)
{
	// Return the label string of the specified label slot.

	itsLabel.Empty();

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
		{
			StringHandle	labelStringHandle = (*fTabInfo)[tabIndex].fLabel;
			
			if (labelStringHandle != NULL)
			{
				Ptr	sourceStringPtr = (Ptr) *labelStringHandle;

				::BlockMoveData(sourceStringPtr, itsLabel, 1 + *sourceStringPtr);
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabEnable
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabEnable(SInt32 tabIndex, bool enable)
{
	// Enable or disable the specified tab.

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
			(*fTabInfo)[tabIndex].fDisabled = ! enable;
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::IsTabEnabled
//----------------------------------------------------------------------------------------

bool CTabbedView_AC::IsTabEnabled(SInt32 tabIndex)
{
	// Return the enable state of the specified tab.
	
	bool	isEnabled = true;

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
			isEnabled = ! (*fTabInfo)[tabIndex].fDisabled;
	}
	
	return isEnabled;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetTabUserData
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetTabUserData(SInt32 tabIndex, void* userData)
{
	// Stash the userData in the specified tab's info.

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
			(*fTabInfo)[tabIndex].fUserData = userData;
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetTabUserData
//----------------------------------------------------------------------------------------

void* CTabbedView_AC::GetTabUserData(SInt32 tabIndex)
{
	// Return the specified tab's fUserData.
	
	void*	userData = NULL;

	if (fTabInfo != NULL)
	{
		if (tabIndex < fNumTabs)
			userData = (*fTabInfo)[tabIndex].fUserData;
	}
	
	return userData;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::SetLabelsStyle
//----------------------------------------------------------------------------------------

void CTabbedView_AC::SetLabelsStyle(const CTextStyle_AC& textStyle)
{
	// Set the QuickDraw text style for the labels.

	fTextStyle = textStyle;
	
	this->CalculateTabWidths();
	this->CalculateTabSizeInfo();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::CalculateTabWidths
//----------------------------------------------------------------------------------------

void CTabbedView_AC::CalculateTabWidths()
{
	this->CalculateTabWidthsByKind(fTabWidthKind);
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::CalculateTabWidthsByKind
//----------------------------------------------------------------------------------------

void CTabbedView_AC::CalculateTabWidthsByKind(TabWidthKind widthKind)
{
	CTempDrawingEnvironment_AC	env;

	SInt32		widthAvailable;
	CStr255_AC	aLabel;
	SInt32		tabWidth;
	SInt32		totalWidth;
	SInt32		overflow;
	SInt32		tabEndWidth;
	SInt32 		i = 0;				

	if ((fTabsLocation == kTopTabs) || (fTabsLocation == kBottomTabs))
	{
		widthAvailable = GetBounds().right - GetBounds().left - (2 * kPanelEndMargin);
		tabEndWidth = fTabImageSize.h;
	}
	else
	{
		widthAvailable = GetBounds().bottom - GetBounds().top - (2 * kPanelEndMargin);
		tabEndWidth = fTabImageSize.v;
	}

	fTextStyle.SetPortTextStyle();
	
	switch (widthKind)
	{
		case kVariableTabWidth:
		
			totalWidth = 0;

			for (i = 0; i < fNumTabs; ++i)
			{
				this->GetTabLabel(i, aLabel);
				tabWidth = ::StringWidth(aLabel) + (2 * tabEndWidth);

				totalWidth += tabWidth;
				(*fTabInfo)[i].fTabWidth = tabWidth;
			}
			
			// If we overflowed the available width, reduce the tab sizes.

			overflow = totalWidth - widthAvailable;

			if (overflow > 0)
				for (i = 0; i < fNumTabs; ++i)
					(*fTabInfo)[i].fTabWidth -= (1 + (overflow / fNumTabs));
			break;

		case kCommonTabWidth:

			tabWidth = 0;

			for (i = 0; i < fNumTabs; ++i)
			{
				this->GetTabLabel(i, aLabel);
				tabWidth = MaxSInt32(tabWidth, ::StringWidth(aLabel));
			}
			
			tabWidth += (2 * tabEndWidth);

			for (i = 0; i < fNumTabs; ++i)
				(*fTabInfo)[i].fTabWidth = tabWidth;

			// If we overflowed the available width, reduce the tab sizes.

			overflow = (tabWidth * fNumTabs) - widthAvailable;

			if (overflow > 0)
				for (i = 0; i < fNumTabs; ++i)
					(*fTabInfo)[i].fTabWidth -= (1 + (overflow / fNumTabs));

			break;
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawBackground
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawBackground()
{
	CTempDrawingEnvironment_AC	env;

	// Paint the background.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
		{
		if (deep)
			this->ApplyBackgroundColor();
		else
			kRGBColorRamp[rW].SetForeColor();

		::PaintRect(GetBounds());
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawFrame
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawFrame()
{
	// If we have a frame, draw it; otherwise, draw the line/shading
	// for just the tabs edge.

	if (fDrawFrame)
		this->DrawFrameBox();
	else
		this->DrawFrameTop();
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawFrameTop
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawFrameTop()
{
	// Draw the line and shading for the tabs without a frame.

	CTempDrawingEnvironment_AC	env;

	CRect_AC	frameBounds(GetBounds());
	bool		drawEnabled = fEnabled && fActive;

	frameBounds.top = frameBounds.bottom - 3;
	
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		CRect_AC	r = frameBounds;
		
		if (deep && !drawEnabled)
			kRGBColorRamp[r10].SetForeColor();
		else
			kRGBColorRamp[rB].SetForeColor();

		::MoveTo(r.left, r.top);
		::LineTo(r.right - 1, r.top);
		
		if (deep)
		{
			if (drawEnabled)
				kRGBColorRamp[r3].SetForeColor();
			else
				kRGBColorRamp[r2].SetForeColor();

			::MoveTo(r.left + 1, r.top + 1);
			::LineTo(r.right - 2, r.top + 1);

			if (drawEnabled)
				kRGBColorRamp[rW].SetForeColor();
			else
				kRGBColorRamp[r2].SetForeColor();

			::MoveTo(r.left + 2, r.top + 2);
			::LineTo(r.right - 3, r.top + 2);
		}
	}
}

//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawFrameBox()
{
	// Draw the tab panel frame box.

	CTempDrawingEnvironment_AC	env;

	enum { Frame, Fill, LightOuter, LightInner, DarkOuter, DarkInner, kNumFrameColors };

	UInt8		colorIndexes[kNumFrameColors];
	CRect_AC	frameBounds(GetBounds());
	bool		drawEnabled = fEnabled && fActive;
	
	if (drawEnabled)
	{
		colorIndexes[Frame] = rB;
		colorIndexes[Fill] = r1;
		colorIndexes[LightOuter] = r3;
		colorIndexes[LightInner] = rW;
		colorIndexes[DarkOuter] = r6;
		colorIndexes[DarkInner] = r4;
	}
	else
		{
		colorIndexes[Frame] = r10;
		colorIndexes[Fill] = r2;
		colorIndexes[LightOuter] = r2;
		colorIndexes[LightInner] = r2;
		colorIndexes[DarkOuter] = r2;
		colorIndexes[DarkInner] = r2;
	}

	frameBounds.top += (fTabImageSize.v - 3);
	
	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		CRect_AC	r = frameBounds;
		
		if (deep)
			kRGBColorRamp[colorIndexes[Frame]].SetForeColor();
		else
		{
			kRGBColorRamp[rB].SetForeColor();
			
			if (! drawEnabled)
				PenPatGray();
		}
		
		::FrameRect(&r);
		
		if (deep)
			kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
		else
		{
			kRGBColorRamp[rW].SetForeColor();
			
			if (! drawEnabled)	// put it back if we set to gray above
				PenPatBlack();
		}
			
		r.Inset(1, 1);
		::PaintRect(&r);
		
		if (deep && drawEnabled)	// disabled has no raised light/dark edges
		{
			kRGBColorRamp[colorIndexes[LightOuter]].SetForeColor();
			::MoveTo(r.left, r.bottom - 2);
			::LineTo(r.left, r.top);
			::LineTo(r.right - 2, r.top);

			kRGBColorRamp[colorIndexes[DarkOuter]].SetForeColor();
			::Move(1, 1);
			::LineTo(r.right - 1, r.bottom - 1);
			::LineTo(r.left + 1, r.bottom - 1);

			kRGBColorRamp[colorIndexes[LightInner]].SetForeColor();
			::Move(0, -2);
			::LineTo(r.left + 1, r.top + 1);
			::LineTo(r.right - 3, r.top + 1);

			kRGBColorRamp[colorIndexes[DarkInner]].SetForeColor();
			::Move(1, 1);
			::LineTo(r.right - 2, r.bottom - 2);
			::LineTo(r.left + 2, r.bottom - 2);
			
			kRGBColorRamp[r2].SetForeColor();
			::MoveTo(r.left, r.bottom - 1);
			::Line(0, 0);
			::MoveTo(r.top, r.right - 1);
			::Line(0, 0);
		}
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawTabs
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawTabs()
{
	// Draw the tabs, one at a time.

	for (SInt32 i = 0; i < fNumTabs; ++i)
		this->DrawOneTab(i, kNotPressed);
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DrawOneTab
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DrawOneTab(SInt32 tabIndex, bool pressed)
{
	// Draw the specified tab.

	CTempDrawingEnvironment_AC	env;

	enum { EdgeFrame, OuterEdge, InnerEdge, Fill, InteriorFrame, InteriorOuterEdge, InteriorInnerEdge, kNumTabColors };

	UInt8	colorIndexes[kNumTabColors];

	bool				selected = (tabIndex == fCurrentTabIndex);
	CRect_AC			tabBounds;
	CStr255_AC			tabLabel;
	CPoint_AC			tabOrigin;
	AGAStringOutColor	textColor;
	UInt32				leftImageIndex;
	UInt32				rightImageIndex;
	bool				drawEnabled = fEnabled && fActive;

	this->GetTabBounds(tabIndex, tabBounds);
	this->GetTabLabel(tabIndex, tabLabel);
	
	tabBounds.left += fTabImageSize.h;
	tabBounds.right -= fTabImageSize.h;

	if ((! drawEnabled) || ! this->IsTabEnabled(tabIndex))
	{
		textColor = kDisabledOutput;
		leftImageIndex = selected ? kTabDisabledSelectedL : kTabDisabledDeselectedL;
		rightImageIndex = selected ? kTabDisabledSelectedR : kTabDisabledDeselectedR;

		colorIndexes[EdgeFrame] = r10;
		colorIndexes[OuterEdge] = r2;
		colorIndexes[InnerEdge] = r2;
		colorIndexes[Fill] = r2;
		colorIndexes[InteriorFrame] = selected ? r2 : r10;
		colorIndexes[InteriorOuterEdge] = r2;
		colorIndexes[InteriorInnerEdge] = r2;
	}
	else if (pressed)
	{
		textColor = kInverseOutput;
		leftImageIndex = kTabPressedL;
		rightImageIndex = kTabPressedR;

		colorIndexes[EdgeFrame] = rB;
		colorIndexes[OuterEdge] = r11;
		colorIndexes[InnerEdge] = r10;
		colorIndexes[Fill] = r9;
		colorIndexes[InteriorFrame] = rB;
		colorIndexes[InteriorOuterEdge] = r3;
		colorIndexes[InteriorInnerEdge] = rW;
	}
	else if (selected)
	{
		textColor = kNormalOutput;
		leftImageIndex = kTabNormalSelectedL;
		rightImageIndex = kTabNormalSelectedR;

		colorIndexes[EdgeFrame] = rB;
		colorIndexes[OuterEdge] = r3;
		colorIndexes[InnerEdge] = rW;
		colorIndexes[Fill] = r1;
		colorIndexes[InteriorFrame] = r1;
		colorIndexes[InteriorOuterEdge] = r1;
		colorIndexes[InteriorInnerEdge] = r1;
	}
	else
	{
		textColor = kNormalOutput;
		leftImageIndex = kTabNormalDeselectedL;
		rightImageIndex = kTabNormalDeselectedR;

		colorIndexes[EdgeFrame] = rB;
		colorIndexes[OuterEdge] = r3;
		colorIndexes[InnerEdge] = r2;
		colorIndexes[Fill] = r3;
		colorIndexes[InteriorFrame] = rB;
		colorIndexes[InteriorOuterEdge] = r3;
		colorIndexes[InteriorInnerEdge] = rW;
	}
	
	// Note that at this point, tabBounds is inset to avoid
	// the corner images.

	CGDIterator_AC	iter;
	bool			deep;

	while (iter.NextDevice(deep))
	{
		// Draw the tab corner images.

		tabOrigin.h = tabBounds.left - fTabImageSize.h;
		tabOrigin.v = tabBounds.top;
		
		this->CopyTabImage(leftImageIndex, tabOrigin, deep);
		tabOrigin.h = tabBounds.right;
		this->CopyTabImage(rightImageIndex, tabOrigin, deep);
		
		// Draw the straight edge shading.

		CRect_AC	r(tabBounds);

		kRGBColorRamp[colorIndexes[EdgeFrame]].SetForeColor();
		::MoveTo(r.left, r.top);
		::LineTo(r.right - 1, r.top);
		
		if (deep)
		{
			++r.top;
			--r.bottom;

			kRGBColorRamp[colorIndexes[OuterEdge]].SetForeColor();
			::MoveTo(r.left, r.top);
			::LineTo(r.right - 1, r.top);
			++r.top;

			kRGBColorRamp[colorIndexes[InnerEdge]].SetForeColor();
			::MoveTo(r.left, r.top);
			::LineTo(r.right - 1, r.top);
			++r.top;
			
			kRGBColorRamp[colorIndexes[InteriorInnerEdge]].SetForeColor();
			::MoveTo(r.left, r.bottom);
			::LineTo(r.right - 1, r.bottom);

			--r.bottom;
			kRGBColorRamp[colorIndexes[InteriorOuterEdge]].SetForeColor();
			::MoveTo(r.left, r.bottom);
			::LineTo(r.right - 1, r.bottom);

			--r.bottom;
			kRGBColorRamp[colorIndexes[InteriorFrame]].SetForeColor();
			::MoveTo(r.left, r.bottom);
			::LineTo(r.right - 1, r.bottom);

			kRGBColorRamp[colorIndexes[Fill]].SetForeColor();
			::PaintRect(&r);
		}
		else
		{
			// For 1-bit drawing, there's a pixel at the bottom that
			// may need to be erased or re-drawn. It's either leftover
			// from the frame (need to erase), or missing from being
			// previously selected (need to draw).
			
			r.bottom -= 3;
			++r.top;

			if (selected)
				kRGBColorRamp[rW].SetForeColor();
			else
				kRGBColorRamp[rB].SetForeColor();

			::MoveTo(r.left, r.bottom);
			::LineTo(r.right - 1, r.bottom);
			
			// The enable state of the tabs as a whole, and this
			// tab in particular require us to paint a gray pattern over
			// some or all of the tab corner images.

			if ((! drawEnabled) || ! this->IsTabEnabled(tabIndex))
			{
				CRect_AC	rectToPaint(tabBounds);
				
				rectToPaint.left -= fTabImageSize.h;
				rectToPaint.right += fTabImageSize.h;
				
				// If whole thing is enabled but this tab is disabled,
				// make sure we don't gray out the box frame line segment.
				if (drawEnabled && ! this->IsTabEnabled(tabIndex))
					rectToPaint.bottom -= 3;

				PenPatGray();
				::PenMode(patBic);
				::PaintRect(&rectToPaint);
				
				::PenNormal();
			}
			
			// We also need to paint the tab rectangular inside area
			// black or white, depending on whether the tab is pressed,
			// in preparation for the label text.
			
			if (pressed)
				kRGBColorRamp[rB].SetForeColor();
			else
				kRGBColorRamp[rW].SetForeColor();

			::PaintRect(&r);
		}
		
		AGAStringOut(tabLabel, tabBounds, smTruncMiddle, teCenter, textColor, deep, fTextStyle);
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::GetTabBounds
//----------------------------------------------------------------------------------------

void CTabbedView_AC::GetTabBounds(SInt32 tabIndex, CRect_AC& tabBounds)
{
	// Return the bounds of the specified tab.

	SInt16	tabOrigin = kPanelEndMargin;
	SInt16	tabWidth = (*fTabInfo)[tabIndex].fTabWidth;
	
	for (SInt32 i = 0; i < tabIndex; ++i)
		tabOrigin += (*fTabInfo)[i].fTabWidth;;

	switch (fTabsLocation)
	{
		case kTopTabs:
		case kBottomTabs:

			tabBounds.left = GetBounds().left + tabOrigin;
			tabBounds.right = tabBounds.left + tabWidth;
			
			if ((fDrawFrame && (fTabsLocation == kTopTabs)) ||
				(!fDrawFrame && (fTabsLocation == kBottomTabs)))
			{
				tabBounds.top = GetBounds().top;
				tabBounds.bottom = GetBounds().top + fTabImageSize.v;
			}
			else
			{
				tabBounds.top = GetBounds().bottom - fTabImageSize.v;
				tabBounds.bottom = GetBounds().bottom;
			}

			break;

		case kLeftTabs:
		case kRightTabs:

			tabBounds.top = GetBounds().top + tabOrigin;
			tabBounds.bottom = tabBounds.top + tabWidth;
			
			if ((fDrawFrame && (fTabsLocation == kLeftTabs)) ||
				(!fDrawFrame && (fTabsLocation == kRightTabs)))
			{
				tabBounds.left = GetBounds().left;
				tabBounds.right = tabBounds.left + fTabImageSize.h;
			}
			else
			{
				tabBounds.left = GetBounds().right - fTabImageSize.h;
				tabBounds.right = GetBounds().right;
			}

			break;
	}
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::CalculateTabSizeInfo
//----------------------------------------------------------------------------------------

void CTabbedView_AC::CalculateTabSizeInfo()
{
	// Recalculate the tab image size based on the tab size kind and
	// the tab location orientation.
	// Decide whether our bounds are large enough to draw a frame.

	switch (fTabSize)
	{
		case kSmallTabs:
		
			switch (fTabsLocation)
			{
				case kTopTabs:
				case kBottomTabs:
					fTabImageSize.Set(kSmallTabImageWidth, kSmallTabImageHeight);
					break;

				case kLeftTabs:
				case kRightTabs:
					fTabImageSize.Set(kSmallTabImageHeight, kSmallTabImageWidth);
					break;
			}

			break;

		case kLargeTabs:
		
			switch (fTabsLocation)
			{
				case kTopTabs:
				case kBottomTabs:
					fTabImageSize.Set(kLargeTabImageWidth, kLargeTabImageHeight);
					break;

				case kLeftTabs:
				case kRightTabs:
					fTabImageSize.Set(kLargeTabImageHeight, kLargeTabImageWidth);
					break;
			}

			break;
	}
	
	//
	// If the bounds are small, we don't draw the frame. We allow some
	// leeway in layout so that the user doesn't have to make the bounds
	// exactly match the image size to suppress the frame. This is fine
	// because if there is a frame, it won't be that small.
	//

	fDrawFrame = ((GetBounds().bottom - GetBounds().top) > (fTabImageSize.v + 16));
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DisposeTabInfo
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DisposeTabInfo()
{
	// Release the memory allocated for tab labels.

	if (fTabInfo != NULL)
	{
		for (SInt32 i = 0; i < fNumTabs; ++i)
			::DisposeHandle((Handle) (*fTabInfo)[i].fLabel);

		::DisposeHandle((Handle) fTabInfo);
		fTabInfo = NULL;
	}

	fNumTabs = 0;
}

//----------------------------------------------------------------------------------------
// STATIC DATA
//----------------------------------------------------------------------------------------

bool CTabbedView_AC::fgImagesAllocated = false;

// Each radio button drawing state has an offscreen image.
CAutoPtr_AC<CSimpleOffscreenImage_AC>	CTabbedView_AC::fgOffscreenTabImages [kNumTabImages] [kNumImageTypes];
CAutoPtr_AC<CSimpleOffscreenImage_AC>	CTabbedView_AC::fg1BitOffscreenTabImages [kNumTabImages] [kNumImageTypes];

// These are the color ramp indexes that are used to build the small tab offscreen images.
SInt8				CTabbedView_AC::kSmallTabImageValues [kNumTabImages] [kSmallTabImageHeight] [kSmallTabImageWidth] =
	{
		{	// kTabNormalDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB,  r5,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  r5,  r3,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12,  r5,  r2,  r2,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, r12,  r3,  r2,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3 },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3 },
//		{ OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
//		{ OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
//		{ OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
//		{ OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r4,  r9, r12, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT },
//		{  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT },
//		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT },
//		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT },
//		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB,  r4,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  r4,  r3,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12,  r4,  rW,  rW,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1 },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1 },
//		{ OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
//		{ OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
//		{ OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
//		{ OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		},

		{	// kTabNormalSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r4,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT },
//		{  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT },
//		{  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT },
//		{  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT },
//		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3, r11, r12, OUT },
//		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r7,  rB, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2, r11,  rB },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r7,  rB },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r3 },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  rW },
		},

		{	// kTabPressedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB, r11, r11 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12, r11, r11, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12, r11, r10, r10,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9 },
//		{ OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9 },
//		{ OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
//		{ OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
//		{ OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
//		{ OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{ r11,  r7,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{ r10, r10,  r7,  r8,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT },
//		{  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT },
//		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT },
//		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT },
//		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledDeselectedR
		{ r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2, r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{ r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
//		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledSelectedR
		{ r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2, r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
//		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

	};

// These are the color ramp indexes that are used to build the large tab offscreen images.
SInt8				CTabbedView_AC::kLargeTabImageValues [kNumTabImages] [kLargeTabImageHeight] [kLargeTabImageWidth] =
	{
		{	// kTabNormalDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB,  r5,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  r5,  r3,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12,  r5,  r2,  r2,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT, r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ OUT,  rB,  r3,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{ r12,  r5,  r2,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r4, r11, r12, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r4, r11, r12, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4, r11, r12, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4, r11, r12, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r7,  rB, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r5,  rB, OUT },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r4,  r9, r12 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB,  r4,  r3 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  r4,  r3,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12,  r4,  r3,  rW,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT, r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ OUT,  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{ r12,  r4,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  rB,  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  r3,  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		{  rW,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1 },
		},

		{	// kTabNormalSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r3,  r4,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r4, r11, r12, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r7,  rB, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3, r11, r12, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2,  r7,  rB, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r7,  rB, OUT },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r2, r11,  rB },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r7,  rB },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r3,  r3 },
		{  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  r1,  rW },
		},

		{	// kTabPressedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12,  rB, r11, r11 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r12, r11, r11, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r12, r11, r11, r10,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT, r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ OUT,  rB, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{ r12, r11, r10,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{ r11,  r7,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{ r10, r10,  r7,  r8,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r8,  rB, OUT },
		{  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r9,  r7, r11, r12 },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3,  r3 },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledDeselectedR
		{ r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2, r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{ r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10, r10,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ OUT, r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{ r10,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

		{	// kTabDisabledSelectedR
		{ r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2, r10, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10, OUT },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2, r10 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		{  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2,  r2 },
		},

	};

// These are the color ramp indexes that are used to build the small 1-bit tab offscreen images.
SInt8				CTabbedView_AC::k1BitSmallTabImageValues [kNumTabImages] [kSmallTabImageHeight] [kSmallTabImageWidth] =
	{
		{	// kTabNormalDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
//		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
//		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
//		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
//		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
//		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
//		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
//		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
//		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
//		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
//		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

	};

// These are the color ramp indexes that are used to build the large 1-bit tab offscreen images.
SInt8				CTabbedView_AC::k1BitLargeTabImageValues [kNumTabImages] [kLargeTabImageHeight] [kLargeTabImageWidth] =
	{
		{	// kTabNormalDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabNormalSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT, OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{ OUT,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabPressedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB, OUT },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledDeselectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledSelectedL
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rB,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT, OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{ OUT,  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rB,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

		{	// kTabDisabledSelectedR
		{  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rB,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB, OUT },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rB },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		{  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW,  rW },
		},

	};

//----------------------------------------------------------------------------------------
// CTabbedView_AC::AllocateTabImages
//----------------------------------------------------------------------------------------

void CTabbedView_AC::AllocateTabImages()
{
	// Allocate the offscreen images for the various sizes & states.

	CPoint_AC	imageSize(kLargeTabImageWidth, kLargeTabImageHeight);
	UInt32 		imageIndex = 0;
	
	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
	{
		fgOffscreenTabImages [imageIndex] [kLargeTabImages] = TH_new CSimpleOffscreenImage_AC;
	
		fgOffscreenTabImages [imageIndex] [kLargeTabImages] ->CreateImageData(
					imageSize,
					&kLargeTabImageValues[imageIndex][0][0],
					NULL, 
					kRGBColorRamp[r2]);
	}

	
	imageSize.Set(kSmallTabImageWidth, kSmallTabImageHeight);

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
	{
		fgOffscreenTabImages [imageIndex] [kSmallTabImages] = TH_new CSimpleOffscreenImage_AC;
	
		fgOffscreenTabImages [imageIndex] [kSmallTabImages] ->CreateImageData(
					imageSize,
					&kSmallTabImageValues[imageIndex][0][0],
					NULL, 
					kRGBColorRamp[r2]);
	}
		
	imageSize.Set(kLargeTabImageWidth, kLargeTabImageHeight);

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
	{
		fg1BitOffscreenTabImages [imageIndex] [kLargeTabImages] = TH_new CSimpleOffscreenImage_AC;
	
		fg1BitOffscreenTabImages [imageIndex] [kLargeTabImages] ->CreateImageData(
					imageSize,
					&k1BitLargeTabImageValues[imageIndex][0][0],
					NULL, 
					kRGBColorRamp[r2]);
	}
		
	imageSize.Set(kSmallTabImageWidth, kSmallTabImageHeight);

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
	{
		fg1BitOffscreenTabImages [imageIndex] [kSmallTabImages] = TH_new CSimpleOffscreenImage_AC;
	
		fg1BitOffscreenTabImages [imageIndex] [kSmallTabImages] ->CreateImageData(
					imageSize,
					&k1BitSmallTabImageValues[imageIndex][0][0],
					NULL, 
					kRGBColorRamp[r2]);
	}
		
	fgImagesAllocated = true;
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::CopyTabImage
//----------------------------------------------------------------------------------------

void CTabbedView_AC::CopyTabImage(UInt32 imageIndex, const CPoint_AC& imageDestination, bool deep)
{
	// Blast the specified offscreen image into the radio button's button area.

	if (deep)
		fgOffscreenTabImages [imageIndex] [(fTabSize == kSmallTabs) ? kSmallTabImages : kLargeTabImages]->DrawImage(imageDestination.h, imageDestination.v);
	else
		fg1BitOffscreenTabImages [imageIndex] [(fTabSize == kSmallTabs) ? kSmallTabImages : kLargeTabImages]->DrawImage(imageDestination.h, imageDestination.v);
}

//----------------------------------------------------------------------------------------
// CTabbedView_AC::DisposeTabImages
//----------------------------------------------------------------------------------------

void CTabbedView_AC::DisposeTabImages()
{
	UInt32 imageIndex = 0;
	
	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
		if (!fgOffscreenTabImages [imageIndex] [kLargeTabImages].IsNULL())
			fgOffscreenTabImages [imageIndex] [kLargeTabImages].reset();

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
		if (!fgOffscreenTabImages [imageIndex] [kSmallTabImages].IsNULL())
			fgOffscreenTabImages [imageIndex] [kSmallTabImages].reset();

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
		if (!fg1BitOffscreenTabImages [imageIndex] [kLargeTabImages].IsNULL())
			fg1BitOffscreenTabImages [imageIndex] [kLargeTabImages].reset();

	for (imageIndex = 0; imageIndex < kNumTabImages; ++imageIndex)
		if (!fg1BitOffscreenTabImages [imageIndex] [kSmallTabImages].IsNULL())
			fg1BitOffscreenTabImages [imageIndex] [kSmallTabImages].reset();
}

