//----------------------------------------------------------------------------------------
// CNetworkTransaction_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CNetworkTransaction_AC__
#define __CNetworkTransaction_AC__

// Network

#ifndef __Net_AC__
#include "Net_AC.h"
#endif

// Patterns

#ifndef __CBuffer_AC__
#include "CBuffer_AC.h"
#endif

#ifndef __CStateMachine_AC__
#include "CStateMachine_AC.h"
#endif

#ifndef __CErrorHandler_AC__
#include "CErrorHandler_AC.h"
#endif

// Core

#ifndef __MRefCountable_AC__
#include "MRefCountable_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

// Streams
			
#ifndef __CBufferStream_AC__
#include "CBufferStream_AC.h"
#endif				

// fwd references

class CEvent_AC;
class CServiceHandler_AC;
class CNetworkTransaction_AC;
class MEventTransceiver_AC;


// typedef 

typedef CRefCountingPtr_AC<CNetworkTransaction_AC> CNetworkTransactionPtr_AC;

//----------------------------------------------------------------------------------------
// class CNetworkTransaction_AC is conceptually a task to performed over the net.
// These can be queued up in the ServiceHandler, and do tasks such as send or 
// receive data.
//----------------------------------------------------------------------------------------

class EXPORT_AC CNetworkTransaction_AC : 	public CStateMachine_AC, 
											public MRefCountable_AC,
											public MErrorProne_AC
{
public:
	enum eStates
	{
		kComplete = CStateMachine_AC::kNullState,
		kOpening,
		kProcessing,
		kBlocking,
		kSuspending,
		kResuming,
		kError,
		kAbort,
		kLastState = kError
	};
	enum eMisc
	{
		kDefaultBufferSize 	= 10000
	};
		
	CNetworkTransaction_AC(bool doExpedite = false);
		// ctor
	
	CNetworkTransaction_AC(const CNetworkTransaction_AC&);

	CNetworkTransaction_AC(unsigned long size, char* inPtr  = NULL, bool doExpedite = false );
		// This will make a CBuffer_AC for you.
		// if you hand the messageblock a buffer, it will *not* delete it.
		// if you hand it a size, and not a buffer, it will create a buffer and it *will*
		// delete it. 

	CNetworkTransaction_AC(CBuffer_AC* inBuffer, bool doExpedite = false);
		// This will aquire, not copy inBuffer. The buffer will be refcounted.
	
	virtual ~CNetworkTransaction_AC();

	CNetworkTransaction_AC& operator = (const CNetworkTransaction_AC& inTransaction);
		// assignment operator copies data, if any.
			
//-- Info Accessors.	
		
	bool IsExpedited() const;

	bool IsData() const;	

	bool IsSender() const;

	CError_AC GetError() const;

	void SetError(CError_AC inError);
	
	virtual void Complete();
		// Tell the transaction to complete. This will remove it from the CServiceHandler's
		// queue.

	bool IsComplete() const;	
	
//-- Transfer Management 

	long GetTransferedAmount() const
		// This returns how much data was transfered between when this transaction was
		// opened and closed. Changing the buffer while the transaction is open will
		// reset the tally.
	{
		return fTransferedAmt;
	}
	void SetExpectedTransferAmount(long inAmt)
		// This sets how much the transaction is expecting to transfer. This defaults
		// to the size of the buffer. Note that setting a new buffer will revert the
		// expected transfer amount, so set this after you set your buffer.
	{
		fExpectedAmt = inAmt;
	}
	long GetExpectedTransferAmount() const
		// This returns how much this transaction is planning to transfer before it completes.
	{
		return fExpectedAmt;
	}

//-- Buffer management

	void SetBuffer(CBuffer_AC* inBuffer);
		// This releases old buffer and aquires inBuffer, i.e. ref counts it. 

	void SetBuffer(unsigned long size, char* inPtr  = NULL);
		// This releases old buffer.
		// This will make a new CBuffer_AC.
		// If you hand the messageblock a buffer, it will *not* delete it. 
		// If you hand it a size, and not a buffer, it will create a buffer and it *will*
		
	void CopyBuffer(CBuffer_AC* inBuffer);
		// This makes fBuffer an identical copy of inBuffer.
		
	CBuffer_AC* GetBuffer();

	void ReleaseBuffer();

// streaming data
	CBufferStream_AC& GetStream();
		// use the stream to insert or remove formatted data. This will make
		// a stream if there isn't one assigned.
		
	void SetStream(CBufferStream_AC* inStream);
	
	void ReleaseStream();
	
	void SyncStreamWithBuffer();
		// Make sure the stream is iterating over the transactions buffer.

	virtual CNetworkTransaction_AC *Clone() = 0;
	
//-- Implementation
protected:

	virtual void Process( CServiceHandler_AC* inServiceHandler) = 0 ;
		// Override this to do your work. 
				
	virtual void ReceiveEvent(CEvent_AC* inEvent, MEventTransceiver_AC* inSender);
		// Override this to Receive an event from the service handler.
		// Use this if you need to block on an event like kSentData.
	
	void SetIsSender(bool);
		// Called by service handler.

	long GetPriority() const;	
		// Called by service handler.

	virtual void Open();
		// Changes state to kOpening. Called by service handler.
	
	void MakeStream();
				
	virtual void Expedite(bool isExpedited = true);
		// Call this to set the transaction to expedited.
		// Changing expidited status isn't supported after construction.
		
	enum ePriorities
	{
		kCommandPriority = 10,
		kDataPriority	 = 20
	};	
			
	void SetPriority(long inPriority);
		// The priority detirmines in what order the transaction is 
		// processed. It does not determine where in the queue the
		// transaction is put.
		
	void SetIsData(bool);
		// Called by service handler.
		
protected:
	
	enum eIdentityFlags
	{
		kExpedited			= 	1,		 
		kSender				= 	1 << 2,  
		kData				=  	1 << 3   
	};

	unsigned long								fFlags;
	CRefCountingPtr_AC<CBuffer_AC>				fBuffer;
	
	CAutoPtr_AC<CBufferStream_AC>				fStream;
	
	long										fTransferedAmt;
	long										fExpectedAmt;
	
	long										fPriority;
	CError_AC									fError;
	
	
	friend class CServiceHandler_AC;
};

// inline methods

inline bool CNetworkTransaction_AC::IsExpedited() const
{	
	return ::TestBits_AC(fFlags, kExpedited); 
}

inline void CNetworkTransaction_AC::SetIsSender(bool inBool)
{
	::SetBits_AC(fFlags, kSender, inBool);
}

inline bool CNetworkTransaction_AC::IsSender() const
{
	return ::TestBits_AC(fFlags, kSender);
}	

inline bool CNetworkTransaction_AC::IsData() const
{
	return ::TestBits_AC(fFlags, kData);
}	

inline void CNetworkTransaction_AC::SetPriority(long inPriority)
{
	fPriority = inPriority;
}

inline long CNetworkTransaction_AC::GetPriority() const
{
	return fPriority;
}

inline CError_AC CNetworkTransaction_AC::GetError() const
{
	return fError;
}

inline void CNetworkTransaction_AC::SetError(CError_AC inError)
{
	fError = inError;
}

inline CBufferStream_AC& CNetworkTransaction_AC::GetStream()
{
	if(!fStream) MakeStream();
	return *fStream;
}

inline bool CNetworkTransaction_AC::IsComplete() const
{
	return GetState() == kComplete;
}

#endif