//----------------------------------------------------------------------------------------
// CAbstractThreadContext_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------


#ifndef __CAbstractThreadContext_AC__
#define __CAbstractThreadContext_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

// ACS

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#if qWantsMP
#ifndef MP_h
#include <MP.h>
#endif
#endif


//----------------------------------------------------------------------------------------
// FORWARD DECLARATIONS
//----------------------------------------------------------------------------------------

class CMonitorLock_AC;
class CThread_AC;
class CThreadRunner_AC;

//----------------------------------------------------------------------------------------
// Shared type (not used in the abstract base class, but in subclasses)
//----------------------------------------------------------------------------------------

typedef void (*SimpleTaskProcPtr)(void *parameter);

//----------------------------------------------------------------------------------------
// CAbstractThreadContext
//----------------------------------------------------------------------------------------

class EXPORT_AC CAbstractThreadContext_AC
// Abstract base class for objects containing contextual information for a single
// execution context; thread or task.  The context object is not normally accessed
// or created directly, but is used to implement the services provided by thread
// handles and through global utility functions.
{
public:
	static CAbstractThreadContext_AC* GetCurrentContext();
		// Returns a pointer to the concrete context object for the currently 
		// active thread or task.

	CAbstractThreadContext_AC(CThread_AC* adoptedThread = NULL);
		// The context object takes ownership of the thread to be executed, 
		// deleting it when the context is destructed.

	virtual ~CAbstractThreadContext_AC();

	static void SuspendCurrentContext();
		// Suspends the current context.
	
	static void YieldCurrentContext();
		// Yields the current context.
		
	virtual void Suspend() = 0;	
		// Causes the thread or task to stop executing until Resume is called.

	virtual void Resume() = 0;
		// Resumes the execution of a suspended thread or task.

	virtual bool IsAlive() = 0;
		// Returns true if the thread or task has not been terminated.

	virtual void Yield() = 0;
		// Yields the processor to other contexts of the same type.


	void	Acquire();
		// Increments the reference count.  Contexts are reference counted so they will
		// not be deleted before execution has terminated and all referencing handles
		// have been destructed.

	void	Release();
		// Decrements the reference count.  Contexts are reference counted so they will
		// not be deleted before execution has terminated and all referencing handles
		// have been destructed.

	long	GetRefCount();
		// Returns the current reference count.

	CThread_AC*	GetThread();
		// Getter returns a pointer to the adopted thread.


#if qWantsMP

	static MPQueueID	GetCooperativeRequestQueue();
		// This MP queue is used to receive requests for execution in a cooperative 
		// (toolbox safe) context from MP tasks.  Requests are executed in the order
		// received by the yield thread each time it gets scheduled to execute.
		// This call is actually handled by the CYieldThread, which is used in the 
		// implementation to manage cooperative tasks.

#endif
	
protected:

#if qYieldToOtherProcesses || qWantsMP

	static void InitializeYieldThread();
		// Starts up a thread manager thread which yields time to, and processes requests
		// for, MP tasks.  This is essential on a single processor machine when using
		// MP tasks, since MP tasks do not preempt the main context, and would otherwise
		// only get limited time at certain system calls, such as WaitNextEvent.
		// This call is actually handled by the CYieldThread, which is used in the
		// implementation to manage cooerpative tasks.
		
#endif

private:

	CAutoPtr_AC<CMonitorLock_AC> fLock;
		// Monitor lock for protecting reference counting of the context object.

	CAutoPtr_AC<CThread_AC> fAdoptedThread;
		// thread object comprising the functions to be executed in the thread or
		// task.  The context takes care of deleting the thread.

	long fRefCount;
		// The reference count.  When the count is decremented to zero, the context is
		// deleted.

#if qThreadDebug
public:
	void* fStackFrame;
		// When compiled with debugging on, fStackFrame gets set to the current stack
		// pointer whenever a method on the context gets called.  This can help in 
		// debugging a task that has stopped, especially an MP task, since the MP API
		// does not make it easy to get the current stack pointer for an MP task that
		// is not stopped at a breakpoint.  Use the value in this field to get an
		// approximate stack crawl for the task with either MacsBug or the Power
		// Macintosh debugger.
#endif
};

//----------------------------------------------------------------------------------------
// Global functions
//----------------------------------------------------------------------------------------

#if __POWERPC__ && qThreadDebug
EXPORT_AC asm void* GetSP();
		// Calling this routine returns a copy of the current stack pointer for saving
		// in CAbstractThreadContext_AC::fStackFrame to assist debugging.

#define DEBUG_SETUP_STACK_FRAME() (CAbstractThreadContext_AC::GetCurrentContext()->fStackFrame = GetSP())
#else
#define DEBUG_SETUP_STACK_FRAME() (NULL)
#endif

//----------------------------------------------------------------------------------------
// Forward declarations of template collection classes.
//----------------------------------------------------------------------------------------

template <class T>
struct less;

template <class T>
class allocator;
#ifdef MSIPL_DEF_TEMPARG
template <class Key, class T, class Compare = less<Key>,
          class Allocator = allocator<T> >
#else
template <class Key, class T, class Compare, class Allocator>
#endif
class map;


#endif
