//----------------------------------------------------------------------------------------
// CThreadRunner_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CThreadRunner_AC__
#define	__CThreadRunner_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif


//----------------------------------------------------------------------------------------
//	Forward declarations and includes
//----------------------------------------------------------------------------------------

class	CAbstractThreadSchedule_AC;
class	CThread_AC;
class	CAbstractThreadContext_AC;

//----------------------------------------------------------------------------------------
//	CThreadHandle_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CThreadHandle_AC
{
//
// CThreadHandle_AC is a class used to increment the reference count for a starting thread's context,
// so that the context isn't deleted until the thread has had a chance to complete.
// The thread runner subclasses from this class as well, so that the context is guaranteed
// to be valid as long as the thread runner is valid.
//
public:
	CThreadHandle_AC(CAbstractThreadContext_AC* itsContext);
		// Increments the reference count for the currently executing thread.

	virtual ~CThreadHandle_AC();
		// Destructor decrements the reference count for the currently executing thread

protected:

	CAbstractThreadContext_AC* fContext;
};

//----------------------------------------------------------------------------------------
//	CThreadRunner_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC CThreadRunner_AC : public CThreadHandle_AC
{
//
// CThreadRunner_AC is the class used to launch and interact with the state of a thread
// or MP task.  Multiple handles may be associated with a single executing thread, and
// reference counting is used to ensure that valid information about the thread state is
// available to any outstanding handles, even after the thread has terminated.
//
// Construct a CThreadRunner_AC using a new object of a subclass of CThread_AC 
// to start a new thread of execution.  The CThread_AC object will be deleted
// when the thread is no longer executing and all CThreadRunners_AC referencing it have 
// been destructed.
//
// To cooperatively terminate a running thread, call RequestShutdown.  To synchronize on 
// the termination of the thread, call Join.
//
public:

	//--------------------------------------------------------------------------------
	//	Constructors, destructor, and assignment
	//--------------------------------------------------------------------------------

	CThreadRunner_AC(CThread_AC* aThread);
		// create new thread in the calling task - acquires aThread
		// which is deleted upon termination of the thread by its context.
		// The resources owned by the thread are also discarded.

	CThreadRunner_AC(const CThreadRunner_AC& aThread);
		// copy constructor

	CThreadRunner_AC& operator=(const CThreadRunner_AC& aThread);
		// copy assignment

	bool operator==(const CThreadRunner_AC& toWhat) const;
		// Comparison operator for comparing CThreadRunners

	bool operator!=(const CThreadRunner_AC& toWhat) const;
		// Comparison operator for comparing CThreadRunners

	virtual ~CThreadRunner_AC();
		// Destructor

	//--------------------------------------------------------------------------------
	//	General Thread methods
	//--------------------------------------------------------------------------------

	virtual bool IsSelf() const;
		// return true if the thread handle represents the current thread.

	virtual bool Exists() const;
		// Determines if the thread still exists (e.g., it is either running or suspended).

	virtual void RequestShutdown();
		// Call to request a graceful shutdown of the thread
		// Override this method only if immediate processing other than setting the
		// fShudownRequested flag in the thread is required.

	virtual void Join();
		// Calls RequestShutdown and blocks the calling thread until the
		// thread terminates.
				
	//--------------------------------------------------------------------------------
	// General Thread protocols
	//--------------------------------------------------------------------------------
		
	virtual void Resume();
		// Resumes the execution of this thread if the thread stopped itself by 
		// calling Suspend.
		
	virtual void Kill();
		// Terminates the thread - this call is unsafe since no user-level
		// clean-up code gets to run.  CancelWaitAndPostException() or your
		// own ad hoc protocol should be used to provide safe termination.

	virtual void CancelWaitAndPostException() const;
		// Causes a blocking kernel call to be unblocked and an exception to be
		// thrown in the thread (*this).
	
};


#endif	// __CThreadRunner_AC__

