//----------------------------------------------------------------------------------------
// CThread_AC.h
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CThread_AC__
#define	__CThread_AC__

#ifndef __Threads_AC__
#include "Threads_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

//----------------------------------------------------------------------------------------
//	Forward declarations and includes
//----------------------------------------------------------------------------------------

class	CAbstractThreadSchedule_AC;

//----------------------------------------------------------------------------------------
//	CThread_AC
//----------------------------------------------------------------------------------------

class EXPORT_AC	CThread_AC
//
//	Abstract base class representing the code that a thread should execute
//	The derived class should override Run() to provide the code to execute
//	Prepare() is called before Run() is called. Prepare() allows for
//	initialization before start-up, while still synchronized with the launching
//	thread.
//
{

public:

	//--------------------------------------------------------------------------------
	// Constructors, destructor, and assignment
	//--------------------------------------------------------------------------------

	CThread_AC();
		// default constructor

	CThread_AC(const CThread_AC& source);
		// copy constructor

	virtual ~CThread_AC();
		// Destructor

	CThread_AC& operator=(const CThread_AC& sourceThreadProgram);
		// Assignment operator

	//--------------------------------------------------------------------------------
	// Methods a client may or must override.
	//--------------------------------------------------------------------------------

	virtual void Prepare();
		// Prepare() is called in the new thread before the CThreadRunner_AC constructor completes.
		// It should not be called by clients.

	virtual void Run() = 0;
		// Represents the entry point for the code to be run in the thread.
		// Override this method to provide the code the thread is to execute.

	//--------------------------------------------------------------------------------
	// Methods a subclass should always call (in Run, if implementing a loop in Run)
	//--------------------------------------------------------------------------------

	bool ShutdownRequested();
		// Returns true if RequestShutdown has been called.  Subclasses should gracefully
		// terminate their execution when this call returns true.

	static void Yield();
		// Yields this thread, allowing other (cooperatively run) threads time to execute.

protected:
	
	static void Suspend();
		// Suspends this thread.  Must be called from within the thread (e.g., Run).
		// Must be awakened by another thread by calling CThreadRunner's Resume method.
		
	//--------------------------------------------------------------------------------
	// Other methods a client may optionally override or call.
	//--------------------------------------------------------------------------------

public:

	virtual CAbstractThreadSchedule_AC* GetThreadSchedule();
		// Returns the schedule object to be used in the thread, creating one if not created yet.
		// Override this method if you dont want the default thread schedule.

	virtual size_t GetStackSize() const;
		// Returns the size of the stack to be set up for the thread.
		// Override this method if you dont want the default stack size.
	
	virtual void RequestShutdown();
		// RequestShutdown() may be called either directly or through the Thread runner to
		// request that the thread gracefully terminate itself.  It will have no effect
		// unless the CThread_AC subclass periodically checks the value returned by
		// ShutdownRequested in its Run method.
		// Override this method only if immediate processing other than setting the
		// fShudownRequested flag is required.

private:

	bool fShutdownRequested;		// Indicates whether this thread has a shutdown request.

	static CAutoPtr_AC<CAbstractThreadSchedule_AC> fSchedule;
};

#endif	// CThread_AC

