//----------------------------------------------------------------------------------------
// CMPThreadContext_AC.cp
// Copyright  1996 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CMPThreadContext_AC__
#include "CMPThreadContext_AC.h"
#endif

#if qWantsMP

#ifndef __CMPAllocator_AC__
#include "CMPAllocator_AC.h"
#endif

#ifndef __CException_AC__
#include "CException_AC.h"
#endif

#ifndef MAP_H
#include <map.h>
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CMPMonitorLock_AC__
#include "CMPMonitorLock_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Instantiate static globals.
//----------------------------------------------------------------------------------------

CAutoPtr_AC<map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> > > CMPThreadContext_AC::fMap;
CAutoPtr_AC<CAbstractMonitorLock_AC> CMPThreadContext_AC::fMapLock;
CAutoPtr_AC<CMPThreadContext_AC> CMPThreadContext_AC::fMainContext;

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

CMPThreadContext_AC* CMPThreadContext_AC::GetCurrentMPContext()
{
	CMPThreadContext_AC* result = NULL;
	// Lookup context in static list using ID as key.
	if (fMap.get() != NULL && fMapLock.get() != NULL) // Check both for safety during static destruction.
	{
		CWhileSynchronized_AC entry(fMapLock);
		if (!fMap->empty()) 
		{
			MPTaskID currentTask = ::MPCurrentTaskID();
			map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> >::iterator taskMapIterator = fMap->find(currentTask);
			if ( (*taskMapIterator).first == currentTask) 
			{
				result = (*taskMapIterator).second;
			}
		}
	}
	if (result == NULL)
	{
		// Main context may still need to be created!
		if (fMainContext.get() == NULL)
		{
			fMainContext = TH_new CMPThreadContext_AC(::HasMP_AC() ? ::MPCurrentTaskID(): NullMPTaskID);
		}
		// Return main context if not an MP context.
		result = fMainContext.get();
	}
	return result;
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

CMPThreadContext_AC::CMPThreadContext_AC(
			TaskProc			entryPoint,
			void				*parameter,
			ByteCount		stackSize,
			MPQueueID		notifyQueue,
			void 			*terminationParameter1,
			void 			*terminationParameter2,
			MPTaskOptions	options,
			CThread_AC* adoptedThread) :
	CAbstractThreadContext_AC(adoptedThread),
	fTaskID(NullMPTaskID),
	fLock(NullMPSemaphoreID),
	fEntry(NULL),
	fParameter(NULL),
	fDone(false)
{
	OSErr err;
	if (fMapLock.get() == NULL) 
	{
		fMapLock = TH_new CMPMonitorLock_AC();
		fMap = TH_new map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> >();

#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}
	CWhileSynchronized_AC entry(fMapLock);
	err = ::MPCreateTask(entryPoint, parameter, stackSize, notifyQueue, 
			terminationParameter1, terminationParameter2, options, &fTaskID);
	::ThrowIfError_AC(err);
	err = 	::MPCreateBinarySemaphore(&fLock);
	::ThrowIfError_AC(err);
	::MPWaitOnSemaphore(fLock, kDurationForever);  // Should aquire immediately.
	// Add context to static list of contexts keyed by Task ID.
	pair<const MPTaskID, CMPThreadContext_AC* > taskMapPair(fTaskID, this);
	fMap->insert(taskMapPair);
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

CMPThreadContext_AC::CMPThreadContext_AC(
			MPTaskID	taskID) :
	CAbstractThreadContext_AC(NULL),
	fTaskID(taskID),
	fLock(NullMPSemaphoreID),
	fEntry(NULL),
	fParameter(NULL),
	fDone(false)
{
	// This constructor will be called from static construction even when MP is not present.
	if (::HasMP_AC()) {
		// Lookup context in static list using ID as key.
		bool foundContext = false;
		if (fMapLock.get() == NULL) 
		{
			fMapLock = TH_new CMPMonitorLock_AC();
			fMap = TH_new map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
			bool settingMainContext = false;
			// To prevent recursion, fMainContext should be set before calling out.
			if (fMainContext.get() == NULL)
			{
				settingMainContext = true;
				fMainContext = this;
			}
			InitializeYieldThread();	// Make sure the yielding thread is initialized.
			// Clear the auto pointer so it doesn't destruct this when reassigned!
			if (settingMainContext) fMainContext.release();
#endif
		}
		CWhileSynchronized_AC entry(fMapLock);
		if (!fMap->empty()) 
		{
			map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> >::iterator taskMapIterator = fMap->find(taskID);
			foundContext = ((*taskMapIterator).first == taskID);
		}
		if (!foundContext) 
		{
			OSErr err;
			err = 	::MPCreateBinarySemaphore(&fLock);
			::ThrowIfError_AC(err);
			::MPWaitOnSemaphore(fLock, kDurationForever);  // Should aquire immediately.
			// Add context to static list of contexts keyed by Task ID.
			pair<const MPTaskID, CMPThreadContext_AC* > taskMapPair(fTaskID, this);
			fMap->insert(taskMapPair);
		}
	}
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

OSStatus CMPThreadContext_AC::MPWrapperProc(CMPThreadContext_AC* context)
{
	(context->fEntry)(context->fParameter);
	context->fDone = true;
	return 0;
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

CMPThreadContext_AC::CMPThreadContext_AC(
			SimpleTaskProcPtr	entryPoint,
			void				*parameter,
			ByteCount		stackSize,
			MPQueueID		notifyQueue,
			void 			*terminationParameter1,
			void 			*terminationParameter2,
			MPTaskOptions	options,
			CThread_AC* adoptedThread) :
	CAbstractThreadContext_AC(adoptedThread),
	fTaskID(NullMPTaskID),
	fLock(NullMPSemaphoreID),
	fEntry(entryPoint),
	fParameter(parameter),
	fDone(false)
{
	OSErr err;
	if (fMapLock.get() == NULL) 
	{
		fMapLock = TH_new CMPMonitorLock_AC();
		fMap = TH_new map<MPTaskID,CMPThreadContext_AC* ,less<MPTaskID> ,CMPAllocator_AC<CMPThreadContext_AC*> >();
#if qYieldToOtherProcesses || qWantsMP
		InitializeYieldThread();	// Make sure the yielding thread is initialized.
#endif
	}
	CWhileSynchronized_AC entry(fMapLock);
	err = ::MPCreateTask((TaskProc)MPWrapperProc, this, stackSize, notifyQueue, 
			terminationParameter1, terminationParameter2, options, &fTaskID);
	::ThrowIfError_AC(err);
	err = 	::MPCreateBinarySemaphore(&fLock);
	::ThrowIfError_AC(err);
	::MPWaitOnSemaphore(fLock, kDurationForever);  // Should aquire immediately.
	// Add context to static list of contexts keyed by Task ID.
	pair<const MPTaskID, CMPThreadContext_AC* > taskMapPair(fTaskID, this);
	fMap->insert(taskMapPair);
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

CMPThreadContext_AC::~CMPThreadContext_AC()
{
	// Main context gets statically constructed even if MP is not installed.
	if (::HasMP_AC()) 
	{
		::MPDeleteSemaphore(fLock);
		CWhileSynchronized_AC entry(fMapLock);
		if (fMap.get() != NULL)
			fMap->erase(fTaskID);
	}
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

void  CMPThreadContext_AC::Suspend()
{
	DEBUG_SETUP_STACK_FRAME();

	::MPWaitOnSemaphore(fLock, kDurationForever);	// Second attempt to aquire lock should block.
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

void  CMPThreadContext_AC::Resume()
{
	::MPSignalSemaphore(fLock);	// Unblock the task.
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

bool CMPThreadContext_AC::IsAlive()
{
	// Need way to check if it has been killed.
	return (fDone == false);
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

void  CMPThreadContext_AC::Yield()
{
	::MPYield();
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

MPSemaphoreID CMPThreadContext_AC::GetLock()
{
	return fLock;
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

MPTaskID CMPThreadContext_AC::GetTaskID()
{
	return fTaskID;
}

//----------------------------------------------------------------------------------------
// CMPThreadContext_AC
//----------------------------------------------------------------------------------------

long CMPThreadContext_AC::GetCurrentTaskCount()
{
	long result = 0;
	if (fMap.get() != NULL)
	{
		result = fMap->size();
		if (fMainContext.get() != NULL)
			result -= 1;	// Don't count main context.
	}
	return result;
}

#endif
