//----------------------------------------------------------------------------------------
// CMonitorCondition_AC.cp
// Copyright  1996-1997 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------

#ifndef __CMonitorCondition_AC__
#include "CMonitorCondition_AC.h"
#endif

#if __POWERPC__
#ifndef __CMPAllocator_AC__
#include "CMPAllocator_AC.h"
#endif

// Trick to use MP safe allocator for deque.
#ifndef Allocator
#define Allocator CMPAllocator_AC
#endif

#endif // __POWERPC__

#ifndef DEQUE_H
#include <Deque.h>
#endif

#if __POWERPC__
#ifndef MP_h
#include <MP.h>
#endif
#endif

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

// Specialization for Pointer type iterators
null_template
struct iterator_trait <CAbstractThreadContext_AC*const*> {
    typedef ptrdiff_t                   		distance_type;
    typedef CAbstractThreadContext_AC* const	value_type;
    typedef random_access_iterator_tag   		iterator_category;
};

//----------------------------------------------------------------------------------------
// Constructor with monitor lock
//----------------------------------------------------------------------------------------

CMonitorCondition_AC::CMonitorCondition_AC ( CAbstractMonitorLock_AC* lock ):
    fLock( lock ),
    fDefaultDelay( CTime_AC::kZero ),
    fWaiters(NULL)
#if qWantsMP
	,fUsingQueue(::UsingMP_AC()),
	fQueue(NullMPQueueID)
#endif
{
#if qWantsMP
	if (fUsingQueue)
	{
		::MPCreateQueue(&fQueue);
	}
	else
#endif
	{
    	fWaiters = TH_new deque<CAbstractThreadContext_AC*, allocator<CAbstractThreadContext_AC*> >();
	}
}

//----------------------------------------------------------------------------------------
// Constructor with monitor lock and default delay
//----------------------------------------------------------------------------------------

CMonitorCondition_AC::CMonitorCondition_AC ( CAbstractMonitorLock_AC* lock, const CTime_AC& defaultDelay ):
    fLock( lock ),
    fDefaultDelay( CTime_AC::kZero ),
    fWaiters(NULL)
#if qWantsMP
	,fUsingQueue(::UsingMP_AC()),
	fQueue(NullMPQueueID)
#endif
{
#if qWantsMP
	if (fUsingQueue)
	{
		::MPCreateQueue(&fQueue);
	}
	else
#endif
	{
    	fWaiters = TH_new deque<CAbstractThreadContext_AC*, allocator<CAbstractThreadContext_AC*> >();
	}
	
    SetDefaultDelay( defaultDelay );
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CMonitorCondition_AC::~CMonitorCondition_AC()
{
	Broadcast();

#if qWantsMP
	if (fUsingQueue)
	{
		::MPDeleteQueue(fQueue);
	}
	else
#endif
	{
    	delete fWaiters;
	}
}

//----------------------------------------------------------------------------------------
// GetDefaultDelay
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::GetDefaultDelay( CTime_AC& delay ) const
{
    delay = fDefaultDelay;
}

//----------------------------------------------------------------------------------------
// SetDefaultDelay
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::SetDefaultDelay( const CTime_AC& delay )
{
    if ( delay <= CTime_AC::kZero )
    	fDefaultDelay = CTime_AC::kPositiveInfinity;
    else
    	fDefaultDelay = delay;
}

//----------------------------------------------------------------------------------------
// Wait
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::Wait()
{   
    if ( fDefaultDelay > CTime_AC::kZero)
        Wait( fDefaultDelay );
    else
    	Wait( CTime_AC::kPositiveInfinity );
}

//----------------------------------------------------------------------------------------
// Wait
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::Wait( const CTime_AC& /*maxDelay*/ )
{
	OSStatus result = 0;

	CAbstractThreadContext_AC* context = CAbstractThreadContext_AC::GetCurrentContext();
#if qWantsMP
	if (fUsingQueue)
	{
		::MPNotifyQueue(fQueue, context, NULL, NULL);
	}
	else
#endif
	{
		fWaiters->push_back(context);
	}

	fLock->Release();
	CAbstractThreadContext_AC::SuspendCurrentContext();	// cause the executing thread to be suspended	
	fLock->Acquire();
}

//----------------------------------------------------------------------------------------
// Notify
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::Notify()
{
#if qWantsMP
	if (fUsingQueue)
	{
		CAbstractThreadContext_AC* context = NULL;
		if (::MPWaitOnQueue(fQueue, &context, NULL, NULL, kDurationImmediate) == noErr) 
		{
			if (context) 
				context->Resume();
		}
	}
	else
#endif
	{
		if (!fWaiters->empty()) 
		{
			CAbstractThreadContext_AC* context = (*fWaiters->begin());
			if (context)
				context->Resume();
			fWaiters->pop_front();
		}
	}
}

//----------------------------------------------------------------------------------------
// Broadcast
//----------------------------------------------------------------------------------------

void CMonitorCondition_AC::Broadcast()
{
#if qWantsMP
	if (fUsingQueue)
	{
		CAbstractThreadContext_AC* context = NULL;
		while (::MPWaitOnQueue(fQueue, &context, NULL, NULL, kDurationImmediate) == noErr) 
		{
			if (context) 
				context->Resume();
		}
	}
	else
#endif
	{
		while (!fWaiters->empty()) 
		{
			CAbstractThreadContext_AC* context = (*fWaiters->begin());
			if (context)
				context->Resume();
			fWaiters->pop_front();
		}
	}
}

