//----------------------------------------------------------------------------------------
// CThreadRunner_AC.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Copyright (C) 1994 Taligent, Inc.
//----------------------------------------------------------------------------------------


#ifndef __CThreadRunner_AC__
#include "CThreadRunner_AC.h"
#endif


#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#ifndef __CAbstractThreadSchedule_AC__
#include "CAbstractThreadSchedule_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

#ifndef __CMonitorCondition_AC__
#include "CMonitorCondition_AC.h"
#endif

//========================================================================================
// Thread Prologue
//========================================================================================

struct ThreadPrologue_AC 
{
	CThread_AC* fThread;
	CAbstractMonitorLock_AC *fMonitor;
	CMonitorCondition_AC *fCondition;
};

//----------------------------------------------------------------------------------------

extern "C" { // to avoid name mangling match problems
	void	NewThreadPrologue(ThreadPrologue_AC *aPrologue);
}

//----------------------------------------------------------------------------------------

void NewThreadPrologue(ThreadPrologue_AC *aPrologue)
{
	CThread_AC* aThread = aPrologue->fThread;
	CThreadHandle_AC aHandle(CAbstractThreadContext_AC::GetCurrentContext());	// Increments current thread context ref count.  Decrements on destruction at exit of scope.
	CAbstractThreadSchedule_AC* aSchedule = aThread->GetThreadSchedule();
	bool gotException = false;

	{ // scope thread synchronization
		CWhileSynchronized_AC anEntry(aPrologue->fMonitor);
		try 
		{
			aThread->Prepare();
		}
		catch (...) 
		{
			gotException = true;
		}

		aPrologue->fCondition->Notify();
	}

	// Now don't refer to aPrologue object at all
	// Don't touch the condition after released

	// Call the Run method if we did not get an exception.
	if (!gotException)
	{
		try 
		{
			aThread->Run();
		}
		catch(CException_AC& inException)
		{
			DoCatchMessage_AC(inException);
		}
		catch (...) 
		{
#if qThreadDebug
			ProgramBreak_AC("Caught Exception in NewThreadPrologue");
#endif		
			// Don't let exceptions propogate beyond the top of the stack.
		}	
	}
		
	// Thread cleanup is handled by the reference counted context
}

//========================================================================================
//	CThreadHandle_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// Constructor:
//----------------------------------------------------------------------------------------

CThreadHandle_AC::CThreadHandle_AC(CAbstractThreadContext_AC* itsContext) :
	fContext(itsContext)
{
	if (fContext)
	{
		fContext->Acquire();
	}
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadHandle_AC::~CThreadHandle_AC()
{
	if (fContext)
	{
		fContext->Release();
	}
}


//========================================================================================
//	CThreadRunner_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// Constructor: Create a thread handle for the calling thread
//----------------------------------------------------------------------------------------

CThreadRunner_AC::CThreadRunner_AC(CThread_AC* aThread) :
	CThreadHandle_AC(NULL)
//	acquires aThread which is deleted upon termination of the thread.
//	The resources owned by the thread are also discarded.
{
	// Check the thread is valid
	Assertion_AC(aThread != NULL, "CThreadRunner_AC: Adopted CThread_AC* is NULL");
	
	CAbstractThreadSchedule_AC* schedule = aThread->GetThreadSchedule();
	CAbstractMonitorLock_AC* aMonitor = schedule->CreateMonitorLock();
	CMonitorCondition_AC* aCondition = TH_new CMonitorCondition_AC(aMonitor);
	
	// Create and initialize the prologue struct
	ThreadPrologue_AC aPrologue;
	aPrologue.fThread = aThread;
	aPrologue.fMonitor = aMonitor;
	aPrologue.fCondition = aCondition;
	
	// adoptThreadCode->Prepare(); runs in created thread not here -sandeep
	// start up the new thread, but don't let it run yet.
	
	{
		CWhileSynchronized_AC anEntry(aPrologue.fMonitor);
		if (schedule != NULL) 
			fContext = schedule->CreateThreadContext( 
				aThread->GetStackSize(), 
				(SimpleTaskProcPtr)NewThreadPrologue, 
				(void*)&aPrologue,
				aPrologue.fThread);
			// Context is created with ref count of one, representing its creator, this handle.

		// Wait for the new thread to Notify that he has started
		aPrologue.fCondition->Wait();
	}
	delete aCondition;
	delete aMonitor;
}

//----------------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------------

CThreadRunner_AC::~CThreadRunner_AC()
{
	Join();
}


//----------------------------------------------------------------------------------------
//	copy constructor
//----------------------------------------------------------------------------------------

CThreadRunner_AC::CThreadRunner_AC(const CThreadRunner_AC& aThread) :
	CThreadHandle_AC(aThread.fContext)
{
}


//----------------------------------------------------------------------------------------
//	assignment
//----------------------------------------------------------------------------------------

CThreadRunner_AC& CThreadRunner_AC::operator=(const CThreadRunner_AC& aThread)
{
	if (this != &aThread)
	{
		if (fContext) 
			fContext->Release();
		fContext = aThread.fContext;
		if (fContext) 
			fContext->Acquire();
	}
	
	return *this;
}

//----------------------------------------------------------------------------------------
// Comparison operator for comparing CThreadRunners
//----------------------------------------------------------------------------------------

bool CThreadRunner_AC::operator==(const CThreadRunner_AC& toWhat) const
{
	return (fContext == toWhat.fContext);
}


//----------------------------------------------------------------------------------------
// Comparison operator for comparing CThreadRunners
//----------------------------------------------------------------------------------------

bool CThreadRunner_AC::operator!=(const CThreadRunner_AC& toWhat) const
{
	return !(fContext == toWhat.fContext);
}



//----------------------------------------------------------------------------------------
// return true if the thread handle represents the current thread.
//----------------------------------------------------------------------------------------

bool CThreadRunner_AC::IsSelf() const
{
	return (fContext == CAbstractThreadContext_AC::GetCurrentContext());
}


//----------------------------------------------------------------------------------------
// determine if the thread still exists
//----------------------------------------------------------------------------------------

bool CThreadRunner_AC::Exists() const
{
	return ((fContext != NULL) && fContext->IsAlive());
}

//----------------------------------------------------------------------------------------
// Request Shutdown
//----------------------------------------------------------------------------------------

void CThreadRunner_AC::RequestShutdown()
{
	if (fContext) 
	{
		CThread_AC* thread = fContext->GetThread();
		if (thread) 
		{
			thread->RequestShutdown();
		}
	}
}
		
//----------------------------------------------------------------------------------------
// Resume
//----------------------------------------------------------------------------------------

void CThreadRunner_AC::Resume()
{
	if (fContext)
	{
		fContext->Resume();
	}
}

//----------------------------------------------------------------------------------------
//	Instruct OS to terminate the thread. The call is very low-level. This
//	is not the correct method for general use as it does not allow any user
//	clean-up code to run. The client should use an ad hoc protocol to
//	provide safe termination.
//----------------------------------------------------------------------------------------

void CThreadRunner_AC::Kill()
{
	// not implemented yet
}

//----------------------------------------------------------------------------------------
// CThreadRunner_AC::CancelWaitAndPostException: 
// causes a blocking call to be unblocked and an exception to be
// thrown in the thread (*this).
//----------------------------------------------------------------------------------------

void CThreadRunner_AC::CancelWaitAndPostException() const
{
	// not implemented yet.
}

//----------------------------------------------------------------------------------------
// CThreadRunner_AC::Join: 
// Calls RequestShutdown and blocks the calling thread until the
// thread terminates.
//----------------------------------------------------------------------------------------

void CThreadRunner_AC::Join()
{
	RequestShutdown();
	if (!IsSelf())
	{
		while (Exists())
	 	{
			fContext->Yield();
		}
	}
}
