//----------------------------------------------------------------------------------------
// CLinkedList_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CLinkedList_AC__
#define __CLinkedList_AC__

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

// This class is meant to do the following:
//		1. Provide support for reasonably safe inserts/deletes while iterating
//		2. Iterate forward only
//		3. Good performance.
//		4. Typesafe interface
//		5. Be simple - not meant to be general purpose container.

template <class T>
class CLinkedList_AC
{
public:

	struct SLink;

	// Eventually, we'll break the implementation of CIterator out of the header.
	// We can't get the compiler to compile it outside of the linked list
	// header. Also can't break it out into a seperate file because the iterator
	// and list have intimate knowledge of each other which would cause header file
	// chicken and egg probs. sorry.

	class EXPORT_AC CIterator
	{
	private:
		CIterator(CLinkedList_AC<T> *inList, SLink* startingLink) 
			: fList(inList), fCurrentLink(startingLink)
		{	
			::AssertNotNULL_AC(fList); 
			fList->AddIterator();  
			if(fCurrentLink && fCurrentLink->fIsMarked)
				Next();
		}

	public:
		CIterator(const CIterator& inIter)
			: fList(inIter.fList), fCurrentLink(inIter.fCurrentLink)
		{	if(fList) fList->AddIterator(); }	

		CIterator()
			: fList(NULL), fCurrentLink(NULL)
		{}	

		~CIterator()
		{	if(fList) fList->RemoveIterator(); }

		CIterator& operator = (const CIterator& inIter)
		{	
			if(&inIter != this)
			{
				if(fList) fList->RemoveIterator();
				fCurrentLink = inIter.fCurrentLink; 
				fList = inIter.fList; 
				if(fList)  fList->AddIterator();
			}
			return *this; 
		}

		void First()
		{
			::AssertNotNULL_AC(fList);
			fCurrentLink = fList->fHead; 
			if(fCurrentLink && fCurrentLink->fIsMarked)
				Next();
		}
		
		void Last()
		{
			::AssertNotNULL_AC(fList);
			fCurrentLink = fList->fTail; 
			if(fCurrentLink && fCurrentLink->fIsMarked)
				Prev();
		}
		
		bool IsValid()
		{	
			
			if( (fCurrentLink != NULL) && (fCurrentLink->fIsMarked == true))
				Next();	

			return	fCurrentLink != NULL; 
		}

		T& Current() const 
		{ 	
			::Assertion_AC(	fCurrentLink != NULL && fCurrentLink->fIsMarked == false,
							"Current item is invalid.");
			::AssertNotNULL_AC(fCurrentLink); 
			return fCurrentLink->fData; 
		}
		
		void Next()
		{
			::AssertNotNULL_AC(fList);
			if(fCurrentLink)
			{
				do
				{
					fCurrentLink = fCurrentLink->fNext;
				}
				while( (fCurrentLink != NULL) && (fCurrentLink->fIsMarked == true));
			}
		}
		void Prev()
		{
			::AssertNotNULL_AC(fList);
			if(fCurrentLink)
			{
				do
				{
					fCurrentLink = fCurrentLink->fPrev;
				}
				while( (fCurrentLink != NULL) && (fCurrentLink->fIsMarked == true));
			}
		}

		inline void operator++() 
		{ Next(); }
		// PreIncrement

		inline void operator++(int) 
		{ Next(); }
		// PostIncrement

		inline void operator--() 
		{ Prev(); }
		// PreDecrement

		inline void operator--(int) 
		{ Prev(); }
		// PostDecrement

		inline operator T&() const 
		{ return Current(); }
		// Coerce to current element
	
		T& operator *() const 
		{ return Current();	}
			
	private:
		SLink* fCurrentLink;
		CLinkedList_AC<T>* fList;
		
		friend class CLinkedList_AC<T>;
	};
	
	CLinkedList_AC();

	virtual ~CLinkedList_AC();

// for peeking at data	
	T& GetFirstElement() const;

	T& GetLastElement() const;

// inserting	
	void Push(const T& inElement);
		
	void Append(const T& inElement);

	void InsertAfter(CIterator& inAfter, const T& inElement);

	void InsertBefore(CIterator& inBefore, const T& inElement);

// removing
	void Remove(const T& inElement);
		
	void Remove(CIterator& inIterator);
	
// misc info	
	bool InList(const T& inElement);
	
	long GetLength() const
	{	return fCount; }
		
	bool IsEmpty() const
	{	return fCount == 0; }	
		

// searching
	CIterator Find(const T& inElement);
		// this will return a NULL iterator if item isn't found.

// iterating
	CIterator First();
	
	CIterator Last();
	
// misc utils	
	void AppendElementsFrom (CLinkedList_AC<T>* inList);
	
	void Delete();
		// delete all the links in the list

// Implementation.
		
protected:

	typedef struct SLink
	{
		SLink(const T& inData) 
			: fIsMarked(false), fNext(NULL), fPrev(NULL), fData(inData) {}
		T fData;
		bool fIsMarked;
		SLink* fNext;
		SLink* fPrev;
	} SLink;

	void RemoveLink(SLink*& toRemove);
	
	void AddIterator();

	void RemoveIterator();
	
	void RemoveMarkedLinks();
	
	SLink* FindLink(const T& inElement);
		
protected:
	SLink* 	fHead;
	SLink*	fTail;
	
	long 	fIteratorCount;	
	long	fCount;
	
	friend class CIterator;
};

// implementation

// inlines


template <class T>
void CLinkedList_AC<T>::RemoveLink(SLink*& toRemove)
{
	// if prev == NULL this means toRemove is fHead

	::AssertNotNULL_AC(toRemove);

	if(fIteratorCount > 0)
	{
		toRemove->fIsMarked = true;
	}
	else
	{
		if(toRemove->fPrev) 
			toRemove->fPrev->fNext = toRemove->fNext;
		
		if(toRemove->fNext)
			toRemove->fNext->fPrev = toRemove->fPrev;
		
		if(toRemove == fHead)
			fHead = fHead->fNext;
			
		if(toRemove == fTail)
			fTail = fTail->fPrev;	
	
		fCount--;	
			
		delete toRemove;
		toRemove = NULL;
	}	
	
}
	
template <class T>
inline T& CLinkedList_AC<T>::GetFirstElement() const
{
	::AssertNotNULL_AC(fHead);
	return fHead->fData;
}
template <class T>
inline T& CLinkedList_AC<T>::GetLastElement() const
{
	::AssertNotNULL_AC(fHead);
	return fTail->fData;
}	
template <class T>
inline void CLinkedList_AC<T>::AddIterator()
{	
	++fIteratorCount; 
}

template <class T>
inline void CLinkedList_AC<T>::RemoveIterator()
{
	--fIteratorCount;
	
	if(fIteratorCount == 0)
		RemoveMarkedLinks();
}

template <class T>
inline void CLinkedList_AC<T>::Remove(CIterator& inIter)
{
	RemoveLink(inIter.fCurrentLink);
}
template <class T>
inline CLinkedList_AC<T>::CIterator CLinkedList_AC<T>::First()
{
	return CIterator(this, fHead);
}
template <class T>
inline CLinkedList_AC<T>::CIterator CLinkedList_AC<T>::Last()
{
	return CIterator(this, fTail);
}

// non-inlines

template <class T>
CLinkedList_AC<T>::CLinkedList_AC()
	: fHead(NULL),
	fTail(NULL),
	fIteratorCount(0),
	fCount(0)
{
}
template <class T>
CLinkedList_AC<T>::~CLinkedList_AC()
{	
	::Assertion_AC(fIteratorCount == 0, "Can't delete list with outstanding iterators!");

	Delete(); 
}
template <class T>
void CLinkedList_AC<T>::Push(const T& inElement)
{
	SLink* newLink = TH_new SLink(inElement);
	
	if(fHead) fHead->fPrev = newLink;
	newLink->fNext = fHead;
	fHead = newLink;

	if(!fTail) fTail = newLink;

	++fCount;
}
template <class T>
void CLinkedList_AC<T>::Append(const T& inElement)
{
	SLink* newLink = TH_new SLink(inElement);

	newLink->fPrev = fTail;
	if(fTail) fTail->fNext = newLink;
	fTail = newLink;

	if(!fHead) fHead = newLink;

	++fCount;
}

template <class T>
inline void CLinkedList_AC<T>::InsertAfter(CIterator& inAfter, const T& inElement)
{
	::AssertNotNULL_AC(inAfter.fCurrentLink);

	SLink* newLink = TH_new SLink(inElement);
	
	if(inAfter.fCurrentLink->fNext) 
		inAfter.fCurrentLink->fNext->fPrev = newLink;
		
	newLink->fPrev = inAfter.fCurrentLink;
	newLink->fNext = inAfter.fCurrentLink->fNext;
	inAfter.fCurrentLink->fNext = newLink;
	
	if(inAfter.fCurrentLink == fTail)
		fTail = newLink;
}

template <class T>
void CLinkedList_AC<T>::InsertBefore(CIterator& inBefore, const T& inElement)
{
	::AssertNotNULL_AC(inBefore.fCurrentLink);

	SLink* newLink = TH_new SLink(inElement);
	
	if(inBefore.fCurrentLink->fPrev) 
		inBefore.fCurrentLink->fPrev->fNext = newLink;
		
	newLink->fNext = inBefore.fCurrentLink;
	newLink->fPrev = inBefore.fCurrentLink->fPrev;
	inBefore.fCurrentLink->fPrev = newLink;
	
	if(inBefore.fCurrentLink == fHead)
		fHead = newLink;
}

template <class T>
bool CLinkedList_AC<T>::InList(const T& inElement)
{
	bool out = false;
	SLink* linkWalker = fHead;
	
	while(linkWalker)
	{
		if(linkWalker->fData == inElement)
		{
			out = true;
			break;
		}
		
		linkWalker = linkWalker->fNext;
	}
	
	return out;
}


template <class T>
void CLinkedList_AC<T>::Remove(const T& inElement)
{
	SLink* linkWalker = fHead;
	SLink* deleter = NULL;
	
	while(linkWalker)
	{	
		deleter = linkWalker;
		linkWalker = linkWalker->fNext;
	
		if(deleter->fData == inElement)
			RemoveLink(deleter);
	}
}


template <class T>
void CLinkedList_AC<T>::Delete()
{
	SLink* linkWalker = fHead;
	SLink* deleter = NULL;
	
	while(linkWalker)
	{	
		deleter = linkWalker;
		linkWalker = linkWalker->fNext;
	
		RemoveLink(deleter);
	}
}

template <class T>
CLinkedList_AC<T>::SLink* CLinkedList_AC<T>::FindLink(const T& inElement)
{
	bool found = false;
	SLink* linkWalker = fHead;
	
	while(linkWalker)
	{
		if(linkWalker->fData = inElement)
		{
			found = true;
			break;
		}
		linkWalker = linkWalker->fNext;
	}
	
	return found? linkWalker : NULL;
}

template <class T>
CLinkedList_AC<T>::CIterator CLinkedList_AC<T>::Find(const T& inData)
{
	CLinkedList_AC<T>::SLink* link = FindLink(inData);
	
	return link != NULL ? CIterator(this, link) : CIterator();	
}

template <class T>
void CLinkedList_AC<T>::AppendElementsFrom (CLinkedList_AC<T>* inList)
{
	CIterator listWalker = inList->Begin();
		
	while(listWalker.HasMore())
	{
		Append( listWalker ); 
		listWalker.Next();
	}
}


template <class T>
void CLinkedList_AC<T>::RemoveMarkedLinks()
{
	SLink* linkWalker = fHead;
	SLink* deleter = NULL;
	
	while(linkWalker)
	{
		deleter = linkWalker;
		linkWalker = linkWalker->fNext;
	
		if(deleter->fIsMarked)
			RemoveLink(deleter);
	}
}

#endif