// UList.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __CList_AC__
#define __CList_AC__

// ACS

#ifndef __CDynamicArray_AC__
#include "CDynamicArray_AC.h"
#endif

//----------------------------------------------------------------------------------------
// Unit UList defines object types CList_AC, a simple dynamic array whose elements are
// references to TObjects, and CSortedList_AC, which maintains the object references in a
// sequence defined by overriding CSortedList_AC::Compare.
//
// Peter Gaston's scheme for chunky memory allocation has been used by permission. Thanks
// Peter!.
//
// CList_AC: CList_AC is an object which provides a very nice dynamic array of objects. It is
// implemented by appending the dynamic array onto the end of the object.Growing and
// shrinking the array is done by growing and shrinking the handle which holds the object.
//
// CSortedList_AC: CSortedList_AC maintains the object references in a sequence defined by
// overriding CSortedList_AC.Compare
//
//  Some Specific uses that CList_AC is especially good for.
//  LIFO Stack: A push/pop operation is provided.
//  FIFO Stack: (Queue) Use InsertLast for Queue and First for Dequeue.
// Pre-allocation of your list: If you know the size that your list will eventually reach,
// then you can pre-set it to the proper size.
//
//  Delete by Index:
// If you keep track of items by index number, then you can avoid a linear search when you
// want to delete it. Remember though, it is a dynamic list and any insertion/ deletion
// activity can cause the index to CPoint_AC at a different element than you expect.
//
//  Caveats:
// A _small_ amount of memory is always unused with this method.An active list will
// generally waste one-half of the chunk size, or 4*2=8 bytes/CList_AC.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// some helpful compatibility goop for T[Sorted]VoidList
//----------------------------------------------------------------------------------------
class CSortedList_AC;
class CList_AC;
typedef CSortedList_AC CSortedVoidList_AC;
typedef CList_AC CVoidList_AC;

//----------------------------------------------------------------------------------------
// procedure typedefs for parameters
//----------------------------------------------------------------------------------------
typedef CompareResult_AC (*CompareVoidType)(void* anItem, void* yourDataPtr);

typedef CompareResult_AC (*CompareVoidsType)(void* item1,
											void* item2,
											void* yourDataPtr);

typedef CompareResult_AC (*CompareHandleType)(Handle anItem, void* yourDataPtr);

typedef CompareResult_AC (*CompareLongType)(long anItem, void* yourDataPtr);


//----------------------------------------------------------------------------------------
// CSortedList_AC: A dynamic list of void*'s. It IS permissible for descendants to
// add more named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CSortedList_AC : public CSortedDynamicArray_AC
{
public:
	inline CSortedList_AC(ArrayIndex_AC initialSize = 0) : CSortedDynamicArray_AC(initialSize, sizeof(void*)) { }
		// constructor

	virtual ~CSortedList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual void Insert(const void* item);

#if qRangeCheck || qDebugMsg
	void* At(ArrayIndex_AC index) const;
#else
	inline void* At(ArrayIndex_AC index) const { return *((void**)ComputeAddress(index)); }
#endif
		// Return the index'th element of the list.

#if qRangeCheck || qDebugMsg
	void AtDelete(ArrayIndex_AC index);
#else
	inline void AtDelete(ArrayIndex_AC index) { DeleteElementsAt(index, 1); }
#endif
		// Deletes via an index.

	virtual void Delete(void* item);

	void* First() const;
		// Return the first element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	void* Last() const;
		// Return the last element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	virtual ArrayIndex_AC GetIdentityItemNo(const void* item) const;
		// Find the first reference to the IDENTITY item in the list. If item does not
        // occur, return 0.

	virtual ArrayIndex_AC GetEqualItemNo(const void* item) const;
		// Find the first reference to item in the list. If item does not occur, return 0.

	virtual CompareResult_AC Compare(const void* item1,
										  const void* item2) const;
		// By default just compare the ordinal value of the items. Subclasses that want to
        // use the items as pointers or other types should override if the comparison is
        // based on the pointed at value

	virtual CompareResult_AC CompareElements(const void* element1,
												  const void* element2) const;
		// Forward into the Compare function.

	virtual void* DoSearch(CompareVoidType TestItem,
									 void* yourDataPtr,
									 ArrayIndex_AC& index) const;
		// DON'T use EXIT to get out of this routine from your TestItem function or you
        // will be really sad!(our debugger will check for you) That's why you can return
        // true to stop enumerating. Signaling Failure is OK too.

	virtual ArrayIndex_AC Search(CompareVoidType TestItem, void* yourDataPtr);

	virtual void SortBy(CompareVoidsType CompareItems, void* yourDataPtr);
		// Sorts the list using the supplied CompareItems function.
};

//----------------------------------------------------------------------------------------
// CList_AC: A dynamic list of void*'s. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CList_AC : public CSortedList_AC
{
public:
	inline CList_AC(ArrayIndex_AC initialSize = 0) : CSortedList_AC(initialSize) { }
		// constructor

	virtual ~CList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual bool DoSearchElement(CompareIndexType_AC TestItem,
										   void* yourDataPtr,
										   ArrayIndex_AC& index) const; 
		// Overridden to do a linear search

	virtual void InsertElementInOrder(const void* ElementPtr); 
		// Overridden to do arrival sequence insertion

#if qRangeCheck || qDebugMsg
	void AtPut(ArrayIndex_AC index, void* newItem);
#else
	inline void AtPut(ArrayIndex_AC index, void* newItem) { *((void**)ComputeAddress(index)) = newItem; }
#endif
		// Replace the index'th element of the list. Range check only if the compile-flag
        // qRangeCheck is true.

#if qRangeCheck || qDebugMsg
	void InsertBefore(ArrayIndex_AC index, void* item);
#else
	inline void InsertBefore(ArrayIndex_AC index, void* item) { InsertElementsBefore(index, &item, 1); }
#endif
		// Insert a reference to item at the indicated index. If the compile-flag qDebug
        // is true & SetEltType was called, verify item's type.

	inline void InsertFirst(void* item) { InsertBefore(1, item); }
		// Insert a reference to item at the front of the list. If the compile-flag qDebug
        // is true & SetEltType was called, verify item's type.

	void InsertLast(void* item);
		// Insert a reference to item at the back of the list. If qDebug is true &
        // SetEltType was called, check that item's type is correct.

	inline void Push(void* item) { InsertLast(item); }
		// LIFO stack push.

	void* Pop();
		// LIFO stack pop
};

//----------------------------------------------------------------------------------------
// CSortedHandleList_AC: A dynamic list of Handles. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CSortedHandleList_AC : public CSortedDynamicArray_AC
{
public:
	inline CSortedHandleList_AC(ArrayIndex_AC initialSize = 0) : CSortedDynamicArray_AC(initialSize, sizeof(Handle)) { }
		// Constructor

	virtual ~CSortedHandleList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual void FreeAll();
		// Frees each element in the list. Leave fSize at 0.

	void FreeList();
		// Frees each handle in the list, then frees the list.
		

#if qRangeCheck || qDebugMsg
	Handle At(ArrayIndex_AC index) const;
#else
	inline Handle At(ArrayIndex_AC index) const { return *static_cast<Handle*>(ComputeAddress(index)); }
#endif
		// Return the index'th element of the list.

#if qRangeCheck || qDebugMsg
	void AtDelete(ArrayIndex_AC index);
#else
	inline void AtDelete(ArrayIndex_AC index) { DeleteElementsAt(index, 1); }
#endif
		// Deletes via an index.

	void Delete(Handle item);

	Handle First() const;
		// Return the first element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	Handle Last() const;
		// Return the last element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	virtual ArrayIndex_AC GetIdentityItemNo(const Handle item) const;
		// Find the first reference to the IDENTITY item in the list. If item does not
        // occur, return 0.

	virtual ArrayIndex_AC GetEqualItemNo(const Handle item) const;
		// Find the first reference to item in the list. If item does not occur, return 0.

	virtual void Insert(const Handle item);
		// Inserts the given item into the list in sorted order, using the Compare method
        // to determine the item's location relative to other items in the list.

	virtual Handle DoSearch(CompareHandleType TestItem,
								   void* yourDataPtr,
								   ArrayIndex_AC& index) const;

	virtual CompareResult_AC Compare(const Handle item1,
										 const Handle item2) const;
		// By default just compare the ordinal value of the items. Subclasses that want to
        // use the items as pointers or other types should override if the comparison is
        // based on the pointed at value

	virtual CompareResult_AC CompareElements(const void* element1,
												 const void* element2) const;
		// Forward into the Compare function.
};

//----------------------------------------------------------------------------------------
// CHandleList_AC: A dynamic list of Handles. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CHandleList_AC : public CSortedHandleList_AC
{
public:

	inline CHandleList_AC(ArrayIndex_AC initialSize = 0) : CSortedHandleList_AC(initialSize) { }
		// constructor

	virtual ~CHandleList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual void InsertElementInOrder(const void* ElementPtr);
		// Overridden to do arrival sequence insertion
};

//----------------------------------------------------------------------------------------
// CSortedLongintList_AC: A dynamic list of Longints. It IS permissible for descendants to
// add more named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CSortedLongintList_AC : public CSortedDynamicArray_AC
{
public:
	inline CSortedLongintList_AC(ArrayIndex_AC initialSize = 0) : CSortedDynamicArray_AC(initialSize, sizeof(long)) { }
		// constructor

	virtual ~CSortedLongintList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual void Insert(long item);

#if qRangeCheck || qDebugMsg
	long At(ArrayIndex_AC index) const;
#else
	inline long At(ArrayIndex_AC index) const { return *static_cast<long*>(ComputeAddress(index)); }
#endif
		// Return the index'th element of the list.

#if qRangeCheck || qDebugMsg
	void AtDelete(ArrayIndex_AC index);
#else
	inline void AtDelete(ArrayIndex_AC index) { DeleteElementsAt(index, 1); }
#endif
		// Deletes via an index.

	virtual void Delete(long item);

	long First() const;
		// Return the first element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	long Last() const;
		// Return the last element of the list. If the compile-flag qDebug is true &
        // SetEltType was called, verify item's type.

	virtual ArrayIndex_AC GetIdentityItemNo(long item) const;
		// Find the first reference to the IDENTITY item in the list. If item does not
        // occur, return 0.

	virtual ArrayIndex_AC GetEqualItemNo(long item) const;
		// Find the first reference to item in the list. If item does not occur, return 0.

	virtual CompareResult_AC Compare(long item1,
										  long item2) const;
		// By default just compare the ordinal value of the items. Subclasses that want to
        // use the items as pointers or other types should override if the comparison is
        // based on the pointed at value

	virtual CompareResult_AC CompareElements(const void* element1,
												  const void* element2) const;
		// Forward into the Compare function.

	virtual bool DoSearch(CompareLongType TestItem,
									 void* yourDataPtr,
									 ArrayIndex_AC& index) const;
		// DON'T use EXIT to get out of this routine from your TestItem function or you
        // will be really sad!(our debugger will check for you) That's why you can return
        // true to stop enumerating. Signaling Failure is OK too.

	virtual ArrayIndex_AC Search(CompareLongType TestItem, void* yourDataPtr);
};

//----------------------------------------------------------------------------------------
// CLongintList_AC: A dynamic list of Longints. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------
class EXPORT_AC CLongintList_AC : public CSortedLongintList_AC
{
public:
	inline CLongintList_AC(ArrayIndex_AC initialSize = 0) : CSortedLongintList_AC(initialSize) { }
		// constructor

	virtual ~CLongintList_AC();
		// Destructor
		
	virtual void* Clone() const;

	virtual bool DoSearchElement(CompareIndexType_AC TestItem,
										   void* yourDataPtr,
										   ArrayIndex_AC& index) const; 
		// Overridden to do a linear search

	virtual void InsertElementInOrder(const void* ElementPtr); 
		// Overridden to do arrival sequence insertion

#if qRangeCheck || qDebugMsg
	void AtPut(ArrayIndex_AC index, long newItem);
#else
	inline void AtPut(ArrayIndex_AC index, long newItem) { *static_cast<long*>(ComputeAddress(index)) = newItem; }
#endif
		// Replace the index'th element of the list. Range check only if the compile-flag
        // qRangeCheck is true.

#if qRangeCheck || qDebugMsg
	void InsertBefore(ArrayIndex_AC index, long item);
#else
	inline void InsertBefore(ArrayIndex_AC index, long item) { InsertElementsBefore(index, &item, 1); }
#endif
		// Insert a reference to item at the indicated index. If the compile-flag qDebug
        // is true & SetEltType was called, verify item's type.

	inline void InsertFirst(long item) { InsertBefore(1, item); }
		// Insert a reference to item at the front of the list. If the compile-flag qDebug
        // is true & SetEltType was called, verify item's type.

	void InsertLast(long item);
		// Insert a reference to item at the back of the list. If qDebug is true &
        // SetEltType was called, check that item's type is correct.

	inline void Push(long item) { InsertLast(item); }
		// LIFO stack push.

	long Pop();
		// LIFO stack pop
};

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------
EXPORT_AC  CList_AC* NewList_AC();
	// A convenience function. Create a CList_AC, initialize it, and return a reference to
    // it. Signals Failure if it cannot be created.

EXPORT_AC  CSortedList_AC* NewSortedList_AC();
	// A convenience function. Create a CSortedList_AC, initialize it, and return a reference
    // to it. Signals Failure if it cannot allocate the object.

EXPORT_AC  CList_AC* NewAllocatedList_AC(ArrayIndex_AC iSize);
	// A convenience function. The same as NewList_AC, but the initial allocation size can be
    // set.

#endif


