//----------------------------------------------------------------------------------------
// CMap_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CMap_AC__
#define __CMap_AC__

#if 0
#undef Allocator
#define Allocator CTidyHeapAllocator_AC
#include "CTidyHeapAllocator_AC.h"
#endif

// #include <allocator.h>

#if qMSWindows
	#include <map>
	#include <algorithm>
#else
	#ifndef MSIPL_MAP_H
	#include <map.h>
	#endif
	
	#ifndef ALGO_H
	#include <algorithm.h>
	#endif
#endif

/*
	This class provides _reasonably_ safe insertions deletions while iterating the multimap provided
	you use it _responsibly_.
	
	rules of thumb:
	
		To begin iterations use begin_iterating instead of begin 
	 	To end iterations call end_iterating
	 	call safe_erase or safe_insert instead of insert or erase
	
	implementation notes:
	
		deletions mananged by a simple mark and sweep algorithm
		Note: it is _okay_ to delete data the node in the multimap holds once it is marked
		for deletion. I.E. calling delay_erase then deleting the data is fine (one of 
		the design criterion) even if the node in the multimap is deleted long after the
		user data is deleted.
		insertions are queued while iteration is in progress
	
	todo:
	
		Allocators for insertion queue?
		better way of marking data
*/


template <class KEY, class TYPE, class COMPARE>
class EXPORT_AC CMap_AC : public multimap<KEY,TYPE,COMPARE, allocator<TYPE> >
{
public:

#if 0	
	class Marker
	{
	public:
		Marker(KEY) : fMarked(0), fKey(0) {}
		operator KEY() { return fKey; }
		void Mark() { fMarked = true; }
		bool IsMarked() const { return fMarked; }
	
	private:
		KEY fKey;
		bool fMarked;
	};
#endif

	typedef multimap<KEY ,TYPE, COMPARE, allocator<TYPE> > map_type;
	typedef KEY 	Key;
	typedef TYPE 	Type;

public:
	// ctors
	CMap_AC(const COMPARE& comp = COMPARE());

	CMap_AC(const value_type* first, const value_type* last, const COMPARE& comp = COMPARE());

    CMap_AC(const map_type& x);

   	CMap_AC(const CMap_AC<KEY,TYPE,COMPARE>& x);

	virtual ~CMap_AC();
    
  	// assignment operators 
  	CMap_AC<KEY, TYPE, COMPARE>& operator=(const CMap_AC<KEY, TYPE, COMPARE>& x);
  
  	map_type& operator=(const multimap<KEY, TYPE, COMPARE, allocator<TYPE> >& x);

	// inserters/deleters

	void safe_erase(const value_type& inValue);
		// 	this will erase immediately or delay erase depending on whether list is
		// 	being iterated.

	void safe_erase(iterator);

	void delay_erase( const value_type& inValue);
		// 	delay erase until handle_delayed_deletions is called (called by 
		//	handle_delayed_operations which is called by stop_iterating
		
	void safe_insert(const value_type& inValue);
		// 	this will insert immediately or delay insert depending on whether list is
		// 	being iterated.
	
	void delay_insert(const value_type& x);
		// 	delay inserts until handle_delayed_insertions is called (called by 
		//	handle_delayed_insertions which is called by stop_iterating
	
	bool is_marked( const TYPE& inT ) const;
		// 	is this data marked for deletion?
	
	bool delete_if_marked(iterator& in);
		// 	if the node the iterator is pointing at is marked, remove and delete
		// 	it from the multimap

	void handle_delayed_deletions();
		//	this is the sweep. Search tree for marked nodes and then delete them.

	long get_marked_count() const;
		// 	how many nodes are marked for deletion?
		
	void handle_delayed_insertions();
		//	empty insertion queue into the multimap

	void handle_delayed_operations();
		// 	calls handle_delayed_deletions and handle_delayed_insertions

	iterator begin_iterating();
		//	begin iterating. Nested interations are supported. 
	
	void stop_iterating();
		// stop iterating: DON'TYPE FORGET TO CALL THIS!
	
	bool is_iterating() const;
	
	bool value_in_map(const value_type& x);

	iterator find(const value_type& x);

protected:
	class Inserter
	{
	public:
		Inserter(const value_type& inValue) : fValue(inValue) {}
		Inserter* fNext;
		value_type fValue;
	};
	
	void DeleteInserters();
	
	enum { kMarkedForDelete = 0 } ;

private:
	long fMarkCount;
	Inserter *fTop;	
	long fIterating;
};

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>::CMap_AC(const COMPARE& comp) 
: 	fMarkCount(0), 
	map_type(comp), 
	fIterating(0), 
	fTop(NULL) { }

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>
::CMap_AC(const value_type* first, const value_type* last, const COMPARE& comp) 
:	fMarkCount(0), 
	map_type(first,last,comp), 
	fIterating(0), 
	fTop(NULL) { }
	
template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>::CMap_AC(const map_type& x) 
:	fMarkCount(0), 
	map_type(x), 
	fIterating(0), 
	fTop(NULL) { }

// assignment operators 

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::delay_erase( const value_type& inValue)
{	
	if(!empty())
	{
#if qMSWindows
		std::replace((iterator)begin(), end(), inValue, 
					 value_type(inValue.first, reinterpret_cast<TYPE>(kMarkedForDelete)));
#else
		::replace( begin(), end(), inValue, value_type( inValue.first, static_cast<TYPE>(kMarkedForDelete )) );
#endif
		++fMarkCount;
	}
}

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::delay_insert(const value_type& x)
{ 	
	Inserter* inserter = TH_new Inserter(x); 
	inserter->fNext = fTop; 
	fTop = inserter; 
}

template <class KEY, class TYPE, class COMPARE>
inline bool CMap_AC<KEY,TYPE,COMPARE>::is_marked( const TYPE& inT ) const 
{ 
	return inT == static_cast<TYPE>(kMarkedForDelete ); 
}

template <class KEY, class TYPE, class COMPARE>
inline bool CMap_AC<KEY,TYPE,COMPARE>::delete_if_marked(iterator& in)
{ 
	bool is = is_marked( (*in).second ); 
	if(is) 
	{ 
		erase(in); 
		--fMarkCount; 
	} 
	return is;
}

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::handle_delayed_deletions()
{
	if(fMarkCount && !empty())
	{
		iterator iter = begin();
		iterator iter_end = end();
		iterator deleter;
		
		while(iter != iter_end)
		{
			deleter = iter;
			++iter;
			
			if( is_marked((*deleter).second))
			{
				erase(deleter);
				--fMarkCount;	
			}
		
			if(!fMarkCount)
				break;
		}
	}
}

template <class KEY, class TYPE, class COMPARE>
inline long CMap_AC<KEY,TYPE,COMPARE>::get_marked_count() const 
{ 
	return fMarkCount; 
}
	
template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::handle_delayed_insertions()
{	
	Inserter* old = NULL;
	while(fTop)
	{
		insert(fTop->fValue);
		old = fTop;
		fTop = fTop->fNext;
		delete old;
	}
}		

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::handle_delayed_operations()
{	
	handle_delayed_deletions();
	handle_delayed_insertions();
}

template <class KEY, class TYPE, class COMPARE>
inline multimap<KEY,TYPE, COMPARE, allocator<TYPE> >::iterator CMap_AC<KEY,TYPE,COMPARE>::begin_iterating() 
{ 
	++fIterating; 
	return begin();
}

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::stop_iterating() 
{ 
	--fIterating; 
	
	if(!is_iterating()) handle_delayed_operations();
}

template <class KEY, class TYPE, class COMPARE>
inline bool CMap_AC<KEY,TYPE,COMPARE>:: is_iterating() const 
{ 
	return fIterating > 0; 
}

template <class KEY, class TYPE, class COMPARE>
inline void CMap_AC<KEY,TYPE,COMPARE>::DeleteInserters()
{
	Inserter* old = NULL;
	while(fTop)
	{
		old = fTop;
		fTop = fTop->fNext;
		delete old;
	}
}

template <class KEY, class TYPE, class COMPARE>
CMap_AC<KEY,TYPE,COMPARE>::~CMap_AC()
{
	DeleteInserters();
}

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY, TYPE, COMPARE>& CMap_AC<KEY,TYPE,COMPARE>::operator=(const CMap_AC<KEY, TYPE, COMPARE>& x) 
{ 	
	map_type::operator=(x); 
		
	
	
	fMarkCount = x.fMarkCount; 
	DeleteInserters();
	return *this; 
}

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>::map_type& CMap_AC<KEY,TYPE,COMPARE>::operator=(const multimap<KEY, TYPE, COMPARE, allocator<TYPE> >& x) 
{
  	map_type::operator=(x); 
	fMarkCount = 0; 
	DeleteInserters();
	return *this; 
}

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>::CMap_AC(const CMap_AC<KEY,TYPE,COMPARE>& x) 
: 	fMarkCount(0),
	map_type(x), 
	fIterating(0), 
	fTop(NULL) 
{
	*this = x;
}

template <class KEY, class TYPE, class COMPARE>
void CMap_AC<KEY,TYPE,COMPARE>::safe_erase(const value_type& inValue)
{
	if(!empty())
	{
		if(is_iterating())
			delay_erase( inValue);
		else
		{
#if qMSWindows
			iterator iter = std::find(begin(),end(), inValue);
#else
			iterator iter = ::find(begin(),end(), inValue);
#endif
			erase(iter);
		}
	}
}

	
template <class KEY, class TYPE, class COMPARE>
void CMap_AC<KEY,TYPE,COMPARE>::safe_insert(const value_type& inValue)
{
	if(is_iterating())
		delay_insert(inValue);
	else
		insert(inValue);
}


template <class KEY, class TYPE, class COMPARE>
void CMap_AC<KEY,TYPE,COMPARE>::safe_erase(iterator inIter)
{
 	(*inIter).second = static_cast<TYPE>(kMarkedForDelete );
 	++fMarkCount;
}

template <class KEY, class TYPE, class COMPARE>
inline CMap_AC<KEY,TYPE,COMPARE>::iterator CMap_AC<KEY,TYPE,COMPARE>::find(const value_type& value)
{
	return ::find(begin(), end(), value);
}
	
template <class KEY, class TYPE, class COMPARE>
inline bool CMap_AC<KEY,TYPE,COMPARE>::value_in_map(const value_type& value)
{
	return (find(value) == end()) ? false : true;
}	

#endif