//----------------------------------------------------------------------------------------
// UListIterator.cp 
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

//========================================================================================
// CLASS CArrayIterator_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CArrayIterator_AC::CArrayIterator_AC: 
//----------------------------------------------------------------------------------------
CArrayIterator_AC::CArrayIterator_AC(const CDynamicArray_AC* itsDynamicArray,
				   ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
				   bool itsForward)
{
	IArrayIterator(itsDynamicArray, itsLowBound, itsHighBound, itsForward);

	First();
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::CArrayIterator_AC: 
//----------------------------------------------------------------------------------------
CArrayIterator_AC::CArrayIterator_AC(const CDynamicArray_AC* itsDynamicArray, bool itsForward)
{
	IArrayIterator(itsDynamicArray, 1, LONG_MAX, itsForward);

	First();
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::~CArrayIterator_AC: 
//----------------------------------------------------------------------------------------
CArrayIterator_AC::~CArrayIterator_AC()
{
	if (fDynamicArray)
		const_cast<CDynamicArray_AC*>(fDynamicArray)->fIteratorPtr = fNextLink;
	fNextLink = NULL;
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::IArrayIterator: 
//----------------------------------------------------------------------------------------
void CArrayIterator_AC::IArrayIterator(const CDynamicArray_AC* itsDynamicArray,
									ArrayIndex_AC itsLowBound,
									ArrayIndex_AC itsHighBound,
									bool itsForward)
{
	fNextLink = NULL;
	fLowBound = fHighBound = fCurrentIndex = kEmptyIndex_AC;

	fIterateForward = itsForward;

	fDynamicArray = itsDynamicArray;

	if (itsDynamicArray)
	{
		// link me in to the list of iterations in progress
		// We alway install the current iteration at the head of the list.
		fNextLink = fDynamicArray->fIteratorPtr;
		const_cast<CDynamicArray_AC*>(fDynamicArray)->fIteratorPtr = this;

		// Restrict the bounds of the iteration to valid values
		fHighBound = MinMax_AC((ArrayIndex_AC) kEmptyIndex_AC, itsHighBound, itsDynamicArray->GetSize());
		fLowBound = MinMax_AC(1, itsLowBound, fHighBound + 1);

		// empty iteration
		if (fLowBound > fHighBound)
			fLowBound = fHighBound = kEmptyIndex_AC;
	}
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::AbortIteration: 
//----------------------------------------------------------------------------------------
void CArrayIterator_AC::AbortIteration()
{
	fDynamicArray = NULL;
	
	CArrayIterator_AC * nextLink = fNextLink;
	while (nextLink)
	{
		nextLink->AbortIteration();
		
		nextLink = nextLink->fNextLink;
	}
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::First: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CArrayIterator_AC::First()
{
	return fCurrentIndex = fIterateForward ? fLowBound : fHighBound;
}


//----------------------------------------------------------------------------------------
// CArrayIterator_AC::Next: 
//----------------------------------------------------------------------------------------
ArrayIndex_AC CArrayIterator_AC::Next()
{
	return fCurrentIndex = fIterateForward ? ((fCurrentIndex < fHighBound) ? fCurrentIndex + 1 : kEmptyIndex_AC) : ((fCurrentIndex > fLowBound) ? fCurrentIndex - 1 : kEmptyIndex_AC);
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::DeleteElementAt: 
//----------------------------------------------------------------------------------------
void CArrayIterator_AC::DeleteElementAt(ArrayIndex_AC theIndex,
									 ArrayIndex_AC theCount)
{
	// tuck the endpoints of the iteration in to match
	if (theIndex < fLowBound)
		fLowBound = Max_AC(fLowBound - theCount, theIndex);
	// This is equivalent to:
	//      if (the Index + theCount <= fLowBound)
	//              fLowBound -= theCount;
	//              // whole deleted range is below fLowBound
	//      else
	//              fLowBound = theIndex;
	//              // new index of the first element not deleted

	if (theIndex <= fHighBound)
		fHighBound = Max_AC(fHighBound - theCount, theIndex - 1);
	// Similar: theIndex - 1 is the last element not deleted.

	// Now bend the fCurrentIndex to account for the deletion.
	// In case the current item is deleted, we give fCurrentIndex such a
	// value that AFTER calling the method advance it will point to the next
	// element to be processed.
	if (fIterateForward)
	{
		if (theIndex <= fCurrentIndex)
			fCurrentIndex = Max_AC(fCurrentIndex - theCount, theIndex - 1);
	}
	else
	{											// Iterating backwards
		if (theIndex < fCurrentIndex)
			fCurrentIndex = Max_AC(fCurrentIndex - theCount, theIndex);
	}

	// hand off control to the next link until you hit the last link in the chain
	if (fNextLink)
		fNextLink->DeleteElementAt(theIndex, theCount);
}

//----------------------------------------------------------------------------------------
// CArrayIterator_AC::InsertElementBefore: 
//----------------------------------------------------------------------------------------
void CArrayIterator_AC::InsertElementBefore(ArrayIndex_AC theIndex,
										 ArrayIndex_AC theCount)
{
	// bump the endpoints of this iteration out to match
	if (theIndex <= fLowBound)
		fLowBound += theCount;

	if (theIndex <= fHighBound)
		fHighBound += theCount;

	if (fIterateForward)
	{
		// If the inserted element was !in the range yet to be
		// iterated then bend the fCurrentIndex to account for it.
		if (theIndex <= fCurrentIndex)
			fCurrentIndex += theCount;
	}
	else										// Iterating backward 
		{
			// If the inserted element was IN the range yet to be
			// iterated then bend the fCurrentIndex to account for it.
			if (theIndex < fCurrentIndex)
				fCurrentIndex += theCount;
		}

	// hand off control to the next link until you hit the last link in the chain
	if (fNextLink)
		fNextLink->InsertElementBefore(theIndex, theCount);
}

//========================================================================================
// CLASS CCommonArrayIterator_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CCommonArrayIterator_AC::CCommonArrayIterator_AC: 
//----------------------------------------------------------------------------------------
CCommonArrayIterator_AC::CCommonArrayIterator_AC(const CDynamicArray_AC* itsList,
					ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					bool itsForward) :

CArrayIterator_AC(itsList, itsLowBound, itsHighBound, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CCommonArrayIterator_AC::CCommonArrayIterator_AC: 
//----------------------------------------------------------------------------------------
CCommonArrayIterator_AC::CCommonArrayIterator_AC(const CDynamicArray_AC* itsList, bool itsForward) :
CArrayIterator_AC(itsList, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CCommonArrayIterator_AC::First: 
//----------------------------------------------------------------------------------------
void* CCommonArrayIterator_AC::First()
{
	return fCurrentElement = CArrayIterator_AC::First() ? fDynamicArray->ComputeAddress(fCurrentIndex) : NULL;
}

//----------------------------------------------------------------------------------------
// CCommonArrayIterator_AC::Next: 
//----------------------------------------------------------------------------------------
void* CCommonArrayIterator_AC::Next()
{
	return fCurrentElement = CArrayIterator_AC::Next() ? fDynamicArray->ComputeAddress(fCurrentIndex) : NULL;
}

//========================================================================================
// CLASS CCommonVoidIterator_AC
//========================================================================================
//----------------------------------------------------------------------------------------
// CCommonVoidIterator_AC::CCommonVoidIterator_AC: 
//----------------------------------------------------------------------------------------
CCommonVoidIterator_AC::CCommonVoidIterator_AC(const CDynamicArray_AC* itsList,
					ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					bool itsForward) :

CCommonArrayIterator_AC(itsList, itsLowBound, itsHighBound, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CCommonVoidIterator_AC::CCommonVoidIterator_AC: 
//----------------------------------------------------------------------------------------
CCommonVoidIterator_AC::CCommonVoidIterator_AC(const CDynamicArray_AC* itsList, bool itsForward) :
CCommonArrayIterator_AC(itsList, itsForward)
{
	First();
}

//----------------------------------------------------------------------------------------
// CCommonVoidIterator_AC::First: 
//----------------------------------------------------------------------------------------
void* CCommonVoidIterator_AC::First()
{
	return fCurrent = CCommonArrayIterator_AC::First() ? *static_cast<void**>(CCommonArrayIterator_AC::Current()) : NULL;
}

//----------------------------------------------------------------------------------------
// CCommonVoidIterator_AC::CCommonVoidIterator_AC: 
//----------------------------------------------------------------------------------------
void* CCommonVoidIterator_AC::Next()
{
	return fCurrent = CCommonArrayIterator_AC::Next() ? *static_cast<void**>(CCommonArrayIterator_AC::Current()) : NULL;
}

//----------------------------------------------------------------------------------------
// End of UListIterator.cp
