//----------------------------------------------------------------------------------------
// CAutoPtr_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CAutoPtr_AC__
#define __CAutoPtr_AC__

// ACS core includes.

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CAutoPtr_AC
//----------------------------------------------------------------------------------------

template <class T>
class CAutoPtr_AC  
{
public:	
	CAutoPtr_AC(T* ptr = NULL);
		// Assumes ownership of ptr. Will delete ptr upon destruction of the AutoPtr.

	CAutoPtr_AC(const CAutoPtr_AC<T> &toCopy);
		// The result of this will be a transfer of ownership from toCopy
		// to this AutoPtr, i.e. toCopy will become NULL.
		
	~CAutoPtr_AC();
		// This will delete fPointee if not NULL.
		
	T* get() const;
		// Get the data.
			
	T* operator ->() const;
		// operate on the data
		
	operator T* () const;
		// convert to the data
		
	T& operator *() const;
		// similate derefencing the pointer. Returns a ref to the data.
		
	void operator = (T* ptr);
		// Assumes ownership of ptr after delete fPointee if fPointee is not
		// NULL. Will delete ptr upon destruction of the AutoPtr.
	
	CAutoPtr_AC<T> &operator = (const CAutoPtr_AC<T> &toCopy );
		// The result of this will be a transfer of ownership from toCopy
		// to this AutoPtr, i.e. toCopy will become NULL. Also,  fPointee will 
		// be deleted if not NULL.
	
	bool operator == (const CAutoPtr_AC<T> &p) const;
		// compare value of fPointee to p.fPointee.
		
	bool operator != (const CAutoPtr_AC<T> &p) const;
		// compare value of fPointee to p.fPointee.
		
	bool operator == (const T* p) const;
		// compare value of fPointee to p
	
	bool operator != (const T* p) const;
		// compare value of fPointee to p
		
	T* release() const;
		// Returns a pointer to the data and sets fPointee to NULL.
		// Does not delete the data.
		
	void reset();
		// Sets the pointer to NULL after deleting the data if its not NULL.

	bool IsNULL() const;
		// Test for NULL. Use this instead of MyPtr == NULL or MyPtr != NULL, 
		// i.e. use MyPtr.IsNULL.

private:
	
#if qCoreDebug	

// These are here as helpers so if you call MyPtr == NULL it you'll get an
// error that lands on the offending line of code instead of somewhere way 
// up in the headers that is confusing as heck.

	bool operator == (const void* ) const; // Use IsNULL
	bool operator != (const void* ) const; // Use IsNULL
	bool operator == (int ) const; // Use IsNULL
	bool operator != (int ) const; // Use IsNULL
#endif
	
private:
	T* fPointee;
};

//----------------------------------------------------------------------------------------
//	Inline Implementation
//----------------------------------------------------------------------------------------

template <class T>
inline	CAutoPtr_AC<T>::CAutoPtr_AC(T* ptr) 
: 	fPointee(ptr)			
{ 
}
template <class T>
inline	CAutoPtr_AC<T>::CAutoPtr_AC(const CAutoPtr_AC<T> &toCopy) 
:	fPointee(toCopy.release())	
{ 
}

template <class T>
inline	CAutoPtr_AC<T>::~CAutoPtr_AC() 
{ 
	reset();
}
template <class T>
inline	T* CAutoPtr_AC<T>::get() const 
{ 
	return fPointee; 
}
template <class T>
inline	T* CAutoPtr_AC<T>::operator ->() const
{ 
	::AssertNotNULL_AC(fPointee);
	return fPointee; 
}
template <class T>
inline	CAutoPtr_AC<T>::operator T* () const
{ 
	return fPointee; 
}
		
template <class T>
inline	void CAutoPtr_AC<T>::operator = ( T* ptr) 
{ 
	if(fPointee != ptr)
	{
		reset();
		fPointee = ptr;
	}
}

template <class T>
inline	T&  CAutoPtr_AC<T>::operator *() const
{
	::AssertNotNULL_AC(fPointee);
	return *fPointee; 
} 
template <class T>
inline CAutoPtr_AC<T> &CAutoPtr_AC<T>::operator = (const CAutoPtr_AC<T> &ptr )
{ 
	if (this != &ptr)
	{
		if(*this != ptr)
			reset();
	
		fPointee = ptr.release(); 
	}
	return *this; 
}
template <class T>
inline	bool CAutoPtr_AC<T>::operator == (const CAutoPtr_AC<T> &p) const 
{	
	return fPointee == p.fPointee; 
}
template <class T>
inline	bool CAutoPtr_AC<T>::operator != (const CAutoPtr_AC<T> &p) const 
{	
	return fPointee != p.fPointee;	
}
template <class T>
inline	bool CAutoPtr_AC<T>::operator == (const T* p) const 
{	
	return fPointee == p; 
}
template <class T>
inline	bool CAutoPtr_AC<T>::operator != (const T* p) const 
{	
	return fPointee != p;	
}

template <class T>
inline void CAutoPtr_AC<T>::reset() 
{ 	
	::DeleteIfObject_AC(fPointee);
}

template <class T>
inline T* CAutoPtr_AC<T>::release() const
{ 	
	T* old = fPointee;
	const_cast<CAutoPtr_AC*>(this)->fPointee = NULL;
	return old;
}

template <class T>
inline bool CAutoPtr_AC<T>::IsNULL() const
{
	return fPointee == NULL;
}


#endif