//----------------------------------------------------------------------------------------
// CRefCountingLock_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


/*
	This class will never delete a ref counted object even if the ref count goes to 
	zero. Be careful using this that you don't allow the block to never be deleted.
	
	Use this when you need to hold onto a pointer to ref counted object and you
	explicitely do not own and you don't want it to be deleted.
	
	
*/


#ifndef __CRefCountingLock_AC__
#define __CRefCountingLock_AC__

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __MRefCountable_AC__
#include "MRefCountable_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CRefCountingLock_AC
//----------------------------------------------------------------------------------------

template <class T>
class CRefCountingLock_AC  
{
public:	
	CRefCountingLock_AC(T* ptr);

	CRefCountingLock_AC(const CRefCountingLock_AC<T> &toCopy);

	CRefCountingLock_AC();

	~CRefCountingLock_AC();
	
	T* get() const;
	
	T* operator ->() const;
		
	operator T* () const;
	
	T & operator *() const;
	
	void operator = (T* ptr);
	
	CRefCountingLock_AC<T> &operator = ( const CRefCountingLock_AC<T> &ptr );
	
	bool operator == (const CRefCountingLock_AC<T> &p) const;
	
	bool operator != (const CRefCountingLock_AC<T> &p) const;
		
	bool operator == (const T* p) const;
	
	bool operator != (const T* p) const;

	T* release();
		// This release the pointer and does not delete it even if the ref
		// count goes to zero.
	
	bool IsNULL() const;

private:

#if qCoreDebug	
	bool operator == (const void* ) const; // Use IsNULL
	bool operator != (const void* ) const; // Use IsNULL
	bool operator == (int ) const; // Use IsNULL
	bool operator != (int ) const; // Use IsNULL
#endif

private:
	T* fPointee;
};

//----------------------------------------------------------------------------------------
// Inline Implementation 
//----------------------------------------------------------------------------------------

template <class T>
inline	CRefCountingLock_AC<T>::CRefCountingLock_AC(T* ptr) 
: 	fPointee(ptr)			
{ 
	if(fPointee) fPointee->aquire();
}
template <class T>
inline	CRefCountingLock_AC<T>::CRefCountingLock_AC(const CRefCountingLock_AC<T> &toCopy) 
:	fPointee(toCopy.fPointee)	
{ 
	if(fPointee) fPointee->aquire();
}
template <class T>
inline	CRefCountingLock_AC<T>::CRefCountingLock_AC() 
: 	fPointee(NULL) 
{
}
template <class T>
inline	CRefCountingLock_AC<T>::~CRefCountingLock_AC() 
{ 
	release();
}
template <class T>
inline	T* CRefCountingLock_AC<T>::get() const 
{ 
	return fPointee; 
}
template <class T>
inline	T* CRefCountingLock_AC<T>::operator ->() const
{ 
	::AssertNotNULL_AC(fPointee);
	return fPointee; 
}
template <class T>
inline	CRefCountingLock_AC<T>::operator T* () const
{ 
	return fPointee; 
}
		
template <class T>
inline	void CRefCountingLock_AC<T>::operator = ( T* ptr) 
{ 
	release();
	fPointee = ptr; 
	if(fPointee) fPointee->aquire();
}

template <class T>
inline	T & CRefCountingLock_AC<T>::operator *() const
{
	::AssertNotNULL_AC(fPointee);
	return *fPointee; 
} 
template <class T>
inline	CRefCountingLock_AC<T> &CRefCountingLock_AC<T>::operator = ( const CRefCountingLock_AC<T> &ptr )
{ 
	if(this != &ptr)
	{
		release();
		fPointee = ptr.fPointee; 
		if(fPointee) fPointee->aquire();
	}
	return *this; 
}
template <class T>
inline	bool CRefCountingLock_AC<T>::operator == (const CRefCountingLock_AC<T> &p) const 
{	
	return fPointee == p.fPointee; 
}
template <class T>
inline	bool CRefCountingLock_AC<T>::operator != (const CRefCountingLock_AC<T> &p) const 
{	
	return fPointee != p.fPointee;	
}
template <class T>
inline T* CRefCountingLock_AC<T>::release() 
{ 	
	T* old = fPointee;
	fPointee = NULL;
	if(old) old->release();
	return old;
}
template <class T>
inline bool CRefCountingLock_AC<T>::operator == (const T* inPtr) const
{
	return fPointee == inPtr;
}
	
template <class T>
inline bool CRefCountingLock_AC<T>::operator != (const T* inPtr) const
{
	return fPointee != inPtr;
}

template <class T>
inline bool CRefCountingLock_AC<T>::IsNULL() const
{
	return fPointee == NULL;
}


#endif