//----------------------------------------------------------------------------------------
// CException_AC.cp
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// note on the terminate handler:
//
// The terminate handler gets called if an exception is thrown that is not 
// caught. The terminate_handler is not returned from and no informatation
// about the thrown exception is provided with the handler. Abort is immediately called after
// the terminate_handler. The best we can hope for is somesort of feedback saying sorry,
// we're about to terminate.
//
// The terminate_handler is called from terminate() and unexpected(), both of
// which can be user defined procs.
//
// Note also that MacApp wraps most places where an exception would be thrown
// with try/catch blocks that catch the exceptions so that appropriate
// action can be taken. Terminate will only be called in unusual circumstances,
// perhaps during application init time, for example.
//----------------------------------------------------------------------------------------


#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __typeinfo__
#include <typeinfo.h>
#endif


#if qDebugExceptions
#ifndef __stdio__
#include <stdio.h>
#endif

#ifndef __string__
#include <string.h>
#endif

#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __Resources__
#include <Resources.h>
#endif

#ifndef __Errors__
#include <Errors.h>
#endif

//----------------------------------------------------------------------------------------
// class CSetDefaultTerminateHandler_AC
//----------------------------------------------------------------------------------------

class CSetDefaultTerminateHandler_AC
{
public:
	CSetDefaultTerminateHandler_AC()
	{
		CException_AC::ResetTerminateHandler();
	}

};

#if qDebugExceptions == qDebugLevelTwo_AC
	bool CException_AC::fgDoThrowMessage = true;
	bool CException_AC::fgDoCatchMessage = false;
#elif qDebugExceptions == qDebugLevelThree_AC
	bool CException_AC::fgDoThrowMessage = false;
	bool CException_AC::fgDoCatchMessage = true;
#elif qDebugExceptions == qDebugLevelFour_AC
	bool CException_AC::fgDoThrowMessage = true;
	bool CException_AC::fgDoCatchMessage = true;
#endif

terminate_handler CException_AC::fgPreviousTerminateHandler = NULL;

static CSetDefaultTerminateHandler_AC gSetDefault;

//----------------------------------------------------------------------------------------
// CException_AC::GetPreviousTerminateHandler
//----------------------------------------------------------------------------------------

void CException_AC::ResetTerminateHandler()
{
	fgPreviousTerminateHandler = set_terminate(DefaultTerminateHandler);
}
	
//----------------------------------------------------------------------------------------
// CException_AC::GetPreviousTerminateHandler
//----------------------------------------------------------------------------------------

terminate_handler CException_AC::GetPreviousTerminateHandler()
{
	return fgPreviousTerminateHandler;
}


//----------------------------------------------------------------------------------------
// CException_AC::SetTerminateHandler
//----------------------------------------------------------------------------------------

terminate_handler CException_AC::SetTerminateHandler( terminate_handler inHandler)
{
	fgPreviousTerminateHandler = set_terminate(inHandler);

	return fgPreviousTerminateHandler;
}

//----------------------------------------------------------------------------------------
// CException_AC::DefaultTerminateHandler
//----------------------------------------------------------------------------------------

void CException_AC::DefaultTerminateHandler()
{
#if qDebugExceptions
	::ProgramReport_AC(
		"The application encountered an unhandled and fatal exception of unknown type.",
		true);
#endif
		
	fgPreviousTerminateHandler();
}

//----------------------------------------------------------------------------------------
// CException_AC::BuildMessage
//----------------------------------------------------------------------------------------

long CException_AC::BuildMessage(short lowWord, short highWord) 
{ 
	return (static_cast<long>(highWord) << 16) | lowWord; 
}

//----------------------------------------------------------------------------------------
// CException_AC::SetMessage
//----------------------------------------------------------------------------------------

void CException_AC::SetMessage(short lowWord, short highWord)
{
	fMessage = BuildMessage(lowWord, highWord);
}


#if qDebugExceptions

#if qDebugExceptions >= qDebugLevelTwo_AC	

//----------------------------------------------------------------------------------------
// CException_AC::DoThrowMessage
//----------------------------------------------------------------------------------------

void CException_AC::DoThrowMessage()
{
	if(fgDoThrowMessage)
	{
		if(fError == memFullErr)
			::ProgramReport_AC( false, "%s Thrown: type:memFullErr, msg:%ld\nThrown At:%s, Line:%d", what(), fMessage,fFile,fLine);
		else
			::ProgramReport_AC( false, "%s Thrown: type:%ld, msg:%ld\nThrown At:%s, Line:%d", what(), fError, fMessage,fFile,fLine);
	}
}

//----------------------------------------------------------------------------------------
// CException_AC::DoCatchMessage_AC
//----------------------------------------------------------------------------------------

void CException_AC::DoCatchMessage(	const char* fileName,
									short lineNum,
									bool doBreak)
{
	if(fgDoCatchMessage)
	{
		if(fError == memFullErr)
			::ProgramReport_AC( doBreak, "%s Caught: type:memFullErr, msg:%ld\nCaught at:%s, Line:%d", what(), fMessage,fileName,lineNum);
		else
			::ProgramReport_AC( doBreak, "%s Caught: type:%ld, msg:%ld\nCaught At:%s, Line:%d", what(), fError, fMessage,fileName,lineNum);
	}
}


#endif //	qDebugExceptions >= qDebugLevelTwo_AC

//----------------------------------------------------------------------------------------
// CException_AC::SetThrownFrom
//----------------------------------------------------------------------------------------

void CException_AC::SetThrownFrom(const char* inFile, short inLine)
{
	::strcpy(fFile, inFile);
	fLine = inLine;
}


//----------------------------------------------------------------------------------------
// DebugThrowError_AC
//----------------------------------------------------------------------------------------

void DebugThrowError_AC( CError_AC inErr, const char* inFile, short inLine)
{
	CException_AC ex(inErr);
	ex.SetThrownFrom(inFile, inLine);
	DoThrowMessage_AC(ex);
	throw(ex);
}


//----------------------------------------------------------------------------------------
// DebugThrowIfError_AC
//----------------------------------------------------------------------------------------

void DebugThrowIfError_AC( CError_AC inErr, const char* inFile, short inLine)
{
	if(inErr) 
		DebugThrowError_AC(inErr, inFile, inLine);
}


//----------------------------------------------------------------------------------------
// DebugThrowIfMemError_AC
//----------------------------------------------------------------------------------------

void DebugThrowIfMemError_AC( const char* inFile, short inLine)
{
	OSErr e = MemError();

	if (e != noErr)
		::DebugThrowError_AC(e, inFile, inLine);
}


//----------------------------------------------------------------------------------------
// DebugThrowException_AC
//----------------------------------------------------------------------------------------

void DebugThrowException_AC(CError_AC inErr, CException_AC::Message inMsg,  
							const char *inFileName, short inLineNum)
{
	CException_AC ex(inErr, inMsg);
	ex.SetThrownFrom(inFileName, inLineNum);
	DoThrowMessage_AC(ex);
	throw ex;
}


//----------------------------------------------------------------------------------------
// DebugThrowIfNULLResource
//----------------------------------------------------------------------------------------

void DebugThrowIfNULLResource_AC(void* r, const char* inFileName, short inLineNum)
{
	if (!r)
	{
		OSErr e = ResError();
		if (e == noErr)
			e = resNotFound;
		::DebugThrowError_AC(e, inFileName, inLineNum);
	}
}

//----------------------------------------------------------------------------------------
// DebugThrowIfOSErr_AC
//----------------------------------------------------------------------------------------

void DebugThrowIfOSErr_AC(OSErr error, const char* inFileName, short inLineNum)
{
	if (error != noErr)
		::DebugThrowError_AC(error, inFileName, inLineNum);
}

//----------------------------------------------------------------------------------------
// DebugThrowIfNonObject_AC:
//----------------------------------------------------------------------------------------

void DebugThrowIfNonObject_AC(void* obj, const char* inFileName, short inLineNum)
{
	if (!obj || ::Odd_AC(obj))
	{
#if qCoreDebug
		::ProgramBreak_AC("Object that failed discipline: %p", obj);
#endif
		::DebugThrowError_AC(minErr, inFileName, inLineNum);
	}
}

//----------------------------------------------------------------------------------------
// DebugThrowIfResError_AC: 
//----------------------------------------------------------------------------------------

void DebugThrowIfResError_AC(const char* inFileName, short inLineNum)
{
	OSErr e = ::ResError();

	if (e != noErr)
		::DebugThrowError_AC(e, inFileName, inLineNum);
}

//----------------------------------------------------------------------------------------
// DebugThrowNewMessage_AC
//----------------------------------------------------------------------------------------

void DebugThrowNewMessage_AC(	CError_AC inError, 
								CException_AC::Message oldMessage, 
								CException_AC::Message newMessage,
								const char* inFileName, 
								short inLineNum)
{
	CException_AC ex(inError, oldMessage ? oldMessage : newMessage);
	ex.SetThrownFrom(inFileName, inLineNum);
	DoThrowMessage_AC(ex);
	throw(ex);
}

//----------------------------------------------------------------------------------------
// DebugThrowIfNULL_AC
//----------------------------------------------------------------------------------------

void DebugThrowIfNULL_AC( const void* inPtr, const char* inFile, short inLine)
{
	if(inPtr == NULL)
	{
		CException_AC ex(memFullErr);
		ex.SetThrownFrom(inFile, inLine);
		DoThrowMessage_AC(ex);
		throw(ex);
	}
}

//----------------------------------------------------------------------------------------
// DebugThrowBadCast_AC
//----------------------------------------------------------------------------------------

void DebugThrowBadCast_AC(const void* castTo, const void* castFrom, const char* inFile, short inLine)
{
	ProgramBreak_AC("Cast From %s* to %s failed. File: %s, Line: %d", castFrom, castTo, inFile, inLine);
	throw bad_cast("bad_cast");
}


//=======================================================================================
// Non Debug Methods	
//=======================================================================================
#else


//----------------------------------------------------------------------------------------
// ThrowIfMemError_AC
//----------------------------------------------------------------------------------------

void ThrowIfMemError_AC()
{
	OSErr e = ::MemError();

	if (e != noErr)
		::ThrowError_AC(e);
}

//----------------------------------------------------------------------------------------
// ThrowIfResError_AC
//----------------------------------------------------------------------------------------

void ThrowIfResError_AC()
{
	OSErr e = ::ResError();

	if (e != noErr)
		::ThrowError_AC(e);
}

//----------------------------------------------------------------------------------------
// ThrowIfNonObject_AC
//----------------------------------------------------------------------------------------


void ThrowIfNonObject_AC(void* obj)
{
	if (!obj || ::Odd_AC(obj))
	{
		::ThrowError_AC(minErr);
	}
}

//----------------------------------------------------------------------------------------
// ThrowIfNULLResource_AC
//----------------------------------------------------------------------------------------

void ThrowIfNULLResource_AC(void* r)
{
	if (!r)
	{
		OSErr e = ::ResError();
		if (e == noErr)
			e = resNotFound;
		::ThrowError_AC(e);
	}
}

//----------------------------------------------------------------------------------------
// ThrowIfNULL_AC
//----------------------------------------------------------------------------------------

void ThrowIfNULL_AC( const void* inPtr )
{
	if(inPtr == NULL)
	{
		CException_AC ex = memFullErr;
		throw(ex);
	}
}

//----------------------------------------------------------------------------------------
// ThrowIfOSErr_AC
//----------------------------------------------------------------------------------------

void ThrowIfOSErr_AC(OSErr error)
{
	if (error != noErr)
		::ThrowError_AC(error);
}

//----------------------------------------------------------------------------------------
// ThrowIfOSErr_AC
//----------------------------------------------------------------------------------------

void ThrowIfBadCast_AC( const void* inPtr )
{
	if(inPtr == NULL)
	{
		throw bad_cast("bad_cast");
	}
}

#endif


