//----------------------------------------------------------------------------------------
// MemoryUtils_AC.cp 
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __MemoryUtils_AC__
#include "MemoryUtils_AC.h"
#endif

#if q68K || (UINT_MAX <= ULONG_MAX)
// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif
#endif

//========================================================================================
// GLOBAL Functions
//========================================================================================

//----------------------------------------------------------------------------------------
// StripLong_AC
//----------------------------------------------------------------------------------------
#if q68K && !qNeedsSystem7_6
long StripLong_AC(void* address)
{
	//----------------------------------------------------------------------------------------
	// gStrippedAddress is used on 68K systems where some of the bits aren't part of the address
	// it is embedded in this function to control initialization ordering so that it will
	// be initialized when needed for this function.
	//----------------------------------------------------------------------------------------
	// -1 == 0xFFFFFFFF, the largest 32 bit address. Our routine StripLong_AC uses a
	// pre-stripped address gStrippedAddress_AC to avoid the yucky MPW glue. 
	static const void* gStrippedAddress_AC = StripAddress((void*) -1);

	return reinterpret_cast<long>(address) & reinterpret_cast<long>(gStrippedAddress_AC);
}
#endif

//----------------------------------------------------------------------------------------
// BlockMove_AC:
//----------------------------------------------------------------------------------------

#if (UINT_MAX < ULONG_MAX)
void BlockMove_AC(const void* srcPtr, void* destPtr, long byteCount)
{
	// We think size_t is at least a long, so we don't need a cast.
	// If we get a compiler warning, we're in trouble. 
	if (byteCount < UINT_MAX) // size_t could be an unsigned 16 bit value
		::memcpy(destPtr, srcPtr, byteCount);
	else
		BlockMoveData(srcPtr, destPtr, byteCount);
}
#endif

//----------------------------------------------------------------------------------------
// BlockMoveOverlap_AC:
//----------------------------------------------------------------------------------------

#if (UINT_MAX < ULONG_MAX)
void BlockMoveOverlap_AC(const void* srcPtr, void* destPtr, long byteCount)
{
	// We think size_t is at least a long, so we don't need a cast.
	// If we get a compiler warning, we're in trouble. 
	if (byteCount < UINT_MAX) // size_t could be an unsigned 16 bit value
		::memmove(destPtr, srcPtr, byteCount);
	else
		BlockMoveData(srcPtr, destPtr, byteCount);
}
#endif

//----------------------------------------------------------------------------------------
// BlockSet: Much thanks for this improved version to Scott D. Schmitz
//----------------------------------------------------------------------------------------

void BlockSet_AC(void* destPtr,
			  long byteCount,
			  unsigned char setVal)
{
	// here are the results of using chars (1), longs (4) or doubles (8) on 68K and PowerPC
	// 				68K				601
	//---------------------------------------
	// char			261	(100)		237 (100)
	// long			156	(60)		68 	(29)
	// double		933	(357)		46	(19)
	//---------------------------------------
	// the poor result for 68K/double is to be expected. Jeroen Schalk, DTS.
	
#if qPowerPC
	const unsigned long kBytesPerPunch = 8;
	typedef double Punch, *PunchPtr;
		// select type double for best performance
#else
	const unsigned long kBytesPerPunch = 4;
	typedef unsigned long Punch, *PunchPtr;
		// select type long for best performance
#endif

	char* begPtr = reinterpret_cast<char*>(StripLong_AC(destPtr));
	char* endPtr = begPtr + byteCount;

	// this looks like the right cutoff, since for the optimization 
	// O(n) = kBPP + kBPP/2 + n/kBPP + kBPP/2 = 2*kBPP + n/kBPP
	// a simple linear fill has O(n) = n
	// n > 2*kBPP + n/kBPP if n > 2*kBPP, since 1/kBPP  0.
	
	if (byteCount >= (4 * kBytesPerPunch))
	{
		// progress to nearest 4 byte boundary. O(n) = kBytesPerPunch/2
		while (reinterpret_cast<long>(begPtr) & (kBytesPerPunch - 1))
			*begPtr++ = setVal;

		// lets get a kBytesPerPunch byte 'punch'. O(n) = kBytesPerPunch
		register PunchPtr punchSetValPtr = (PunchPtr)begPtr;	// save pointer to the first value
#if qPowerPC
		*begPtr++ = setVal;										// fill it in
		*begPtr++ = setVal;
		*begPtr++ = setVal;
		*begPtr++ = setVal;
#endif
		*begPtr++ = setVal;
		*begPtr++ = setVal;
		*begPtr++ = setVal;
		*begPtr++ = setVal;
		register Punch punchSetVal = *punchSetValPtr++;			// pick it up

		// truncate to nearest kBytesPerPunch bytes
		PunchPtr punchEndPtr = (PunchPtr)(reinterpret_cast<long>(endPtr) & (0xFFFFFFFF - (kBytesPerPunch - 1)));
	
		// assign in kBytesPerPunch byte chunks what we can. O(n) = n/kBytesPerPunch
		while (punchSetValPtr < punchEndPtr)
			*punchSetValPtr++ = punchSetVal;
			
		begPtr = (char*)punchSetValPtr;
	}
	
	// now finish assigning odd bytes. O(n) = kBytesPerPunch/2 if optimization chosen.
	while (begPtr < endPtr)
		*begPtr++ = setVal;

}

//----------------------------------------------------------------------------------------
// End of MemoryUtils_AC.cp

