//----------------------------------------------------------------------------------------
// CBuffer_AC.h
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CBuffer_AC__
#define __CBuffer_AC__

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CRefCountingPtr_AC__
#include "CRefCountingPtr_AC.h"
#endif

#ifndef __CByteIterator_AC__
#include "CByteIterator_AC.h"
#endif

// forward refs

class CBuffer_AC;

// typedefs

typedef class EXPORT_AC CRefCountingPtr_AC<CBuffer_AC> CBufferPtr_AC;


//----------------------------------------------------------------------------------------
// CBuffer_AC is a class for managing a raw data buffer. If the CBuffer created the 
// the buffer, it can grow or shrink the data. The CBuffer is refcounted.
//----------------------------------------------------------------------------------------

class EXPORT_AC CBuffer_AC : public MRefCountable_AC
{
public:

	enum eMisc
	{
		kDefaultGrowChunkPercent = 10
			// the percent by which to grow or shrink the buffer by
	};

	enum eOwnership
	{
		kOwnData,		// The buffer owns the data
		kBorrowData,	// The buffer does not own the data
		kAutoSet		// The buffer will decide if it owns the data
			
	};

	CBuffer_AC( const CBuffer_AC& in );
		// copy ctor

	CBuffer_AC(long size, void* buf = NULL, eOwnership ownerFlags = kAutoSet);
		// If owner flag is kAutoSet: If buf is not NULL, CBuffer will assume
		// it doesn't own the data. If it's NULL it will make a buffer the size
		// of 'size'.

	CBuffer_AC();
		// default ctor
		
	~CBuffer_AC();
		// dtor
	
	CBuffer_AC& operator = (const CBuffer_AC& inBlock);
		// assignment operator

// for accessing the buffer
	operator char* ();
	char* GetBuffer() const;
		// returns a pointer to the start of the buffer
	
	CByteIterator_AC GetIterator();
		// get an interator for accessing and looking at the data.
			
// buffer management

	// Note: Attempts to resize a buffer that isn't owned will result in an exception.
	
	void SetBuffer(long size, void* buf = NULL, eOwnership ownerFlags = kAutoSet);
		// If owner flag is kAutoSet: If buf is not NULL, CBuffer will assume
		// it doesn't own the data. If it's NULL it will make a buffer the size
		// of 'size'.
		
	void SetSize(long inSize, bool preserveData = true);
		// Resize the buffer (if CBuffer owns it). Preserve data will have the
		// CBuffer copy the old data into the new data.

	long GetSize() const;
		// Current size of the buffer.

	void Grow(bool preserveData = true);
		// grow by chunk size
	
	void Shrink(bool preserveData = true);
		// Shrink by chunk size

	void ReleaseBuffer();
		// this will delete the data if this buffer owns the data.

	void SetGrowChunkSize(long inSize);
		// the default is kGrowChunkPercent of size
	
	void CalcGrowChunkSize();
		// this will calc the grow size, 
		// which by default size divided by kGrowChunkPercent
	
	long GetGrowChunkSize() const;

	void SetGrowChunkPercent(short newPercent);
		// The percent is how much the CBuffer should grow or shrink
		// the buffer by.
	
	short GetGrowChunkPercent() const;
	
	long GetGrownAmount() const;
		// get the amount the buffer has been grown or shrunk from 
		// original size param. Use this to keep growing under control.
		
	void ResetGrownAmount();	
		// sets the amount the buffer has been resize to zero
			
	void ResetBufferSize(bool preserveData = true);
		// This will reset the buffer to the original size.
				
private:	
	char* fBuffer;
	long fSize;
	long fGrownAmt;
	long fGrowChunkSize;
	bool fOwner;
	short fGrowChunkPercent;
};

// Inline functions

inline char* CBuffer_AC::GetBuffer() const
{	
	return fBuffer;
}

inline long  CBuffer_AC::GetSize() const
{	
	return fSize; 
}
inline CBuffer_AC::operator char* ()
{
	return fBuffer;
}

inline CByteIterator_AC CBuffer_AC::GetIterator()
{
	 return CByteIterator_AC(fBuffer, fSize);
}

inline void CBuffer_AC::Grow(bool preserveData)
{	
	SetSize( fSize + fGrowChunkSize, preserveData); 
}

inline void CBuffer_AC::Shrink(bool preserveData)
{	
	SetSize( fSize - fGrowChunkSize, preserveData); 
}

inline void CBuffer_AC::SetGrowChunkSize(long inSize)
{	
	fGrowChunkSize = inSize; 
}

inline long CBuffer_AC::GetGrowChunkSize() const
{	
	return fGrowChunkSize; 
}

inline long CBuffer_AC::GetGrownAmount() const
{	
	return fGrownAmt; 
}

inline void CBuffer_AC::ResetBufferSize(bool preserveData)
{
	if(fGrownAmt != 0)
		SetSize( fSize + fGrownAmt, preserveData );
}

inline void CBuffer_AC::ResetGrownAmount()
{
	fGrownAmt = 0;
}

inline void CBuffer_AC::SetGrowChunkPercent(short newPercent)
{
	fGrowChunkPercent = newPercent;
}

inline short CBuffer_AC::GetGrowChunkPercent() const
{
	return fGrowChunkPercent;
}

inline void CBuffer_AC::CalcGrowChunkSize()
{
	
	fGrowChunkSize = fSize / fGrowChunkPercent;
}
#endif