//----------------------------------------------------------------------------------------
// CByteIterator_AC.h
// Copyright  1985-96 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __CByteIterator_AC_AC__
#define __CByteIterator_AC_AC__

// Patterns

#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

//----------------------------------------------------------------------------------------
// CByteIterator_AC
//----------------------------------------------------------------------------------------
	
class EXPORT_AC CByteIterator_AC
{
public:
	CByteIterator_AC(void* bytes, long size);
		// Iterate over any chunk of bytes
		
	CByteIterator_AC(const CByteIterator_AC& inIter);
		// copy ctor

	CByteIterator_AC();
		// default ctor		

	~CByteIterator_AC();
		// dtor
		
// assignment operators	

	CByteIterator_AC& operator = (const CByteIterator_AC& inIter);
		// make this iterator point to the same place and string as inIter
		
	void operator = (char inChar);
		// sets current char
	
// misc
	bool IsValid() const;
		// Assuming the bytes and size are valid, is the iterator
		// pointing within the assigned chunk of memory?
	
	bool IsAtEnd() const;
		// The position is as far to the end as it can go. If the pos is 
		// at the end, trying to get the byte ptr will result in a assertion 
		// failure.

	void SetBuffer(void* bytes, long size);
		// Set an arbritrary chunk of data to iterate over
		
	char* GetBuffer();
	
	long GetBufferSize() const;					

// for peeking at the data - these don't move the iterator
	char operator + (unsigned long in) const;
	char operator - (unsigned long in) const;
	char operator[] (long in) const;
	char&operator[] (long in);

// for moving the iterator	
			
	void operator += (long inAmt);
	void operator -= (long inAmt);
	// prefix
	void operator ++ ();
	void operator -- ();
	// postfix
	void operator ++ (int in);
	void operator -- (int in);

	void SetPosition(long pos);
	long GetPosition() const;
		// Position is where the iterator is currently pointing at in the data.

	long GetOffsetToEnd() const;
		// returns how far it is from current iterator pos to the end of data.

	void Offset(long inAmt);
		// Move the iterator.

	void Rewind();
		// set to first byte	
	void SetToLast();
		// set to last byte
	void SetToEnd();
		// set to end (i.e. can't read or write)

// misc
	
	operator char() const;	
		// returns char at current pos
		
	char* operator* () const;
	char* GetPositionPtr() const;
	operator char* () const;
		// all return byte ptr at current pos

	bool operator == (char inChar) const;
	bool operator != (char inChar) const;
		// compare against char in current pos

#if qPatternsDebug_AC
	void RangeCheckWalker(bool canBeAtEnd) const;
	void AssertValidAddress(const char* addr) const;
#endif	
								
protected:
	char*		fBytes;
	long		fSize;
	char*		fWalker;
};	

// Inline functions

inline bool CByteIterator_AC::IsValid() const
{
	return 	(fWalker != NULL) && 
			(fBytes != NULL) && 
			(fWalker >= fBytes) && 
			(fWalker < (fBytes + fSize)) ;
}

#if qPatternsDebug_AC
inline void CByteIterator_AC::RangeCheckWalker(bool CanBeAtEnd) const
{
	if(CanBeAtEnd)
	{
		::Assertion_AC( ( fBytes != NULL) &&
						( fWalker != NULL )  &&
						( fWalker <= fBytes + fSize ) &&	
						( fWalker >= fBytes ),
						"TextIterator_AC: Range Error." );
	}
	else
	{
		::Assertion_AC(IsValid(), "TextIterator_AC: Range Error.");
	}
}

inline void CByteIterator_AC::AssertValidAddress(const char* inAddress) const
{
	RangeCheckWalker(false);
	::Assertion_AC(	((inAddress >= fBytes) && (inAddress < (fBytes + fSize))),
					"TextIterator_AC: Invalid Index");
}
#endif

inline void  CByteIterator_AC::SetBuffer(void* bytes, long inSize)
{
	fBytes = static_cast<char*>(bytes);
	fSize = inSize;
	fWalker = fBytes;
}

inline CByteIterator_AC& CByteIterator_AC::operator = (const CByteIterator_AC& inIter)
{	
	fBytes = inIter.fBytes; 
		// point at same buffer
	fWalker = inIter.fWalker;
		// point into same place in buffer.
	fSize = inIter.fSize;
		
	return *this; 
}

inline bool CByteIterator_AC::IsAtEnd() const
{
#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif

	return fWalker >= fBytes + fSize;
}

inline CByteIterator_AC::CByteIterator_AC(void* bytes, long size)
{
	SetBuffer(bytes, size);
}

inline CByteIterator_AC::CByteIterator_AC(const CByteIterator_AC& inIter)
{	
	*this = inIter;
}
	 
inline CByteIterator_AC::CByteIterator_AC()
	: fBytes(NULL), fSize(0), fWalker(NULL)
{
} 

inline CByteIterator_AC::~CByteIterator_AC()
{
}

inline void CByteIterator_AC::operator += (long inAmt)
{ 	
	fWalker += inAmt; 
	
#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::operator -= (long inAmt) 
{ 	
	fWalker -= inAmt; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::operator ++ ()
{	
	fWalker ++; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::operator -- ()
{	
	fWalker --; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void  CByteIterator_AC::operator ++ (int)
{
	fWalker ++; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void  CByteIterator_AC::operator -- (int)
{
	fWalker --; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline CByteIterator_AC::operator char* () const 
{ 
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif
	return fWalker; 
}

inline long CByteIterator_AC::GetPosition() const
{	
#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif

	return fWalker - fBytes; 
}

inline long CByteIterator_AC::GetOffsetToEnd() const
{
#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
	
	return IsAtEnd() ? 0 : fBytes + fSize - fWalker;
}

inline char* CByteIterator_AC::operator* () const
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif

	return fWalker;
}

inline void CByteIterator_AC::Offset(long inAmt)
{
	 fWalker += inAmt; 

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::Rewind()
{
	fWalker = fBytes;

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::SetToEnd()
{
	fWalker = fBytes+fSize;

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::SetToLast()
{
	fWalker = fBytes+fSize-1;

#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline void CByteIterator_AC::SetPosition(long inAmt)
{

	fWalker = fBytes + inAmt;
	
#if qPatternsDebug_AC
	RangeCheckWalker(true);
#endif
}

inline char* CByteIterator_AC::GetBuffer()
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif

	return fWalker;
}

inline char CByteIterator_AC::operator + (unsigned long inLong) const
{

#if qPatternsDebug_AC
	AssertValidAddress( fWalker + inLong );
#endif
			
	return *(fWalker + inLong);
}

inline char CByteIterator_AC::operator - (unsigned long inLong) const
{
#if qPatternsDebug_AC
	AssertValidAddress( fBytes - inLong );
#endif
			
	return *(fWalker - inLong);
}

inline char CByteIterator_AC::operator[] (long inLong) const
{
#if qPatternsDebug_AC
	AssertValidAddress( fBytes + inLong );
#endif
	return fBytes[inLong];
}

inline char& CByteIterator_AC::operator[] (long inLong)
{
#if qPatternsDebug_AC
	AssertValidAddress( fBytes + inLong );
#endif
	return fBytes[inLong];
}

inline void CByteIterator_AC::operator = (char inChar)
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif
	*fWalker = inChar;
}

inline bool CByteIterator_AC::operator == (char inChar) const
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif
	return *fWalker == inChar;	
}

inline bool CByteIterator_AC::operator != (char inChar) const
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif
	return *fWalker != inChar;	
}

inline long CByteIterator_AC::GetBufferSize() const
{
	return fSize;
}

inline CByteIterator_AC::operator char() const
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif
	return *fWalker;
}

inline char* CByteIterator_AC::GetPositionPtr() const
{
#if qPatternsDebug_AC
	RangeCheckWalker(false);
#endif

	return fWalker;	
}

#endif		