// CDependencies.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __CDependencies_AC__
#define __CDependencies_AC__

// ACS

#ifndef __Core_AC__
#include "Core_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CList__AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator_AC__
#include "CListIterator_AC.h"
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

//----------------------------------------------------------------------------------------
/*	Theory of Operation
	-------------------
	
	The dependency relationships are stored in two arrays of triples, where each
	triple contains a primary object, a secondary object and a label. The arrays are
	sorted first by the primary object.
	
	One array, gDependents, contains notifier-dependent relationships, so the primary 
	object is a notifier and the secondary objects are dependents. For example, 
	suppose A has two dependents C and D, each of which has B as a dependent. 
	The dependents array looks as follows:
	
		A	C 	label
		A	D 	label
		C	B 	label
		D	B	label
	
	The other array, gNotifiers, contains dependent-notifier relationships, so the 
	primary object is a dependent and the secondary objects are notifiers:

		B	C 	label
		B	D 	label
		C	A 	label
		D	A	label
	
	The labels are used for marking objects, though they may have other uses in future.
	
	Iteration over the secondary objects for a given primary object is performed by 
	doing a binary search for a matching primary object, then iterating backwards to 
	find the beginning of the cluster of entries with that primary object. Iteration then
	proceeds forwards until the primary object changes. To iterate over the dependents of 
	an object we use gDependents, and to iterate over its notifiers we use gNotifiers.
	
	These data structures are not particularly space-efficient, but allow fast iteration 
	over both notifiers and dependents. Each entry is only twelve bytes, so you should be 
	able to have hundreds of dependencies with no problem.*/

//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------
enum
{
	kNoLabel_AC = LONG_MAX - 1
};

//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------
typedef long DependencyLabels_AC;
	// The valid range of labels

typedef long LabelRange_AC;
	// Used internally because we flip the label's sign to mark an object

//----------------------------------------------------------------------------------------
// SNotification_AC
//----------------------------------------------------------------------------------------
struct SNotification_AC
{
	inline SNotification_AC() { }
	
	inline SNotification_AC(MDependable_AC* notifier,
				 ChangeID_AC theChange,
				 void* changeData) :
	fNotifier(notifier),
	fChange(theChange),
	fChangeData(changeData)
	{ }		

	MDependable_AC* fNotifier;
	void* fChangeData;
	ChangeID_AC fChange;
};

//----------------------------------------------------------------------------------------
// CNotificationQueue_AC
//----------------------------------------------------------------------------------------
class EXPORT_AC CNotificationQueue_AC : public CDynamicArray_AC
{
public:

	inline CNotificationQueue_AC() : CDynamicArray_AC(1, sizeof(SNotification_AC)) { }

	virtual ~CNotificationQueue_AC();
		
	//------------------------------------------------------------------------------------
	// Insertion and Deletion
	//------------------------------------------------------------------------------------

	inline void Queue(const SNotification_AC& item) { InsertElementsBefore(1, &item, 1); }
		// FIFO queue addition.(same as insertLast)

	bool Dequeue(SNotification_AC& item);
		// FIFO queue relief
};

//----------------------------------------------------------------------------------------
// SDependency_AC
//----------------------------------------------------------------------------------------
struct SDependency_AC
{
	inline SDependency_AC(const MDependable_AC* primaryObject,
				MDependable_AC* secondaryObject,
				LabelRange_AC dependencyLabel) :
	fPrimaryObject(primaryObject),
	fSecondaryObject(secondaryObject),
	fLabel(dependencyLabel)
	{ }
		// Constructor

	const MDependable_AC* fPrimaryObject;
	MDependable_AC* fSecondaryObject;
	LabelRange_AC fLabel;
};

//----------------------------------------------------------------------------------------
// CDependencies_AC: An array for storing the dependency relationships
//----------------------------------------------------------------------------------------
class EXPORT_AC CDependencies_AC : public CSortedDynamicArray_AC
{
public:

	inline CDependencies_AC() : CSortedDynamicArray_AC(1, sizeof(SDependency_AC)) { }
		// constructor

	virtual ~CDependencies_AC();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Insertion and Deletion
	//------------------------------------------------------------------------------------

	virtual void AddDependency(MDependable_AC* primaryObject,
									  MDependable_AC* secondaryObject,
									  DependencyLabels_AC dependencyLabel);
		// Register an object ("secondaryObject") as dependent on the specified change in
        // another object ("primaryObject")

	virtual void InsertDependencyBefore(ArrayIndex_AC index,
											   MDependable_AC* primaryObject,
											   MDependable_AC* secondaryObject,
											   DependencyLabels_AC dependencyLabel);
		// Add a dependency relation to the array

	virtual void RemoveDependency(MDependable_AC* primaryObject,
										 MDependable_AC* secondaryObject,
										 DependencyLabels_AC dependencyLabel);
		// Remove a dependency relation from the array if it is present


	//------------------------------------------------------------------------------------
	// Searching
	//------------------------------------------------------------------------------------

	virtual CompareResult_AC CompareElements(const void* Element1,
											 const void* Element2) const;
		// Orders the elements of the array by the ordinal value of the notifier.

	virtual bool FindDependency(MDependable_AC* primaryObject,
										  MDependable_AC* secondaryObject,
										  DependencyLabels_AC dependencyLabel,
										  ArrayIndex_AC& index) const;
		// Returns true if the specified dependency is in the array. The position of the
        // dependency is returned in "index". If the dependency is not present the
        // position at which it should be inserted is returned in "index"

	virtual bool FindPrimaryObject(const MDependable_AC* primaryObject, ArrayIndex_AC& index) const;
		// Returns true if a binary search for the specified primary object succeeds.
		// The index of the found entry is also returned.

	inline const MDependable_AC* PrimaryObjectAt(ArrayIndex_AC i) const { return static_cast<SDependency_AC*>(ComputeAddress(i))->fPrimaryObject; }
		// Returns the primary object at a given index

	inline MDependable_AC* SecondaryObjectAt(ArrayIndex_AC i) const { return static_cast<SDependency_AC*>(ComputeAddress(i))->fSecondaryObject; }
		// Returns the secondary object at a given index
};

//----------------------------------------------------------------------------------------
// CDependencySpace_AC: Abstract Superclass of SDependency_AC Spaces
//----------------------------------------------------------------------------------------
class CDependencySpace_AC;
class EXPORT_AC CAutoPtr_AC<CDependencySpace_AC>;

class EXPORT_AC CDependencySpace_AC
{
public:
	inline CDependencySpace_AC() { }
		// Empty constructor

	virtual ~CDependencySpace_AC();
		// Destructor
	
	virtual void AddDependency(MDependable_AC* notifier,
									  MDependable_AC* dependent,
									  DependencyLabels_AC dependencyLabel);
		// Register an object ("dependent") as a dependent of another object ("Notifier"),
		// with specified label (can be kNoLabel_AC)
	
	virtual void RemoveDependency(MDependable_AC* notifier,
										 MDependable_AC* dependent,
										 DependencyLabels_AC dependencyLabel);
		// Remove the dependency relation if it is present
	
	virtual void RemoveDependencies(MDependable_AC* anObject);
		// Remove all dependencies containing the object, either as Notifier or dependent.
		// Called by MDependable_AC::Free.

	virtual void NotifierChanged(MDependable_AC* notifier,
										ChangeID_AC theChange,
							 			void* changeData);
		// Notifies dependents of "notifier" that it has changed

	static CAutoPtr_AC<CDependencySpace_AC> fgDefaultDependencySpace;
};

//----------------------------------------------------------------------------------------
// CSimpleDependencySpace_AC: Used to store dependency relationships and propagate
// changes to immediate dependents. ACS has a single global instance fgDefaultDependencySpace
//----------------------------------------------------------------------------------------
class EXPORT_AC CSimpleDependencySpace_AC : public CDependencySpace_AC
{
public:
	CSimpleDependencySpace_AC() { }
		// Constructor
		
	virtual ~CSimpleDependencySpace_AC();
		// Free this object
		
	virtual void AddDependency(MDependable_AC* notifier,
									  MDependable_AC* dependent,
									  DependencyLabels_AC dependencyLabel);	// Override
		// Register an object ("dependent") as a dependent of another object ("Notifier"),
		// with specified label (can be kNoLabel_AC)
	
	virtual void RemoveDependency(MDependable_AC* notifier,
										 MDependable_AC* dependent,
										 DependencyLabels_AC dependencyLabel);	// Override
		// Remove the dependency relation if it is present


	virtual void RemoveDependencies(MDependable_AC* anObject);	// Override
		// Remove all dependencies containing the object, either as Notifier or dependent.
		// Called by MDependable_AC::Free.

	virtual void NotifierChanged(MDependable_AC* notifier,
										ChangeID_AC theChange,
							 			void* changeData);	// Override
		// Notifies direct dependents of "notifier" that it has changed
		
	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
public:

	CDependencies_AC fDependents;
	CDependencies_AC fNotifiers;
};

//----------------------------------------------------------------------------------------
// CStandardDependencySpace_AC: Can be used to store dependency relationships and propagate
// changes to immediate and indirect dependents. 
//----------------------------------------------------------------------------------------

class EXPORT_AC CStandardDependencySpace_AC : public CSimpleDependencySpace_AC
{
public:
	CStandardDependencySpace_AC();
		// Constructor

	virtual ~CStandardDependencySpace_AC();
		// Destructor
				
	virtual void NotifierChanged(MDependable_AC* notifier,
								 ChangeID_AC theChange,
							 	 void* changeData);	// Override
		// Notifies direct and indirect dependents of "notifier" that it has changed
		// Handles nested calls to Changed. 

	void NotifyDependents(MDependable_AC* notifier,
						  ChangeID_AC theChange,
						  void* changeData);
		// Notifies direct and indirect dependents of "notifier" that it has changed
		// Does not handle nested calls to Changed. 
	
	virtual void MarkAllDependents(MDependable_AC* notifier, bool state);
		// Marks all the direct and indirect dependents of "notifier"

	virtual void MarkObject(MDependable_AC* anObject, bool state);	// Override
		// Used to mark or unmark an object as needing updating. Used during a change/update
		// phase
	
	virtual bool IsObjectMarked(MDependable_AC* anObject);	// Override
		// Returns true if the specified object is currently marked

	virtual void UpdateMarkedObjects(ChangeID_AC theChange,
									 MDependable_AC* changedObject,
									 void* changeData);
		// Updates all marked objects by calling HandleUpdate

	virtual void HandleUpdate(MDependable_AC* anObject,
							  ChangeID_AC theChange,
							  MDependable_AC* changedObject,
							  void* changeData);
		// If "anObject" is marked as needing updating, unmark it, update all its notifiers,
        // then update the object itself by calling its DoUpdate method
		
	virtual void UpdateNotifiers(MDependable_AC* anObject,
								 ChangeID_AC theChange,
								 MDependable_AC* changedObject,
								 void* changeData);
		// Recursively updates each notifier of "anObject"

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
public:
	CNotificationQueue_AC fNestedUpdates;					// Queue of updates to be processed
	bool				fUpdating;						// An update is in progress
};

//----------------------------------------------------------------------------------------
// Iterators 
//----------------------------------------------------------------------------------------

class EXPORT_AC CDependencyIterator_AC : public CArrayIterator_AC
{
  public:
	CDependencyIterator_AC(const CDependencies_AC* theArray,
										 const MDependable_AC* theObject);
  	
	//
	// Required functions 
	//
	MDependable_AC* First();
	inline MDependable_AC* Current() const { return fCurrent; }
	MDependable_AC* Next();

	//
	// Convenience functions for C++meisters
	//
	inline MDependable_AC* operator++() { return Next(); }
	inline MDependable_AC* operator++(int) { MDependable_AC* returnVal = Current(); Next(); return returnVal; }
	inline operator MDependable_AC*() const { return Current(); }
	inline MDependable_AC* operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline MDependable_AC* FirsMDependable() { return First(); }
	inline MDependable_AC* CurrenMDependable() const { return Current(); }
	inline MDependable_AC* NexMDependable() { return Next(); }

  protected:
	MDependable_AC* fCurrent;
	const MDependable_AC* fObject;	// The primary object
};

class EXPORT_AC CDependentIterator : public CDependencyIterator_AC
{
  public:
  	CDependentIterator(const MDependable_AC* theObject,
									   CSimpleDependencySpace_AC* dependencySpace);
};


class EXPORT_AC CNotifierIterator_AC : public CDependencyIterator_AC
{
  public:
  	CNotifierIterator_AC(const MDependable_AC* theObject, CSimpleDependencySpace_AC* dependencySpace);
};


#endif // __UDEPENDENCIES__
