//----------------------------------------------------------------------------------------
// CReactor_AC.h
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __CReactor_AC__
#define __CReactor_AC__

// core
#ifndef __Patterns_AC__
#include "Patterns_AC.h"
#endif

#ifndef __CAutoPtr_AC__
#include "CAutoPtr_AC.h"
#endif

#ifndef __CIdleTimeDispenser_AC__
#include "CIdleTimeDispenser_AC.h"
#endif

// forward references
class CEvent_AC;
class MEventTransceiver_AC;

template <class T> class CLinkedList_AC;

//----------------------------------------------------------------------------------------
// CReactor_AC is a basically an event dispatching object. The Reactor has a list of 
// senders and receivers (which are MEventTransceiver_AC objects). Event receiver has
// registered to received events from the sender. The receiver will only receive events
// from the sender it has registered with. The only exception to this is if filtering
// is turned off, in which case all receivers will receive every event sent through 
// the reactor.
//----------------------------------------------------------------------------------------
	
class EXPORT_AC CReactor_AC : public CIdleTimeDispenser_AC
{
public:
	CReactor_AC();
		// ctor
		
	virtual ~CReactor_AC();
		// dtor
		
// event posting

	virtual void PostEvent( CEvent_AC* inEvent, MEventTransceiver_AC* inSender);
		// Send event to all MEventTransceivers who have registered for events from inTransceiver. This
		// sends inEvent and inTransceiver to the registered MEventTransceiver's ReceiveEvent method.			
		// Nested posts are supported. Note that both inEvent and inSender may be deleted if their 
		// ref count goes to zero.

// event registration
	
	// Note: registration is generally handled through the MEventTransceiver_AC api, i.e. RegisterForEventsFrom 
	// and UnregisterForEventsFrom.
	
	virtual void RegisterForEvents( MEventTransceiver_AC* inReceiver, MEventTransceiver_AC* inTransceiver);
		// RegisterForEvents prevents duplicate registrations and prevents registrations
		// where inReceiver == inTransceiver.

	
	virtual void Unregister( MEventTransceiver_AC* inReceiver, MEventTransceiver_AC* inTransceiver);
		// Remove the dependency - inReceiver will stop recieving events from inTransceiver.
		
	virtual void UnregisterSender(MEventTransceiver_AC* inTransceiver);
		// All Receivers recieving events from inTransceiver will cease to Receive events
		// from inTransceiver.

	virtual void UnregisterReceiver(MEventTransceiver_AC* inTransceiver);
		// Tell Reactor to no longer send any events to this object
		
	virtual void Unregister(MEventTransceiver_AC* inTransceiver);
		// Tell Reactor to remove all dependencies where inTrasceiver is either a sender or a receiver.

	virtual void UnregisterAll();	
		// Remove all dependencies. 

	void SetDoFilter(bool doFilter);
		// filtering is on by default
		
	bool IsFiltering() const;

// Implementation

	bool IsRegisteredDependency(MEventTransceiver_AC* inReceiver, MEventTransceiver_AC* inTransceiver);
					
protected:	
	
	class CEventDependency
	{	
	public:
		CEventDependency(MEventTransceiver_AC* receiver, MEventTransceiver_AC* sender) :
			fReceiver(receiver), fSender(sender) 
		{}
		CEventDependency(const CEventDependency& inDependency) 
		{ *this = inDependency; }
		CEventDependency() 
			: fSender(NULL), fReceiver(NULL) 
		{}
		CEventDependency& operator = (const CEventDependency& inDependency) 
		{	fSender = inDependency.fSender; fReceiver = inDependency.fReceiver; return *this; }
		bool operator == (const CEventDependency& inDependency) const
		{	return fSender == inDependency.fSender && fReceiver == inDependency.fReceiver; }
		
		MEventTransceiver_AC* fSender;
		MEventTransceiver_AC* fReceiver;
	};

	typedef class CLinkedList_AC<CEventDependency> CDependencyList;
	
// member data
	CAutoPtr_AC<CDependencyList>	fDependencies;
	bool fFiltering;
		
private:

// Disallow these operations.
	CReactor_AC(const CReactor_AC& /*in*/);
	CReactor_AC& operator = (const CReactor_AC& /*in*/);

};

// inline methods

inline void CReactor_AC::SetDoFilter(bool doFilter)
{
	fFiltering = doFilter;
}
	
inline bool CReactor_AC::IsFiltering() const
{
	return fFiltering;
}


#endif


