//----------------------------------------------------------------------------------------
// CStream_AC.cp 
// Copyright  1990-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein.  Used by permission 
//----------------------------------------------------------------------------------------

#ifndef __CStream_AC__
#include "CStream_AC.h"
#endif

#ifndef __CStreamContext_AC__
#include "CStreamContext_AC.h"
#endif

//========================================================================================
// CLASS CStream_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CStream_AC constructor
//----------------------------------------------------------------------------------------

CStream_AC::CStream_AC(CStreamRep_AC* inRep)
 	: fClassMap(NULL),
	fClassMapSize(0),
	fFilters(NULL),
	fStreamRep(inRep)
{
	AssertNotNULL_AC(inRep);
}

//----------------------------------------------------------------------------------------
// CStream_AC destructor
//----------------------------------------------------------------------------------------

CStream_AC::~CStream_AC()
{
	MLinkable_AC<CDataFilter_AC>::DeleteList(fFilters);
}

//----------------------------------------------------------------------------------------
// CStream_AC::GetWritePosition
//----------------------------------------------------------------------------------------

long CStream_AC::GetWritePosition() const
{
	return fStreamRep->GetWritePosition();
}

//----------------------------------------------------------------------------------------
// CStream_AC::GetReadPosition
//----------------------------------------------------------------------------------------

long CStream_AC::GetReadPosition() const
{
	return fStreamRep->GetReadPosition();
}

//----------------------------------------------------------------------------------------
// CStream_AC::SetReadPosition
//----------------------------------------------------------------------------------------

void CStream_AC::SetReadPosition(long newPosition)
{
	fStreamRep->SetReadPosition(newPosition);
}

//----------------------------------------------------------------------------------------
// CStream_AC::SetWritePosition
//----------------------------------------------------------------------------------------

void CStream_AC::SetWritePosition(long newPosition)
{
	fStreamRep->SetWritePosition(newPosition);
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadBytes
//----------------------------------------------------------------------------------------

void CStream_AC::ReadBytes(void* inBytes, long inAmt)
{
	fStreamRep->ReadBytes(inBytes, inAmt);
	FilterReadBytes(inBytes, inAmt);
}

//----------------------------------------------------------------------------------------
// CStream_AC::PipeOutputTo
//----------------------------------------------------------------------------------------

void CStream_AC::PipeOutputTo(CStream_AC* inStream)
{
	if (fOutputPipes.IsNULL())
		fOutputPipes = TH_new CStreamList;
		
	fOutputPipes->Push(inStream);
}

//----------------------------------------------------------------------------------------
// CStream_AC::BreakPipeWith
//----------------------------------------------------------------------------------------

void CStream_AC::BreakPipeWith(CStream_AC* inStream)
{
	if (!fOutputPipes.IsNULL())
		fOutputPipes->Remove(inStream);
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteBytes
//----------------------------------------------------------------------------------------

void CStream_AC::WriteBytes(const void* inBytes, long inAmt)
{
	const void* bytes = FilterWriteBytes(inBytes, inAmt);
	
	fStreamRep->WriteBytes(bytes, inAmt);
	PipeBytes(bytes, inAmt);
}

//----------------------------------------------------------------------------------------
// CStream_AC::PipeBytes
//----------------------------------------------------------------------------------------

void CStream_AC::PipeBytes(const void* inBytes, long inAmt)
{
	if (!fOutputPipes.IsNULL())
	{
		CStreamList::CIterator iter = fOutputPipes->First();
	
		while (iter.IsValid())
		{
			(*iter)->WriteBytes(inBytes, inAmt);
			iter.Next();
		}
	}
}

//----------------------------------------------------------------------------------------
// CStream_AC::Flush
//----------------------------------------------------------------------------------------

CStream_AC& CStream_AC::Flush()
{
	fStreamRep->Flush();

	if (!fOutputPipes.IsNULL())
	{
		CStreamList::CIterator iter = fOutputPipes->First();
	
		while (iter.IsValid())
		{
			(*iter)->Flush();
			iter.Next();
		}
	}

	return *this;
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteStreamObject: 
//----------------------------------------------------------------------------------------

void CStream_AC::WriteStreamObject(MStreamable_AC* inData,bool stdObject)
{
	long sizePosition;
	short delimiter = kClassIDAndClassname;
	LocalObjectNumber itsObjectNumber;
	bool newEntry = false;

	if (!inData)
		delimiter = kNullObject;
	else
	{
		if (stdObject)
			delimiter = kSignatureAndClassname;

		CStreamContext_AC * itsContext = GetContext();
		if (itsContext)
		{
			itsObjectNumber = itsContext->Add(inData, kDontForceReplacement, newEntry);
			if (!newEntry)
				delimiter = kLocalObjectNumber;
		}
	}

	WriteInteger(delimiter);

	if (delimiter == kNullObject)
		;										// write nothing
	else if (delimiter == kLocalObjectNumber)
		WriteBytes(&itsObjectNumber, sizeof(itsObjectNumber));
	else
	{
		if (delimiter == kClassIDAndClassname)
			WriteObject(inData, sizePosition);	// Write the object header. 
		else
			WriteStdObject(inData, sizePosition);	// Write the object header. 

		if (inData)								// Write the private inData. 
			inData->WriteTo(this);

		WriteObjectSize(sizePosition);			// Write the object size. 
	}
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteStdObject: 
//----------------------------------------------------------------------------------------

void CStream_AC::WriteStdObject(MStreamable_AC* inData, long& sizePosition)
{
	IDType itsSignature = inData ? inData->GetStandardSignature() : kNoIdentifier_AC;

	WriteIDType(itsSignature);					// Write the signature. 

	sizePosition = GetWritePosition();				// Return the position of the object size

	WriteLong(LONG_MAX);						// Write a dummy size (maximum object size).

	CStr255_AC className;
	if (inData)									// inData is not NULL
	{
		className = typeid(*inData).name();			// Get the actual class name. 
		if (itsSignature != kNoIdentifier_AC)
		{
			const CClassRegistration_AC* signatureClassDesc = CClassRegistration_AC::GetClassDescFromSignature(itsSignature);
			if (signatureClassDesc && className == signatureClassDesc->GetName())
				className.Empty();				// then no need for class name in stream
		}
	}

	WriteString(className);						// Write the class name (which may be empty).
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteObject: 
//----------------------------------------------------------------------------------------

void CStream_AC::WriteObject(MStreamable_AC* inData, long& sizePosition)
{
	long streamID = inData ? CClassRegistration_AC::GetClassDescFromClassName(typeid(*inData).name())->GetClassID() : kNilClass_AC;

	WriteLong(streamID);						// Write the class ID. 

	sizePosition = GetWritePosition();				// Remember the position of the object size
	// in the stream.

	WriteLong(LONG_MAX);						// Write a dummy size (maximum object size).

	CStr255_AC className;
	if ((inData) && (LookupClassID(streamID) == kNilClass_AC))
	// inData is not NULL, and this is the first object of the class. 
	{
		className = typeid(*inData).name();			// Get the actual class name. 

		// Register the ID, so we don't write the class name the next time
		//	we write an object of this class.  The second parameter
		//	doesn't matter,  as long as it isn't kNilClass_AC.
		RegisterClassID(streamID, streamID);
	}

	WriteString(className);						// Write the class name (which may be empty).
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteObjectSize: 
//----------------------------------------------------------------------------------------

// This fills in the object's size at the position passed in. 
void CStream_AC::WriteObjectSize(long sizePosition)
{
	long currentPosition = GetWritePosition();		// Remember where we are. 
	SetWritePosition(sizePosition);					// Go back to position of size. 

	WriteLong(currentPosition - sizePosition);	// Write the size. 

	SetWritePosition(currentPosition);				// Go back to where we were. 
}


void CStream_AC::WriteHandle(const Handle aHandle)
{
	if (aHandle)
	{
		// write the size of the handle
		Size itsSize = GetHandleSize(aHandle);
		WriteLong(itsSize);

		// Write the contents of the handle. 
		// We definitely need to lock the handle here because
		// the code in Write may move memory before reading
		// the inData. 
		CTempHandleLock_AC tempHandleLock(aHandle);
		::ThrowIfMemError_AC();
		WriteBytes(*aHandle, itsSize);
	}
	else
		WriteLong(-1);
}

//----------------------------------------------------------------------------------------
// CStream_AC::LookupClassID: This returns the new class ID given a class ID found in the
// stream, or kNilClass_AC if it wasn't found. It does a simple linear search.
//----------------------------------------------------------------------------------------

long CStream_AC::LookupClassID(long id) const
{
	long returnVal = kNilClass_AC;

	if (id != static_cast<long>(kNilClass_AC))
		for (short i = 0; i < fClassMapSize; ++i)
			if ((*fClassMap)[i].fOldID == id)
			{
				returnVal = (*fClassMap)[i].fNewID;
				break;
			}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStream_AC::RegisterClassID: This simply adds the mapping to the end of the list. It does
// not check to see if the mapping exists already.
//----------------------------------------------------------------------------------------

void CStream_AC::RegisterClassID(long oldID,
							  long newID)
{
	if (oldID != kNilClass_AC)
	{
		if (!fClassMap)
			fClassMap = (HClassArray) NewPermHandle_AC(0);

		++fClassMapSize;
		SetPermHandleSize_AC((Handle)(HClassArray)fClassMap, fClassMapSize * sizeof(ClassIDEntry));
		

		(*fClassMap)[fClassMapSize - 1].fOldID = oldID;
		(*fClassMap)[fClassMapSize - 1].fNewID = newID;
	}
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadString: 
//----------------------------------------------------------------------------------------

CStr255_AC CStream_AC::ReadString(short maxSize)
{
	CStr255_AC inData;

	ReadBytes(&inData, 1);						// Read the length byte. 

	if (inData.Length() + 1 > maxSize)
		::ThrowIfOSErr_AC(paramErr);					// Can't read that many bytes. 
	else
		ReadBytes(&inData[1], inData.Length());

	return inData;
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadHandle: 
//----------------------------------------------------------------------------------------

Handle CStream_AC::ReadHandle()
{
	Handle returnVal = NULL;

	// read the size of the handle in from the stream
	Size itsSize = ReadLong();

	if (itsSize >= 0)
	{
		// allocate the handle
		CTempHandle_AC aHandle = NewPermHandle_AC(itsSize); 

		// Lock the handle and read into it from the stream. 
		// We definitely need to lock the handle here because
		// the code in Read may move memory before reading
		// the inData. 
		CTempHandleLock_AC tempHandleLock(aHandle);
		::ThrowIfMemError_AC();
		ReadBytes(*(Handle)aHandle, itsSize);

		returnVal = aHandle.release();
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadStdObject: 
//----------------------------------------------------------------------------------------

MStreamable_AC* CStream_AC::ReadStdObject(bool& wasValid)
{
	wasValid = false;

	IDType itsSignature = ReadIDType();				// Read the object's signature. 

	long sizePosition = GetReadPosition();				// Read its size and remember the current position.
	long objectSize = ReadLong();

	CStr255_AC className = ReadString(sizeof(className));		// Read its name. 

	MStreamable_AC* returnVal = NewObjectBySignature_AC(itsSignature, className);

	if (returnVal)
		wasValid = true;
	else										// Don't know about the class. 
		{
			// If the streamID is kNilClass_AC then the object was realy
			//	NULL, which is a known object, otherwise the class
			//	isn't available.
			wasValid = false;

			// Skip over the object's inData. 
			SetReadPosition(sizePosition + objectSize);// Skip over its inData. 
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadObject: 
//----------------------------------------------------------------------------------------

MStreamable_AC* CStream_AC::ReadObject(bool& wasValid)
{
	MStreamable_AC* returnVal = NULL;
	wasValid = false;

	long currentID;									// Class ID in program. 

	long streamID = ReadLong();						// Read the object's class ID in stream.

	long sizePosition = GetReadPosition();				// Read its size and remember the current position.
	long objectSize = ReadLong();

	CStr255_AC className = ReadString(sizeof(className));		// Read its name. 

	if (className.IsEmpty())					// No class name. Must have seen the class
	// before, so convert the streamID to the
	// currentID.
		currentID = LookupClassID(streamID);
	else
	{											// Get ID from name ...
		currentID = CClassRegistration_AC::GetClassDescFromClassName(className)->GetClassID();
		RegisterClassID(streamID, currentID);	// ... and register the IDs. 
	}

	if (currentID == kNilClass_AC)					// Don't know about the class. 
	{
		// If the streamID is kNilClass_AC then the object was realy
		//	NULL, which is a known object, otherwise the class
		//	isn't available.
		wasValid = streamID == kNilClass_AC;

		// Skip over the object's inData. 
		SetReadPosition(sizePosition + objectSize);	// Skip over its inData. 
	}
	else
	{											// Create the object. 
		returnVal = CClassRegistration_AC::NewByClassID((ClassID_AC)currentID);
		wasValid = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStream_AC::ReadStreamObject: 
//----------------------------------------------------------------------------------------

MStreamable_AC* CStream_AC::ReadStreamObject(bool& wasValid)
{
	MStreamable_AC* returnVal = NULL;

	wasValid = false;

	short delimiter;
	LocalObjectNumber itsObjectNumber;
	CStreamContext_AC * itsContext = GetContext();

	delimiter = ReadInteger();

	if (delimiter == kNullObject)
	{
		returnVal = NULL;
		wasValid = true;
	}
	else if (delimiter == kLocalObjectNumber)
	{
		ReadBytes(&itsObjectNumber, sizeof(itsObjectNumber));
		if (itsContext)
			returnVal = itsContext->Find(itsObjectNumber);
#if qDebugMsg
		else
			::ProgramBreak_AC("###Need a CStreamContext_AC in CStream_AC::ReadStreamObject.");
#endif

		if (returnVal)
			wasValid = true;
	}
	else
	{
		if (delimiter == kClassIDAndClassname)
			returnVal = ReadObject(wasValid);			// Create an uninitialized object. 
		else if (delimiter == kSignatureAndClassname)
			returnVal = ReadStdObject(wasValid);		// Create an uninitialized object by signature. 
#if qDebugMsg
		else
			::ProgramBreak_AC("###Unknown delimiter in CStream_AC::ReadStreamObject.");
#endif

		if (wasValid)
		{
			bool newEntry = false;
			if (itsContext)
				itsObjectNumber = itsContext->Add(returnVal, kDontForceReplacement, newEntry);

			try
			{
				if (returnVal)						// If we got an object, then tell it to initialize itself.
					returnVal->ReadFrom(this);
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				delete returnVal;					// we don't want ReadFrom/Fields to delete object, we will
				returnVal = NULL;					// make sure that we return a null null reference.
				throw;
			}
			
		}
	}
	return returnVal;
}

//----------------------------------------------------------------------------------------
// CStream_AC::AddFilter
//----------------------------------------------------------------------------------------

void CStream_AC::AddFilter(CDataFilter_AC* inFilter)
{
	AssertNotNULL_AC(inFilter);
	fFilters = inFilter->Append(fFilters);
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteString
//----------------------------------------------------------------------------------------

void CStream_AC::WriteString(const char* inCString)
{	
	const char* strPtr = inCString;
	long len = ::strlen(strPtr);
	CDataFilter_AC* walker = fFilters; 
	while (walker)	
	{	
		strPtr = walker->FilterOutput(strPtr, len);
			// get new ptr from filter if any.	
		walker = walker->GetNextLink();	
	}	
	WriteBytes(strPtr, len);
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteString
//----------------------------------------------------------------------------------------

void CStream_AC::WriteString(const unsigned char* inPString)
{	
	const unsigned char* str = inPString;
	long len = *str;
	CDataFilter_AC* walker = fFilters; 
	while (walker)	
	{	
		str = walker->FilterOutput(str);	
		walker = walker->GetNextLink();	
	}	
	WriteBytes(str, *str);
}

//----------------------------------------------------------------------------------------
// CStream_AC::WriteString
//----------------------------------------------------------------------------------------

void CStream_AC::WriteString(const CPascalStr_AC& inStr)
{		
	const unsigned char* str = inStr;
	CDataFilter_AC* walker = fFilters; 
	while (walker)	
	{	
		str = walker->FilterOutput(str);	
		walker = walker->GetNextLink();	
	}	
	WriteBytes(str, *str + 1);
}

//----------------------------------------------------------------------------------------
// CStream_AC::RemoveFilter
//----------------------------------------------------------------------------------------

void CStream_AC::RemoveFilter(CDataFilter_AC* inFilter)
{
	if (fFilters)
	{
		fFilters = fFilters->Remove(inFilter);
	}
}
