// UBackgroundAdorner.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.

/*
	Change History:
		07/25/97	When updating with platinum appearance we were not respecting the window's content
					region color. We needed to set the background color for the window when focusing.
					However, to allow drawing environments to provide alternate coloration we also needed
					to call TView::SetupDrawingEnvironment() when focusing. TWhiteBackgroundAdorner installs
					a DrawingEnvironment to cause the color environment to change to the desired one.
*/

#pragma once

#ifndef __UBACKGROUNDADORNER__
#define __UBACKGROUNDADORNER__

// ACS

#ifndef __CControl_AC__
#include "CControl_AC.h"
#endif

#ifndef __AGAImaging_AC__
#include "AGAImaging_AC.h"
#endif

// MacApp

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

EXPORT_MA void InitUBackgroundAdorner();

#pragma mark TBackgroundAdorner

//----------------------------------------------------------------------------------------
// TBackgroundAdorner
// An adorner that you should attach to typical modeless windows if you
// want the standard AGA gray background. It should be placed BEFORE the
// "Draw Adorner".
//----------------------------------------------------------------------------------------

class EXPORT_MA TBackgroundAdorner : public TAdorner
{
	MA_DECLARE_CLASS;

public:

	TBackgroundAdorner();

	// MacApp overrides
	virtual void DoHighlightSelection(TView* itsView, const CViewRect& area, HLState fromHL, HLState toHL);
	virtual void Draw(TView* itsView, const CViewRect& area);
	virtual void ViewChangedFrame(TView* itsView, const CViewRect& oldFrame, const CViewRect& newFrame, bool	invalidate);
	virtual void AddedToView(TView* itsView);

protected:

	virtual void DrawBackground(TView* itsView, const CViewRect& area, bool fill, bool active);
	virtual bool HasGrowBox(TView* itsView);

	AGABackgroundKind	fBackgroundKind;
};

#pragma mark TModalBackgroundAdorner

//----------------------------------------------------------------------------------------
// TModalBackgroundAdorner
// An adorner that you should attach to typical modal windows (dialogs)
// in order to get the standard AGA modal gray background. It should be placed
// BEFORE the "Draw Adorner".
//
// Usage note: This adorner draws the raised gray background in the window
// content region. Some window kinds (notably dBoxProc and movableDBoxProc
// for modal dialogs) pad the window frame with a few pixels around the edge
// of the content area. This adorner will leave a white ring around the usable
// content area, because you can't draw outside it.
// If you want a solid gray background with these kinds of windows, use an
// TFlatBackgroundBehavior attached to the window. It will install the
// gray background into the window's color table, so that gray is drawn in
// the area outside the content region.
//----------------------------------------------------------------------------------------

class EXPORT_MA TModalBackgroundAdorner : public TBackgroundAdorner
{
	MA_DECLARE_CLASS;

public:

	TModalBackgroundAdorner();
};

#pragma mark TFlatBackgroundBehavior

//----------------------------------------------------------------------------------------
// TFlatBackgroundBehavior
// TBehavior subclass that handles DoPostCreate by installing a gray
// background color in the owner's window color table.
// This MUST be only attached to a TWindow or TView in a window.
// The reason we use a behavior rather than an adorner, is that adorners
// don't get notified in a timely fashion when they are attached to the
// host view, while behaviors get a DoPostCreate call, which is at the
// right time to install the color.
//----------------------------------------------------------------------------------------

class EXPORT_MA TFlatBackgroundBehavior : public TBehavior
{
	MA_DECLARE_CLASS;

public:

	TFlatBackgroundBehavior();
	virtual ~TFlatBackgroundBehavior();

	// MacApp overrides
	virtual void DoPostCreate(TDocument* itsDocument);
};

#pragma mark TWhiteBackgroundAdorner

//----------------------------------------------------------------------------------------
// TWhiteBackgroundAdorner
// An adorner that you should attach to any subview where the window has
// a gray background but the particular subview needs a white background.
// Editable text fields and list boxes are the most common examples. It should
// be placed BEFORE the "Draw Adorner".
//----------------------------------------------------------------------------------------

class EXPORT_MA TWhiteBackgroundAdorner : public TAdorner
{
	MA_DECLARE_CLASS;

public:
	//------------------------------------------------------------------------------------
	// notification
	//------------------------------------------------------------------------------------
	virtual void AddedToView(TView* itsView);
		// Notification that this adorner has been added to a view

	virtual void RemovedFromView(TView* itsView);
		// Notification that this adorner has been removed from a view

	// other MacApp overrides
	virtual void Draw(TView* itsView, const CViewRect& area);
};

#pragma mark TNotchedWhiteBackgroundAdorner

//----------------------------------------------------------------------------------------
// TNotchedWhiteBackgroundAdorner
// Same as TWhiteBackgroundAdorner, but leaves a "notch" in bottom
// right corner for area created by a pair of scroll bars.
//----------------------------------------------------------------------------------------

class EXPORT_MA TNotchedWhiteBackgroundAdorner : public TAdorner
{
	MA_DECLARE_CLASS;

public:

	// MacApp overrides
	virtual void Draw(TView* itsView, const CViewRect& area);
};

#pragma mark TBorderFrameAdorner

//----------------------------------------------------------------------------------------
// TBorderFrameAdorner
// An adorner that you should attach to any subview where the subview
// needs a "3D" sunken frame around it. Editable text fields and list
// boxes are the most common examples. It should be placed AFTER the
// "Draw Adorner".
//----------------------------------------------------------------------------------------

class EXPORT_MA TBorderFrameAdorner : public TAdorner
{
	MA_DECLARE_CLASS;

public:

	// MacApp overrides
	virtual void DoHighlightSelection(TView* itsView, const CViewRect& area, HLState fromHL, HLState toHL);
	virtual void Draw(TView* itsView, const CViewRect& area);
};

#pragma mark TNotchedBorderFrameAdorner

//----------------------------------------------------------------------------------------
// TNotchedBorderFrameAdorner
// An adorner that you should attach to any subview where you want a
// "3D" sunken frame around it, contoured to fit the "notch" at the
// bottom right of a pair of scroll bars.
//----------------------------------------------------------------------------------------

class EXPORT_MA TNotchedBorderFrameAdorner : public TAdorner
{
	MA_DECLARE_CLASS;

public:

	// MacApp overrides
	virtual void DoHighlightSelection(TView* itsView, const CViewRect& area, HLState fromHL, HLState toHL);
	virtual void Draw(TView* itsView, const CViewRect& area);
};

#endif // __UBACKGROUNDADORNER__
