//----------------------------------------------------------------------------------------
// UQuickTime.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UQUICKTIME__
#include "UQuickTime.h"
#endif

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

//Toolbox

#ifndef __COMPONENTS__
#include <Components.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif


const CommandNumber cPlayEveryFrame = 1001;
const CommandNumber cHalfSize = 1002;
const CommandNumber cNormalSize = 1003;
const CommandNumber cDoubleSize = 1004;
const CommandNumber cLoop = 1005;
const CommandNumber cAutoRewind = 1006;
const CommandNumber cPlayWhenInactive = 1007;

//----------------------------------------------------------------------------------------
// Prototype

void FailComponentErr(ComponentResult theResult);

//----------------------------------------------------------------------------------------
// InitUQuickTime
//----------------------------------------------------------------------------------------

void InitUQuickTime()
{
	if (!HasQuickTime_AC())						// balk if QuickTime isn't present
	{
#if qDebug
		ProgramBreak_AC("InitUQuickTime called but QuickTime isn't present.");
#endif

		return;
	}

	// Initialize the QuickTime movie environment
	::ThrowIfOSErr_AC(EnterMovies());
}

//----------------------------------------------------------------------------------------
// FailComponentErr
//----------------------------------------------------------------------------------------

void FailComponentErr(ComponentResult theResult)
{
	if (theResult)
		ProgramBreak_AC("A ComponentResultError has occured.");
}

//========================================================================================
// CLASS TMovie
//========================================================================================
#undef Inherited

// MA_DEFINE_CLASS(TMovie);

//----------------------------------------------------------------------------------------
// TMovie destructor:
//----------------------------------------------------------------------------------------

TMovie::~TMovie()
{
	if (fMovie)
		DisposeMovie(fMovie);

	fMovie = NULL;
}

//----------------------------------------------------------------------------------------
// TMovie constructor
//----------------------------------------------------------------------------------------

TMovie::TMovie(TFile* file) :
	fMovie(NULL),
	fPlayEveryFrame(false)
{
	//
	//	make a movie from the file
	//
	file->OpenFile();							// in case it's not already open

	Movie newMovie;
	ResNumber actualResID = static_cast<ResNumber>(DoTheRightThing);
	::ThrowIfOSErr_AC(NewMovieFromFile(&newMovie, file->GetRsrcRefNum(), &actualResID, NULL, newMovieActive, NULL));
	fMovie = newMovie;

	//
	// spruce up the movie bounds
	//
	CRect_AC aRect(GetBox());
	if (aRect.Empty())
		aRect.Set(20, 100, 200, 116);		// use a default size in this case

	fOriginalBox = aRect - aRect[topLeft];		// make sure the top left is 0,0
	SetBox(fOriginalBox);
}

//----------------------------------------------------------------------------------------
// TMovie::GetDuration
//----------------------------------------------------------------------------------------

TimeValue TMovie::GetDuration() const
{
	// ??? in seconds?
	return GetMovieDuration(fMovie) / GetMovieTimeScale(fMovie);
}

//----------------------------------------------------------------------------------------
// TMovie::GetPoster
//----------------------------------------------------------------------------------------

PicHandle TMovie::GetPoster() const
{
	PicHandle picHandle = GetMoviePosterPict(fMovie);
	CRect_AC picFrame = (*picHandle)->picFrame;

	//	If the picture frame is empty, return a NULL handle
	if (picHandle && picFrame.Empty())
		picHandle = (PicHandle)DisposeIfHandle_AC((Handle)picHandle);

	return picHandle;
}

//----------------------------------------------------------------------------------------
// TMovie::GetBox
//----------------------------------------------------------------------------------------

CRect_AC TMovie::GetBox() const
{
	CRect_AC itsBox;
	GetMovieBox(fMovie, &itsBox);

	return itsBox;
}

//----------------------------------------------------------------------------------------
// TMovie::SetBox
//----------------------------------------------------------------------------------------

void TMovie::SetBox(const CRect_AC& itsBox)
{
	SetMovieBox(fMovie, itsBox);
}

//----------------------------------------------------------------------------------------
// TMovie::PutOnScrap
//----------------------------------------------------------------------------------------

OSErr TMovie::PutOnScrap(long movieScrapFlags)
{
	return PutMovieOnScrap(fMovie, movieScrapFlags);
}

//----------------------------------------------------------------------------------------
// TMovie::IsDone
//----------------------------------------------------------------------------------------

bool TMovie::IsDone() const
{
	return IsMovieDone(fMovie);
}

//----------------------------------------------------------------------------------------
// TMovie::IsPlaying
//----------------------------------------------------------------------------------------

bool TMovie::IsPlaying() const
{
	return (fPlayEveryFrame || GetMovieRate(fMovie)) && !IsDone();
}

//----------------------------------------------------------------------------------------
// TMovie::Start
//----------------------------------------------------------------------------------------

void TMovie::Start()
{
	if (!fPlayEveryFrame)
		StartMovie(fMovie);
}

//----------------------------------------------------------------------------------------
// TMovie::Stop
//----------------------------------------------------------------------------------------

void TMovie::Stop()
{
	if (!fPlayEveryFrame)
		StopMovie(fMovie);
}

//----------------------------------------------------------------------------------------
// TMovie::SetPlayEveryFrame
//----------------------------------------------------------------------------------------

void TMovie::SetPlayEveryFrame(bool playEveryFrame)
{
	fPlayEveryFrame = playEveryFrame;
}

//========================================================================================
// CLASS TMovieViewIdler
//========================================================================================
#undef Inherited
#define Inherited TEventHandler

MA_DEFINE_CLASS(TMovieViewIdler);

//----------------------------------------------------------------------------------------
// TMovieViewIdler Constructor
//----------------------------------------------------------------------------------------

TMovieViewIdler::TMovieViewIdler(TMovieView* itsMovieView) :
	TEventHandler(NULL),

	fMovieView(itsMovieView)
{
}

//----------------------------------------------------------------------------------------
// TMovieViewIdler destructor
//----------------------------------------------------------------------------------------

TMovieViewIdler::~TMovieViewIdler()
{
}

//----------------------------------------------------------------------------------------
// TMovieViewIdler::DoIdle
//----------------------------------------------------------------------------------------

bool TMovieViewIdler::DoIdle(IdlePhase phase)
{
	// if no longer playing then I got freed
	return fMovieView && phase == idleContinue ? !fMovieView->IdleMovies() : Inherited::DoIdle(phase);
}

//========================================================================================
// CLASS TMovieView
//========================================================================================
#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(TMovieView);

//----------------------------------------------------------------------------------------
// TMovieView Constructor
//----------------------------------------------------------------------------------------

TMovieView::TMovieView() :
	fMovie(NULL),
	fMovieController(NULL),
	fPlayWhenInactive(false),
	fWasPlaying(false),
	fMovieClipIsValid(false),
	fLoop(false),
	fAutoRewind(false),
	fMovieControllerIdentifier(kNoIdentifier)
{
}

//----------------------------------------------------------------------------------------
// TMovieView Destructor
//----------------------------------------------------------------------------------------

TMovieView::~TMovieView()
{
	if (fIdler)
		gApplication->InstallCohandler(fIdler, false);

	if (fMovieController)
		fMovieController->AttachMovieView(NULL);
	fMovieController = NULL;
}

//----------------------------------------------------------------------------------------
// TMovieView::DoPostCreate
//----------------------------------------------------------------------------------------

void TMovieView::DoPostCreate(TDocument* itsDocument)
{
	fMovieControllerIdentifier = 'MCtl';		//### temporary until new template is available

	Inherited::DoPostCreate(itsDocument);

	fLoop = false;
	fAutoRewind = false;

	fMovieSize[hSel] = normalSize;
	fMovieSize[vSel] = normalSize;

	SetMovieSize(fMovieSize[hSel], fMovieSize[vSel]);
	if (fSuperView)
	{
		if (fMovieControllerIdentifier != kNoIdentifier)
		{
			TMovieController * itsController = dynamic_cast<TMovieController*>(GetRootView()->FindSubView(fMovieControllerIdentifier));
			if (itsController)
				itsController->AttachMovieView(this);
#if qDebugMsg
			else
			{
				IDType itsID = fMovieControllerIdentifier;
				ProgramBreak_AC("###In TMovieView::DoPostCreate, can't find MovieController: %.4s\n", &itsID);
			}
#endif

		}
	}
}

//----------------------------------------------------------------------------------------
// TMovieView::HaveMovie
//----------------------------------------------------------------------------------------

void TMovieView::HaveMovie(TMovie* movie)
{
	fMovie = movie;

	Focus();

	if (fMovie)
		SetMovieGWorld(fMovie->GetMovie(), (CGrafPtr)GetGrafPort(), NULL);

	if (fMovieController)
		fMovieController->HaveMovie(fMovie);

	AdjustFrame();
	if (fSuperView)
		fSuperView->AdjustFrame();

	SetMovieSize(fMovieSize[hSel], fMovieSize[vSel]);
}

//----------------------------------------------------------------------------------------
// TMovieView::SetLoop
//----------------------------------------------------------------------------------------

void TMovieView::SetLoop(bool loop)
{
	fLoop = loop;

	if (fMovieController)
		FailComponentErr(MCDoAction(fMovieController->GetController(), mcActionSetLooping, (Ptr)fLoop));
}

//----------------------------------------------------------------------------------------
// TMovieView::SetMovieSize
//----------------------------------------------------------------------------------------

void TMovieView::SetMovieSize(VariableMovieSize newSizeH,
							  VariableMovieSize newSizeV)
{
	fMovieSize[hSel] = newSizeH;
	fMovieSize[vSel] = newSizeV;

	if (GetMovie())
	{
		CRect_AC box(GetMovie()->GetOriginalBox());

		// Adjust view size based on size determiners. 
		for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
		{
			if (fSizeDeterminer[vhs] == sizeVariable)
				switch (fMovieSize[vhs])
				{
					case halfSize:
						box[botRight][vhs] /= 2;
						break;

					case doubleSize:
						box[botRight][vhs] *= 2;
						break;
				}
			else
				box[botRight][vhs] = fSize[vhs] - 1;
		}

		SetFrame(CViewRect(box), kInvalidate);
	}
}

//----------------------------------------------------------------------------------------
// TMovieView::SetPlayWhenInactive
//----------------------------------------------------------------------------------------

bool TMovieView::SetPlayWhenInactive(bool shouldPlay)
{
	bool returnVal = fPlayWhenInactive;

	if (fPlayWhenInactive != shouldPlay)
	{
		fPlayWhenInactive = shouldPlay;

		if (!fIsActive && fWasPlaying)
			fMovie->Start();

		IdleMovies();
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMovieView::SetFrame
//----------------------------------------------------------------------------------------

void TMovieView::SetFrame(const CViewRect& newFrame,
						  bool invalidate)
{
	if (newFrame != GetFrame())
	{
		Inherited::SetFrame(newFrame, invalidate);

		if (invalidate)
			ForceRedraw();

		AdjustOffsetsForController();

		if (fMovieController)
			fMovieController->MovieChangedFrame(newFrame, invalidate);

	}
}

//----------------------------------------------------------------------------------------
// TMovieView::BeInPort
//----------------------------------------------------------------------------------------

void TMovieView::BeInPort(GrafPtr itsPort)		// override 
{
	Inherited::BeInPort(itsPort);

	if (fMovie)
		SetMovieGWorld(fMovie->GetMovie(), (CGrafPtr)itsPort, NULL);
}

//----------------------------------------------------------------------------------------
// TMovieView::Draw
//----------------------------------------------------------------------------------------

void TMovieView::Draw(const CViewRect&				/*area*/ )
{
	if (fMovie)
	{
		bool wasActive = true;
		if (!IsPlaying())
		{
			wasActive = GetMovieActive(fMovie->GetMovie());
			if (wasActive)
				SetMovieActive(fMovie->GetMovie(), true);
		}

		UpdateMovie(fMovie->GetMovie());
		MoviesTask(fMovie->GetMovie(), 0);

		if (!IsPlaying())
		{
			if (!wasActive)
				SetMovieActive(fMovie->GetMovie(), wasActive);
		}
	}
}

//----------------------------------------------------------------------------------------
// TMovieView::Focus
//----------------------------------------------------------------------------------------

bool TMovieView::Focus()
{
	bool returnVal = Inherited::Focus();

	//	keep the movie up to date with our coordinate sys only if absolutely necessary
	if (returnVal && !fMovieClipIsValid)
	{
		CTempRegion_AC visRgn;
		GetVisibleRegion(visRgn);

		if (fMovieController && fMovieController->GetController())
			FailComponentErr(MCSetClip(fMovieController->GetController(), NULL, visRgn));
		else if (fMovie)
			SetMovieDisplayClipRgn(fMovie->GetMovie(), visRgn);

		fMovieClipIsValid = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMovieView::InvalidateMovieClip
//----------------------------------------------------------------------------------------

void TMovieView::InvalidateMovieClip()
{
	fMovieClipIsValid = false;
}

//----------------------------------------------------------------------------------------
// TMovieView::EnsureMovieClipIsValid
//----------------------------------------------------------------------------------------

void TMovieView::EnsureMovieClipIsValid()
{
	if (!fMovieClipIsValid)
		Focus();
}

//----------------------------------------------------------------------------------------
// TMovieView::Activate
//----------------------------------------------------------------------------------------

void TMovieView::Activate(bool entering)
{
	Inherited::Activate(entering);

	if (entering != fIsActive)
	{
		fIsActive = entering;

		if (!fPlayWhenInactive)
			if (entering)
			{
				if (fWasPlaying)
					fMovie->Start();
			}
			else
			{
				fWasPlaying = IsPlaying();
				fMovie->Stop();
			}
		IdleMovies();
	}
}

//----------------------------------------------------------------------------------------
// TMovieView::HaveMovieController
//----------------------------------------------------------------------------------------

void TMovieView::HaveMovieController(TMovieController* itsMovieController)
{
	fMovieController = itsMovieController;

	AdjustOffsetsForController();

	if (fMovieController && fMovie)
		fMovieController->HaveMovie(fMovie);
}

//----------------------------------------------------------------------------------------
// TMovieView::AdjustOffsetsForController
//----------------------------------------------------------------------------------------

void TMovieView::AdjustOffsetsForController()
{
	if (fMovieController && Focus())
	{
		// reset to zero every time
		SetLocalOrigin(gZeroVPt, kDontRedraw);

		// calculate the distance between us and the controller
		CRect_AC movieQDRect(GetQDExtent());

		CPoint_AC movieControllerQDOffset(fMovieController->ViewToQD(fMovieController->RootViewToLocal(LocalToRootView(QDToView(movieQDRect[topLeft])))));

		if (fMovie)
			fMovie->SetBox(movieQDRect + movieControllerQDOffset);

		SetLocalOrigin(CViewPoint(gZeroPt - movieControllerQDOffset), kDontRedraw);

		if (fMovie)
			MCMovieChanged(fMovieController->GetController(), fMovie->GetMovie());

		InvalidateMovieClip();
	}
}

//----------------------------------------------------------------------------------------
// TMovie::HasMovie
//----------------------------------------------------------------------------------------

bool TMovieView::HasMovie() const
{
	//not totally safe - could have a movie object with no fMovie...change
	return fMovie ? true : false;
}
//----------------------------------------------------------------------------------------
// TMovieView::DoMouseCommand
//----------------------------------------------------------------------------------------

void TMovieView::DoMouseCommand(CViewPoint& theMouse,
								TToolboxEvent* event,
								CPoint_AC hysteresis)
{
	// if a movieController is attached pass off mouse clicks to it
	if (IsEnabled())
	{
		if (fMovieController)
		{
			if (fMovieController->Focus())
				fMovieController->DoMouseCommand(fMovieController->RootViewToLocal(LocalToRootView(theMouse)), event, hysteresis);
		}
		else
		{
			if (event->GetClickCount() > 1)
			{
				if (fMovie->IsDone())
					GoToBeginningOfMovie(fMovie->GetMovie());
				fMovie->Start();
			}
			else
				fMovie->Stop();
			IdleMovies();
		}
	}
	else
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TMovieView::SetIdleIfMoviePlaying
//----------------------------------------------------------------------------------------

bool TMovieView::SetIdleIfMoviePlaying()
{
	const long kMagicIdleNumber = 1;

	if (fLoop && fMovie && fMovie->IsDone())
		GoToBeginningOfMovie(fMovie->GetMovie());

	if (fAutoRewind && fMovie && fMovie->IsDone())
	{
		GoToBeginningOfMovie(fMovie->GetMovie());
		IdleMovies();						// since we will stop the idling process the frame wouldn't redraw
	}

	bool returnVal = IsPlaying();

	if (returnVal)
	{
		if (!fIdler)
		{
			fIdler = TH_new TMovieViewIdler(this);
			gApplication->InstallCohandler(fIdler, true);
		}
		fIdler->SetIdleFreq(kMagicIdleNumber);
	}
	else
	{
		if (fIdler)
			fIdler->SetIdleFreq(kMaxIdleTime);
	}

	return returnVal;
}
//----------------------------------------------------------------------------------------
// TMovieView::IsPlaying
//----------------------------------------------------------------------------------------

bool TMovieView::IsPlaying() const
{
	bool returnVal = false;

	if (fMovie)
		returnVal = fMovie->IsPlaying();

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMovieView::IdleMovies
//----------------------------------------------------------------------------------------

bool TMovieView::IdleMovies()
{
	if (fMovieController)
		fMovieController->IdleMovies();
	else
	{
		if (fMovie && Focus())
			MoviesTask(fMovie->GetMovie(), 0);
	}

	return SetIdleIfMoviePlaying();
}


//----------------------------------------------------------------------------------------
// TMovieView::UpdateCoordinates
//----------------------------------------------------------------------------------------

void TMovieView::UpdateCoordinates()
{
	Inherited::UpdateCoordinates();

	InvalidateMovieClip();
}

//----------------------------------------------------------------------------------------
// TMovieView::IsActive
//----------------------------------------------------------------------------------------

bool TMovieView::IsActive() const
{
	return fIsActive && Inherited::IsActive();
}

//----------------------------------------------------------------------------------------
// TMovieView::ComputeFrame
//----------------------------------------------------------------------------------------

void TMovieView::ComputeFrame(CViewRect& newFrame)
{
	Inherited::ComputeFrame(newFrame);

	if (fMovie)
	{
		CRect_AC box(fMovie->GetBox());

		if (fSizeDeterminer[hSel] == sizeVariable)
			newFrame.right = newFrame.left + box.GetLength(hSel);

		if (fSizeDeterminer[vSel] == sizeVariable)
			newFrame.bottom = newFrame.top + box.GetLength(vSel);
	}
}

//----------------------------------------------------------------------------------------
// TMovieView::DoMenuCommand
//----------------------------------------------------------------------------------------

void TMovieView::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	switch (aCommandNumber)
	{
		case cAutoRewind:
			fAutoRewind = !fAutoRewind;
			break;

		case cLoop:
			SetLoop(!fLoop);
			break;

		case cPlayWhenInactive:
			SetPlayWhenInactive(!fPlayWhenInactive);
			break;

		case cHalfSize:
		case cNormalSize:
		case cDoubleSize:
			switch (aCommandNumber)
			{
				case cHalfSize:
					SetMovieSize(halfSize, halfSize);
					break;

				case cNormalSize:
					SetMovieSize(normalSize, normalSize);
					break;

				case cDoubleSize:
					SetMovieSize(doubleSize, doubleSize);
					break;
			}
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//========================================================================================
// CLASS TMovieController
//========================================================================================
#undef Inherited
#define Inherited TView

MA_DEFINE_CLASS(TMovieController);

//----------------------------------------------------------------------------------------
// TMovieController Constructor
//----------------------------------------------------------------------------------------

TMovieController::TMovieController() :
	fMovieController(NULL),
	fMovieView(NULL),
	fIsActive(false),
	fPlayEveryFrame(false),
	fAutoSizeToMovie(true)
{
}

//----------------------------------------------------------------------------------------
// TMovieController Destructor
//----------------------------------------------------------------------------------------

TMovieController::~TMovieController()
{
	if (fMovieView)
		fMovieView->HaveMovieController(NULL);
	fMovieView = NULL;

	if (fMovieController)
		CloseComponent(fMovieController);
	fMovieController = NULL;
}


//----------------------------------------------------------------------------------------
// TMovieController::DoPostCreate
//----------------------------------------------------------------------------------------

void TMovieController::DoPostCreate(TDocument* itsDocument)
{
	Inherited::DoPostCreate(itsDocument);

	fMovieController = (MovieController)OpenDefaultComponent(MovieControllerComponentType, kAnyComponentSubType);
	::ThrowIfNULL_AC(fMovieController);

	BeInPort(GetGrafPort());
	FailComponentErr(MCSetVisible(fMovieController, fShown));
	FailComponentErr(MCDoAction(fMovieController, mcActionSetKeysEnabled, (Ptr)TRUE));

	SetPlayEveryFrame(fPlayEveryFrame);
}

//----------------------------------------------------------------------------------------
// TMovieController::SetFrame
//----------------------------------------------------------------------------------------

void TMovieController::SetFrame(const CViewRect& newFrame,
								bool invalidate)
{
	Inherited::SetFrame(newFrame, invalidate);

	if (invalidate)
		ForceRedraw();

	if (fMovieController)
		FailComponentErr(MCSetControllerBoundsRect(fMovieController, &GetQDExtent()));

	if (fMovieView)
		fMovieView->AdjustOffsetsForController();
}

//----------------------------------------------------------------------------------------
// TMovieController::Show
//----------------------------------------------------------------------------------------

void TMovieController::Show(bool state,
							bool redraw)
{
	Inherited::Show(state, redraw);

	FailComponentErr(MCSetVisible(fMovieController, state));
}

//----------------------------------------------------------------------------------------
// TMovieController::BeInPort
//----------------------------------------------------------------------------------------

void TMovieController::BeInPort(GrafPtr itsPort)// override 
{
	Inherited::BeInPort(itsPort);

	if (fMovieController)
		FailComponentErr(MCSetControllerPort(fMovieController, (CGrafPtr)itsPort));
}

//----------------------------------------------------------------------------------------
// TMovieController::IdleMovies
//----------------------------------------------------------------------------------------

void TMovieController::IdleMovies()
{
	if (Focus())
		FailComponentErr(MCIdle(fMovieController));
}

//----------------------------------------------------------------------------------------
// TMovieController::DoEvent
//----------------------------------------------------------------------------------------

void TMovieController::DoEvent(EventNumber eventNumber,
							   TEventHandler* source,
							   TEvent* event)
{
	if (eventNumber == mBecameTarget)
		Activate(true);
	else if (eventNumber == mResignedTarget)
		Activate(false);

	Inherited::DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TMovieController::DoMouseCommand
//----------------------------------------------------------------------------------------

void TMovieController::DoMouseCommand(CViewPoint& theMouse,
									  TToolboxEvent* event,
									  CPoint_AC hysteresis)
{
	bool handled = false;

	if (IsEnabled())
	{
		handled = MCClick(fMovieController, GetGrafPort(), ViewToQD(theMouse), event->GetWhen(), event->GetModifiers());
		if (handled)
		{
			if (fMovieView)
				fMovieView->IdleMovies();
			else
				IdleMovies();
		}
	}

	if (!handled)
		Inherited::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TMovieController::DoKeyEvent
//----------------------------------------------------------------------------------------

void TMovieController::DoKeyEvent(TToolboxEvent* event)// override 
{
	bool handled = false;

	if (IsEnabled() && Focus())
	{
		handled = MCKey(fMovieController, event->GetText()[1], event->GetModifiers());
		if (event->IsDoubleEvent())
			handled = MCKey(fMovieController, event->GetText()[2], event->GetModifiers(1));

		if (handled)
		{
			if (fMovieView)
				fMovieView->IdleMovies();
			else
				IdleMovies();
		}
	}

	if (!handled)
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TMovieController::DoCommandKeyEvent
//----------------------------------------------------------------------------------------

void TMovieController::DoCommandKeyEvent(TToolboxEvent* event)// override 
{
	bool handled = false;

	if (IsEnabled() && Focus())
	{
		handled = MCKey(fMovieController, event->GetText()[1], event->GetModifiers());
		if (event->IsDoubleEvent())
			handled = MCKey(fMovieController, event->GetText()[2], event->GetModifiers(1));

		if (handled)
		{
			if (fMovieView)
				fMovieView->IdleMovies();
			else
				IdleMovies();
		}
	}

	if (!handled)
		Inherited::DoCommandKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TMovieController::Draw
//----------------------------------------------------------------------------------------

void TMovieController::Draw(const CViewRect& /*area*/ )
{
	// I only want to draw the controller not the associated movie. (It will draw itself)
	// So I set the movie's clip to empty in order to defeat the drawing of the movie that
	// would otherwise occur here. When finished I restore the original movie clipping.

	RgnHandle oldClip;
	FailComponentErr(MCGetClip(fMovieController, NULL, &oldClip));

	CTempRegion_AC emptyClip;
	SetEmptyRgn(emptyClip);
	FailComponentErr(MCSetClip(fMovieController, qd.thePort->clipRgn, emptyClip));

	FailComponentErr(MCDraw(fMovieController, GetGrafPort()));

	FailComponentErr(MCSetClip(fMovieController, NULL, oldClip));
}

//----------------------------------------------------------------------------------------
// TMovieController::Focus
//----------------------------------------------------------------------------------------

bool TMovieController::Focus()
{
	if (fMovieView)
		fMovieView->EnsureMovieClipIsValid();

	return Inherited::Focus();
}

//----------------------------------------------------------------------------------------
// TMovieController::Activate
//----------------------------------------------------------------------------------------

void TMovieController::Activate(bool entering)
{
	Inherited::Activate(entering);

	if (entering != fIsActive)
	{
		fIsActive = entering;

		SetActive(entering);
	}
}

//----------------------------------------------------------------------------------------
// TMovieController::SetActive
//----------------------------------------------------------------------------------------

void TMovieController::SetActive(bool state)
{
	Focus();

	FailComponentErr(MCActivate(fMovieController, GetGrafPort(), state));

	// Because Movie ToolBox invalidates the extent of the control but we do
	// direct drawing in the unclipped area.

	CTempRegion_AC visibleRgn;
	GetVisibleRegion(visibleRgn);

	Validate(visibleRgn);

	DrawContents();						// draw in activated or deactivated state
}

//----------------------------------------------------------------------------------------
// TMovieController::AttachMovieView
//----------------------------------------------------------------------------------------

void TMovieController::AttachMovieView(TMovieView* itsMovieView)
{
	fMovieView = itsMovieView;
	if (fMovieView)
		fMovieView->HaveMovieController(this);
}

//----------------------------------------------------------------------------------------
// TMovieController::HaveMovie
//----------------------------------------------------------------------------------------

void TMovieController::HaveMovie(TMovie* itsMovie)
{
	FailComponentErr(MCSetMovie(fMovieController, itsMovie ? itsMovie->GetMovie() : NULL, fMovieView ? fMovieView->GetGrafPort() : GetGrafPort(), gZeroPt));
	FailComponentErr(MCSetControllerAttached(fMovieController, false));
	FailComponentErr(MCDoAction(fMovieController, mcActionSetLooping, fMovieView ? (Ptr)fMovieView->fLoop : false));

	SetPlayEveryFrame(fPlayEveryFrame);

	AdjustFrame();
	if (fSuperView)
		fSuperView->AdjustFrame();
}

//----------------------------------------------------------------------------------------
// TMovieController::SetIdleIfMoviePlaying
//----------------------------------------------------------------------------------------

bool TMovieController::SetIdleIfMoviePlaying()
{
	return fMovieView ? fMovieView->SetIdleIfMoviePlaying() : false;
}

//----------------------------------------------------------------------------------------
// TMovieController::IsPlaying
//----------------------------------------------------------------------------------------

bool TMovieController::IsPlaying() const
{
	return fMovieView ? fMovieView->IsPlaying() : false;
}

//----------------------------------------------------------------------------------------
// TMovieController::IsActive
//----------------------------------------------------------------------------------------

bool TMovieController::IsActive() const
{
	return fIsActive && Inherited::IsActive();
}

//----------------------------------------------------------------------------------------
// TMovieController::MovieChangedFrame
//----------------------------------------------------------------------------------------

void TMovieController::MovieChangedFrame(const CViewRect&/*newFrame*/  ,
										 bool/*invalidate*/ )
{
	AdjustFrame();
	if (fSuperView)
		fSuperView->AdjustFrame();
}

//----------------------------------------------------------------------------------------
// TMovieController::UpdateCoordinates
//----------------------------------------------------------------------------------------

void TMovieController::UpdateCoordinates()
{
	Inherited::UpdateCoordinates();

	if (fMovieView)
		fMovieView->InvalidateMovieClip();
}


//----------------------------------------------------------------------------------------
// TMovieController::SetAutoSizeToMovie
//----------------------------------------------------------------------------------------

void TMovieController::SetAutoSizeToMovie(bool autoSize)
{
	fAutoSizeToMovie = autoSize;
}


//----------------------------------------------------------------------------------------
// TMovieController::SetPlayEveryFrame
//----------------------------------------------------------------------------------------

void TMovieController::SetPlayEveryFrame(bool playEveryFrame)
{
	fPlayEveryFrame = playEveryFrame;

	bool wasPlaying = false;
	if (fMovieView && fMovieView->GetMovie())
	{
		fMovieView->GetMovie()->SetPlayEveryFrame(fPlayEveryFrame);
		wasPlaying = fMovieView->IsPlaying();
	}

	if (wasPlaying)
		fMovieView->GetMovie()->Stop();

	FailComponentErr(MCDoAction(fMovieController, mcActionSetPlayEveryFrame, (Ptr)fPlayEveryFrame));

	if (wasPlaying)
		fMovieView->GetMovie()->Start();
}

//----------------------------------------------------------------------------------------
// TMovieController::ComputeFrame
//----------------------------------------------------------------------------------------

void TMovieController::ComputeFrame(CViewRect& newFrame)
{
	Inherited::ComputeFrame(newFrame);

	if (fAutoSizeToMovie && fMovieView)
	{
		newFrame = newFrame - newFrame[topLeft];// zero base

		CViewRect movieFrame(fMovieView->GetFrame());
		newFrame.Offset(movieFrame.left, movieFrame.bottom);
		newFrame.right = newFrame.left + movieFrame.GetLength(hSel);
	}
}

//----------------------------------------------------------------------------------------
// TMovieController::DoSetupMenus
//----------------------------------------------------------------------------------------

void TMovieController::DoSetupMenus()			// override 
{
	Inherited::DoSetupMenus();

	if (fMovieView->HasMovie())
	{
		EnableCheck(cLoop, true, fMovieView->fLoop);
		EnableCheck(cAutoRewind, true, fMovieView->fAutoRewind);

		EnableCheck(cPlayWhenInactive, true, fMovieView->fPlayWhenInactive);

		EnableCheck(cHalfSize, true, fMovieView->fMovieSize[hSel] == halfSize || fMovieView->fMovieSize[vSel] == halfSize);
		EnableCheck(cNormalSize, true, fMovieView->fMovieSize[hSel] == normalSize || fMovieView->fMovieSize[vSel] == normalSize);
		EnableCheck(cDoubleSize, true, fMovieView->fMovieSize[hSel] == doubleSize || fMovieView->fMovieSize[vSel] == doubleSize);

		EnableCheck(cPlayEveryFrame, true, fPlayEveryFrame);
	}
}

//----------------------------------------------------------------------------------------
// TMovieController::DoMenuCommand
//----------------------------------------------------------------------------------------

void TMovieController::DoMenuCommand(CommandNumber aCommandNumber)// override 
{
	switch (aCommandNumber)
	{
			//
			// Forward some cases to the MovieView
			//
		case cAutoRewind:
		case cLoop:
		case cPlayWhenInactive:
		case cHalfSize:
		case cNormalSize:
		case cDoubleSize:
			fMovieView->DoMenuCommand(aCommandNumber);
			break;

		case cPlayEveryFrame:
			SetPlayEveryFrame(!fPlayEveryFrame);
			break;

		default:
			Inherited::DoMenuCommand(aCommandNumber);
			break;
	}
}

//----------------------------------------------------------------------------------------
// End of UQuickTime.cp


