//----------------------------------------------------------------------------------------
// UAreaSelector.cp
// Copyright  1991-96 by Apple Computer, Inc. All rights reserved. 
//----------------------------------------------------------------------------------------

#ifndef __UAREASELECTOR__
#include "UAreaSelector.h"
#endif

#ifndef __UFRACVIEWS__
#include "UFracViews.h"
#endif

#ifndef __UFRACDOC__
#include "UFracDoc.h"
#endif

#ifndef __CThread_AC__
#include "CThread_AC.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

//-------------------------------------------------------------------------------------------


/* Initialize the selector object itself by calling ICommand. Also set the boolean that says
  we constrain the mouse, so that our TrackConstrain method gets called. */

#undef Inherited
#define Inherited TTracker

TAreaSelector::TAreaSelector(TFracAppView* itsView, TFracAppDocument* itsDocument,
										 const CViewPoint& theMouse) :
	TTracker(cMouseCommand, itsDocument, kCantUndo, kDoesNotCauseChange, itsDocument,
			 itsView, itsView->GetScroller(kAnySuperView), theMouse)
{
	
	fConstrainsMouse = true;					// do the constrain to match to screen.

	fOwnerView = itsView;						// save the view for use in tracking.
	fPlotWidth = itsDocument->GetPlotWidth();
	fPlotHeight = itsDocument->GetPlotHeight();
}

TAreaSelector::TAreaSelector()
{
}

//-------------------------------------------------------------------------------------------


/* Track the mouse while the button is down. This is overridden so we can make sure that the
  selection is cleared when we start tracking, and properly set when we are done tracking. */

TTracker* TAreaSelector::TrackMouse(TrackPhase aTrackPhase, CViewPoint& anchorPoint,
										   CViewPoint& /*previousPoint*/, CViewPoint& nextPoint,
										   bool /*mouseDidMove*/)
{
	switch (aTrackPhase)
	{
		case trackBegin:
			{
				fOwnerView->SetSelection(gZeroRect, kRedraw); // clear rect, there isnt one.
			}
			break;

		case trackEnd:
			{
				CRect_AC tempRect;
				::Pt2Rect(anchorPoint.AsPoint(), nextPoint.AsPoint(), tempRect);
				fOwnerView->SetSelection(tempRect, kRedraw);
			}
			break;
	}
	return this; 						// Make sure we return something valid.
}

//-------------------------------------------------------------------------------------------


/* Track the mouse giving the feedback of a different rectangle kind.  This is so we can use
  the selection pattern to give a preferred rectangle.	The selection pattern comes out of
  temporary memory so as to not fail needlessly. */

void TAreaSelector::TrackFeedback(TrackPhase /*aTrackPhase*/, const CViewPoint& anchorPoint,
										 const CViewPoint& /*previousPoint*/, const CViewPoint& nextPoint,
										 bool mouseDidMove, bool /*turnItOn*/)
{
	if (mouseDidMove)
	{
		// the pen is already in patXOR mode, black, one wide
		PatHandle selPatHandle = ::GetPattern(kSelPattern);	// get the pattern we use.
		if (selPatHandle)
		{
			// use our pattern if available.
			Pattern safePat = **selPatHandle;
			::PenPat(&safePat); 	// set pen pattern to our selection kind.
		}
		CRect_AC selBoy;
		::Pt2Rect(anchorPoint.AsPoint(), nextPoint.AsPoint(), selBoy);
		::FrameRect(selBoy);
	}
	CThread_AC::Yield();
}

//-------------------------------------------------------------------------------------------


/* Constrain the mouse to a rectangle that is the same proportion as the document, so we can
  make the selection match better without having to guess at the length or width, or scaling
  the chosen rect to fit the document.	Small piece chosen will blow up to fit easily.	This
  will make it easier to choose a selection that gives a 1:1 aspect ratio.	This also
  chooses which direction the mouse has moved, deciding which is larger in order to decide
  the direction to constrain. */

	inline long abs(long n)		{ return (n >= 0) ? n : -n; }

	static void ChangeWidth(long plotWidth, long plotHeight, const CViewPoint& anchorPoint,
								   CViewPoint& nextPoint)
	{
		// Get the new width as a positive number, a displacement that is constrained.
		long newWidth = abs(nextPoint.v - anchorPoint.v) * plotWidth / plotHeight;

		// Decide which quadrant we are in, moving the right direction.
		if (nextPoint.h < anchorPoint.h)
			newWidth = - newWidth;

		// Actually change the final point to pass back.
		nextPoint.h = anchorPoint.h + newWidth; // add offset to get new pt.
	}

	static void ChangeHeight(long plotWidth, long plotHeight, const CViewPoint& anchorPoint,
									CViewPoint& nextPoint)
	{
		// Get the new height as a positive number, a displacement that is constrained.
		long newHeight = abs(nextPoint.h - anchorPoint.h) * plotHeight / plotWidth;

		// Decide which quadrant we are in, moving the right direction.
		if (nextPoint.v < anchorPoint.v)
			newHeight = - newHeight;

		// Actually change the final point to pass back.
		nextPoint.v = anchorPoint.v + newHeight; // add offset to get new pt.
	}

	// Pin the rectangle to the edge of the document.
	static void PinPoint(long plotWidth, long plotHeight, CViewPoint& nextPoint)
	{
		CViewRect	constrainRect(0, 0, plotWidth, plotHeight);
		nextPoint.ConstrainTo(constrainRect);
	}

void TAreaSelector::TrackConstrain(TrackPhase /*aTrackPhase*/, const CViewPoint& anchorPoint,
										  const CViewPoint& /*previousPoint*/, CViewPoint& nextPoint,
										  bool /*mouseDidMove*/)
{
	long dragHeight = nextPoint.v - anchorPoint.v;
	if (dragHeight == 0)
		dragHeight = 1;
	float mouseRatio = abs((nextPoint.h - anchorPoint.h) / dragHeight);
	float plotRatio = fPlotWidth / fPlotHeight;

	/* The deltaX, deltaY can be thought of as a rect too. If the ratio of sides on
	that rect (width/height) is greater than the ratio of width/height of the
	plot rectangle, then we need to grow the height of the rect. If it is less,
	we need to grow the width. This is a ratio of sides to decide which way to
	grow. We grow to make the new rect still touch the mouse position. It can be
	thought of as the rectangle being thicker than tall wanting to grow the tall
	part in a constrained way, and the corollary for the width. */

	if (mouseRatio > plotRatio)
	{
		// constrain height to new value.
		ChangeHeight(fPlotWidth, fPlotHeight, anchorPoint, nextPoint);
		PinPoint(fPlotWidth, fPlotHeight, nextPoint);
		ChangeWidth(fPlotWidth, fPlotHeight, anchorPoint, nextPoint);
	}
	else
	{
		// constrain width to new value.
		ChangeWidth(fPlotWidth, fPlotHeight, anchorPoint, nextPoint);
		PinPoint(fPlotWidth, fPlotHeight, nextPoint);
		ChangeHeight(fPlotWidth, fPlotHeight, anchorPoint, nextPoint);
	}
}
