//	UEngine.cp

#ifndef __UMEMORY__
#include <UMemory.h>
#endif

#ifndef __FRACAPP__
#include "UFracApp.h"
#endif

#ifndef __UENGINE__
#include "UEngine.h"
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __URECTSTACK__
#include "URectStack.h"
#endif

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

#ifndef __UFRACDOC__
#include "UFracDoc.h"
#endif

// Threads

#ifndef __ThreadUtilities_AC__
#include "ThreadUtilities_AC.h"
#endif

#ifndef __CWhileSynchronized_AC__
#include "CWhileSynchronized_AC.h"
#endif

//-------------------------------------------------------------------------------------------

const short kMinCCRectSize		= 18;				// minimum CalcCity rect size

//-------------------------------------------------------------------------------------------
//---------------------------------CFracAppEngine Methods------------------------------------
//-------------------------------------------------------------------------------------------


CFracAppEngine::CFracAppEngine() :
	fFracAppDocument(NULL),
	fDwells(NULL),
	fLastDwell(0),
	fPaintQueue(NULL),
#if qUseThreads
	fThreadList(NULL),
	fActiveThreadCounter(-1)
#endif
{
}

//-------------------------------------------------------------------------------------------


CFracAppEngine::~CFracAppEngine()
{
#if qUseThreads
	if (!fThreadList.IsNULL()) {
		for (TemplateIterator_AC<CThreadRunner_AC*> iter(fThreadList); iter; ++iter)
		{
			iter->RequestShutdown();
		}
		for (TemplateIterator_AC<CThreadRunner_AC*> iter(fThreadList); iter; ++iter)
		{
			iter->Join();
			
			delete iter.Current();
		}
	}
#endif
}

//-------------------------------------------------------------------------------------------


/* Inits the engine object itself. Remembers a reference to its document, and makes a working
  copy of its FracHeader. This is done for speed so that we dont have to continually query
  the document for its info. */

void CFracAppEngine::IFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth, Byte* dwells)
{
	fFracAppDocument = itsDocument;
	fThreadCount = fFracAppDocument->GetThreadCount();
	fThreadStyle = fFracAppDocument->GetThreadStyle();
	if (fThreadCount > 0) 
		fThreadList = TH_new CList_AC(fThreadCount);

	FracRecord f = fFracAppDocument->GetFracHeader();
	
	CExtended realMin(f.pages.realMin);
	CExtended imagMin(f.pages.imagMin);
	CExtended deltaP(f.deltaP);
	CExtended deltaQ(f.deltaQ);

#if qPowerPC
	fRealMin = realMin;
	fImagMin = imagMin;
	fDeltaP = deltaP;
	fDeltaQ = deltaQ;
#else	
	// SCpp can't figure this out
	fRealMin = *(extended80 *)realMin;
	fImagMin = *(extended80 *)imagMin;
	fDeltaP = *(extended80 *)deltaP;
	fDeltaQ = *(extended80 *)deltaQ;
#endif

	fPlotWidth = f.plotWidth;
	fPlotHeight = f.plotHeight;
	fCalcRect = f.calcRect;

	fDwellWidth = dwellWidth;
	fDwells = dwells;

	fPaintQueue = fFracAppDocument->GetPaintQueue();
}

//-------------------------------------------------------------------------------------------
/* This is the heart and soul of this program. Given a point and some other defining
  constants, this routine figures out what color the point should be. Basically, it works
  like this:

  Mandelbrot fractals are calculated on the complex coordinate plane. This means that
  complex numbers of the form a + ib are ploted in x,y fashion on a two dimensional grid.
  The value a is plotted in the x, or real, direction, and the value b is plotted in the
  y, or imaginary, direction.

  Given a point a + ib, we square it and add a complex constant C = Po + iQo. We then check
  to see how far the result is away from the first point. If it is farther than some limit
  (called M here), we are done. If the result is within that limit, we apply the formula
  again to that result. We keep this process up until we either go outside the limit M, or
  we have performed this process a certain number of times (called the dwell limit). It is
  this number of iterations that determines the color of the pixel. If we exceed our maximum
  number of iterations, we map the color to black.
*/
//-------------------------------------------------------------------------------------------


#if ! qUseAsmComputation
inline short CFracAppEngine::GoFigger(const long double Po, const long double Qo, const short M, const short K)
{
	register short countDown = K;
	register long double radius = M;
	register long double p = Po;
	register long double q = Qo;
	register long double x = p;
	register long double y = q;

	do
	{
		//	Given a point: pt = x + iy,
		//	and a constant: C = Po + iQo,
		//	iteratively calculate pt2 = pt^2 + C
		//							  = (x + iy)^2 + (Po + iQo)
		//							  = (x^2 + 2ixy - y^2) + (Po + iQo)
		//							  = (x^2  - y^2 + Po) + i(2xy + Qo)
		//	until pt2 is at least M units from the original point.
		//	The number of times that it took us to get to this state gets
		//	mapped into the color that we use for that point. If we iterated more
		//	than a certain maximum, then we cap the color to that maximum.

		register long double xx = x * x;
		register long double yy = y * y;
		if (xx + yy > radius) break;

		register long double xy = x * y;

		x = xx - yy + p;
		y = xy + xy + q;

	} while (--countDown > 0);

	return K - countDown + 1;
}
#endif

//-------------------------------------------------------------------------------------------


/* Bottleneck used to inform the document that part of the fractal has been
  freshly calculated and needs to be set in the offscreen bitmap.  If the dwell is
  the same as the last one, we can optimize by trying to combining rectangles
  (often single pixels) into one. */

void CFracAppEngine::PaintDwellRect(Byte dwell, const CRect_AC& r)
{
/*
	if (dwell == fLastDwell)
		fPaintQueue->AddRectSync(r);
	else
	{
		fLastDwell = dwell;
		fPaintQueue->PushRectSync(r);
	}
*/
}

//-------------------------------------------------------------------------------------------


/* Called by the engines CalcCity method. This method takes a coordinate and calculates
  the dwell that it should be by calling GoFigger. */

short CFracAppEngine::PixWhap(short col, short row)
{
	const short M				= 100;			/* This decides what infinity is. If
												 value less than this, then loop. */
	const short K				= kNumColors;	/* Dwell time, ie, the number of
												 iterations we make before giving up. */
	const short kBlackIndex		= 255;			/* entry in our offscreen color table for
												 black. */
//	volatile UnsignedWide startTime, endTime, endTime2, elapsedTime, elapsedTime2;
//	::Microseconds(&startTime);
	
	long double Po = fRealMin + row * fDeltaP; 	// next starting point
	long double Qo = fImagMin + col * fDeltaQ;

	short dwell = this->GoFigger(Po, Qo, M, K);	// go hammer out a pixel color index
	// dwell = short((long(dwell - 1) * kNumColors) / K) + 1;
	if (dwell > kNumColors)
		dwell = kBlackIndex; 					// Clip it to black if were too high.

//	::Microseconds(&endTime);
	this->SetDwell(col, row, dwell);
	
//	::Microseconds(&endTime2);
//	elapsedTime.lo = endTime.lo - startTime.lo;
//	elapsedTime2.lo = endTime2.lo - startTime.lo;

	return dwell;
}

//-------------------------------------------------------------------------------------------


/* Called by the engines CalcCity method. This method takes a rect and calculates
  the dwells for the points that it contains by calling GoFigger. */

short CFracAppEngine::RectWhap(const CRect_AC& r, bool wantsMatch)
{
//	TRACE_SETUP;
//	LOG_ENTRY( "CFracAppEngine:RectWhap" );
	const short M				= 100;			/* This decides what infinity is. If
												 value less than this, then loop. */
	const short K				= kNumColors;	/* Dwell time, ie, the number of
												 iterations we make before giving up. */
	const short kBlackIndex		= 255;			/* entry in our offscreen color table for
												 black. */
//	volatile UnsignedWide startTime, endTime, endTime2, elapsedTime, elapsedTime2;
//	::Microseconds(&startTime);
	
	long double Po; 	// next starting point
	long double Qo;

	long baseIndex;
	long index;
	short dwell, previousDwell = 0;
	bool match = wantsMatch;	// Do all dwells match?
	for (short row = r.top; row <= r.bottom - 1; ++row)
	{
		baseIndex = (long)fDwellWidth * row;
		for (short col = r.left; col <= r.right - 1; ++col)
		{
			index = baseIndex + col;
			if (fDwells[index] == 0) {
				Po = fRealMin + row * fDeltaP; 	// next starting point
				Qo = fImagMin + col * fDeltaQ;
				
				dwell = this->GoFigger(Po, Qo, M, K);	// go hammer out a pixel color index
				// dwell = short((long(dwell - 1) * kNumColors) / K) + 1;
				if (dwell > kNumColors)
					dwell = kBlackIndex; 					// Clip it to black if were too high.

				fDwells[index] = dwell;
			}
			else {
				dwell = fDwells[index];
			}
			if (match) {
				match = ((previousDwell == dwell) || (previousDwell == 0));
//				if (!match) return 0;
				previousDwell = dwell;
			}
//			this->SetDwell(col, row, dwell);
		}
	}
//	::Microseconds(&endTime);
//	this->PaintDwellRect(0, r);
//	::Microseconds(&endTime2);
//	elapsedTime.lo = endTime.lo - startTime.lo;
//	elapsedTime2.lo = endTime2.lo - startTime.lo;

//	LOG_EXIT;
	return (match?dwell:0);
//	return dwell;
}

//-------------------------------------------------------------------------------------------


/* Bottleneck used to inform the document that part of the fractal has been
  freshly calculated and needs to be copied to the screen. */

void CFracAppEngine::ReportRectCompleted(CRect_AC& dirtyRect)
{
	fPaintQueue->AddRectSync(dirtyRect);
}

//-------------------------------------------------------------------------------------------


/* Sets a point in the dwell array to the specified value. */

void CFracAppEngine::SetDwell(short x, short y, short dwell)
{
	long index = (long)fDwellWidth * y + x;
	fDwells[index] = dwell;

	/* Move to the pixel we calculated for, then draw the pixel in right color. This
	  could be done by setting the bytes in pixel map directly, since we own the
	  PixMap and the buffer. However, since most of the applications time is spent
	  calculating the pixel color (ie, number of iterations through the loop), the
	  overhead of using QuickDraw is very small, and we dont get much speed
	  improvement by bypassing it. */
	{
		CRect_AC itsRect;
		itsRect.top = y;
		itsRect.bottom = y + 1;
		itsRect.left = x;
		itsRect.right = x + 1;

		this->PaintDwellRect(dwell, itsRect);
	}
}

//-------------------------------------------------------------------------------------------


/* Sets a rectangle in the dwell array to the specified value. */

void CFracAppEngine::SetDwellRect(const CRect_AC& r, short dwell)
{
	short width = r.right - r.left;
	short height = r.bottom - r.top;
	long index = (long)fDwellWidth * r.top + r.left;
	Byte* dwellRow = &fDwells[index];
	while (height-- > 0)
	{
		Byte* dwells = dwellRow;
		short i = width;
		while (i-- > 0)
			*dwells++ = dwell;

		dwellRow += fDwellWidth;
	}

	this->PaintDwellRect(dwell, r);
}

//-------------------------------------------------------------------------------------------


/* Documents idling routine. Called by TFracAppApplication::DoIdle when its this documents
  turn to calculate part of itself. Takes care of calling the CFracAppEngine. */

#if qUseThreads
pascal Ptr CalcTown(void * threadParam);
pascal Ptr CalcTown(void * threadParam)
{
	while (!((CFracAppEngine*) threadParam)->CalcCity());
	return noErr;
}
#endif

//-------------------------------------------------------------------------------------------


/* Documents idling routine. Called by TFracAppApplication::DoIdle when its this documents
  turn to calculate part of itself. Takes care of calling the CFracAppEngine, and updating
  the window timer displays.

  It determines how much time to give to the FracAppEngine in 3 ways. First, if we are in
  the background, we dont want to be CPU hogs, so we take up as little time as possible.
  Mostly this mean limiting ourselves to about one call to the engine. */

OSErr CFracAppEngine::StartCalculating()
{
#if qUseThreads
	fActiveThreadCounter = fThreadCount;

	TFracThread* program = NULL;
	CThreadRunner_AC* threadHandle;
	for (int i = 0; i < fThreadCount; ++i)
	{
		program = TH_new TFracThread(this);
		threadHandle = TH_new CThreadRunner_AC(program);
		fThreadList->InsertLast(threadHandle);
	}
	return noErr;
#else
	return noErr;
#endif
}

//-------------------------------------------------------------------------------------------
//------------------------------CNormalFracAppEngine Methods---------------------------------
//-------------------------------------------------------------------------------------------


CNormalFracAppEngine::CNormalFracAppEngine()
{
	fDocumentIsFinished = false;
}

//-------------------------------------------------------------------------------------------


/* Initialize our CNormalFracAppEngine object.  Initializes the pen to (0,0). */

void CNormalFracAppEngine::INormalFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth,
												Byte* dwells)
{
	this->IFracAppEngine(itsDocument, dwellWidth, dwells);

	fCurrentLocation = gZeroPt;
}

//-------------------------------------------------------------------------------------------


CNormalFracAppEngine::~CNormalFracAppEngine()
{
}

//-------------------------------------------------------------------------------------------


/* The procedure to do the idle time processing for the fractal. It does it one pixel at a
  time to avoid any hit on performance for other applications.	This is called in response
  to the DoIdle for the application. */

bool CNormalFracAppEngine::CalcCity()
{
	if (this->GetDone())
		return true;

	/* Calculate the fractal as we go.  Do next pixel here, based on the state saved
	  in fCurrentLocation. When done, those variables are updated to go to the next
	  location to do. It sets the pixel in the offscreen port to be whatever we
	  calculate it to be. After an entire row has been calculated, the appropriate
	  rectangle on the screen is invalidated. This will cause the screen to get
	  updated the next time through the event loop. */
	bool isFinished = false;		// Assume document is not done.
	CRect_AC tempRect = fCalcRect;
	{
#if qUseThreads
		CWhileSynchronized_AC entry(&fLock);
#endif
		tempRect.top = fCurrentLocation.v;
		++fCurrentLocation.v;				// Up the counter of next row to do.
	}
	tempRect.bottom = tempRect.top+1;
	// Check if we are done, and if so, set the flag to stop calculations.
	if (tempRect.top > fPlotHeight-1) 
	{
		isFinished = true;
	}
	else 
	{
		this->RectWhap(tempRect);
		this->ReportRectCompleted(tempRect);
	}
#if !qUseThreads
	fDocumentIsFinished = isFinished;
#endif
	if (fThreadCount == 0)
		fDocumentIsFinished = isFinished;
	return isFinished;

} // TNormalFracAppDocument.CalcCity

//-------------------------------------------------------------------------------------------


/* Routine to read the data from the data fork of the file into our engine. This consists
  only of fCurrentLocation. This method is called by our TDocument.DoRead. The file mark
  is already set, so we only have to read whats there. */

void CNormalFracAppEngine::DoRead(short aRefNum, bool /*forPrinting*/)
{
	long recsize = sizeof(fCurrentLocation);
	::ThrowIfError_AC(::FSRead(aRefNum, &recsize, &fCurrentLocation));
}

//-------------------------------------------------------------------------------------------


/* Write out our state variables to disk: fCurrentLocation. This method is called by
  TFracAppDocument::DoWrite, and it sets up the file mark for us, so we just need to start
  writing to whereever we happen to be. */

void CNormalFracAppEngine::DoWrite(short aRefNum, bool /*makingCopy*/)
{
	long recsize = sizeof(fCurrentLocation);
	::ThrowIfError_AC(::FSWrite(aRefNum, &recsize, &fCurrentLocation));
}

//-------------------------------------------------------------------------------------------
//-------------------------------CFastFracAppEngine Methods----------------------------------
//-------------------------------------------------------------------------------------------


/* Constructor.  Initialize data members to safe values. */

CFastFracAppEngine::CFastFracAppEngine()
{
	fRectStack = NULL;
	fDocumentIsFinished = false;
}

//-------------------------------------------------------------------------------------------


/* Initializer for CFastFracAppEngine. Creates a CRectStack to hold the subdivided
   rectangles. After that, it seeds the RectStack with 4 rectangles that cover
   the entire document. */

void CFastFracAppEngine::IFastFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth,
											Byte* dwells)
{
	const short kInitialNumberOfRects = 160;

	this->IFracAppEngine(itsDocument, dwellWidth, dwells);

	Boolean oldPerm = PermAllocation(true);
	CRectStack* aRectStack = TH_new CRectStack(kInitialNumberOfRects);
	PermAllocation(oldPerm);
	::ThrowIfNULL_AC(aRectStack);
	fRectStack = aRectStack;

	CRect_AC tempRect = fCalcRect;
	this->DivideAndConquer(tempRect);
}

//-------------------------------------------------------------------------------------------


CFastFracAppEngine::~CFastFracAppEngine()
{
	if (fRectStack)
	{
		delete fRectStack;
		fRectStack = NULL;
	}
}

//-------------------------------------------------------------------------------------------


/* The procedure to do the idle time processing in the document. For a TFastFracAppDocument,
  we use the Mariani/Silver algorithm. This method assumes that if we are given a bounded
  region and the boundary of that region is all one color, then the interior of that region
  is the same color. We can take advantage of this by recursively quartering our pixmap
  until we start getting rectangles that have solid boundaries. Once we get one, we fill it
  in all at once with a big PaintRect. To take into account that we dont live in a perfect
  world (well...*I* dont, anyway), and that things start to degrade once we get down to
  smaller scales, we forego the big PaintRect thing once either side of the rectangle we are
  looking at gets to be 4 pixels or smaller. This should take care of any skinny color
  snakes that sneak their way in just because they are too skinny to show up at 72dpi. */

bool CFastFracAppEngine::CalcCity()
{
	if (this->GetDone())
		return true;

	/* Get the next rectangle off of the stack, and start to analyze it. It should
	  fall into one of three states. It could be very small (less than 4 pixels on
	  a side), in which case we just go ahead and fill it in. It could be that
	  color of the entire border is the same, in which case we call PaintRect to
	  fill it in. Finally, we have the rectangle whose border colors are not the
	  same. In this case, we call DivideAndConquer to divvy it up into four smaller
	  rectangles, and push them on the stack for later. */

	short itsDwell;
	CRect_AC thisRect;
	bool isFinished = !fRectStack->PopRect(thisRect);
	if (!isFinished) {

		if (((thisRect.right - thisRect.left) <= kMinCCRectSize) ||
			((thisRect.bottom - thisRect.top) <= kMinCCRectSize))
		{
			this->RectWhap(thisRect);

			/* Our rectangles are chosen so that there is a little bit of overlap between them
			  and the rects next to them. We have to get rid of that overlap so that updating
			  is more efficient, and so that our areaCompleted running total is accurate. */

			if (thisRect.top > 0)
				++thisRect.top;
			if (thisRect.left > 0)
				++thisRect.left;

			this->ReportRectCompleted(thisRect);
		}
		else if ((itsDwell = this->BordersAreTheSame(thisRect)) != 0)
		{
			if (thisRect.top > 0)
				++thisRect.top;
			if (thisRect.left > 0)
				++thisRect.left;

			CRect_AC doneRect = thisRect;

			// Do an InsetRect by hand for speed.
			//	++thisRect.top;
			//	++thisRect.left;
			--thisRect.bottom;
			--thisRect.right;

			this->SetDwellRect(thisRect, itsDwell);

			this->ReportRectCompleted(doneRect);
		}
		else
		{
			this->DivideAndConquer(thisRect);
		}
	}
#if qUseThreads
	else {
		/*	The stack is empty.  This may either be because the document is done or
			because other threads are currrently processing all outstanding rects.
			We yield here to give any other threads a chance to add more to the stack.
			If there still isn't anything to process, we'll return true to terminate
			the thread. */
		CThread_AC::Yield();
		isFinished = (fRectStack == NULL) || fRectStack->IsEmpty();
	}

#else
	fDocumentIsFinished = isFinished;
#endif
	if (fThreadCount == 0) fDocumentIsFinished = isFinished;
	return isFinished;

} // CFastFracAppEngine::CalcCity

//-------------------------------------------------------------------------------------------


/* Takes the proferred rectangle, divides it into four parts, and pushes them onto the
  CRectStack. This is done carefully so as to make sure that the rectangles dont just abut
  each other, but that they actually overlap. This is so we dont go and calculate the right
  border of one rectangle, and then the left border of the rectangle just to its right. If
  they overlap, we only calculate one line of pixels, which doubles as the border of
  two rectangles. */

void CFastFracAppEngine::DivideAndConquer(const CRect_AC& r)
{
	CRect_AC quarterRect;

	// Push on the lower right corner

	quarterRect.top = (r.bottom + r.top - 1) / 2;
	quarterRect.left = (r.right + r.left - 1) / 2;
	quarterRect.bottom = r.bottom;
	quarterRect.right = r.right;
	fRectStack->PushRect(quarterRect);

	// Push on the lower left corner

	quarterRect.right = quarterRect.left + 1;
	quarterRect.left = r.left;
	fRectStack->PushRect(quarterRect);

	// Push on the upper right corner

	quarterRect.bottom = quarterRect.top + 1;
	quarterRect.top = r.top;
	quarterRect.left = (r.right + r.left - 1) / 2;
	quarterRect.right = r.right;
	fRectStack->PushRect(quarterRect);

	// Push on the upper left corner

	quarterRect.right = quarterRect.left + 1;
	quarterRect.left = r.left;
	fRectStack->PushRect(quarterRect);
}

//-------------------------------------------------------------------------------------------


/* Modified version of PixWhap. This routine first sees if we calculated the
  value for the specified pixel already. If so, we just return its color. If
  not, we call PixWhap to calculate and plot it, and then return that color to
  the caller. */

short CFastFracAppEngine::CalcAndGetDwell(short curCol, short curRow)
{
	short dwell = this->GetDwell(curCol, curRow);
	if (dwell == 0)
	{
		dwell = this->PixWhap(curCol, curRow);
	}
	return dwell;
}


//-------------------------------------------------------------------------------------------


/* Takes the given rect, and proceeds to calculate and compare the border
  colors. If they are all the same, we return true. However, as soon as we
  find a different color, we stop calculating, and return false. */

bool CFastFracAppEngine::ComeAcross(short y, short left, short right,
									   short baseDwell)
{
	bool mismatch;
	short x = left;
	do
	{
		mismatch = CalcAndGetDwell(++x, y) != baseDwell;
	} while (!mismatch && (x < right));

	return mismatch;
}

bool CFastFracAppEngine::ComeDown(short x, short top, short bottom,
									 short baseDwell)
{
	bool mismatch;
	short y = top;
	do
	{
		mismatch = CalcAndGetDwell(x, ++y) != baseDwell;
	} while (!mismatch && (y < bottom));
	return mismatch;
}

short CFastFracAppEngine::BordersAreTheSame(const CRect_AC& r)
{
/*
	short baseDwell = CalcAndGetDwell(r.left, r.top);

	if (ComeAcross(r.top, r.left, r.right - 1, baseDwell))
		return 0;

	if (ComeAcross(r.bottom - 1, r.left, r.right - 1, baseDwell))
		return 0;

	if (ComeDown(r.left, r.top, r.bottom - 1, baseDwell))
		return 0;

	if (ComeDown(r.right - 1, r.top, r.bottom - 1, baseDwell))
		return 0;
*/

/* Sides overlap by one at each corner, so if each side is a single color, the 
  whole border matches. */
  
	short baseDwell;
	CRect_AC side = r;
	side.bottom = r.top + 1;
	baseDwell = RectWhap(side, true);
	if (baseDwell != 0) {
		side = r;
		side.top = r.bottom-1;
		baseDwell = RectWhap(side, true);
		if (baseDwell != 0) {
			side = r;
			side.right = r.left+1;
			baseDwell = RectWhap(side, true);
			if (baseDwell != 0) {
				side = r;
				side.left = r.right-1;
				baseDwell = RectWhap(side, true);
			}
		}
	}
	return baseDwell;
}


//-------------------------------------------------------------------------------------------


/* Sort of a small version of the CNormalFracAppEngine. This routine gets called
  when our rectangle has gotten small enough. It just loops through and
  calculates the pixels one by one. The rectangle shouldnt be larger than
  about 20 or 25 pixels, so this doesnt take very long. */

void CFastFracAppEngine::FillMeUp(const CRect_AC& thisRect)
{
	RectWhap(thisRect);
/*
	for (short x = thisRect.left; x <= thisRect.right - 1; ++x)
	{
		for (short y = thisRect.top; y <= thisRect.bottom - 1; ++y)
		{
			this->CalcAndGetDwell(x, y);
		}
	}
*/
}	// FillMeUp


//-------------------------------------------------------------------------------------------


/* Routine to read the data from the data fork of the file into our engine. This consists of
  all the Rects that we had in the CRectStack. First, tell the CRectStack to clear itself
  out. Then, read in a count of the number of rectangles that we wrote out. Next, loop for
  that number of times, reading a rectangle and pushing it onto the stack. This method is
  called by our TDocument.DoRead. The file mark is already set, so we only have to read
  whats there. */

void CFastFracAppEngine::DoRead(short aRefNum, bool forPrinting)
{
	fRectStack->DoRead(aRefNum, forPrinting);
}

//-------------------------------------------------------------------------------------------


/* Write out our state variables to disk. This consists of all the rectangles we may have on
  the CRectStack, in addition to a count of the number of rectangles there actually are (for
  when we need to read them back in). To do this, we call a CRectStack method that performs
  a routine for each item on the stack. The routine that we tell it to perform is one that
  takes the current rectangle, and writes it to disk.. This method is called by
  TFracAppDocument::DoWrite, and it sets up the file mark for us, so we just need to start
  writing to wherever we happen to be. */

void CFastFracAppEngine::DoWrite(short aRefNum, bool makingCopy)
{
	fRectStack->DoWrite(aRefNum, makingCopy);
}
