/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

Main unit for FracApp 3.1. It defines the following classes:

CFracAppEngine - Base class for a fractal generating engine. Again, this was
	myopically designed with FracApp in mind, so you may have a chore of adopting
	it for Fractals other than the Mandelbrot set. It provides a core routine for
	calculating the value of a pixel, and defines methods you should override.
	The methods define what should be done at Idle time, and how to read and
	write Engine specific data to disk.
CNormalFracAppEngine - Calculates the entire document pixel-by-pixel.
CFastFracAppEngine - Calculates the Mandelbrot set using the Mariani/Silver
	algorithm. This algorithm assumes that the content of any area that is
	bounded by a single color is the same color. This algorithm recursively
	divides the document up into 4 parts, and examines the borders of each
	rectangle. If the border is all the same color, then the entire rectangle is
	filled with that color, and we go to the next rectangle. If the border colors
	are not all the same, then THAT rectangle is quartered, and we do the process
	again.

--------------------------------------------------------------------------------------------*/

#ifndef __UENGINE__
#define __UENGINE__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

#if qUseThreads

	#ifndef __UFracThread__
	#include "UFracThread.h"
	#endif

	#ifndef __CThreadRunner_AC__
	#include "CThreadRunner_AC.h"
	#endif
	
	#ifndef __CMonitorLock_AC__
	#include "CMonitorLock_AC.h"
	#endif
	
#endif

#ifndef  __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif


#ifndef __CList_AC__
#include "CList_AC.h"
#endif


class TFracAppDocument; // external
class CRectQueue; // external
class CRectStack; // external

inline long IntMultiple(short x, short y) {return (long)x * (long)y;}

//-------------------------------  Engine  -------------------------------

class CFracAppEngine
{
  protected:
	TFracAppDocument*	fFracAppDocument;	/* Private - Used to tell the document
												that new parts of the fractal have
												been completed, and that the window
												needs to be updated. */
	long double			fRealMin;			// These are valid for any doc
	long double			fImagMin;			//
	long double		 	fDeltaP;			// horz step size in fractal space.
	long double		 	fDeltaQ;			// vert step size in fractal space.
	long				fPlotWidth;			// width of fractal area in pixels.
	long			 	fPlotHeight;		// height of fractal area in pixels.
	CRect_AC				fCalcRect;			// rectangle surrounding the window it was built for
	Byte*				fDwells;			// Array of dwell values
	short				fDwellWidth;		// Width of the dwell array
	Byte				fLastDwell;			// Save recent dwell value for optimizing painting
	CRectQueue*			fPaintQueue;		// Computed Rects to be painted
#if qUseThreads
	friend class TFracThread;
	CAutoPtr_AC<CList_AC>	fThreadList;	// List of references to my threads.
	CMonitorLock_AC		fLock;				// Concurrent access control for fields.
	long				fActiveThreadCounter;	// Active thread count.
	long				fThreadCount;		// Number of threads or tasks to use to
											// generate the fractal.  If this is zero,
											// idle processing is used instead of
											// threading or tasks 
	ThreadStyle			fThreadStyle;		// Type of thread or task scheduling desired.
											// Valid values are kCooperativeThread or
											// kPreemptiveThread, as defined in Threads.h.
											// When the preemptive style is selected on
											// PowerPC, multiprocessor tasks are used
											// instead of thread manager threads.
#endif

  public:
	CFracAppEngine();

	virtual ~CFracAppEngine();
	/* Destructor.  Ties up loose thread. */

	void IFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth,
						Byte* dwells);
	/* Inits the engine object itself. Remembers a reference to its document, and
	   makes a working copy of its FracHeader. This is done for speed so that we
	   dont have to continually query the document for its info. */

	virtual bool CalcCity() = 0;
	/* Abstract method for calculating a little bit of the fractal. Called by
	   TFracAppDocument.CalcTown. This one here doesnt do anything; it is meant to
	   be overridden. */

	virtual void DoRead(short aRefNum, bool forPrinting) = 0;
	/* Abstract method for reading what the engine needs from disk. Called by
	   TFracAppDocument.DoRead. This one here doesnt do anything; it is meant to
	   be overridden. */

	virtual void DoWrite(short aRefNum, bool makingCopy) = 0;
	/* Abstract method for writing what the engine needs to disk. Called by
	   TFracAppDocument.DoWrite. This one here doesnt do anything; it is meant to
	   be overridden. */

	bool fDocumentIsFinished;
	bool GetDone()	{ return fDocumentIsFinished; }
	/* Returns whether the calculations are done by checking the result. */

	short GetDwell(short x, short y)
	{
		long index = (long)fDwellWidth * y + x;
		return fDwells[index];
	}
	/* Gets the value for a point in the dwell array. */

	short GoFigger(const long double Po, const long double Qo, const short M, const short K);
	/* Heart and soul of the program. Called by the CalcCity routines to calculate
	  the color of the specified point. */

	void PaintDwellRect(Byte dwell, const CRect_AC& r);
	/* Paints a rectangle in the offscreen bitmap, based on the value in the dwell array.
	  It is assumed that all the dwell values in the rectangle are the same. */

	short PixWhap(short col, short row);
	/* Called by the engines CalcCity method. Given a point, figure out its color
	  and plot it. Also returns the actual rectangle used for plotting the thing,
	  and the color used. */

	short RectWhap(const CRect_AC& r, bool wantsMatch = false);
	/* Called by the engines CalcCity method. Given a rect, figure out its colors
	  and plot it.*/

	void ReportRectCompleted(CRect_AC& dirtyRect);
	/* Bottleneck used by CalcCity to inform the document that part of the fractal
	  has been freshly calculated and needs to be copied to the screen. */

	void SetDwell(short x, short y, short dwell);
	/* Sets a point in the dwell array to the specified value. */

	void SetDwellRect(const CRect_AC& r, short dwell);
	/* Sets a rectangle in the dwell array to the specified value. */

	OSErr StartCalculating();
	/* Starts a thread to do the computations. */

};

class CNormalFracAppEngine: public CFracAppEngine
{
  protected:
	Point	fCurrentLocation;		/* Private - holds current positon of the
									   pen. */

  public:
	CNormalFracAppEngine();

	virtual ~CNormalFracAppEngine();

	void INormalFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth,
							  Byte* dwells);
	/* Initializes the CNormalFracAppEngine, as well as its superclass. Tells the
	   document its version number, and initializes the pen to (0,0). */

	virtual bool CalcCity();
	/* Calculates the fractal one pixel at a time. If weve calculated a whole row,
	   tell the document so that it can copy it to the screen. */

	virtual void DoRead(short aRefNum, bool forPrinting);
	// Called by TFracAppDocument.DoRead. Reads fCurrentLocation from disk.

	virtual void DoWrite(short aRefNum, bool makingCopy);
	// Called by TFracAppDocument.DoWrite. Writes fCurrentLocation to disk.

};

class CFastFracAppEngine: public CFracAppEngine
{
  protected:
	CRectStack*		fRectStack; 	 /* Private - maintains the areas of the
										document we need to still calculate. */

  public:
	CFastFracAppEngine();
	/* Constructor.  Initialize data members to safe values. */

	virtual void IFastFracAppEngine(TFracAppDocument* itsDocument, short dwellWidth,
									Byte* dwells);
	/* Initializes the CFastFracAppEngine, as well as its superclass. Tells the
	   document its version number, creates a TRectStack, and seeds it with 4
	   starting rectangles. */

	virtual ~CFastFracAppEngine();
	// Frees fRectStack.

	virtual bool CalcCity();
	// Calculates the fractal using the divide and conquer method.

	void DivideAndConquer(const CRect_AC& r);
	/* Takes a rectangle, quarters it, and pushes it on the rect stack. Called by
	   CalcCity when it finds a rectangle whose border is not all the same color. */

	short CalcAndGetDwell(short curCol, short curRow);
	/* Modified version of PixWhap. This routine first sees if we calculated the
	  value for the specified pixel already. If so, we just return its color. If
	  not, we call PixWhap to calculate and plot it, and then return that color to
	  the caller. */

	bool ComeAcross(short y, short left, short right, short baseDwell);
	bool ComeDown(short x, short top, short bottom, short baseDwell);
	// Utilities used by BordersAreTheSame

	short BordersAreTheSame(const CRect_AC& r);
	/* Takes the given rect, and proceeds to calculate and compare the border
	  dwell values. If they are all the same, we return the value. However, as
	  soon as we find a different dwell, we stop calculating, and return 0. */

	void FillMeUp(const CRect_AC& thisRect);
	/* Sort of a small version of the CNormalFracAppEngine. This routine gets called
	  when our rectangle has gotten small enough. It just loops through and
	  calculates the pixels one by one. The rectangle shouldnt be larger than
	  about 20 or 25 pixels, so this doesnt take very long. */

	virtual void DoRead(short aRefNum, bool forPrinting);
	// Called by TFracAppDocument.DoRead. Reads the rect stack from disk.

	virtual void DoWrite(short aRefNum, bool makingCopy);
	// Called by TFracAppDocument.DoWrite. Writes the rect stack to disk.

};

#endif
