/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker, Steven McGrath, Brian Arnold.
PowerPC version by Jeroen Schalk

Copyright  1988-1997 Apple Computer, Inc. All rights reserved.

--------------------------------------------------------------------------------------------

TFracAppDocument - Sub-Class of TPICTDocument. Reads and writes PICT header
	information that describes the fractals we make. Also manages the FracApp
	engines we use to create the fractals in the first place. Finally, it acts as
	a go-between for the TFracAppView, and the engine that is generating our
	data.

--------------------------------------------------------------------------------------------*/

#ifndef __UFRACDOC__
#define __UFRACDOC__

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

#ifndef __UPICTDOC__
#include "UPictDoc.h"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UFRACGLOBALS__
#include "UFracGlobals.h"
#endif

class CRectQueue; // external
class CFracAppEngine; // external
class COffscreen; // external
class TFracAppWindow; // external

	// Constants used for identification

const OSType kSignature		= 'SS01';		// Application signature
const OSType kFileType		= 'PICT';		// File-type code used for document files
											// created by this application
const short kHeaderID		= 0x4641;		// Header ID = FA for FracApp

/* TFracAppDocument::LockThePixels takes a bool value that determines if we are actually
  locking the pixels, or really unlocking them. These constants define what action to take. */

const bool kLock				= true;
const bool kUnlock 			= ! kLock;

/* When setting the strings in the infobar (elapsed time, etc.) we need to differentiate
  between setting those strings for the FIRST time, and all subsequent times. When calling
  them for the first time, we pass kForceUpdate. All other times, we pass kDontForceUpdate. */

const bool kForceUpdate		= true;
const bool kDontForceUpdate	= ! kForceUpdate;

//---------------------------  CPaintRectCache	---------------------------

class CPaintRectCache
{
  protected:
	CRGBColor_AC fColor;
	CRect_AC fRect;

  public:
	CPaintRectCache();
	void Put(const CRGBColor_AC& aColor, const CRect_AC& aRect);
	void Flush(void);
};

//-------------------------------  Document	-------------------------------

DeclareClassDesc(TFracAppDocument);

class TFracAppDocument : public TPICTDocument 
{
	DeclareClass(TFracAppDocument);

  public:
	FracRecord		fFracHeader; 	  /* Global state of any fractal. It can be
										 accessed with the following methods:
										 GetDone, GetFracHeader, GetPlotHeight,
										 GetPlotWidth */
	CFracAppEngine*	fFracAppEngine;   /* Private - Used by CalcTown, DoRead, and
										 DoWrite */
	TFracAppWindow*	fFracAppWindow;   /* Private - Used to animate the windows
										 palettes. */
	COffscreen*		fOffWorlder; 	  /* Private - Object that handle offscreen
										 management. */
	Byte*			fDwells;		  /* Array of dwell values */
	short			fDwellWidth;	  /* Width of the dwell array */
	CRectQueue*		fPaintQueue;	  /* Computed Rects to be painted */
	CPaintRectCache	fPaintRect;		  /* Cache for faster painting */
	CommandNumber	fStartupMode;	  /* Private - Used in DoInitialState to set
										 variables that are different depending
										 on whether we are multipaging or not,
										 or if we are creating this document
										 afresh or based on the selection of
										 another document. */
#if qUseThreads
	long			fThreadCount;	  /* Number of threads or tasks to use to
										 generate the fractal.  If this is zero,
										 idle processing is used instead of
										 threading or tasks */
	ThreadStyle		fThreadStyle;	  /* Type of thread or task scheduling desired.
										 Valid values are kCooperativeThread or
										 kPreemptiveThread, as defined in Threads.h.
										 When the preemptive style is selected on
										 PowerPC, multiprocessor tasks are used
										 instead of thread manager threads. */
#endif

	TFracAppDocument(TFile* itsFile = NULL, bool use32BitCQD = true,
							CommandNumber startupMode = cNewFromSelection);
	/* Init routine for the document. Called when the new document is created in
	   DoMakeDocument. Sets up the object, then the fractal
	   default state. */

	virtual ~TFracAppDocument();
	/* Free method for our document.  It calls FreeData to dispose the data block of
	   the picture data that was read in from the disk, and kills the offscreen
	   stuff. */

	virtual void AnimateColors(void);
	// Called by the application at Idle time to do the palette animation.

	virtual void BumpAreaComplete(long areaAmount);
	/* Adds areaAmount to the areaComplete field. We keep track of this stuff so
	   that we can display the percentage complete. This is my own method that the
	   TFracAppEngines call when theyve updated a new area of the fractal. */

	void BumpChangeCount(void)	{ ++fChangeCount; }
	/* Increments the fChangeCount field. This is my own method that the
	   TFracAppEngines call when theyve updated a new area of the fractal. */

	virtual void BumpCalculationTime(long elapsedAmount);
	/* Increments the elapsedTime field and updates the window display. Called
	   by the document idling routine: CalcTown. */

	virtual bool CalcTown(void);
	/* Documents idling routine. Called by TFracAppApplication.DoIdle when its
	   this documents turn to calculate part of itself. Takes care of calling the
	   CFracAppEngine, and updating the window timer displays. */

	virtual void DoDrawPicture(void);
	/* Called by TPICTDocument.DoRead and DoWrite. Images the picture for saving to
	   disk. Just calls Copybits. */

	virtual void DoInitialState(void);
	/* Called by MacApp if we are making a new document (as opposed to reading one
	   from disk). Does the work for a New operation, where we start with a new
	   fractal that doesnt have any stored data. */

	virtual void DoMakeViews(bool forPrinting);
	/* Called when launching a new document, be it newly created or being read from
	   disk. Launches the view which is seen in the documents window.  Every
	   document which has any screen display or which can be printed MUST override
	   this method. */

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
	/* Called by MacApp when a menu item has been chosen. Handles the Copy menu
	   item, as well as everything in the Zoomy menu */

	virtual void DoNeedDiskSpace(TFile* itsFile, long& dataForkBytes,
										long& rsrcForkBytes);
	/* Called by MacApp when preparing to save a document to disk. This override
	   doesnt do any calculations itself, but it does prepare the offscreen world
	   before we call the INHERITED method. */

	virtual void DoRead(TFile* aFile, bool forPrinting);
	/* Called by MacApp to read the data from the data fork of the file. Sets up the
	   offscreen world so that we can call the INHERITED method to do the reading of
	   the PICT. Also resets the vars in the document object to match the saved
	   values from the header. */

	virtual void DoSetupMenus(void);
	/* Called by MacApp when the user clicks on the menu. Handles the Copy menu
	   item, as well as everything in the Animate menu. The Animate menu is only
	   enabled if we have 32-bit Color QuickDraw. */

	virtual void DoWrite(TFile* aFile, bool makingCopy);
	/* Called by MacApp to write out the documents data. Sets up the offscreen
	   world, and calls the INHERITED method to actually write out the PICT. Also
	   writes out the standard FracHeader, and calls the engines DoWrite method to
	   have it write out anything it needs. */

	virtual void FreeData(void);
	/* Free method usually used for Revert operation, kept here to show the normal
	   Free approach. */

	bool GetDone(void)	{ return fFracHeader.done; }
	/* Returns the fFracHeader.done field. Called by various FracApp routines to see
	   if this document needs any calculation time. */

	FracRecord GetFracHeader(void)	{ return fFracHeader; }
	/* Returns a copy of the fFracHeader field. This is an easy way for the
	   CFracAppEngine to get a lot of working values it needs. */

	long GetThreadCount()	{ return fThreadCount; }
	/* Returns the number of threads to be used in generating the fractal.  Zero
	   indicates that threads are not used for this document. */
	
	ThreadStyle GetThreadStyle()	{ return fThreadStyle; }
	/* Returns the style of thread or task to be used in generating the fractal */

	virtual CGrafPtr GetFracPort(void);
	// Returns the offscreen port. Used by the TFracAppView when updating the screen.

	virtual Ptr GetOffPixBase(void);
	/* Returns the pointer to the bits for the offscreen port. Called by the
	   CFracAppEngine when doing its own version of GetCPixel. */

	CRectQueue* GetPaintQueue()		{ return fPaintQueue; }
	/* Return a pointer to the stack of rectangles to be painted. */

	long GetPlotHeight(void)	{ return fFracHeader.plotHeight; }
	/* Returns fFracHeader.plotHeight. Called by TAreaSelector so that it knows what
	   dimensions the document is, so that it can etch a proportional selection
	   rectangle. */

	long GetPlotWidth(void)	{ return fFracHeader.plotWidth; }
	/* Returns fFracHeader.plotWidth. Called by TAreaSelector so that it knows what
	   dimensions the document is, so that it can etch a proportional selection
	   rectangle. */

	bool IsMultiPaging(void)	{ return fFracHeader.pages.multiPaging; }
	// Return true if this document is one in a series of multi-paged documents.

	virtual void JumblePalette(void);
	/* Jumbles up the palette so that we get a nice banding effect. Called by
	   DoMenuCommand when the user selects the Jump Palette menu
	   item. */

	virtual void LockThePixels(bool lockIt);
	/* Called by the TFracAppView before it does any CopyBits operations. This is
	   so the View only needs to know about the TFracAppDocument; it never needs
	   to know that a TOffScreen object exists. */

	virtual Byte* MakeDwells(void);
	/* Make a dwells buffer. */

	virtual Byte* MakeDwellsFromBitmap(void);
	/* Make a dwells buffer and initialize it from the bitmap in our offscreen graphics
	  world. */

	virtual CFracAppEngine* MakeFracAppEngine(short version);
	/* Handy utility for making a CFracAppEngine. Makes a different kind depending
	   on what version is set to. Called by DoRead and
	   DoInitialState. */

	virtual COffscreen* MakeOffWorlder(bool use32BitCQD, const CRect_AC& bounds);
	/* Handy utility for making a TOffScreen. Makes a different kind depending on
	   what use32BitCQD is set to. Called by DoRead and
	   DoInitialState. */

	virtual bool PaintDwellQueue(void);

	virtual void PaintDwellRect(const CRect_AC& r);
	/* Paints a rectangle in the offscreen bitmap, based on the value in the dwell array.
	  It is assumed that all the dwell values in the rectangle are the same. */

	virtual void BlitDwellRect(const CRect_AC& r);
	/* Blits a rectangle in the offscreen bitmap, from the values in the dwell array. */

	virtual void PreDraw(void);
	/* Called by CFracAppEngine before it images to the offscreen bitmap. This is
	   so the Engine only needs to know about the TFracAppDocument; it never needs
	   to know that a TOffScreen object exists. */

	virtual void PostDraw(void);
	// Called by CFracAppEngine when it is all done imaging to the offscreen dude.

	virtual void RestorePalette(void);
	/* Recovers from the effects of Animating or Jumbling. Called by
	   DoMenuCommand when the user selects Everything back to
	   Normal, or turns off Jumble Palette. */

	virtual void SetAreaComplete(long newComplete, bool forceUpdate);
	/* Updates the areaCompleted field in fFracHeader. Also updates the percentage
	   field in the window. */

	virtual void SetCalculationTime(long newElapsed, bool forceUpdate);
	/* Updates the elapsedTime field in fFracHeader. Also updates the calculation time
	   field in the windows infobar. */

	virtual void SetElapsedTime(bool forceUpdate);
	/* Updates the elapsed time field in the windows infobar based on the starting
	   and current time. */

	virtual void SetMethodName(bool forceUpdate);
	/* Sets the string that displays the algorithm being used to create the fractal.
	   Called by DoMakeViews when the document is being created. */

	virtual OSErr StartCalculating(void);
	/* Starts a thread to do the computations. */

	virtual void StashSelectedRange(void);
	/* Called by the routines that handle creating a new document based on the
	   current selection Transfers information relating to the selection on the
	   screen into some global variables. This is so we can create a new document
	   from them. */
};								// TFracAppDocument

//-------------------------------------------------------------------------------------------

void InitUFracDoc(void);

//-------------------------------------------------------------------------------------------

#endif
