/*-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

Offscreen management. Herein, we have three objects. There is a COffscreen
object, that defines some common routines and method names. It really doesnt
do a whole lot, as	most of the functionality is implemented in its
sub-classes: COldGrossOffscreen, and CNewCoolOffscreen.

COldGrossOffscreen is a set of home brew routines that implement offscreen
PixMaps totally by hand. These are the routines that are in the original
version of FracApp. In the past, this was the only way to create offscreen
PixMaps.When initializing COldGrossOffscreen, we create a new PixMap and data
buffer, set up all the values for them, clear the buffer, create a gDevice
record, initialize it, attach the two together, and hope that it will still
work under new Systems. CNewCoolOffscreen uses the new 32-Bit Color QuickDraw
routines if they are available. It just calls NewGWorld, and attaches our color
table to it.

There are several support routines for TOffscreens. ChangeCTFlag is used to set
or clear color table flags. This was hacked in to support the need to clear and
set bit 14 when we save the PixMap to disk. LockThePixels and UnlockThePixels
is called whenever we have to perform some QuickDraw operation, like CopyBits.
This is only needed for the CNewCoolOffscreen stuff, and is a null operation
uner COldGrossOffscreen. SaveOldWorld and SetupOffWorld are used for saving
whatever port and gDevice are set, and swapping in the offscreen drawing
environment. RestoreOffWorld is called when we are done to reverse the effects
of SetupOffWorld. None of these last 5 routines (LockThePixels,
UnlockThePixels, SaveOldWorld, SetupOffWorld, RestoreOffWorld) should be called
directly, as they are combined in the PreDraw and PostDraw routines, which take
care of all the management. Whenever you need to do offscreen drawing, call
PreDraw, and then PostDraw when you are done. Finally, if you need to get the
offscreen port for some direct manipulation (like for CopyBits), call
GetOffPort.

--------------------------------------------------------------------------------------------*/

#ifndef __UOFFSCREEN__
#define __UOFFSCREEN__

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __QDOFFSCREEN__
#include <QDOffScreen.h>
#endif

const short kAnimationFlag		= 14;
const bool kClearFlag		= false;
const bool kSetFlag			= true;

class COffscreen
{
  protected:
	GDHandle 		fOldDevice;		  /* Holds the previous device while were
										 busy playing in the offscreen world.
										 You can fetch the device if you need to
										 with GetOldDevice. */
	GrafPtr			fOldPort;		  /* Holds the previous GrafPort while were
										 busy playing in the offscreen world.
										 You can fetch the port if you need to
										 with GetOldPort. */
  public:
	COffscreen(void);
	// Initialize the offscreen world.

	virtual ~COffscreen(void);
	// Destructor.

	void ChangeCTFlag(short whichBit, bool whichWay);
	// Set or clear a color table flag.

	virtual GDHandle GetOffDevice(void) = 0;
	/* Return the GDHandle of the device belonging to the offscreen port. Must be
	   overridden. */

	virtual Ptr GetOffPixBase(void) = 0;
	// Return the base address of our offscreen pixels. Must be overridden.

	virtual CGrafPtr GetOffPort(void) = 0;
	// Return a CGrafPtr to the offscreen port. Must be overridden.

	GDHandle GetOldDevice(void)	{ return fOldDevice; }
	// Return a GDHandle to the saved device.

	GrafPtr GetOldPort(void)	{ return fOldPort; }
	// Return a GrafPtr to the saved port.

	virtual void LockThePixels(void);
	// For 32BCQD support. Does nothing by default.

	void PreDraw(void);
	/* Call this method to save off the current drawing world and swap in the
	   offscreen drawing world. */

	virtual void PostDraw(void);
	// Call this method to restore the drawing world saved by PreDraw.

	void RestoreOldWorld(void);
	/* Called by PostDraw to restore the old port and gDevice. Should not need
	   to be called directly. PostDraw will call this for you. */

	void SaveOldWorld(void);
	/* Called by PreDraw to save the old port and gDevice. Should not need
	   to be called directly. PreDraw will call this for you.  */

	virtual void SetupOffWorld(void) = 0;
	/* Called by PreDraw to set the offscreen port and gDevice. Must be overridden. */

	virtual void UnlockThePixels(void);
	// For 32BCQD support. Does nothing by default.
};

class COldGrossOffscreen: public COffscreen
{
  protected:
	Ptr				fBigBuff;		  /* The memory to use for our pixels in the
										 offscreen world. You can get their
										 address with GetOffPixBase. */
	GDHandle 		fOffDevice;		  /* Holds the offscreen device to use when
										 playing with the offscreen world. You
										 can fetch the device if you need to
										 with GetOffDevice. */
	CGrafPtr		fOffPort;		  /* Holds the offscreen GrafPort to use
										 when playing with the offscreen world.
										 You can fetch the port if you need to
										 with GetOffPort. */

  public:
	COldGrossOffscreen(void);
	// Constructor - initializes data members to safe values.

	void IOldGrossOffscreen(const CRect_AC& offBounds, CTabHandle itsColors);
	/* Create and initialize an offscreen drawing environment using the routines
	   described in Inside Mac V. The offscreen world size is specified by
	   offBound, and uses the colors in itsColors. It makes a copy of the
	   table. */

	virtual ~COldGrossOffscreen(void);

	virtual GDHandle GetOffDevice(void);
	virtual Ptr GetOffPixBase(void);
	virtual CGrafPtr GetOffPort(void);

	virtual void SetupOffWorld(void);
	/* The default SetupOffWorld doesnt do anything because it doesnt know what
	  form our offscreen management takes. So we have to override it here. */
};

class CNewCoolOffscreen: public COffscreen
{
  protected:
	GWorldPtr		fOffWorld;

  public:
	CNewCoolOffscreen(void);
	// Constructor - initializes data members to safe values.

	virtual void INewCoolOffscreen(const CRect_AC& offBounds, CTabHandle itsColors);
	/* Create and initialize an offscreen drawing environment using the 32-bit Color
	   QuickDraw routines. The offscreen world size is specified by offBound, and
	   uses the colors in itsColors. It makes a copy of the table. */

	virtual ~CNewCoolOffscreen(void);

	virtual GDHandle GetOffDevice(void);
	virtual Ptr GetOffPixBase(void);
	virtual CGrafPtr GetOffPort(void);

	virtual void LockThePixels(void);

	virtual void PostDraw(void);
	/* The base PostDraw will set the port and gDevice correctly. However, before it
	   does that, we have to make sure that we unlock our offscreen bit so that the
	   can float around and not fragment our heap. */

	virtual void SetupOffWorld(void);
	/* The default SetupOffWorld doesnt do anything because it doesnt know what
	  form our offscreen management takes. So we have to override it here. */

	virtual void UnlockThePixels(void);

};

#endif
