/*
-------------------------------------------------------------------------------------------

Program:	FracApp 3.1

by Keith Rollin & Bo3b Johnson
of Apple Macintosh Developer Technical Support

Threads support by Tom Becker
PowerPC version by Jeroen Schalk

Copyright  1988-1994 Apple Computer, Inc.
All rights reserved.

--------------------------------------------------------------------------------------------

This unit implements a stack for storing Rects. It defines a TRectStack class
that is based on the new CDynamicArray_AC class in MacApp 2.0 final. Its used by
the fast FracApp engine in keeping track of the areas of the fractal document
that need to be calculated.

There are 4 major methods of interest. PushRect and PopRect take care of adding
Rects to and removing Rects from the stack. EachRect is used to iterate over
all of the  Rects on the stack. This is used by the CFracAppEngines DoWrite
method when it needs to save the stack to disk. Finally, there is the
ClearStack method that is used to put the stack into a virgin state.

--------------------------------------------------------------------------------------------
*/

#ifndef __URECTSTACK__
#define __URECTSTACK__

#ifndef __FRACRECORD__
#include "FracRecord.h"
#endif

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef  __TYPES__
#include <Types.h>
#endif

#if qUseThreads
	#ifndef __CMonitorLock_AC__
	#include "CMonitorLock_AC.h"
	#endif
	
	#ifndef __CWhileSynchronized_AC__
	#include "CWhileSynchronized_AC.h"
	#endif
#endif

#ifndef qTuneRectStack
	#define qTuneRectStack		1
#endif

//----------------------------------------------------------------------------------------
// CRectStack
//----------------------------------------------------------------------------------------

class CRectStack
{
  protected:
	long fSize;									// number of elements ACTUALLY in the
                                                // array, from 0 to the limit of memory

	long fAllocatedSize;						// Number of elements for which storage is
                                                // ALLOCATED
#if qTuneRectStack
	long fHighWaterMark;						// For tuning
#endif

	CRect_AC *fData;								// Pointer to CRect_AC storage array
#if qUseThreads
	CMonitorLock_AC fLock;
#endif

	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

  public:
	CRectStack(long numberOfRects);
		// Initialize a new array with initialNumberOfRects rects.

	~CRectStack();
		// Free the array.


	//------------------------------------------------------------------------------------
	// Array manipulation primitives
	//------------------------------------------------------------------------------------

	long GetSize(void)	{ return fSize; }
		// Returns the ACTUAL number of elements in the array.

	void DeleteAll(void)	{ fSize = 0; }
		// Remove all Rects from the stack. The stack is left empty.

	void PushRect(const CRect_AC& theRect);
		// Takes the CRect_AC you pass it, and pushes it on the stack.

	bool PopRect(CRect_AC& theRect);
		// Returns the first CRect_AC on the stack. That CRect_AC is then removed from
		// the stack.

	//------------------------------------------------------------------------------------
	// Misc. functions
	//------------------------------------------------------------------------------------

	bool IsFull(void)	{ return (fSize >= fAllocatedSize); }
		// Test if this array is full or not.

	bool IsEmpty(void)	{ 
#if qUseThreads
		CWhileSynchronized_AC entry(&fLock);
#endif
		return (fSize <= 0);
	}
		// Test if this array is empty or not.

	void DoRead(short aRefNum, bool forPrinting);
		// Read the stack size and data from the file.

	void DoWrite(short aRefNum, bool makingCopy);
		// Write the stack size and data to the file.

};

//----------------------------------------------------------------------------------------
// CRectQueue
//----------------------------------------------------------------------------------------

class CRectQueue
{
  protected:
	long fSize;									// number of elements ACTUALLY in the
                                                // array, from 0 to the limit of memory

	long fAllocatedSize;						// Number of elements for which storage is
                                                // ALLOCATED
#if qTuneRectStack
	long fHighWaterMark;						// For tuning
#endif

	long fHead;									// index of first element in the queue

	long fTail;									// index of next free slot in the queue
												// (1 + index of last element)

	CRect_AC *fData;								// Pointer to CRect_AC storage array

#if qUseThreads
	CMonitorLock_AC fLock;
	CMonitorCondition_AC fNotFull;
	bool fFilled;
#endif

	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

  public:
	CRectQueue(long numberOfRects);
		// Initialize a new queue with initialNumberOfRects rects.

	~CRectQueue();
		// Free the queue.


	//------------------------------------------------------------------------------------
	// Array manipulation primitives
	//------------------------------------------------------------------------------------

	long GetSize(void)	{ return fSize; }
		// Returns the ACTUAL number of elements in the queue.

	void DeleteAll(void)	{ fSize = fHead = fTail = 0; }
		// Remove all Rects from the queue. The queue is left empty.

	void PushRect(const CRect_AC& theRect);
		// Takes the CRect_AC you pass it, and appends it to the queue.

	void PushRectSync(const CRect_AC& theRect);
		// Wait if the queue is full, then push the CRect_AC into it.

	void AddRect(const CRect_AC& theRect);
		// Takes the CRect_AC you pass it, and tries to add it to the last CRect_AC in
		// the queue.  If it can't be added, it is just appended to the queue.

	void AddRectSync(const CRect_AC& theRect);
		// Wait if the queue is full, then add the CRect_AC into it.

	void PopRect(CRect_AC& theRect);
		// Returns the first CRect_AC in the queue. That CRect_AC is then removed from
		// the queue.

	//------------------------------------------------------------------------------------
	// Misc. functions
	//------------------------------------------------------------------------------------

	bool IsFull(void)	{ return (fSize >= fAllocatedSize); }
		// Test if this queue is full or not.

	bool IsEmpty(void)	{ return (fSize <= 0); }
		// Test if this queue is empty or not.
};

#endif
