//----------------------------------------------------------------------------------------
// UCalcCells.cp
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCALCCELL__
#include "UCalcCells.h"
#endif

// Calc

#ifndef __UCALC__
#include "UCalc.h"
#endif

#ifndef __UCALCCOLUMNS__
#include "UCalcColumns.h"
#endif

#ifndef __UCALCROWS__
#include "UCalcRows.h"
#endif

// MacApp

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif


//========================================================================================
// Global Functions
//========================================================================================
static CompareResult_AC CompareCells(void* anItem, void* staticLink);

//========================================================================================
// struct CCellComparer
//========================================================================================

struct CCellComparer
{
public:
	GridCoordinate& r;
	GridCoordinate& c;

	// Constructor
	CCellComparer(GridCoordinate& theRow,
				  GridCoordinate& theColumn) :
		r(theRow),
		c(theColumn)
	{
	}
};

//----------------------------------------------------------------------------------------
// CompareCells: 
//----------------------------------------------------------------------------------------

CompareResult_AC CompareCells(void* anItem, void* staticLink)
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	CCellComparer * comparisonInfo = (CCellComparer *)staticLink;

	if (static_cast<TCell*>(anItem)->fLocation.v > comparisonInfo->r)
		returnVal = kItemGreaterThanCriteria_AC;
	else if (static_cast<TCell*>(anItem)->fLocation.v < comparisonInfo->r)
		returnVal = kItemLessThanCriteria_AC;
	else
	{
		if (static_cast<TCell*>(anItem)->fLocation.h > comparisonInfo->c)
			returnVal = kItemGreaterThanCriteria_AC;
		else if (static_cast<TCell*>(anItem)->fLocation.h < comparisonInfo->c)
			returnVal = kItemLessThanCriteria_AC;
	}

	return returnVal;
}

//========================================================================================
// CLASS TCellList
//========================================================================================
MA_DEFINE_CLASS(TCellList);

//----------------------------------------------------------------------------------------
// TCellList destructor
//----------------------------------------------------------------------------------------

TCellList::~TCellList()
{
}

//----------------------------------------------------------------------------------------
// TCellList::Compare: 
//----------------------------------------------------------------------------------------

CompareResult_AC TCellList::Compare(const void* item1,
									const void* item2) const// override 
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	// since this is a cell list and we make sure that everything in it are cells, we
	// can typecast freely without side effects of insomnia

	// row is more important than column when ordering cells
	if (static_cast<const TCell*>((item1))->fLocation.v > static_cast<const TCell*>((item2))->fLocation.v)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (static_cast<const TCell*>((item1))->fLocation.v < static_cast<const TCell*>((item2))->fLocation.v)
		returnVal = kItem1LessThanItem2_AC;
	else
	{											// same row, must check column instead
		if (static_cast<const TCell*>((item1))->fLocation.h > static_cast<const TCell*>((item2))->fLocation.h)
			returnVal = kItem1GreaterThanItem2_AC;
		else if (static_cast<const TCell*>((item1))->fLocation.h < static_cast<const TCell*>((item2))->fLocation.h)
			returnVal = kItem1LessThanItem2_AC;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCellList::GetCell: 
//----------------------------------------------------------------------------------------

TCell* TCellList::GetCell(const GridCell& itsLoc)
{
	GridCell localLoc(itsLoc);
	CCellComparer aCellComparer(localLoc.v, localLoc.h);
	ArrayIndex_AC index;

	return static_cast<TCell*>(DoSearch(::CompareCells, &aCellComparer, index));
}

//----------------------------------------------------------------------------------------
// TCellList::FreeAll: 
//----------------------------------------------------------------------------------------

void TCellList::FreeAll()
{
	for (CTCellIterator iter(this); iter.Current(); ++iter)
		delete iter.Current();

	DeleteAll();
}

//========================================================================================
// CLASS TCell
//========================================================================================
//----------------------------------------------------------------------------------------
// TCell constructors 
//----------------------------------------------------------------------------------------

TCell::TCell(TCalcDocument* owningDocument, const GridCell& itsLoc) :
	fCalcDocument(owningDocument),
	fLocation(itsLoc),
	fKind(kEmptyCell),
	fError(kNoEvalError),
	fValue(0),
	fEvaluating(false),
	fMarked(false),
	fDeleted(false)
{
	::ThrowIfMemError_AC();
}

//----------------------------------------------------------------------------------------
TCell::TCell(const TCell& otherCell) :
	fCalcDocument(otherCell.fCalcDocument),
	fLocation(otherCell.fLocation),
	fKind(otherCell.fKind),
	fError(kNoEvalError),
	fEvaluating(false),
	fValue(otherCell.fValue),
	fValueString(otherCell.fValueString),
	fFormula(otherCell.fFormula),
	fMarked(false),
	fDeleted(false)
{
}

//----------------------------------------------------------------------------------------
// TCell destructor
//----------------------------------------------------------------------------------------

TCell::~TCell()
{
}

//----------------------------------------------------------------------------------------
// TCell::CopyContents: 
//----------------------------------------------------------------------------------------

void TCell::CopyContents(TCell* sourceCell)
{
	fKind = sourceCell->fKind;
	fFormula = sourceCell->fFormula;
}

//----------------------------------------------------------------------------------------
// TCell::EvaluateFormula: 
//----------------------------------------------------------------------------------------

void TCell::EvaluateFormula(bool updateDependencies, bool& cellChanged)
{
	KindOfCell theKind = kEmptyCell;
	EvalResult theError = kNoEvalError;
	ValueType theValue = 0;

	if (updateDependencies)
	{
		// Use nesting to conserve stack space -- the iterator is destroyed before
		// we get here, so the parser and CStr255_AC can be allocated in its place!!

		for (CReferencedCellIterator iter(this); iter.Current(); ++iter)
			iter->RemoveDependentCell(this);
	}

	fValueString.Empty();

	if (!fDeleted)	// MEB
	{
		fEvaluating = true;							// prevent self reference loop 

		CCellParser theParser(this);

		theParser.EvaluateFormula(theKind, theError, theValue, fFormula, updateDependencies);

		fEvaluating = false;
	}

	cellChanged = (theValue != fValue) || (theKind != fKind);
	if (cellChanged)
	{
		fCalcDocument->SetChangeCount(Max_AC(fCalcDocument->GetChangeCount() + 1, 1L));
		if ((theError == kNoEvalError) || (theError == kGarbageAtEnd))
			fValue = theValue;
		fKind = theKind;
		invalidate();
	}
}

//----------------------------------------------------------------------------------------
// TCell::GetAsString: 
//----------------------------------------------------------------------------------------

CStr255_AC TCell::GetAsString()
{
	return fFormula;
}

//----------------------------------------------------------------------------------------
// TCell::GetValueAsString: 
//----------------------------------------------------------------------------------------

CStr255_AC TCell::GetValueAsString()
{
	CStr255_AC returnVal;

	switch (fKind)
	{
		case kTextCell:
			returnVal = fFormula;
			break;

		case kValueCell:
			if (fValueString.IsEmpty())
				ValueToString();
			returnVal = fValueString;
			break;

		case kErrorCell:
		{
			CStr255_AC errorVal;
			NumToString(fError, errorVal);
			returnVal = "**ERROR " + errorVal;
			break;
		}
	}
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCell::GetCoordinates:  Returns a CPoint_AC describing the Cell's location on the GridView
//----------------------------------------------------------------------------------------

GridCell TCell::GetCoordinates()
{
	return fLocation;
}

//----------------------------------------------------------------------------------------
// TCell::invalidate: 
//----------------------------------------------------------------------------------------

void TCell::invalidate()
{
	if (fCalcDocument && fCalcDocument->fCellsView)
		fCalcDocument->fCellsView->InvalidateCell(fLocation);
}

//----------------------------------------------------------------------------------------
// TCell::IsEmpty: 
//----------------------------------------------------------------------------------------

bool TCell::IsEmpty()
{
	return fKind == kEmptyCell && fFormula.Length() == 0;
}

//----------------------------------------------------------------------------------------
// TCell::SetDeleteState: 
//----------------------------------------------------------------------------------------

void TCell::SetDeleteState(bool deleted)
{
	fDeleted = deleted;
}

//----------------------------------------------------------------------------------------
// TCell::SetToString: 
//----------------------------------------------------------------------------------------

void TCell::SetToString(const CStr255_AC& theString)
{
	fFormula = theString;
	fKind = kEmptyCell;						// Force Recalculate to invalidate the cell 
	Recalculate(!kForceAutomatic, kUpdateDependencies);// we want to set new dependents 
}

//----------------------------------------------------------------------------------------
// TCell::Recalculate: 
//----------------------------------------------------------------------------------------

void TCell::Recalculate(bool forceAutomatic, bool updateDependencies)
{
	bool cellChanged;

	EvaluateFormula(updateDependencies, cellChanged);
	if (cellChanged && (forceAutomatic || fCalcDocument->IsAutoCalc()))
		Changed(mValueChanged, this);
}

//----------------------------------------------------------------------------------------
// TCell::GetDependencySpace: 
//----------------------------------------------------------------------------------------

CDependencySpace_AC* TCell::GetDependencySpace()	// Override
{
	return &fCalcDocument->fDependencies;
}

//----------------------------------------------------------------------------------------
// TCell::DoUpdate: 
//----------------------------------------------------------------------------------------

void TCell::DoUpdate(ChangeID_AC theChange,
					 MDependable_AC* changedObject,
					 void* changeData,
					 CDependencySpace_AC* dependencySpace)// override 
{
	bool cellChanged;

	if (theChange == mValueChanged)
	{
		EvaluateFormula(!kUpdateDependencies, cellChanged);
		if (!cellChanged)
			theChange = kNoChange_AC;
	}
	else if (theChange == mWidthChanged)
		theChange = kNoChange_AC;
	else
		MDependable_AC::DoUpdate(theChange, changedObject, changeData, dependencySpace);
}

//----------------------------------------------------------------------------------------
// TCell::GetNoOfDependentCells: 
//----------------------------------------------------------------------------------------

ArrayIndex_AC TCell::GetNoOfDependentCells()
{
	ArrayIndex_AC count = 0;

	for (CDependentCellIterator iter(this); iter.Current(); ++iter)
		++count;

	return count;
}

//----------------------------------------------------------------------------------------
// TCell::GetNoOfReferencedCells: 
//----------------------------------------------------------------------------------------

ArrayIndex_AC TCell::GetNoOfReferencedCells()
{
	ArrayIndex_AC count = 0;

	for (CReferencedCellIterator iter(this); iter.Current(); ++iter)
		++count;

	return count;
}

//----------------------------------------------------------------------------------------
// TCell::ValueToString: Fills fValueString with the CPascalStr_AC representation of the cell's
// value. The "general" format is complicated because we try to ensure that the CPascalStr_AC
// will fit within the cell's column width. Thus we have to figure out the appropriate
// representation for the value.
//----------------------------------------------------------------------------------------

void TCell::ValueToString()
{
	CStr255_AC aString;
	char aCString[255];							// a char* type with preallocated 
												// space.  Used for a temp value later;
	decform theFormat;
	decimal d;
	decimal aDecimal;
	short i;
	short len;
	short sigDigits;
	short leftDigits;
	short rightDigits;
	short width;
	short digitWidth;
	short dotWidth;
	short eWidth;
	short minusWidth;
	short plusWidth;
	TColumn * aColumn;
	ValueType aValueType;


	aColumn = fCalcDocument->GetColumn(fLocation.h);

	switch (aColumn->fFormat.fStyle)
	{
		case kGeneral:
			{
				SetTheFont(aColumn->fFormat.fFontNumber, aColumn->fFormat.fFontSize, aColumn->fFormat.fFontStyle);
				digitWidth = CharWidth('0');
				dotWidth = CharWidth('.');
				eWidth = CharWidth('e');
				minusWidth = CharWidth('-');
				plusWidth = CharWidth('+');
				width = fCalcDocument->fCellsView->GetColWidth(fLocation.h) - (kCellHBorder * 2);

				theFormat = gGeneralFormat;
				aValueType = fValue;
				num2dec(&theFormat, aValueType, &d);
				aString = *((CStr255_AC *)(&d.sig));// sig is actually a CPascalStr_AC, and we want it!

				len = aString.Length();

				if (d.sgn == 1)
					width = width - minusWidth;

				if (d.exp > 0)
				// We've exceeded the precision of our value representation 
				{
					theFormat.style = FLOATDECIMAL;
					// account for '.' and '+e99' 
					width -= dotWidth - CharWidth('+') - eWidth - digitWidth - digitWidth;
					theFormat.digits = Min_AC(kValuePrecision, static_cast<short>(width / digitWidth));
				}
				else
				{

					// determine number of significant digits 
					sigDigits = 1;				// in case all digits are zero 
					for (i = len; i >= 2; --i)	// this was DOWNTO
						if (aString[i] != '0')
						{
							sigDigits = i;
							break;
						}

					leftDigits = kValuePrecision + d.exp;
					rightDigits = sigDigits - leftDigits;

					if (leftDigits < sigDigits)
						width = width - dotWidth;// account for decimal CPoint_AC 

					if (leftDigits * digitWidth > width)
					{
						// We must convert to scientific notation 

						// account for decimal CPoint_AC and 'e9' 
						width = width - dotWidth - eWidth - digitWidth;

						/* For positive numbers, SANE puts a space (' ') before
						  the first significant digit. */
						if (d.sgn != 1)
							width -= CharWidth(' ');
						if (d.exp < -kValuePrecision)
							width -= minusWidth;
						else
							width -= plusWidth;
						if (leftDigits > 10)
							width -= digitWidth;// account for double-digit exponents 

						theFormat.style = FLOATDECIMAL;
						theFormat.digits = Min_AC(sigDigits, static_cast<short>(width / digitWidth));
					}
					else
					{
						theFormat.style = FIXEDDECIMAL;
						theFormat.digits = Min_AC(static_cast<short>((width - (leftDigits * digitWidth) - dotWidth) / digitWidth), rightDigits);
						if (theFormat.digits < 0)
							theFormat.digits = 0;
					}
				}
				break;
			}
		case kDecimalStyle:
			{
				theFormat = gDecimalFormat;
				break;
			}
		case kNoDecimal:
			{
				theFormat = gNoDecimalFormat;
				break;
			}
		case kScientific:
			{
				theFormat = gScientificFormat;
				break;
			}
	}

	aValueType = fValue;
	num2dec(&theFormat, aValueType, &aDecimal);
	dec2str(&theFormat, &aDecimal, aCString);
	aString = aCString;
	//  That should have been "Num2Str(theFormat, aValueType, (decstr)(aString));"
	if (aString[1] == ' ')
		aString.Delete(1, 1);					// Remove leading space 
	fValueString = aString.Copy(1, Min_AC(static_cast<short>(kMaxValueLength), aString.Length()));
}

//----------------------------------------------------------------------------------------
// TCell::GetDiskSize: 
//----------------------------------------------------------------------------------------

short TCell::GetDiskSize(bool infoRecordOnly)
{
	short cellSize = sizeof(CellDiskInfo) - sizeof(CStr255_AC) + fFormula.Length() + 1;
	if (!infoRecordOnly)
		cellSize = cellSize + static_cast<short>(GetNoOfReferencedCells() + GetNoOfDependentCells() + 1) * sizeof(GridCell);

	return cellSize;
}

//----------------------------------------------------------------------------------------
// TCell::ReadFromDisk: 
//----------------------------------------------------------------------------------------

void TCell::ReadFromDisk(TFile* aFile)
{
	short cellLength;

	long count = sizeof(cellLength);
	::ThrowIfOSErr_AC(aFile->ReadData(&cellLength, count));

	count = cellLength;
	CellDiskInfo cellInfo;
	::ThrowIfOSErr_AC(aFile->ReadData(&cellInfo, count));

	fKind = cellInfo.kind;
	fError = cellInfo.error;
	fValue = cellInfo.value;
	fFormula = cellInfo.formula;

	for (short refIndex = 1; refIndex <= cellInfo.noOfReferences; ++refIndex)
	{
		TCell * referencedCell = fCalcDocument->GetCell(ReadCellCoordinate(aFile));
		referencedCell->AddDependentCell(this);
	}
}

//----------------------------------------------------------------------------------------
// TCell::WriteToDisk: 
//----------------------------------------------------------------------------------------

void TCell::WriteToDisk(TFile* aFile)
{
	// write out the data that TCalcDocument.DoRead expects to see 
	WriteCellCoordinateToDisk(aFile);

	short cellLength = GetDiskSize(true);
	long count = sizeof(cellLength);
	::ThrowIfOSErr_AC(aFile->WriteData(&cellLength, count));

	// write out the data that TCell.ReadFromDisk expects to see 
	CellDiskInfo cellInfo;
	cellInfo.kind = fKind;
	cellInfo.error = fError;
	cellInfo.value = fValue;
	cellInfo.noOfReferences = static_cast<short>(GetNoOfReferencedCells());
	cellInfo.formula = fFormula;
	count = cellLength;
	::ThrowIfOSErr_AC(aFile->WriteData(&cellInfo, count));

	for (CReferencedCellIterator iter(this); iter.Current(); ++iter)
		iter->WriteCellCoordinateToDisk(aFile);
}

//----------------------------------------------------------------------------------------
// TCell::WriteCellCoordinateToDisk: 
//----------------------------------------------------------------------------------------

void TCell::WriteCellCoordinateToDisk(TFile* aFile)
{
	long count = sizeof(GridCell);

	::ThrowIfOSErr_AC(aFile->WriteData(&fLocation, count));
}

//----------------------------------------------------------------------------------------
// TCell::ReadFromScrap: 
//----------------------------------------------------------------------------------------

void TCell::ReadFromScrap(Handle theScrap, long& scrapOffset)
{
	short cellLength;
	CellDiskInfo cellInfo;

	ReadScrap(theScrap, scrapOffset, &cellLength, sizeof(cellLength));
	ReadScrap(theScrap, scrapOffset, &cellInfo, cellLength);

	fKind = cellInfo.kind;
	fError = cellInfo.error;
	fValue = cellInfo.value;
	fFormula = cellInfo.formula;

	for (short refIndex = 1; refIndex <= cellInfo.noOfReferences; ++refIndex)
	{
		GridCell cellCoord;
		ReadScrap(theScrap, scrapOffset, &cellCoord, sizeof(GridCell));
		TCell* referencedCell = fCalcDocument->GetCell(cellCoord);
		referencedCell->AddDependentCell(this);
	}
}

//----------------------------------------------------------------------------------------
// TCell::WriteToScrap: 
//----------------------------------------------------------------------------------------

void TCell::WriteToScrap(Handle theScrap, long& scrapOffset)
{
	WriteScrap(theScrap, scrapOffset, &fLocation, sizeof(GridCell));

	short cellLength = GetDiskSize(true);				// don't include ref's && dependents 
	WriteScrap(theScrap, scrapOffset, &cellLength, sizeof(cellLength));

	CellDiskInfo cellInfo;
	cellInfo.kind = fKind;
	cellInfo.error = fError;
	cellInfo.value = fValue;
	cellInfo.noOfReferences = 0;
	cellInfo.formula = fFormula;

	WriteScrap(theScrap, scrapOffset, &cellInfo, cellLength);
}

//========================================================================================
// CLASS TCellEditCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCellEditCommand constructor 
//----------------------------------------------------------------------------------------

TCellEditCommand::TCellEditCommand(TCalcDocument* itsDocument, CommandNumber itsCommand)
:	TCommand(itsCommand, itsDocument, kCanUndo, kCausesChange, itsDocument),
	fSelection(*itsDocument->fCellsView->GetSelections()),
	fCalcDocument(itsDocument)
{
	fChangesClipboard = (itsCommand != cClear);
	fCausesChange = (itsCommand != cCopy);
	fCanUndo = fCausesChange;					// Copy is not undoable
}

//----------------------------------------------------------------------------------------
// TCellEditCommand destructor
//----------------------------------------------------------------------------------------

TCellEditCommand::~TCellEditCommand()			// override 
{
	fClearedCells.FreeAll();
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TCellEditCommand::DoIt()
{
	if (fIdentifier != cClear)
		CopySelection();
	if (fIdentifier != cCopy)
	{
		SaveSelection();
		DeleteSelection();
	}
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::UndoIt: If the user has changed the selection since this command was
// created, restore it before Undoing so that the correct cells are affected.
//----------------------------------------------------------------------------------------

void TCellEditCommand::UndoIt()
{
	if (fIdentifier != cCopy)
	{
		ReSelect();								// restore command's original selection 
		RestoreSelection();
		fCalcDocument->fCellsView->ScrollSelectionIntoView(true);
	}
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::RedoIt: If the user has changed the selection since this command was
// created, restore it before Redoing so that the correct cells are affected.
//----------------------------------------------------------------------------------------

void TCellEditCommand::RedoIt()
{
	if (fIdentifier != cCopy)
	{
		ReSelect();								// restore command's original selection 
		DeleteSelection();
		fCalcDocument->fCellsView->ScrollSelectionIntoView(true);
	}
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::CopySelection: 
//----------------------------------------------------------------------------------------

void TCellEditCommand::CopySelection()
{
	GridRect clipRect(fSelection.GetBounds());
	clipRect.bottom = clipRect.bottom - 1;
	clipRect.right = clipRect.right - 1;

	TCalcDocument * clipDocument = TH_new TCalcDocument(NULL, clipRect);
	clipDocument->DoInitialState();

	{
		CSelectedCellIterator iter(fCalcDocument->fRowsView);
		for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
		{
			TRow * newRow = TH_new TRow(*fCalcDocument->GetRow(aCell.v));
			newRow->fNumber -= clipDocument->fOffset.v;
			clipDocument->AddRow(newRow);
		}
	}

	{
		CSelectedCellIterator iter(fCalcDocument->fColumnsView);
		for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
		{
			TColumn * newColumn = TH_new TColumn(*fCalcDocument->GetColumn(aCell.h));
			newColumn->fNumber -= clipDocument->fOffset.h;
			clipDocument->AddColumn(newColumn);
		}
	}

	{
		for (CSelectedExistingCalcCellIterator iter(fCalcDocument); iter.Current(); ++iter)
		{
			TCell * newCell = TH_new TCell(*iter.Current()); //Clone
			clipDocument->AddCell(newCell, newCell->fLocation - clipDocument->fOffset);
		}
	}

	TCellsView * clipView = TH_new TCellsView(true);
	clipView->HandlePostCreate(clipDocument); //SRF new requirement
	clipView->UpdateFromDocument();				// update view (number of rows, columns, etc)
	clipDocument->fCellsView = clipView;
	ClaimClipboard(clipView);
	clipView->AdjustFrame();
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::DeleteSelection - clear selected cells in document
//----------------------------------------------------------------------------------------

void TCellEditCommand::DeleteSelection()
{
	CSelectedCellIterator iter(fCalcDocument->fCellsView);

	for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
	{
		TCell* destCell = fCalcDocument->GetCell(aCell);
		fCalcDocument->DeleteCell(aCell);
	}

	UpdateViews();
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::RestoreSelection - restore the cut/cleared cells
//----------------------------------------------------------------------------------------

void TCellEditCommand::RestoreSelection()
{
	for (CCalcCellIterator iter(&fClearedCells); iter.Current(); ++iter)
	{
		GridCell aCell = iter->fLocation;
		TCell* destCell = fCalcDocument->GetCell(aCell);
		destCell->CopyContents(iter);
		fCalcDocument->UndeleteCell(aCell);
	}

	UpdateViews();
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::ReSelect: 
//----------------------------------------------------------------------------------------

void TCellEditCommand::ReSelect()
{
	fCalcDocument->fCellsView->ReSelect(&fSelection);
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::SaveSelection - save selected cells in fClearedCells list
//----------------------------------------------------------------------------------------

void TCellEditCommand::SaveSelection()
{
	try
	{
		CSelectedCellIterator iter(fCalcDocument->fCellsView);

		for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
		{
			TCell* destCell = fCalcDocument->GetCell(aCell);
			TCell* clearedCell = TH_new TCell(*destCell);
			fClearedCells.InsertLast(clearedCell);
		}
	}
	catch(CException_AC&)
	{
		fClearedCells.FreeAll();
		throw;
	}
}

//----------------------------------------------------------------------------------------
// TCellEditCommand::UpdateViews 
//----------------------------------------------------------------------------------------

void TCellEditCommand::UpdateViews()
{
	fCalcDocument->DoRecalculate(!kForceAutomatic, !kUpdateDependencies);
	fCalcDocument->fCellsView->InvalidateSelection();

	GridCell firstCell = fCalcDocument->fCellsView->FirstSelectedCell();
	fCalcDocument->SetEntry(firstCell);	// update entry view 
}

//========================================================================================
// CLASS TCellPasteCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCellPasteCommand::TCellPasteCommand: 
//----------------------------------------------------------------------------------------

TCellPasteCommand::TCellPasteCommand(TCalcDocument* itsDocument) :
	TCommand(cPaste, itsDocument, kCanUndo, kCausesChange, itsDocument),

	fSelection(*itsDocument->fCellsView->GetSelections()),
	fCalcDocument(itsDocument),
	fClipDocument(dynamic_cast_AC(TCellsView*,(gClipboardMgr->fClipView))->fCalcDocument)
{
#if qDebug
	if (!dynamic_cast<TCellsView*>(gClipboardMgr->fClipView))
		ProgramBreak_AC("Attempt to paste a non-TCellsView clipboard");
#endif
}

//----------------------------------------------------------------------------------------
// TCellPasteCommand::Free: 
//----------------------------------------------------------------------------------------

TCellPasteCommand::~TCellPasteCommand()			// override 
{
	fReplacedCells.FreeAll();
}

//----------------------------------------------------------------------------------------
// TCellPasteCommand::DoIt: 
//----------------------------------------------------------------------------------------

void TCellPasteCommand::DoIt()
{
	try
	{
		short r = 1;
		short c = 1;

		CSelectedCellIterator iter(fCalcDocument->fCellsView);

		for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
		{
			TCell * sourceCell;
			TCell * replacedCell;
			TCell * destCell;

			destCell = fCalcDocument->GetCell(aCell);
			replacedCell = TH_new TCell(*destCell);
			fReplacedCells.InsertLast(replacedCell);

			sourceCell = fClipDocument->GetCell(GridCell(c, r));
			++c;
			if (c > fClipDocument->fNoOfColumns)
			{
				c = 1;
				++r;
				if (r > fClipDocument->fNoOfRows)
					r = 1;
			}
			destCell->CopyContents(sourceCell);// copy the necessary fields 
		}
	}
	catch(CException_AC&)
	{
		// Extra block to ensure that throw is called in the proper order
		{
			for (CCalcCellIterator iter(&fReplacedCells); iter.Current(); ++iter)
			{
				TCell * pastedCell = fCalcDocument->GetCell(iter->fLocation);
				if (iter->IsEmpty())			// free up memory used by empty cell 
				{
					fCalcDocument->DeleteCell(pastedCell->fLocation);
					fCalcDocument->FreeCell(pastedCell);
				}
				else
					pastedCell->CopyContents(iter);
			}
		}
		fReplacedCells.FreeAll();
		UpdateViews();
		throw;
	}

	UpdateViews();
}

//----------------------------------------------------------------------------------------
// TCellPasteCommand::UndoIt: 
//----------------------------------------------------------------------------------------

void TCellPasteCommand::UndoIt()
{
	fCalcDocument->fCellsView->ReSelect(&fSelection);// restore original selection 

	for (CCalcCellIterator iter(&fReplacedCells); iter.Current(); ++iter)
	{
		TCell * pastedCell = fCalcDocument->GetCell(iter->fLocation);
		pastedCell->CopyContents(iter);
	}

	UpdateViews();
}

//----------------------------------------------------------------------------------------
// TCellPasteCommand::RedoIt: 
//----------------------------------------------------------------------------------------

void TCellPasteCommand::RedoIt()
{
	short r = 1;
	short c = 1;

	fCalcDocument->fCellsView->ReSelect(&fSelection);// restore original selection

	CSelectedCellIterator iter(fCalcDocument->fCellsView);

	for (GridCell aCell = iter.FirstCell(); iter.Current() != gZeroVPt; aCell = ++iter)
	{
		TCell * destCell = fCalcDocument->GetCell(aCell);
		TCell * sourceCell = fClipDocument->GetCell(GridCell(c, r));
		++c;
		if (c > fClipDocument->fNoOfColumns)
		{
			c = 1;
			++r;
			if (r > fClipDocument->fNoOfRows)
				r = 1;
		}

		destCell->CopyContents(sourceCell);		// copy the necessary fields 
	}

	UpdateViews();
}

//----------------------------------------------------------------------------------------
// TCellPasteCommand::UpdateViews: 
//----------------------------------------------------------------------------------------

void TCellPasteCommand::UpdateViews()
{
	fCalcDocument->DoRecalculate(!kForceAutomatic, kUpdateDependencies);
	fCalcDocument->fCellsView->InvalidateSelection();

	GridCell aCell(fCalcDocument->fEditLocation);
	if (fCalcDocument->fCellsView->IsCellSelected(aCell))
	{
		fCalcDocument->fEditCell = fCalcDocument->GetCell(aCell);
		fCalcDocument->fEntryView->SetToString(fCalcDocument->fEditCell->GetAsString());
	}
}

//========================================================================================
// CLASS CCalcCellIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CCalcCellIterator::CCalcCellIterator: 
//----------------------------------------------------------------------------------------

CCalcCellIterator::CCalcCellIterator(TCalcDocument* theCalcDocument) :
	CTCellIterator(theCalcDocument ? &theCalcDocument->fCells : NULL)
{
}

//========================================================================================
// CLASS CSelectedExistingCalcCellIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CSelectedExistingCalcCellIterator::CSelectedExistingCalcCellIterator: 
//----------------------------------------------------------------------------------------

CSelectedExistingCalcCellIterator::CSelectedExistingCalcCellIterator(TCalcDocument* theCalcDocument) :
	CCalcCellIterator(theCalcDocument),
	fSelectedRegion(theCalcDocument->fCellsView->GetSelections())
{
	First();
}

//----------------------------------------------------------------------------------------
// CSelectedExistingCalcCellIterator::Reset: 
//----------------------------------------------------------------------------------------

TCell* CSelectedExistingCalcCellIterator::First()
{
	CCalcCellIterator::First();

	// It doesn't, so we'll get the next  one, which is guaranteed to either meet the criteria, or be NULL
	if (!IsCellSelected(Current()))
		Next();

	return Current();
}

//----------------------------------------------------------------------------------------
// CSelectedExistingCalcCellIterator::Next: 
//----------------------------------------------------------------------------------------

TCell* CSelectedExistingCalcCellIterator::Next()
{
	do
	{
		CCalcCellIterator::Next();

		if (IsCellSelected(Current()))
			break;
	}
	while (Current());

	return Current();
}

//----------------------------------------------------------------------------------------
// CSelectedExistingCalcCellIterator::IsCellSelected: 
//----------------------------------------------------------------------------------------

bool CSelectedExistingCalcCellIterator::IsCellSelected(TCell* theCell)

{
	return Current() ? fSelectedRegion->Contains(theCell->GetCoordinates()) : false;
}

//========================================================================================
// CLASS CExistingCalcCellIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CExistingCalcCellIterator::CExistingCalcCellIterator: 
//----------------------------------------------------------------------------------------

CExistingCalcCellIterator::CExistingCalcCellIterator(TCalcDocument* theCalcDocument) :
	CCalcCellIterator(theCalcDocument),

	fCalcDocument(theCalcDocument)
{
	First();
}

//----------------------------------------------------------------------------------------
// CExistingCalcCellIterator::Reset: 
//----------------------------------------------------------------------------------------

TCell* CExistingCalcCellIterator::First()
{
	CCalcCellIterator::First();

	// It doesn't, so we'll get the next  one, which is guaranteed to either meet the criteria, or be NULL
	if (!CellExists())
		Next();

	return Current();
}

//----------------------------------------------------------------------------------------
// CExistingCalcCellIterator::CellExists: 
//----------------------------------------------------------------------------------------

bool CExistingCalcCellIterator::CellExists()

{
	return Current() ? fCalcDocument->IsExistingCell(Current()) : false;
}

//----------------------------------------------------------------------------------------
// CExistingCalcCellIterator::Advance: 
//----------------------------------------------------------------------------------------

TCell* CExistingCalcCellIterator::Next()
{
	do
		CCalcCellIterator::Next(); 
	while (Current() && !CellExists());

	return Current();
}

//========================================================================================
// CLASS CReferencedCellIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CReferencedCellIterator::CReferencedCellIterator: 
//----------------------------------------------------------------------------------------

CReferencedCellIterator::CReferencedCellIterator(TCell* theCell) :
	CNotifierIterator_AC(theCell, dynamic_cast_AC(CStandardDependencySpace_AC*,theCell->GetDependencySpace()))
{
	First();
}

//----------------------------------------------------------------------------------------
// CReferencedCellIterator::Reset: 
//----------------------------------------------------------------------------------------

TCell* CReferencedCellIterator::First()
{
	CNotifierIterator_AC::First();

	// It doesn't, so we'll get the next  one, which is guaranteed to either meet the criteria, or be NULL
	if (!NotifierIsCell())
		Next();

	return Current();
}

//----------------------------------------------------------------------------------------
// CReferencedCellIterator::Next: 
//----------------------------------------------------------------------------------------

TCell* CReferencedCellIterator::Next()
{
	do
		CNotifierIterator_AC::Next();
	while (Current() && !NotifierIsCell());

	return Current();
}

//----------------------------------------------------------------------------------------
// CReferencedCellIterator::NotifierIsCell: 
//----------------------------------------------------------------------------------------

bool CReferencedCellIterator::NotifierIsCell()
{
	return Current() ? dynamic_cast<TCell*>(Current()) != NULL : false;// is it a cell?
}

//========================================================================================
// CLASS CDependentCellIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CDependentCellIterator::CDependentCellIterator: 
//----------------------------------------------------------------------------------------
CDependentCellIterator::CDependentCellIterator(TCell* theCell) :
	CDependentIterator(theCell, dynamic_cast_AC(CStandardDependencySpace_AC*,theCell->GetDependencySpace()))
{
	First();
}

//----------------------------------------------------------------------------------------
// CDependentCellIterator::Reset: 
//----------------------------------------------------------------------------------------
TCell* CDependentCellIterator::First()
{
	CDependentIterator::Next();

	// It doesn't, so we'll get the next  one, which is guaranteed to either meet the criteria, or be NULL
	if (!DependentIsCell())
		Next();

	return Current();
}

//----------------------------------------------------------------------------------------
// CDependentCellIterator::Next: 
//----------------------------------------------------------------------------------------

TCell* CDependentCellIterator::Next()
{
	do
		CDependentIterator::Next();
	while (Current() && !DependentIsCell());

	return Current();
}

//----------------------------------------------------------------------------------------
// CDependentCellIterator::DependentIsCell: 
//----------------------------------------------------------------------------------------

bool CDependentCellIterator::DependentIsCell()
{
	return Current() ? dynamic_cast<TCell*>(Current()) != NULL : false;// is it a cell?
}

//========================================================================================
// CLASS CParseSubscribedText
//========================================================================================
//----------------------------------------------------------------------------------------
// CParseSubscribedText::CParseSubscribedText: 
//----------------------------------------------------------------------------------------

CParseSubscribedText::CParseSubscribedText(long length,
										   const GridRect& subBounds)
:	fLength(length),
	fSubBounds(subBounds),							// bounds of the subscriber
	fText(NULL),
	fCharPtr(NULL),
	fOffset(0)
{
}

//----------------------------------------------------------------------------------------
// CParseSubscribedText::ParseSubscribedText: 
//----------------------------------------------------------------------------------------

CStr255_AC CParseSubscribedText::ParseSubscribedText(Handle textHdl, GridCell aGridCell)

{
	CStr255_AC parseSubscribedTextResult;			// temporary for the return result

	fText = textHdl;
	if (fLength)
	{
		GridCell normalizedCell = aGridCell - fSubBounds[topLeft];
		fOffset = 0;

		// skip over returns 
		short i;
		for (i = 1; i <= normalizedCell.v; ++i)
		{
			ScanTo(chReturn);
			++fOffset;							// skip over return 
			if (fOffset > fLength)				// there's no more input 
				return parseSubscribedTextResult;
		}

		// is there a relevant data item to read in? 
		if (NumTabs() < normalizedCell.h)
			return parseSubscribedTextResult;

		// skip over tabs 
		for (i = 1; i <= normalizedCell.h; ++i)
		{
			ScanTo(chTab);
			++fOffset;							// skip over tab 
			if (fOffset > fLength)				// there's no more input 
				return parseSubscribedTextResult;
		}

		fCharPtr = reinterpret_cast<unsigned char*>(*fText) + fOffset;// set where to start
		long maxLength = Min_AC(fLength - fOffset, kStr255Len);
		fOffset = 0;
		char aChar = *(fCharPtr + fOffset);		// get the "fOffset"th character

		parseSubscribedTextResult.Empty();

		while (((fOffset < maxLength) && (aChar != chTab) && (aChar != chReturn)))
		{
			++fOffset;
			parseSubscribedTextResult += aChar;	// save the current character
			aChar = *(fCharPtr + fOffset);		// get the next one
		}
	}

	return parseSubscribedTextResult;
}

//----------------------------------------------------------------------------------------
// CParseSubscribedText::ScanTo: 
//----------------------------------------------------------------------------------------

void CParseSubscribedText::ScanTo(char ch)
{
	unsigned char* p = reinterpret_cast<unsigned char*>(*fText) + fOffset;
	while ((*p != ch) && (fOffset < fLength))
	{
		fOffset = fOffset + 1;
		p = reinterpret_cast<unsigned char*>(*fText) + fOffset;
	}
}

//----------------------------------------------------------------------------------------
// CParseSubscribedText::NumTabs: 
//----------------------------------------------------------------------------------------

short CParseSubscribedText::NumTabs()
// number of tabs between *fText + localOffset and the next chReturn 
{
	long localOffset = fOffset;
	short num = 0;
	unsigned char* p = reinterpret_cast<unsigned char*>(*fText) + localOffset;
	while (*p != chReturn && localOffset < fLength)
	{
		if (*p == chTab)
			++num;
		++localOffset;
		p = reinterpret_cast<unsigned char*>(*fText) + localOffset;
	}

	return num;
}

//========================================================================================
// CLASS CCellParser
//========================================================================================
//----------------------------------------------------------------------------------------
// CCellParser::CCellParser: 
//----------------------------------------------------------------------------------------

CCellParser::CCellParser(TCell* theCell)
{
	fCell = theCell;
	fCalcDocument = fCell->fCalcDocument;
}

//----------------------------------------------------------------------------------------
// CCellParser::GetNextChar: 
//----------------------------------------------------------------------------------------

void CCellParser::GetNextChar()
{
	do
	{
		if (fFormulaIndex < fFormulaLength)
		{
			++fFormulaIndex;
			fChar = fFormula[fFormulaIndex];	// get the "fFormulaIndex"th character of fFormula
		}
		else
			fChar = 0;

	} while (fChar == ' ');

	if ((fChar >= 'a') && (fChar <= 'z'))
		fChar = fChar - 32;
}

//----------------------------------------------------------------------------------------
// CCellParser::DoInteger: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::DoInteger(short& theInteger)
{
	theInteger = 0;
	while (IsDigit(fChar))
	{
		theInteger = theInteger * 10 + fChar - '0';
		GetNextChar();
	}

	return kNoEvalError;
}

//----------------------------------------------------------------------------------------
// CCellParser::DoNumber: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::DoNumber(ValueType& theValue)
{
	// DoNumber is called only when fChar is a numeric digit or a decimal CPoint_AC 
	decimal decimalNumber;
	short IsValidNumber = false;				// this is because it is passed in as a
												// reference in str2dec and str2dec's
												// interface is dumb and requires a
												// reference to a short.
	bool startsWithDecPt = (fChar == '.');
	short newIndex = fFormulaIndex - 1;

	str2dec(fFormula, &newIndex, &decimalNumber, &IsValidNumber);
	// Str2Dec returns IsValidNumber == false if the digits are followed by non-numerics like * 

	if (startsWithDecPt && (!IsValidNumber))
	{
		GetNextChar();
		if (!IsDigit(fChar))
			return kBadNumber;
	}
	fFormulaIndex = newIndex;
	GetNextChar();
	theValue = dec2num(&decimalNumber);

	return kNoEvalError;
}

//----------------------------------------------------------------------------------------
// CCellParser::DoCellReference: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::DoCellReference(ValueType& theValue)
{
	EvalResult theResult = kNoEvalError;
	short r;									// Can't use GridCoordinate because it may be out of range 

	GridRect aRect = fCalcDocument->fDimensions;			// valid coordinates 
	short c = 0;
	while ((fChar >= 'A') && (fChar <= 'Z') && (c <= aRect.right))
	{
		c = c * 26 + fChar - 'A' + 1;
		GetNextChar();
	}
	if (c > aRect.right)						// column is out of range, so 
		r = 0;									// don't bother looking for row number 
	else
		theResult = DoInteger(r);
	if (fCalcDocument->CellInRange(r, c, aRect))
	{
		GridCell refLoc = GridCell(c, r) - fCalcDocument->fOffset;
		TCell* referencedCell = fCalcDocument->GetExistingCell(refLoc);
		if (!referencedCell)					// cell doesn't exist
		{
			theValue = 0;
			if (fUpdateDependencies)
			{
				referencedCell = fCalcDocument->GetCell(refLoc);	// create cell so we can setup a dependency
				referencedCell->AddDependentCell(fCell);
			}
		}
		else if (referencedCell->IsDeleted())	// cell exists, but was deleted
		{
			theValue = 0;
		}
		else
		{
			if (referencedCell->fEvaluating)
				theResult = kSelfReference;
			else if (referencedCell->fKind == kErrorCell)
				theResult = kErrorCellReference;
			else if (referencedCell->fKind == kTextCell)
				theValue = 0;
			else
				theValue = referencedCell->fValue;

			if (fUpdateDependencies && (theResult != kSelfReference))
				referencedCell->AddDependentCell(fCell);
		}
	}
	else
		theResult = kBadCellReference;

	return theResult;
}

//----------------------------------------------------------------------------------------
// CCellParser::Term: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::Term(ValueType& theValue)
{
	EvalResult theTempResult;

	if (fChar == '(')
	{
		GetNextChar();
		theTempResult = Expression(theValue);
		if (fChar == ')')
			GetNextChar();
		else
			theTempResult = kMissingRightParen;
	}
	else if (IsDigit(fChar) || (fChar == '.'))
		theTempResult = DoNumber(theValue);
	else if ((fChar >= 'A') && (fChar <= 'Z'))
		theTempResult = DoCellReference(theValue);
	else if (fChar == '+')
	{
		GetNextChar();
		theTempResult = Term(theValue);
	}
	else if (fChar == '-')
	{
		GetNextChar();
		theTempResult = Term(theValue);
		theValue = -theValue;
	}
	else
		theTempResult = kIllegalCharacter;

	return theTempResult;
}

//----------------------------------------------------------------------------------------
// CCellParser::Factor: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::Factor(ValueType& theValue)
{
	ValueType factorValue;

	EvalResult theResult = Term(theValue);
	if (theResult == kNoEvalError)
		if (fChar == '*')
		{
			GetNextChar();
			theResult = Factor(factorValue);
			theValue *= factorValue;
		}
		else if (fChar == '/')
		{
			GetNextChar();
			theResult = Factor(factorValue);
			theValue /= factorValue;
		}

	return theResult;
}

//----------------------------------------------------------------------------------------
// CCellParser::Expression: 
//----------------------------------------------------------------------------------------

EvalResult CCellParser::Expression(ValueType& theValue)
{
	ValueType factorValue;

	EvalResult factorResult = Factor(theValue);
	if (factorResult == kNoEvalError)
		do
		{
			if (fChar == '+')
			{
				GetNextChar();
				factorResult = Factor(factorValue);
				theValue += factorValue;
			}
			else if (fChar == '-')
			{
				GetNextChar();
				factorResult = Factor(factorValue);
				theValue -= factorValue;
			}
		} while ((fChar == '+') || (fChar == '-'));

	return factorResult;
}

//----------------------------------------------------------------------------------------
// CCellParser::EvaluateFormula: 
//----------------------------------------------------------------------------------------

void CCellParser::EvaluateFormula(KindOfCell& theKind,
								  EvalResult& theError,
								  ValueType& theValue,
								  CStr255_AC& theFormula,
								  bool doUpdateDependencies)
{
	fFormula = theFormula;
	fUpdateDependencies = doUpdateDependencies;
	fFormulaLength = fFormula.Length();
	fFormulaIndex = 0;
	GetNextChar();

	if (IsDigit(fChar) || fChar == '=' || fChar == '-' || fChar == '+' || fChar == '.')
	{
		if (fChar == '=')
			GetNextChar();

		theError = Expression(theValue);

		if (theError == kNoEvalError)
			if (fFormulaIndex <= fFormulaLength && fChar)
				theError = kGarbageAtEnd;

		theKind = (theError == kNoEvalError) ? kValueCell : kErrorCell;
	}
	else
		theKind = kTextCell;
}

//----------------------------------------------------------------------------------------
// End of UCalcCells.cp


