// UCalcCells.h
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __UCALCCELL__
#define __UCALCCELL__

//
// INTERNAL INCLUDES FOR CALC
//

#ifndef __CALCUTILITIES__
#include "CalcUtilities.h"
#endif

// ACS
#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp
#ifndef __UGRIDVIEW__
#include "UGridView.h"
#endif

class TCell;
class TCellList;
class TCalcDocument;

enum
{
	kCellWidth = 80,							// default width of each cell 
	kCellHeight = 17,							// default height of each cell 
	kCellHBorder = 2							// pixels separating cell contents from left/right edge 
};

typedef double DiskValueType;					// extendeds are for internal use only

enum KindOfCell
{
	kEmptyCell, kValueCell, kTextCell, kErrorCell
};

enum EvalResult
{
	kNoEvalError, kMissingRightParen, kSelfReference, kErrorCellReference, kBadNumber, kIllegalCharacter, kBadCellReference, kGarbageAtEnd
};

enum TypeOfStyle
{
	kNoStyle, kGeneral, kDecimalStyle, kNoDecimal, kScientific, kUnknownStyle
};

enum CalcMode
{
	kManual, kAutomatic
};

enum TypeOfSelection
{
	kNoSelection, kCellSelection, kRowSelection, kColumnSelection, kAllSelection
};


typedef double ValueType;
typedef CStr31_AC ValueString;						// This isn't exactly what it used to be
// Should be constrained by kMaxValueLength!!!!!

//============================================================================================
// Data read from or written to disk files or exchanged via clip must be built to use the same
// alignment. We choose to center on mac68K alignment. In your application you may choose any
// data alignment that you wish, so long as both your PPC and 68K code agree on it.
//============================================================================================
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct FormatRecord
{
	TypeOfStyle fStyle;
	SignedByte fJustification;
	char fDigits;
	char fFontNumber;
	char fFontSize;
	Style fFontStyle;
};
typedef FormatRecord * FormatPtr;

struct CalcDocDiskInfo
{
	GridRect dimensions;
	GridCoordinate allocatedCells;
	GridCoordinate allocatedRows;
	GridCoordinate allocatedColumns;
	GridCoordinate editRow;
	GridCoordinate editColumn;
	CalcMode calcMode;
	TypeOfSelection selectionType;
};

// Info about the document's rows
struct RowDiskInfo
{
	GridCoordinate number;
};

// Info about the document's columns
struct ColumnDiskInfo
{
	FormatRecord format;
	GridCoordinate number;
	ViewCoordinate width;
};

// The document's cells 
// row/column coordinate read/written separately
struct CellDiskInfo
{
	short noOfReferences;
	DiskValueType value;
	KindOfCell kind;
	EvalResult error;
	CStr255_AC formula;
};


struct ScrapInfoRecord
{
	GridRect selection;
	GridCoordinate noOfCells;
};

// row/column coordinate read/written separately 
// cell length read/written separately 
struct CellScrapInfo
{
	FormatRecord format;
	CStr255_AC formula;
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

//--------------------------------------------------------------------------------------------------
// TCellList:
//--------------------------------------------------------------------------------------------------

class TCellList : public CList_AC
{
	MA_DECLARE_CLONE;
	
public:
	inline TCellList() : CList_AC() { }

	virtual ~TCellList();
		// Destructor

	virtual CompareResult_AC Compare(const void* item1,
								 const void* item2) const;	// override 
	// Compares cells based on row and column number 

	TCell* GetCell(const GridCell& itsLoc);
	// Returns the cell whose row,column is r,c 

	void FreeAll();
		// Frees all the cells and then deletes all the elements
};


//--------------------------------------------------------------------------------------------------
// TCell: TCell describes one element of the spreadsheet grid. 
//--------------------------------------------------------------------------------------------------

class TCell : public MDependable_AC
{
public:
	TCell(TCalcDocument* owningDocument = NULL,
		  const GridCell& itsLoc = GridCell(gZeroPt));		
	// Constructor
	
	TCell(const TCell& otherCell);
	// Copy constructor

	virtual ~TCell();			// override 
	// Free my lists and then free me 

	virtual void CopyContents(TCell* sourceCell);
	// Copy contents of sourceCell 

	virtual void EvaluateFormula(bool updateDependencies,
								 bool& cellChanged);
	/* Calculate my value by evaluating fFormula. If updateDependencies is true, add me
	  to the list of dependents of any cells referenced in my formula. Return true in
	  cellChanged if my fValue or fKind has changed */

	CStr255_AC GetAsString();
	// Return my formula CPascalStr_AC 

	CStr255_AC GetValueAsString();
	// Return a CPascalStr_AC representation of my value 

	virtual void invalidate();
	// Cause a cell to be marked invalid (in need of re-drawing). 

	virtual GridCell GetCoordinates();
	// Returns a CPoint_AC describing the Cell's location on the GridView

	virtual bool IsEmpty();
	// Return true if I have no formula or value 

	bool IsDeleted() { return fDeleted; }
	// Return true if this cell is deleted

	virtual void SetDeleteState(bool deleted);
	// Mark a cell as deleted or not deleted 

	virtual void SetToString(const CStr255_AC& theString);
	// Set my formula to theString and recalculate my value if necessary 

	virtual void Recalculate(bool forceAutomatic,
							 bool updateDependencies);
	// Re-evaluate my formula CPascalStr_AC. Notify dependent cells of the change 

	virtual void DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void* changeData,
						  CDependencySpace_AC* dependencySpace);// Override 
	/* Update this cell in response to a change in a cell referenced in this
	  cell's formula */

	virtual ArrayIndex_AC GetNoOfDependentCells();
	// Returns the size of the dependent list 

	virtual ArrayIndex_AC GetNoOfReferencedCells();
	// Returns the size of the dependent list 

	virtual CDependencySpace_AC* GetDependencySpace(); // Override
	// Overridden to return the document's dependency space

	inline void AddDependentCell(TCell* aCell)
	{ AddDependent(aCell); }
	// Add aCell to this cell's list of dependents 
	
	void RemoveDependentCell(TCell* aCell)
	{ RemoveDependent(aCell); }
	// Remove aCell from this cell's list of dependents 

	virtual void ValueToString();
	// Fill fValueString with the CPascalStr_AC representation of my value 

	virtual short GetDiskSize(bool infoRecordOnly);

	virtual void ReadFromDisk(TFile* aFile);

	virtual void WriteToDisk(TFile* aFile);

	virtual void WriteCellCoordinateToDisk(TFile* aFile);

	virtual void ReadFromScrap(Handle theScrap, long& scrapOffset);

	virtual void WriteToScrap(Handle theScrap, long& scrapOffset);

public:
	TCalcDocument*	fCalcDocument;				// The document I belong to 

	GridCell		fLocation;					// Which row/column I'm in 

	KindOfCell		fKind;						// What kind of cell: text, value, empty, error
	EvalResult		fError;						// Error code from my latest evaluation 
	bool			fEvaluating;				// flag for preventing self reference loops
	ValueType		fValue;						// My value represented as a number 
	ValueString		fValueString;				// My value represented as a CPascalStr_AC, for display
	CStr255_AC		fFormula;					// My contents as typed in by the user 
	bool			fMarked;					// !!!RCR For change notification 
	bool			fDeleted;					// Whether the cell has been deleted 
};												// TCell 


//--------------------------------------------------------------------------------------------------
// TCellEditCommand: TCellEditCommand is a command object created to handle the cCopy,
// cCut, and cClear menu commands.
//--------------------------------------------------------------------------------------------------

class TCellEditCommand : public TCommand
{
public:
	TCellEditCommand(TCalcDocument* itsDocument = NULL,
					 CommandNumber itsCommand = cNoCommand);
	// Construct the command object. Save the current cell selection for later Undo/Redo

	virtual ~TCellEditCommand();
	// Clean up our stuff 

	virtual void DoIt();			// override 
	// Edit the currently selected cells 

	virtual void UndoIt();			// override 
	// Undo the edit of the cells 

	virtual void RedoIt();			// override 
	// Redo the edit of the cells 

	virtual void CopySelection();
	// Copy the currently selected cell(s) into the clipboard 

	virtual void DeleteSelection();
	// Delete the currently selected cell(s) 

	virtual void RestoreSelection();
	// Restore the deleted cells 

	virtual void ReSelect();
	// Change the selection back to what it was when this command was created 

	virtual void SaveSelection();
	// Save the currently selected cell(s) for later undo

	virtual void UpdateViews();
	// Redisplay the affected cells 

public:
	TGridViewDesignator	fSelection;				// cells selected when command was created 
	TCellList			fClearedCells;			// the cells we cleared (cCut, cClear) 
	TCalcDocument*		fCalcDocument;
};


//--------------------------------------------------------------------------------------------------
// TCellPasteCommand: A command object created to handle the cPaste menu command. 
//--------------------------------------------------------------------------------------------------

class TCellPasteCommand : public TCommand
{
public:
	TCellPasteCommand(TCalcDocument* itsDocument = NULL);
	// Initialize the command object 

	virtual ~TCellPasteCommand();			// override 
	// We don' need this command no more 

	virtual void DoIt();			// override 
	// Paste to the currently selected cells. Save this selection for Undo/Redo. 

	virtual void UndoIt();			// override 
	// Undo the paste of the cells 

	virtual void RedoIt();			// override 
	// Redo the paste of the cells 

	virtual void UpdateViews();
	// Redisplay the affected cells 

public:
	TCellList fReplacedCells;		// the cells we pasted over 
	TGridViewDesignator fSelection;					// cells selected when command was created 

	TCalcDocument*	fClipDocument;
	TCalcDocument*	fCalcDocument;
};

//--------------------------------------------------------------------------------------------------
typedef TemplateIterator_AC<TCell*> CTCellIterator;

class CCalcCellIterator : public CTCellIterator
{
public:
	CCalcCellIterator(CDynamicArray_AC* inList) :
	CTCellIterator(inList)
	{ }

	CCalcCellIterator(TCalcDocument* theCalcDocument);
};


//--------------------------------------------------------------------------------------------------
class CExistingCalcCellIterator : public CCalcCellIterator
{
public:
	CExistingCalcCellIterator(TCalcDocument* theCalcDocument);
	
	//
	// Required functions 
	//
	TCell* First();
	inline TCell* Current() const { return static_cast<TCell*>(CCalcCellIterator::Current()); }
	TCell* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TCell* operator++() { return Next(); }
	inline TCell* operator++(int) { TCell* returnVal = Current(); Next(); return returnVal; }
	inline operator TCell*() const { return Current(); }
	inline TCell* operator->() const { return Current(); }

protected:
	bool CellExists();

protected:
	TCalcDocument* fCalcDocument;
};


//--------------------------------------------------------------------------------------------------
class CSelectedExistingCalcCellIterator : public CCalcCellIterator
{
public:
	CSelectedExistingCalcCellIterator(TCalcDocument* theCalcDocument);

	//
	// Required functions 
	//
	TCell* First();
	inline TCell* Current() const { return static_cast<TCell*>(CCalcCellIterator::Current()); }
	TCell* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TCell* operator++() { return Next(); }
	inline TCell* operator++(int) { TCell* returnVal = Current(); Next(); return returnVal; }
	inline operator TCell*() const { return Current(); }
	inline TCell* operator->() const { return Current(); }

protected:
	bool IsCellSelected(TCell* theCell);

protected:
	TGridViewDesignator* fSelectedRegion;
};


//--------------------------------------------------------------------------------------------------
class CReferencedCellIterator : public CNotifierIterator_AC
{
public:
	CReferencedCellIterator(TCell* theCell);
	
	//
	// Required functions 
	//
	TCell* First();
	inline TCell* Current() const { return static_cast<TCell*>(CNotifierIterator_AC::Current()); }
	TCell* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TCell* operator++() { return Next(); }
	inline TCell* operator++(int) { TCell* returnVal = Current(); Next(); return returnVal; }
	inline operator TCell*() const { return Current(); }
	inline TCell* operator->() const { return Current(); }

protected:
	bool NotifierIsCell();
};


//--------------------------------------------------------------------------------------------------
class CDependentCellIterator : public CDependentIterator
{
public:
	CDependentCellIterator(TCell* theCell);
	
	//
	// Required functions 
	//
	TCell* First();
	inline TCell* Current() const { return dynamic_cast<TCell*>(CDependentIterator::Current()); }
	TCell* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TCell* operator++() { return Next(); }
	inline TCell* operator++(int) { TCell* returnVal = Current(); Next(); return returnVal; }
	inline operator TCell*() const { return Current(); }
	inline TCell* operator->() const { return Current(); }

protected:
	bool DependentIsCell();
};

//--------------------------------------------------------------------------------------------------
class CParseSubscribedText
{
public:
	CParseSubscribedText(long length,
						 const GridRect& subBounds);	// bounds of the subscriber 

	virtual CStr255_AC ParseSubscribedText(Handle	textHdl, GridCell	aGridCell);

private:
	GridRect		fSubBounds;					// bounds of the subscriber 
	Handle			fText;
	long			fLength;
	unsigned char*	fCharPtr;
	long			fOffset;
	
	virtual void ScanTo(char ch);

	virtual short NumTabs();
};

//--------------------------------------------------------------------------------------------------
class CCellParser
{												// takes the text of a cell and evaluates its contents
public:
	CCellParser(TCell* theCell);

	void EvaluateFormula(KindOfCell&	theKind,
						 EvalResult&	theError,
						 ValueType&		theValue,
						 CStr255_AC&	theFormula,
						 bool			doUpdateDependencies);

protected:
	TCell*			fCell;
	TCalcDocument*	fCalcDocument;
	ValueString		fValueString;
	CStr255_AC		fFormula;
	short			fFormulaLength;
	short			fFormulaIndex;
	char			fChar;
	bool			fUpdateDependencies;

	void GetNextChar();
	
	EvalResult DoInteger(short& theInteger);

	EvalResult DoNumber(ValueType& theValue);

	EvalResult DoCellReference(ValueType& theValue);

	EvalResult Term(ValueType& theValue);

	EvalResult Factor(ValueType& theValue);

	EvalResult Expression(ValueType& theValue);
};


#endif

