// UCalcColumns.h
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved. 

#ifndef __UCALCCOLUMNS__
#define __UCALCCOLUMNS__

//
// INTERNAL INCLUDES FOR CALC
//

#ifndef __UCALCCELLS__
#include "UCalcCells.h"
#endif

class TCalcDocument;
class TColumn;
class TColumnsView;
class TColumnList;
class TCellsView;
class CColumnIterator;

//--------------------------------------------------------------------------------------------------
// TColumnsView: TColumnsView is used to show the column headers, represented by the
// letters A-BL. Click in this view to select an entire column or group of columns.
//--------------------------------------------------------------------------------------------------

class TColumnsView : public TTextGridView
{
	MA_DECLARE_CLONE;
	
public:

	TCalcDocument* fCalcDocument;

	virtual ~TColumnsView();
		// Destructor
	
	virtual void DoPostCreate(TDocument* itsDocument);// override 
		// Create the view from a resource template

	virtual void AdornCol(GridCoordinate aCol, const CViewRect&	/*area*/);// override 
		// Draw a column delimiter in the given area 

	virtual void CoordToString(GridCoordinate coord, CStr255_AC& theString);

	virtual void DoMouseCommand(CViewPoint& theMouse,
									   TToolboxEvent*	event,
									   CPoint_AC			/*hysteresis*/);// override 

	virtual void DoSetCursor(const CViewPoint&	localPoint,
									RgnHandle		cursorRegion);// override 

	virtual void DrawCell(const GridCell&		aCell,
								 const CViewRect&	aRect);// override 

	virtual void ReSelect(const TGridViewDesignator* cellRegion);

};												// TColumnsView 


//--------------------------------------------------------------------------------------------------
// TColumn:
//--------------------------------------------------------------------------------------------------

class TColumn
{
public:
	TColumn(GridCoordinate number = 0);		
		// Constructor
	
	virtual ~TColumn();
		// Destructor

	virtual void ReadFromDisk(TFile* aFile);

	virtual void WriteToDisk(TFile* aFile);

	virtual void ReadFromScrap(Handle	theScrap,
									  long&		scrapOffset);

	virtual void WriteToScrap(Handle	theScrap,
									 long&	scrapOffset);

public:

	GridCoordinate fNumber;
	FormatRecord fFormat;
	FormatRecord fOldFormat;					// save original format for Undo 
	ViewCoordinate fWidth;
};


//--------------------------------------------------------------------------------------------------
// TColumnList:
//--------------------------------------------------------------------------------------------------

class TColumnList : public CSortedList_AC
{
	MA_DECLARE_CLONE;
	
public:
	inline TColumnList() : CSortedList_AC() { }
	// Constructor 

	virtual ~TColumnList();
		// Destructor
	
	virtual CompareResult_AC Compare(const void* item1,
										  const void* item2) const;// override 
	// Compares columns based on columns number 

	virtual TColumn* GetColumn(GridCoordinate c);
	// Returns the column whose number is c 

	void FreeAll();
		// Frees all the columns and then deletes all the elements
};


//--------------------------------------------------------------------------------------------------
// TColumnFormatter: TColumnFormatter is a command object created to change the style or
// justification of a column. It is not undoable.
//--------------------------------------------------------------------------------------------------

class TColumnFormatter : public TCommand
{
public:
	TColumnFormatter(TCalcDocument* itsDocument = NULL,
								   CommandNumber itsCommand = cNoCommand);

	virtual ~TColumnFormatter();				// override 

	virtual void DoIt();				// override 
	virtual void UndoIt();			// override 
	virtual void RedoIt();			// override 

protected:
	void SaveFormat(const GridCell& aCell);

	void SetFormat(const GridCell& aCell);

	void RestoreFormat(const GridCell& aCell);

public:
	TGridViewDesignator		fSelection;						// columns selected for formatting 

	TCalcDocument*	fCalcDocument;
};


//--------------------------------------------------------------------------------------------------
// TColumnSizer: TColumnSizer is a command object created to handle mouse movement when
// you click on the boundary between two columns.
//--------------------------------------------------------------------------------------------------

class TColumnSizer : public TTracker
{
public:
	TColumnSizer(TCalcDocument* itsDocument = NULL,
									 GridCoordinate	c = 0,
									 const CViewPoint& theMouse = gZeroVPt);

	virtual ~TColumnSizer();
		// Destructor
	
	virtual TTracker* TrackMouse(TrackPhase	/*aTrackPhase*/,
										CViewPoint&		/*anchorPoint*/,
										CViewPoint&		/*previousPoint*/,
										CViewPoint&		nextPoint,
										bool		/*mouseDidMove*/);

	virtual void TrackFeedback(TrackPhase	/*aTrackPhase*/,
									  const CViewPoint&	/*anchorPoint*/ ,
									  const CViewPoint&	/*previousPoint*/ ,
									  const CViewPoint&	nextPoint,
									  bool		mouseDidMove,
									  bool		/*turnItOn*/);// override 

	virtual void TrackConstrain(TrackPhase		/*aTrackPhase*/,
									   const CViewPoint&	/*anchorPoint*/ ,
									   const CViewPoint&	/*previousPoint*/ ,
									   CViewPoint&			nextPoint,
									   bool			/*mouseDidMove*/);// override 

	virtual void DoIt();				// override 

	virtual void UndoIt();			// override 

	virtual void SetColumnWidth(ViewCoordinate newWidth);

public:
	TCalcDocument*	fCalcDocument;
	TCellsView*		fCellsView;
	TColumn*		fColumn;

	ViewCoordinate			fLeftEdge;
	ViewCoordinate			fNewWidth;
	ViewCoordinate			fOldWidth;				// remember previous width for Undo 
 };


//--------------------------------------------------------------------------------------------------
typedef TemplateIterator_AC<TColumn*> CTColumnIterator;

class CCalcColumnIterator : public CTColumnIterator
{
public:
	inline CCalcColumnIterator(TColumnList* theColumnList) :
	CTColumnIterator(theColumnList)
	{ }

	CCalcColumnIterator(TCalcDocument* theCalcDocument);

};

#endif  //UCALCCOLUMNS

