//----------------------------------------------------------------------------------------
// UStructureInspectors.cp
//  Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USTRUCTUREINSPECTORS__
#include "UStructureInspectors.h"
#endif

// MacApp

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif


//========================================================================================
// Global Functions
//========================================================================================
static bool ListsMatch(CList_AC* firstList,
						  CList_AC* secondList);
static short GetDepth(TView* aView);
static void FlattenTargetChain(TEventHandler* head,
							   CList_AC* theList);

//========================================================================================
// CLASS TIdler
//========================================================================================
//----------------------------------------------------------------------------------------
// TIdler constructor 
//----------------------------------------------------------------------------------------
TIdler::TIdler(TEventHandler* itsHandler) :
	TEventHandler(NULL),

	fHandler(itsHandler)
{
	SetIdleFreq(fHandler->GetIdleFreq());
}

//----------------------------------------------------------------------------------------
// TIdler destructor
//----------------------------------------------------------------------------------------
TIdler::~TIdler()
{
}

//----------------------------------------------------------------------------------------
// TIdler::DoIdle: 
//----------------------------------------------------------------------------------------
bool TIdler::DoIdle(IdlePhase phase)			// override 
{
	return fHandler ? fHandler->DoIdle(phase) : false;
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// ListsMatch: 
//----------------------------------------------------------------------------------------
bool ListsMatch(CList_AC* firstList,
				   CList_AC* secondList)
{
	bool returnVal = false;

	if (firstList->GetSize() == secondList->GetSize())
	{
		for (ArrayIndex_AC i = 1; i <= firstList->GetSize(); ++i)
			if (firstList->At(i) != secondList->At(i))
				break;

		returnVal = true;
	}
	
	return returnVal;
}


//========================================================================================
// CLASS TObjectListView
//========================================================================================
//----------------------------------------------------------------------------------------
// TObjectListView constructor 
//----------------------------------------------------------------------------------------
TObjectListView::TObjectListView() :
	fDisplayedList(NULL)
{
}

//----------------------------------------------------------------------------------------
// TObjectListView::DoPostCreate: 
//----------------------------------------------------------------------------------------
void TObjectListView::DoPostCreate(TDocument* itsDocument)// override 
{
	TTextListView::DoPostCreate(itsDocument);

	fAList = NewList_AC();
	fBList = NewList_AC();

	fIdler = TH_new TIdler(this);

	fDisplayedList = fAList;
}

//----------------------------------------------------------------------------------------
// TObjectListView::Free: 
//----------------------------------------------------------------------------------------
TObjectListView::~TObjectListView()				// Override 
{
}

//----------------------------------------------------------------------------------------
// TObjectListView::BuildList: 
//----------------------------------------------------------------------------------------
void TObjectListView::BuildList(CList_AC* newList)
{
	newList->DeleteAll();
}

//----------------------------------------------------------------------------------------
// TObjectListView::IsSynchronized: 
//----------------------------------------------------------------------------------------
bool TObjectListView::IsSynchronized()
{
	CList_AC * theNewList;
	CList_AC * theOldList;

	if (fDisplayedList == fAList)
	{
		theNewList = fBList;
		theOldList = fAList;
	}
	else
	{
		theNewList = fAList;
		theOldList = fBList;
	}

	BuildList(theNewList);

	return ListsMatch(theOldList, theNewList);
}

//----------------------------------------------------------------------------------------
// TObjectListView::Synchronize: 
//----------------------------------------------------------------------------------------
void TObjectListView::Synchronize(bool		/* redraw */)
{
	CList_AC * theNewList;
	CList_AC * theOldList;

	if (fDisplayedList == fAList)
	{
		theNewList = fBList;
		theOldList = fAList;
	}
	else
	{
		theNewList = fAList;
		theOldList = fBList;
	}

	BuildList(theNewList);
	if (!ListsMatch(theOldList, theNewList))
	{
		void * selectedObject = GetSelectedObject();

		if (theOldList->GetSize() != theNewList->GetSize())
		{
			DelItemLast(fNumOfRows);
			InsItemLast(static_cast<GridCoordinate>(theNewList->GetSize()));
		}
		fDisplayedList = theNewList;
		if (selectedObject)
			SelectObject(selectedObject);
		ScrollSelectionIntoView(!kRedraw);
		ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TObjectListView::Draw: 
//----------------------------------------------------------------------------------------
void TObjectListView::Draw(const CViewRect& area)	// Override 
{
	if (IsSynchronized())
		TTextListView::Draw(area);
	// Otherwise, let next idle synchronize
}

//----------------------------------------------------------------------------------------
// TObjectListView::DoIdle: 
//----------------------------------------------------------------------------------------
bool TObjectListView::DoIdle(IdlePhase		/*phase*/)// override 
{
	Synchronize(kRedraw);

	return false;
}

//----------------------------------------------------------------------------------------
// TObjectListView::Open: 
//----------------------------------------------------------------------------------------
void TObjectListView::Open()					// override 
{
	gApplication->InstallCohandler(fIdler, TRUE);

	TTextListView::Open();
}

//----------------------------------------------------------------------------------------
// TObjectListView::Close: 
//----------------------------------------------------------------------------------------
void TObjectListView::Close()					// override 
{
	gApplication->InstallCohandler(fIdler, false);

	TTextListView::Close();
}

//----------------------------------------------------------------------------------------
// TObjectListView::GetSelectedObject: 
//----------------------------------------------------------------------------------------
void* TObjectListView::GetSelectedObject()
{
	return GetNthObject(FirstSelectedItem());
}

//----------------------------------------------------------------------------------------
// TObjectListView::GetNthObject: 
//----------------------------------------------------------------------------------------
void* TObjectListView::GetNthObject(GridCoordinate n) const
{
	return fDisplayedList && (n > 0) && (n <= fDisplayedList->GetSize()) ? fDisplayedList->At(n) : NULL;
}

//----------------------------------------------------------------------------------------
// TObjectListView::SelectObject: 
//----------------------------------------------------------------------------------------
void TObjectListView::SelectObject(void* anObject)
{
	SelectItem(static_cast<GridCoordinate>(fDisplayedList->GetIdentityItemNo(anObject)), false, false, true);
}


//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

//----------------------------------------------------------------------------------------
// GetDepth: 
//----------------------------------------------------------------------------------------
short GetDepth(TView* aView)
{
	return aView->GetSuperView() ? GetDepth(aView->GetSuperView()) + 1 : 0;
}


//========================================================================================
// CLASS TViewHierarchyView
//========================================================================================
DEFINE_REGISTERED_AC(TViewHierarchyView);

//----------------------------------------------------------------------------------------
// TViewHierarchyView destructor
//----------------------------------------------------------------------------------------
TViewHierarchyView::~TViewHierarchyView()
{
}

//----------------------------------------------------------------------------------------
// TViewHierarchyView::BuildList: 
//----------------------------------------------------------------------------------------
void TViewHierarchyView::BuildList(CList_AC* newList)// Override
{
	newList->DeleteAll();
	FlattenHierarchy(NULL, newList);
}

//----------------------------------------------------------------------------------------
// TViewHierarchyView::FlattenHierarchy: 
//----------------------------------------------------------------------------------------

void TViewHierarchyView::FlattenHierarchy(TView* root,
										  CList_AC* theList)
{
	if (root)
	{
		theList->InsertLast(root);

		for (CSubViewIterator iter(root); iter; ++iter)
			FlattenHierarchy(iter, theList);
	}
	else
		for (CWMgrIterator iter; iter; ++iter)
		{
			TWindow * aWindow = TWindow::WMgrToWindow(iter);
			if (aWindow && (aWindow != GetWindow()))
				FlattenHierarchy(aWindow, theList);
		}
}

//----------------------------------------------------------------------------------------
// TViewHierarchyView::GetItemText: 
//----------------------------------------------------------------------------------------
CStr255_AC TViewHierarchyView::GetItemText(GridCoordinate anItem) const// override 
{
	CStr255_AC returnVal;
	if (anItem <= fDisplayedList->GetSize())
	{
		TView* theView = static_cast<TView*>(GetNthObject(anItem));

		if (theView)
		{
			CStr255_AC indentString = "......................................................................................................................";
			indentString.Length() = GetDepth(theView) * 2;

			returnVal = indentString + typeid(*theView).name() + " '" + CStr15_AC(theView->GetIdentifier()) + "'";
		}
		else
			returnVal = "Error";
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TViewHierarchyView::GetSelectedView: 
//----------------------------------------------------------------------------------------
TView* TViewHierarchyView::GetSelectedView()
{
	return static_cast<TView*>(GetSelectedObject());
}


//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// FlattenTargetChain: 
//----------------------------------------------------------------------------------------
void FlattenTargetChain(TEventHandler* head,
						CList_AC* theList)
{
	theList->InsertLast(head);
	if (head->GetNextHandler())
		FlattenTargetChain(head->GetNextHandler(), theList);
}

//========================================================================================
// CLASS TTargetChainView
//========================================================================================
DEFINE_REGISTERED_AC(TTargetChainView);

//----------------------------------------------------------------------------------------
// TTargetChainView destructor
//----------------------------------------------------------------------------------------
TTargetChainView::~TTargetChainView()
{
}

//----------------------------------------------------------------------------------------
// TTargetChainView::BuildList: 
//----------------------------------------------------------------------------------------
void TTargetChainView::BuildList(CList_AC* newList)// Override
{
	newList->DeleteAll();
	FlattenTargetChain(GetTarget(), newList);
}

//----------------------------------------------------------------------------------------
// TTargetChainView::GetItemText: 
//----------------------------------------------------------------------------------------
CStr255_AC TTargetChainView::GetItemText(GridCoordinate anItem) const// override 
{
	CStr255_AC returnVal;
	if (anItem <= fDisplayedList->GetSize())
	{
		TEventHandler* theHandler = static_cast<TEventHandler*>(GetNthObject(anItem));

		returnVal = theHandler ? CStr255_AC(typeid(*theHandler).name()) + " '" + CStr15_AC(theHandler->GetIdentifier()) + "'" : CStr255_AC("Error");
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TTargetChainView::GetSelectedHandler: 
//----------------------------------------------------------------------------------------
TEventHandler* TTargetChainView::GetSelectedHandler()
{
	return static_cast<TEventHandler*>(GetSelectedObject());
}

//----------------------------------------------------------------------------------------
// End of UStructureInspectors.cp
