// UMemory.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UMEMORY__
#define __UMEMORY__

//----------------------------------------------------------------------------------------
// This unit implements MacApp's memory management and segment management schemes.
//
// The memory management scheme works by distinguishing between "permanent" and
// "temporary" heap allocation requests. Permanent requests are typically used for data
// your application allocates: objects and handles. Temporary requests are used for code
// segments and Toolbox resources and data, such as WDEF's and CDEF's.
//
// Permanent memory objects are considered permanent because they are not purged from
// memory until you explicitly dispose of or free them. (Of course, the Macintosh Memory
// Manager will purge them if they're marked purgeable.) Permanent objects are allocated
// with PermAllocation(true). This prevents the MacApp GrowZone from purging temporary objects to
// accommodate a permanent one.In MacApp, all TObjects and its descendants are considered
// permanent, and are allocated with PermAllocation(true). Temporary memory objects are
// considered temporary because MacApp's GrowZone procedure may purge them from memory to
// satisfy a memory allocation request. This is true regardless of whether the object is
// marked non-purgeable, although the GrowZone procedure will not purge locked objects
// (such as code segments in use). Typically, temporary objects are marked non-purgeable
// so that MacApp's GrowZone can control when they are purged.
//
// MacApp reserves a specific amount of heap space for temporary objects, the idea being
// that the space reserved is large enough to handle the largest number of temporary
// objects (e.g. code and system resources) needed at any given time. If this amount is
// sufficiently large, your program will never fail loading a segment or system resource.
// This amount is defined by the internal variable pSzTemporaryReserve. You can retrieve its
// value by calling GetReserveSize. It is initially set by the 'seg!' and 'mem!'
// resources, and can be changed at run-time by calling SetReserveSize. The procedure
// BuildTemporaryReserve reserves the space by allocating the handle pTemporaryReserve and setting
// its size to pSzTemporaryReserve - (the total size of all temporary objects in memory [at
// that time]).
//
// When a temporary object is loaded into memory, the size of pTemporaryReserve is adjusted
// accordingly. Permanent objects can be loaded into memory only so long as there will
// still be at least pSzTemporaryReserve bytes available for temporary objects.
//
// To identify which objects in the heap are temporary, MacApp maintains four lists of
// handles. The objects identified by these handles are considered temporary and fall
// under the control of MacApp's GrowZone procedure. The lists are:
//
// gCodeSegs - A list of handles to all CODE resources in the application and system
// resource forks.
//
//  gSysMemList - A list handles to RAM-based system resources.
//
// By default this list includes all PACK, LDEF, MDEF, CDEF and WDEF resources in the
// system and application resource forks. You can add other resources, such as fonts, by
// calling AddHandle. gApp1MemList, gApp2MemList - Lists of handles to application data or
// resources. MacApp initializes both lists to nil. The difference between the two lists is
// that handles in gApp1MemList are purged before those in gApp2MemList. You may add your
// own handles to these lists by allocating them with NewHandle and calling AddHandle for
// each handle to be added to the list.

// The Macintosh Memory Manager calls MacApp's GrowZone procedure only when all purgeable
// objects have been purged from the heap and there is still insufficient space to satisfy
// a memory request. The GrowZone will look through the lists of temporary objects in
// memory, making one purgeable so long as there is still at least pSzTemporaryReserve bytes
// allocated to temporary memory (the total size of the temporary objects in memory and
// the pTemporaryReserve handle). The GrowZone procedure attempts to never allow the size of the
// temporary objects and reserve to fall below pSzTemporaryReserve, thereby guaranteeing that
// space is always available for code segments and system resources (provided
// pSzTemporaryReserve is large enough to handle your application at its most memory intensive
// state).
//
// The value of pSzTemporaryReserve is determined at startup-time by adding up the size of all
// the segments named in the 'seg!' resources, and adding the first value of each of the
// 'mem!' resources. You can derive these resources by observing your program and using the
// MacApp debugger to help you determine when your application uses the largest amount of
// code and system resources. Typically, we've found that this happens while printing on
// the LaserWriter, or during initialization or termination.
//
// MacApp maintains another reserve, known as the low memory reserve. This is a kind of
// emergency reserve--when all else fails we release the pLowSpaceReserve handle. Its size is
// initially determined by the second number of each 'mem!' resource, and can be changed
// by calling SetReserveSize. You can retrieve its size by calling GetReserveSize.
// Internally, the low-memory reserve is allocated with the pLowSpaceReserve handle, and its
// size is stored in pLowSpaceReserveSize.
//
// The procedure InitUMemory is responsible for initially setting up the temporary and
// low-memory reserves, setting up the GrowZone procedure, initializing handle lists, and
// patching LoadSeg. It signals failure if the temporary reserve could not be allocated.
//----------------------------------------------------------------------------------------

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif


// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

const long kGZMaxAlloc = 2147483647;

typedef Handle* HandleListPtr, ** HandleListHandle;

//----------------------------------------------------------------------------------------
// CLASS CGrowZoneHook
//----------------------------------------------------------------------------------------

// An abstract class for tracking memory used internal to UMemory Manager.

class EXPORT_MA CGrowZoneHook
{
  protected:
	static CGrowZoneHook*	fFirst;

  public:
	static Size TotalTempSize(bool justLocked);
		// Calls TotalSize for all the memory hooks installed in the list. 
		// TotalTempSize returns the total number of bytes of the temporary handles
		// currently in RAM (or only locked/ in use handles if justLocked is true). 

	static Size EachPurge(Size needed);
		// Calls Purge for all the memory hooks installed in the list. 
		// Returns the actual amount of memory purged. 

  protected:
	CGrowZoneHook*		fNext;

  protected:
	CGrowZoneHook();
		// Construct a memory hook object. Protected because it is an abstract class. 

  public:
	virtual ~CGrowZoneHook();

	void InsertFirst();
		// Add a hook at the beginning of the list of memory hooks. 

	void InsertLast();
		// Add a hook to the end of the list of memory hooks. 

	inline void Insert()	{ InsertLast(); }
		// Add a hook to the list of memory hooks. Defaults to add at the end. 

	virtual Size TotalSize(bool justLocked) = 0;
		// Returns the total number of bytes managed (or only locked
		// handles if justLocked is true). 

	virtual Size Purge(Size needed) = 0;
		// Make memory available. Returns the actual amount of memory purged. 
};


//----------------------------------------------------------------------------------------
// CLASS CMMHandleList
//----------------------------------------------------------------------------------------

// A little class for tracking handles used internal to UMemory Manager.

class EXPORT_MA CMMHandleList : public CGrowZoneHook
{
  protected:
	int fSize;									// number of entries in list. This must be
												// maintained when you add or delete handles.
	HandleListHandle fHandleList;				// actual handles

  public:
	CMMHandleList();
	virtual ~CMMHandleList();

	inline int GetSize() const	{ return fSize; }

	inline Handle operator[](int i) const	{ return (*fHandleList)[i]; }

	//----------------------------------------------------------------------------------------
	// Handle list management
	//----------------------------------------------------------------------------------------

	void AddHandle(Handle h);
		// Adds a handle to the list of handles; does not check if the handle already exists
		// in the list. Simply calls Munger to add to the front of the list.

	void AddAllRsrc(ResType rType);
		// Adds all the resources of type rType to the list. Filters out all ROM
		// resources. Calls AddHandle.
	
	virtual Size TotalSize(bool justLocked);
		// Returns the total number of bytes managed (or only locked
		// handles if justLocked is true). 

	virtual Size Purge(Size needed);
		// Make memory available by emptying handles. Returns the actual amount of
		// memory purged. 
};


//----------------------------------------------------------------------------------------
// CLASS CSysMemList
//----------------------------------------------------------------------------------------

// This class doesn't make the handles in its list non-purgeable. 

class EXPORT_MA CSysMemList : public CMMHandleList
{
  public:
	CSysMemList();
	virtual ~CSysMemList();

	//----------------------------------------------------------------------------------------
	// Handle list management
	//----------------------------------------------------------------------------------------
	
	virtual Size TotalSize(bool justLocked);
		// Returns the total number of bytes managed (or only locked
		// handles if justLocked is true). 

	virtual Size Purge(Size needed);
		// Make memory available by emptying handles. Returns the actual amount of
		// memory purged. 
};


//----------------------------------------------------------------------------------------
// Global variable declarations.
//----------------------------------------------------------------------------------------

EXPORT_MA extern  short gRsrcCheck;

EXPORT_MA extern  bool gCheckRsrcUsage;
	// Enable/disable the operation of CheckRsrcUsage

EXPORT_MA extern  Size gMaxLockedRsrc;
	// The maximum memory consumed by locked resources. See CheckRsrcUsage procedure.

EXPORT_MA extern  CSysMemList gSysMemList;
	// List of system handles used to compute current allocated temporary memory (all
	// PACK, LDEF, MDEF, CDEF, WDEF resources).

EXPORT_MA extern  CMMHandleList gApp1MemList;
	// an application defined memlist

EXPORT_MA extern  CMMHandleList gApp2MemList;
	// These start out empty. You can create lists of handles and place then in either of
    // these variables. (One list might be permanent handles in your application, and the
    // other based on the current situation.) The values stored here should be a handle to
    // a list of other handles. If you modify either of these lists, call CheckReserve. It
    // calls BuildAllReserves to recompute the code and low space reserve. CheckReserve's
    // result indicates whether the full temorary reserve is present. If not, then your
    // program may crash because a segment (or defproc) can't be loaded. The handles in
    // the lists should normally be resources that your application might require at any
    // given time. All these handles will normally be marked non-purgeable. The GrowZone
    // proc, however, can purge any of these handles that are not locked.

EXPORT_MA extern  UniversalProcPtr gGZPurgeNotify;
	// If non-nil, then this will be called before the Grow Zone proc purges a handle. The
    // proc will be passed the actual handle. (Not called for code segments.)


//----------------------------------------------------------------------------------------
// Initialization
//----------------------------------------------------------------------------------------

EXPORT_MA void InitUMemory(short callsToMoreMasters);
	// Initializes this unit. Must be called before using this unit. The caller must be in
    // the program's main segment. Sets up the gCodeSegs and gSysMemList handle lists,
    // sets the grow zone, calls MaxApplZone, sets gApp1MemList and gApp2MemList to nil,
    // patches LoadSeg and allocates the temporary and low-memory reserves. Fails if
    // unable to allocate the temporary reserve.


//----------------------------------------------------------------------------------------
// Memory Management
//----------------------------------------------------------------------------------------

EXPORT_MA void BuildAllReserves();
	// BuildAllReserves creates the temporary memory (code) and low space reserves. These
    // are kept for use at a time when an out-of-memory condition has occurred to allow
    // most such occurrances to recover smoothly by deallocating the space.

EXPORT_MA bool CheckReserve();
	// Checks to see if the temorary reserve is OK. Calls BuildAllReserves and returns true if
    // the full temorary reserve is present. If this returns false your application may bomb
    // because a segment or system resource can't be loaded.

EXPORT_MA void CheckRsrcUsage();
	// Checks to see if the total size of the currently loaded resources exceeds the
    // maximum (gMaxLockedRsrc). If so, the new maximum is set. If gRsrcReport is true, then
    // the new maximum is reported in the debugger window. If gMemMgtBreak then program
    // execution is stopped.

EXPORT_MA void FailNoReserve();
	// IF NOT CheckReserve THEN Failure(memFullErr, 0).

EXPORT_MA void FailSpaceIsLow();
	// IF MemSpaceIsLow THEN Failure(memFullErr, 0).

EXPORT_MA void GetReserveSize(Size& szTemporaryReserve, Size& szMemReserve);
	// Returns the amount of memory that is to be reserved for the code and memory
    // reserve. This is not a true indication of whether this amount of memory has in fact
    // been reserved. Call CheckReserve to find out if the temorary reserve could be
    // allocated, and call MemSpaceIsLow to find out if the low-memory reserve could be
    // allocated.

EXPORT_MA void EmptyAHandle(Handle h);
	// Call this to empty a handle. Notifies gGZPurgeNotify if any

EXPORT_MA bool MemSpaceIsLow();
	// Returns true if the low space reserve is missing.

EXPORT_MA Ptr NewPermPtr(Size logicalSize);
	// Allocates a permanent Pointer; you should call this instead of NewPointer if
    // allocating some permanent memory.

EXPORT_MA Ptr NewPermPtrClear(Size logicalSize);
	// Allocates a permanent Pointer and clears the allocated block; you should call
	// this instead of NewPointerClear if allocating some permanent memory.

EXPORT_MA Handle NewPermHandle(Size logicalSize);
	// Allocates a permanent handle; you should call this instead of NewHandle if
    // allocating some permanent memory.

EXPORT_MA Handle NewPermHandleClear(Size logicalSize);
	// Allocates a permanent handle and clears the allocated block; you should call
	// this instead of NewHandleClear if allocating some permanent memory.

EXPORT_MA bool IsTemporaryAllocation();
	// Returns current state of the temporary flag.

EXPORT_MA bool TemporaryAllocation(bool temporary);
inline bool PermAllocation(bool permanent) { return !TemporaryAllocation(!permanent); } // Provided for compatibility.
	// TemporaryAllocation controls whether subsequent memory allocations are considered
	// permanent or temporary. Pass true to setup things for a temporary allocation. Returns
	// the previous state of the temporary flag.

EXPORT_MA Handle PermCopyHandle(const Handle inHandle);
inline void PermHandToHand(Handle& ioHandle) { ioHandle = PermCopyHandle(ioHandle); }
	// PermHandToHand copies the information to which theHndl is a handle and returns a new
    // permanent handle to the copy. Signals failure if the new handle can't be allocated.

EXPORT_MA bool HasTemporaryReserve();
	// True if the temorary reserve is known to be fully allocated

EXPORT_MA void SetPermHandleSize(Handle h, Size newSize);
	// Use this call to size permanent handles. It sets/ resets the permanent flag
    // correctly and does a FailMemError.

EXPORT_MA void SetPermPtrSize(Ptr p, Size newSize);
	// Use this call to size permanent pointers. It sets/ resets the permanent flag
    // correctly and does a FailMemError.

EXPORT_MA void SetReserveSize(Size temporary, Size forOther);
	// Call this to set the size of the memory reserved for temporary (code) and permanent
    // (low memory) requests.

#if qDebug
EXPORT_MA void WriteReserves();
	// fprintf's the temporary reserve and low-memory reserves to stderr.
#endif

//----------------------------------------------------------------------------------------
// Manage TempAllocation State
//----------------------------------------------------------------------------------------
class EXPORT_MA CTempAllocation 
{
public:
	inline CTempAllocation(bool temporary) : fOldState(TemporaryAllocation(temporary)) { }
	// Sets the fData to itsData

	inline ~CTempAllocation() { TemporaryAllocation(fOldState); }
	// Deallocates the data

	inline operator bool&() { return fOldState; }
		// Conversion operator, for converting data to desired return type.
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	bool	fOldState;
};

#endif // __UMEMORY__
