// UStream.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein. Used by permission


#ifndef __USTREAM__
#define __USTREAM__

#ifndef __CStream_AC__
#include "CStream_AC.h"
#endif
#ifndef __CResourceStream_AC__
#include "CResourceStream_AC.h"
#endif
#ifndef __CCountingStream_AC__
#include "CCountingStream_AC.h"
#endif
#ifndef __CHandleStream_AC__
#include "CHandleStream_AC.h"
#endif

#define TStream 		CStream_AC
#define TResourceStream	CResourceStream_AC
#define TCountingStream CCountingStream_AC
#define THandleStream	CHandleStream_AC

#if 0

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

// Toolbox


//----------------------------------------------------------------------------------------
// Forward class declarations. 
//----------------------------------------------------------------------------------------

class CList_AC;
class MStreamable;

//----------------------------------------------------------------------------------------
// This unit implements a rudimentary stream abstract type, and subclasses that do I/ O
// to files and handles. This can be used to implement document I/ O and clipboard I/ O
// with the same code. Streams can write out and read back objects, using the features
// in UObject to get an object's class ID and name. The format of an object in the stream
// is as follows:
//
//		2 bytes		delimiter
//		if (delimiter == kNullObject)			no object, so...
//			that's all folks!
//		else if (delimiter == kIndex)			object has been seen
//			4 bytes		index into TContext
//		else
//		{
//			if (delimiter == kClassID)
//				4 bytes		class ID (at the time it was written) 4 bytes for model far code
//			else if (delimiter == kSignature)
//				4 bytes		signature
//			4 bytes		size of object in stream, excluding class ID
//			1 byte		size of class name
//			n bytes		class name
//			m bytes		data specific to object
//		}
//
// The class ID in the stream might not match the class ID of the same class in the
// current application. (It depends on how the class IDs were assigned.) The stream code
// uses the class name to identify the actual class. It is not necessary, however, to
// write out the class name for every class. The stream will associate the classID in
// the stream with the correct class ID for the application. The first object of a given
// class has its name written into the stream, along with its classID. When read back,
// the name is used to find the class ID in the current application, which is associated
// with the class ID in the stream.  Subsequent objects of that class can be written with
// an empty class name, and the table of class IDs will be consulted to find out the
// proper class ID.
//
// TStream handles structures in which objects are referenced more than once using
// TContext which maintains a table of all the objects written out. The first time an
// object is written it is added to the table.If the same object is written again, only
// its table index is written. A similar process is used to read the object back.
//
// TStream will skip over objects that it can't understand.To do this, it writes the size
// of each object in a known position.This means that writing an object is a 2 step
// process. You first write the object header, by calling WriteObject, and after the entire
// object is written you fill in its size, by calling WriteObjectSize.
//
// TStream::ReadObject will return nil if it can't understand a particular object, or if
// the object was actually nil.  It returns a flag indicating the difference.
//----------------------------------------------------------------------------------------

// passed to WriteStreamObject
enum
{
	kNoStandardObject, kStandardObject
};

//  written by WriteStreamObject, read by ReadStreamObject 
enum
{
	kNullObject, kLocalObjectNumber, kClassIDAndClassname, kSignatureAndClassname
};

// passed to TContext::Add()
enum
{
	kDontForceReplacement, kForceReplacement
};

//----------------------------------------------------------------------------------------
// TContext: This is the context which holds references to objects "seen" by the stream.
//----------------------------------------------------------------------------------------
typedef long LocalObjectNumber;

class EXPORT_MA TContext
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction
	//------------------------------------------------------------------------------------

	TContext();
		// Constructor
		
	TContext(const TContext& inContext);
		// Copy Constructor

	virtual ~TContext();
		// Called to dispose of an object. Gives object a chance to clean up after itself.
        // Default simply calls ShallowFree, which makes no attempt to free instance
        // variables.  Should be overridden by any class which allocates space or owns
        // other objects in its instance variables.  Be sure to call Inherited!

	//------------------------------------------------------------------------------------
	// Miscellaneous protocol and support methods.
	//------------------------------------------------------------------------------------

	//------------------------------------------------------------------------------------
	// Context Methods
	//------------------------------------------------------------------------------------

	virtual LocalObjectNumber Add(MStreamable* objectToAdd,
										 bool forceReplacement,
										 bool& newEntry);

	virtual MStreamable* Find(LocalObjectNumber name);

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CList_AC	fOtherObjects;						// the objects "seen" by this context
};


typedef struct ClassIDEntry
{
	long fOldID;
	long fNewID;
} *PClassIDEntry, **HClassArray;


//----------------------------------------------------------------------------------------
// TStream: This is the abstract stream class. You would normally create instances of one
// of the concrete subclasses below. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TStream
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction
	//------------------------------------------------------------------------------------

	TStream();
		// Constructor
		
	virtual ~TStream();

	//------------------------------------------------------------------------------------
	// Access: These methods have default implementations that assume a 0-length stream. 
	//------------------------------------------------------------------------------------

	virtual long GetPosition();

	virtual void SetPosition(long newPosition);

	virtual long GetSize();

	virtual void SetSize(long newSize);


	//------------------------------------------------------------------------------------
	// TContext mgmt.
	//------------------------------------------------------------------------------------

	inline void SetContext(TContext* itsContext)
	{ fContext = itsContext; }

	inline TContext* GetContext()
	{ return fContext; }

	//------------------------------------------------------------------------------------
	// Note - all reading and writing methods signal failure in case of an I/ O error.
	//------------------------------------------------------------------------------------

	bool AtEnd();				
		// Return true if at the end of the stream. 

	virtual void ReadBytes(void* p, long count); 
		// This must be overridden by subclasses; all other reading methods call this. 

	Byte ReadByte();

	bool ReadBoolean();

	short ReadCharacter();
	inline void ReadCharacter(short& data)
	{ data = ReadCharacter(); }

	short ReadInteger();

	long ReadLong();

	CPoint_AC ReadPoint();

	CViewPoint ReadVPoint();
	inline void ReadVPoint(CViewPoint& data)
	{ data = ReadVPoint(); }

	CRect_AC ReadRect();
	inline void ReadRect(CRect_AC& data)
	{ data = ReadRect(); }

	CViewRect ReadVRect();
	inline void ReadVRect(CViewRect& data)
	{ data = ReadVRect(); }

	CRGBColor_AC ReadRGBColor();
	inline void ReadRGBColor(CRGBColor_AC& data)
	{ data = ReadRGBColor(); }

	IDType ReadIDType();

	CStr255_AC ReadString(short maxSize); 
	inline void ReadString(CPascalStr_AC& data, short maxSize)
	{ data = ReadString(maxSize); }
		// Read a CPascalStr_AC.maxSize is the amount of memory available, which is the maximum
        // CPascalStr_AC length + 1.If the size of the CPascalStr_AC in the stream is too large, then
        // it signals Failure with err=paramErr. 

	Handle ReadHandle(); 
		// Read a handle from the stream and return it. 

	MStreamable* ReadObject(bool& wasValid);
		// Returns an uninitialized object; known is set to true if the object was valid
        // (i.e., the class of the object is known to the program. The function result
        // will be nil if the object was unknown or if nil was found in the stream. 

	MStreamable* ReadStdObject(bool& wasValid);

	MStreamable* ReadStreamObject(bool& wasValid);
		// If stdObject is true, calls ReadStdObject, else calls ReadObject. If the
		// result is not nil, calls ReadFrom on the new object. 

	void ReadWordAlign();
		// calls ReadBytes to bump ptr 1 byte to word align


	//------------------------------------------------------------------------------------
	// Writing
	//------------------------------------------------------------------------------------

	virtual void WriteBytes(const void* p,
								   long count); 
		// This must be overridden by subclasses; all other writing methods call this. 

	void WriteByte(Byte data);

	void WriteBoolean(bool data);

	void WriteCharacter(short data);

	void WriteInteger(short data);

	void WriteLong(long data);

	void WritePoint(CPoint_AC data);

	void WriteVPoint(const CViewPoint& data);

	void WriteRect(const CRect_AC& data);

	void WriteVRect(const CViewRect& data);

	void WriteRGBColor(const CRGBColor_AC& data);
	
	void WriteIDType(const IDType data);

	void WriteString(const CPascalStr_AC& data); 
		// Write a CPascalStr_AC. This takes a CPascalStrPtr_AC, so it can accommodate any size CPascalStr_AC. 

	void WriteHandle(const Handle aHandle); 
		// Locks the handle, dereferences it, and calls WriteBytes for the size of the
		// handle.  Restores the handle to the state it was in.

	//------------------------------------------------------------------------------------
	// These methods are used to write an object to a stream.You must call WriteObject
    // first, then tell the object to write its specific data, and then call
    // WriteObjectSize. 
	//------------------------------------------------------------------------------------

	void WriteObject(MStreamable* data, long& sizePosition);

	void WriteStdObject(MStreamable* data,
									   long& sizePosition);

	void WriteObjectSize(long sizePosition);

	void WriteStreamObject(MStreamable* data,
										  bool stdObject = kNoStandardObject);
		// This calls WriteObject, then data->WriteTo, and finally WriteObjectSize. 

	void WriteWordAlign();
		// calls WriteBytes to bump ptr 1 byte to word align


	//------------------------------------------------------------------------------------
	// Class handling: used internally 
	//------------------------------------------------------------------------------------

	long LookupClassID(long id);

	void RegisterClassID(long oldID, long newID);

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:

	CAutoPtr_AC<TContext> fContext;				// remembers "seen" objects
	TemplateTempHandle<HClassArray> fClassMap;	// handle to class ID mapping table 

	short fClassMapSize;						// number of entries in the table 
};


//----------------------------------------------------------------------------------------
// THandleStream: This implements a stream into or out of a handle.The client is
// responsible for allocating and disposing the handle. 
//----------------------------------------------------------------------------------------

class EXPORT_MA THandleStream : public TStream
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction
	//------------------------------------------------------------------------------------
	THandleStream(Handle itsHandle = NULL, long growth = 1);
		// Initialize stream; growth specifies how much to grow the handle when it needs
        // to grow. You use this so that the handle isn't grown by a small number of bytes
        // on each write operation. 
		
	virtual ~THandleStream();
		// This also shrinks the handle to the current position, eliminating any extra
        // space at the end. 

	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------
	virtual long GetPosition();

	virtual void SetPosition(long newPosition);

	virtual long GetSize();

	virtual void SetSize(long newSize);

	virtual void ReadBytes(void* p, long count); 

	virtual void WriteBytes(const void* p, long count); 

	//------------------------------------------------------------------------------------
	// Growing 
	//------------------------------------------------------------------------------------
	virtual long GrowthSize(long needed);
		// Returns amount to grow by, given that we need 'needed' bytes.Default is to grow
        // by Max(fGrowth, needed). 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	Handle fHandle;								// the actual handle 
	long fPosition;								// offset into the handle 
	long fSize;									// size of the stream (may be smaller than handle) 
	long fGrowthSize;							// how much to grow the handle when needed
	char fSavedState;							// handle's saved state
};


//----------------------------------------------------------------------------------------
// TCountingStream: This implements a stream that counts up the total number of bytes. It
// is only usable for writing. You will get a debugging break if you try to read from this
// kind of stream. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TCountingStream : public TStream
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction 
	//------------------------------------------------------------------------------------

	TCountingStream();
		// Constructor

	virtual ~TCountingStream();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------

	virtual long GetPosition();					// Override
	virtual void SetPosition(long newPosition);	// Override
	virtual long GetSize();						// Override
	virtual void SetSize(long newSize);			// Override

	virtual void WriteBytes(const void* p, long count); 	// Override

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:

	long fPosition;								// current position in the stream 
	long fSize;									// current size of stream 
};


//----------------------------------------------------------------------------------------
// TResourceStream: This implements a stream that reads and writes to partial resources.
// Since the partial resources traps are only available in System 7, this stream class can
// only be used under System 7. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TResourceStream : public TStream
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	// Creation/Destruction 
	//------------------------------------------------------------------------------------
	TResourceStream(ResType theType = kNoIdentifier, ResNumber theID = kNoResource);
		// Constructor
	virtual ~TResourceStream();
		// Destructor

	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------
	virtual long GetPosition();
		// Returns fPosition.

	virtual void SetPosition(long newPosition);
		// Sets fPosition.
	
	virtual long GetSize();
		// Returns fSize.

	virtual void SetSize(long newSize);
		// Sets fSize.
	
	virtual void ReadBytes(void* p, long count);
		// Reads bytes from the resource calling ReadPartialResource.

	virtual void WriteBytes(const void* p, long count);
		// Write bytes to the resource calling WritePartialResource.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	Handle fResource;							// the resource to write to/ read from
	long fPosition;								// offset into the resource
	long fSize;									// offset into the resource
};

#endif // #if 0

#endif // __USTREAM__
