//----------------------------------------------------------------------------------------
// UMemory.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

// ACS

#ifndef __CHandle_AC__
#include "CHandle_AC.h"
#endif

// MacApp

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif

#if qDebugMsg
#endif

typedef pascal void(* NotifyType)(Handle h);

enum
{
	kHeapOverhead = 8
};

//----------------------------------------------------------------------------------------
// Globals to this module and externally available.

#if qDebug
bool gCheckRsrcUsage = true;
#else
bool gCheckRsrcUsage;
#endif

short gRsrcCheck = kRsrcCheckInterval;

Size gMaxLockedRsrc;

// Memory hooks
CGrowZoneHook* CGrowZoneHook::fFirst;

CMMHandleList gApp1MemList;
CSysMemList gSysMemList;
CMMHandleList gApp2MemList;

UniversalProcPtr gGZPurgeNotify;

long pTemporaryReserveShortfall;				// amt. that we are lacking in the
												// temporary reserve

static Handle pTemporaryReserve;				// Contains temporary (code) reserve.
static Handle pLowSpaceReserve;					// Allocates low memory reserve.
static Size pTotalTemporaryReserveSize;			// Attempt to reserve this much memory for
												// the temporary (code) reserve.
												// This is the total size necessary to
												// account for temporary requests,
												// NOT the size of the temporaryReserve
static Size pLowSpaceReserveSize;				// Attempt to reserve this much memory for
												// the low-memory reserve.

static long pLastBuildAllReservesFreeMem = -1;	// amount of freemem for which the reserves
												// are valid

static bool pInGrowZone;						// true when the growzoneproc has been entered

static bool pTemporaryAllocation = true;		// Controls behavior of the GrowZone proc.
												// true if it's ok to dip into the temp reserve

static bool pGotLowSpaceReserve;				// true if the low space reserve could be allocated

//========================================================================================
// GLOBAL Procedures
//========================================================================================

pascal long GrowZoneProc(Size needed);

static bool IsHandleEligible(Handle h);

static void BuildTemporaryReserve(Size allocLim);
static void BuildTemporaryReserve(Size allocLim, Size tempInMemorySize);

static void InstallGrowZoneProc();

static Size ReleaseTemporaryReserve();

static Size ReleaseLowSpaceReserve();

static bool HasLowSpaceReserve();

//----------------------------------------------------------------------------------------
// TPermHandleHandler uses UMemory calls to differentiate temp vs. permanent memory allocations.
// It is installed into the ACS so calls to PermHandle_AC use MacApp's calls (an interim solution).
//----------------------------------------------------------------------------------------

class TPermHandleHandler : public CPermHandleHandler_AC
{
public:
	TPermHandleHandler();
	virtual ~TPermHandleHandler();
	
	virtual Handle DoNewHandle(Size size);
		// Calls NewPermHandle
		
	virtual void DoSetHandleSize(Handle h, Size newSize);
		// Calls SetPermHandleSize

};

//----------------------------------------------------------------------------------------
// TPermHandleHandler constructor:
//----------------------------------------------------------------------------------------

TPermHandleHandler::TPermHandleHandler()
{
}

//----------------------------------------------------------------------------------------
// TPermHandleHandler destructor:
//----------------------------------------------------------------------------------------

TPermHandleHandler::~TPermHandleHandler()
{
}
	
//----------------------------------------------------------------------------------------
// TPermHandleHandler::DoNewHandle:
//----------------------------------------------------------------------------------------

Handle TPermHandleHandler::DoNewHandle(Size size)
{
	return ::NewPermHandle(size);
}
		
//----------------------------------------------------------------------------------------
// TPermHandleHandler::DoSetHandleSize:
//----------------------------------------------------------------------------------------

void TPermHandleHandler::DoSetHandleSize(Handle h, Size newSize)
{
	::SetPermHandleSize(h, newSize);
}

//----------------------------------------------------------------------------------------
// InlineIsHandleEligible: 
//----------------------------------------------------------------------------------------

static bool IsHandleEligible(Handle h)
{
	return h && *h  && h != GZSaveHnd();
}

//----------------------------------------------------------------------------------------
// abs: 
//----------------------------------------------------------------------------------------

static long abs(long x)
{
	return (x < 0) ? -x : x;
}

//========================================================================================
// CLASS CGrowZoneHook
//========================================================================================

//----------------------------------------------------------------------------------------
// CGrowZoneHook constructor: 
//----------------------------------------------------------------------------------------
CGrowZoneHook::CGrowZoneHook() :
fNext(NULL)
{
}

//----------------------------------------------------------------------------------------
// CGrowZoneHook destructor: 
//----------------------------------------------------------------------------------------
CGrowZoneHook::~CGrowZoneHook()
{
}

//----------------------------------------------------------------------------------------
// CGrowZoneHook::InsertFirst: 
//----------------------------------------------------------------------------------------
void CGrowZoneHook::InsertFirst()
{
	fNext = fFirst;
	fFirst = this;
}

//----------------------------------------------------------------------------------------
// CGrowZoneHook::InsertLast: 
//----------------------------------------------------------------------------------------
void CGrowZoneHook::InsertLast()
{
	if (!fFirst)
		fFirst = this;
	else
	{
		CGrowZoneHook* hook = fFirst;
		CGrowZoneHook* next;
		while ((next = hook->fNext) != NULL)
			hook = next;
		
		hook->fNext = this;
	}
}

//----------------------------------------------------------------------------------------
// CGrowZoneHook::TotalTempSize: 
//----------------------------------------------------------------------------------------
Size CGrowZoneHook::TotalTempSize(bool justLocked)
{
	Size total = 0;
	CGrowZoneHook* hook = fFirst;
	while (hook)
	{
		total += hook->TotalSize(justLocked);
		hook = hook->fNext;
	}
	return total;
}

//----------------------------------------------------------------------------------------
// CGrowZoneHook::EachPurge: 
//----------------------------------------------------------------------------------------
Size CGrowZoneHook::EachPurge(Size needed)
{
	Size purged = 0;
	CGrowZoneHook* hook = fFirst;
	while (hook && (purged < needed))
	{
		purged += hook->Purge(needed);
		hook = hook->fNext;
	}
	return purged;
}

//========================================================================================
// CLASS CMMHandleList
//========================================================================================

//----------------------------------------------------------------------------------------
// CMMHandleList constructor: 
//----------------------------------------------------------------------------------------
CMMHandleList::CMMHandleList() :
	fSize(0),
	fHandleList(NULL)
{
}

//----------------------------------------------------------------------------------------
// CMMHandleList destructor: 
//----------------------------------------------------------------------------------------
CMMHandleList::~CMMHandleList()
{
	fSize = 0;
	if (fHandleList)
		DisposeHandle((Handle)fHandleList);
	fHandleList = NULL;
}

//----------------------------------------------------------------------------------------
// CMMHandleList::AddAllRsrc: 
//----------------------------------------------------------------------------------------
void CMMHandleList::AddAllRsrc(ResType rType)
{
	enum
	{
		kROMResident = 1
	};

	bool oldResLoad = MASetResLoad(false);

	short nResources = CountResources(rType);
	for (short i = 1; i <= nResources; ++i)
	{
		Handle h = GetIndResource(rType, i);

		if (h)
		{
			short theID;
			ResType theType;
			CStr255_AC theName;
			GetResInfo(h, &theID, &theType, theName);

			// If there is a ROM resource for this type and ID, don't put it
			// on the list since it will not consume any memory.

			UseROMMap(false);
			Handle romHandle = GetResource(rType, theID);
			UseROMMap(true);
			if (romHandle && (HomeResFile(romHandle) != kROMResident) && !(GetResAttrs(h) & resSysHeap))
				AddHandle(romHandle);
		}
	}

	SetResLoad(oldResLoad);
}

//----------------------------------------------------------------------------------------
// CMMHandleList::AddHandle: 
//----------------------------------------------------------------------------------------
void CMMHandleList::AddHandle(Handle h)
{
	if (!fHandleList)
	{
		fHandleList = (HandleListHandle)NewHandle(0);
		::ThrowIfNULL_AC(fHandleList);
	}
	Munger((Handle)fHandleList, 0, NULL, 0, &h, sizeof(Handle));
	::ThrowIfMemError_AC();
	++fSize;
}

//----------------------------------------------------------------------------------------
// CMMHandleList::TotalSize: 
//----------------------------------------------------------------------------------------
Size CMMHandleList::TotalSize(bool justLocked)
{
	Size total = 0;
	int i = GetSize();
	while (--i > 0)
	{
		Handle h = (*fHandleList)[i];
		if (h && !::IsHandlePurged_AC(h))	// in memory already
		{
			//HNoPurge(h);
			if (!justLocked || IsHandleLocked_AC(h))
				total += ::GetHandleSize(h) + kHeapOverhead;
		}
	}
	return total;
}

//----------------------------------------------------------------------------------------
// CMMHandleList::Purge: 
//----------------------------------------------------------------------------------------
Size CMMHandleList::Purge(Size needed)
{
	Size purged = 0;
	int i = GetSize();
	while (--i > 0)
	{
		Handle h = (*fHandleList)[i];
		if (::IsHandleEligible(h) && !IsHandleLocked_AC(h))
		{
			purged += ::GetHandleSize(h);
			EmptyAHandle(h);
			if (purged >= needed)
				break;
		}
	}
	return purged;
}

//========================================================================================
// CLASS CSysMemList
//========================================================================================

//----------------------------------------------------------------------------------------
// CSysMemList constructor: 
//----------------------------------------------------------------------------------------
CSysMemList::CSysMemList()
{
}

//----------------------------------------------------------------------------------------
// CSysMemList destructor: 
//----------------------------------------------------------------------------------------
CSysMemList::~CSysMemList()
{
}

//----------------------------------------------------------------------------------------
// CSysMemList::TotalSize: 
//----------------------------------------------------------------------------------------
Size CSysMemList::TotalSize(bool justLocked)
{
	Size total = 0;

	int i = GetSize();
	while (--i > 0)
	{
		Handle h = (*fHandleList)[i];
		if (h && !::IsHandlePurged_AC(h))	// in memory already
		{
			// don't fool with system defprocs
			// don't HNoPurge(h);
			
			if (!justLocked || IsHandleLocked_AC(h))
				total += ::GetHandleSize(h) + kHeapOverhead;
		}
	}

	return total;
}

//----------------------------------------------------------------------------------------
// CSysMemList::Purge: 
//----------------------------------------------------------------------------------------
Size CSysMemList::Purge(Size needed)
{
	Size purged = 0;
	int i = GetSize();
	while (--i > 0)
	{
		Handle h = (*fHandleList)[i];
		if (::IsHandleEligible(h) && !IsHandleLocked_AC(h))
		{
			purged += ::GetHandleSize(h);
			EmptyAHandle(h);
			if (purged >= needed)
				break;
		}
	}
	return purged;
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================

//----------------------------------------------------------------------------------------
// BuildTemporaryReserve: 
//----------------------------------------------------------------------------------------
void BuildTemporaryReserve(Size allocLim)
{
	BuildTemporaryReserve(allocLim, CGrowZoneHook::TotalTempSize(false));
}

//----------------------------------------------------------------------------------------
// BuildTemporaryReserve: 
//----------------------------------------------------------------------------------------
void BuildTemporaryReserve(Size allocLim, Size tempInMemorySize)
{
#if qDebug
	if (IsTemporaryAllocation())
		::ProgramBreak_AC("BuildTemporaryReserve called with IsTemporaryAllocation() = true");
#endif

	pTemporaryReserveShortfall = 0;

	// compute amt actually needed
	Size needed = Min_AC(pTotalTemporaryReserveSize - tempInMemorySize - kHeapOverhead, allocLim);

	if (needed > 0)
	{
		long tryThis = Min_AC(allocLim, needed);

		if (::IsHandlePurged_AC(pTemporaryReserve) || ::IsHandleEligible(pTemporaryReserve))
		{
			ReallocateHandle(pTemporaryReserve, tryThis);
			if (MemError() != noErr)
				ReleaseLowSpaceReserve();
		}

		if (!pInGrowZone)
		{
			if (::IsHandlePurged_AC(pTemporaryReserve))
			{
				//
				// Didn't get it before, now we try a little harder
				//
				PurgeMem(needed);
				long avail = CompactMem(needed);
	
				ReallocateHandle(pTemporaryReserve, Min_AC(avail, needed));
	
				if (!::IsHandlePurged_AC(pTemporaryReserve))
					pTemporaryReserveShortfall = pTotalTemporaryReserveSize - ::GetHandleSize(pTemporaryReserve);
				else
					pTemporaryReserveShortfall = pTotalTemporaryReserveSize - avail;
			}

			// Large handles are almost as obnoxious as nonrelocatable blocks. Try to get
      		// this guy out of the way, just in case.
			if (!::IsHandlePurged_AC(pTemporaryReserve))
				MoveHHi(pTemporaryReserve);
		}
	}
	else // free the current temporary reserve
		ReleaseTemporaryReserve();

}

//----------------------------------------------------------------------------------------
// BuildAllReserves: 
//----------------------------------------------------------------------------------------
void BuildAllReserves()
{
	//
	// The reserves are valid for a given amount of Free Memory. If the amount of free
	// memory hasn't changed then the state of the reserves hasn't really changed either.
	//
	enum
	{
		kSlopFactor = 64
	};

	if (abs(FreeMem() - pLastBuildAllReservesFreeMem) > kSlopFactor)
	{
		// set the permanent flag to ensure that the temporary reserve is actually allocated and
		// not given up to the low space reserve
	
		bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	
		// reallocate the low space Reserve, if necessary
		if (!HasLowSpaceReserve())
		{
			pGotLowSpaceReserve = false;
	
			if (!pTemporaryReserveShortfall)
			{
				ReallocateHandle(pLowSpaceReserve, pLowSpaceReserveSize);
				pGotLowSpaceReserve = MemError() == noErr;
			}
		}
	
		TemporaryAllocation(priorTemp);
		
		pLastBuildAllReservesFreeMem = FreeMem();
	}
}

//----------------------------------------------------------------------------------------
// CheckReserve: 
//----------------------------------------------------------------------------------------
bool CheckReserve()
{
	BuildAllReserves();

	return !pTemporaryReserveShortfall;
}

//----------------------------------------------------------------------------------------
// CheckRsrcUsage: 
//----------------------------------------------------------------------------------------
void CheckRsrcUsage()
{
	if (gCheckRsrcUsage)
	{
		long sz = CGrowZoneHook::TotalTempSize(true);
		if (sz > gMaxLockedRsrc)
		{
			// In non debug mode, one can calculate seg! and mem! resources by loading the application
			// under with a debugger, setting the variable gCheckRsrcUsage to true, and setting a break
			// point on the following line. Everytime the debugger breaks on this line, you can 
			// look at your heap and loaded segments.

			gMaxLockedRsrc = sz;

#if qDebug
			if (gRsrcReport)
			{
				CStr255_AC s;
				NumToString(gMaxLockedRsrc, s);
				s = "  = New maximum resources usage: " + s + " =";
				::ProgramReport_AC(s, gMemMgtBreak);
			}
#endif

		}
	}
}

//----------------------------------------------------------------------------------------
// FailNoReserve: 
//----------------------------------------------------------------------------------------
void FailNoReserve()
{
	if (!CheckReserve())
		::ThrowException_AC(memFullErr, 0);
}

//----------------------------------------------------------------------------------------
// FailSpaceIsLow: 
//----------------------------------------------------------------------------------------
void FailSpaceIsLow()
{
	if (MemSpaceIsLow())
		::ThrowException_AC(memFullErr, 0);
}

//----------------------------------------------------------------------------------------
// GetReserveSize: 
//----------------------------------------------------------------------------------------
void GetReserveSize(Size& szTemporaryReserve,
					Size& szMemReserve)
{
	szTemporaryReserve = pTotalTemporaryReserveSize;
	szMemReserve = pLowSpaceReserveSize;
}


//----------------------------------------------------------------------------------------
// EmptyAHandle: 
//----------------------------------------------------------------------------------------
void EmptyAHandle(Handle h)
{
	if (gGZPurgeNotify)
		((NotifyType)gGZPurgeNotify)(h);
	EmptyHandle(h);
}


//----------------------------------------------------------------------------------------
// GrowZoneProc: 
//----------------------------------------------------------------------------------------
pascal long GrowZoneProc(Size needed)
{
	long OldA5 = SetCurrentA5();				// Can be called from other worlds 
	long result = 0;							// default is to fail 

	if (!pInGrowZone)						// prevent re-entrane 
	{
		pInGrowZone = true;

		if (IsTemporaryAllocation())
		{
			result = CGrowZoneHook::EachPurge(needed);
		}
		else 
		{
			//
			// If this is a permanent request; see if the temporary reserve is too large 
			//
			if (::IsHandleEligible(pTemporaryReserve))
			{
				// compute size of resources currently in memory 
				Size tempInMemorySize = CGrowZoneHook::TotalTempSize(false);
				
				long tempReserveSize = ::GetHandleSize(pTemporaryReserve);
				if (tempInMemorySize + tempReserveSize + kHeapOverhead > pTotalTemporaryReserveSize)
				{
					// this should cut down the existing temporary reserve. 
					BuildTemporaryReserve(tempReserveSize, tempInMemorySize);
	
					// see if we succeeded in freeing some memory 
					long newTempReserveSize = ::GetHandleSize(pTemporaryReserve);
					if (::IsHandlePurged_AC(pTemporaryReserve))
						result = tempReserveSize;
					else if ((newTempReserveSize < tempReserveSize))
						result = tempReserveSize - newTempReserveSize;
				}
			}
	
			// Didn't free anything above. However, we may still be able to get someone
			// to purge something for us; only purge it if there is too much temp stuff
			// in memory already
			if (!result && ::IsHandlePurged_AC(pTemporaryReserve))
			{
				result = CGrowZoneHook::EachPurge(needed);
//				BuildTemporaryReserve(result);
				BuildTemporaryReserve(MaxBlock());
			}
	
			// last ditch attempt-free emergency reserve
			if (!result)
				result = ReleaseLowSpaceReserve();
		}

		pInGrowZone = false;
	}
	else
	{
#if qDebug
		DebugStr((StringPtr)"\pGrowZoneProc has been re-entered.");
#endif
	}
	
	OldA5 = SetA5(OldA5);

	return result;
}

//----------------------------------------------------------------------------------------
// InstallGrowZoneProc: Once called the grow zone proc's segment CANNOT be moved since
// we're passing a NON-JT address to SetGrowZone (so we can be called from "other worlds"
//----------------------------------------------------------------------------------------
void InstallGrowZoneProc()
{
	GrowZoneUPP gzUPP = NewGrowZoneProc(GrowZoneProc);
	::ThrowIfNULL_AC(gzUPP);
	SetGrowZone(gzUPP);
}

//----------------------------------------------------------------------------------------
// InitUMemory: 
//----------------------------------------------------------------------------------------
void InitUMemory(short callsToMoreMasters)
{
	// Here is a trick suggested by Jerome C.--it allocates several large blocks of master
	// pointers. Now it's even approved in the new IM!
	THz applZone = ApplicationZone();
	short oldMoreMast = applZone->moreMast;
	long numMasterPointersDesired = oldMoreMast * callsToMoreMasters;
	do
	{
		applZone->moreMast = Min_AC(numMasterPointersDesired, SHRT_MAX);
		MoreMasters();
		numMasterPointersDesired -= applZone->moreMast;
	} while (numMasterPointersDesired > 0);
	applZone->moreMast = oldMoreMast;

	//
	// Pickup the starting points 
	//
	Size heapSize = GetSizeObjectHeap();
	Size heapIncrement = GetSizeHeapIncrement();
	Size temporaryReserve = GetSizeTempReserve();
	Size lowSpaceRes = GetSizeLowSpaceReserve() + heapIncrement;	// Include a heap block in our low
																	// space reserve to ensure that temporary 
																	// objects can still be allocated.

	// Initialize memory management globals

	pLowSpaceReserve = NewEmptyHandle();
	::ThrowIfNULL_AC(pLowSpaceReserve);

	pTemporaryReserve = NewEmptyHandle();
	::ThrowIfNULL_AC(pTemporaryReserve);

	// init the gSysMemList 

	gSysMemList.AddAllRsrc('LDEF');
	gSysMemList.AddAllRsrc('CDEF');
	gSysMemList.AddAllRsrc('MDEF');
	gSysMemList.AddAllRsrc('WDEF');
	gSysMemList.AddAllRsrc('PACK');

	gUMemoryInitialized = true;

	SetReserveSize(temporaryReserve, lowSpaceRes);

	if (pTemporaryReserveShortfall)					// couldn't get temporary reserve. Can't continue
		::ThrowException_AC(memFullErr, 0);

	// Memory hooks
	gApp1MemList.InsertLast();
	gSysMemList.InsertLast();
	gApp2MemList.InsertLast();
	
	InstallGrowZoneProc();
	
	SetPermHandleHandler_AC(TH_new TPermHandleHandler);
}

//----------------------------------------------------------------------------------------
// MemSpaceIsLow: 
//----------------------------------------------------------------------------------------
bool MemSpaceIsLow()
{
	BuildAllReserves();

	return !pGotLowSpaceReserve;
}

//----------------------------------------------------------------------------------------
// NewPermHandle: 
//----------------------------------------------------------------------------------------
Handle NewPermHandle(Size logicalSize)
{
	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	Handle aHandle = NewHandle(logicalSize);
	TemporaryAllocation(priorTemp);

	::ThrowIfNULL_AC(aHandle);
#if qDebug
	BlockSet_AC(*aHandle, logicalSize, kNewInitVal_AC);
#endif

	return aHandle;
}

//----------------------------------------------------------------------------------------
// NewPermHandleClear: 
//----------------------------------------------------------------------------------------
Handle NewPermHandleClear(Size logicalSize)
{
	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	Handle aHandle = NewHandleClear(logicalSize);
	TemporaryAllocation(priorTemp);

	::ThrowIfNULL_AC(aHandle);

	return aHandle;
}

//----------------------------------------------------------------------------------------
// NewPermPtr: 
//----------------------------------------------------------------------------------------
Ptr NewPermPtr(Size logicalSize)
{
	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	Ptr aPtr = NewPtr(logicalSize);
	TemporaryAllocation(priorTemp);

	::ThrowIfNULL_AC(aPtr);
#if qDebug
	BlockSet_AC(aPtr, logicalSize, kNewInitVal_AC);
#endif

	return aPtr;
}

//----------------------------------------------------------------------------------------
// NewPermPtrClear: 
//----------------------------------------------------------------------------------------
Ptr NewPermPtrClear(Size logicalSize)
{
	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	Ptr aPtr = NewPtrClear(logicalSize);
	TemporaryAllocation(priorTemp);

	::ThrowIfNULL_AC(aPtr);

	return aPtr;
}

//----------------------------------------------------------------------------------------
// TemporaryAllocation: 
//----------------------------------------------------------------------------------------
bool TemporaryAllocation(bool temporary)
{
	bool returnVal = pTemporaryAllocation;

	if (temporary != pTemporaryAllocation)
	{
		pTemporaryAllocation = temporary;
		if (pTemporaryAllocation)
			ReleaseTemporaryReserve();
		else
			BuildTemporaryReserve(kGZMaxAlloc);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TemporaryAllocation: 
//----------------------------------------------------------------------------------------
bool IsTemporaryAllocation()
{
	return pTemporaryAllocation;
}

//----------------------------------------------------------------------------------------
// PermCopyHandle: 
//----------------------------------------------------------------------------------------
Handle PermCopyHandle(const Handle inHandle)
{
	Handle returnVal = NULL;

	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	OSErr err = CopyHandle_AC(inHandle, returnVal);
	TemporaryAllocation(priorTemp);

	::ThrowIfOSErr_AC(err);
	
	return returnVal;
}


//----------------------------------------------------------------------------------------
// SetPermHandleSize: 
//----------------------------------------------------------------------------------------
void SetPermHandleSize(Handle h,
					   Size newSize)
{
#if qDebug
	Size oldSize = ::GetHandleSize(h);
#endif

	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	SetHandleSize(h, newSize);
	OSErr theError = MemError();
	TemporaryAllocation(priorTemp);

	// This lets us be assured that no
	// operations have occurred that would invalidate
	// the MemErr flag thus the following call will
	// give a true result
	if (theError != noErr)
		::ThrowException_AC(theError, 0);

#if qDebug
	if (oldSize < newSize)
		BlockSet_AC((Ptr) * h + oldSize, newSize - oldSize, kNewInitVal_AC);
#endif

}

//----------------------------------------------------------------------------------------
// SetPermPtrSize: 
//----------------------------------------------------------------------------------------
void SetPermPtrSize(Ptr p,
					Size newSize)
{
#if qDebug
	Size oldSize = GetPtrSize(p);
#endif

	bool priorTemp = TemporaryAllocation(false);// make sure temporary reserve is OK
	SetPtrSize(p, newSize);
	OSErr theError = MemError();
	TemporaryAllocation(priorTemp);

	// This lets us be assured that no
	// operations have occurred that would invalidate
	// the MemErr flag thus the following call will
	// give a true result
	if (theError != noErr)
		::ThrowException_AC(theError, 0);

#if qDebug
	if (oldSize < newSize)
		BlockSet_AC((Ptr)p + oldSize, newSize - oldSize, kResizedPtrInitVal_AC);
#endif

}

//----------------------------------------------------------------------------------------
// SetReserveSize: 
//----------------------------------------------------------------------------------------
void SetReserveSize(Size temporary,
					Size forOther)
{
	pTotalTemporaryReserveSize = temporary;
	pLowSpaceReserveSize = forOther;

	// Since the numbers have changed, make sure we start from scratch. 
	ReleaseTemporaryReserve();
	ReleaseLowSpaceReserve();

	BuildAllReserves();
}

//----------------------------------------------------------------------------------------
// ReleaseLowSpaceReserve: 
//----------------------------------------------------------------------------------------
Size ReleaseLowSpaceReserve()
{
	Size returnVal = 0;
	if (::IsHandleEligible(pLowSpaceReserve))
	{
		returnVal = ::GetHandleSize(pLowSpaceReserve);
		EmptyAHandle(pLowSpaceReserve);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// ReleaseTemporaryReserve: 
//----------------------------------------------------------------------------------------
Size ReleaseTemporaryReserve()
{
	Size returnVal = 0;
	if (::IsHandleEligible(pTemporaryReserve))
	{
		returnVal = ::GetHandleSize(pTemporaryReserve);
		EmptyAHandle(pTemporaryReserve);
	}

	Size didRelease = ReleaseLowSpaceReserve();

	return returnVal + didRelease;
}

//----------------------------------------------------------------------------------------
// HasLowSpaceReserve: 
//----------------------------------------------------------------------------------------
bool HasLowSpaceReserve()
{
	return !::IsHandlePurged_AC(pLowSpaceReserve);
}

//----------------------------------------------------------------------------------------
// HasTemporaryReserve: 
//----------------------------------------------------------------------------------------
bool HasTemporaryReserve()
{
	return !::IsHandlePurged_AC(pTemporaryReserve) && !pTemporaryReserveShortfall;
}

//----------------------------------------------------------------------------------------
// WriteReserves: fprintf's the temporary reserve and low-memory reserves to stderr
//----------------------------------------------------------------------------------------

#if qDebug

void WriteReserves()
{
	WrLblPtr("Temporary reserve (pTemporaryReserve)", pTemporaryReserve);
	ProgramReport_AC("\n");
	
	WrLblPtr("Low-space reserve (pLowSpaceReserve)", pLowSpaceReserve);
	ProgramReport_AC("\n");
}

#endif

//----------------------------------------------------------------------------------------
// End of UMemory.cp
