//----------------------------------------------------------------------------------------
// UniversalStartup.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

// MacApp

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

#ifndef __SEGLOAD__
#include <SegLoad.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#ifndef _STDLIB
#include <stdlib.h>
#endif

#ifndef _STDDEF
#include <stddef.h>
#endif


// B E G I N   U N I V E R S A L   C O D E
//
// The following is universal code. That is, code that must be compiled to run on any
// processor. This is because this is the code that is called during startup, but after
// we've checked for PPC so this can be PPC code. This code will properly initialize the
// toolbox, thus an alert can be shown if necessary. It must not make any calls into code
// that is not "universal" code. 
//

static bool gHasProcessMgr;				// saved from InitToolBox for configuration info
static bool gIsOnlyBackground;			// saved from InitToolBox for configuration info
static bool gIsHighLevelEventAware;		// saved from InitToolBox for configuration info
static short gProcessor;				// saved from ValidateHardware for configuration info
static bool gHasFPU;					// saved from ValidateHardware for configuration info

static Size gSizeObjectHeap;
static Size gSizeHeapIncrement;
static Size gSizeTempReserve;
static Size gSizeLowSpaceReserve;
static Size gStackTot;

//========================================================================================
// GLOBAL Functions
//========================================================================================

//----------------------------------------------------------------------------------------
// ClearTheFPU: Essential one-time initialization
//
//	INLINE 0x42A7,										// CLR.L -(A7) 
//		   0x42A7,										// CLR.L -(A7) 
//		   0xF21F, 0x9800;								// FMOVEM (A7)+, FPCR/FPSR 
//----------------------------------------------------------------------------------------
#if q68K
static void ClearTheFPU() = { 0x42A7, 0x42A7, 0xF21F, 0x9800 };
#endif

static void InitToolBox();
static void FailedInitToolBox();
static void ExpandHeap();
static void SetStackSpace(long numBytes);

//----------------------------------------------------------------------------------------
// UniversalStartup
//----------------------------------------------------------------------------------------
void UniversalStartup()
{
	static bool gDidUniversalStartup;

	if (!gDidUniversalStartup)
	{
		bool validProcessor = ValidateProcessor();

#if q68K
		// the main procedure is always compiled with universal code so, the FPU must be
		// reset before it is used. We could get spurious crashes or worse.
		//  Remember: 2 + 2 = 4... every time!
		if (gHasFPU)
			ClearTheFPU();
#endif

		InitToolBox();

		if (!validProcessor)
			AlertUnsupportedConfiguration();
		
		ExpandHeap();

		gDidUniversalStartup = true; // Don't come back
	}
}

//----------------------------------------------------------------------------------------
// ValidateProcessor: 
//----------------------------------------------------------------------------------------
bool ValidateProcessor()
{
	long response;
	OSErr err = Gestalt(gestaltProcessorType, &response);
	gProcessor = static_cast<short>(response);
	
	err = Gestalt(gestaltFPUType, &response);
	gHasFPU = response != gestaltNoFPU;

	bool isSupported = true;

	// Run the gauntlet of hardware support tests using the conditionally set constants. If any
    // single test fails then the app is considered unsupported on this machine.

	if (qNeedsMC68020)
		isSupported = isSupported && (gProcessor != gestalt68000 &&
						gProcessor != gestalt68010);

	if (qNeedsMC68030)
		isSupported = isSupported && (gProcessor != gestalt68000 &&
						gProcessor != gestalt68010 &&
						gProcessor != gestalt68020);

	if (qNeedsMC68040)
		isSupported = isSupported && (gProcessor != gestalt68000 &&
						gProcessor != gestalt68010 &&
						gProcessor != gestalt68020 &&
						gProcessor != gestalt68030);
	
	if (qNeedsFPU)
		isSupported = isSupported && gHasFPU;

	return isSupported;
}

//----------------------------------------------------------------------------------------
// UnsupportedConfiguration: 
//----------------------------------------------------------------------------------------
void AlertUnsupportedConfiguration()
{
	::SetCursor(&qd.arrow);
	
	// This alert is set to preload
	::Alert(phUnsupportedConfiguration, NULL);

	exit(EXIT_FAILURE);							// leave the application pronto! 
}

//----------------------------------------------------------------------------------------
// InitToolBox: 
//----------------------------------------------------------------------------------------
void InitToolBox()
{
	if (!gToolBoxInitialized)
	{		
		gToolBoxInitialized = true;

		// Check for process mgr then check for background only mode
		long response;
		if (Gestalt(gestaltOSAttr, &response) == noErr && (response >> gestaltLaunchControl) & 1)
		{
			gHasProcessMgr = true;				// save it for later
			ProcessSerialNumber psn;
			psn.highLongOfPSN = 0;
			psn.lowLongOfPSN = kCurrentProcess;
			
			ProcessInfoRec info;
			info.processInfoLength = sizeof(ProcessInfoRec);
			info.processName = NULL;
			info.processAppSpec = NULL;

			if (::GetProcessInformation(&psn, &info) == noErr)
			{
				gIsOnlyBackground = (info.processMode & modeOnlyBackground) != 0;
				gIsHighLevelEventAware = (info.processMode & modeHighLevelEventAware) != 0;
			}
		}

		// init the toolbox managers
		::InitGraf(&qd.thePort);

		// Are we background only?
		if (gIsOnlyBackground)
			// _DON'T_ flush disk-inserted or os events or you'll be sorry! 
			::FlushEvents(everyEvent - diskMask - osMask, 0);
		else
		{
			::InitFonts();
			::InitWindows();							// creates non-relocatable for the WM port 

			// _DON'T_ flush disk-inserted or os events or you'll be sorry! 
			::FlushEvents(everyEvent - diskMask - osMask, 0);
		
			::InitMenus();
			::TEInit();
			::InitDialogs(NULL);
		
			::InitCursor();
		}
		
		// The refnum where the application's resources should be found 
		gApplicationRefNum = LMGetCurMap();  // Don't use CurResFile_AC, it's not universal
		gCodeRefNum = LMGetCurApRefNum();	// Not necessarily the same as gApplicationRefNum under some IDE's
		
		// -1 == 0xFFFFFFFF, the largest 32 bit address. Our routine StripLong uses a
	    // pre-stripped address gStrippedAddress to avoid the yucky MPW glue. 
		gStrippedAddress = StripAddress((Ptr) - 1);
	}
}

//----------------------------------------------------------------------------------------
// ExpandHeap: 
//----------------------------------------------------------------------------------------
void ExpandHeap()
{
	// Sum up the standard memory requirements
	// and add on the processor dependent memory requirements
	// Then expand the heap while preserving the requested stack space
	
	const ResType memResType[2] = { 'mem!'
#if qPowerPC
	, 'ppc!'
#elif q68K
	, '68k!'
#elif qMSWindows
	, 'wn32'
#endif
	};
	
	for (int whichType = 0; whichType < 2; ++whichType)
	{
		ResType whichResType = memResType[whichType];
		ResNumber rsrcCnt = ::CountResources(whichResType);
		for (ResNumber i = 1; i <= rsrcCnt; ++i)
		{
			Handle h = ::GetIndResource(whichResType, i);
#if qDebug
			if (GetHandleSize(h) != sizeof(Mem))
				ProgramBreak_AC("\pI found a Mem! resource which is either corrupt or old style!");
#endif
			{
				const Mem &memH = **((MemHandle)h);
	
				gSizeObjectHeap += memH.objectHeap;
				gSizeHeapIncrement += memH.heapIncrement;
				gSizeTempReserve += memH.codeVal;
				gSizeLowSpaceReserve += memH.lowSpaceVal;
				gStackTot += memH.stackVal;
			}
			::ReleaseResource(h);
		}
	}
	
	SetStackSpace(gStackTot);

	::MaxApplZone();
}

//----------------------------------------------------------------------------------------
// SetStackSpace: 
//----------------------------------------------------------------------------------------
void SetStackSpace(long numBytes)
{
	// Make sure numBytes is even (don't use the "odd" macro in case it outlines
	if (Odd_AC(numBytes))
		++numBytes;

	// Check that we aren't running with a corrupt heap. If we are,
	// fix the problem. This was a bug with FBA's before System 7.5.5.
	// With System Software later than 7.5.5, this "fix" is harmless
	if (gIsOnlyBackground)
	{
		THz myZone = ::GetZone();
		if (myZone->bkLim != LMGetHeapEnd())
			LMSetHeapEnd(myZone->bkLim);
	}

	ptrdiff_t newLimit = static_cast<ptrdiff_t>(LMGetCurStackBase()) - numBytes;
	if (reinterpret_cast<ptrdiff_t>(::GetApplLimit()) > newLimit)
		::SetApplLimit((Ptr)newLimit);
}

//----------------------------------------------------------------------------------------
// Accessors: 
//----------------------------------------------------------------------------------------
Size GetSizeObjectHeap()
{
	UniversalStartup();
	
	return gSizeObjectHeap;
}

Size GetSizeHeapIncrement()
{
	UniversalStartup();
	
	return gSizeHeapIncrement;
}

Size GetSizeTempReserve()
{
	UniversalStartup();
	
	return gSizeTempReserve;
}

Size GetSizeLowSpaceReserve()
{
	UniversalStartup();
	
	return gSizeLowSpaceReserve;
}

Size GetStackTot()
{
	UniversalStartup();
	
	return gStackTot;
}

bool GetHasProcessMgr()
{
	UniversalStartup();
	
	return gHasProcessMgr;
}

bool IsOnlyBackground()
{
	UniversalStartup();
	
	return gIsOnlyBackground;
}

bool IsHighLevelEventAware()
{
	UniversalStartup();
	
	return gIsHighLevelEventAware;
}

short GetProcessor()
{
	UniversalStartup();
	
	return gProcessor;
}

bool HasFPU()
{
	UniversalStartup();
	
	return gHasFPU;
}


// E N D   U N I V E R S A L   C O D E


//----------------------------------------------------------------------------------------
// End of UniversalStartup.cp

