// USection.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __USECTION__
#define __USECTION__

//----------------------------------------------------------------------------------------
//  Unit: USection
//
//  OVERVIEW
//
// TSection is designed to provide Edition Manager support. TSection is an abstract
// superclass which references the main Edition Manager data structure (the
// SectionHandle), and a designator (designating which portion of the document is
// published/ subscribed). Specialized subclasses of TSection are required to represent the
// TPublisher and TSubscriber.
//
//  CLASS HIERARCHY
//
//  TSection		abstract superclass for sections
//  TPublisher		an Edition Manager publisher
//  TSubscriber		an Edition Manager subscriber
//
//  USAGE
//
// TPublisher/ TSubscriber is designed to be used with the Edition Manager support that is
// built into MacApp. Whenever a portion of the document is publisher/ subscribed, a
// TSection object is instantiated. The TSection object references a TDesignator.
//----------------------------------------------------------------------------------------

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CRandomAccessStream_AC__
#include "CRandomAccessStream_AC.h"
#endif

// MacApp

#ifndef __UDESIGNATOR__
#include "UDesignator.h"
#endif

// Toolbox

#ifndef __EDITIONS__
#include <Editions.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TDesignator;
class TEditionDocument;
class TFile;


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const bool kEditionFound = true;				// the edition file was found
const bool kEditionNotFound = false;			// the edition file wasn't found


//----------------------------------------------------------------------------------------
// passed into TSection::CancelSection()
//----------------------------------------------------------------------------------------

const bool kCancel = true;						// cancel the section
const bool kDontCancel = false;					// don't cancel the section


//----------------------------------------------------------------------------------------
// Section Change ID's
//----------------------------------------------------------------------------------------

const ChangeID_AC cSubscriberChanged = 610;		// change id indicating that the
                                                // subscriber changed

const ChangeID_AC cPublisherChanged = 611;			// change id indicating that the publisher
                                                // changed


//----------------------------------------------------------------------------------------
// TSection: TSection is an abstract superclass providing common support for sections -
// both subscribers and publishers.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSection : public MDependable_AC
{
	MA_DECLARE_CLONE;
	
public:
	//------------------------------------------------------------------------------------
	//  Contructors/Destructors
	//------------------------------------------------------------------------------------

	TSection(TEditionDocument* itsDocument = NULL,
								 TDesignator* itsDesignator = NULL,
								 SectionHandle itsSectionHandle = NULL,
								 short itsRsrcID = 0);
		// constructor
		
	virtual ~TSection();
		// calls Unregister, disposes the SectionHandle and AliasHandle


	//------------------------------------------------------------------------------------
	// Accessors
	//------------------------------------------------------------------------------------

	virtual void AttachSectionHandle(SectionHandle itsSectionHandle);
		// attach a SectionHandle to this TSection object

	virtual void GetEditionName(CStr255_AC& name);
		// Returns the name of the edition container file.

	//	virtual TFileHandler* GetFileHandler();
	//		// returns the TFileHandler of the document with which this section is associated

	virtual SectionType GetSectionType();
		// returns the (*fSectionHandle)->kind

	virtual UpdateMode GetUpdateMode();
		// returns the (*fSectionHandle)->mode


	//------------------------------------------------------------------------------------
	// Saving/Restoring
	//------------------------------------------------------------------------------------

	virtual void DoWrite(TFile* aFile, bool makingCopy);
		// writes the section handle as a sect resource and the alias handle as a alis
        // resource

	virtual void DoRead(TFile* aFile);
		// reads the section handle from a sect resource and the alias handle from a alis
        // resource

	virtual void DoNeedDiskSpace(long& dataForkBytes, long& rsrcForkBytes);
		// Bytes required to store saved sections on disk. When called, both parameters
        // are set to 0 for you.

	virtual void Associate(TFile* aFile);
		// call this to associate the section handle with a new section document


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual void Delete();
		// removes the section resources

	virtual TFile* GetCurrentFile(TFile* aFile);
		// given a file being written to in DoWrite, this returns the correct file

	virtual bool IsCanceled();
		// returns fCanceled

	virtual bool IsChanged();
		// returns true if the mdDate of the SectionRecord is newer than the mdDate of the
        // edition on disk

	virtual bool IsRegistered();
		// returns true if fRegistered & (IsRegisteredSection returns noErr)

	virtual void MarkAsChanged();
		// marks this publisher as changed; should be called each time the user edits the
        // data in the publisher

	virtual OSErr Register();
		// calls RegisterSection on the Section Handle and sets fRegisteredSection

	virtual void UnRegister();
		// calls UnRegisterSection on the Section Handle and sets fRegisteredSection

	virtual void UpdateEditionInfo();
		// updates fEditionInfo for later use

	virtual void CancelSection(bool cancel);
		// if cancel is true then calls UnRegister and sets fCanceled to true else if
        // cancel is false then calls Register and sets fCanceled to false

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline TDesignator* GetDesignator() const { return fDesignator; }
	inline TEditionDocument* GetDocument() const { return fDocument; }
	inline SectionHandle GetSectionHandle() const { return fSectionHandle; }
	inline short GetRsrcID() const { return fRsrcID; }
	inline bool GetNewSection() const { return fNewSection; }

#if qBackwardCompatible
public:
#else
protected:
#endif
	CAutoPtr_AC<TDesignator> fDesignator;		// the designated portion of the document

	TEditionDocument* fDocument;				// the document that owns this section

	SectionHandle fSectionHandle;				// reference to the section record for
                                                // this edition

	short fRsrcID;								// rsrc id of stored form (needed for
                                                // Read)

	EditionInfoRecord fLastEditionInfo;			// stored at various points for later use

	bool fCanceled;								// whether this section is canceled

	bool fFound;								// whether this edition was found by
                                                // RegisterSection

	bool fNewSection;							// this is a brand spankin' new section
};


//----------------------------------------------------------------------------------------
// TPublisher: each instance of this class represents a part of the document that is
// published
//----------------------------------------------------------------------------------------

class EXPORT_MA TPublisher : public TSection
{
	MA_DECLARE_CLONE;
	
public:

	//------------------------------------------------------------------------------------
	// Contructors/Destructors
	//------------------------------------------------------------------------------------

	inline TPublisher(TEditionDocument* itsDocument = NULL,
								   TDesignator* itsDesignator = NULL,
								   SectionHandle itsSectionHandle = NULL,
								   short itsRsrcID = 0) :
	TSection(itsDocument, itsDesignator, itsSectionHandle, itsRsrcID)
	{ }

	virtual ~TPublisher();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Change Notification
	//------------------------------------------------------------------------------------

	virtual void DoUpdate(ChangeID_AC theChange,
								 MDependable_AC* changedObject,
								 void* changeData,
								 CDependencySpace_AC* dependencySpace);
		// marks this publisher as changed calling MarkAsChanged


	//------------------------------------------------------------------------------------
	// Saving/Restoring
	//------------------------------------------------------------------------------------

	virtual void Publish(TFile* aFile);
		// publishes this edition to the edition file

	virtual void DoWrite(TFile* aFile, bool makingCopy);
	// calls INHERITTED Write if (mode is pumOnSave) & IsChanged then Publish


	//------------------------------------------------------------------------------------
	//  Miscellaneous
	//------------------------------------------------------------------------------------

	virtual void Delete();
		// deletes the edition file; called when the doc is saved if the user canceled the
        // publisher

	virtual void DeleteEditionFile();
		// delete the edition container file

	virtual bool IsChanged();
		// returns true if the mdDate of the SectionRecord is newer than the mdDate of the
        // edition on disk

	virtual void MarkAsChanged();
		// marks this publisher as changed; should be called each time the user edits the
        // data in the publisher

};


//----------------------------------------------------------------------------------------
// TSubscriber - each instance represents a subscriber for this document.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSubscriber : public TSection
{
	MA_DECLARE_CLONE;
	
public:
    //------------------------------------------------------------------------------------
	//  Contructors/Destructors
    //------------------------------------------------------------------------------------

	TSubscriber(TEditionDocument* itsDocument = NULL,
									TDesignator* itsDesignator = NULL,
									SectionHandle itsSectionHandle = NULL,
									short itsRsrcID = 0);

	virtual ~TSubscriber();
		// Destructor
		
    //------------------------------------------------------------------------------------
	// Saving/Restoring
    //------------------------------------------------------------------------------------

	virtual void Subscribe();
		// updates this subscriber from the edition on disk

	virtual void SubscribeIfNewer();
		// calls Subscribe if a newer edition is available


    //------------------------------------------------------------------------------------
	// Miscellaneous
    //------------------------------------------------------------------------------------

	virtual bool IsNewerEditionAvailable();
		// return true if there is a newer version of this edition on disk

	virtual void OpenPublisher();
		// calls GetEditionInfo & GotoPublisherSection

	virtual bool IsChanged();
		// returns true if the mdDate of the SectionRecord is newer than the mdDate of the
        // edition on disk

	virtual void MarkAsChanged();
		// marks this publisher as changed; should be called each time the user edits the
        // data in the publisher

#if qBackwardCompatible
public:
#else
protected:
#endif
	bool			fChanged;					// whether or not this guy has changed.
};


//----------------------------------------------------------------------------------------
// TSectionStream reads data from and writes data to an already open EditionContainerFile.
// Should be enhanced to support streaming data concurrently of various OS Types (e.g.
// PICT, TEXT, etc) rather than simply streaming one type at a time.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionStream : public CRandomAccessStream_AC
{
	MA_DECLARE_CLONE;
	
public:
    //------------------------------------------------------------------------------------
	// Contructors/Destructors
    //------------------------------------------------------------------------------------
	TSectionStream(EditionRefNum itsEditionRefNum = 0, OSType itsEditionOSType = kNoIdentifier);
		// Constructor

	virtual ~TSectionStream();
		// Destructor
		
    //------------------------------------------------------------------------------------
	// Miscellaneous
    //------------------------------------------------------------------------------------
	virtual void Reset(OSType itsEditionOSType, long itsSize);
		// Resets fPosition to 0, fEditionOSType to itsEditionOSType, and fSize to
        // itsSize.
};

//----------------------------------------------------------------------------------------
// TSectionList: Manages a sorted list of TSection objects.
//----------------------------------------------------------------------------------------

class EXPORT_MA TSectionList : public CSortedList_AC
{
	MA_DECLARE_CLONE;
	
public:

	inline TSectionList() : CSortedList_AC() { }
		// constructor

	virtual ~TSectionList();
		// Destructor
		
	virtual CompareResult_AC Compare(const void* item1,
								   const void* item2) const;
		// Compares 'item'1 with 'item2' returning an integer indicating the results of
        // the comparison
};

#endif // __USECTION__
