// UAdorners.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UADORNERS__
#define __UADORNERS__

// ACS

#ifndef __CDynamicArray_AC__
#include "CDynamicArray_AC.h"
#endif

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

// MacApp

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------
class CStream_AC;
class TView;
class TAdornerList;
class TAdorner;
class TDimAdorner;
class TDrawAdorner; 
class TEraseAdorner;
class THiliteAdorner;
class TPrintAdorner;
class TResizeIconAdorner;
class TSelectionAdorner;

//----------------------------------------------------------------------------------------
// Constants used to indicate adorner priorities
//----------------------------------------------------------------------------------------
enum
{
	kAdornLast = 127,				// Low priority adorners are considered last
	kAdornAfter = kAdornLast - 32,	// Low priority adorners are considered last
	kDrawView = 64,					// View priority is special case
	kAdornBefore = kDrawView - 32,	// High priority adorners take precedence
	kAdornFirst = 0					// High priority adorners take precedence
};

//----------------------------------------------------------------------------------------
// Constants for indicating whether an adorner should be destroyed.  After calling
// TAdorner::RemovedFromView you must test fFreeOnDeletion to see if the object should
// be destroyed.
//----------------------------------------------------------------------------------------
const bool kFreeOnDeletion = true;
const bool kDontFreeOnDeletion = !kFreeOnDeletion;

//----------------------------------------------------------------------------------------
// Constant signatures for some default adorners.
//----------------------------------------------------------------------------------------
enum
{
	kDimAdorner		= 'dima',
	kDrawAdorner		= 'draw',
	kEraseAdorner		= 'eras',
	kFrameAdorner		= 'fram',
	kHiliteAdorner		= 'hili',
	kLineBottomAdorner = 'lnbo',
	kLineLeftAdorner	= 'lnlf',
	kLineRightAdorner	= 'lnrt',
	kLineTopAdorner	= 'lntp',
	kOvalAdorner		= 'oval',
	kPrintAdorner		= 'prnt',
	kResizeIconAdorner = 'resz',
	kRRectAdorner		= 'rrct',
	kSelectionAdorner	= 'sele',
	kShadowedFrameAdorner	= 'sfra',
	kCompatibilityAdorner	= 'comp'
};

//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------
typedef char AdornPriority;

//----------------------------------------------------------------------------------------
// CAdornerIterator
//----------------------------------------------------------------------------------------
class EXPORT_MA CAdornerIterator : public CArrayIterator_AC
{
public:
	CAdornerIterator(const TView* theView,
					   ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					   bool itsForward);

	CAdornerIterator(const TView* theView, bool itsForward);

	CAdornerIterator(const TView* theView);

	CAdornerIterator(const TAdornerList* theAdornerList);

	//
	// Required functions 
	//
	TAdorner* First();
	inline TAdorner* Current() const { return fCurrent; }
	TAdorner* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TAdorner* operator++() { return Next(); }
	inline TAdorner* operator++(int) { TAdorner* returnVal = Current(); Next(); return returnVal; }
	inline operator TAdorner*() const { return Current(); }
	inline TAdorner* operator->() const { return Current(); }

protected:
	TAdorner* fCurrent;
};

//----------------------------------------------------------------------------------------
// TAdorner: abstract superclass for adorners which are basically  rendering algorithms  
//----------------------------------------------------------------------------------------
class EXPORT_MA TAdorner : public MStreamable
{
public:
	//------------------------------------------------------------------------------------
	// constructor and destructor
	//------------------------------------------------------------------------------------

	TAdorner(IDType itsID = kNoIdentifier, bool freeOnDeletion = true, bool isOpaque = false);

	virtual ~TAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	void Set(IDType itsID = kNoIdentifier, bool freeOnDeletion = true);

	//------------------------------------------------------------------------------------
	// notification
	//------------------------------------------------------------------------------------

	virtual void AddedToView(TView* itsView);
		// Notification that this adorner has been added to a view

	virtual void RemovedFromView(TView* itsView);
		// Notification that this adorner has been removed from a view


	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFrom(CStream_AC* aStream);	// Override
	
	virtual void WriteTo(CStream_AC* aStream) const;	// Override


	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);

	virtual void DrawLine(VHSelect vhs, short lh, short lv, short lto);
		// Utility routine for adorners - draws a line. 

	virtual void DoHighlightSelection(TView* itsView, 
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL);
		// Adorns the selection before the view's DoHighlightSelection. 

	virtual void InvalidateAdorner(TView* itsView);
		// Invalidates the adorner in itsView. 

	virtual void ViewChangedFrame (TView* itsView,
											const CViewRect& oldFrame,
											const CViewRect& newFrame,
											bool	invalidate);
		// This method is called when the owning view changes size 
		
	virtual bool DoesAdorn (const TView* itsView) const;
		// Default returns true. The adornment only includes enabled adorners
		// in its iteration.

	bool IsOpaque() const { return GetOpacity(); }

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline IDType GetIdentifier() const { return fIdentifier; }
	inline bool GetFreeOnDeletion() const { return fFreeOnDeletion; }

	inline bool GetOpacity() const { return fIsOpaque; }
	inline void SetOpacity(bool inOpacity) { fIsOpaque = inOpacity; }

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	IDType fIdentifier;							// The ID of this adorner, to distinguish
                                                // it from other kinds.

	bool fFreeOnDeletion;						// Should the adorner be freed when
                                                // removed from an adornment

	bool fIsOpaque;								// true if this adorner should be considered "opaque"
												// and none of the bits behind it will be visible

public:
	//----------------------------------------------------------------------------------------
	// Global, preallocated, shared adorners:
	//----------------------------------------------------------------------------------------
	static TDimAdorner fgDimAdorner;					// Calls the view's Dim method  
	static TDrawAdorner fgDrawAdorner;					// Calls the view's Draw method  
	static TEraseAdorner fgEraseAdorner;				// Erases the area passed in the DoDraw method
	static THiliteAdorner fgHiliteAdorner;				// Calls the view's Hilite method  
	static TPrintAdorner fgPrintAdorner;				// Calls the view's DoDrawPrintFeedback method
	static TResizeIconAdorner fgResizeIconAdorner; 		// Calls the TWindow's DrawResizeIcon method
	static TSelectionAdorner fgSelectionAdorner;		// Calls the view's DoHighlightSelection method


public:
	static TAdorner* NewStdAdorner(IDType signature,
									  const char* className,
									  IDType itsID,
									  bool freeOnDeletion);
};


//----------------------------------------------------------------------------------------
// TCompatibilityAdorner: Draws the adornment available in MacApp 2.0.
//----------------------------------------------------------------------------------------
class EXPORT_MA TCompatibilityAdorner : public TAdorner
{
public:
	TCompatibilityAdorner(CntlAdornment itsAdornment = 0);

	virtual ~TCompatibilityAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFrom(CStream_AC* aStream);	// Override
	
	virtual void WriteTo(CStream_AC* aStream) const;	// Override

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CntlAdornment fAdornment;	
};

//----------------------------------------------------------------------------------------
// TDimAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TDimAdorner : public TAdorner
{
public:
	inline TDimAdorner(IDType itsID = kDimAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TDimAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// calls itsView->Dim
};


//----------------------------------------------------------------------------------------
// TDrawAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TDrawAdorner : public TAdorner
{
public:
	inline TDrawAdorner(IDType itsID = kDrawAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TDrawAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// calls itsView->Draw(area)
};


//----------------------------------------------------------------------------------------
// TEraseAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TEraseAdorner : public TAdorner
{
public:
	inline TEraseAdorner(IDType itsID = kEraseAdorner, bool freeOnDeletion = kDontFreeOnDeletion, bool isOpaque = true) :
	TAdorner(itsID, freeOnDeletion, isOpaque)
	{}
	// erasure makes a view opaque

	virtual ~TEraseAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// calls itsView->Draw(area)
};

//----------------------------------------------------------------------------------------
// THiliteAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA THiliteAdorner : public TAdorner
{
public:
	inline THiliteAdorner(IDType itsID = kHiliteAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~THiliteAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		//  calls itsView->Hilite  
};

//----------------------------------------------------------------------------------------
// TPrintAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TPrintAdorner : public TAdorner
{
public:
	inline TPrintAdorner(IDType itsID = kPrintAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TPrintAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void DoHighlightSelection(TView* itsView, 
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL);	// Override
		// calls itsView->DoHighlightSelection(fromHL, toHL)
};

//----------------------------------------------------------------------------------------
// TResizeIconAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TResizeIconAdorner : public TAdorner
{
public:
	inline TResizeIconAdorner(IDType itsID = kResizeIconAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TResizeIconAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void DoHighlightSelection(TView* itsView, 
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL);	// Override
		// calls dynamic_cast<TWindow*>(itsView)->DrawResizeIcon
};

//----------------------------------------------------------------------------------------
// TSelectionAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TSelectionAdorner : public TAdorner
{
public:
	inline TSelectionAdorner(IDType itsID = kSelectionAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TSelectionAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void DoHighlightSelection(TView* itsView, 
											 const CViewRect& area,
											 HLState fromHL,
											 HLState toHL);	// Override
		//  calls itsView->DoHighlightSelection(fromHL, toHL)  
};

//----------------------------------------------------------------------------------------
// TFrameAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TFrameAdorner : public TAdorner
{
public:
	inline TFrameAdorner(IDType itsID = kFrameAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TFrameAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// frames the extent of itsView 
};

//----------------------------------------------------------------------------------------
// TShadowedFrameAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TShadowedFrameAdorner : public TAdorner
{
public:
	inline TShadowedFrameAdorner(IDType itsID = kShadowedFrameAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TShadowedFrameAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// frames the extent of itsView 
};

//----------------------------------------------------------------------------------------
// TLineBottomAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TLineBottomAdorner : public TAdorner
{
public:
	inline TLineBottomAdorner(IDType itsID = kLineBottomAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TLineBottomAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// draws bottom line 
};


//----------------------------------------------------------------------------------------
// TLineLeftAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TLineLeftAdorner : public TAdorner
{
public:
	inline TLineLeftAdorner(IDType itsID = kLineLeftAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TLineLeftAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// draws left line 
};

//----------------------------------------------------------------------------------------
// TLineRightAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TLineRightAdorner : public TAdorner
{
public:
	inline TLineRightAdorner(IDType itsID = kLineRightAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TLineRightAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// draws right line 
};

//----------------------------------------------------------------------------------------
// TLineTopAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TLineTopAdorner : public TAdorner
{
public:
	inline TLineTopAdorner(IDType itsID = kLineTopAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TLineTopAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// draws top line 
};

//----------------------------------------------------------------------------------------
// TOvalAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TOvalAdorner : public TAdorner
{
public:
	inline TOvalAdorner(IDType itsID = kOvalAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TOvalAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// draws oval adorner 
};

//----------------------------------------------------------------------------------------
// TRRectAdorner
//----------------------------------------------------------------------------------------
class EXPORT_MA TRRectAdorner : public TAdorner
{
public:
	inline TRRectAdorner(IDType itsID = kRRectAdorner, bool freeOnDeletion = kDontFreeOnDeletion) :
	TAdorner(itsID, freeOnDeletion)
	{}

	virtual ~TRRectAdorner();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual void Draw(TView* itsView, const CViewRect& area);	// Override
		// adorns itsView with a round rectangle 
};

//----------------------------------------------------------------------------------------
// AdornerElement
//----------------------------------------------------------------------------------------
struct AdornerElement
{
	inline AdornerElement(AdornPriority itsPriority, TAdorner* itsAdorner) :
		priority(itsPriority),
		adorner(itsAdorner)
	{ }
		// Constructor

	AdornPriority	priority;
	TAdorner* 		adorner;
};

typedef AdornerElement* AdornerElementPtr;

//----------------------------------------------------------------------------------------
// TAdornerList: A list for keeping TAdorner objects ordered by priority 
//----------------------------------------------------------------------------------------
class EXPORT_MA TAdornerList : public CSortedDynamicArray_AC
{
public:
	inline TAdornerList() : CSortedDynamicArray_AC(0,sizeof(AdornerElement)) { }

	virtual ~TAdornerList();
		// Destructor

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFrom(CStream_AC* aStream);	// Override
	
	virtual void WriteTo(CStream_AC* aStream) const;	// Override


	virtual CompareResult_AC CompareElements(const void* Element1,
											const void* Element2) const;	// Override
		// Compares Two TAdorner objects based on their priority 

	void AddAdorner(TAdorner* itsAdorner, AdornPriority itsPriority);
		// Add the adorner to the list 
	
	inline TAdorner* AdornerAt(ArrayIndex_AC index) const
	{ return ((AdornerElementPtr) ComputeAddress(index))->adorner; }
	
	inline AdornPriority PriorityAt(ArrayIndex_AC index) const
	{ return ((AdornerElementPtr) ComputeAddress(index))->priority; }
		// Return the priority stored in the element record associated with this
		// adorner.  Used when cloning a list of adorners.
	
};

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------
EXPORT_MA void InitUAdorners();

#endif


