// UBehavior.h
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UBEHAVIOR__
#define __UBEHAVIOR__

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

//----------------------------------------------------------------------------------------
// Some miscellaneous declarations.
//----------------------------------------------------------------------------------------

enum IdlePhase { idleBegin, idleContinue, idleEnd };

class MScriptableObject;
class TAppleEvent;
class TDocument;
class TEventHandler;
class TEvent;
class TToolboxEvent;

//----------------------------------------------------------------------------------------
// TBehavior
//----------------------------------------------------------------------------------------

class EXPORT_MA TBehavior : public MStreamable
{
public:
	TBehavior(IDType itsIdentifier = kNoIdentifier);
		// Constructor

	TBehavior(const TBehavior& inBehavior);
		// Copy Constructor
		
	virtual ~TBehavior();
	
	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Chain Management Methods
	//------------------------------------------------------------------------------------
	inline void SetNextBehavior(TBehavior* nextBehavior) { fNextBehavior.release(); fNextBehavior = nextBehavior; }
		// Set the fNextBehavior field

	inline void SetPreviousBehavior(TBehavior* previousBehavior) { fPreviousBehavior = previousBehavior; }
		// Set the fPreviousBehavior field

	inline TBehavior* GetNextBehavior() const { return fNextBehavior; }
		// Return the next behavior	

	TBehavior* GetNextEnabledBehavior() const;
		// Return the next enabled behavior	

	virtual void SetOwner(TEventHandler* itsOwner);
		// Set the fOwner field

	inline bool IsOwnerSelected() const { return fIsOwnerSelected; }
		// Returns the current state of the 'fIsOwnerSelected' flag 

	virtual void SelectOwner (bool select);
		// Called when this behavior's owner is selected. Sets the fIsOwnerSelected flag

	inline bool IsEnabled() const { return fEnabled; }
		// Returns the current state of the 'fEnabled' flag 
		
	virtual void SetEnabled(bool enabled);
		// Sets the 'fEnabled' flag.

	void InsertBefore(TBehavior* insertMe,
									 TBehavior* before);
		// Insert behavior 'insertMe' ahead of behavior 'before' in the chain 

	void InsertAfter(TBehavior* insertMe,
									TBehavior* after);
		// Insert behavior 'insertMe' after of behavior 'after' in the chain 

	void AppendBehavior(TBehavior* aBehavior);
		// Add 'aBehavior' to the end of the list 

	void RemoveBehavior(TBehavior* removeMe);
		// Remove behavior 'removeMe' from the chain 

	//------------------------------------------------------------------------------------
	// Dispatching
	//------------------------------------------------------------------------------------
	void HandleIdle(IdlePhase phase);
		// Calls DoIdle if the handler is eligible.

	//------------------------------------------------------------------------------------
	// Idling.
	//------------------------------------------------------------------------------------
	virtual void SetIdleFreq(long newIdleFreq);
		// Call to set the handler's idling frequency.

	unsigned long NextIdle();
		// Returns the number Tickcount for when the handler should next be idled.

	virtual bool DoIdle(IdlePhase phase);
		// Returns true if it frees itself so caller can know

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFrom(CStream_AC* aStream);

	virtual void WriteTo(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	//  Event Filtering Methods
	//------------------------------------------------------------------------------------
	virtual void DoCommandKeyEvent(TToolboxEvent* event);
		// Allows behaviors to respond to CommandKey events, if there is a next behavior
        // it passes the message on down the chain otherwise it calls the corresponding
        // method of its Owner 

	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// Allows behaviors to handle events, if there is a next behavior it passes the
        // message on down the chain otherwise it calls the corresponding method of its
        // Owner 

	virtual void DoKeyEvent(TToolboxEvent* event);
		// Allows behaviors to respond to KeyCommands, if there is a next behavior it
        // passes the message on down the chain otherwise it calls the corresponding
        // method of its Owner 

	virtual void DoKeyUp(TToolboxEvent* event);
		// Allows behaviors to respond to KeyUps, if there is a next behavior it passes
        // the message on down the chain otherwise it calls the corresponding method of
        // its Owner 

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Allows behaviors to handle menu events, if there is a next behavior passes the
        // message on down the chain otherwise it calls the corresponding method of its
        // Owner 

	virtual bool DoScriptCommand(CommandNumber		aCommandNumber,
									TAppleEvent*		message,
									TAppleEvent*		reply);
		// Allows behaviors to handle appleevents. Returns true if the event is handled,
		// and false if it was not.
		
	virtual void DoSetupMenus();
		// Allows behaviors to handle menu setup if there is a next behavior passes the
        // message on down the chain otherwise it calls the corresponding method of its
        // Owner 

	//  Special hooks which are used when attached to a TView

	virtual bool DoMouseCommand(CViewPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint_AC hysteresis);
		// Allow view behaviors to intercept mouse commands, if there is a next behavior
        // it passes the message on down the chain otherwise by default returns false 
		
	virtual bool DoMouseUp(CViewPoint& theMouse,
									   TToolboxEvent* event,
									   CPoint_AC hysteresis);
		// Allow view behaviors to intercept mouse up commands.

	virtual void Draw ( const CViewRect& area );
		// Called during a views drawing process, after the view has been called to draw
        // itself and before the views subviews are called to draw themselves. The 'area'
        // in need of drawing is passed in. 
		
	virtual bool DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion);
		// Gives the view behavior the opportunity to setup the cursor, if there is a nex
        // behavior it passes the message on down the chain otherwise by default returns
        // false 

	virtual bool DoUnsetCursor();
		// Gives the view behavior the opportunity to unset the cursor or othewise draw.
		// if there is a next behavior it passes the message on down the chain otherwise
		// by default returns false 

	//  Special hooks which are used when attached to a TApplication

	virtual bool DoToolboxEvent(TToolboxEvent* event);
		// When this is called by MacApp, theEvent is the event to be processed.
		
	virtual void DoPostCreate(TDocument* itsDocument);
		// Gives the behavior the opportunity to do any processing after its
		// associated view has been fully created

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline IDType GetIdentifier() const { return fIdentifier; }

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	CAutoPtr_AC<TBehavior> fNextBehavior;	//	The next behavior in the chain

	TBehavior* fPreviousBehavior;			//	The previous behavior in the chain
	TEventHandler* fOwner;					// The event handler that owns this behavior
	IDType fIdentifier;

	long fIdleFreq;							// the minimum number of ticks that can pass
                                            // before my DoIdle is called. 0 = call as
                                            // often as possible. n..kMaxIdleTime = call
                                            // every fIdleFreq ticks.
												
	long fLastIdle;							// the tick count the last time my DoIdle was
                                            // called.

	bool		fEnabled;					// True if this behavior is enabled. If false,
                                            // this behavior will be skipped over as
                                            // though it were not in the chain.

	bool		fIsOwnerSelected;			// True if this behavior's owner is "selected"
                                            // as in a prototyping environment. Treat this
                                            // instance variable as private, and call
                                            // TEventHandler::SetSelect
};


//----------------------------------------------------------------------------------------
// CBehaviorIterator: A simple iterator for the behavior chain.
//----------------------------------------------------------------------------------------

class EXPORT_MA CBehaviorIterator : public CIterator_AC
{
public:
	CBehaviorIterator(const TEventHandler* itsEventHandler);
	
	//
	// Required functions 
	//
	TBehavior* First();
	inline TBehavior* Current() const { return fCurrent; }
	TBehavior* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TBehavior* operator++() { return Next(); }
	inline TBehavior* operator++(int) { TBehavior* returnVal = Current(); Next(); return returnVal; }
	inline operator TBehavior*() const { return Current(); }
	inline TBehavior* operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline TBehavior* FirstBehavior() { return First(); }
	inline TBehavior* CurrentBehavior() const { return Current(); }
	inline TBehavior* NextBehavior() { return Next(); }

private:
	TBehavior*	fCurrent;
	TBehavior*	fFirstBehavior;
	TBehavior*	fNextBehavior;
};

#endif
