// UCommand.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UCOMMAND__
#define __UCOMMAND__

// ACS

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TAppleEvent;
class TCommand;
class TCommandHandler;
class TView;

//----------------------------------------------------------------------------------------
// Some typedefs
//----------------------------------------------------------------------------------------

typedef EventPriority CommandPriority;

//----------------------------------------------------------------------------------------
// Some constants
//----------------------------------------------------------------------------------------

const bool kCanUndo = true;
const bool kCantUndo = false;
const bool kCausesChange = true;
const bool kDoesNotCauseChange = false;

//----------------------------------------------------------------------------------------
// Global Function Prototypes
//----------------------------------------------------------------------------------------

EXPORT_MA TCommand* CommitACommand(TCommand *aCommand);

//----------------------------------------------------------------------------------------
// TCommand:Handles a user command, including undoing and mouse-tracking if required.
// Usually created by the part of the Application with the greatest specific knowledge
// about the action to take in response to events and passed back to be performed at
// several well defined places.
//----------------------------------------------------------------------------------------

class EXPORT_MA TCommand : public TEvent, public MDependable_AC
{
public:
	//------------------------------------------------------------------------------------
	// Init & Free  
	//------------------------------------------------------------------------------------
	TCommand(CommandNumber itsCommandNumber = cNoCommand,
								 TCommandHandler* itsContext = NULL,
								 bool canUndo = true,
								 bool causesChange = true,
							 	 MDependable_AC* objectToNotify = NULL);
		// constructor
		
	virtual ~TCommand();
		// Frees fClipboardView and fUndoClipboardView when appropriate
		
	virtual void AbandonClipboardView();
		// Frees fClipboardView when appropriate
		
	virtual void AbandonUndoClipboardView();
		// Frees fUndoClipboardView when appropriate

	virtual void PrepareForUndoRedo();
		// Make the command context visible for undo/redo. 
	
	virtual void FinishUndoRedo();
		// Common undo/redo code. 

	virtual void RevealUndoRedo();
		// Make this context visible after undo/redo. 

	//------------------------------------------------------------------------------------
	// command processing  
	//------------------------------------------------------------------------------------

	virtual void Process(); // override
		// Performs the command (instead of processing as an event). 
	
	virtual void SetValidationError(OSErr validationError);
		// Set the validation state of this command
		
	virtual void Abort();
		// Clean up after command fails. 
		
	//------------------------------------------------------------------------------------
	// command processing - usually overridden   
	//------------------------------------------------------------------------------------

	//	virtual void Commit();
	//		// Stub  

	virtual void DoIt();
		// Stub  

	virtual void RedoIt();
		// Stub  

	virtual void UndoIt();
		// Stub  

	virtual void DoNotification();
		// Default calls fNotify->Changed if fCausesChange is true
	
	virtual void SetupDependencies();
		// Undoable commands are dependents of their contexts. When 
		// the context is changed, the command is committed. 

	virtual void DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject, 
						  void* changeData,
						  CDependencySpace_AC* dependencySpace); // override
		// If the context closed, the command gets committed

	virtual void Completed();
		// Called after the command's DoIt method has been called. Also called if
		// failure occurs during the process of DoIt, UndoIt, or RedoIt.
	
	//------------------------------------------------------------------------------------
	// Clipboard handling  
	//------------------------------------------------------------------------------------

	virtual void ClaimClipboard(TView* clipboardView);
		// Installs the specified view in the clipboard window. Saves the current
		// clipboard view for purposes of Undo
		
	virtual void UndoRedoClipboard();
		// Installs the appropriate view in the clipboard window
		
	//------------------------------------------------------------------------------------
	// Scheduling  
	//------------------------------------------------------------------------------------

	virtual bool IsReadyToExecute();
		// Returns true (the Default) when the command is ready to execute. Override to
		// base the execution of the command on more sophisticated criteria. Most programs
		// won't have to mess with this..  

	virtual bool ShouldFreeOnCompletion();
		// Overridden to return the value stored in fFreeOnCompletion  

	virtual bool IsRecurring();
		// Overridden to return the value stored in fRecurring  

	virtual TAppleEvent* MakeAppleEvent();  
		// Make an Apple Event corresponding to the command. Override for your commands. 

	inline bool CanBeUndone() const { return fCanUndo; }
		// Default behavior is to return the value stored in fCanUndo  

	inline bool WillCauseChange() const { return fCausesChange; }
		// Default behavior is to return the value stored in fCausesChange
		
	//------------------------------------------------------------------------------------
	// Change notification  
	//------------------------------------------------------------------------------------

	virtual ChangeID_AC GetChangeID();
		// Returns a ChangeID_AC for TApplication::PerformCommand to pass to Changed. Returns
		// fIdentifier as a default.  

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline bool GetUseAppleEvent() const { return fUseAppleEvent; }
	inline bool GetCommandDone() const { return fCommandDone; }
	inline TCommandHandler* GetContext() const { return fContext; }
	inline OSErr GetValidationError() const { return fValidationError; }

	inline void SetUseAppleEvent(bool inUseAppleEvent) { fUseAppleEvent = inUseAppleEvent; }
	inline void SetCommandDone(bool inCommandDone) { fCommandDone = inCommandDone; }
	//------------------------------------------------------------------------------------
	// data members  
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	MDependable_AC* fObjectToNotify;				// object to notify of changes  

	TCommandHandler* fContext;					// A document or view. If the command is
												// undoable it will be committed when the
												// context is closed  
										  
	TView* fClipboardView;						// The view installed in the clipboard by this 
												// command in its DoIt and RedoIt phases
	TView* fUndoClipboardView;					// The view installed in the clipboard by this 
												// command in its UndoIt phase
	TCommandHandler* fUndoClipboardViewContext; // The context of the command which originally 
												// placed the current saved fUndoClipboardView
												// in the clipboard window
 
	OSErr fValidationError;						// noErr (the default) indicates that neither
												// this command nor a linked command have
												// failed.
	
	bool fCommandDone;							// "Done" as in DoIt/UndoIt/RedoIt. 

	bool fCanUndo;								// Defaults to true  

	bool fCausesChange;							// Defaults to true; Marks document
												// changed when command is done  

	bool fFreeOnCompletion;						// True to cause the command to be freed
												// on completion (the default). Completion
												// is after DoIt for non-undoable commands
												// and Commit for undoable commands.  

	bool fReadyToExecute;						// True (the default) will enable this
												// command to be returned from any queues
												// that may be holding it. Most programs
												// won't have to mess with this.  

	bool fRecurring;							// False (the default) will enable this
												// command to be removed from any command
												// queue that holds it because it is only
												// executed once.
												
	bool fUseAppleEvent;						// True if the command should be handled by
												// sending an Apple Event
	
	bool fChangesClipboard;						// Defaults to false. Set to true for
												// command subclasses representing CUT and/
												// or COPY commands which change the
												// Clipboard.  
};


//----------------------------------------------------------------------------------------
// TAppleCommand: Abstract class for TServerCommand and TClientCommand.
//----------------------------------------------------------------------------------------

class EXPORT_MA TAppleCommand : public TCommand
{
public:

	inline TAppleCommand(CommandNumber itsCommandNumber = cNoCommand,
					 TCommandHandler* itsContext = NULL,
					 bool canUndo = true,
					 bool causesChange = true,
					 MDependable_AC* objectToNotify = NULL,
					 TAppleEvent* message = NULL,
					 TAppleEvent* reply = NULL) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),
	fMessage(message),
	fReply(reply)
	{ }
		// Constructor
		
	virtual ~TAppleCommand();
	
	virtual void FreeTheMessage();
		// deletes the fMessage and fReply.
		
//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
protected:
	CAutoPtr_AC<TAppleEvent> fMessage;
	CAutoPtr_AC<TAppleEvent> fReply;
};


//----------------------------------------------------------------------------------------
// TServerCommand: Should be used to service an AppleEvent.  It should also be used for all
// commands that can be invoked by an AppleEvent _OR_ some user event (DoMenuCommand).
// A good example is the 'odoc' command. The 'odoc' command's DoIt method simply opens
// a list of files. TServerCommand lets that list come from an AppleEvent or from some user
// action.
//----------------------------------------------------------------------------------------

class EXPORT_MA TServerCommand : public TAppleCommand
{
public:

	TServerCommand(CommandNumber itsCommandNumber = cNoCommand,
						TCommandHandler* itsContext = NULL,
						bool canUndo = true,
						bool causesChange = true,
						MDependable_AC* objectToNotify = NULL,
						TAppleEvent* itsMessage = NULL,
						TAppleEvent*itsReply = NULL,
						bool suspendTheEvent = false);
		// Construct a TServerCommand from MacApp TAppleEvent objects. 
		// Replaces the old InitializeFromAppleEvent method. 
		
	TServerCommand(CommandNumber itsCommandNumber,
						TCommandHandler* itsContext,
						bool canUndo,
						bool causesChange,
						MDependable_AC* objectToNotify,
						const AppleEvent& itsMessage,
						const AppleEvent& itsReply,
						bool suspendTheEvent = false);
		// Construct a TServerCommand from Apple Events. 
		
	virtual ~TServerCommand();
		// Destructor

	virtual void Process(); 			// override
		// Check the state of fRequiresUserInteraction and call AEInteractWithUser
		// if necessary.  Report any errors back to the AppleEvent manager.
	
	void ResumeAndFreeMessage();
		// If the event has been suspended then resume it and free the message.
	
	virtual void SetValidationError(OSErr error);
		// Report any errors in the supplied reply.
		
	virtual void Completed();		// override
		// Called when the command's DoIt method has been called, and after commit
		// if undoable.  Also called if failure ocurred during the process of doing
		// the command.  Overridden to Resume the suspended AppleEvent.

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	bool fRequiresUserInteraction;		// If true AEInteractWithUser will be called
											// before the command is processed
	bool fSuspendTheEvent;				// If true AESuspendTheCurrentEvent will be
											// called before the command is processed
		
protected:
	bool fReplySent;		// Set to TRUE in completed if an fReply was sent.

};


//----------------------------------------------------------------------------------------
// TClientCommand: can be used to send an AppleEvent.  If the message is to be sent
// kAENoReply then the default behavior is to send its fMessage in the DoIt Method.
// kAEWaitReply will send the message and process the reply before being posted to
// the event queue for normal command processing.  kAEQueueReply will send the message
// and then place itself in a pending queue until its matching reply is received.  Then it
// will process the reply and allow itself to be posted to the event queue for normal
// processing.
//----------------------------------------------------------------------------------------

class EXPORT_MA TClientCommand : public TAppleCommand
{
public:

	TClientCommand(CommandNumber itsCommandNumber = cNoCommand,
									TCommandHandler* itsContext = NULL,
									bool canUndo = true,
									bool causesChange = true,
									MDependable_AC* objectToNotify = NULL,
									TAppleEvent* theMessage = NULL);
		// Constructor
		
	virtual ~TClientCommand();
		// Destructor

	virtual long GetReturnID();
		// Return the return ID of fMessage.  Used to match an incoming reply with
		// this command.
									  
	virtual bool IsReadyToPost();	// override
		// check that message has been sent and the reply received before posting the
		// command to the event queue.
									  
	virtual void ProcessReply(TAppleEvent* theReply);
		// Assign theReply to fReply.  Optionally one could process the contents of the
		// reply here and assign them to local instance variables

	virtual TAppleEvent* SendMessage();
		// Send the message stored in fMessage and return the message if one is returned

	virtual void DoIt();	// override
		// Default behavior for a kAENoReply message is to send the message
		// If your message is to be sent by WaitReply or QueueReply you should override
		// DoIt to do your own thing

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	bool fMessageSent;
	
};


#endif // __UCOMMAND__