// UDesignator.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UDESIGNATOR__
#define __UDESIGNATOR__

//----------------------------------------------------------------------------------------
// OVERVIEW
// ========
// TDesignator is designed as an abstract superclass which designates a portion of the
// document. Each specialized document will require its own specialized designator.
// Specialized subclasses of TDesignator are required to designate portions of different
// documents.
//  
// CLASS HIERARCHY
// ===============
// TDesignator			abstract superclass for designators
// TLinearDesignator	a linear data designator (e.g. for text models)
// TRectDesignator		a rectangular designator (e.g. for spreadsheet models)
// TRgnDesignator		a region designator (e.g. for discontiguous designations)
//
// USAGE
// =====
// TDesignator is used with the Edition Manager support that is built into MacApp.
// Whenever a TSection object is instantiated, it requires a TDesignator object that lets
// the TSection object determine which portion of the document is being designated as a
// section, whether a publisher or a subscriber.
//----------------------------------------------------------------------------------------

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

// ACS

#ifndef __CTempRegion_AC__
#include "CTempRegion_AC.h"
#endif

// Toolbox

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

//----------------------------------------------------------------------------------------
// Containment constants
//----------------------------------------------------------------------------------------

enum ContainmentType { kNotContained, kPartiallyContained, kExactlyContained, kFullyContained };

//----------------------------------------------------------------------------------------
// Object Signtures: These follow the same reservation rules as resource types. The
// signature is used to search the signature table and create an object "by signature"If
// the signature's entry in the table has been updated with a new class then an object of
// that class will be created.
//----------------------------------------------------------------------------------------

const IDType kLinearDesignator = 'ldsg';

const IDType kVRectDesignator = 'vdsg';

const IDType kRegionDesignator = 'rdsg';

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TStream;

//----------------------------------------------------------------------------------------
// TDesignator: is designed as an abstract superclass. Use subclasses of TDesignator.
//----------------------------------------------------------------------------------------

class EXPORT_MA TDesignator : public MScriptableObject, public MStreamable
{
	MA_DECLARE_CLONE;
	
public:
	//----------------------------------------------------------------------------------------
	// contructors/destructors
	//----------------------------------------------------------------------------------------

	TDesignator();
		// constructor

	virtual ~TDesignator();
		// Destructor
		
	//----------------------------------------------------------------------------------------
	// utility methods
	//----------------------------------------------------------------------------------------

	virtual Size GetSize() const;
		// returns the size of this designation

	virtual ContainmentType IsContained(TDesignator* aDesignator);
		// Override this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in this

	virtual bool IsEmpty() const;
		// Returns false if the size of this designation is greater than 0

};


//----------------------------------------------------------------------------------------
// TLinearDesignator: is designed to designate any linear portion of data that may be
// designated by a start position and an end position.
//----------------------------------------------------------------------------------------

class EXPORT_MA TLinearDesignator : public TDesignator
{
	MA_DECLARE_CLONE;
	
public:
	//----------------------------------------------------------------------------------------
	//  contructors/ destructors
	//----------------------------------------------------------------------------------------

	TLinearDesignator(long startPos = 0,
										  long endPos = 0);
		// Constructor
	virtual ~TLinearDesignator();
		// Destructor
		
	//----------------------------------------------------------------------------------------
	// utility methods
	//----------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;
		// returns the signature for this class

	virtual Size GetSize() const;
		// returns the size of this designation

	virtual ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in this

	//----------------------------------------------------------------------------------------
	//  saving
	//----------------------------------------------------------------------------------------

	virtual void WriteTo(CStream_AC* aStream) const;
		// OVERRIDE this; writes specification of designator to stream

	virtual void ReadFrom(CStream_AC* aStream);
		// initializes this object from a stream specification

public:
	long fStartPos;								// offset to start
	long fEndPos;								// offset to end
};


//----------------------------------------------------------------------------------------
// TViewRectDesignator: is designed to designate any rectangular portion of data that may be
// designated by top, left, bottom, and right ViewCoordinates.
//----------------------------------------------------------------------------------------

class EXPORT_MA TViewRectDesignator : public TDesignator
{
	MA_DECLARE_CLONE;
	
public:
	//----------------------------------------------------------------------------------------
	//  contructors/destructors
	//----------------------------------------------------------------------------------------

	TViewRectDesignator(const CViewRect& itsDesignation = gZeroVRect);

	virtual ~TViewRectDesignator();
		// Destructor
		
	//----------------------------------------------------------------------------------------
	// utility methods
	//----------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;
		// returns the signature for this class

	virtual Size GetSize() const;
		// returns the size of this designation

	virtual ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in this

	//----------------------------------------------------------------------------------------
	// saving
	//----------------------------------------------------------------------------------------

	virtual void WriteTo(CStream_AC* aStream) const;
		// OVERRIDE this; writes specification of designator to stream

	virtual void ReadFrom(CStream_AC* aStream);
		// initializes this object from a stream specification

public:
	CViewRect fDesignation;							// CViewRect representing designation
};


//----------------------------------------------------------------------------------------
// TRegionDesignator: is designed to designate any portion of data that may be designated by
// a region. The caller must create the region, and the designator optionally frees the
// region. In general, the TStream passed into this object should be a THandleStream.
//----------------------------------------------------------------------------------------

class EXPORT_MA TRegionDesignator : public TDesignator
{
	MA_DECLARE_CLONE;
	
public:
	//----------------------------------------------------------------------------------------
	// contructors/destructors
	//----------------------------------------------------------------------------------------

	TRegionDesignator(RgnHandle itsDesignation = NULL,
									   bool freeRegionOnFree = true);
		// Constructor
		
	TRegionDesignator(const TRegionDesignator& inRegionDesignator);
		// Copy Constructor

	virtual ~TRegionDesignator();
		// Frees fDesignation and calls Inherited::Free.


	//----------------------------------------------------------------------------------------
	// utility methods
	//----------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;
		// returns the signature for this class

	virtual Size GetSize() const;
		// returns the size of this designation

	virtual ContainmentType IsContained(TDesignator* aDesignator);
		// returns a ContainmentType indicating to what extent aDesignator is contained in this

	virtual void SetDesignationRect(const CRect_AC& theDesignation);
		// sets fDesignation to contain this rectangle

	//----------------------------------------------------------------------------------------
	//  saving
	//----------------------------------------------------------------------------------------

	virtual void WriteTo(CStream_AC* aStream) const;
		// OVERRIDE this; writes specification of designator to stream

	virtual void ReadFrom(CStream_AC* aStream);
		// initializes this object from a stream specification

public:
	bool fFreeRegionOnFree;							// whether to free the RgnHandle when this
                                                	// designator is freed

	CTempRegion_AC fDesignation;						// rgn representing designation
};

#endif
