// UDispatcher.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UDISPATCHER__
#define __UDISPATCHER__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

#ifndef __CListIterator__
#include "CListIterator_AC.h"
#endif

#ifndef __CProcess_AC__
#include "CProcess_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __USCRIPTABLEOBJECT__
#include "UScriptableObject.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

//----------------------------------------------------------------------------------------
// Arguments of TDispatcher::GetActiveWindow
//----------------------------------------------------------------------------------------
const bool kFloatersOK = true;
const bool kNoFloaters = false;

//----------------------------------------------------------------------------------------
// Timeout Argument of TDispatcher::InteractWithUser
//----------------------------------------------------------------------------------------

const long kMacAppTimeout = 901;	// let MacApp decide how to handle the timeout

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TDocument;
class TEvent;
class TDispatcher;
class TFile;
class TMenuBarManager;
class TToolboxEvent;
class TTracker;
class TView;
class TWindow;

//----------------------------------------------------------------------------------------
//  TEventList: A list for keeping TEvents ordered by priority
//----------------------------------------------------------------------------------------

class EXPORT_MA TEventList : public CSortedList_AC
{
public:
	inline TEventList() : CSortedList_AC() { }
		// constructor

	virtual ~TEventList();
		// Destructor

	virtual CompareResult_AC Compare(const void* item1,
								  const void* item2) const;
		// Compares Two TEvents based on their priority

	virtual void Insert(const void* item);
		// Overridden to guarantee insertion even in low memory and to preserve arrival
        // sequence within "Compare" ordering.
};

//----------------------------------------------------------------------------------------
// TDispatcher: The purpose of this class is to implement the main event loop that all
// Macintosh applications must have. There is only one instance of TDispatcher. One thing
// that TDispatcher does is to interpret the raw events that are posted and convert them
// into higher-level events. For example, it converts a click in the menu bar into a 'menu
// event' that contains the menu ID/ item number that was chosen by the user.
//----------------------------------------------------------------------------------------

class EXPORT_MA TDispatcher :	public TCommandHandler,
								public MDefaultScriptableObject,
								public CProcess_AC
{
public:
	//------------------------------------------------------------------------------------
	// Initialization and Free
	//------------------------------------------------------------------------------------
	TDispatcher(OSType itsMainFileType = kTextType_AC,
									 OSType itsCreator = kAllQuestionsType_AC);
		// itsMainFileType should be the 4-byte file type that this application reads and
		// writes.  itsCreator is the application's signiture.  They are used when creating
		// a default file with DoMakeFile and to display the proper file types in the
		// StandardFile dialogs.

	virtual ~TDispatcher();
		// Destructor
	
	//------------------------------------------------------------------------------------
	// Dispatching
	//------------------------------------------------------------------------------------
	virtual void HandleToolboxEvent(TToolboxEvent* event);
		// Handles the dispatching of DoToolboxEvent messages between itself and Behaviors.

	virtual void HandleAlienEvent(TToolboxEvent* event);
		// This method is called upon to deal with 'application-specific' events

	virtual void HandleActivateEvent(TToolboxEvent* event);
		// Handles activate (and deactivate) events. Calls the window's Activate method,
        // and invalidates the menubar if activating the window.

	virtual void HandleDiskEvent(TToolboxEvent* event);
		// Handles disk events. Calls DIBadMount if the high word of theEvent's message is
        // <> noErr

	virtual void HandleHighLevelEvent(TToolboxEvent* event);
		// Process a high level event. Default behavior is to assume it is an AppleEvent.

	virtual void HandleKeyDownEvent(TToolboxEvent* event);
		// Handles key down events. If the command key is down, then this calls
        // CommandKey. Otherwise it calls GetTarget()->DoKeyEvent.

	virtual void HandleKeyUpEvent(TToolboxEvent* event);
		// Handles key up events. Calls GetTarget()->DoKeyEvent.

	virtual void HandleMouseDown(TToolboxEvent* event);
		// Called when a mouse down occurs; this cases on whereMouseDown and does the
        // right thing.

	virtual void HandleMouseUp(TToolboxEvent* event);
		// Sets fLastUpTime.

	virtual void HandleSystemEvent(TToolboxEvent* event);
		// Handles "system" events, in particular suspend/ resume events and mouse moved
        // events. Calls RegainControl if switching in, AboutToLoseControl if switching
        // out, or TrackCursor if it is a mouse-moved event.

	virtual void HandleUpdateEvent(TToolboxEvent* event);
		// Handles window update events. Calls UpdateEvent for the window indicate by the
        // event.

	virtual void HandleCursor(CPoint_AC globalMouse, RgnHandle cursorRegion);

	virtual void HandleHelp(CPoint_AC globalMouse, RgnHandle helpRegion);

#if qAttachable
	virtual bool HandleOSAEvent(CommandNumber aCommandNumber,
									TAppleEvent* message,
									TAppleEvent* reply);
	// Overridden to not give OSA the run AppleEvent.
#endif

	//------------------------------------------------------------------------------------
	// Event Handling
	//------------------------------------------------------------------------------------
	virtual TToolboxEvent* GetEvent(short eventMask, unsigned long sleep, 
											RgnHandle sleepRegion);
		// Calls WaitNextEvent (or GetNextEvent) with the given event mask, sleep, and
        // sleep region parameters. You can override this if you have an alternate source
        // for events.

	virtual void MainEventLoop();
		// Loop on PollEvent until the application is done

	virtual void DoToolboxEvent(TToolboxEvent* event);
		// When this is called by MacApp, theEvent is the event to be processed.

	virtual void DidEvent(TToolboxEvent* event);

	virtual void KeyEventToComponents(TToolboxEvent* event);
		// re-extracts components of an event in a script manager compatible way.

	virtual TTracker* TrackMouse(const CViewPoint& theMouse, CPoint_AC hysteresis,
										TTracker* theCommand);
		// Responsible for mouse tracking. !!! Candidate for eventhandler method?

	virtual void OpenDeskAccessory(const CStr255_AC& deskAccName);
		// Attempts to open the desk accessory whose name is deskAccName. Called from
        // TDispatcher::MenuEvent when an Apple menu item is chosen, and the item has a
        // negative command number.

	virtual void PollEvent(bool allowApplicationToSleep);
		// Polls for events/commands and processes them.

	virtual bool PollToolboxEvent(bool allowApplicationToSleep);
		// Polls for events calling Idle before waitNextEvent if there are no events
        // pending and the application is allowed to go to sleep
        // (kAllowApplicationToSleep), calls HandleEvent with the event, if there is one.
        // and then idles if necessary.
	
	virtual void DoInitialState();
		// Run calls this to perform polymorphic initialization before Run calls the main event loop.
		// Override and call inherited first, to add your own application-specific initialization prior to Run.
		
	void Run();
		// call MainEventLoop

	virtual void UpdateAllWindows();
		// Process all update events in the queue.

	virtual bool InModalState();
		// Returns true if the front window is modal.

	short GetMainEventMask();
		// Retrieve the current main event mask.
		
	short SetMainEventMask(short newMask);
		// Set the main event mask and return the old one.

	//------------------------------------------------------------------------------------
	// Target and Cohandlers
	//------------------------------------------------------------------------------------

	virtual void InstallCohandler(TEventHandler* aCohandler,
										 bool addIt);
		// Used to add (addIt = true) or delete (addIt = false) a cohandler to/ from the
        // global list of cohandlers

	virtual TWindow* GetFrontWindow();
		// Returns the window object reference for the frontmost application window
        // (whether active or not). 
		// Returns NULL if the FrontWindow window is not a window object.

	virtual TWindow* GetActiveWindow(bool floatersAreOkay);
		// Returns the window object reference for the active application window.
		// Returns NULL if the FrontWindow window is not a window object.

	//------------------------------------------------------------------------------------
	// Opening and Printing Documents
	//------------------------------------------------------------------------------------

	virtual TDocument* FindDocument(TFile* aFile);
		// Given a TFile, if that document is already opened, returns the TDocument.
        // Otherwise returns NULL.

	virtual TFile* DoMakeFile(CommandNumber itsCommandNumber);
		// Default behavior is to return an instance of TFile. May be overridden to create
        // a file of your own special type.

	//------------------------------------------------------------------------------------
	// Termination
	//------------------------------------------------------------------------------------

	virtual void Close();
		// Called when the user chooses Quit from the menu, and tries to save all the open
        // documents. If all succeed the application terminates. Signals Failure with
        // error = noErr and message = messageCancelled if user cancels


	//------------------------------------------------------------------------------------
	// Hierarchy
	//------------------------------------------------------------------------------------

	virtual void AddDocument(TDocument* aNewDocument);
		// Add another document to my list of documents, and make it the current one

	virtual void AddWindow(TWindow* aWindow);
		// Add the window to the free-window list

	virtual void DeleteDocument(TDocument* docToDelete);
		// Delete a document from my list of documents

	virtual void DeleteWindow(TWindow* windowToDelete);
		// Delete a window from my list of free windows


	//------------------------------------------------------------------------------------
	// Window Manager Windows
	//------------------------------------------------------------------------------------
	
	virtual WindowRef NewToolboxWindow(short rsrcId,
										bool& isResizable,
										bool& isClosable);
		// Get a Window Manger window resource whose resource id is rsrId. Return
        // isResizable and isClosable based upon the 'WIND' resource.

#if qBackwardCompatible
	virtual TWindow* WMgrToWindow(WindowRef aWMgrWindow);
		// Returns the window object that represents the given Window Manager window, or
        // NULL if there is no window object.
#endif	

	//------------------------------------------------------------------------------------
	// Idle Events
	//------------------------------------------------------------------------------------
	
	virtual unsigned long GetWaitTicks(bool allowApplicationToSleep);
		// Compute the "wait ticks" to be passed to GetEvent

	virtual void Idle(IdlePhase phase);
		// Parcels out idle time to the DoIdle method in the target and cohandler chains
        // of TEvtHandlers. If your application needs to do something at idle time then
        // override DoIdle instead of this method.

	//------------------------------------------------------------------------------------
	// Cursor and Help
	//------------------------------------------------------------------------------------
	
	virtual RgnHandle GetSleepRegion(const CPoint_AC* forGlobalMouse = NULL);
		// Return the "sleep region" to be passed to GetEvent
		// if globalMouse is non NULL it will be used to calculate the region
		// otherwise the current mouse position will be used. This is useful in
		// responding to mouse moved events, where we want to give the highest fidelity
		// feedback and are less concerned about lazy evaluation.

	virtual void GetDefaultCursorRegion(CPoint_AC globalMouse, RgnHandle cursorRegion);
		// Computes the cursor region for the case where no window has claimed the cursor

	virtual void InstallHelpMenuItems();
		// Override this method to install your help menu item(s).  Use
		// gMenuBarManager->AddHelpMenuItem( "^0 Help", kYourHelpCommandNumber );
		
	virtual void GetDefaultHelpRegion(CPoint_AC globalMouse,  RgnHandle helpRegion);
		// Computes the help region for the case where no window has claimed the cursor

	virtual void GetHelpParameters(ResNumber helpResource,
									short helpIndex,
									short helpState,
									HMMessageRecord& helpMessage,
									CPoint_AC& localQDTip,
									CRect_AC& localQDRect,
									short& balloonVariant);
		// Gets balloon contents from an 'hdlg' resource. Called by TView::GetHelpParameters

	virtual void DoSetCursor(CPoint_AC globalMouse, RgnHandle cursorRegion);
		// Sets the cursor to an arrow and computes the region for which it is claimed.
        // Called when no view has set the cursor

	virtual void TrackCursor(CPoint_AC globalMouse);
		// Allows windows/views to set the cursor and cursor region via TView::HandleCursor
        // and TView::DoSetCursor. If no region is set, the cursor is set to an arrow and
        // the largest possible default cursor region is calculated.

	virtual void DoShowHelp(CPoint_AC globalMouse, RgnHandle helpRegion);
		// !!!

	virtual void TrackHelp(CPoint_AC globalMouse);
		// !!!

	void InvalidateMouseRegions();
		// Invalidates the cursor and help regions

	void InvalidateCursorRgn();
		// Sets the cursor region to the empty region

	void InvalidateHelpRgn();
		// Sets the help region to the empty region

	virtual bool IsCursorRgnInvalid();
		// Returns true if the cursor region needs to be recalculated

	virtual bool IsHelpRgnInvalid();
		// Returns true if the help region needs to be recalculated
	

	//------------------------------------------------------------------------------------
	// Keyboard events
	//------------------------------------------------------------------------------------
	
	virtual void DoKeyEvent(TToolboxEvent* event);
		// Handle keydown events.


	//------------------------------------------------------------------------------------
	// Menu Events
	//------------------------------------------------------------------------------------
	
	virtual void DoCommandKeyEvent(TToolboxEvent* event);
		// Called when a keyDown event is received and the command key is down.

	virtual void MenuEvent(short theMenuNumber, short theItemNumber);
		// Given a value returned by MenuKey or MenuSelect, figure out the command number
        // that was chosen, have the application create a command object, and post it.

	virtual void SetupUndoRedoMenus();
		// Setup the Undo and Redo menu items. 
	
	virtual void SetUndoText(CommandNumber id);
		// Called to setup the Undo menu item. 

	virtual void SetRedoText(CommandNumber id);
		// Called to setup the Redo menu item. 

	virtual void SetUndoRedoText(bool cmdDone, CommandNumber id);
		// Called to setup the UndoRedo menu item. 

	virtual void SetupTheMenus();
		// Initiates the process of enabling & checking menu items.

	//------------------------------------------------------------------------------------
	// Opening and Closing Windows
	//------------------------------------------------------------------------------------
	
	virtual void CloseToolboxWindow(WindowRef aWMgrWindow);
		// Called when user closes a window either with a menu item or with a GoAway box.
		// Returns false if user cancels. 
        // (Used to signal Failure with error = noErr and message = messageCancelled if user cancels)


	//------------------------------------------------------------------------------------
	// Command Management
	//------------------------------------------------------------------------------------
	
	virtual TEvent* RetrieveAnEvent(); // override
		// Retrieves a previously posted event/ command from the queue. NULL if there are no
        // queued commands.

	void RemoveEvent(TEvent* event);
		// Removes a previously posted event/ command from the queue. There is no error
		// if the event is not in the queue. 

	virtual void ProcessEvent(TEvent* event);
		// Processes the given event from the queue of event objects. The event may be a
        // Toolbox event or a command object. This method calls the event's Process method.

	virtual void PostAnEvent(TEvent* event); // override
		// Called to post an event or command to a queue for later processing. Default is
        // to hand off to fNextHandler

	virtual void PostCommand(TCommand* command); // override
		// Called to post a command to the queue for later execution.

	//------------------------------------------------------------------------------------
	// Double/Triple Clicks
	//------------------------------------------------------------------------------------
	
	virtual short CountClicks(TToolboxEvent* event, short whereMouseDown);
		// Called by TDispatcher::HandleMouseDown. Returns the number of clicks that can
        // be considered multiple clicks (e.g. if it returns 1 the mouse down should be
        // treated as a single click, or possibly the first click of a multi-click
        // sequence. If it returns 2 the click should be considered a double-click, etc. A
        // click is considered part of a multi-click sequence if the mouse down was within
        // the proper time range of the previous mouse up, and was within the proper
        // number of pixels of the last mouse down. TDispatcher::HandleMouseDown sets
        // theClickCount of its EventInfo record to the result of this function.
        // aPDownEvent is a pointer to the mouse down event. whereMouseDown is the result
        // of FindWindow on the event.

	//------------------------------------------------------------------------------------
	// Clipboard support
	//------------------------------------------------------------------------------------
	
	virtual void DoLaunchClipboard();
		// calls gClipboardMgr->Launch

	virtual TView* MakeViewForAlienClipboard();
		// Override this to create a view to handle your application's specific data
        // types. Return NULL (the default) to have the clipboard manager create a default
        // view.


	//------------------------------------------------------------------------------------
	// Command Handlers
	//------------------------------------------------------------------------------------
	
	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Handles the application specific menu commands.

	virtual void DoSetupMenus();

	virtual void AboutToLoseControl(bool convertClipboard);
		// Called when about to activate a Desk Accessory, switch out to another
        // application, or Terminate. convertClipboard is generally true, except it will
        // be false if we get a System event that does not require Clipboard conversion

	virtual void RegainControl(bool checkClipboard);
		// Called when switching in or when leaving a desk accessory

	//------------------------------------------------------------------------------------
	// Error Alerts
	//------------------------------------------------------------------------------------

	virtual void Beep(short duration);
		// Call this method when you want to beep at the user. Sound manager users may
        // have to override this method to integrate its functionality in with your
        // application

	virtual void ShowError(OSErr error, long message);
		// Calls ErrorAlert to display an error alert.

	virtual void SpaceIsLowAlert();
		// Called when space is low, at intervals of kLowSpaceInterval ticks. Displays an
        // alerting informing the user that memory space is low. Override this if you wish
        // to take more appropriate action.

	//------------------------------------------------------------------------------------
	// AppleEvent Handling
	//------------------------------------------------------------------------------------

	virtual bool DoUndoRedo(TCommandHandler* theContext,
							   CommandNumber aCommandNumber,
							   MScriptableObject* theTarget);
	// Handles the undo choice from the menu. If you want undo/redo to be recordable
	// your DoMenuCommand method should call this on cUndo.

	virtual bool UndoRedoInContext(TCommandHandler* theContext,
									  CommandNumber aCommandNumber);
	// Handles the undo event. TDispatcher will handle this for itself, but if
	// you handle undo/redo in another context you should call this from that object's
	// DoScriptCommand method.

	virtual void DoScriptCommand(CommandNumber aCommandNumber,
								TAppleEvent* message,
								TAppleEvent* reply);
	// Handles object model AppleEvents for the application.

	virtual long CountContainedObjects(DescType desiredType);
	// Counts contained documents and window.

	virtual MScriptableObject* GetContainedObject(DescType desiredType,
											  DescType selectionForm,
											  const CAEDesc& selectionData);

	virtual MScriptableObject* GetIndContainedObject(DescType desiredType,
													 long index);
	// Returns a contained document or window.

	virtual TWindow* GetWindowByIndex(short wIndex);
	// Returns a window by front to back order.
	
	virtual TWindow* GetPaletteByIndex(short index);
	// Returns a palette by front to back order.

	virtual TDocument* GetDocumentByIndex(DescType desiredType,
										  short wIndex);
	// Returns a document based on its position in the application's fDocumentList.

	//------------------------------------------------------------------------------------
	// Scripting Utilities
	//------------------------------------------------------------------------------------


	virtual bool MakeObjectSpecifier(CAEDesc& theObjectSpecifier,
									 DescType preferredForm = formName);
	// Creates an object specifier for the application.

	virtual bool IsDocumentClass(DescType desiredType);
	// If your application supports more than one document class you should override this.
	// This method is then called in place of using (desiredType == cDocument).

	virtual void ReportReplyError(CommandNumber fromCmdNum,
								  OSErr theError,
								  const CStr255_AC& theErrStr);
	// When a reply returned through a TSendAECommand contains an error it is sent
	// through this method for reporting.

	//------------------------------------------------------------------------------------
	// Dispatcher Properties
	//------------------------------------------------------------------------------------

	virtual void SetObjectProperty(const CAEDesc& thePropertyValue,
								   DescType whichProperty);
	// Sets application properties.

	virtual bool GetObjectProperty(CAEDesc& thePropertyValue,
									  DescType whichProperty,
									  const CAEDesc& desiredType);
	// Gets application properties.

	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual void DoMakeDependencySpace();
		// Creates a dependency space for storing dependencies, and stores it in fgDefaultDependencySpace.

	virtual void DoMakeMenuBarManager();
		// Creates an instance of TMenuBarManager
		
	virtual void DoMakeViewServer();
		// instantiates the global view server

	virtual void InvalidateFocus();
		// Called to indicate that a view is unfocused when you don't know which view was focused.

	virtual bool IsHelpEnabled();
		// Returns true if the Help Manager is present and the user has enabled help

	virtual OSErr InteractWithUser();
		// This is the same as calling MAInteractWithUser except we _know_
		// we have a dispatcher by now.
		
	virtual OSErr InteractWithUser(long timeOutInTicks,
								   NMRecPtr nmReqPtr, 
								   AEIdleUPP idleProc);
		// Wrapper for AEInteractWithUser. Virtual bottleneck for the Notification Manager.
		// Don't use this directly -- call MAInteractWithUser instead. 
		
	virtual bool ActivateBusyCursor(bool entering);
		// Called when the busy cursor mechanism should be activated or deactivated.

	virtual bool KeepCursorBusy(bool keepItSpinning);
		// set this if you want to call any of the patched traps without resetting the
		// cursor

	virtual bool AlertFilter(DialogRef theDialog, EventRecord& theEvent,
									   short& itemHit);
									   
	virtual bool AppleEventIdleProc(EventRecord& theEventRecord,
											  long& sleepTime,
								 		      RgnHandle& mouseRgn);
		// Called by the default AppleEvent idle proc MacAppAppleEventIdleProc.
		// Provides default idle behavior by handling update and activate events.
		
	virtual bool StandardFileFilter(DialogRef theDialog,
						EventRecord& theEvent, short& itemHit, void* yourDataPtr);
		// Called by the default Standard File filter proc MacAppStandardFileFilterProc.

		
	virtual void GetApplicationName(CStr255_AC& theName);
		// The default behavior is to return the name supplied by GetAppParms

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline CList_AC* GetDocumentList() const { return const_cast<CList_AC*>(&fDocumentList); }
	inline bool GetSysWindowActive() const { return fSysWindowActive; }
	inline void SetSysWindowActive(bool inSysWindowActive) { fSysWindowActive = inSysWindowActive; }
	inline bool GetAllowApplicationToSleep() const { return fAllowApplicationToSleep; }
	inline bool GetDone() const { return fDone; }
#if qDebug
	inline TEventList* GetEventList() const { return const_cast<TEventList*>(&fEventList); }
#endif
	inline OSType GetCreator() const { return fCreator; }
	inline short GetClickCount() const { return fClickCount; }
	inline short GetEventLevel() const { return fEventLevel; }

	inline void SetDone(bool inDone) { fDone = inDone; }
	inline void SetLastClickPart(short inLastClickPart) { fLastClickPart = inLastClickPart; }
	inline void SetEventLevel(short inEventLevel) { fEventLevel = inEventLevel; }

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	OSType fCreator;							// The application signature; Creator type
												// for files.  This is used by DoMakeFile.
												
	OSType fMainFileType;						// principal file type opened/printed by
                                                // application; by default,
                                                // TDispatcher::SFGetFilters returns a
                                                // list of just this. Used by DoMakeFile

	CPoint_AC fLastMousePoint;						// coordinates of mouse in last event
                                                // passed to TDispatcher.CountClicks
												
	TEventList fEventList;						// Events/Commands that were posted for execution
	CList_AC fDocumentList;						// list of documents
	CList_AC fFreeWindowList;						// list of free-standing (documentless) windows
	
	TEventHandler* fHeadCohandler;				// head of linked list of global
                                                // co-handlers

	CTempRegion_AC fSleepRegion;				// the region that will be passed to
                                                // WaitNextEvent as the sleep region.
                                                // Computed from the cursor and help
                                                // regions
												
	CTempRegion_AC fCursorRegion;				// the current cursor region

	CTempRegion_AC fHelpRegion;					// the current help region

	CommandNumber fUndoCommand;					// the command number currently in the undo menu
	
	CommandNumber fRedoCommand;					// the command number currently in the redo menu
	
	CommandNumber fUndoRedoCommand;				// the command number currently in the undo/redo menu
	
	bool fUndoRedoState;						// the state of undo/redo at the last undo/redo command

	unsigned long fLastUpTime;				// time of last mouse up event

	unsigned long fNextSpaceMessage;			// time when next low space message should
                                                // be displayed
												
	unsigned long fLowSpaceInterval;			// If >= 0, the frequency (in Ticks) with
                                                // which MacApp displays a low space
                                                // alert. (Defaults to kLowSpaceInterval.)
                                                // If < 0, MacApp doesn't display an
                                                // alert.
												
	IdlePhase fIdlePhase;						// The current idle phase

	ResNumber fDisplayedMenus;					// menus that are read in and installed in
                                                // menu bar
												
	ResNumber fMBarHierarchical;				// menus that pop up when a menu item is
                                                // choosen
	
	ResNumber fOtherMenus;						// menus that are read in but not
                                                // installed
	
	short fClickCount;							// number of 'saved up' mouse clicks. If a
                                                // mouseDown, the value is that returned
                                                // by the call to
                                                // TDispatcher::CountClicks. If a mouseUp,
                                                // the value is left alone. Otherwise, set
                                                // to 0. Also set to 0 is the 2 clicks
                                                // were in different parts ofthe window.
												
	short fLastClickPart;						// the list window part clicked in-used
                                                // for double click detection
												
	short fMainEventMask;						// Event mask used in main event loop.
                                                // Initialized by InitUMacApp.
												
	short fEventLevel;							// A count of the number of nested calls
                                                // to PollToolboxEvent.
												
	bool fAllowApplicationToSleep;				// Whether the application should be
                                                // allowed to "sleep" and thus share
                                                // processor time, when getting events

	bool fAlwaysTrackCursor;					// set this to true when you want the
                                                // application to track the cursor even if
                                                // it doesn't stray outside gCursorRgn.
												
	bool fDone;									// set this to true when you want the
                                                // application to terminate

	bool fSysWindowActive;						// True if the front window is a system
                                                // window
	
	bool fWantKeyUpEvents;						// set this to true in your constructor to set
												// the system mask to allow key-up events

	bool fAutoKeyCommandKeys;					// set this to true in your constructor to
												// Allow AutoKey Events for Command keys

};

//----------------------------------------------------------------------------------------
// CDocumentIterator
//----------------------------------------------------------------------------------------

class EXPORT_MA CDocumentIterator : public TemplateIterator_AC<TDocument*>
{
public:
	CDocumentIterator(const TDispatcher* itsDispatcher,
					  ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					  bool itsForward) :
	TemplateIterator_AC<TDocument*>(itsDispatcher ? itsDispatcher->GetDocumentList() : NULL, itsLowBound, itsHighBound, itsForward)
	{ }

	CDocumentIterator(const TDispatcher* itsDispatcher, bool itsForward = kIterateForward_AC) :
	TemplateIterator_AC<TDocument*>(itsDispatcher ? itsDispatcher->GetDocumentList() : NULL, itsForward)
	{ }
};

//----------------------------------------------------------------------------------------
// CNoGhostDocsIterator
//----------------------------------------------------------------------------------------

class EXPORT_MA CNoGhostDocsIterator : public CDocumentIterator
{
public:
	CNoGhostDocsIterator(const TDispatcher* itsApplication,
					  ArrayIndex_AC itsLowBound, ArrayIndex_AC itsHighBound,
					  bool itsForward);

	CNoGhostDocsIterator(const TDispatcher* itsApplication, bool itsForward = kIterateForward_AC);

	//
	// Required functions 
	//
	TDocument* First();
	inline TDocument* Current() const { return reinterpret_cast<TDocument*>(CCommonVoidIterator_AC::Current()); }
	TDocument* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TDocument* operator++() { return Next(); }
	inline TDocument* operator++(int) { TDocument* returnVal = Current(); Next(); return returnVal; }
	inline operator TDocument*() const { return Current(); }
	inline TDocument* operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline TDocument* FirstDocument() { return First(); }
	inline TDocument* CurrentDocument() const { return Current(); }
	inline TDocument* NextDocument() { return Next(); }

protected:

	bool IsGhostDoc();
};

//----------------------------------------------------------------------------------------
// TEventRetrieverCommand: Retrieve Events and process them.
//----------------------------------------------------------------------------------------

class EXPORT_MA TEventRetrieverCommand : public TCommand
{
public:
	TEventRetrieverCommand(CommandNumber itsCommandNumber = cNoCommand);
		// constructor

	virtual ~TEventRetrieverCommand();
		// Destructor
		
	virtual void DoIt();
		// Retrieve and process an event
};


//----------------------------------------------------------------------------------------
// Globals defined by this unit
//----------------------------------------------------------------------------------------

extern TDispatcher* gDispatcher;			// the application object

extern bool gInitialized;					// Set to true at top of TDispatcher::Run()

extern CTempDesc gServerAddress;			// Address of the AppleEvent Server. Initialized
                                        	// to send to ourselves.

#endif // __UDISPATCHER__
