// UEvent.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UEVENT__
#define __UEVENT__

// ACS

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

// MacApp

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif

// Toolbox

#ifndef __EVENTS__
#include <Events.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

enum
{
	kPriorityLowest = 127,						// Low priority commands are considered last
	kPriorityLow = kPriorityLowest - 32,		// Low priority commands are considered last
	kPriorityNormal = 64,						// Normal priority: command default priority
	kPriorityHigh = kPriorityNormal - 32,		// High priority commands take precedence
	kPriorityHighest = 0						// High priority commands take precedence
};

//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef char EventPriority;
	//  Event priorities


//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TEventHandler;


//----------------------------------------------------------------------------------------
// TEvent
//----------------------------------------------------------------------------------------

class EXPORT_MA TEvent
{
public:
	TEvent(EventNumber itsEventNumber = cNoEvent,
				TEventHandler* itsSource = NULL,
				TEventHandler* itsHandler = NULL);
		// Constructor
		
	virtual ~TEvent();
		// Write out a message if we are attempting to free an event that is still on the
		// event queue.

	virtual bool ShouldFreeOnCompletion();
	
	virtual bool IsReadyToPost();
		// True if the event can be posted to the event queue.  Default returns true.

	virtual bool IsReadyToExecute();

	virtual bool IsRecurring();


	virtual TEventHandler* GetHandler();

	virtual void Process();

#if qDebug
	virtual void ReportEvent();
#endif

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline EventNumber GetIdentifier() const { return fIdentifier; }
	inline bool GetAffectsMenus() const { return fAffectsMenus; }
	inline EventPriority GetPriority() const { return fPriority; }

	inline void SetIdentifier(EventNumber inIdentifier) { fIdentifier = inIdentifier; }
	inline void SetAffectsMenus(bool inAffectsMenus) { fAffectsMenus = inAffectsMenus; }
	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	TEventHandler* fSource;
	TEventHandler* fHandler;					// The handler for this event
	EventNumber fIdentifier;					// Can be a many-to-one mapping of fIdentifier to TEvent
	EventPriority fPriority;					// How this event should be prioritized if
                                                // it has do compete on a prioritized
                                                // basis for execution. kNormalPriority is
                                                // the Default. Most programs won't have
                                                // to mess with this.
	bool fAffectsMenus;							// True means the menus must be setup after the event
};


//----------------------------------------------------------------------------------------
// TToolboxEvent
//----------------------------------------------------------------------------------------

class EXPORT_MA TToolboxEvent : public TEvent
{
public:
	TToolboxEvent(TEventHandler* itsHandler = NULL, const EventRecord* theEvent = NULL);		
		// Constructor

	virtual ~TToolboxEvent();
		// Destructor
		
	bool IsDoubleEvent();
		// returns TRUE if the event was the first of a two event sequence

	void AddSecondaryKeyEvent(const EventRecord& theEvent);
		// If IsDoubleEvent returns true then call this method with the next event retrieved
		// from the Event Manager. It will be added to the TToolBoxEvent.

	bool IsButtonPressed() const;
		// Was the button pressed?
		
	bool IsCommandKeyPressed() const;
		// Was Command key depressed?
		
	bool IsShiftKeyPressed() const;
		// Was Shift key depressed?
		
	bool IsAlphaLock() const;
		// Was Alpha Lock depressed?
		
	bool IsOptionKeyPressed() const;
		// Was Option key depressed?
		
	bool IsControlKeyPressed() const;
		// Was Control key depressed?
		
	bool IsAutoKeyEvent() const;
		// True if this was an auto key event

	virtual void Process();		//override
		// Call gApplication->DoToolBoxEvent directly to dispatch the event

#if qDebug
	virtual void ReportEvent();
#endif

	short GetWhat() const;
		// returns the event code

	long GetWhen() const;
		// returns the event time (in ticks)

	CPoint_AC GetWhere() const;
		// returns the event location

	long GetMessage(short zeroOrOne = 0) const;
		// returns the event message

	short GetModifiers(short zeroOrOne = 0) const;
		// returns the event modifiers

	//------------------------------------------------------------------------------------
	// Additional simple accessors
	//------------------------------------------------------------------------------------
	inline short GetClickCount() const { return fClickCount; }
	inline short GetKeyCode() const { return fKeyCode; }
	inline ScriptCode GetKeyScript() const { return fKeyScript; }
	inline const CStr2_AC& GetText() const { return fText; }
	inline const EventRecord& GetEventRecord(short zeroOrOne = 0) const { return fEventRecords[zeroOrOne]; }

	inline void SetClickCount(short inClickCount) { fClickCount = inClickCount; }
	inline void SetText(const CStr2_AC& inText) { fText = inText; }
	inline void SetKeyCode(short inKeyCode) { fKeyCode = inKeyCode; }

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif
	EventRecord fEventRecords[2];				// IF a key event, The primary original
												// packed ToolBox event record (and secondary, when this is
												// a multibyte character)

	CStr2_AC fText;								// IF a key event, the character that the event
                                                // translates to. length of 1 for single byte
												// characters and length of 2 for 16 bit characters


	ScriptCode fKeyScript;						// The KeyScript at the time the event was retrieved

	short fKeyCode;								// IF a key event the virtual key code
                                                // that the event translates to

	short fClickCount;							// 0 = event was not a mouse down; 1-N = #
                                                // of multiple clicks
	bool fDoubleEvent;						// true if there is a second event stored herein (fEventRecords[1])

#if qBackwardCompatible
	const EventRecord& fEventRecord;			// Reference to the first record, left in for compatibility
	const unsigned char& fCharacter;			// Reference to the first character, left in for compatibility
#endif
};


#endif


