// UEventHandler.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UEVENTHANDLER__
#define __UEVENTHANDLER__

// ACS

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

// Toolbox

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------
class TBehavior;
class TCommand;
class TEvent;
class TToolboxEvent;
class TWindow;

//----------------------------------------------------------------------------------------
// TEventHandler: This type represents abstract objects that handle certain kinds of events:
// Key events: both key down and auto key events, menu events: both enabling menus and
// menu items and processing menu commands, Read to/ write from Disk, Idle events: when
// there are no other events to handle, Actual events: EvtHandlers - through their method
// 'DoHandleEvent' may intercept and handle actual ToolBox events Termination: when the
// application quits TEventHandler are linked into a list with the most specific object
// (usually a selection) at the head of the list. 'TApplication->fTarget' contains the head
// of the list. (When a window is deactivated, this global variable is cached in the
// window object, and retrieved when it is later activated.) For these kinds of events,
// the TApplication->fTarget gets the first crack at handling the event. The default
// implementation of the methods of TEventHandler is to pass the event to the next element
// of the list.
//----------------------------------------------------------------------------------------
class EXPORT_MA TEventHandler : public MDependable_AC, public MStreamable
{
  public:
	//------------------------------------------------------------------------------------
	// Construction and destruction.
	//------------------------------------------------------------------------------------
	TEventHandler(TEventHandler* itsNextHandler = NULL);
		// Constructor
				
	TEventHandler(const TEventHandler& inEventHandler);
		// Copy Constructor

	virtual ~TEventHandler();
		// Before handing to inherited destructor, changes the target to the application if the
        // next handler is NULL or the next handler if there is one.

	virtual void* Clone() const;
		// Clone

	//------------------------------------------------------------------------------------
	// Termination
	//------------------------------------------------------------------------------------
	virtual void Terminate();
		// Does nothing in this class.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFrom(CStream_AC* aStream);
	
	virtual void WriteTo(CStream_AC* aStream) const;

	//------------------------------------------------------------------------------------
	// Chainwalking
	//------------------------------------------------------------------------------------
	inline TEventHandler* GetNextHandler() const { return fNextHandler; }
		// Return the next handler in the chain

	inline void SetNextHandler(TEventHandler* inEventHandler) { fNextHandler = inEventHandler; }

	TEventHandler* AddHandler(TEventHandler* headOfChain);
		// Adds this from the chain of handlers starting at headOfChain. Returns the new
        // head of chain.

	TEventHandler* RemoveHandler(TEventHandler* headOfChain);
		// Removes this from the chain of handlers starting at headOfChain. Returns the
        // new head of chain (if this was the head then there _must_ be a new head.

	//------------------------------------------------------------------------------------
	// Dispatching
	//------------------------------------------------------------------------------------
	void HandleKeyCommand(TToolboxEvent* event);
		// Handles the dispatching of DoKeyEvent messages between itself and Behaviors
	
	void HandleKeyUp(TToolboxEvent* event);
		// Handles the dispatching of DoKeyUp messages between itself and Behaviors
	
	void HandleCommandKey(TToolboxEvent* event);
		// Handles the dispatching of DoCommandKeyEvent messages between itself and Behaviors
	
	void HandleMenuCommand(CommandNumber aCommandNumber);
		// Handles the dispatching of DoMenuCommand messages
	
	void HandleSetupMenus();
		// Handles the dispatching of DoSetupMenus messages
	
	void HandleIdle(IdlePhase phase);
		// Calls DoIdle if the handler is eligible.

	void HandleEvent(EventNumber eventNumber,
							 TEventHandler* source,
							 TEvent* event);
		// Responsible for dispatching DoHandleEvent messages
	
	//------------------------------------------------------------------------------------
	// Events
	//------------------------------------------------------------------------------------
	unsigned long NextIdle();
		// Returns the number Tickcount for when the handler should next be idled.

	virtual bool DoIdle(IdlePhase phase);
		// Returns true if it frees itself so caller can know

	virtual bool DoCoHandlerEvent(TEvent* event);
		// Handle the event and return true if I want it, else I don't handle it, and
        // return false

	virtual void DoEvent(EventNumber eventNumber, TEventHandler* source, TEvent* event);
		// By default, asks the next handler to handle the event

	virtual void SetEnable(bool state);
		// Enables based on 'state'

	inline bool IsEnabled() const { return fEnabled; }
		// Returns the enabled state.

	//------------------------------------------------------------------------------------
	// Double/ Triple Clicks
	//------------------------------------------------------------------------------------
	virtual bool DoMultiClick(CPoint_AC lastDownPt, CPoint_AC newDownPt);
		// Called by TApplication::CountClicks. Should return true if the 2 points are
        // close enough to be considered part of a double/ triple click. (Both points are
        // in global coordinates.) This is only called if the mouse down was within the
        // proper time range of the previous mouse up. Default is to pass message to the
        // nextEventHandler (if one exists), otherise to require that the sum of the x & y
        // distances is <= gStdHysteresis.

	//------------------------------------------------------------------------------------
	// Key Events
	//------------------------------------------------------------------------------------
	virtual void KeyEventToComponents(TToolboxEvent* event);
		// Intended to extract character components of an event in a script manager
        // compatible way.Default just forwards to next handler

	virtual void DoKeyEvent(TToolboxEvent* event);
		// Handles all keystrokes except those with the command key held down.

	virtual void DoKeyUp(TToolboxEvent* event);
		// Handles all key upstrokes.

	virtual void DoCommandKeyEvent(TToolboxEvent* event);
		// Handles command-key combinations only.

	//------------------------------------------------------------------------------------
	// MenuEvents
	//------------------------------------------------------------------------------------
	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// If the handler can perform the command it does so by either performing the
        // command directly or by posting a TCommand.

	virtual void DoSetupMenus();
		// Handles the setup of menus.

	//------------------------------------------------------------------------------------
	// Help
	//------------------------------------------------------------------------------------
	virtual void DoHelp(TToolboxEvent* event, long& message);
		// If fNextHandler is not NULL calls its DoHelp method, otherwise returns NULL.

	//------------------------------------------------------------------------------------
	// Target management
	//------------------------------------------------------------------------------------
	inline bool IsTarget() { return this == fgTarget; }
		// returns true if this is the target

	virtual TWindow *GetWindow() const;
		// returns the TWindow of this TEventHandler (if any). Returns NULL if this TEventHandler isn't a TView
		// or isn't installed in a TWindow.
		
	virtual bool WantsToBeTarget();
		// Called to find out if this object wishes to become the target. Default returns
        // false

	virtual long WillingToResignTarget();
		// Called to find out if this object is willing to cease being the target. Default
        // returns true

	virtual void TargetValidationSucceeded();
	
	virtual void TargetValidationFailed(long reason);

	virtual void BecameWindowTarget();
		// Called when this object becomes the current target of a window

	virtual void ResignedWindowTarget();
		// Called when this object ceases to be the current target of a window

	virtual void BecameTarget();
		// Called when this object becomes the current target

	virtual void ResignedTarget();
		// Called when this object ceases to be the current target

	virtual bool BecomeTarget();
		// Called to make this object be the current target

	virtual bool ResignTarget();
		// Called to inform this object to cease being the current target

	virtual void SetTargetSelection(bool redraw);
		// Called when this object becomes the target, except as a result of a mouse click
		// Overridden by EditTexts to select all characters

	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------
	virtual void DoKeySelection(const CStr255_AC& selectionString);
		// Derived classes override this method to set the selection based on the value of
        // 'selectionString'. This method is called by a TKeySelectionBehavior which can
        // be added to a TEventHandler and any of its derived classes.
		
	inline IDType GetIdentifier() const { return fIdentifier; }
		// Return the fIdentifier of the object.
		
	inline void SetIdentifier(IDType inIdentifier) { fIdentifier = inIdentifier; }
		// Return the fIdentifier of the object.
		
	inline unsigned long GetIdleFreq() const { return fIdleFreq; }
		// Return the current value of fIdleFreq.
		
	virtual void SetIdleFreq(unsigned long newIdleFreq);
		// Call to set the handler's idling frequency.

	void RemoveBehavior (TBehavior* theBehavior);
		// Remove theBehavior from the behavior chain
	
	void AddBehavior (TBehavior* theBehavior);
		// Call to add theBehavior to this object
	
	inline TBehavior* GetFirstBehavior() const { return fBehavior; }
		// Call to retrieve the first behavior currently attached to EvtHandler

	TBehavior* GetFirstEnabledBehavior () const;
		// Call to retrieve the first enabled behavior currently attached to EvtHandler

	virtual TBehavior* GetBehaviorWithIdentifier(IDType itsIdentifier) const;
		// Returns the first behavior with the specified ID, or NULL
		
	virtual void SelectOwner (bool select);
		// "Selects" this object by calling the SelectOwner method of each behavior
		
	virtual bool IsSelected();
		// Returns the value of the first behavior's IsOwnerSelected(), or false
		// if there are no behaviors

	//------------------------------------------------------------------------------------
	// Command Management
	//------------------------------------------------------------------------------------
	virtual TEvent* RetrieveAnEvent();
		// Retrieves a previously posted event/ command from the queue. NULL if there are no
        // queued commands. Default is to hand off to fNextHandler

	virtual void PostAnEvent(TEvent* event);
		// Called to post an event or command to a queue for later processing. Default is
        // to hand off to fNextHandler

	virtual void PostCommand(TCommand* command);
		// Called to post a command to a queue for later execution. Left in for
        // compatibility - calls PostAnEvent

	//----------------------------------------------------------------------------------------
	// static functions
	//----------------------------------------------------------------------------------------
	static inline CPoint_AC GetStdHysteresis() { return fgStdHysteresis; }
	static inline TEventHandler* GetTarget() { return fgTarget; }
		// Returns the current target

	static void SetTarget(TEventHandler* newTarget);
		// Sets the current target (the head of the target chain).

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
#if qBackwardCompatible
public:
#else
protected:
#endif

	TEventHandler* fNextHandler;				// the next element of the list, or NULL
	
	CAutoPtr_AC<TBehavior> fBehavior;			// Behavioral modifier object
	
	IDType fIdentifier;							// the ID if any
												
	unsigned long fIdleFreq;					// the minimum number of ticks that can
                                                // pass before my DoIdle is called. 0 =
                                                // call as often as possible.
                                                // n..kMaxIdleTime = call every fIdleFreq
                                                // ticks.
												
	unsigned long fLastIdle;					// the tick count the last time my DoIdle
                                                // was called.

	bool fEnabled;								// is this eventHandler enabled. (does it
                                                // take clicks/keystrokes and other
                                                // events? 

//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
protected:
	static TEventHandler* fgTarget;				// the TEventHandler that gets the first
                                                // chance at DoCommand, DoSetupMenu,
                                                // DoKeyEvent, Idle; should never be NULL
                                                // -- If you do not want your own target
                                                // set this to the application object
												
	static CPoint_AC fgStdHysteresis;			// standard hysteresis used in TCommand::TCommand
};


//----------------------------------------------------------------------------------------
// CHandlerIterator: A simple iterator for the handler chain.
//----------------------------------------------------------------------------------------

class EXPORT_MA CHandlerIterator : public CIterator_AC
{
public:
	CHandlerIterator(const TEventHandler* chainHead);
	
	//
	// Required functions 
	//
	TEventHandler* First();
	inline TEventHandler* Current() const { return fCurrent; }
	TEventHandler* Next();

	//
	// Convenience functions for C++meisters
	//
	inline TEventHandler* operator++() { return Next(); }
	inline TEventHandler* operator++(int) { TEventHandler* returnVal = Current(); Next(); return returnVal; }
	inline operator TEventHandler*() const { return Current(); }
	inline TEventHandler* operator->() const { return Current(); }

	//
	// Compatibility functions
	//
	inline TEventHandler* FirstEventHandler() { return First(); }
	inline TEventHandler* CurrentEventHandler() const { return Current(); }
	inline TEventHandler* NextEventHandler() { return Next(); }

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
private:
	TEventHandler*	fCurrent;
	const TEventHandler* fFirstHandler;
	TEventHandler*	fNextHandler;
};

#endif
