// UUndo.h
// Copyright  1995-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UUNDO__
#define __UUNDO__

// ACS

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __CPascalString_AC__
#include "CPascalString_AC.h"
#endif
#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef qUseODUndoStyle
#define qUseODUndoStyle 0
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TAppleEvent;

//------------------------------------------------------------------------------------
// Constants
//------------------------------------------------------------------------------------

enum EActionType {kSingleAction, kBeginAction, kEndAction};

enum ERespectMarksChoices {kDontRespectMarks, kRespectMarks};

enum EActionState {kInSingleAction, kInBeginAction, kInCollectActions};

const kDefaultUndoStackDepth = 25;

//----------------------------------------------------------------------------------------
// CLASS CUndoAction
//----------------------------------------------------------------------------------------

class EXPORT_MA CUndoAction
{
  public:
	//------------------------------------------------------------------------------------
	CUndoAction(TCommand* command = NULL, EActionType actionType = kSingleAction, CommandNumber id = cNoCommand);
	// Constructor

	CUndoAction(const CUndoAction& rhs);
	// Copy Constructor
				
	inline ~CUndoAction() { }
	// destructor
	
	void DeleteCommand();
	
	CUndoAction& operator=(const CUndoAction& rhs);
	// assignment operator
				
	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------
	bool IsDone();
	

	inline TCommand* GetCommand() const { return fCommand; }
	inline CommandNumber GetCommandNumber() const { return fID; }
	inline EActionType GetActionType() const { return fActionType; }
	inline bool GetMark() const { return fMark; }
	
	inline void SetCommand(TCommand* inCommand) { fCommand = inCommand; }
	inline void SetCommandNumber(CommandNumber inID) { fID = inID; }
	inline void SetActionType(EActionType inActionType) { fActionType = inActionType; }
	inline void SetMark(bool inMark) { fMark = inMark; }
	
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	TCommand*		fCommand;
	CommandNumber	fID;
	EActionType		fActionType;
	bool			fMark;
};

//----------------------------------------------------------------------------------------
// CLASS TActionStack
//----------------------------------------------------------------------------------------

class EXPORT_MA TActionStack : public CDynamicArray_AC
{
  public:
	TActionStack();
				
	virtual ~TActionStack();
	// destructor
	
	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------
	CUndoAction* Top();
	
	void PushAction(const CUndoAction& action);

	void Delete(CUndoAction* action);
	// finds and deletes the action

	void ClearActions(bool toMark = false, TCommandHandler* filterby = NULL, bool itsForward = kIterateForward_AC);
		// Delete and free actions.
		// Can run until a marked one is found, or the stack is empty.
		// Can delete only actions related to a supplied commandhandler 
	
	CUndoAction* FindTransaction(TCommand* command, bool itsForward);
		// Find the action or transaction containing a command. 
		// Use kIterateForward_AC for the undo stack, kIterateBackward_AC for redo. 

	Size PurgeFirstPhase(Size needed) { return Purge(needed, 1); }
		// Unload as much of the stack as needed up to, but not including,
		// the topmost entry in the stack.

	Size Purge(Size needed, ArrayIndex_AC numToLeave = 0);
		// Unload up to top.
};

//----------------------------------------------------------------------------------------
// CLASS TUndoHandler
//----------------------------------------------------------------------------------------

class EXPORT_MA TUndoHandler
{
  public:
	//------------------------------------------------------------------------------------
	// Initializer, I<Method> and Free.
	//------------------------------------------------------------------------------------
	TUndoHandler();
				
	virtual ~TUndoHandler();
	
	//------------------------------------------------------------------------------------
	// Action History Manipulation
	//------------------------------------------------------------------------------------
	void AddActionToHistory(TCommand* command, CommandNumber id);
		// Add an action to the list of commands that could be undone. 
		// Also, clears the redo history. 
	
	void BeginAction(CommandNumber inCommand = cNoCommand);
		// Begin grouping actions together 
	
	void EndAction();
		// End grouping actions together. 
	
	void AbortCurrentTransaction();
		// Remove the current transaction (and any nested transactions) from
		// the action history. Must be called before the End action is added. 
	
	void ClearActionHistory(ERespectMarksChoices respectMarks, TCommandHandler* filterby = NULL);
		// Clear the undo and redo stacks. 
	
	void MarkActionHistory();
		// Marks the top of the undo and redo stacks so they can be cleared to the mark
		// later. Usually used for modal dialogs so that commands can be cleared to the
		// mark when the dialog is dismissed (even if the dialog is not freed).
	
	CUndoAction* FindTransaction(TCommand* command);
		// Find the action or transaction containing a particular command. 
	
	void Abort();
		// Abort the top (trans)action in the undo stack. 
	
	Size Purge(Size needed);
		// Unload as much of the undo and redo stack as needed.

	CUndoAction* GetTopAction(bool isUndo);
		// Returns the first action that matches

	inline ArrayIndex_AC GetMaxDesiredItems() { return fMaxDesiredItems; }
		// returns the max number of undoable items to track

	ArrayIndex_AC SetMaxDesiredItems(ArrayIndex_AC newDepth);
		// set the max number of undoable items to track.
		// returns the previous setting.

	void RemoveCommandAction(TCommand* theCommand);
	// the command went away so silently slink up and remove the top action

	//------------------------------------------------------------------------------------
	// Undo
	//------------------------------------------------------------------------------------
	void Undo();
		// Undo the top (trans)action in the undo stack. 
	
	CommandNumber GetUndoID();
		// Get the ID for the undo menu item for a command. 
		// Returns cNoCommand if no matching command is found
	
	//------------------------------------------------------------------------------------
	// Redo
	//------------------------------------------------------------------------------------
	void Redo();
		// Redo the top (trans)action in the redo stack. 
	
	CommandNumber GetRedoID();
		// Get the ID for the redo menu item for a command. 
		// Returns cNoCommand if no matching command is found
	
	void ClearRedoHistory();
		// Clear the redo stack. 
	
	//------------------------------------------------------------------------------------
	// For internal use only
	//------------------------------------------------------------------------------------
  protected:
	EActionType AbortAction();
		// Abort the top action in the undo stack. 
	
	void AbortCommand(TCommand* command);
		// Abort an individual command. 
	
	EActionType UndoAction();
		// Undo the top action in the undo stack. 
	
	void UndoCommand(TCommand* command);
		// Undo a command. 
	
	EActionType RedoAction();
		// Redo the top action in the undo stack. 
	
	void RedoCommand(TCommand* command);
		// Redo a command. 

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
protected:
	TActionStack fUndoStack;					// Stack of commands that could be undone. 
	TActionStack fRedoStack;					// Stack of commands that could be redone. 

	long				fInTransaction;			// Keep track of transaction level in case
												// we need to abort. 
	EActionState		fActionState;			// track begin/endaction calls
	CommandNumber		fActionCommand;			// if !cNoCommand: identifies a commandNumber that
												// should be stamped on the ID of all commands
												// grouped in a single action.

	// We need to better tune the following number.  This number indicates what we would like
	// to be the maximum number of stacked undoable actions at any one time.  If your commands
	// retain large amounts of data for performing undo/redo then you may want to reduce this
	// limit.  If they are smaller then perhaps the number could be increased.  We should also
	// consider having a maximum memory usage setpoint as well but the overhead of keeping
	// a running total is too costly right now.

	ArrayIndex_AC			fMaxDesiredItems;	// Maximum number of entried in stack.

	//------------------------------------------------------------------------------------
	// static data members
	//------------------------------------------------------------------------------------
public:
	static TUndoHandler	fgUndoHandler;		// The undo handler object. 
};

//----------------------------------------------------------------------------------------
// CLASS TUndoRedoCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TUndoRedoCommand : public TCommand
{
  public:
	
	TUndoRedoCommand(CommandNumber itsCommandNumber = cNoCommand);
		// Constructor

	virtual ~TUndoRedoCommand();
		// Destructor
			
	virtual TAppleEvent* MakeAppleEvent();
		// Create a Undo or Redo Apple Event. 

	virtual void DoIt();
		// Calls TUndoHandler::fgUndoHandler->Undo/Redo as necessary
			
	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
  protected:	
	AEEventID	fAEEventID;
};

//----------------------------------------------------------------------------------------
// CLASS CUndoStackHook
//----------------------------------------------------------------------------------------

// A class for unloading portions of the undo stack when memory is needed.

class EXPORT_MA CUndoStackHook : public CGrowZoneHook
{
  public:
	CUndoStackHook() { }
	virtual ~CUndoStackHook() { }

	virtual Size TotalSize(bool justLocked);
		// Returns the total number of bytes managed (or only locked
		// handles if justLocked is true). 

	virtual Size Purge(Size needed);
		// Make memory available by unloading the undo stack. Returns 
		// the actual amount of memory purged. 
};

//========================================================================================
// GLOBAL Procedures
//========================================================================================

EXPORT_MA CStr255_AC GetUndoRedoText(bool cmdDone, CommandNumber aCommandNumber);
	// Get the text for the undo or redo menu item for a command. 
	

#endif
