//----------------------------------------------------------------------------------------
// UViewDragger.h
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __UVIEWDRAGGER__
#define __UVIEWDRAGGER__

// MacApp

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

//----------------------------------------------------------------------------------------
// TViewDragger
//----------------------------------------------------------------------------------------
//
//	This behavior allows the view to which it is attached to be moved by dragging within
//	constraints inside its parent view.  The constraint is specified as a rectangle giving
//	the limits for motion of the origin of the view, along with an enum describing the
//	allowed motion.  All coordinates are in the coordinate system of the parent of the
//	view to be moved.  The bounding rectangle can also be made dependant on changes in
//	other views, in a manner similar to TSizer behaviors.  Up to four dependencies may be
//	specified.
//
//	This behavior is intended to be attached to a view to be used as a resizing control.
//	The size of other views can be made dependent on the position of this behavior's view
//	through the use of Sizer behaviors.
//

#define kNumTargets 4

class EXPORT_MA TViewDragger : public TBehavior, public MDependable_AC
{
	MA_DECLARE_CLASS;
	
public:
	
	TViewDragger(IDType itsIdentifier = kNoIdentifier);
		// Constructor

	TViewDragger(const TViewDragger& inViewDragger);
		// Copy Constructor
		
	virtual ~TViewDragger();

	virtual	void		InitializeViewDragger();
			void		DoUpdate(	ChangeID_AC			theChange,
									MDependable_AC*		changedObject,
									void*				changeData,
									CDependencySpace_AC*	dependencySpace);
	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------
	virtual void ReadFrom(CStream_AC* aStream);

	virtual void WriteTo(CStream_AC* aStream) const;


	//------------------------------------------------------------------------------------
	// Event Filtering Methods
	//------------------------------------------------------------------------------------
	virtual bool DoMouseCommand(CViewPoint& theMouse,
										  TToolboxEvent* event,
										  CPoint_AC hysteresis);
		// Override which creates a Tracker to drag the view holding this behavior.
		
	virtual bool DoSetCursor(const CViewPoint& localPoint, RgnHandle cursorRegion);
		// Override to set the cursor to show that the view for this behavior is
		// locally dragable 

	// An enum which specifies the type of dragging supported.
	enum DragMode
	{
		eHorizontal				= 0,
		eVertical				= 1,
		eHorizontalAndVertical	= 2
	};
	// An enum which specifies where to look for the target views.
	enum AssociationMode
	{
		eChildAssociation		= 0,
		ePeerAssociation		= 1,
		eParentAssociation		= 2
	};
	// An enum which specifies how to respond to any change we see.
	enum ChangeTypeMode
	{
		eNoMode								= 0,
		eTopMode							= 1,
		eLeftMode							= 2,
		eHeightMode							= 3,
		eWidthMode							= 4
	};
	
	// An enum which specifies how to react in relation to the target view's frame change.
	enum AdjustMode
	{
		eDontAdjust					= 0,
		eAdjPosByTopOrLeft			= 1,
		eAdjNegByTopOrLeft			= -1,
		eAdjPosByBotOrRight			= 2,
		eAdjNegByBotOrRight			= -2,
		
		// Adjust proportional to the height/width.  Top/Left can be proportional to 
		// any parent's width and Height/Width can be proportional to any other width.
		eAdjProportional				= 3
	};
	
	// Method for procedurally adding dependencies. A resizer can have up to four constraints.
	virtual int				AddConstraint(	TView* 			target,
											ChangeTypeMode	theChangeType,
											AdjustMode		theAdjustMode);
											
	virtual void			SetLimitFrame( CViewRect& limitFrame, DragMode theDragMode);
											
protected:

	virtual	void			RespondToFrameAdjustment(	CViewRect&		/* newFrame */,
														CViewRect&		/* deltaFrame */,
														TView*		/* changedView */);
	virtual	void			RespondToOtherViewChange(	TView* 		/* changedView */) { } // = 0;

	virtual	void			SetGeneralMode(	int 			targetIndex,
											ChangeTypeMode	theChangeType,
											AdjustMode		theAdjustMode);


private:
	int						fNumTargets; // Number of active dependencies.
	DragMode				fDragMode;	// Type of dragging supplied.
	AssociationMode			fChildPeerOrParent[kNumTargets];
	ChangeTypeMode			fChangeTypeMode[kNumTargets];
	AdjustMode				fAdjustMode[kNumTargets];
	double					fProportion[kNumTargets];		// Proportion if eAdjProportional
	IDType					fTargetViewID[kNumTargets];	// ID of view who notifies us of changed frame.
	TView*					fTarget[kNumTargets];		// Pointers to views which notify us of changes.
	CViewRect					fOldFrame[kNumTargets];
	CViewRect					fLimitFrame;		// Current drag limits.
	bool					fInitialized;		// Prevents double initialization.
};

//----------------------------------------------------------------------------------------
// TDragViewCommand
//----------------------------------------------------------------------------------------

class EXPORT_MA TDragViewCommand : public TTracker
{
public:
	//------------------------------------------------------------------------------------
	// Constructor & Destructor  
	//------------------------------------------------------------------------------------
	TDragViewCommand(TView* viewToDrag = NULL,
					const CViewPoint& itsMouse = gZeroVPt, const CViewRect& constraint = CViewRect::kZeroRect);
		// constructor

	virtual ~TDragViewCommand();
		// Destructor
		
	//------------------------------------------------------------------------------------
	// Mouse Tracking 
	//------------------------------------------------------------------------------------
	virtual void TrackConstrain(TrackPhase aTrackPhase,
										const CViewPoint& anchorPoint,
										const CViewPoint& previousPoint,
										CViewPoint& nextPoint,
										bool mouseDidMove);

	virtual void TrackFeedback(TrackPhase aTrackPhase,
										const CViewPoint& anchorPoint,
										const CViewPoint& previousPoint,
										const CViewPoint& nextPoint,
										bool mouseDidMove,
										bool turnItOn);

	virtual TTracker* TrackMouse(TrackPhase aTrackPhase,
										CViewPoint& anchorPoint,
										CViewPoint& previousPoint,
										CViewPoint& nextPoint,
										bool mouseDidMove);
										
	//------------------------------------------------------------------------------------
	// Command Execution  
	//------------------------------------------------------------------------------------
	virtual void DoIt();
	
private:
	TView*	fViewToDrag;
	CViewRect	fConstraint;
	CViewRect	fOldFrame;
};

//----------------------------------------------------------------------------------------
// Global initialization procedure
//----------------------------------------------------------------------------------------

EXPORT_MA  void InitUViewDragger();

#endif
