//----------------------------------------------------------------------------------------
// UBehavior.cp
// Copyright  1991-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UBEHAVIOR__
#include "UBehavior.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif


//========================================================================================
// CLASS TBehavior
//========================================================================================

#undef Inherited


MA_DEFINE_CLASS(TBehavior);

//----------------------------------------------------------------------------------------
// TBehavior constructor
//----------------------------------------------------------------------------------------


TBehavior::TBehavior(IDType itsIdentifier) :
	fOwner(NULL),
	fPreviousBehavior(NULL),
	fIdentifier(itsIdentifier),
	fIdleFreq(kMaxIdleTime),
	// Assume it never wants idle time 
	fLastIdle(0),
	fEnabled(true),
	fIsOwnerSelected(false)
{
}

//----------------------------------------------------------------------------------------
// TBehavior copy constructor
//----------------------------------------------------------------------------------------


TBehavior::TBehavior(const TBehavior& inBehavior) :
	MStreamable(inBehavior),

	fOwner(NULL),
	fPreviousBehavior(NULL),
	fIdentifier(inBehavior.fIdentifier),
	fIdleFreq(inBehavior.fIdleFreq),
	fLastIdle(inBehavior.fLastIdle),
	fEnabled(inBehavior.fEnabled),
	fIsOwnerSelected(inBehavior.fIsOwnerSelected)
{
}

//----------------------------------------------------------------------------------------
// TBehavior::Free:
//----------------------------------------------------------------------------------------


TBehavior::~TBehavior()
{
}

//----------------------------------------------------------------------------------------
// TBehavior::ReadFrom:
//----------------------------------------------------------------------------------------


void TBehavior::ReadFrom(CStream_AC* aStream)		// Override 
{
	MStreamable::ReadFrom(aStream);

	fIdentifier = aStream->ReadIDType();
	fEnabled = aStream->ReadBoolean();
	fIdleFreq = aStream->ReadLong();
}

//----------------------------------------------------------------------------------------
// TBehavior::WriteTo:
//----------------------------------------------------------------------------------------


void TBehavior::WriteTo(CStream_AC* aStream) const		// Override 
{
	MStreamable::WriteTo(aStream);

	aStream->WriteIDType(fIdentifier);
	aStream->WriteBoolean(fEnabled);
	aStream->WriteLong(fIdleFreq);
}

//----------------------------------------------------------------------------------------
// TBehavior::AppendBehavior:
//----------------------------------------------------------------------------------------


void TBehavior::AppendBehavior(TBehavior* aBehavior)
{
	if (aBehavior)
	{
		if (fNextBehavior)
			fNextBehavior->AppendBehavior(aBehavior);
		else
		{
			SetNextBehavior(aBehavior);
			aBehavior->SetPreviousBehavior(this);
		}
	}
}

//----------------------------------------------------------------------------------------
// TBehavior::SetOwner:
//----------------------------------------------------------------------------------------


void TBehavior::SetOwner(TEventHandler* itsOwner)
{
	fOwner = itsOwner;
}

//----------------------------------------------------------------------------------------
// TBehavior::SelectOwner:
//----------------------------------------------------------------------------------------


void TBehavior::SelectOwner(bool select)
{
	fIsOwnerSelected = select;
}

//----------------------------------------------------------------------------------------
// TBehavior::SetEnabled:
//----------------------------------------------------------------------------------------


void TBehavior::SetEnabled(bool enabled)
{
	fEnabled = enabled;
}

//----------------------------------------------------------------------------------------
// TBehavior::GetNextEnabledBehavior:
//----------------------------------------------------------------------------------------


TBehavior* TBehavior::GetNextEnabledBehavior() const
{
	TBehavior* returnVal = NULL;

	for (TBehavior* test = GetNextBehavior(); test; test = test->GetNextBehavior())
		if (test->IsEnabled())
		{
			returnVal = test;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TBehavior::InsertBefore:
//----------------------------------------------------------------------------------------


void TBehavior::InsertBefore(TBehavior* insertMe,
							 TBehavior* before)
{
	if (insertMe)
	{
		// We are one that insertion is going to be before
		if (this == before)
		{
			if (fPreviousBehavior)
				fPreviousBehavior->SetNextBehavior(insertMe);

			insertMe->SetPreviousBehavior(fPreviousBehavior);
			insertMe->SetNextBehavior(this);

			SetPreviousBehavior(insertMe);
		}
		else if (fNextBehavior)				// Pass message on down chain
			fNextBehavior->InsertBefore(insertMe, before);
	}
}

//----------------------------------------------------------------------------------------
// TBehavior::InsertAfter:
//----------------------------------------------------------------------------------------


void TBehavior::InsertAfter(TBehavior* insertMe,
							TBehavior* after)
{
	if (insertMe)
	{
		// We are one that insertion is going to be after
		if (this == after)
		{
			TBehavior * nextBehavior = GetNextBehavior();

			insertMe->SetPreviousBehavior(this);
			insertMe->SetNextBehavior(nextBehavior);

			if (nextBehavior)
				nextBehavior->SetPreviousBehavior(insertMe);

			SetNextBehavior(insertMe);
		}
		else if (fNextBehavior)				// Pass message on down chain
			fNextBehavior->InsertAfter(insertMe, after);
	}
}

//----------------------------------------------------------------------------------------
// TBehavior::RemoveBehavior:
//----------------------------------------------------------------------------------------


void TBehavior::RemoveBehavior(TBehavior* removeMe)
{
	// We are the one being removed
	if (this == removeMe)
	{
		TBehavior * itsNextBehavior = GetNextBehavior();

		if (fPreviousBehavior)
			fPreviousBehavior->SetNextBehavior(itsNextBehavior);

		if (itsNextBehavior)
			itsNextBehavior->SetPreviousBehavior(fPreviousBehavior);

		// remove it from the linked list
		SetNextBehavior(NULL);
		SetPreviousBehavior(NULL);
		SetOwner(NULL);
	}
	else if (fNextBehavior) 					// Pass message on down chain as we aren't one to be removed
			fNextBehavior->RemoveBehavior(removeMe);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoCommandKeyEvent:
//----------------------------------------------------------------------------------------


void TBehavior::DoCommandKeyEvent(TToolboxEvent* event)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoCommandKeyEvent(event);
	else if (fOwner)
		fOwner->DoCommandKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoEvent:
//----------------------------------------------------------------------------------------


void TBehavior::DoEvent(EventNumber eventNumber,
						TEventHandler* source,
						TEvent* event)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoEvent(eventNumber, source, event);
	else if (fOwner)
		fOwner->DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoKeyEvent:
//----------------------------------------------------------------------------------------


void TBehavior::DoKeyEvent(TToolboxEvent* event)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoKeyEvent(event);
	else if (fOwner)
		fOwner->DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoKeyUp:
//----------------------------------------------------------------------------------------


void TBehavior::DoKeyUp(TToolboxEvent* event)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoKeyUp(event);
	else if (fOwner)
		fOwner->DoKeyUp(event);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoMenuCommand:
//----------------------------------------------------------------------------------------


void TBehavior::DoMenuCommand(CommandNumber aCommandNumber)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoMenuCommand(aCommandNumber);
	else if (fOwner)
		fOwner->DoMenuCommand(aCommandNumber);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoScriptCommand:
//----------------------------------------------------------------------------------------


bool TBehavior::DoScriptCommand(CommandNumber aCommandNumber,
								   TAppleEvent* message,
								   TAppleEvent* reply)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoScriptCommand(aCommandNumber, message, reply) : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoSetupMenus:
//----------------------------------------------------------------------------------------


void TBehavior::DoSetupMenus()
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoSetupMenus();
	else if (fOwner)
		fOwner->DoSetupMenus();
}

//----------------------------------------------------------------------------------------
// TBehavior::NextIdle:
//----------------------------------------------------------------------------------------


unsigned long TBehavior::NextIdle()
{
	unsigned long returnVal = kMaxIdleTime;

	if (fIdleFreq != kMaxIdleTime && IsEnabled())// Does it idle at all? 
		returnVal = fLastIdle ? fLastIdle + fIdleFreq : TickCount();		// if never idled then it needs idling now 

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TBehavior::HandleIdle:
//----------------------------------------------------------------------------------------


void TBehavior::HandleIdle(IdlePhase phase)
{
	// Do first, because 'this' may be freed by DoIdle
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	// Does it idle at all? 
	// idleBegin and idleEnd don't respect fIdleFreq because they are
	// notifications that the application is beginning/ending idle.
	// idleContinues, however, are parceled out at the fIdleFreq.
	// ALSO: don't store the fLastIdle if I FREE'd myself or
	// the idlePhase is not idleContinue

	if ((IsEnabled() && fIdleFreq != kMaxIdleTime)
	&& (phase != idleContinue || (TickCount() - fLastIdle >= fIdleFreq))
	&& (!DoIdle(phase) && phase == idleContinue))
		fLastIdle = TickCount();

	if (nextBehavior)
		nextBehavior->HandleIdle(phase);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoIdle:
//----------------------------------------------------------------------------------------


bool TBehavior::DoIdle(IdlePhase)
{
	return false;								// Did not free myself 
}

//----------------------------------------------------------------------------------------
// TBehavior::SetIdleFreq:
//----------------------------------------------------------------------------------------


void TBehavior::SetIdleFreq(long newIdleFreq)
{
	fIdleFreq = newIdleFreq;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoMouseCommand:
//----------------------------------------------------------------------------------------


bool TBehavior::DoMouseCommand(CViewPoint& theMouse,
								  TToolboxEvent* event,
								  CPoint_AC hysteresis)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoMouseCommand(theMouse, event, hysteresis) : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoMouseUp:
//----------------------------------------------------------------------------------------


bool TBehavior::DoMouseUp(CViewPoint& theMouse,
							 TToolboxEvent* event,
							 CPoint_AC hysteresis)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoMouseUp(theMouse, event, hysteresis) : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::Draw:
//----------------------------------------------------------------------------------------


void TBehavior::Draw(const CViewRect& area)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->Draw(area);
}

//----------------------------------------------------------------------------------------
// TBehavior::DoSetCursor:
//----------------------------------------------------------------------------------------


bool TBehavior::DoSetCursor(const CViewPoint& localPoint,
							   RgnHandle cursorRegion)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoSetCursor(localPoint, cursorRegion) : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoUnsetCursor:
//----------------------------------------------------------------------------------------


bool TBehavior::DoUnsetCursor()
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoUnsetCursor() : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoToolboxEvent:
//----------------------------------------------------------------------------------------


bool TBehavior::DoToolboxEvent(TToolboxEvent* event)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	return nextBehavior ? nextBehavior->DoToolboxEvent(event) : false;
}

//----------------------------------------------------------------------------------------
// TBehavior::DoPostCreate:
//----------------------------------------------------------------------------------------


void TBehavior::DoPostCreate(TDocument* itsDocument)
{
	TBehavior * nextBehavior = GetNextEnabledBehavior();

	if (nextBehavior)
		nextBehavior->DoPostCreate(itsDocument);
}

//========================================================================================
// CLASS CBehaviorIterator
//========================================================================================

#undef Inherited
#define Inherited CIterator_AC

//----------------------------------------------------------------------------------------
// CBehaviorIterator::CBehaviorIterator.
//----------------------------------------------------------------------------------------


CBehaviorIterator::CBehaviorIterator(const TEventHandler* itsEventHandler) :
	fCurrent(NULL),
	fFirstBehavior(NULL),
	fNextBehavior(NULL)
{
	if (itsEventHandler)
		fFirstBehavior = itsEventHandler->GetFirstBehavior();

	First();
}

//----------------------------------------------------------------------------------------
// CBehaviorIterator::First:
//----------------------------------------------------------------------------------------


TBehavior* CBehaviorIterator::First()
{
	fCurrent = fFirstBehavior;
	if (fFirstBehavior)
		fNextBehavior = fFirstBehavior->GetNextBehavior();
	
	return fCurrent;
}

//----------------------------------------------------------------------------------------
// CBehaviorIterator::Next:
//----------------------------------------------------------------------------------------


TBehavior* CBehaviorIterator::Next()
{
	if (fCurrent)
	{
		fCurrent = fNextBehavior;
		if (fCurrent)
			fNextBehavior = fCurrent->GetNextBehavior();
	}
	
	return fCurrent;
}

//----------------------------------------------------------------------------------------
// End of UBehavior.cp


