//----------------------------------------------------------------------------------------
// UCommand.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

// ACS

#ifndef __CDependencies_AC__
#include "CDependencies_AC.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

// Toolbox

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// Global Functions
//========================================================================================


TCommand* CommitACommand(TCommand *aCommand)
{	
	if (aCommand)
	{
#if qDebug
		::ThrowIfNonObject_AC(aCommand);
#endif
		//	if (aCommand->fCommandDone)
		//		aCommand->Commit();
			
		aCommand->Completed();
		
		if (aCommand->ShouldFreeOnCompletion())
			delete aCommand;
	}
		
	return NULL;
}

//========================================================================================
// CLASS TCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TCommand constructor
//----------------------------------------------------------------------------------------


TCommand::TCommand(CommandNumber itsCommandNumber,
								 TCommandHandler* itsContext,
								 bool canUndo,
								 bool causesChange,
							 	 MDependable_AC* objectToNotify) :
	TEvent(itsCommandNumber, NULL, itsContext ? itsContext : gDispatcher),

	fObjectToNotify(objectToNotify),
	fContext(dynamic_cast<TCommandHandler*>(fHandler)),
	fCommandDone(false),
	fCanUndo(canUndo),
	fCausesChange(causesChange),
	fChangesClipboard(false),
	fClipboardView(NULL),
	fUndoClipboardView(NULL),
	fUndoClipboardViewContext(NULL),
	fFreeOnCompletion(true),				// command will be freed after DoIt or Commit
	fRecurring(false),						// Command is a one-shot deal 
	fReadyToExecute(true),					// Always executable by default 
	fValidationError(noErr),				// Hasn't failed yet...
	fUseAppleEvent(false)
{
}

//----------------------------------------------------------------------------------------
// TCommand::Free:
//----------------------------------------------------------------------------------------


TCommand::~TCommand()
{
	if (fChangesClipboard)
	{
		AbandonClipboardView();						
		AbandonUndoClipboardView();						
	}
	
	// If this command is in the action history, it should
	// be freed only through the undo handler, so the
	// action history can be cleaned up before the command is freed. 
	
	//	LinkToSecondary(NULL); // clear the reference to this command in its linked cmd
}

//----------------------------------------------------------------------------------------
// TCommand::AbandonClipboardView
//----------------------------------------------------------------------------------------


void TCommand::AbandonClipboardView() // override
{
//	fClipboardView is owned by this command, which created it, unless it
//	is the current clipboard, in which case it is owned by gClipboardMgr.

// Note:: TView::FreeFromClipboard calls gClipboardMgr->Changed
// so that if another command has a reference to this fClipboardView
// in its fUndoClipboardView field, it can NULL it.

	if (fClipboardView)
	{
		if (fClipboardView == gClipboardMgr->fClipView)
			gClipboardMgr->fClipContext = NULL;
		else
		{
			fClipboardView->FreeFromClipboard();
			fClipboardView = NULL;
		}
	}
}

//----------------------------------------------------------------------------------------
// TCommand::AbandonClipboardView
//----------------------------------------------------------------------------------------


void TCommand::AbandonUndoClipboardView()  
{
// Note:: TView::FreeFromClipboard calls gClipboardMgr->Changed
// so that if another command has a reference to this fUndoClipboardView
// in its fClipboardView field, it can NULL it.

	if (fUndoClipboardView 
		&& (fUndoClipboardView != gClipboardMgr->fClipView)
		&& (!fUndoClipboardViewContext || (fUndoClipboardViewContext == fContext))	)
	{
		fUndoClipboardView->FreeFromClipboard();
		fUndoClipboardView = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TCommand::PrepareForUndoRedo:
//----------------------------------------------------------------------------------------


void TCommand::PrepareForUndoRedo()
{
	if (fContext)
		fContext->PrepareForUndoRedo(this);
}

//----------------------------------------------------------------------------------------
// TCommand::FinishUndoRedo:
//----------------------------------------------------------------------------------------


void TCommand::FinishUndoRedo()
{
	if (fChangesClipboard)
		UndoRedoClipboard();
	
	fCommandDone = !fCommandDone;
	DoNotification();
}

//----------------------------------------------------------------------------------------
// TCommand::RevealUndoRedo:
//----------------------------------------------------------------------------------------


void TCommand::RevealUndoRedo()
{
	if (fContext)
		fContext->RevealUndoRedo(this);
}

//----------------------------------------------------------------------------------------
// TCommand::Completed:
//----------------------------------------------------------------------------------------



void TCommand::Completed()
{
}

//----------------------------------------------------------------------------------------
// TCommand::DoIt:
//----------------------------------------------------------------------------------------



void TCommand::DoIt()
{
}

//----------------------------------------------------------------------------------------
// TCommand::GetChangeID:
//----------------------------------------------------------------------------------------


ChangeID_AC TCommand::GetChangeID()
{
	return fIdentifier;
}

//----------------------------------------------------------------------------------------
// TCommand::IsReadyToExecute:
//----------------------------------------------------------------------------------------


bool TCommand::IsReadyToExecute() // override
{
	return fReadyToExecute;
}

//----------------------------------------------------------------------------------------
// TCommand::IsRecurring:
//----------------------------------------------------------------------------------------


bool TCommand::IsRecurring() // override
{
	return fRecurring;
}

//----------------------------------------------------------------------------------------
// TCommand::MakeAppleEvent:
//----------------------------------------------------------------------------------------


TAppleEvent* TCommand::MakeAppleEvent()
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TCommand::DoNotification:
//----------------------------------------------------------------------------------------


void TCommand::DoNotification()
{
	// Notify the proper object that a change has occurred 	
	if (WillCauseChange() && fObjectToNotify)
	{
		ChangeID_AC theChange;
		if (fCommandDone)
			theChange = GetChangeID();
		else
			theChange = cUndo;
		fObjectToNotify->Changed(theChange, this);
	}
}

//----------------------------------------------------------------------------------------
// TCommand::SetupDependencies:
//----------------------------------------------------------------------------------------


void TCommand::SetupDependencies()
{
	if (CanBeUndone())
	{
		fContext->AddDependent(this);
		if (fChangesClipboard || (fIdentifier == cPaste))
			gClipboardMgr->AddDependent(this);
	}
}

//----------------------------------------------------------------------------------------
// TCommand::DoUpdate:
//----------------------------------------------------------------------------------------


void TCommand::DoUpdate(ChangeID_AC theChange,
						MDependable_AC* changedObject,
						void* changeData,
						CDependencySpace_AC* /* dependencySpace */) // override
{
	switch (theChange)
	{
		case mClipViewChanged:
			// This code appears to be there to support Paste commands
			// whose RedoIt methods go back to the clipboard document for the
			// data to paste. *** This won't work with multiple undo. ***
			// From now on, commands need to save all the data needed for 
			// their UndoIt and RedoIt methods. 
			
			//	if ((changedObject == gClipboardMgr) &&
			//		(fIdentifier == cPaste))
			//		fContext->CommitLastCommand();
			break;

		case mFreeClipboardView:
			if (changedObject == gClipboardMgr) 
				if (changeData == fUndoClipboardView)
				{
					fUndoClipboardView = NULL;
					fUndoClipboardViewContext = NULL;
				}
				else if (changeData == fClipboardView)
					fClipboardView = NULL;
			break;

		case mClosed:
			if ((changedObject == fContext)
				/* || (theChange == mAboutToLoseControl) */
				|| (theChange == mScrapChanged)) 
				fContext->CommitLastCommand();
			break;
	}
}

//----------------------------------------------------------------------------------------
// TCommand::RedoIt:
//----------------------------------------------------------------------------------------


void TCommand::RedoIt()
{
	DoIt();
}

//----------------------------------------------------------------------------------------
// TCommand::ShouldFreeOnCompletion:
//----------------------------------------------------------------------------------------


bool TCommand::ShouldFreeOnCompletion()
{
	return (!fRecurring && fFreeOnCompletion);
}

//----------------------------------------------------------------------------------------
// TCommand::UndoIt:
//----------------------------------------------------------------------------------------


void TCommand::UndoIt()
{
}

//----------------------------------------------------------------------------------------
// TCommand::Process:
//----------------------------------------------------------------------------------------


void TCommand::Process()	
{
	if (fContext)
		fContext->PerformCommand(this);
}

//----------------------------------------------------------------------------------------
// TCommand::SetValidationError:
//----------------------------------------------------------------------------------------


void TCommand::SetValidationError(OSErr	validationError)
{
	fValidationError = validationError;
}

//----------------------------------------------------------------------------------------
// TCommand::Abort:
//----------------------------------------------------------------------------------------


void TCommand::Abort()
{
	// Try to remove the command in case it is no longer recurring.
	gDispatcher->RemoveEvent(this);
	
	if (gClipboardMgr->fClipView && gClipboardMgr->fClipView == fClipboardView)
	{
		gClipboardMgr->SetClipView(fUndoClipboardView, fUndoClipboardViewContext);
		fUndoClipboardView = NULL;
	}

	if (fCommandDone && CanBeUndone())
	{
		UndoIt();
		if (fChangesClipboard)
			UndoRedoClipboard();
		fCommandDone = false;
		DoNotification();
	}
	
	Completed();
}

//----------------------------------------------------------------------------------------
// TCommand::ClaimClipboard:
//----------------------------------------------------------------------------------------


void TCommand::ClaimClipboard(TView* clipboardView)
{
	// For RedoIt only
	AbandonUndoClipboardView();

	fUndoClipboardView = gClipboardMgr->fClipView;	// Copy current clipboard contents to the
													// Undo side
	fUndoClipboardViewContext = fContext;			// gClipboardMgr->fClipContext; 
	if (clipboardView)
	{
		gClipboardMgr->SetClipView(clipboardView, fContext);			// Will install it as fClipView 
		fClipboardView = clipboardView;	
	}	
#if qDebugMsg
	else
		::ProgramBreak_AC("Claiming clipboard with NULL view");
#endif
}

//----------------------------------------------------------------------------------------
// TCommand::UndoRedoClipboard:
//----------------------------------------------------------------------------------------


void TCommand::UndoRedoClipboard()
{	
	if (fCommandDone)
	{
		// Undoing
		if (fUndoClipboardView && (gClipboardMgr->fClipContext == fContext))
		{
			gClipboardMgr->SetClipView(fUndoClipboardView, fUndoClipboardViewContext);
		}
	}
	else if (fValidationError == noErr)
	{
		// Redoing
		ClaimClipboard(fClipboardView);		
	}
}

//========================================================================================
// CLASS TAppleCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TAppleCommand::Free:
//----------------------------------------------------------------------------------------


TAppleCommand::~TAppleCommand()
{
}

//----------------------------------------------------------------------------------------
// TAppleCommand::FreeTheMessage:
//----------------------------------------------------------------------------------------


void TAppleCommand::FreeTheMessage()
{
	fMessage.reset();
	fReply.reset();
}

//========================================================================================
// CLASS TServerCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TServerCommand constructor
//----------------------------------------------------------------------------------------


TServerCommand::TServerCommand(CommandNumber itsCommandNumber,
									TCommandHandler* itsContext,
									bool canUndo,
									bool causesChange,
									MDependable_AC* objectToNotify,
									TAppleEvent* itsMessage,
									TAppleEvent* itsReply,
									bool suspendTheEvent) :
	TAppleCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify, itsMessage, itsReply),

	fRequiresUserInteraction(false),
	fSuspendTheEvent(suspendTheEvent),
	fReplySent(false)
{
	// Inform the AppleEvent manager that this event will be processed asynchronously.
	// Any reply that needs to be returned will be returned in TServerCommand::Completed().
	if (fSuspendTheEvent)
		AESuspendTheCurrentEvent(fMessage);
}

//----------------------------------------------------------------------------------------
// TServerCommand constructor
//----------------------------------------------------------------------------------------


TServerCommand::TServerCommand(CommandNumber itsCommandNumber,
									TCommandHandler* itsContext,
									bool canUndo,
									bool causesChange,
									MDependable_AC* objectToNotify,
									const AppleEvent& itsMessage,
									const AppleEvent& itsReply,
									bool suspendTheEvent) :
	TAppleCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify, TH_new TAppleEvent(itsMessage, false), TH_new TAppleEvent(itsReply, false)),

	fRequiresUserInteraction(false),
	fSuspendTheEvent(suspendTheEvent),
	fReplySent(false)
{
	// Inform the AppleEvent manager that this event will be processed asynchronously.
	// Any reply that needs to be returned will be returned in TServerCommand::Completed().
	if (fSuspendTheEvent)
		AESuspendTheCurrentEvent(fMessage);
}

//----------------------------------------------------------------------------------------
// TServerCommand destructor
//----------------------------------------------------------------------------------------


TServerCommand::~TServerCommand()
{
}

//----------------------------------------------------------------------------------------
// TServerCommand::Completed:
//----------------------------------------------------------------------------------------


void TServerCommand::Completed() // override
{
	if (fMessage && fReply)
		ResumeAndFreeMessage();
}

//----------------------------------------------------------------------------------------
// TServerCommand::Process:
//----------------------------------------------------------------------------------------


void TServerCommand::Process()
{
	if (fRequiresUserInteraction)
	{
		try
		{
			::ThrowIfOSErr_AC(MAInteractWithUser());
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			SetValidationError(theException.GetError());
			throw;
		}
	}

	try
	{
		TAppleCommand::Process();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		// If an error arose and a reply was sent then ignore the error
		// at this point.
		if (!fReplySent)
		{
			if (theException.GetError() == userCanceledErr)
				::ThrowNewMessage_AC(noErr, theException.GetMessage(), messageCancelled);
			throw;
		}
	}
}

//----------------------------------------------------------------------------------------
// TServerCommand::ResumeAndFreeMessage:
//----------------------------------------------------------------------------------------


void TServerCommand::ResumeAndFreeMessage()
{
	// If the event has been suspended then resume it and free the message.
	if (fSuspendTheEvent)
	{
		// Inform the AppleEvent manager that we are through processing the event. If an error
		// has occurred the default reply will contain the error number and error string.
		AEEventHandlerUPP dispatcher = static_cast<AEEventHandlerUPP>(kAENoDispatch);

		AEResumeTheCurrentEvent(fMessage, fReply, dispatcher, 0);
		fReplySent = true;
	}
	// Free the message and its reply since they are no longer valid
	FreeTheMessage();
}

//----------------------------------------------------------------------------------------
// TServerCommand::SetValidationError:
//----------------------------------------------------------------------------------------


void TServerCommand::SetValidationError(OSErr error)
{
	// The finder passes a null reply with required events
	// so don't try to stuff any info into the reply
	if (fReply && fReply->descriptorType != typeNull)
		fReply->WriteOSError(error);
	TAppleCommand::SetValidationError(error);
}

//========================================================================================
// CLASS TClientCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TClientCommand constructor
//----------------------------------------------------------------------------------------


TClientCommand::TClientCommand(CommandNumber itsCommandNumber,
									TCommandHandler* itsContext,
									bool canUndo,
									bool causesChange,
									MDependable_AC* objectToNotify,
									TAppleEvent* theMessage) :
	TAppleCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify, theMessage),

	fMessageSent(false)
{
}

//----------------------------------------------------------------------------------------
// TClientCommand destructor
//----------------------------------------------------------------------------------------


TClientCommand::~TClientCommand()
{
}

//----------------------------------------------------------------------------------------
// TClientCommand::GetReturnID:
//----------------------------------------------------------------------------------------


long TClientCommand::GetReturnID()
{
	return fMessage ? fMessage->GetReturnID() : 0;
}

//----------------------------------------------------------------------------------------
// TClientCommand::IsReadyToPost:
//----------------------------------------------------------------------------------------


bool TClientCommand::IsReadyToPost() // override
{
	bool ready = true;
	
	if (fMessage)
	{
		if (!fMessageSent)
		{
			if ((fMessage->GetSendingMode() & kAEReplyModeMask) == kAEWaitReply)
			{
				TAppleEvent* theReply = NULL;
				
				theReply = SendMessage();
				ProcessReply(theReply);
			}
			else if ((fMessage->GetSendingMode() & kAEReplyModeMask) == kAEQueueReply)
			{
				SendMessage();	// No reply will be returned
				TOSADispatcher::fgDispatcher->PostPendingReplyCommand(this);
				ready = false;
			}
		}
		else if (((fMessage->GetSendingMode() & kAEReplyModeMask) == kAEQueueReply) && !fReply)
			ready = false;				// Even though the message has been sent we still
										// haven't received our reply so don't post it yet.
	}
	return ready;
}

//----------------------------------------------------------------------------------------
// TClientCommand::SendMessage:
//----------------------------------------------------------------------------------------


TAppleEvent* TClientCommand::SendMessage()
{
	TAppleEvent* returnVal = NULL;

	if (fMessage)
	{
		fMessageSent = true;
		returnVal = fMessage->Send();
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TClientCommand::ProcessReply:
//----------------------------------------------------------------------------------------


void TClientCommand::ProcessReply(TAppleEvent* theReply)
{
	fReply = theReply;
	
	// Check for errors in the reply.
	long 	errorNumber;
	CStr255_AC	errorString;
	theReply->ReadError(errorNumber, errorString);

	if ((errorNumber != noErr) || !errorString.IsEmpty())
	{
		if (gDispatcher)
			gDispatcher->ReportReplyError(fIdentifier, (OSErr)errorNumber, errorString);
	}
}

//----------------------------------------------------------------------------------------
// TClientCommand::DoIt:
//----------------------------------------------------------------------------------------


void TClientCommand::DoIt() // override
{
	if (fMessage && ((fMessage->GetSendingMode() & kAEReplyModeMask) == kAENoReply))
		SendMessage();
}

//----------------------------------------------------------------------------------------
// End of UCommand.cp


