//----------------------------------------------------------------------------------------
// UCommandHandler.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOMMANDHANDLER__
#include "UCommandHandler.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOREERRORMGR__
#include "UCoreErrorMgr.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __USCRIPTING__
#include "UScripting.h"
#endif

#ifndef __UUNDO__
#include "UUndo.h"
#endif


unsigned long TCommandHandler::fgTransactionIDCount;

//========================================================================================
// CLASS TCommandHandler
//========================================================================================
#undef Inherited
#define Inherited TEventHandler

MA_DEFINE_CLASS(TCommandHandler);

//----------------------------------------------------------------------------------------
// TCommandHandler constructor
//----------------------------------------------------------------------------------------

TCommandHandler::TCommandHandler(TEventHandler* itsNextHandler) :
	TEventHandler(itsNextHandler),
	fPendingActionID(0)
{
}

//----------------------------------------------------------------------------------------
// TCommandHandler destructor
//----------------------------------------------------------------------------------------

TCommandHandler::~TCommandHandler()
{
}

//----------------------------------------------------------------------------------------
// TCommandHandler::GetContext:
//----------------------------------------------------------------------------------------

TCommandHandler* TCommandHandler::GetContext(CommandNumber /* aCommandNumber */)
{
	return this;
}

//----------------------------------------------------------------------------------------
// TCommandHandler::PerformCommand:
//----------------------------------------------------------------------------------------

void TCommandHandler::PerformCommand(TCommand* command)
{
#if qDebug
	::ThrowIfNonObject_AC(command);
#endif
	
#if qDebugMsg
	if (gIntenseDebugging)
		ProgramReport_AC(false, "The Command to perform: %s\n", typeid(*command).name());
#endif

	if (command->GetUseAppleEvent() && PerformCommandAppleEvent(command))
	{
		// The command has been successfully converted to an Apple Event
		// and performed as an Apple Event. We don't need it any more. 
		//CommitACommand(command);
		command->Completed();
		if (command->ShouldFreeOnCompletion())
			delete command;
	}
	else
	{
		bool wasUndoAble = command->CanBeUndone();

		try
		{
			command->DoIt();

			if (wasUndoAble && !TUndoHandler::fgUndoHandler.FindTransaction(command))
				command = NULL;								// The command went away
			else
				command->SetCommandDone(true);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
	
			bool isOnUndoStack = false;
			if (wasUndoAble)
			{
				isOnUndoStack = TUndoHandler::fgUndoHandler.FindTransaction(command);
				if (!isOnUndoStack)
					command = NULL;							// The command went away
			}
	
			if (command)
			{
				command->SetValidationError(theException.GetError());
				
				// Save the ID because the command is going away. 
				CommandNumber aCommandNumber = command->GetIdentifier();
				if (isOnUndoStack)
				{
					// Abort the current transaction in the undo handler. 
					TUndoHandler::fgUndoHandler.Abort();
					if (!command->IsRecurring())
						delete command;
				}
				else if (!command->IsRecurring())
				{
					// Try to remove the command in case it is no longer recurring.
					gDispatcher->RemoveEvent(command);
					delete command;
				}
				
				::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), CException_AC::BuildMessage(static_cast<short>(aCommandNumber), messageCommandError));
			}
			throw;
		}
		
	
		if (command)
		{
			// This is done after DoIt, so DoIt can change the fCausesChange flag. 
			command->DoNotification();
			
			// Try to remove the command in case it is no longer recurring.
			gDispatcher->RemoveEvent(command);
			
			// Either set up for undo, or get rid of it
			if (command->CanBeUndone())
				command->SetupDependencies();
			else
			{
				command->Completed();
				if (command->ShouldFreeOnCompletion())
					delete command;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TCommandHandler::PerformCommandAppleEvent:
//----------------------------------------------------------------------------------------

bool TCommandHandler::PerformCommandAppleEvent(TCommand* command)
{
	OSErr dispatchError = noErr;
	long processError = 0L;
	
	CAutoPtr_AC<TAppleEvent> sendEvent;
	CAutoPtr_AC<TAppleEvent> resultEvent;
	
	try
	{
		sendEvent = command->MakeAppleEvent();
		if (sendEvent)
		{
#if qDebug
			::Assertion_AC(sendEvent->GetSendingMode() != kAEQueueReply,
					  "Command-based apple events must be sent synchronously");
#endif
			if (!fgTransactionIDCount)
				fgTransactionIDCount = 1;
				
			long transactionID = fgTransactionIDCount++;
			
			sendEvent->SetTransactionID(transactionID);
			fPendingActionID = transactionID;
			TAppleEvent* tempEvent = resultEvent;
			dispatchError = sendEvent->SendEvent(tempEvent);
			resultEvent = tempEvent;
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		dispatchError = theException.GetError();
	}
	
	
	if (dispatchError == noErr && resultEvent)
	{
		try
		{
			if (resultEvent->HasParameter(keyErrorNumber))
				processError = resultEvent->ReadLong(keyErrorNumber);
		}
		catch(CException_AC& theException)
		{
			DoCatchMessage_AC(theException);
			// ignore the error
		}
		
	}
	
	return dispatchError == noErr && !processError;
}

//----------------------------------------------------------------------------------------
// TCommandHandler::GetLastCommand:
//----------------------------------------------------------------------------------------

TCommand* TCommandHandler::GetLastCommand()
{
	
	
	
	return NULL;	// fLastCommand;
}

//----------------------------------------------------------------------------------------
// TCommandHandler::CommitLastCommand:
//----------------------------------------------------------------------------------------

void TCommandHandler::CommitLastCommand()
{
	// Clear the action history for this context 
	TUndoHandler::fgUndoHandler.ClearActionHistory(kDontRespectMarks, GetContext(cNoCommand));
}

//----------------------------------------------------------------------------------------
// TCommandHandler::PrepareForUndoRedo:
//----------------------------------------------------------------------------------------

void TCommandHandler::PrepareForUndoRedo(TCommand* /*command*/)
{
	if (WantsToBeTarget())
		BecomeTarget();
}

//----------------------------------------------------------------------------------------
// TCommandHandler::RevealUndoRedo:
//----------------------------------------------------------------------------------------

void TCommandHandler::RevealUndoRedo(TCommand* /*command*/)
{
}

//----------------------------------------------------------------------------------------
// End of UCommandHandler.cp


