//----------------------------------------------------------------------------------------
// UDeskScrapView.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDESKSCRAPVIEW__
#include "UDeskScrapView.h"
#endif

// ACS

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

// MacApp

#ifndef __UCLIPBOARDMGR__
#include "UClipboardMgr.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

// Toolbox

#ifndef __ERRORS__
#include <Errors.h>
#endif

// ANSI




//========================================================================================
// static data members
//========================================================================================
bool TDeskScrapView::gVarClipPicSize;

static bool LookForScrapType(ResType theResType, ResType& resTypeFound);

//========================================================================================
// CLASS TDeskScrapView
//========================================================================================
#undef Inherited
#define Inherited TView


MA_DEFINE_CLASS(TDeskScrapView);

//----------------------------------------------------------------------------------------
// TDeskScrapView constructor
//----------------------------------------------------------------------------------------


TDeskScrapView::TDeskScrapView() :
	TView(gZeroVPt, CViewPoint(200, 200), sizeVariable, sizeVariable),

	fDataHandle(NULL),
	fHavePicture(false),
	fHaveText(false),
	fScrapCount(gClipboardMgr->fClipboard->GetScrapCountMinusOne())
{
	// Can't know the extent required until we actually read the data in, which we won't bother
	// to do unless forced to by receiving an Update event
}

//----------------------------------------------------------------------------------------
// TDeskScrapView destructor
//----------------------------------------------------------------------------------------


TDeskScrapView::~TDeskScrapView()
{
	if (this == gClipboardMgr->fClipOrphanage.get())
		gClipboardMgr->fClipOrphanage.release();
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::ReadFields: 
//----------------------------------------------------------------------------------------


void TDeskScrapView::ReadFields(CStream_AC* aStream)
{
	Inherited::ReadFields(aStream);

	fScrapCount = gClipboardMgr->fClipboard->GetScrapCountMinusOne();
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::CalcMinFrame: 
//----------------------------------------------------------------------------------------

CViewRect TDeskScrapView::CalcMinFrame()// override 
{
	CViewRect minFrame(Inherited::CalcMinFrame());

	if (fDataHandle)
		if (fHaveText)
		{
			// TEXT 
			if (Focus())
			{
				SetPortTextStyle(gSystemStyle);
				
				if (fSuperView)
					minFrame = SuperViewToLocal(fSuperView->GetFrame());

				CRect_AC qdArea(ViewToQD(minFrame));
				
				if (GetHandleSize(fDataHandle) < 32000)	// more than 32000 characters will kill TECalText
				{
					TEHandle aTEHandle = TENew(qdArea, qdArea);
					::ThrowIfNULL_AC((Ptr)aTEHandle);
	
					Handle oldHText = (*aTEHandle)->hText;	// We'll need this for TEDispose 
					(*aTEHandle)->hText = fDataHandle;
	
					TECalText(aTEHandle);
	
					minFrame.bottom = minFrame.top + Min_AC((*aTEHandle)->nLines * (*aTEHandle)->lineHeight, static_cast<short>(kMaxQDCoord));
	
					(*aTEHandle)->hText = oldHText;	// So TEDispose doesn't dispose our handle 
	
					TEDispose(aTEHandle);
				}
			}
		}
		else
		{
#if qDebugMsg
			if (gIntenseDebugging)
			{
				CRect_AC picFrame = (**(PicHandle)fDataHandle).picFrame;
				ProgramReport_AC("Picture Frame:");
				ProgramReport_AC(false, "%s\n", (const char*)(picFrame));
			}
#endif
			if (!gVarClipPicSize)
			{
				CRect_AC picFrame = (*((PicHandle)fDataHandle))->picFrame;
				minFrame[botRight] = minFrame[topLeft] + CViewPoint(picFrame.GetSize());
			}
		}

	return minFrame;
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::CheckScrapContents: 
//----------------------------------------------------------------------------------------

void TDeskScrapView::CheckScrapContents()
{
	if (!gClipboardMgr->fClipboard->ScrapCountStillValid(fScrapCount))
	{
		// Make sure we get rid of the old scrap handle if necessary 
		fDataHandle = ::DisposeIfHandle_AC(fDataHandle);

		ResType resTypeFound;
		fHavePicture = LookForScrapType(kPictType_AC, resTypeFound);
		fHaveText = LookForScrapType(kTextType_AC, resTypeFound);

		if (fHavePicture || fHaveText)
		{
			CTempHandle_AC aHandle(NewPermHandle(0));

			try
			{
				bool savedTemp = TemporaryAllocation(false);
				long offset;
				long length = CClipboard_AC::GetScrap(aHandle, resTypeFound, &offset);
				savedTemp = TemporaryAllocation(savedTemp);

				if (length < 0)					// Only results < 0 are really errors 
					::ThrowIfOSErr_AC((OSErr)length);
				fDataHandle = aHandle;
				AdjustFrame();
				Focus();					// Make sure we're still focused 

				aHandle.DontRelease();
			}
			catch(CException_AC& theException)
			{
				DoCatchMessage_AC(theException);
				fDataHandle = NULL;
				throw;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::Draw: 
//----------------------------------------------------------------------------------------


void TDeskScrapView::Draw(const CViewRect& area)
{
	// Draw routine must not fail. Otherwise alert can cause an infinite loop of draw
	// fail, alert, update event, draw fail, etc..
	try
	{
#if qDebug
		AssumeFocused();
#endif

		// Has scrap been updated since we last drew? 
		CheckScrapContents();
	
		CRect_AC aRect;
		if (fHaveText)
		{
			SetPortTextStyle(gApplicationStyle);
			aRect = GetQDExtent();
	
			CTempHandleLock_AC tempHandleLock(fDataHandle);
			
			TextBox_AC(*fDataHandle, GetHandleSize(fDataHandle), aRect, teFlushDefault, kAutoWrap, NULL, kNoEraseFirst, kNoSpaceForCaret);
		}
		else if (fHavePicture)
		{
			if (gVarClipPicSize)
				aRect = GetQDExtent();
			else
				aRect.Set(gZeroPt, ((CRect_AC &) (**((PicHandle)fDataHandle)).picFrame).GetSize());
	
			DrawPicture((PicHandle)fDataHandle, &aRect);
		}
	
		if (!gClipboardMgr->fClipboard->ScrapCountStillValid(fScrapCount))
		{
			Validate(aRect);				// avoid flicker upon clipboard installation
			fScrapCount = gClipboardMgr->fClipboard->GetScrapCount();
			gClipboardMgr->fClipWrittenToDeskScrap = true;// Don't need to write it back out. 
		}
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		// do nothing. We could draw a string "unable to show the clipboard at this time"
		// definitely *don't* resignal.
	}
	

	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::SuperViewChangedFrame: 
//----------------------------------------------------------------------------------------


void TDeskScrapView::SuperViewChangedFrame(const CViewRect& oldFrame,
												 const CViewRect& newFrame,
												 bool invalidate)// override 
{
	if (oldFrame.GetSize() != newFrame.GetSize())
	{
		AdjustFrame();

		if (invalidate)
			ForceRedraw();		// because of word wrap
	}
}

//----------------------------------------------------------------------------------------
// TDeskScrapView::WriteToDeskScrap: 
//----------------------------------------------------------------------------------------

void TDeskScrapView::WriteToDeskScrap()// override 
{
	// This view represents data that's already written to the desk scrap 
}

//----------------------------------------------------------------------------------------
// LookForScrapType: 
//----------------------------------------------------------------------------------------

bool TDeskScrapView::LookForScrapType(ResType theResType, ResType& resTypeFound)
{
	long offset;

	long dataLength = CClipboard_AC::GetScrap(NULL, theResType, &offset);
	if (dataLength > 0)
		resTypeFound = theResType;
#if qDebugMsg
	if ((dataLength != noTypeErr) && (dataLength < 0))
		ProgramReport_AC(false, "LookForScrapType: err = %d\n", dataLength);
#endif

	return (dataLength > 0);
}

//----------------------------------------------------------------------------------------
// End of UDeskScrapView.cp


