//----------------------------------------------------------------------------------------
// UDialogBehavior.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDIALOGBEHAVIOR__
#include "UDialogBehavior.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UUNDO__
#include "UUndo.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//========================================================================================
// CLASS TDialogBehavior
//========================================================================================
#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TDialogBehavior);

//----------------------------------------------------------------------------------------
// TDialogBehavior constructor
//----------------------------------------------------------------------------------------


TDialogBehavior::TDialogBehavior(bool isModal,
											 IDType  itsDefaultItem,
											 IDType  itsCancelItem) :
	TBehavior(kDialogBehavior),

	fModal(isModal),
	fDefaultItem(itsDefaultItem),
	fCancelItem(itsCancelItem),
	fDismisser(kNoIdentifier),
	fDismissed(true)
{
}

//----------------------------------------------------------------------------------------
// TDialogBehavior destructor
//----------------------------------------------------------------------------------------


TDialogBehavior::~TDialogBehavior()
{
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::ReadFrom: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::ReadFrom(CStream_AC* aStream)				// override 
{
	Inherited::ReadFrom(aStream);

	fModal = aStream->ReadBoolean();
 	fDefaultItem = aStream->ReadIDType();
	fCancelItem = aStream->ReadIDType();
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::WriteTo: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::WriteTo(CStream_AC* aStream) const				// override 
{
	Inherited::WriteTo(aStream);

	aStream->WriteBoolean(fModal);
	aStream->WriteIDType(fDefaultItem);
	aStream->WriteIDType(fCancelItem);
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::Dismiss: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::Dismiss(IDType dismisser, bool validate)
{
	TView* owner = dynamic_cast<TView*>(fOwner);

	if (owner)
	{
		if (validate)
		{
			if (owner->IsHierarchyValid())
			{
				fDismisser = dismisser;
				fDismissed = true;
			}
		}
		else
		{
			fDismisser = dismisser;
			fDismissed = true;			
		}
		
	}
	
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::DoEvent: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::DoEvent(EventNumber eventNumber,
								 TEventHandler* source,
								 TEvent* event)// override 
{
	if (eventNumber == mDismiss /*dismiss*/)
		Dismiss(source->GetIdentifier(), source->GetIdentifier() != fCancelItem);
	else
		Inherited::DoEvent(eventNumber, source, event);
	
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::DoKeyEvent: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::DoKeyEvent(TToolboxEvent* event)// override 
{
	// Maps the Enter and Return keys to the default item (view)
	// Maps the Escape key to the cancel item (view)
	
	TView* owner = dynamic_cast<TView*>(fOwner);
	bool wasHandled = false;

	if (owner && owner->IsEnabled())
	{
		if (event->GetText() == chEscape && event->GetKeyCode() != kClearVirtualCode) // esc double for two different keys! 
		{
			if (fCancelItem != kNoIdentifier)
			{
				wasHandled = true;
				TView * cancelView = owner->FindSubView(fCancelItem);
				if (cancelView)
				{
					if (cancelView->IsEnabled())
						cancelView->HandleEvent(cancelView->GetEventNumber(), owner, NULL);
				}
				else
					owner->HandleEvent(mCancelKey, owner, NULL);
			}
		}
		else if(event->GetText() == chEnter || event->GetText() == chReturn)
		{
			if (fDefaultItem != kNoIdentifier)
			{
				wasHandled = true;
				::AssertNotNULL_AC(owner);
				
				TView * defaultView = owner->FindSubView(fDefaultItem);
				if (defaultView)
				{
					if (defaultView->IsEnabled())
						defaultView->HandleEvent(defaultView->GetEventNumber(), owner, NULL);
				}
				else
					owner->HandleEvent(mDefaultKey, owner, NULL);
			}
		}
	}

	if (!wasHandled)
		Inherited::DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::DoCommandKeyEvent: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::DoCommandKeyEvent(TToolboxEvent* event)// override 

{
	// Maps Command-Period to the cancel item (view)
	TView* owner = dynamic_cast<TView*>(fOwner);

	if (owner && owner->IsEnabled() && event->GetText() == '.' && fCancelItem != kNoIdentifier)
	{
		TView* cancelView = owner->FindSubView(fCancelItem);
		if (cancelView)
		{
			if (cancelView->IsEnabled())
				cancelView->HandleEvent(cancelView->GetEventNumber(), owner, NULL);
		}
		else
			owner->HandleEvent(mCancelKey, owner, NULL);
	}
	else
		Inherited::DoCommandKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::PoseModally: 
//----------------------------------------------------------------------------------------


void TDialogBehavior::PoseModally()
{
	TUndoHandler::fgUndoHandler.MarkActionHistory();
	fDismissed = false;
	fDismisser = kNoIdentifier;
	
	short oldEventMask;
	
	oldEventMask = gDispatcher->GetMainEventMask();
	gDispatcher->SetMainEventMask(oldEventMask & ~highLevelEventMask);

	try
	{
		while (!fDismissed)
			gDispatcher->PollEvent(kAllowApplicationToSleep);

		gDispatcher->SetMainEventMask(oldEventMask);
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		gDispatcher->SetMainEventMask(oldEventMask);
		TUndoHandler::fgUndoHandler.ClearActionHistory(kRespectMarks);
		throw;
	}
		
	TUndoHandler::fgUndoHandler.ClearActionHistory(kRespectMarks);
}

//----------------------------------------------------------------------------------------
// TDialogBehavior::GetStandardSignature:
//----------------------------------------------------------------------------------------


IDType TDialogBehavior::GetStandardSignature() const	// override 
{
	return kStdDialogBehavior;
}

//----------------------------------------------------------------------------------------
// End of UDialogBehavior.cp


