//----------------------------------------------------------------------------------------
// UEventHandler.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UEVENTHANDLER__
#include "UEventHandler.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Toolbox

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// ANSI


#ifndef __STDLIB__
#include <stdlib.h>
#endif


//----------------------------------------------------------------------------------------
// static data members
//----------------------------------------------------------------------------------------
CPoint_AC TEventHandler::fgStdHysteresis(4, 4);
TEventHandler* TEventHandler::fgTarget;

//========================================================================================
// CLASS CHandlerIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CHandlerIterator::Reset: 
//----------------------------------------------------------------------------------------
CHandlerIterator::CHandlerIterator(const TEventHandler* chainHead) :
	fCurrent(NULL),
	fFirstHandler(NULL),
	fNextHandler(NULL)
{
	fFirstHandler = chainHead;

	First();
}

//----------------------------------------------------------------------------------------
// CHandlerIterator::First:
//----------------------------------------------------------------------------------------
TEventHandler* CHandlerIterator::First()
{
	fCurrent = const_cast<TEventHandler*>(fFirstHandler);
	if (fFirstHandler)
		fNextHandler = fFirstHandler->GetNextHandler();
	
	return fCurrent;
}

//----------------------------------------------------------------------------------------
// CHandlerIterator::Next:
//----------------------------------------------------------------------------------------
TEventHandler* CHandlerIterator::Next()
{
	if (fCurrent)
	{
		fCurrent = fNextHandler;
		if (fCurrent)
			fNextHandler = fCurrent->GetNextHandler();
	}
	
	return fCurrent;
}

//========================================================================================
// CLASS TEventHandler
//========================================================================================
MA_DEFINE_CLASS(TEventHandler);

//----------------------------------------------------------------------------------------
// TEventHandler constructor
//----------------------------------------------------------------------------------------
TEventHandler::TEventHandler(TEventHandler* itsNextHandler) :
	fNextHandler(itsNextHandler),
	fIdentifier(kNoIdentifier),
	fIdleFreq(kMaxIdleTime),
	// Assume it never wants idle time 
	fLastIdle(0),
	fEnabled(true)								// Default to taking events 
{
}

//----------------------------------------------------------------------------------------
// TEventHandler::BecameWindowTarget: 
//----------------------------------------------------------------------------------------
void TEventHandler::BecameWindowTarget()
{
}

//----------------------------------------------------------------------------------------
// TEventHandler::BecameTarget: 
//----------------------------------------------------------------------------------------
void TEventHandler::BecameTarget()
{
	HandleEvent(mBecameTarget, this, NULL);
}

//----------------------------------------------------------------------------------------
// TEventHandler::BecomeTarget: 
//----------------------------------------------------------------------------------------
bool TEventHandler::BecomeTarget()
{
	bool returnVal = true;

	TEventHandler* currentTarget = GetTarget();
	if (this != currentTarget)
	{
		long reason = currentTarget->WillingToResignTarget();

		if (reason)
		{
			currentTarget->TargetValidationFailed(reason);
			returnVal = false;	
		}
		else
		{
			currentTarget->TargetValidationSucceeded();
			SetTarget(this);
		}
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEventHandler::ResignTarget: 
//----------------------------------------------------------------------------------------
bool TEventHandler::ResignTarget()
{
	bool returnVal = false;

	TEventHandler * currentTarget = GetTarget();
	long reason = currentTarget->WillingToResignTarget();

	if (reason)
		currentTarget->TargetValidationFailed(reason);
	else
	{
		currentTarget->TargetValidationSucceeded();

		SetTarget(gDispatcher);
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEventHandler::SetEnable: 
//----------------------------------------------------------------------------------------
void TEventHandler::SetEnable(bool state)
{
	fEnabled = state;
}

//----------------------------------------------------------------------------------------
// TEventHandler::Free: 
//----------------------------------------------------------------------------------------
TEventHandler::~TEventHandler()
{
	if (gDispatcher && gDispatcher != this && GetTarget() == this)
		SetTarget(GetNextHandler() ? GetNextHandler() : gDispatcher);

	fNextHandler = NULL;
}

//----------------------------------------------------------------------------------------
// TEventHandler copy constructor
//----------------------------------------------------------------------------------------
TEventHandler::TEventHandler(const TEventHandler& inEventHandler) :
	MDependable_AC(inEventHandler),
	MStreamable(inEventHandler),

	fNextHandler(inEventHandler.fNextHandler),
	fIdentifier(inEventHandler.fIdentifier),
	fIdleFreq(inEventHandler.fIdleFreq),
	fLastIdle(inEventHandler.fLastIdle),
	fEnabled(inEventHandler.fEnabled)
{
	for(CBehaviorIterator iter(&inEventHandler); iter; ++iter)
		AddBehavior(static_cast<TBehavior*>(iter->Clone()));
}

//----------------------------------------------------------------------------------------
// TEventHandler::AddHandler: 
//----------------------------------------------------------------------------------------
TEventHandler* TEventHandler::AddHandler(TEventHandler* headOfChain)
{
	// add headOfChain to the end of the chain 
	if (fNextHandler)
		fNextHandler->AddHandler(headOfChain);
	else
		fNextHandler = headOfChain;

	return this;								// new head of chain 
}

//----------------------------------------------------------------------------------------
// TEventHandler::RemoveHandler: 
//----------------------------------------------------------------------------------------
TEventHandler* TEventHandler::RemoveHandler(TEventHandler* headOfChain)
{
	TEventHandler * prevCohandler = NULL;
	TEventHandler * currCohandler = headOfChain;
	TEventHandler * newHeadOfChain = headOfChain;

	while (currCohandler)
	{
		if (currCohandler == this)				// found it 
		{
			if (!prevCohandler)			// I was the head of the chain, so there
			// will be a new head of chain
				newHeadOfChain = fNextHandler;
			else
				prevCohandler->fNextHandler = fNextHandler;// take me out of the link 
			fNextHandler = NULL;				// remember that i'm not in the chain
			// anymore
			currCohandler = NULL;				// So loop will end 
		}
		else
		{
			prevCohandler = currCohandler;
			currCohandler = currCohandler->fNextHandler;
		}
	}

	return newHeadOfChain;
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoCommandKeyEvent: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoCommandKeyEvent(TToolboxEvent* event)
{
	if (GetNextHandler())
		GetNextHandler()->HandleCommandKey(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleCommandKey: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleCommandKey(TToolboxEvent* event)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoCommandKeyEvent(event);
	else
		DoCommandKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoCoHandlerEvent: 
//----------------------------------------------------------------------------------------
bool TEventHandler::DoCoHandlerEvent(TEvent* /*event*/)
{
	return false;
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoHelp: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoHelp(TToolboxEvent* event,
						   long& message)
{
	if (GetNextHandler())
		GetNextHandler()->DoHelp(event, message);
}

//----------------------------------------------------------------------------------------
// TEventHandler::NextIdle: 
//----------------------------------------------------------------------------------------
unsigned long TEventHandler::NextIdle()
{
	// first compute the next needed idle time based on myself
	unsigned long returnVal = kMaxIdleTime;

	if (IsEnabled() && fIdleFreq != kMaxIdleTime)// Does it idle at all? 
		returnVal = fLastIdle ? fLastIdle + fIdleFreq : TickCount();// if never idled then it needs idling now 

	// then bound it by the attached behaviors
	for(CBehaviorIterator iter(this); iter; ++iter)
		returnVal = Min_AC(returnVal, iter->NextIdle());

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleIdle: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleIdle(IdlePhase phase)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->HandleIdle(phase);

	// Does it idle at all? 
	// idleBegin and idleEnd don't respect fIdleFreq because they are
	// notifications that the application is beginning/ending idle.
	// idleContinues, however, are parceled out at the fIdleFreq.
	// ALSO: don't store the fLastIdle if I FREE'd myself or
	// the idlePhase is not idleContinue

	if ((IsEnabled() && fIdleFreq != kMaxIdleTime)
	&& (phase != idleContinue || (TickCount() - fLastIdle >= fIdleFreq))
	&& (!DoIdle(phase) && phase == idleContinue))
		fLastIdle = TickCount();
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoIdle: 
//----------------------------------------------------------------------------------------
bool TEventHandler::DoIdle(IdlePhase)
{
	return false;								// Did not free myself 
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoKeyEvent: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoKeyEvent(TToolboxEvent* event)
{
	if (GetNextHandler())
		GetNextHandler()->HandleKeyCommand(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoKeyUp: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoKeyUp(TToolboxEvent* event)
{
	if (GetNextHandler())
		GetNextHandler()->HandleKeyUp(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleKeyCommand: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleKeyCommand(TToolboxEvent* event)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoKeyEvent(event);
	else
		DoKeyEvent(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleKeyUp: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleKeyUp(TToolboxEvent* event)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoKeyUp(event);
	else
		DoKeyUp(event);
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoMenuCommand: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoMenuCommand(CommandNumber aCommandNumber)
{
	if (GetNextHandler())
		GetNextHandler()->HandleMenuCommand(aCommandNumber);
#if qDebugMsg
	else
		ProgramReport_AC(false, "No one handled the command %1d\n", aCommandNumber);
#endif
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleMenuCommand: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleMenuCommand(CommandNumber aCommandNumber)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoMenuCommand(aCommandNumber);
	else
		DoMenuCommand(aCommandNumber);
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoMultiClick: 
//----------------------------------------------------------------------------------------
bool TEventHandler::DoMultiClick(CPoint_AC lastDownPt,
									CPoint_AC newDownPt)
{
	return GetNextHandler() ? GetNextHandler()->DoMultiClick(lastDownPt, newDownPt) : 
	(abs(lastDownPt.h - newDownPt.h) <= GetStdHysteresis().h && abs(lastDownPt.v - newDownPt.v) <= GetStdHysteresis().v);
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoSetupMenus: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoSetupMenus()
{
	if (GetNextHandler())
		GetNextHandler()->HandleSetupMenus();
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleSetupMenus: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleSetupMenus()
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoSetupMenus();
	else
		DoSetupMenus();
}

//----------------------------------------------------------------------------------------
// TEventHandler::RetrieveAnEvent: 
//----------------------------------------------------------------------------------------
TEvent* TEventHandler::RetrieveAnEvent()
{
	return GetNextHandler() ? GetNextHandler()->RetrieveAnEvent() : NULL;
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoKeySelection: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoKeySelection(const CStr255_AC&/* selectionString */  )
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TEventHandler::SetIdleFreq: 
//----------------------------------------------------------------------------------------
void TEventHandler::SetIdleFreq(unsigned long newIdleFreq)
{
	fIdleFreq = newIdleFreq;
}

//----------------------------------------------------------------------------------------
// TEventHandler::RemoveBehavior: 
//----------------------------------------------------------------------------------------
void TEventHandler::RemoveBehavior(TBehavior* theBehavior)
{
	if (fBehavior)
	{
		if (fBehavior == theBehavior)
		{
			fBehavior.release();
			fBehavior = theBehavior->GetNextBehavior();
			if (fBehavior)
				fBehavior->SetPreviousBehavior(NULL);

			// remove theBehavior from the linked list
			theBehavior->SetNextBehavior(NULL);
			theBehavior->SetPreviousBehavior(NULL);
			theBehavior->SetOwner(NULL);
		}
		else
			fBehavior->RemoveBehavior(theBehavior);
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::AddBehavior: 
//----------------------------------------------------------------------------------------
void TEventHandler::AddBehavior(TBehavior* theBehavior)
{
	if (theBehavior)
	{
#if qDebug
		for(CBehaviorIterator iter(this); iter; ++iter)
			if (iter == theBehavior)
				::ProgramReport_AC("AddBehavior: Attempting to add a behavior that has already been added.");
#endif

		if (fBehavior)
			fBehavior->AppendBehavior(theBehavior);
		else
			fBehavior = theBehavior;

		theBehavior->SetOwner(this);			// make this object its owner
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::GetFirstEnabledBehavior: 
//----------------------------------------------------------------------------------------
TBehavior* TEventHandler::GetFirstEnabledBehavior() const
{
	return fBehavior ? (fBehavior->IsEnabled() ? fBehavior.get() : fBehavior->GetNextEnabledBehavior()) : NULL;
}

//----------------------------------------------------------------------------------------
// TEventHandler::GetBehaviorWithIdentifier: 
//----------------------------------------------------------------------------------------
TBehavior* TEventHandler::GetBehaviorWithIdentifier(IDType itsIdentifier) const
{
	TBehavior * returnVal = NULL;

	for(CBehaviorIterator iter(this); iter; ++iter)
		if (iter->GetIdentifier() == itsIdentifier)
		{
			returnVal = iter;
			break;
		}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEventHandler::SelectOwner: 
//----------------------------------------------------------------------------------------
void TEventHandler::SelectOwner(bool select)
{
	for(CBehaviorIterator iter(this); iter; ++iter)
		iter->SelectOwner(select);
}

//----------------------------------------------------------------------------------------
// TEventHandler::IsSelected: 
//----------------------------------------------------------------------------------------
bool TEventHandler::IsSelected()
{
	TBehavior * firstBehavior = GetFirstBehavior();

	return firstBehavior ? firstBehavior->IsOwnerSelected() : false;
}

//----------------------------------------------------------------------------------------
// TEventHandler::PostAnEvent: 
//----------------------------------------------------------------------------------------
void TEventHandler::PostAnEvent(TEvent* event)
{
	if (GetNextHandler())
		GetNextHandler()->PostAnEvent(event);
	else
	{
#if qDebug
		WrLblHexLongint("No one posted the event ", static_cast<long>(event));
#endif

		if (event->ShouldFreeOnCompletion())
			delete event;
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::PostCommand: 
//----------------------------------------------------------------------------------------
void TEventHandler::PostCommand(TCommand* command)

{
	if (command)
	{
		if (GetNextHandler())
			GetNextHandler()->PostCommand(command);
		else
		{
			// Make next best effort to post somewhere sensible. Try the dispatcher
			if (gDispatcher)
				gDispatcher->PostCommand(command);
			else
			{
#if qDebug
				WrLblHexLongint("No one posted the command ", static_cast<long>(command));
				ProgramReport_AC("\n");
#endif
				if (command->ShouldFreeOnCompletion())
					delete command;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::ResignedWindowTarget: 
//----------------------------------------------------------------------------------------
void TEventHandler::ResignedWindowTarget()

{
	HandleEvent(mResignedWindowTarget, this, NULL);
}

//----------------------------------------------------------------------------------------
// TEventHandler::GetWindow: 
//----------------------------------------------------------------------------------------
TWindow* TEventHandler::GetWindow() const

{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TEventHandler::ResignedTarget: 
//----------------------------------------------------------------------------------------
void TEventHandler::ResignedTarget()

{
	HandleEvent(mResignedTarget, this, NULL);
}

//----------------------------------------------------------------------------------------
// TEventHandler::SetTargetSelection: 
//----------------------------------------------------------------------------------------
void TEventHandler::SetTargetSelection(bool	/* redraw */)
{
}

//----------------------------------------------------------------------------------------
// TEventHandler::SetTarget: 
//----------------------------------------------------------------------------------------
void TEventHandler::SetTarget(TEventHandler* newTarget)
{
	if (!newTarget)
	{
		newTarget = gDispatcher;
#if qDebug
		::ProgramBreak_AC("In TEventHandler::SetTarget  you're setting the global target to NULL!");
#endif
	}

	if (newTarget != fgTarget)
	{
		TEventHandler* oldTarget = fgTarget;
		fgTarget = newTarget;

		if(oldTarget)
			oldTarget->ResignedTarget();
		fgTarget->BecameTarget();

		if (gDispatcher)
			gDispatcher->InvalidateMouseRegions();	// Must re-calc cursor and help regions
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::Terminate: 
//----------------------------------------------------------------------------------------
void TEventHandler::Terminate()
{
}

//----------------------------------------------------------------------------------------
// TEventHandler::ReadFrom: 
//----------------------------------------------------------------------------------------
void TEventHandler::ReadFrom(CStream_AC* aStream)
{
	MStreamable::ReadFrom(aStream);

	fIdentifier = aStream->ReadIDType();
	fEnabled = aStream->ReadBoolean();
	fIdleFreq = aStream->ReadUnsignedLong();

	short numBehaviors = aStream->ReadInteger();
	for (short count = 0; count < numBehaviors; ++count)
	{
		bool wasValid;
		TBehavior * aBehavior = dynamic_cast<TBehavior*>(aStream->ReadStreamObject(wasValid));
		if (wasValid && aBehavior)
			AddBehavior(aBehavior);
#if qDebugMsg
		else
			::ProgramBreak_AC("###Unknown behavior in TEventHandler::ReadFrom: Attempting to create a TBehavior.");
#endif
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::WriteTo: 
//----------------------------------------------------------------------------------------
void TEventHandler::WriteTo(CStream_AC* aStream) const
{
	MStreamable::WriteTo(aStream);

	aStream->WriteIDType(fIdentifier);
	aStream->WriteBoolean(fEnabled);
	aStream->WriteUnsignedLong(fIdleFreq);

	short numBehaviors = 0;
	long numPosition = aStream->GetWritePosition();	// Remember the position of the
	// count in the stream.
	aStream->WriteInteger(numBehaviors);		// Write initial num behaviors.
	TBehavior * aBehavior = GetFirstBehavior();
	while (aBehavior)
	{
		++numBehaviors;
		aStream->WriteStreamObject(aBehavior, kStandardObject);
		aBehavior = aBehavior->GetNextBehavior();
	}

	if (numBehaviors)							// if there were any behaviors...
	{
		long currentPosition = aStream->GetWritePosition();// Remember where we are. 
		aStream->SetWritePosition(numPosition);		// Go back to position of count. 
		aStream->WriteInteger(numBehaviors);	// Write the count. 
		aStream->SetWritePosition(currentPosition);	// Go back to where we were. 
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::WantsToBeTarget: 
//----------------------------------------------------------------------------------------
bool TEventHandler::WantsToBeTarget()

{
	return false;
}

//----------------------------------------------------------------------------------------
// TEventHandler::WillingToResignTarget: 
//----------------------------------------------------------------------------------------
long TEventHandler::WillingToResignTarget()

{
	return 0;									// kValidValue 
}

//----------------------------------------------------------------------------------------
// TEventHandler::KeyEventToComponents: 
//----------------------------------------------------------------------------------------
void TEventHandler::KeyEventToComponents(TToolboxEvent* event)

{
	if (GetNextHandler())
		GetNextHandler()->KeyEventToComponents(event);
	else if (event->GetWhat() == keyDown || event->GetWhat() == keyUp || event->GetWhat() == autoKey)
	{
		// Default extractions 
		event->SetKeyCode(static_cast<unsigned short>((event->GetMessage() & keyCodeMask)) >> 8);

		CStr2_AC newText(static_cast<unsigned char>(event->GetMessage() & charCodeMask));
		if (MACharacterByteType((Ptr) & newText[1], 0, event->GetKeyScript()) == smFirstByte)
			newText += static_cast<unsigned char>(event->GetMessage(1) & charCodeMask);
		event->SetText(newText);
	}
}

//----------------------------------------------------------------------------------------
// TEventHandler::TargetValidationSucceeded: 
//----------------------------------------------------------------------------------------
void TEventHandler::TargetValidationSucceeded()

{
}

//----------------------------------------------------------------------------------------
// TEventHandler::TargetValidationFailed: 
//----------------------------------------------------------------------------------------
void TEventHandler::TargetValidationFailed(long	/* reason */)
{
}

//----------------------------------------------------------------------------------------
// TEventHandler::DoEvent: 
//----------------------------------------------------------------------------------------
void TEventHandler::DoEvent(EventNumber eventNumber,
							TEventHandler* source,
							TEvent* event)
{
	if (GetNextHandler())
		GetNextHandler()->HandleEvent(eventNumber, source, event);
#if qDebugMsg
	else if (gIntenseDebugging)
	{
		ProgramReport_AC(false, "in TEventHandler.DoHandleEvent: no one handled the event: %1d\n", eventNumber);
		WrLblHexLongint(" From source: ", static_cast<long>(source));
		ProgramReport_AC("\n");
		WrLblHexLongint(" Event object: ", static_cast<long>(event));
		ProgramReport_AC("\n");
	}
#endif
}

//----------------------------------------------------------------------------------------
// TEventHandler::HandleEvent: 
//----------------------------------------------------------------------------------------
void TEventHandler::HandleEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event)
{
	TBehavior * aBehavior = GetFirstEnabledBehavior();
	if (aBehavior)
		aBehavior->DoEvent(eventNumber, source, event);
	else
		DoEvent(eventNumber, source, event);
}

//----------------------------------------------------------------------------------------
// End of UEventHandler.cp
