//----------------------------------------------------------------------------------------
// UMenuMgr.cp 
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// Theory of Operation
//
// This unit provides two features: It implements a command numbering system that is
// independent of menu/ item placement, and implements a framework for optimizing menu
// setups. The command numbering system works by assigning commands a unique command
// number, and providing a mechanism for mapping command numbers to menu/ item pairs. A
// set of routines are provide to manipulate commands via their command number rather
// than their menu and item numbers.
//
// Each command can be assigned an integer command number from 1 to 32767. To associate a
// command number with a menu item, menu resources are defined with the 'CMNU' resource
// type--not the 'MENU' type. The 'CMNU' type is just like the 'MENU' type, with an
// additional command number field for each menu item. This unit converts the 'CMNU'
// resource into a 'MENU' dynamically at run time.
//
// The items of some menus cannot be determined until run-time. The font menu is an
// example. In such cases, no command number is assigned by you. Instead, the command
// number is equal to -(256 * menu + item). The procedure CommandToMenuItem converts a
// command number to a menu id and item number by the following method:
//
// - If the command number is positive, the command table is searched for the number. If
// it is found, CommandToMenuItem returns the corre- sponding menu id and item
// number. Otherwise the menu and item are zero.
//
// - If the command number is negative, the menu number is the upper eight bits, and the
// item number the lower eight bits, of the absolute value of the command number. (This
// implies that menu id's must be <= 127 and item numbers must be <= 255.) The function
// CommandFromMenuItem converts menu/ items to command numbers by the following method:
//
// - If the item number is positive, the command table is search for the given menu/ item
// pair.  If found, the corresponding command number is returned. If not found, the number
// returned is equal to
//  -(256 * menu + item).
//  - If the item number is negative, the number returned is equal to
//  -item number.
//
// Note that CommandToMenuItem and CommandFromMenuItem work regardless of whether the
// command is actually installed in the menu bar. MacApp takes advantage of this feature
// by having a set of generic "buzz" commands, whose primary use is to set the name of the
// Undo command (e.g. 'Undo Drawing').
//
// The second feature of this unit is to optimize menu setups (changing the appearance of
// menus and items). Normally, each call to CheckItem, SetItemStyle and SetCommandIcon in
// turn calls CalcMenuSize because the width or height of the menu may have changed. If
// menu setups are done all at one time, then CalcMenuSize can be deferred until the end
// of the setup process. The procedure PerformMenuSetup implements this mechanism. It relies
// on the fact that menu setups are done all at once, and that you will call
// SetCommandName, SetItemStyle and SetCommandIcon instead of SetItem, SetItemStyle and
// SetCommandIcon. It works only for menus whose id is from 1 to mLastMenu. Menus with IDs
// greater than mLastMenu are not affected by this scheme.
//
// PerformMenuSetup accepts one parameter, a procedure which actually implements the menu
// appearance changes. SetupMenus performs the following steps:
//
// - Before calling your menu setup procedure, StartupMenuSetup is called. It disables all
// menus and items and remembers the enabled flags of each menu
//
// - Your procedure is called.It in turn calls SetCommandName, SetCmdStyle, SetCommandIcon,
// Enable, EnableCheck, or a Toolbox routine (provided it isn't SetItem, SetItemStyle or
// SetCommandIcon).
//
// - Menus with at least one enabled item are enabled and InvalMenuBar is called if the
// enabled state of any menu changed.
//----------------------------------------------------------------------------------------

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

// ACS

#ifndef __CIterator_AC__
#include "CIterator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __CPoint_AC__
#include "CPoint_AC.h"
#endif

#ifndef __CRect_AC__
#include "CRect_AC.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Use Memory_AC.h instead of <memory.h> for XPlatform compatability.
#ifndef __Memory_AC__
#include "Memory_AC.h"
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


//----------------------------------------------------------------------------------------
const bool kHierarchical = true;
const bool kNotHierarchical = !kHierarchical;

const bool kDoHierarchical = true;
const bool kDontDoHierarchical = !kDoHierarchical;

enum
{
	kEnablingManagedByMacApp = 0xFFFFFFFF
};

struct MenuCmdRecord
{
	CommandNumber theCmdNumber;
	ResNumber theMenuNumber;
	short theItemNumber;
};

typedef MenuCmdRecord *MenuCmdRecordPtr, ** MenuCmdRecordHandle;

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct CMNUHeader
{
	short							menuID;
	short							menuWidth;
	short							menuHeight;
	Handle							menuProc;
	long							enableFlags;
	CStr255_AC							menuData;	// this is variable length and the array of menu items follows it directly
};

struct CMNUItemAttributes
{
	char							icon;
	char							keyEquivalent;
	char							mark;
	char							style;
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef CMNUHeader *CMNUHeaderPtr, **CMNUHeaderHandle;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
static CompareResult_AC TestMenuResIDs(ArrayIndex_AC anItem,
									void* yourDataPtr);
static CompareResult_AC CmdToMenuItem_TestItem(ArrayIndex_AC anItem,
											void* yourDataPtr);
static char TraceMenuName(CommandNumber aCommand);
static bool IsSetupMenu(MenuRef aMenuRef,
						   bool isHierarchical);



CAutoPtr_AC<TMenuBarManager> TMenuBarManager::fgMenuBarManager;

short 		TMenuBarManager::fgMenuBarState = TMenuBarManager::kShown;
short		TMenuBarManager::fgMenuBarHeight = 0;
RgnHandle	TMenuBarManager::fgSavedGrayRgn = NULL;
RgnHandle	TMenuBarManager::fgNewGrayRgn = NULL;
RgnHandle	TMenuBarManager::fgCoveredRgn = NULL;

//----------------------------------------------------------------------------------------
// TCmdTable:
//----------------------------------------------------------------------------------------
class TCmdTable : public CSortedDynamicArray_AC
{
public:
	inline TCmdTable() : CSortedDynamicArray_AC(0, sizeof(MenuCmdRecord)) { }

	virtual ~TCmdTable();

	virtual CompareResult_AC CompareElements(const void* Element1,
										 const void* Element2) const;// override 

	virtual void CommandToMenuItem(CommandNumber theCommand,
								   MenuID& menu,
								   short& item);
	virtual CommandNumber CommandFromMenuItem(short menu,
											  short item);
	// Given a menuID,item # return the appropriate command number. 
	// If there is no such command number, return -BOR(((menu) << 8), item). 
	// If the item number is <0 then assume that it is a negative command number.

	void AddToTable(CommandNumber commandNumber,
					MenuID menuNumber,
					short itemNumber);

	void AddNewCmdToTableAndUpdateTable(CommandNumber commandNumber,
										MenuID menuNumber,
										short itemNumber);
	// This adds a menu item/command number after the menus have already been 
	// set up, and updates all the tables

	void DeleteCmdAndUpdateTable(CommandNumber theMenuItemCommandNumber);
	// This deletes a menu item/command number after the menus have already
	// been set up, and updates all the tables
};


struct MenuIDRecord
{
	ResNumber fMenuResID;
	MenuID fMenuID;
};
typedef MenuIDRecord *MenuIDRecordPtr, ** MenuIDRecordHandle;


//----------------------------------------------------------------------------------------
// TMenuIDList:
//----------------------------------------------------------------------------------------
class TMenuIDList : public CSortedDynamicArray_AC
{
public:
	inline TMenuIDList() : CSortedDynamicArray_AC(0, sizeof(MenuIDRecord)) { }

	virtual ~TMenuIDList();

	virtual CompareResult_AC CompareElements(const void* Element1,
										   const void* Element2) const;// override 

	MenuID MenuIDFromResID(ResNumber menuResID);

	void AddToTable(ResNumber menuResID,
					MenuID menuID);
};

TCmdTable pCmdTable;							// command numbers and their associated menu and item numbers
TMenuTable pMenuTable;							// list of menus created/managed by MacApp 
TMenuIDList pMenuIDList;						// list of menu IDs sorted by resource ID

//----------------------------------------------------------------------------------------

static bool gMenusAreInvalid = true;
// Used by InvalidateMenus to track whether menus need to be setup.

#if qDebug
bool gTraceSetupMenus;
#endif


//========================================================================================
// struct CGetMenuID
//========================================================================================

struct CGetMenuID
{
public:
	// Fields
	ResNumber& menuResID;
	TMenuIDList* theMenuIDList;

	// Constructor
	inline CGetMenuID(ResNumber& aMenuResID,
			   TMenuIDList* aMenuIDList) :
		menuResID(aMenuResID),
		theMenuIDList(aMenuIDList)
	{ }
};


//----------------------------------------------------------------------------------------
// TestMenuResIDs: 
//----------------------------------------------------------------------------------------


CompareResult_AC TestMenuResIDs(ArrayIndex_AC anItem,
							 void* yourDataPtr)
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	CGetMenuID * testInfo = (CGetMenuID *)yourDataPtr;
	ResNumber theMenuResIDitem = ((MenuIDRecordPtr)testInfo->theMenuIDList->ComputeAddress(anItem))->fMenuResID;

	if (theMenuResIDitem > testInfo->menuResID)
		returnVal = kItemGreaterThanCriteria_AC;
	else if (theMenuResIDitem < testInfo->menuResID)
		returnVal = kItemLessThanCriteria_AC;

	return returnVal;
}


//========================================================================================
// CLASS TMenuIDList
//========================================================================================
//----------------------------------------------------------------------------------------
// TMenuIDList destructor
//----------------------------------------------------------------------------------------


TMenuIDList::~TMenuIDList()
{
}

//----------------------------------------------------------------------------------------
// TMenuIDList::CompareElements: 
//----------------------------------------------------------------------------------------


CompareResult_AC TMenuIDList::CompareElements(const void* Element1,
										   const void* Element2) const// override 
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	ResNumber theResNumber1 = ((MenuIDRecordPtr)Element1)->fMenuResID;
	ResNumber theResNumber2 = ((MenuIDRecordPtr)Element2)->fMenuResID;

	if (theResNumber1 > theResNumber2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (theResNumber1 < theResNumber2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TMenuIDList::AddToTable: 
//----------------------------------------------------------------------------------------


void TMenuIDList::AddToTable(ResNumber menuResID,
							 MenuID menuID)
{
	MenuIDRecord arec;

	arec.fMenuResID = menuResID;
	arec.fMenuID = menuID;

	InsertElementInOrder((Ptr) & arec);
}

//----------------------------------------------------------------------------------------
// TMenuIDList::MenuIDFromResID: 
//----------------------------------------------------------------------------------------


MenuID TMenuIDList::MenuIDFromResID(ResNumber menuResID)
{
	MenuID returnVal = 0;

	ArrayIndex_AC index;
	CGetMenuID aCGetMenuID(menuResID, this);

	if (DoSearchElement(&TestMenuResIDs, &aCGetMenuID, index))
	{
		MenuIDRecordPtr aMenuIDListPtr = (MenuIDRecordPtr)ComputeAddress(index);
		returnVal = aMenuIDListPtr->fMenuID;
	}

	return returnVal;
}


//========================================================================================
// CLASS TCmdTable
//========================================================================================
//----------------------------------------------------------------------------------------
// TCmdTable destructor
//----------------------------------------------------------------------------------------


TCmdTable::~TCmdTable()
{
}

//----------------------------------------------------------------------------------------
// TCmdTable::CompareElements: 
//----------------------------------------------------------------------------------------


CompareResult_AC TCmdTable::CompareElements(const void* Element1,
										 const void* Element2) const// override 
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	CommandNumber theCommanditem1 = ((MenuCmdRecordPtr)Element1)->theCmdNumber;
	CommandNumber theCommanditem2 = ((MenuCmdRecordPtr)Element2)->theCmdNumber;

	if (theCommanditem1 > theCommanditem2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (theCommanditem1 < theCommanditem2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//--------------------------------------------------------------------------------------------------

class CCmdToMenuItem
{
public:
	CommandNumber& theCommand;
	TCmdTable* theCmdTable;

	inline CCmdToMenuItem(CommandNumber& aCommand,
				   TCmdTable* aCmdTable) :
		theCommand(aCommand),
		theCmdTable(aCmdTable)
	{ }
};


CompareResult_AC CmdToMenuItem_TestItem(ArrayIndex_AC anItem,
									 void* yourDataPtr)
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	CCmdToMenuItem * testInfo = (CCmdToMenuItem *)yourDataPtr;
	CommandNumber theCommanditem;

	theCommanditem = ((MenuCmdRecordPtr)testInfo->theCmdTable->ComputeAddress(anItem))->theCmdNumber;

	if (theCommanditem > testInfo->theCommand)
		returnVal = kItemGreaterThanCriteria_AC;
	else if (theCommanditem < testInfo->theCommand)
		returnVal = kItemLessThanCriteria_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCmdTable::CommandToMenuItem: 
//----------------------------------------------------------------------------------------


void TCmdTable::CommandToMenuItem(CommandNumber theCommand,
								  MenuID& menu,
								  short& item)
{
	ArrayIndex_AC index = 0;
	MenuCmdRecordPtr aMenuCmdRecordPtr;
	CCmdToMenuItem aCCmdToMenuItem(theCommand, this);

	if (theCommand < 0)							/* negative commands are mapped not table based */
	{
		menu = static_cast<short>((-theCommand) >> 8);
		item = static_cast<short>((-theCommand) & 255);
	}
	else
	{
		if (DoSearchElement(&CmdToMenuItem_TestItem, &aCCmdToMenuItem, index))
		{
			do
			{
				--index;
				if (!index)
					break;

				aMenuCmdRecordPtr = (MenuCmdRecordPtr)ComputeAddress(index);
			} while (aMenuCmdRecordPtr->theCmdNumber == theCommand);

			++index;

			do
			{
				aMenuCmdRecordPtr = (MenuCmdRecordPtr)ComputeAddress(index);
				if (aMenuCmdRecordPtr->theCmdNumber == theCommand)
				{
					menu = aMenuCmdRecordPtr->theMenuNumber;
					item = aMenuCmdRecordPtr->theItemNumber;
					if (::GetMenuHandle(menu))
						break;
				}
				++index;
			} while (aMenuCmdRecordPtr->theCmdNumber == theCommand);
		}
		else									// not found
			{
				menu = 0;
				item = 0;
			}
	}
}

//----------------------------------------------------------------------------------------
// TCmdTable::CommandFromMenuItem: 
//----------------------------------------------------------------------------------------


CommandNumber TCmdTable::CommandFromMenuItem(short menu,
											 short item)

// Given a menuID/item # return the appropriate command number. if there
// is no such command number, return -BOR(((menu) << 8), item).  If the
// item number is <0 then assume that it is a negative command number.
{
	CommandNumber returnVal = cNoCommand;

	if (item < 0)
		returnVal = -item;
	else
	{
		if (item > 0)
			// Search the table linearly. 
			for (ArrayIndex_AC i = 1; i <= GetSize(); ++i)
			{
				MenuCmdRecordPtr aMenuCmdRecordPtr = (MenuCmdRecordPtr)ComputeAddress(i);
				if ((menu == aMenuCmdRecordPtr->theMenuNumber) && (item == aMenuCmdRecordPtr->theItemNumber))
				{
					returnVal = aMenuCmdRecordPtr->theCmdNumber;
					break;
				}
			}

		if (returnVal == cNoCommand)
		{
#if qDebugMsg
			if ((menu > 127) || (item > 255))
			{
				ProgramReport_AC(false, "menu : %1d , item : %1d \n", menu, item);
				ProgramReport_AC(false, "Menu/item number is too big for a negative command number!\n");
				::ProgramBreak_AC("Try using a negative item number instead.");
			}
#endif
			returnVal = -((menu << 8) | item);
		}
	}
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCmdTable::AddToTable: 
//----------------------------------------------------------------------------------------


void TCmdTable::AddToTable(CommandNumber commandNumber,
						   MenuID menuNumber,
						   short itemNumber)
{
	MenuCmdRecord arec;

	if (commandNumber > 0)						// Negative numbers are mapped not tabled 
	{
		arec.theCmdNumber = commandNumber;
		arec.theMenuNumber = menuNumber;
		arec.theItemNumber = itemNumber;

		InsertElementInOrder((Ptr) & arec);
	}
}


//----------------------------------------------------------------------------------------
// TCmdTable::AddNewCmdToTableAndUpdateTable:
//----------------------------------------------------------------------------------------


void TCmdTable::AddNewCmdToTableAndUpdateTable(CommandNumber commandNumber,
											   MenuID menuNumber,
											   short itemNumber)
{
	MenuCmdRecord arec;

	for (ArrayIndex_AC i = 1; i <= GetSize(); ++i)
	{
		GetElementsAt(i, &arec, 1);

		if ((arec.theMenuNumber == menuNumber) && (arec.theItemNumber >= itemNumber))
		{
			// If we have found an item in the same menu as we are adding to and it is
			// the same index or greater then bump its itemNumber
			++arec.theItemNumber;

			// Write it back out
			ReplaceElementsAt(i, &arec, 1);
		}
	}

	// Insert the new menu item
	arec.theCmdNumber = commandNumber;
	arec.theMenuNumber = menuNumber;
	arec.theItemNumber = itemNumber;

	InsertElementInOrder((Ptr) & arec);
}

//----------------------------------------------------------------------------------------
// TCmdTable::DeleteCmdAndUpdateTable:
//----------------------------------------------------------------------------------------


void TCmdTable::DeleteCmdAndUpdateTable(CommandNumber theMenuItemCommandNumber)
{
	MenuCmdRecord arec;
	short menuNumber;
	short itemNumber;

	CommandToMenuItem(theMenuItemCommandNumber, menuNumber, itemNumber);

	for (ArrayIndex_AC i = 1; i <= GetSize(); ++i)
	{
		GetElementsAt(i, &arec, 1);

		if (arec.theMenuNumber == menuNumber)
		{
			if (arec.theItemNumber == itemNumber)
			{
				// Delete the element
				DeleteElementsAt(i, 1);

				// Back up one element
				--i;
			}
			else if (arec.theItemNumber > itemNumber)
			{
				// If we have found an item in the same menu as we are deleting from and its
				// index is greater then decrement its itemNumber
				--arec.theItemNumber;

				// Write it back out
				ReplaceElementsAt(i, &arec, 1);
			}
		}
	}
}


//========================================================================================
// CLASS TMenuTable
//========================================================================================
//----------------------------------------------------------------------------------------
// TMenuTable destructor
//----------------------------------------------------------------------------------------


TMenuTable::~TMenuTable()
{
}

//----------------------------------------------------------------------------------------
// TMenuTable::GetMenu: 
//----------------------------------------------------------------------------------------


MenuRef TMenuTable::GetMenu(MenuID menuID)
{
	ArrayIndex_AC index;

	return (MenuRef)DoSearch(&TestForMenuID, &menuID, index);// discard index 
}

//----------------------------------------------------------------------------------------
// TMenuTable::Compare: 
//----------------------------------------------------------------------------------------


CompareResult_AC TMenuTable::Compare(const Handle item1,
								  const Handle item2) const	// override 
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;

	MenuID menuID1 = (*((MenuRef)item1))->menuID;
	MenuID menuID2 = (*((MenuRef)item2))->menuID;

	if (menuID1 > menuID2)
		returnVal = kItem1GreaterThanItem2_AC;
	else if (menuID1 < menuID2)
		returnVal = kItem1LessThanItem2_AC;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TestForMenuID: 
//----------------------------------------------------------------------------------------


CompareResult_AC TMenuTable::TestForMenuID(Handle anItem,
										void* yourDataPtr) // static
{
	CompareResult_AC returnVal = kItemEqualCriteria_AC;

	MenuID * menuID = (MenuID *)yourDataPtr;

	short menuIDitem = (*((MenuRef)anItem))->menuID;

	if (menuIDitem > *menuID)
		returnVal = kItemGreaterThanCriteria_AC;
	else if (menuIDitem < *menuID)
		returnVal = kItemLessThanCriteria_AC;

	return returnVal;
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
#undef Inherited

#if qDebugMsg

//----------------------------------------------------------------------------------------
// TraceMenuName: 
//----------------------------------------------------------------------------------------


char TraceMenuName(CommandNumber aCommand)
// For debugging purposes only--used to dump the name and number of a command 
{
	ProgramReport_AC(false, " %1d [%s],", aCommand, (const char*)(CStr255_AC(CommandToName(aCommand))));

	return ' ';
}

#endif

//----------------------------------------------------------------------------------------
// InvalidateMenus: 
//----------------------------------------------------------------------------------------


void InvalidateMenus()
{
	gMenusAreInvalid = true;
}

//----------------------------------------------------------------------------------------


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

struct MenuBarRecord
{
	ResNumber nMenus;
	ResNumber menuResID[1000];
};

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

//----------------------------------------------------------------------------------------
// AddMenuBar: 
//----------------------------------------------------------------------------------------
typedef MenuBarRecord* MenuBarPtr, ** MenuBarHandle;

void AddMenuBar(ResNumber itsID,
				bool isHierarchical)
{
	short hier = 0;
	MenuBarHandle hmbar;

	if (isHierarchical)
		hier = -1;

	hmbar = (MenuBarHandle)GetResource('MBAR', itsID);
	if (hmbar)
	{
		for (ResNumber i = 0; i < (*hmbar)->nMenus; ++i)
		{
			ResNumber menuResID = (*hmbar)->menuResID[i];
			MenuID menuID = pMenuIDList.MenuIDFromResID(menuResID);
			if (!::GetMenuHandle(menuID))		// do nothing if it's already in menubar
			{
				MenuRef aMenu = MAGetMenu(menuResID);
				if (aMenu)
				{
					MAInsertMenu(aMenu, menuResID, hier);

					// a visible menu needed insertion thus the menubar became invalid
					if (!isHierarchical)
						InvalidateMenuBar();
				}
			}
		}

		ReleaseResource((Handle)hmbar);
		// hmbar = NULL;
	}
}

//----------------------------------------------------------------------------------------
// GetIndMenu: 
//----------------------------------------------------------------------------------------

MenuRef GetIndMenu(ResNumber itsID,
				   ResNumber itsIndex)
{
	MenuRef theMenu = NULL;

	MenuBarHandle hmbar = (MenuBarHandle)GetResource('MBAR', itsID);
	if (hmbar)
	{
		ResNumber menuResID = (*hmbar)->menuResID[itsIndex - 1];
		MenuID menuID = pMenuIDList.MenuIDFromResID(menuResID);
		theMenu = MAGetMenu(menuResID);
		ReleaseResource((Handle)hmbar);
	}

	return theMenu;
}

//----------------------------------------------------------------------------------------
// CountMenus: 
//----------------------------------------------------------------------------------------

short CountMenus(ResNumber itsID)
{
	short numMenus = 0;

	MenuBarHandle hmbar = (MenuBarHandle)GetResource('MBAR', itsID);
	if (hmbar)
	{
		numMenus = (*hmbar)->nMenus;
		ReleaseResource((Handle)hmbar);
	}

	return numMenus;
}

//----------------------------------------------------------------------------------------
// ValidateMenus: 
//----------------------------------------------------------------------------------------


void ValidateMenus()
{
	gMenusAreInvalid = false;
}

//----------------------------------------------------------------------------------------
// MenusHavePendingUpdate: 
//----------------------------------------------------------------------------------------


bool MenusHavePendingUpdate()
{
	return gMenusAreInvalid;
}

//----------------------------------------------------------------------------------------
// InvalidateMenuBar: 
//----------------------------------------------------------------------------------------


void InvalidateMenuBar()
{
	InvalidateMenus();							// if the menubar is invalidated then the
												// menu items must be also

	// Systems have "InvalMenuBar"... use that functionality. The system will redraw
	// the menu bar at the next convenient time.

	InvalMenuBar();
}

//----------------------------------------------------------------------------------------
// CommandEnabled: 
//----------------------------------------------------------------------------------------


bool CommandEnabled(CommandNumber command)
{
	bool returnVal = false;

	MenuID menuNo;
	short itemNo;
	MenuRef theMenu = CommandToComponents(command, menuNo, itemNo);

	if (theMenu)
		returnVal = itemNo > 0 && itemNo < 32 ? (((*theMenu)->enableFlags >> itemNo) & 0x01) != 0 : true;

	return returnVal;
}

//----------------------------------------------------------------------------------------
// CommandFromMenuItem: 
//----------------------------------------------------------------------------------------


CommandNumber CommandFromMenuItem(short menu,
								  short item)
{
	return pCmdTable.CommandFromMenuItem(menu, item);
}

//----------------------------------------------------------------------------------------
// CommandToMenuItem: 
//----------------------------------------------------------------------------------------


void CommandToMenuItem(CommandNumber aCommand,
					   MenuID& menu,
					   short& item)
{
	pCmdTable.CommandToMenuItem(aCommand, menu, item);
}

//----------------------------------------------------------------------------------------
// CommandToName: 
//----------------------------------------------------------------------------------------


CStr255_AC CommandToName(CommandNumber aCommand)
{
	CStr255_AC theText;

	short item;
	ResNumber aMenu;
	MenuRef itsMenuRef = CommandToComponents(aCommand, aMenu, item);
	if (itsMenuRef)
	{
		if (item)
			GetMenuItemText(itsMenuRef, item, theText);
		else
			// Toolbox has no interface for retrieving the title of a menu!!
			theText = (*static_cast<MenuHandle>(itsMenuRef))->menuData;
	}
		
	return theText;
}

//----------------------------------------------------------------------------------------
// CommandToComponents: 
//----------------------------------------------------------------------------------------


MenuRef CommandToComponents(CommandNumber command,
							MenuID& menuNo,
							short& itemNo)
{
	MenuRef returnVal = NULL;

	CommandToMenuItem(command, menuNo, itemNo);
	if (menuNo)									// was found 
		returnVal = MAGetMenuRef(menuNo);

	return returnVal;
}

//----------------------------------------------------------------------------------------
// Enable: 
//----------------------------------------------------------------------------------------

void Enable(CommandNumber aCommand,
			bool canDo)
{
#if qDebugMsg
	if (gTraceSetupMenus)
		ProgramReport_AC(false, "..... Enable(%c%s)\n", TraceMenuName(aCommand), (const char*)(gBoolString[canDo]));
#endif

	MenuID menu;
	short item;

	MenuRef aMenuRef = CommandToComponents(aCommand, menu, item);
	if (aMenuRef)
		if (canDo)
			EnableItem(aMenuRef, item);
		else
			DisableItem(aMenuRef, item);
}

//----------------------------------------------------------------------------------------
// EnableCheck: 
//----------------------------------------------------------------------------------------

void EnableCheck(CommandNumber aCommand,
				 bool canDo,
				 bool checkIt)
{
#if qDebugMsg
	if (gTraceSetupMenus)
		ProgramReport_AC(false, "..... EnableCheck(%c%s, %s)\n", TraceMenuName(aCommand), (const char*)(gBoolString[canDo]), (const char*)(gBoolString[checkIt]));
#endif

	MenuID menu;
	short item;

	MenuRef aMenuRef = CommandToComponents(aCommand, menu, item);
	if (aMenuRef)
	{
		if (canDo)
			EnableItem(aMenuRef, item);
		else
			DisableItem(aMenuRef, item);
		CheckItem(aMenuRef, item, checkIt);
	}
}

//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// GetResMenu: Allow us to get a menu when it's not available via ::GetMenuHandle.
//----------------------------------------------------------------------------------------
MenuRef GetResMenu(ResNumber menuResID)
{
	CTempUseResFile_AC tempUseResFile(gApplicationRefNum);

	return (MenuRef)GetResource('MENU', menuResID);
}

//----------------------------------------------------------------------------------------
// ConvertToMenu: 
//----------------------------------------------------------------------------------------


MenuRef ConvertToMenu(Handle CMNUHandle, bool addCommandsToCommandTable)
{
	CTempHandleLock_AC tempHandleLock(CMNUHandle);
	CMNUHeader* theCMNUHeader = reinterpret_cast<CMNUHeaderPtr>(StripLong(*CMNUHandle));

	MenuID menuNo = theCMNUHeader->menuID;
	
	MenuRef theMenuRef = MANewIconMenu(menuNo, theCMNUHeader->menuData);

	if (theCMNUHeader->menuProc) // it is specialized
		(*theMenuRef)->menuProc = theCMNUHeader->menuProc;
		
	if (theCMNUHeader->enableFlags != kEnablingManagedByMacApp) // it is specialized
		(*theMenuRef)->enableFlags = theCMNUHeader->enableFlags;

	Ptr theCMNUPtr = reinterpret_cast<Ptr>(&theCMNUHeader->menuData) + theCMNUHeader->menuData.Length() + 1;

	short itemNo = 0;

	Ptr endPtr = reinterpret_cast<Ptr>(theCMNUHeader) + GetHandleSize(CMNUHandle);
	while (theCMNUPtr < endPtr && !(*reinterpret_cast<CStr255_AC*>(theCMNUPtr)).IsEmpty())
	{
		++itemNo;
		AppendMenu(theMenuRef, "\p "); // Must be at least non-empty
		
		// Set the text this way to defeat metacharacter processing
		SetMenuItemText(theMenuRef, itemNo, *reinterpret_cast<CStr255_AC *>(theCMNUPtr));
		theCMNUPtr += (*reinterpret_cast<CStr255_AC *>(theCMNUPtr)).Length() + 1;

		CMNUItemAttributes& theCMNUAttributes = *(CMNUItemAttributes*)theCMNUPtr;		
		if (theCMNUAttributes.icon)
			SetItemIcon(theMenuRef, itemNo, theCMNUAttributes.icon);

		if (theCMNUAttributes.keyEquivalent)
			SetItemCmd(theMenuRef, itemNo, theCMNUAttributes.keyEquivalent);

		if (theCMNUAttributes.mark)
			SetItemMark(theMenuRef, itemNo, theCMNUAttributes.mark);

		if (theCMNUAttributes.style)
			SetItemStyle(theMenuRef, itemNo, theCMNUAttributes.style);

		theCMNUPtr += sizeof(CMNUItemAttributes);

		if (::Odd_AC(theCMNUPtr))// word align 
			++theCMNUPtr;

		if (addCommandsToCommandTable)
			pCmdTable.AddToTable(*reinterpret_cast<CommandNumber*>(theCMNUPtr), menuNo, itemNo);

		theCMNUPtr += sizeof(CommandNumber);
	}

	return theMenuRef;
}

MenuRef MANewIconMenu(short menuID, ConstStr255Param menuTitle)
{
	/*
		Looks for a menu title of "#nnn" where nnn is the id of an icon suite, 
		represented in text form so it is easy to enter in a resource editor. 
		If the icon suite can be loaded, the menu is created with the icon for
		its title. Otherwise, the menu is created with a normal title string. 
	*/

	MenuRef theMenuRef = NULL;

	// Could be an icon suite resource id
	if ((menuTitle[0] > 1) && (menuTitle[1] == '#'))
	{
		// Convert to a number
		Str15 iconMenuTitle;
		iconMenuTitle[0] = menuTitle[0] - 1;
		if (iconMenuTitle[0] > 15)
			iconMenuTitle[0] = 15;
		BlockMove_AC(&menuTitle[2], &iconMenuTitle[1], iconMenuTitle[0]);
		long iconID = 0;
		StringToNum(iconMenuTitle, &iconID);

		// Load the icon suite
		Handle iconSuite = NULL;
		::GetIconSuite(&iconSuite, iconID, svAllSmallData);
		if (iconSuite)
		{
			// Create the menu with an icon title
			iconMenuTitle[0] = 5;
			iconMenuTitle[1] = 0x01;
			*(long *)&iconMenuTitle[2] = (long)iconSuite;
			theMenuRef = NewMenu(menuID, iconMenuTitle);
		}
	}

	// Create the menu with a text title
	if (!theMenuRef)
		theMenuRef = NewMenu(menuID, menuTitle);

	return theMenuRef;
}

//----------------------------------------------------------------------------------------
// InitUMenuMgr: 
//----------------------------------------------------------------------------------------


void InitUMenuMgr()
{
	// initialize all CMNUs with positive id
	short num = CountResources('CMNU');
	::ThrowIfResError_AC();
	short i;
	for (i = 1; i <= num; ++i)
	{
		Handle aCMNUHandle = GetIndResource('CMNU', i);
		::ThrowIfNULLResource_AC(aCMNUHandle);
		HNoPurge(aCMNUHandle);					// If it was purgeable it might squirt away

		ResNumber menuResID;
		ResType menuResourceType;
		CStr255_AC menuResourceName;
		GetResInfo(aCMNUHandle, &menuResID, &menuResourceType, menuResourceName);

		short menuID = (*(CMNUHeaderHandle)aCMNUHandle)->menuID;
		if (menuID > 0)// not in reserved range
		{
			// load the menudef and store its reference in the menu 
			MenuRef newMenu = ConvertToMenu(aCMNUHandle);
			Handle menuProc = (*(CMNUHeaderHandle)aCMNUHandle)->menuProc;
			if (menuProc) // Was it specialized?
			{
				Handle h = GetResource('MDEF', *reinterpret_cast<short*>(&menuProc));
				(*newMenu)->menuProc = h;
			}

			MAInsertInMenuTable(newMenu, menuResID);
			ReleaseResource(aCMNUHandle);		// won't be needing this since it's we have a MENU now
		}
		else
			ReleaseResource(aCMNUHandle);		// Goodbye!
	}

	// initialize all MENUs with positive id
	num = CountResources('MENU');
	::ThrowIfResError_AC();
	for (i = 1; i <= num; ++i)
	{
		MenuRef newMenu = (MenuRef)GetIndResource('MENU', i);
		if (newMenu)
		{
			ResNumber menuResID;
			ResType menuResourceType;
			CStr255_AC menuResourceName;
			GetResInfo((Handle)newMenu, &menuResID, &menuResourceType, menuResourceName);

			THz itsZone = HandleZone((Handle)newMenu);
			// not a System menu and not in reserved range
			if ((itsZone != SystemZone()) && ((*newMenu)->menuID) > 0)
			{
				// load the menudef and store its reference in the menu 
				Handle h = GetResource('MDEF', *reinterpret_cast<short*>(&(*newMenu)->menuProc));
				(*newMenu)->menuProc = h;

				MAInsertInMenuTable(newMenu, menuResID);
			}
		}
	}

#if qDebugMsg
	if (cUndo - cEditBase != kSysUndo)
		ProgramReport_AC("Invalid UNDO command number\n");
	if (cCut - cEditBase != kSysCut)
		ProgramReport_AC("Invalid CUT command number\n");
	if (cCopy - cEditBase != kSysCopy)
		ProgramReport_AC("Invalid COPY command number\n");
	if (cPaste - cEditBase != kSysPaste)
		ProgramReport_AC("Invalid PASTE command number\n");
	if (cClear - cEditBase != kSysClear)
		ProgramReport_AC("Invalid CLEAR command number\n");
#endif
}

//----------------------------------------------------------------------------------------
// IsManagedMenu: 
//----------------------------------------------------------------------------------------


bool IsManagedMenu(MenuRef theMenu)
{
	return pMenuTable.GetIdentityItemNo((Handle)theMenu) != kEmptyIndex_AC;
}

//----------------------------------------------------------------------------------------
// MAGetMenuRef: 
//----------------------------------------------------------------------------------------


MenuRef MAGetMenuRef(MenuID menuID)
{
	MenuRef theMenu = ::GetMenuHandle(menuID);	// do this first ... the menu is likely to
												// be in the menubar and more likely of
												// success, hence, faster

	if (!theMenu)						// Try the menu list 
		theMenu = pMenuTable.GetMenu(menuID);

	return theMenu;
}

//----------------------------------------------------------------------------------------
// MAGetMenu: 
//----------------------------------------------------------------------------------------


MenuRef MAGetMenu(ResNumber menuResID)
{
	MenuID menuID = pMenuIDList.MenuIDFromResID(menuResID);

	MenuRef theMenu = MAGetMenuRef(menuID);		// First see if the menu is resident

	if (!theMenu)						// darn! Try the resource chain 
		theMenu = GetResMenu(menuResID);

	return theMenu;
}

//----------------------------------------------------------------------------------------
// MAGetNewMBar: 
//----------------------------------------------------------------------------------------


Handle MAGetNewMBar(ResNumber menuRsrcID)
{
	MCTableHandle theColorTab;
	Handle itsRsrcHandle;
	SignedByte savedState;
	Handle itsNewMBar;

	if (qNeedsColorQD || HasColorQD_AC())
		theColorTab = GetMCInfo();

	// make the 'MBAR' resource non-purgeable, so the Toolbox doesn't die 
	itsRsrcHandle = GetResource('MBAR', menuRsrcID);
	if (itsRsrcHandle)
	{
		savedState = HGetState(itsRsrcHandle);
		HNoPurge(itsRsrcHandle);
	}

	itsNewMBar = GetNewMBar(menuRsrcID);

	// restore the 'MBAR' resource 
	if (itsRsrcHandle)
		HSetState(itsRsrcHandle, savedState);

	if (theColorTab && (qNeedsColorQD || HasColorQD_AC()))
	{
		{ // Extra block to destroy handlelocker before DisposeMCInfo
			CTempHandleLock_AC tempHandleLock((Handle)theColorTab);
			SetMCEntries(static_cast<short>(GetHandleSize((Handle)theColorTab) / sizeof(MCEntry)), (*theColorTab));
		}
		DisposeMCInfo(theColorTab);
	}

	return itsNewMBar;
}

//----------------------------------------------------------------------------------------
// MAInsertInMenuTable: 
//----------------------------------------------------------------------------------------


void MAInsertInMenuTable(MenuRef theMenu,
						 ResNumber menuResID)
{
	pMenuTable.Insert((Handle)theMenu);

	// Add the menu to the translation table
	pMenuIDList.AddToTable(menuResID, (*theMenu)->menuID);
}

//----------------------------------------------------------------------------------------
// MAInsertMenu: 
//----------------------------------------------------------------------------------------


void MAInsertMenu(MenuRef theMenu,
				  ResNumber menuResID,
				  MenuID beforeID)
{
	MenuCRsrcHandle theColorRsrc;

	InsertMenu(theMenu, beforeID);

	// Since only GetMenu automatically loads the appropriate color information,
	// and (sigh) since we can only call GetMenu once, and if you call DeleteMenu
	// all the good color stuff goes away (double sigh) we'll have to help out
	// the Menu Manager by doing its job for it

	if (qNeedsColorQD || HasColorQD_AC())
	{
		// n.b. This is using the menuID of the menu as a resource ID, which is incorrect
		theColorRsrc = (MenuCRsrcHandle)GetResource('mctb', menuResID);
		if (theColorRsrc)
		{
			{ // Extra block to destroy handlelocker before releasersrc
				CTempHandleLock_AC tempHandleLock((Handle)theColorRsrc);
				SetMCEntries((*theColorRsrc)->numEntries, &((*theColorRsrc)->mcEntryRecs[0]));
			}
			ReleaseResource((Handle)theColorRsrc);
		}
	}
}

//----------------------------------------------------------------------------------------
// If hierarchical the range of IDs for applications is restricted.  See IM V-236.
// If the menu isn't managed by MacApp and is in a TPopup, its ID must be outside this range.
const ResNumber kHierarchicalMin = 0;
const ResNumber kHierarchicalMax = 235;

//----------------------------------------------------------------------------------------
// IsSetupMenu: 
//----------------------------------------------------------------------------------------


bool IsSetupMenu(MenuRef aMenuRef,
					bool isHierarchical)
{
	bool returnVal = false;

#if qDebug
	if (!IsHandle_AC((Handle)aMenuRef))
	{
		VerboseIsHandle((Handle)aMenuRef);
		::ProgramBreak_AC("In IsSetupMenu: not handed a handle.");
		return false;
	}
#endif

	MenuID menuID = (*aMenuRef)->menuID;

	if (menuID == mApple)						// _NEVER_ managed! 
		;
	
#if qDebug
	else if (menuID == mDebug)
		returnVal = true;
#endif

	else if (isHierarchical && menuID >= kHierarchicalMin && menuID <= kHierarchicalMax)
		returnVal = true;						// must be in valid range for hierarchicals
	else
		returnVal = menuID >= mFirstMenu && menuID <= mLastMenu;// Range of managed menus
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// CSetupMenus:
//----------------------------------------------------------------------------------------

const short kMaxSetupMenus = mLastMenu + 1;

class CSetupMenus
{
public:
	inline CSetupMenus() :
	fCount(0)
	{ }

	void StartMenuSetup(MenuRef aMenuRef);
	void EndMenuSetup();

protected:
	MenuRef fMenuRef[kMaxSetupMenus];
	bool fEnabled[kMaxSetupMenus];
	short fCount;
};

//----------------------------------------------------------------------------------------
// CSetupMenus::StartMenuSetup: 
//----------------------------------------------------------------------------------------


void CSetupMenus::StartMenuSetup(MenuRef aMenuRef)
{
#if qDebug
	if (fCount >= kMaxSetupMenus)
	{
		::ProgramBreak_AC("Array overflow: too many setup menus.");
		return;
	}
#endif

	// Remember if the menu itself was enabled. 
	fMenuRef[fCount] = aMenuRef;
	fEnabled[fCount] = (bool)((*aMenuRef)->enableFlags & 1);
	++fCount;

	// Disable the menu and all of its items. 
	(*aMenuRef)->enableFlags = 0;

	// Uncheck all items.
	short nItems = CountMItems(aMenuRef);
	for (short item = 1; item <= nItems; ++item)// Make sure we don't check items with sub-menus 
	{
		short theCmd;
		GetItemCmd(aMenuRef, item, &theCmd);
		if (theCmd != hMenuCmd)
			CheckItem(aMenuRef, item, false);	// moves/purges memory 
	}
}

//----------------------------------------------------------------------------------------
// CSetupMenus::EndMenuSetup: 
//----------------------------------------------------------------------------------------

void CSetupMenus::EndMenuSetup()
{
	for (short i = 0; i < fCount; ++i)
	{
		MenuRef aMenuRef = fMenuRef[i];

		long newFlags = (*aMenuRef)->enableFlags;
		// If any items are enabled, enable the menu 
		if (newFlags)
		{
			newFlags |= 1;
			(*aMenuRef)->enableFlags = newFlags;
		}

		// If the menu's enabled state changed, we have to draw the menu bar. 
		MenuID menuID = (*aMenuRef)->menuID;
		if (((newFlags & 1) == 1) != fEnabled[i])
			InvalMenuBar();
	}
}

//----------------------------------------------------------------------------------------
// PerformMenuSetup: 
//----------------------------------------------------------------------------------------

void PerformMenuSetup(MenuSetupType TheMenuSetterUpper,
					  void* yourDataPtr)
{
	CSetupMenus setup;

	{											// in a block to destroy the iterator
		for (CMenuIterator iter; iter.Current(); ++iter)
			if (IsSetupMenu(iter, kHierarchical))
				setup.StartMenuSetup(iter);
	}

	TheMenuSetterUpper(yourDataPtr);

	setup.EndMenuSetup();

	ValidateMenus();
}

//----------------------------------------------------------------------------------------
// SetCommandIcon: 
//----------------------------------------------------------------------------------------


void SetCommandIcon(CommandNumber aCommand,
					Byte menuIcon)
{
#if qDebugMsg
	if (gTraceSetupMenus)
		ProgramReport_AC(false, "..... SetCommandIcon(%c%d)\n", TraceMenuName(aCommand), menuIcon);
#endif

	MenuID menuID;
	short item;

	MenuRef aMenuRef = CommandToComponents(aCommand, menuID, item);
	if (aMenuRef)
		SetItemIcon(aMenuRef, item, menuIcon);
}

//----------------------------------------------------------------------------------------
// SetCommandName: 
//----------------------------------------------------------------------------------------


void SetCommandName(CommandNumber aCommand,
					const CStr255_AC& menuText)
{
#if qDebugMsg
	if (gTraceSetupMenus)
		ProgramReport_AC(false, "..... SetCommandName(%c'%s')\n", TraceMenuName(aCommand), menuText);
#endif

#if qDebug
	if (menuText.IsEmpty())
		::ProgramReport_AC("SetCommandName: empty menuText - may corrupt the menu handle!");
#endif

	MenuID menuID;
	short item;

	MenuRef aMenuRef = CommandToComponents(aCommand, menuID, item);
	if (aMenuRef)
		SetMenuItemText(aMenuRef, item, menuText);
}

//----------------------------------------------------------------------------------------
// SetIndividualCommandName: 
//----------------------------------------------------------------------------------------


void SetIndividualCommandName(CommandNumber aCommand,
							  ResNumber rsrcID,
							  short strIndex)
{
	SetCommandName(aCommand, CStr255_AC(rsrcID, strIndex));
}

//----------------------------------------------------------------------------------------
// SetMenuState: 
//----------------------------------------------------------------------------------------


void SetMenuState(CommandNumber aCommand,
				  ResNumber rsrcID,
				  short falseBuzzItem,
				  short trueBuzzItem,
				  bool stateVariable)
{
	SetIndividualCommandName(aCommand, rsrcID, stateVariable ? trueBuzzItem : falseBuzzItem);
}

//----------------------------------------------------------------------------------------
// SetStyle: 
//----------------------------------------------------------------------------------------


void SetStyle(CommandNumber aCommand,
			  /* Style */
			  short aStyle)
{
#if qDebugMsg
	if (gTraceSetupMenus)
		ProgramReport_AC(false, "..... SetStyle(%c%d)\n", TraceMenuName(aCommand), *((Ptr) & aStyle));
#endif

	MenuID menuID;
	short item;

	MenuRef aMenuRef = CommandToComponents(aCommand, menuID, item);
	if (aMenuRef)
		SetItemStyle(aMenuRef, item, aStyle);
}


//----------------------------------------------------------------------------------------
// MAHiliteMenu: 
//----------------------------------------------------------------------------------------


void MAHiliteMenu(short menuID)
{
	if (menuID || ::LMGetTheMenu())
		::HiliteMenu(menuID);
}

//========================================================================================
// CLASS CMenuIterator
//========================================================================================
//----------------------------------------------------------------------------------------
// CMenuIterator::CMenuIterator: A simple iterator for menus.
//----------------------------------------------------------------------------------------


CMenuIterator::CMenuIterator() :
	CHandleIterator_AC(&pMenuTable, kIterateForward_AC)
{
	First();
}

//----------------------------------------------------------------------------------------
// CMenuIterator::First: 
//----------------------------------------------------------------------------------------


MenuRef CMenuIterator::First()
{
	CHandleIterator_AC::First();

	// only point at menus that are in the menulist
	while (Current() && !::GetMenuHandle((*(Current()))->menuID))
		Next();

	return Current();
}

//----------------------------------------------------------------------------------------
// CMenuIterator::Next: 
//----------------------------------------------------------------------------------------


MenuRef CMenuIterator::Next()
{
	CHandleIterator_AC::Next();

	// only point at menus that are in the menulist
	while (Current() && !::GetMenuHandle((*(Current()))->menuID))
		Next();

	return Current();
}

//========================================================================================
// CLASS TMenuBarManager
//========================================================================================
//----------------------------------------------------------------------------------------
// TMenuBarManager destructor
//----------------------------------------------------------------------------------------


TMenuBarManager::~TMenuBarManager()
{
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::IMenuBarManager: 
//----------------------------------------------------------------------------------------

TMenuBarManager::TMenuBarManager(ResNumber otherMenus) :
	fDisplayedMenus(kNoResource),
	fPreferredMenuBarID(kNoResource),
	fDisplayedHierMenus(kNoResource),
	fPreferredHierMenuBarID(kNoResource)
{
	static const CStr2_AC kParamText1 = "^0";
	
	// set my global reference
	fgMenuBarManager = this;
	
	// Add in the "other" menus, then clear them from the MenuBar
	AddMenuBar(otherMenus, kNotHierarchical);
	ClearMenuBar();

	// Initialize the Apple menu 
	MenuRef aMenu = MAGetMenu(mApple);
	if (aMenu)
		AppendResMenu(aMenu, 'DRVR');

	// If the "About" item contains the paramtext keystring (^0) then substitute the
	// Application's name
	CStr255_AC s(CommandToName(cAboutApp));
	short i = s.Pos(kParamText1);
	if (i)
	{
		CStr255_AC apName;

		gDispatcher->GetApplicationName(apName);
		s.Delete(i, kParamText1.Length());
		s.Insert(apName, i);
		SetCommandName(cAboutApp, s);
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::SetPreferredMenuBarID: 
//----------------------------------------------------------------------------------------

void TMenuBarManager::SetPreferredMenuBarID(ResNumber theID)
{
	fPreferredMenuBarID = theID;
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::GetPreferredMenuBarID: 
//----------------------------------------------------------------------------------------

ResNumber TMenuBarManager::GetPreferredMenuBarID()
{
	return fPreferredMenuBarID;
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::SetPreferredHierMenuBarID: 
//----------------------------------------------------------------------------------------

void TMenuBarManager::SetPreferredHierMenuBarID(ResNumber theID)
{
	fPreferredHierMenuBarID = theID;
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::GetPreferredHierMenuBarID: 
//----------------------------------------------------------------------------------------

ResNumber TMenuBarManager::GetPreferredHierMenuBarID()
{
	return fPreferredHierMenuBarID;
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::InstallPreferredMenus: 
//----------------------------------------------------------------------------------------

void TMenuBarManager::InstallPreferredMenus()
{
	if (((fDisplayedMenus != fPreferredMenuBarID) && (fPreferredMenuBarID != kNoResource)) || ((fDisplayedHierMenus != fPreferredHierMenuBarID) && (fPreferredHierMenuBarID != kNoResource)))
	{
		ClearMenuBar();

		if (fPreferredMenuBarID != kNoResource)
		{
			AddMenuBar(fPreferredMenuBarID, kNotHierarchical);
			fDisplayedMenus = fPreferredMenuBarID;
		}
		else
			AddMenuBar(fDisplayedMenus, kNotHierarchical);

		if (fPreferredHierMenuBarID != kNoResource)
		{
			AddMenuBar(fPreferredHierMenuBarID, kHierarchical);
			fDisplayedHierMenus = fPreferredHierMenuBarID;
		}
		else
			AddMenuBar(fDisplayedHierMenus, kHierarchical);

		// We must reinsert the help menus whenever ClearMenuBar is called
		gDispatcher->InstallHelpMenuItems();
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::Reset: 
//----------------------------------------------------------------------------------------

void TMenuBarManager::Reset()
{
	fPreferredMenuBarID = kNoResource;
	fPreferredHierMenuBarID = kNoResource;
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::AddHelpMenuItem: 
//----------------------------------------------------------------------------------------


void TMenuBarManager::AddHelpMenuItem(const CStr255_AC& helpText,
									  CommandNumber helpCommand)
{
	MenuRef helpMenu = NULL;
	short newHelpItemNo = 9999;
	const CStr2_AC kParamText1 = "^0";
	OSErr err = noErr;
	CStr255_AC localHelpText(helpText);

	// If the help item contains the paramtext keystring (^0) then substitute the
	// Application's name
	short i = localHelpText.Pos(kParamText1);
	if (i)
	{
		CStr255_AC apName;

		gDispatcher->GetApplicationName(apName);
		localHelpText.Delete(i, kParamText1.Length());
		localHelpText.Insert(apName, i);
	}

	// Put the help item at the end of the help menu
	err = HMGetHelpMenuHandle(&helpMenu);

	// If we have a reference to a menu, insert the help command
	if (err == noErr && helpMenu)
	{
		short oldMenu;
		short oldItem;

		// If the new help item is to be inserted at the end of
		// the list, make sure that its number is correct
		short nHelpItems = CountMItems(helpMenu);
		if (newHelpItemNo > nHelpItems)
			newHelpItemNo = nHelpItems + 1;

		// Insert the menu, then change its name.  This avoids problems
		// that might occur if the application name included metacharacters
		CStr255_AC helpStr = "Help"; //!!! This should be a string resource!
		InsertMenuItem(helpMenu, helpStr, newHelpItemNo - 1);
		SetMenuItemText(helpMenu, newHelpItemNo, localHelpText);

		// Add the help command number to the command table, but only if it's not already there
		if (!CommandToComponents(helpCommand, oldMenu, oldItem))
			pCmdTable.AddToTable(helpCommand, (*helpMenu)->menuID, newHelpItemNo);
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::AddMenuItem:
//----------------------------------------------------------------------------------------


void TMenuBarManager::AddMenuItem(const CStr255_AC& theMenuItemText,
								  MenuID menuNumber,
								  short insertAfterItem,
								  CommandNumber theMenuItemCommandNumber)
{
	short theMenuItem = insertAfterItem;
	MenuRef theMenuRef = MAGetMenuRef(menuNumber);

	// If we have a reference to a menu, insert the new menu item
	if (theMenuRef && theMenuItemText.Length() > 0)
	{
		// Figure out where to insert the new menu item
		short curNumItems = CountMItems(theMenuRef);
		if (theMenuItem > curNumItems)
			theMenuItem = curNumItems + 1;
		else
			// The user passed in where to insert it after so add one to take this into account
			++theMenuItem;

		// Insert the menu, then change its name.  This avoids problems
		// that might occur if the application name included metacharacters
		// One is subtracted here to take into account insert after item that InsMenuItem does
		::InsertMenuItem(theMenuRef, (ConstStr255Param)"\ptemp", theMenuItem - 1);
		::SetMenuItemText(theMenuRef, theMenuItem, theMenuItemText);

		// Add the help command number to the command table, but only if it's not already there
		short oldMenu;
		short oldItem;

		if (!CommandToComponents(theMenuItemCommandNumber, oldMenu, oldItem))
			pCmdTable.AddNewCmdToTableAndUpdateTable(theMenuItemCommandNumber, (*theMenuRef)->menuID, theMenuItem);
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::DeleteMenuItem:
//----------------------------------------------------------------------------------------


void TMenuBarManager::DeleteMenuItem(CommandNumber theMenuItemCommandNumber)
{
	short theMenuNumber;
	short theItemNumber;

	pCmdTable.CommandToMenuItem(theMenuItemCommandNumber, theMenuNumber, theItemNumber);
	if (theMenuNumber)
	{
		MenuRef theMenu = MAGetMenuRef(theMenuNumber);

		// Delete the menu item from the menu
		if (theMenu)
			::DeleteMenuItem(theMenu, theItemNumber);

		// Delete the command number from the table
		pCmdTable.DeleteCmdAndUpdateTable(theMenuItemCommandNumber);
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::ShowMenuBar:
//----------------------------------------------------------------------------------------


void TMenuBarManager::ShowMenuBar()
{
	WindowPtr	frontWindow;
	GrafPtr		windowPort;
	GrafPtr		oldPort;
	CGrafPtr	windowCPort;
 
	// Make sure the menu bar is hidden
	if (fgMenuBarState == kHidden)
	{
		// Reset the menu bar height
		
		::LMSetMBarHeight(fgMenuBarHeight);

		// Restore the gray region

		::LMSetGrayRgn(fgSavedGrayRgn);

		// Recalculate the windows

		frontWindow = ::FrontWindow();
		::CalcVis( frontWindow );
		::CalcVisBehind( frontWindow, fgNewGrayRgn );

		// Reset the window manager ports clipping

		::GetPort( &oldPort );
		::GetWMgrPort( &windowPort );
		::SetPort( windowPort );
		::SetClip( fgNewGrayRgn );
		::GetCWMgrPort( &windowCPort );
		::SetPort( (GrafPtr) windowCPort );
		::SetClip( fgNewGrayRgn );
		::SetPort( oldPort );

		// Redraw the menu bar

		::HiliteMenu( 0 );
		::DrawMenuBar();

		// Mark that the menu bar is shown

		fgMenuBarState = kShown;
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::HideMenuBar:
//----------------------------------------------------------------------------------------


void TMenuBarManager::HideMenuBar()
{
	GDHandle	mainDisplay;
	Rect		mainDisplayBounds;
	RgnHandle	workRgn;
	GrafPtr		windowPort;
	GrafPtr		oldPort;
	CGrafPtr	windowCPort;
	WindowPtr	frontWindow;
 
	if (fgMenuBarState == kShown && !gDispatcher->IsFrontProcess())
	{
		// If the menu bar is currently shown but we are in the background,
		// set the state to latent hidden: it will be hidden when we are brought
		// into the foreground.
		
		fgMenuBarState = kLatentHidden;
	}
	else if (fgMenuBarState != kHidden && gDispatcher->IsFrontProcess())
	{
		// If the menu bar is not already hidden and we are in the foreground,
		// go ahead and hide the menu bar.
		
		// Store the menu bar height and then set it to zero 

		fgMenuBarHeight = ::LMGetMBarHeight();
		::LMSetMBarHeight(0);

		// Get the rectangle for the main display

		mainDisplay = ::GetMainDevice();
		mainDisplayBounds = (*mainDisplay)->gdRect;

		// Calculate the new regions
		
		workRgn = ::NewRgn();
		::ThrowIfNULL_AC(workRgn);
		fgSavedGrayRgn = ::LMGetGrayRgn();
		::RectRgn( workRgn, &mainDisplayBounds );
		fgNewGrayRgn = ::NewRgn();
		::ThrowIfNULL_AC(fgNewGrayRgn);
		fgCoveredRgn = ::NewRgn();
		::ThrowIfNULL_AC(fgCoveredRgn);
		::UnionRgn( workRgn, fgSavedGrayRgn, fgNewGrayRgn );
		::DiffRgn( fgNewGrayRgn, fgSavedGrayRgn, fgCoveredRgn );
		::DisposeRgn( workRgn );

		// Set the new gray region

		::LMSetGrayRgn( fgNewGrayRgn );

		// Get the window manager port clip region

		::GetPort( &oldPort );
		::GetWMgrPort( &windowPort );
		::SetPort( windowPort );
		::SetClip( fgNewGrayRgn );

		::GetCWMgrPort( &windowCPort );
		::SetPort( (GrafPtr) windowCPort );
		::SetClip( fgNewGrayRgn );

		// Force an update of the desktop

		::PaintOne( NULL, fgNewGrayRgn );

		// Update the windows

		frontWindow = ::FrontWindow();
		::PaintOne( frontWindow, fgCoveredRgn );
		::PaintBehind( frontWindow, fgCoveredRgn );
		::CalcVis( frontWindow );
		::CalcVisBehind( frontWindow, fgCoveredRgn );

		// Mark the menu bar as hidden

		fgMenuBarState = kHidden;

		// Restore the port

		::SetPort( oldPort );
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::AboutToLoseControl:
//----------------------------------------------------------------------------------------


void TMenuBarManager::AboutToLoseControl()
{
	if (fgMenuBarState == kHidden)
	{
		// Application is becoming inactive and we have a hidden menu bar:
		// show the menu bar and set the state to latent.
		ShowMenuBar();
		fgMenuBarState = kLatentHidden;
	}
}

//----------------------------------------------------------------------------------------
// TMenuBarManager::RegainControl:
//----------------------------------------------------------------------------------------


void TMenuBarManager::RegainControl()
{
	if (fgMenuBarState == kLatentHidden)
	{
		// Application is becoming active and we have latent hidden state:
		// hide the menu bar.
		HideMenuBar();
	}
}


//----------------------------------------------------------------------------------------
// UMenuMgr.cp 



