//----------------------------------------------------------------------------------------
// UPrintHandler.cp
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

// MacApp

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

// ANSI


//========================================================================================
// static data members
//========================================================================================
bool TPrintHandler::gFinderPrintingProceed;						// proceed with Finder printing
bool TPrintHandler::gCouldPrint;
CViewPoint TPrintHandler::gPageOffset;
CAutoPtr_AC<TPrintHandler> TPrintHandler::gPrintHandler;
CAutoPtr_AC<TPrintHandler> TPrintHandler::gCurrPrintHandler;


//========================================================================================
// CLASS TPrintMenuBehavior
//========================================================================================
MA_DEFINE_CLASS(TPrintMenuBehavior);

//----------------------------------------------------------------------------------------
// TPrintMenuBehavior constructor
//----------------------------------------------------------------------------------------


TPrintMenuBehavior::TPrintMenuBehavior(TPrintHandler* itsPrintHandler) :
	TBehavior(kPrintMenuBehaviorID),

	fPrintHandler(itsPrintHandler)
{
	if (fPrintHandler)
		fPrintHandler->SetManager(this);
}

//----------------------------------------------------------------------------------------
// TPrintMenuBehavior destructor
//----------------------------------------------------------------------------------------


TPrintMenuBehavior::~TPrintMenuBehavior()
{
}

//----------------------------------------------------------------------------------------
// TPrintMenuBehavior::DoScriptCommand: 
//----------------------------------------------------------------------------------------


bool TPrintMenuBehavior::DoScriptCommand(CommandNumber 	aCommandNumber,	
											TAppleEvent*	messageEvent,
											TAppleEvent*	replyEvent)
{
	bool wasHandled = false;

	if (fPrintHandler)
	{
		wasHandled = fPrintHandler->DoScriptCommand(aCommandNumber, messageEvent, replyEvent);
		// Note: DoScriptCommand is implemented in TStdPrintHandler, not TPrintHandler. 
	}

	if (!wasHandled)
		wasHandled = TBehavior::DoScriptCommand(aCommandNumber, messageEvent, replyEvent);

	return wasHandled;
}

//----------------------------------------------------------------------------------------
// TPrintMenuBehavior::DoMenuCommand: 
//----------------------------------------------------------------------------------------


void TPrintMenuBehavior::DoMenuCommand(CommandNumber aCommandNumber)
{
	// Can print if:
	//   have a print handler, and
	//   print handler has a view, and
	//    either  view is not shown (e.g., Finder printing), or
	//            view is active.
	//			  we are printing from the finder
	// NOTE: we don't support printing of view's that are shown, but not active.

	// This algorithm allows a document to arbitrate between its various
    // TPrintMenuBehavior objects, representing several printable views owned by the
    // document. This arbitration wasn't supported in MacApp prior to version 3.0.

	bool didPrint = false;
	if (fPrintHandler && fPrintHandler->fView && (!fPrintHandler->fView->IsShown() || fPrintHandler->fView->IsActive() || TPrintHandler::gFinderPrintingProceed))
		didPrint = fPrintHandler->DoPrintCommand(aCommandNumber);

	if (!didPrint)
		TBehavior::DoMenuCommand(aCommandNumber);		// if we didn't print, call Inherited
}

//----------------------------------------------------------------------------------------
// TPrintMenuBehavior::DoSetupMenus: 
//----------------------------------------------------------------------------------------


void TPrintMenuBehavior::DoSetupMenus()
{
	TBehavior::DoSetupMenus();

	// see note in TPrintMenuBehavior::DoMenuCommand
	if (fPrintHandler && fPrintHandler->fView && (!fPrintHandler->fView->IsShown() || fPrintHandler->fView->IsActive() || TPrintHandler::gFinderPrintingProceed))
		fPrintHandler->DoSetupPrintMenus();
}


//========================================================================================
// CLASS TPrintHandler
//========================================================================================
MA_DEFINE_CLASS(TPrintHandler);

//----------------------------------------------------------------------------------------
// TPrintHandler constructor
//----------------------------------------------------------------------------------------


TPrintHandler::TPrintHandler(TView* itsView) :
	TBehavior(kPrintBehaviorID),		// attaching to the view is handled by AttachPrintHandler

	fView(itsView),
	fDocument(NULL),
	fManager(NULL),
	fFocusedPage(1),
	fShowBreaks(false),
	fDeviceRes(72, 72) // default matches QD default
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler destructor
//----------------------------------------------------------------------------------------


TPrintHandler::~TPrintHandler()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetViewPerPage: 
//----------------------------------------------------------------------------------------


CViewPoint TPrintHandler::GetViewPerPage()
{
	return fViewPerPage;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetDeviceRes: 
//----------------------------------------------------------------------------------------


CPoint_AC TPrintHandler::GetDeviceRes()
{
	return fDeviceRes;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::BreakFollowing: 
//----------------------------------------------------------------------------------------


ViewCoordinate TPrintHandler::BreakFollowing(VHSelect/* vhs */,
												 ViewCoordinate/* previousBreak */,
												 bool&/* automatic */ )

{
#if qDebug
	::ProgramBreak_AC("BreakFollowing called for non-subclassed Printhandler");
#endif

	return 0;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::CalcPageStrips: 
//----------------------------------------------------------------------------------------


CViewPoint TPrintHandler::CalcPageStrips()
{
#if qDebug
	::ProgramBreak_AC("CalcPageStrips called for non-subclassed Printhandler");
#endif
	return CPoint_AC();
}

//----------------------------------------------------------------------------------------
// TPrintHandler::CalcViewPerPage: 
//----------------------------------------------------------------------------------------


CViewPoint TPrintHandler::CalcViewPerPage()
{
#if qDebug
	::ProgramBreak_AC("CalcViewPerPage called for non-subclassed Printhandler");
#endif
	return CPoint_AC();
}

//----------------------------------------------------------------------------------------
// TPrintHandler::Invalidate:
//----------------------------------------------------------------------------------------


void TPrintHandler::Invalidate()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::Validate: 
//----------------------------------------------------------------------------------------


void TPrintHandler::Update()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::DrawPrintFeedback: 
//----------------------------------------------------------------------------------------


//Draw page-breaks, page-numbers, etc.
void TPrintHandler::DrawPrintFeedback(const CViewRect& /* area */)
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::DrawPageBreak: 
//----------------------------------------------------------------------------------------


void TPrintHandler::DrawPageBreak(VHSelect /* vhs */,
										 long /* whichBreak */,
										 ViewCoordinate /* loc */,
										 bool /* automatic */)
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::Focus: 
//----------------------------------------------------------------------------------------


bool TPrintHandler::Focus()
{
	return false;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::FocusOnInterior: 
//----------------------------------------------------------------------------------------


void TPrintHandler::FocusOnInterior()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetQDOrigin: 
//----------------------------------------------------------------------------------------


CPoint_AC TPrintHandler::GetQDOrigin()
{
	return CPoint_AC();
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetViewToQDOffset: 
//----------------------------------------------------------------------------------------


CViewPoint TPrintHandler::GetViewToQDOffset()
{
	return CPoint_AC();
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetPrintInfo: 
//----------------------------------------------------------------------------------------


TPrintInfo* TPrintHandler::GetPrintInfo()
{
	return NULL;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::LocatePageInterior: 
//----------------------------------------------------------------------------------------


CViewPoint TPrintHandler::LocatePageInterior(long /* pageNumber */)
{
	return CPoint_AC();
}

//----------------------------------------------------------------------------------------
// TPrintHandler::MaxPageNumber: 
//----------------------------------------------------------------------------------------


long TPrintHandler::MaxPageNumber()
{
	return 0;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::Print: 
//----------------------------------------------------------------------------------------


bool TPrintHandler::Print(CommandNumber /* itsCommandNumber */)
{
	return true;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::PrinterChanged: 
//----------------------------------------------------------------------------------------


void TPrintHandler::PrinterChanged()

{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetupForFinder: 
//----------------------------------------------------------------------------------------

bool TPrintHandler::SetupForFinder(CommandNumber /* aCommandNumber */)
{
#if qDebugMsg
	ProgramReport_AC("SetupForFinder called for a view that can't\n");
#endif

	return false;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::RedoPageBreaks: 
//----------------------------------------------------------------------------------------

void TPrintHandler::RedoPageBreaks()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::Reset: 
//----------------------------------------------------------------------------------------

void TPrintHandler::Reset()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::DoPrintCommand: 
//----------------------------------------------------------------------------------------

bool TPrintHandler::DoPrintCommand(CommandNumber /* aCommandNumber */)
{
	return false;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::DoSetupPrintMenus: 
//----------------------------------------------------------------------------------------

void TPrintHandler::DoSetupPrintMenus()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetDefaultPrintInfo: 
//----------------------------------------------------------------------------------------

// Meant to be overridden in a subclass that really prints 
void TPrintHandler::SetDefaultPrintInfo()
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetPageInterior: 
//----------------------------------------------------------------------------------------

void TPrintHandler::SetPageInterior(long /* pageNumber */)
{
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetPageOffset: 
//----------------------------------------------------------------------------------------

void TPrintHandler::SetPageOffset(const CViewPoint& /* coord */)
{
}


//----------------------------------------------------------------------------------------
// TPrintHandler::GetManager: 
//----------------------------------------------------------------------------------------

TPrintMenuBehavior* TPrintHandler::GetManager()
{
	return fManager;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetManager: 
//----------------------------------------------------------------------------------------

void TPrintHandler::SetManager(TPrintMenuBehavior* itsManager)
{
	fManager = itsManager;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SetOwner
//----------------------------------------------------------------------------------------

void TPrintHandler::SetOwner(TEventHandler* itsOwner) // override
{
	TBehavior::SetOwner(itsOwner);

	if (fView)
	{
		TWindow* window = fView->GetWindow();
		if (window)
		{
			if (itsOwner)
			{
				if (itsOwner == fView)
					window->SetScriptingPrintHandler(this);
			}
			else
			{
				window->ReleaseScriptingPrintHandler(this);
				// Note: This does not get called when a window is closed
				// because the window goes away first. 
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TPrintHandler::TerminateUPrinting: 
//----------------------------------------------------------------------------------------

void TPrintHandler::TerminateUPrinting()
{
	// Thank you
}

//----------------------------------------------------------------------------------------
// TPrintHandler::PrepareForFinderPrinting: 
//----------------------------------------------------------------------------------------

void TPrintHandler::PrepareForFinderPrinting(const CFSSpec_AC* /* targetPrinter */)
{
	// Thank you
}

//----------------------------------------------------------------------------------------
// TPrintHandler::DispatcherIsAvailable: 
//----------------------------------------------------------------------------------------

void TPrintHandler::DispatcherIsAvailable()
{
	// Thank you
}

//----------------------------------------------------------------------------------------
// TPrintHandler::GetSavePrintInfoSize: 
//----------------------------------------------------------------------------------------

long TPrintHandler::GetSavePrintInfoSize(TPrintInfo* /* itsPrintInfo */, TFile* /* itsFile */, bool /* useRsrcFork */)
{
	// Thank you
	return 0;
}

//----------------------------------------------------------------------------------------
// TPrintHandler::SavePrintInfo: 
//----------------------------------------------------------------------------------------

void TPrintHandler::SavePrintInfo(TPrintInfo* /* itsPrintInfo */, TFile* /* itsFile */, bool /* useRsrcFork */)
{
	// Thank you
}

//----------------------------------------------------------------------------------------
// TPrintHandler::RestorePrintInfo: 
//----------------------------------------------------------------------------------------

TPrintInfo* TPrintHandler::RestorePrintInfo(TFile* /* itsFile */, bool /* useRsrcFork */)
{
	// Thank you
	return NULL;
}

//----------------------------------------------------------------------------------------
// TStdPrintHandler::DeleteIfPrintInfo: 
//----------------------------------------------------------------------------------------

void TPrintHandler::DeleteIfPrintInfo(TPrintInfo* /* inPrintInfo */)
{
	// Thank you
}

//----------------------------------------------------------------------------------------
// End of UPrintHandler.cp


