//----------------------------------------------------------------------------------------
// UTracker.cp 
// Copyright  1985-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UTRACKER__
#include "UTracker.h"
#endif

// MacApp

#if defined(qDebug) && !defined(__UDEBUG__)
#include "UDebug.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMEMORY__
#include "UMemory.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI


#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// CLASS TTracker
//========================================================================================

//----------------------------------------------------------------------------------------
// TTracker::TTracker:
//----------------------------------------------------------------------------------------


TTracker::TTracker(CommandNumber itsCommandNumber,
							   TCommandHandler* itsContext,	
							   bool canUndo,
							   bool causesChange,
							   MDependable_AC* objectToNotify,
							   TView* itsView,
							   TScroller* itsScroller,
							   const CViewPoint& itsMouse) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),

	fInitialPt(itsMouse),
	fView(itsView),
	fScroller(itsScroller),
	fDeskTopTrackingPort(NULL),
	fTrackPhase(trackBegin),
	fConstrainsMouse(false),
	fViewConstrain(true),
	fTrackNonMovement(false),
	fMovedOnce(false),
	fRestorePattern(false)
{
}

//----------------------------------------------------------------------------------------
// TTracker destructor
//----------------------------------------------------------------------------------------


TTracker::~TTracker()
{
	if(fRestorePattern)
		::PenNormal();
}

//----------------------------------------------------------------------------------------
// TTracker::AutoScroll:
//----------------------------------------------------------------------------------------


void TTracker::AutoScroll(const CViewPoint& delta)
{
	if (fScroller)
		fScroller->ScrollBy(delta, kRedraw);
}

//----------------------------------------------------------------------------------------
// TTracker::IsDoneTracking:
//----------------------------------------------------------------------------------------


bool TTracker::IsDoneTracking()
{
	return fView ? fView->IsDoneTracking() : !StillDown();
}

//----------------------------------------------------------------------------------------
// TTracker::Process:
//----------------------------------------------------------------------------------------

void TTracker::Process() // OVERRIDE
{
#if qDebugMsg
	if (gIntenseDebugging && this)
		ProgramReport_AC(false, "Tracking Command: %s\n", typeid(*this).name());
#endif

	TTracker* finalCommand = gDispatcher->TrackMouse(fInitialPt, TEventHandler::GetStdHysteresis(), this);
	
	if (finalCommand)
	{
		TCommandHandler* handler = finalCommand->fContext;
		if (handler)
			handler->PerformCommand(finalCommand);
	}
}

//----------------------------------------------------------------------------------------
// TTracker::TrackConstrain:
//----------------------------------------------------------------------------------------

void TTracker::TrackConstrain(TrackPhase aTrackPhase,
							  const CViewPoint& anchorPoint,
							  const CViewPoint& previousPoint,
							  CViewPoint& nextPoint,
							  bool mouseDidMove)
{
	if (fView)
		fView->TrackConstrain(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove);
}

//----------------------------------------------------------------------------------------
// TTracker::TrackFeedback:
//----------------------------------------------------------------------------------------

void TTracker::TrackFeedback(TrackPhase aTrackPhase,
							 const CViewPoint& anchorPoint,
							 const CViewPoint& previousPoint,
							 const CViewPoint& nextPoint,
							 bool mouseDidMove,
							 bool turnItOn)
{
	if (fView)
		fView->TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);
}

//----------------------------------------------------------------------------------------
// TTracker::TrackMouse:
//----------------------------------------------------------------------------------------

TTracker* TTracker::TrackMouse(TrackPhase aTrackPhase,
								CViewPoint& anchorPoint,
								CViewPoint& previousPoint,
								CViewPoint& nextPoint,
								bool mouseDidMove)
{
	if (fView)
		fView->TrackMouse(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove);

	return (aTrackPhase == trackEnd && (!fView || !fView->ContainsMouse(nextPoint))) ? NULL : this;
}

//----------------------------------------------------------------------------------------
// TTracker::CleanUpFocus:
//----------------------------------------------------------------------------------------

void TTracker::CleanUpFocus()
{
	if (fDeskTopTrackingPort)
	{
		SetPortWindowPort(gWorkPort);

		if (qNeedsColorQD || HasColorQD_AC())
			CloseCPort((CGrafPtr) fDeskTopTrackingPort);
		else
			ClosePort(fDeskTopTrackingPort);
		fDeskTopTrackingPort = (GrafPtr) ::DisposeIfPtr_AC((Ptr) fDeskTopTrackingPort);
		fDeskTopTrackingPort = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TTracker::DoFocus:
//----------------------------------------------------------------------------------------

void TTracker::DoFocus()
{
	if (fView)
	{
		if (fDeskTopTrackingPort)
			CleanUpFocus();

		if (!fView->Focus())
		{
#if qDebug
			::ProgramBreak_AC("TApplication.TrackMouse: Unable to focus fView.");
#endif
		}
	}
	else
	{											// focus on the desktop 
		if (!fDeskTopTrackingPort)
		{
			fDeskTopTrackingPort = (GrafPtr) NewPermPtr(sizeof(CGrafPort));
			if (qNeedsColorQD || HasColorQD_AC())
				OpenCPort((CGrafPtr) fDeskTopTrackingPort);
			else
				OpenPort(fDeskTopTrackingPort);
		}

		CopyRgn(GetGrayRgn(), TWindow::GetVisRegion(fDeskTopTrackingPort));
		fDeskTopTrackingPort->portRect = (*TWindow::GetVisRegion(fDeskTopTrackingPort))->rgnBBox;
		gDispatcher->InvalidateFocus();
		SetPort(fDeskTopTrackingPort);
	}
}

//----------------------------------------------------------------------------------------
// TTracker::BecomeTracker:
//----------------------------------------------------------------------------------------

void TTracker::BecomeTracker(TTracker* oldTracker)
{
	if (oldTracker)
	{
		fDeskTopTrackingPort = oldTracker->fDeskTopTrackingPort;
		fTrackPhase = oldTracker->fTrackPhase;

		fHysteresis = oldTracker->fHysteresis;
		fTrackPhase = oldTracker->fTrackPhase;
		fAnchorPoint = oldTracker->fAnchorPoint;
		fPreviousPoint = oldTracker->fPreviousPoint;
		fNextPoint = oldTracker->fNextPoint;
		fLastAnchorPoint = oldTracker->fLastAnchorPoint;
		fLastPreviousPoint = oldTracker->fLastPreviousPoint;
		fLastNextPoint = oldTracker->fLastNextPoint;
	}

	DoFocus();
}

//----------------------------------------------------------------------------------------
// TTracker::ConstrainOnce:
//----------------------------------------------------------------------------------------

void TTracker::ConstrainOnce(bool didMouseMove)
{
	if (fViewConstrain && (fView))
		fNextPoint.ConstrainTo(fView->GetExtent());

	if (fConstrainsMouse)
	{
		CViewPoint nextPoint = fNextPoint;
		TrackConstrain(fTrackPhase, fAnchorPoint, fPreviousPoint, nextPoint, didMouseMove);
		fNextPoint = nextPoint;
	}
}

//----------------------------------------------------------------------------------------
// TTracker::FeedbackOnce:
//----------------------------------------------------------------------------------------

void TTracker::FeedbackOnce(bool mouseDidMove, bool turnItOn)
{
	fRestorePattern = true;
	
	PenNormal();
	PenMode(patXor);

	if (turnItOn)
	{
		TrackFeedback(fTrackPhase, fAnchorPoint, fPreviousPoint,
							fNextPoint, mouseDidMove, turnItOn);
		// save these to turn it off with
		if (mouseDidMove)
		{
			fLastAnchorPoint = fAnchorPoint;
			fLastPreviousPoint = fPreviousPoint;
			fLastNextPoint = fNextPoint;
		}
	}
	else
		TrackFeedback(fTrackPhase, fLastAnchorPoint, fLastPreviousPoint,
							fLastNextPoint, mouseDidMove, turnItOn);
}

//----------------------------------------------------------------------------------------
// TTracker::TrackOnce:
//----------------------------------------------------------------------------------------

TTracker* TTracker::TrackOnce(bool didMouseMove)
{
	TView* oldView = fView;
	CViewPoint anchorPoint = fAnchorPoint;
	CViewPoint previousPoint = fPreviousPoint;
	CViewPoint nextPoint = fNextPoint;

	TTracker * newTracker = TrackMouse(fTrackPhase, anchorPoint, previousPoint, nextPoint, didMouseMove);
	
	fAnchorPoint = anchorPoint;
	fPreviousPoint = previousPoint;
	fNextPoint = nextPoint;
	
	if (newTracker == this)
	{
		if (fView != oldView)
			BecomeTracker(this);
	}
	else
	{
		if (newTracker)
			newTracker->BecomeTracker(this);
		else
			CleanUpFocus();
			
		// free me if a new tracker was returned
		if (ShouldFreeOnCompletion())
		{
			TUndoHandler::fgUndoHandler.RemoveCommandAction(this);
			delete this;
		}
	}

	return newTracker;
	
}

//----------------------------------------------------------------------------------------
// TTracker::HandleTrackBegin:
//----------------------------------------------------------------------------------------

TTracker* TTracker::HandleTrackBegin(const CViewPoint& theMouse,
									 CPoint_AC hysteresis)
{
	fTrackPhase = trackBegin;

	fMovedOnce = false;
	fDeskTopTrackingPort = NULL;
	fAnchorPoint = theMouse;
	fPreviousPoint = theMouse;
	fNextPoint = theMouse;
	fHysteresis = hysteresis,

	BecomeTracker(NULL);

	ConstrainOnce(true);
	// in case Constrain changed the fNextPoint guarantee that all 3 are the same on trackBegin
	fAnchorPoint = fNextPoint;
	fPreviousPoint = fNextPoint;

	TTracker* theTracker = TrackOnce(true);
	if (theTracker)
	{
		// in case TrackMouse changed the fNextPoint guarantee that all 3 are the same on trackBegin
		theTracker->fAnchorPoint = theTracker->fNextPoint;
		theTracker->fPreviousPoint = theTracker->fNextPoint;
	
		theTracker->FeedbackOnce(true, true);
	}

	return theTracker;
}

//----------------------------------------------------------------------------------------
// TTracker::HandleTrackContinue:
//----------------------------------------------------------------------------------------

TTracker* TTracker::HandleTrackContinue()
{
	fTrackPhase = trackContinue;				// supply trackphase to all interested parties

	DoFocus();

	CPoint_AC theQDMouse;
	GetMouse(theQDMouse);
	if (fView)
		fNextPoint = fView->QDToView(theQDMouse);
	else
		fNextPoint = theQDMouse;

	if (!fMovedOnce)
	{
		ConstrainOnce(fPreviousPoint != fNextPoint);		// ensure that we are
                                                                // playing on a level
                                                                // field.
		CViewPoint amtMoved = fNextPoint - fAnchorPoint;
		if ((abs((int)amtMoved.h) >= fHysteresis.h) || (abs((int)amtMoved.v) >= fHysteresis.v))
			fMovedOnce = true;
	}

	CViewPoint delta;
	if (fMovedOnce || fTrackNonMovement)
	{
		if (fScroller && fView)
		{
			// convert fNextPoint to scroller coordinates
			CViewPoint mouseInScroller = fScroller->RootViewToLocal(fView->LocalToRootView(fNextPoint));

			// AutoScroll if the CPoint_AC is outside of the scroller's extent
			if (!fScroller->GetExtent().Contains(mouseInScroller))
			{
				fScroller->AutoScroll(mouseInScroller, delta);// Get the amount to autoscroll
				fNextPoint += delta;
			}
		}

		ConstrainOnce(fPreviousPoint != fNextPoint);
	}

	bool willScroll = delta != gZeroVPt;
	bool didMove = fPreviousPoint != fNextPoint;

	FeedbackOnce(didMove || willScroll, false);

	if (willScroll)
	{
		AutoScroll(delta);		// OK, now actually do the scrolling 
		if (fView)
			fView->Update();				// Keep synchronized. ScrollDraw only
                                            // invalidated
		DoFocus();					// the focus changed 
	}

	TTracker* theTracker = TrackOnce(didMove);
	if (theTracker)
	{
		theTracker->FeedbackOnce(didMove || willScroll, true);
		theTracker->fPreviousPoint = theTracker->fNextPoint;
	}

	return theTracker;
}

//----------------------------------------------------------------------------------------
// TTracker::HandleTrackEnd:
//----------------------------------------------------------------------------------------

TTracker* TTracker::HandleTrackEnd()
{
	fTrackPhase = trackEnd;						// supply trackphase to all interested parties

	DoFocus();

	EventRecord peekEvent;

	if (!fMovedOnce)
		fNextPoint = fPreviousPoint;			// normally same as original mouse down;
                                                // we don't use fAnchorPoint in case
                                                // someone has changed that -- it is more
                                                // likely that an app would change
                                                // fAnchorPoint than fPreviousPoint

	else if (OSEventAvail(mUpMask + mDownMask, &peekEvent))
	{
		CPoint_AC theQDMouse = peekEvent.where;
		if (fView)
		{
			GlobalToLocal(theQDMouse);
			fNextPoint = fView->QDToView(theQDMouse);
		}
		else
			fNextPoint = theQDMouse;
		ConstrainOnce(true);
	}
	// else we use the last known mouse position 

	FeedbackOnce(true, false);
	TTracker* theTracker = TrackOnce(true);

	if (theTracker)
		theTracker->CleanUpFocus();

	return theTracker;
}

//----------------------------------------------------------------------------------------
// End of UTracker.cp


