//----------------------------------------------------------------------------------------
// UUndo.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UUNDO__
#include "UUndo.h"
#endif

// MacApp

#ifndef __UAPPLEEVENTS__
#include "UAppleEvents.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

// Toolbox

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

enum
{
	kUndoAction = true,
	kRedoAction = false
};

//========================================================================================
// GLOBAL Variables
//========================================================================================

TUndoHandler TUndoHandler::fgUndoHandler;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
EActionType GetRealAction(EActionType actionType);

EActionType GetRealAction(EActionType actionType)
{
	return qUseODUndoStyle ? actionType : kSingleAction;
}

//----------------------------------------------------------------------------------------
// GetUndoText: 
//----------------------------------------------------------------------------------------
CStr255_AC GetUndoRedoText(bool cmdDone, CommandNumber aCommandNumber)
{
	short newMenuState = bzRedo;

	if (aCommandNumber == cCantUndo)
		newMenuState = bzCantUndo;
	else if (cmdDone)
		newMenuState = bzUndo;

	CStr255_AC returnVal(kIDBuzzString, newMenuState);

	short preCmdName;
	short constChars;
	if (ParseTitleTemplate(returnVal, preCmdName, constChars))
	{
		CStr255_AC cmdName;
		if (aCommandNumber != cNoCommand && aCommandNumber != cCantUndo)
			cmdName = CommandToName(aCommandNumber);
		SubstituteInTitle(returnVal, cmdName, preCmdName, constChars);
	}
	
	return returnVal;
}

//========================================================================================
// CLASS CUndoAction
//========================================================================================
//----------------------------------------------------------------------------------------
// CUndoAction::CUndoAction:
//----------------------------------------------------------------------------------------
CUndoAction::CUndoAction(TCommand* command, EActionType actionType, CommandNumber id) :
	fCommand(command),
	fID(id),
	fActionType(GetRealAction(actionType)),
	fMark(false)
{
}

//----------------------------------------------------------------------------------------
// Copy constructor
//----------------------------------------------------------------------------------------
CUndoAction::CUndoAction(const CUndoAction& rhs) :
	fCommand(rhs.fCommand),
	fID(rhs.fID),
	fActionType(rhs.fActionType),
	fMark(rhs.fMark)
{
}
				
//----------------------------------------------------------------------------------------
// CUndoAction::IsDone:
//----------------------------------------------------------------------------------------
bool CUndoAction::IsDone()
{
	return fCommand ? fCommand->GetCommandDone() : false;
}

//----------------------------------------------------------------------------------------
// CUndoAction::operator=
//----------------------------------------------------------------------------------------
CUndoAction& CUndoAction::operator=(const CUndoAction& rhs)
{
	fCommand = rhs.fCommand;
	fID = rhs.fID;
	fActionType = rhs.fActionType;
	fMark = rhs.fMark;
	
	return *this;
}

//----------------------------------------------------------------------------------------
// CUndoAction::Finalize
//----------------------------------------------------------------------------------------
void CUndoAction::DeleteCommand()
{
	delete fCommand;
	fCommand = NULL;
}
	
//========================================================================================
// CLASS TActionStack
//========================================================================================
//----------------------------------------------------------------------------------------
// TActionStack constructor
//----------------------------------------------------------------------------------------
TActionStack::TActionStack() :
CDynamicArray_AC(0, sizeof(CUndoAction))
{
	SetAllocationIncrement(kDefaultUndoStackDepth); 
}
				
//----------------------------------------------------------------------------------------
// TActionStack destructor
//----------------------------------------------------------------------------------------
TActionStack::~TActionStack()
{
}

//----------------------------------------------------------------------------------------
// TActionStack::Top
//----------------------------------------------------------------------------------------
CUndoAction* TActionStack::Top()
{
	return GetSize() ? static_cast<CUndoAction*>(ComputeAddress(GetSize())) : NULL;
}

//----------------------------------------------------------------------------------------
// TActionStack::PushAction:
//----------------------------------------------------------------------------------------
void TActionStack::PushAction(const CUndoAction& action)
{
	InsertElementsBefore(fSize + 1, &action, 1);
	
	// Purge some actions if we've busted through the max desired.
	if (GetSize() > TUndoHandler::fgUndoHandler.GetMaxDesiredItems())
		Purge(kMaxLong, TUndoHandler::fgUndoHandler.GetMaxDesiredItems());
}

//----------------------------------------------------------------------------------------
// TActionStack::Delete:
//----------------------------------------------------------------------------------------
void TActionStack::Delete(CUndoAction* action)
{
	for (TemplateArrayIterator_AC<CUndoAction> iter(this, kIterateForward_AC); iter; ++iter)
		if (iter == action)
		{
			DeleteElementsAt(iter.CurrentIndex(), 1);
			break;
		}
}

//----------------------------------------------------------------------------------------
// TActionStack::ClearActions:
//----------------------------------------------------------------------------------------
void TActionStack::ClearActions(bool toMark, TCommandHandler* filterby, bool itsForward)
{
	for (TemplateArrayIterator_AC<CUndoAction> iter(this, itsForward); iter; ++iter)
	{
		if (toMark && iter->GetMark())
		{
			iter->SetMark(false);
			break;
		}
		else if (!filterby || iter->GetCommand()->GetContext() == filterby)
		{
			// Always slurp down all the actions if begin - end sets
			long level = 0;
			do
			{
				EActionType actionType = iter->GetActionType();

				if (actionType == kEndAction)
					++level;
				else if (actionType == kBeginAction)
					--level;
				
				iter->DeleteCommand();
				DeleteElementsAt(iter.CurrentIndex(), 1);
				
				// remember iter is pointing at the next item already due to the "while" loop
			} while (level && iter.Current());
		}
	}
}

//----------------------------------------------------------------------------------------
// TActionStack::FindTransaction:
//----------------------------------------------------------------------------------------
CUndoAction* TActionStack::FindTransaction(TCommand* command, bool itsForward)
{
	CUndoAction* transaction = NULL;
	long level = 0;
	bool found = false;
	
	for (TemplateArrayIterator_AC<CUndoAction>iter(this, itsForward); iter; ++iter)
	{
		EActionType actionType = iter->GetActionType();
		if (!level)
			if (actionType == kSingleAction || actionType == kBeginAction)
				transaction = iter;
		
		if (actionType == kBeginAction)
			++level;
		else if (actionType == kEndAction)
			--level;

		if (iter->GetCommand() == command)
		{
			found = true;
			break;
		}
	}
	
	return found ? transaction : NULL;
}

//----------------------------------------------------------------------------------------
// TActionStack::Purge:
//----------------------------------------------------------------------------------------
Size TActionStack::Purge(Size needed, ArrayIndex_AC numToLeave)
{
	// NOTE: This function does not currently support the 'needed' parameter.
	// Unload possibly even the topmost entry in the stack.
	Size	freed = 0;

	// Locate the topmost action to which we will unload.
	ArrayIndex_AC		i = 0;
	ArrayIndex_AC		j = 0;
	ArrayIndex_AC		limit = GetSize() - numToLeave;
	
	while (i <= limit && freed < needed)
	{
		++i;
		CUndoAction*	action = static_cast<CUndoAction*>(ComputeAddress(i));
		Size			freeableSize = 0; // Was: action->Size();
		
		EActionType actionType = action->GetActionType();
		if (actionType == kBeginAction)
		{
			// Skip over this transaction.
			long transaction = 1;
			do
			{
				++i;
				action = static_cast<CUndoAction*>(ComputeAddress(i));
				//freeableSize += action->Size();
				
				actionType = action->GetActionType();
				if (actionType == kBeginAction)
					++transaction;
				else if (actionType == kEndAction)
					--transaction;
			} while (i <= limit && transaction);
		}
		if (i <= limit)
		{
			j = i;
			freed += freeableSize;
		}
	}
	
	// Now flush the targetted entries from the list.
	for (i = 1; i <= j; ++i)
		static_cast<CUndoAction*>(ComputeAddress(i))->DeleteCommand();	
	DeleteElementsAt(1, j);
	
	return freed;
}

//========================================================================================
// CLASS TUndoHandler
//========================================================================================
//----------------------------------------------------------------------------------------
// TUndoHandler constructor
//----------------------------------------------------------------------------------------
TUndoHandler::TUndoHandler() :
	fInTransaction(0),
	fActionState(kInSingleAction),
	fActionCommand(cNoCommand),
	fMaxDesiredItems(kDefaultUndoStackDepth)	// Maximum number of entries in stack
{
}

//----------------------------------------------------------------------------------------
// TUndoHandler destructor
//----------------------------------------------------------------------------------------
TUndoHandler::~TUndoHandler()
{
	ClearActionHistory(kDontRespectMarks, NULL);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::AddActionToHistory:
//----------------------------------------------------------------------------------------
void TUndoHandler::AddActionToHistory(TCommand* command, CommandNumber id)
{
	EActionType action = kSingleAction;
	if (fActionState == kInBeginAction)
	{
		action = kBeginAction;
		fActionState = kInCollectActions;
	}

	EActionType realAction = GetRealAction(action);

	if (realAction == kBeginAction)
		++fInTransaction;

	fUndoStack.PushAction(CUndoAction(command, realAction, (qUseODUndoStyle && fActionCommand != cNoCommand) ? fActionCommand : id));
	
	ClearRedoHistory();
}

//----------------------------------------------------------------------------------------
// TUndoHandler::BeginAction:
//----------------------------------------------------------------------------------------
void TUndoHandler::BeginAction(CommandNumber inCommand)
{
	fActionCommand = inCommand;
	fActionState = kInBeginAction;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::EndAction:
//----------------------------------------------------------------------------------------
void TUndoHandler::EndAction()
{
	if (fActionState == kInCollectActions)
	{
		CUndoAction* action = GetTopAction(kUndoAction);
		if (action)
		{
			EActionType realAction = GetRealAction(kBeginAction ? kSingleAction: kEndAction);

			action->SetActionType(realAction);

			if (realAction == kEndAction)
				--fInTransaction;
		}
	}

	fActionState = kInSingleAction;
	fActionCommand = cNoCommand;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::AbortCurrentTransaction:
//----------------------------------------------------------------------------------------
void TUndoHandler::AbortCurrentTransaction()
{
	if (fInTransaction > 0)
	{
		long nextTransaction = fInTransaction - 1;
		CUndoAction* action;
		while ((action = GetTopAction(kUndoAction)) != NULL)
		{
			EActionType actionType = action->GetActionType();
			action->DeleteCommand();
			fUndoStack.Delete(action);
			
			if (actionType == kEndAction)
				++fInTransaction;
			else if (actionType == kBeginAction)
			{
				if (--fInTransaction <= nextTransaction)
					break;
			}
		}
#if qDebug
		::Assertion_AC(fInTransaction == nextTransaction,
				  "End of undo stack before end of transaction");
#endif
	}
}

//----------------------------------------------------------------------------------------
// TUndoHandler::ClearActionHistory:
//----------------------------------------------------------------------------------------
void TUndoHandler::ClearActionHistory(ERespectMarksChoices respectMarks, TCommandHandler* filterby)
{
	fRedoStack.ClearActions(respectMarks, filterby);
	fUndoStack.ClearActions(respectMarks, filterby, kIterateBackward_AC);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::MarkActionHistory:
//----------------------------------------------------------------------------------------
void TUndoHandler::MarkActionHistory()
{
	CUndoAction* redoAction = fRedoStack.Top();
	if (redoAction)
		redoAction->SetMark(true);

	CUndoAction* undoAction = fUndoStack.Top();
	if (undoAction)
		undoAction->SetMark(true);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::FindTransaction:
//----------------------------------------------------------------------------------------
CUndoAction* TUndoHandler::FindTransaction(TCommand* command)
{
	CUndoAction* action = fUndoStack.FindTransaction(command, kIterateForward_AC);
	if (!action)
		action = fRedoStack.FindTransaction(command, kIterateBackward_AC);
	
	return action;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::Abort:
//----------------------------------------------------------------------------------------
void TUndoHandler::Abort()
{
	long transaction = fInTransaction;
		// start with fInTransaction so we can abort a partial transaction
	do {
		EActionType actionType = AbortAction();
		if (actionType == kEndAction)
			++transaction;
		else if (actionType == kBeginAction)
		{
			--transaction;
#if qDebug
			::Assertion_AC(transaction >= 0, "Bad undo stack");
#endif
		}
	} while (transaction);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::Purge:
//----------------------------------------------------------------------------------------
Size TUndoHandler::Purge(Size needed)
{
	// Unload as much of the undo and redo stack as needed.
	Size totalPurged = fUndoStack.PurgeFirstPhase(kMaxLong);
	if (totalPurged < needed)
	{
		totalPurged = fRedoStack.PurgeFirstPhase(kMaxLong);
		if (totalPurged < needed)
		{
			totalPurged = fUndoStack.Purge(kMaxLong);
			if (totalPurged < needed)
				totalPurged = fRedoStack.Purge(kMaxLong);
		}
	}

	return totalPurged;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::GetTopAction:
//----------------------------------------------------------------------------------------
CUndoAction* TUndoHandler::GetTopAction(bool isUndo)
{
#if qUseODUndoStyle
	return isUndo ? fUndoStack.Top() : fRedoStack.Top();
#else
	CUndoAction* foundAction = NULL;

	for (TemplateArrayIterator_AC<CUndoAction>iter(isUndo ? &fUndoStack : &fRedoStack, kIterateBackward_AC); iter && !iter->GetMark() && !foundAction; ++iter)
	{
		TEventHandler* testHandler = TEventHandler::GetTarget();
		while (testHandler && !foundAction)
		{
			TCommandHandler* testCommandHandler = dynamic_cast<TCommandHandler*>(testHandler);
			if (testCommandHandler && testCommandHandler->GetContext(iter->GetCommand()->GetIdentifier()) == iter->GetCommand()->GetContext())
				foundAction = iter;
			else
				testHandler = testCommandHandler->GetNextHandler();
		}
	}
#endif
	return foundAction;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::Undo:
//----------------------------------------------------------------------------------------
ArrayIndex_AC TUndoHandler::SetMaxDesiredItems(ArrayIndex_AC newDepth)
{
	ArrayIndex_AC returnVal = fMaxDesiredItems;
	
	fMaxDesiredItems = newDepth;
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::Undo:
//----------------------------------------------------------------------------------------
void TUndoHandler::Undo()
{
#if qDebug
	::Assertion_AC(!fInTransaction, "Undo during transaction");
#endif
	long transaction = 0;
	do
	{
		EActionType actionType = UndoAction();
		if (actionType == kEndAction)
			++transaction;
		else if (actionType == kBeginAction)
		{
			--transaction;
#if qDebug
			::Assertion_AC(transaction >= 0, "Bad undo stack");
#endif
		}
	}

	while (transaction);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::GetUndoID:
//----------------------------------------------------------------------------------------
CommandNumber TUndoHandler::GetUndoID()
{
	CUndoAction* foundAction = GetTopAction(kUndoAction);

	return foundAction ? (foundAction->GetMark() ? cNoCommand : foundAction->GetCommandNumber()) : cNoCommand;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::Redo:
//----------------------------------------------------------------------------------------
void TUndoHandler::Redo()
{
#if qDebug
	::Assertion_AC(!fInTransaction, "Redo during transaction");
#endif
	long transaction = 0;
	do
	{
		EActionType actionType = RedoAction();
		if (actionType == kBeginAction)
			++transaction;
		else if (actionType == kEndAction)
		{
			--transaction;
#if qDebug
			::Assertion_AC(transaction >= 0, "Bad redo stack");
#endif
		}
	}
	while (transaction);
}

//----------------------------------------------------------------------------------------
// TUndoHandler::ClearRedoHistory:
//----------------------------------------------------------------------------------------
void TUndoHandler::ClearRedoHistory()
{
	for (TemplateArrayIterator_AC<CUndoAction>iter(&fRedoStack); iter && !iter->GetMark(); ++iter)
	{
		iter->DeleteCommand();	
		fRedoStack.DeleteElementsAt(iter.CurrentIndex(), 1);
	}
}

//----------------------------------------------------------------------------------------
// TUndoHandler::GetRedoID:
//----------------------------------------------------------------------------------------
CommandNumber TUndoHandler::GetRedoID()
{
	CUndoAction* foundAction = GetTopAction(kRedoAction);

	return foundAction ? (foundAction->GetMark() ? cNoCommand : foundAction->GetCommandNumber()) : cNoCommand;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::RemoveCommandAction:
//----------------------------------------------------------------------------------------
void TUndoHandler::RemoveCommandAction(TCommand* theCommand)
{
	CUndoAction* transaction = NULL;
	long level = 0;
	bool found = false;
	
	// find the action containing the command
	for (TemplateArrayIterator_AC<CUndoAction> iter(&fUndoStack, kIterateBackward_AC); iter; ++iter)
	{
		EActionType actionType = iter->GetActionType();
		if (!level)
			if (actionType == kSingleAction || actionType == kBeginAction)
				transaction = iter;
		
		if (actionType == kBeginAction)
			++level;
		else if (actionType == kEndAction)
			--level;

		if (iter->GetCommand() == theCommand)
		{
			// found
			EActionType actionType = iter->GetActionType();
			fUndoStack.DeleteElementsAt(iter.CurrentIndex(), 1);

			switch(actionType)
			{
				case kBeginAction:
					fActionState = kInBeginAction; // just reset it
					break;
					
				case kEndAction:
				{
					++iter;
					if (iter)
						iter->SetActionType(kEndAction);
					break;
				}
			}
			break;
		}
	}
	
}
//----------------------------------------------------------------------------------------
// For internal use only
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TUndoHandler::AbortAction:
//----------------------------------------------------------------------------------------
EActionType TUndoHandler::AbortAction()
{
	CUndoAction* action = GetTopAction(kUndoAction);
	::ThrowIfNonObject_AC(action);

	EActionType actionType = action->GetActionType();
	TCommand* command = action->GetCommand();

	fUndoStack.Delete(action);

	if (command && command->GetCommandDone() && command->CanBeUndone() &&
		(command->GetValidationError() == noErr))
		AbortCommand(command);

	return actionType;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::AbortCommand:
//----------------------------------------------------------------------------------------
void TUndoHandler::AbortCommand(TCommand* command)
{
	try
	{
		command->Abort();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		command->SetValidationError(theException.GetError());
		CommandNumber aCommandNumber = command->GetIdentifier();

		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), CException_AC::BuildMessage(aCommandNumber, messageCommandError));
	}
	
}

//----------------------------------------------------------------------------------------
// TUndoHandler::UndoAction:
//----------------------------------------------------------------------------------------
EActionType TUndoHandler::UndoAction()
{
	CUndoAction* action = GetTopAction(kUndoAction);
	::ThrowIfNonObject_AC(action);

	EActionType actionType = action->GetActionType();
	TCommand* command = action->GetCommand();

	if (command && command->GetCommandDone() && command->CanBeUndone() &&
		(command->GetValidationError() == noErr))
		UndoCommand(command);

	fRedoStack.PushAction(*action);
	fUndoStack.Delete(action);
	
	return actionType;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::UndoCommand:
//----------------------------------------------------------------------------------------
void TUndoHandler::UndoCommand(TCommand* command)
{
	try
	{
		command->PrepareForUndoRedo();
		command->UndoIt();
		command->FinishUndoRedo();
		command->RevealUndoRedo();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		command->SetValidationError(theException.GetError());
		CommandNumber aCommandNumber = command->GetIdentifier();

		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), CException_AC::BuildMessage(aCommandNumber, messageCommandError));
	}
	
}

//----------------------------------------------------------------------------------------
// TUndoHandler::RedoAction:
//----------------------------------------------------------------------------------------
EActionType TUndoHandler::RedoAction()
{
	CUndoAction* action = GetTopAction(kRedoAction);
	::ThrowIfNonObject_AC(action);

	EActionType actionType = action->GetActionType();
	TCommand* command = action->GetCommand();

	if (command && !command->GetCommandDone() && command->CanBeUndone() &&
		(command->GetValidationError() == noErr))
		RedoCommand(command);

	fUndoStack.PushAction(*action);
	fRedoStack.Delete(action);
	
	return actionType;
}

//----------------------------------------------------------------------------------------
// TUndoHandler::RedoCommand:
//----------------------------------------------------------------------------------------
void TUndoHandler::RedoCommand(TCommand* command)
{
	try
	{
		command->PrepareForUndoRedo();
		command->RedoIt();
		command->FinishUndoRedo();
		command->RevealUndoRedo();
	}
	catch(CException_AC& theException)
	{
		DoCatchMessage_AC(theException);
		command->SetValidationError(theException.GetError());
		CommandNumber aCommandNumber = command->GetIdentifier();

		::ThrowNewMessage_AC(theException.GetError(), theException.GetMessage(), CException_AC::BuildMessage(aCommandNumber, messageCommandError));
	}
	
}

//========================================================================================
// CLASS TUndoRedoCommand
//========================================================================================
//----------------------------------------------------------------------------------------
// TUndoRedoCommand::TUndoRedoCommand
//----------------------------------------------------------------------------------------
TUndoRedoCommand::TUndoRedoCommand(CommandNumber itsCommandNumber) :
	TCommand(itsCommandNumber, gDispatcher, kCantUndo, kDoesNotCauseChange, NULL),
	fAEEventID(cNoCommand)
{
	if (itsCommandNumber == cUndoRedo)
	{
		// determine which is desired: undo or redo.
		fAEEventID = TUndoHandler::fgUndoHandler.GetRedoID() == cNoCommand ? static_cast<AEEventID>(kAEUndo) : static_cast<AEEventID>(kAERedo);
	}
	else
		fAEEventID = (itsCommandNumber == cAEUndo || itsCommandNumber == cUndo)
			? static_cast<AEEventID>(kAEUndo)
			: static_cast<AEEventID>(kAERedo);
}


//----------------------------------------------------------------------------------------
// TUndoRedoCommand destructor
//----------------------------------------------------------------------------------------
TUndoRedoCommand::~TUndoRedoCommand()
{
}

//----------------------------------------------------------------------------------------
// TUndoRedoCommand::MakeAppleEvent: 
//----------------------------------------------------------------------------------------
TAppleEvent* TUndoRedoCommand::MakeAppleEvent()
{
	CAutoPtr_AC<TAppleEvent> theEvent(TH_new TAppleEvent(kAEMiscStandards, fAEEventID, gServerAddress, kAEWaitReply));

	CTempDesc targetDesc;
	gDispatcher->MakeObjectSpecifier(targetDesc, gDispatcher->GetSpecifierForm());
	theEvent->WriteParameter(keyDirectObject, targetDesc);

	return theEvent.release();
}

//----------------------------------------------------------------------------------------
// TUndoRedoCommand::DoIt:
//----------------------------------------------------------------------------------------
void TUndoRedoCommand::DoIt()
{
	if (fAEEventID == kAEUndo)
		TUndoHandler::fgUndoHandler.Undo();
	else
		TUndoHandler::fgUndoHandler.Redo();
}
	

//========================================================================================
// CLASS CUndoStackHook
//========================================================================================
//----------------------------------------------------------------------------------------
// CUndoStackHook::Purge:
//----------------------------------------------------------------------------------------
Size CUndoStackHook::Purge(Size needed)
{
	// Make memory available by unloading the undo stack. Returns 
	// the actual amount of memory purged. 

	return TUndoHandler::fgUndoHandler.Purge(needed);
}

//----------------------------------------------------------------------------------------
// End of UUndo.cp
