//----------------------------------------------------------------------------------------
// UCPlusTool.cp
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCPLUSTOOL__
#include "UCPlusTool.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UNIVERSALSTARTUP__
#include "UniversalStartup.h"
#endif

// Toolbox

#ifndef __CURSORCTL__
#include <CursorCtl.h>
#endif

#ifndef __ERRMGR__
#include <ErrMgr.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

// ANSI

#ifndef __SIGNAL__
#include <signal.h>
#endif

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//========================================================================================
// GLOBAL Variables
//========================================================================================

#if !defined(__MWERKS__)
QDGlobals qd;
#endif

// Global variables defined so that storage is allocated:
const char* gCPlusToolVersion = "1.0";
const char* kErrorMarker = "### ";
TCPlusTool* TCPlusTool::fgCPlusTool;

//========================================================================================
// GLOBAL Procedures
//========================================================================================
extern "C" void Intr(int);

//----------------------------------------------------------------------------------------
// InitUCPlusTool:
//----------------------------------------------------------------------------------------
void InitUCPlusTool()
{
	// Do Tool related initialization 
	InitGraf(&qd.thePort);
	SetFScaleDisable(true);							// per chapter in MPW guide on tools 

	InitCursorCtl(NULL);
	RotateCursor(0);

	InitErrMgr("", "", false);

	// -1 == 0xFFFFFFFF, the largest 32 bit address. Our routine StripLong uses a
	// pre-stripped address gStrippedAddress to avoid the yucky MPW glue. 

	gStrippedAddress = StripAddress((Ptr) -1);

	// Find out just what kind of environment we're dealing with here 
	if (!ValidateProcessor())
		; // do something
	if (!ValidateConfiguration())
		; // do something

	// The refnum where the application's resources should be found 
	gApplicationRefNum = CurResFile_AC();

	gToolBoxInitialized = true;
}

//----------------------------------------------------------------------------------------
// Intr:
//----------------------------------------------------------------------------------------
void Intr(int)
{
	TCPlusTool::fgCPlusTool->SetInterrupted();					// we test this switch periodically
}

//========================================================================================
// CLASS TCPlusTool
//========================================================================================
//----------------------------------------------------------------------------------------
// TCPlusTool::Stop:
//----------------------------------------------------------------------------------------
void TCPlusTool::Stop(const CStr255_AC& msg)
{
	if (msg.Length() > 0)
		fprintf(stderr, "\n%s\n", msg);

	if (fInterrupted)
		exit(-9);

	// don't worry about closing the files we opened. The Shell will do so if appropriate.
	exit(static_cast<short>(fRetCode));						// exit, returning the appropriate status code
}

//----------------------------------------------------------------------------------------
// TCPlusTool::SyntaxError:
//----------------------------------------------------------------------------------------
void TCPlusTool::SyntaxError(const CStr255_AC& suffix)
{
	CStr255_AC aStr = fProgName;

	fprintf(stderr, "%s Bad Parameter: %s\n", kErrorMarker, suffix);
	fprintf(stderr, "%s %s<invalid option>\n", kErrorMarker, aStr);
	Stop("");
}

//----------------------------------------------------------------------------------------
// TCPlusTool::DoShowUsage:
//----------------------------------------------------------------------------------------
void TCPlusTool::DoShowUsage()
{
	fprintf(stderr, "# Usage: %s [-p]\n", fProgName);
}

//----------------------------------------------------------------------------------------
// TCPlusTool::GetNextArg:
//----------------------------------------------------------------------------------------
void TCPlusTool::GetNextArg(CStr255_AC& nextArg)
{
	if (++fArgvIndex > fArgc)
		Stop(" 		Not enough arguments ");
	nextArg = fArgv[fArgvIndex];
}

//----------------------------------------------------------------------------------------
// TCPlusTool::InstallKeyWords:
//----------------------------------------------------------------------------------------
void TCPlusTool::InstallKeyWords()
{
	KeyWordStruct keywords [] =
	{
		{"Help", kwHelp},
		{"P", kwP},				{"NoP", kwNoP},
		{"T", kwT},				{"NoT", kwNoT}
	};
	
	for (short i = 0; i < sizeof(keywords) / sizeof(KeyWordStruct); ++i)
		InstallKeyWord(keywords[i].keyword, keywords[i].keyvalue);
}

//----------------------------------------------------------------------------------------
// TCPlusTool::InstallKeyWord:
//----------------------------------------------------------------------------------------
void TCPlusTool::InstallKeyWord(const CStr255_AC& keyword,
									 short kw)
{
	CStr255_AC upperKeyWord(keyword);
	UprStr255(upperKeyWord);

	fKeyWordList.InsertEntry(upperKeyWord, (unsigned short)kw);
}

//----------------------------------------------------------------------------------------
// TCPlusTool::LookupKeyword:
//----------------------------------------------------------------------------------------
bool TCPlusTool::LookupKeyword(const CStr255_AC& keyword,
									   short& kw)
{
	bool returnVal = false;

	CStr255_AC upperKeyWord(keyword);
	UprStr255(upperKeyWord);

	CStr255_AC value;
	if (fKeyWordList.ValueAt(upperKeyWord, value))
	{
		kw = value.ToUShort();
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TCPlusTool::ProcessArg:
//----------------------------------------------------------------------------------------
void TCPlusTool::ProcessArg(const CStr255_AC& arg)
{
	if (arg[1] != '-')
		DoProcessFileArg(arg);
	else
	{
		short akw;
		CStr255_AC realArg(arg);

		realArg.Delete(1,1);					// delete the '-'
		if (LookupKeyword(realArg, akw))
			DoProcessOptionArg(akw);
		else
			SyntaxError(arg + " < invalid option > ");
	}
}

//----------------------------------------------------------------------------------------
// TCPlusTool::DoProcessFileArg:
//----------------------------------------------------------------------------------------
void TCPlusTool::DoProcessFileArg(const CStr255_AC& arg)
{
	SyntaxError(arg + " < invalid option > ");
}

//----------------------------------------------------------------------------------------
// TCPlusTool::DoProcessOptionArg:
//----------------------------------------------------------------------------------------
void TCPlusTool::DoProcessOptionArg(short kw)
{
	switch (kw)
	{
		case kwP:
			fProgress = true;
			break;
		case kwNoP:
			fProgress = false;
			break;
		case kwT:
			fTime = true;
			break;
		case kwNoT:
			fTime = false;
			break;
		case kwHelp:
			DoShowUsage();
			fRetCode = RC_Normal;
			Stop(" 		");
			break;
		default:
			SyntaxError(CStr255_AC(fArgv[fArgvIndex]) + CStr255_AC(" < invalid option > "));
	}
}

//----------------------------------------------------------------------------------------
// TCPlusTool::DoStartProgress:
//----------------------------------------------------------------------------------------
void TCPlusTool::DoStartProgress()
{
	fprintf(stderr, "\n%s (Ver %s) \n\n\n", fProgName, gCPlusToolVersion);
}

//----------------------------------------------------------------------------------------
// TCPlusTool::DoToolAction
//----------------------------------------------------------------------------------------
void TCPlusTool::DoToolAction()
{
	SubClassResponsibility();
}

//----------------------------------------------------------------------------------------
// TCPlusTool destructor
//----------------------------------------------------------------------------------------
TCPlusTool::~TCPlusTool()
{
}

//----------------------------------------------------------------------------------------
// TCPlusTool::TCPlusTool:
//----------------------------------------------------------------------------------------
TCPlusTool::TCPlusTool(int argc, char** argv) :
	fArgc(argc),
	fArgv(argv),
	fArgvIndex(0),
	fCursorCount(0),							// prepare to spin that cursor
	fInterrupted(false),						// becomes true when interrupted
	fProgress(false),
	fRetCode(RC_ParmErrs),
	fStartDateTime(0),							// Date/Time at start of tool 
	fStartTicks(::TickCount()),							// tickcount at start of tool 
	fTime(false)
{
	fgCPlusTool = this;

	unsigned long theDateTime;
	GetDateTime(&theDateTime);
	fStartDateTime = theDateTime;

	SpinCursor(1);
	signal(SIGINT, &Intr);

	fProgName.CopyFrom(argv[fArgvIndex]);

	if (fInterrupted)
		Stop("");

	InstallKeyWords();
}

//----------------------------------------------------------------------------------------
// TCPlusTool::Run:
//----------------------------------------------------------------------------------------
void TCPlusTool::Run()
{
	try
	{
		while (++fArgvIndex < fArgc)		// fArgc is the number of args plus one
		{
			RotateCursor(++fCursorCount);
			ProcessArg(fArgv[fArgvIndex]);
		}

		fRetCode = RC_Normal;

		if (fProgress)
			DoStartProgress();

		DoToolAction();

		fflush(stdout);
	
		if (fTime)
		{
			long currentTicks = ::TickCount();
			long elapsedTicks = currentTicks - fStartTicks;
			long double elapsedSecs = (double)elapsedTicks / 60.0;
			fprintf(stderr, "Elapsed time: %-1.2Lf seconds\n", elapsedSecs);
		}
	}
	catch(CException_AC& caughtException)
	{
		OSErr theErr = caughtException.GetError();
		CChar255_AC theText;

		fflush(stdout);

		if (theErr != noErr)
		{
			GetSysErrText(theErr, theText);
			fprintf(stderr, "%s %s: %s\n", kErrorMarker, fProgName, theText);
			fRetCode = RC_Abort;
		}
	}

	fflush(stderr);
	
	exit(static_cast<short>(fRetCode));
}

//----------------------------------------------------------------------------------------
// End of UCPlusTool.cp
