//----------------------------------------------------------------------------------------
// UCluster.h 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCLUSTER__
#define __UCLUSTER__

// ACS

#if qGrayscaleAppearance

#	ifndef __CCluster_AC__
#	include "CCluster_AC.h"
#	endif

#	ifndef __AGA_AC__
#	include "AGA_AC.h"
#	endif

#endif // qGrayscaleAppearance

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif

class EXPORT_MA TClusterBehavior : public TBehavior
//----------------------------------------------------------------------------------------
// TClusterBehavior: Provides a mechanism whereby radio buttons can be grouped in clusters, such
// that when one of the buttons is selected the others are all deselected.
//----------------------------------------------------------------------------------------
{
	MA_DECLARE_CLASS;
	
public:
	TClusterBehavior(IDType itsIdentifier = kNoIdentifier);
		// Constructor.
		
	virtual ~TClusterBehavior();
		// Destructor.

	virtual void DoEvent(EventNumber eventNumber,
								TEventHandler* source,
								TEvent* event);
		// Responsible for resetting the radio buttons so that the last selected one is
        // the only one selected.

	virtual IDType GetCurrentChoice();
		// Returns the ID of the currently selected button.

	virtual void SetCurrentChoice(IDType newChoice);
		// Highlights the specified button, and turns off all others off.

protected:
	IDType fCurrentChoice;			// the ID of the currently selected radio button.
};

class EXPORT_MA TCluster : public TControl
//----------------------------------------------------------------------------------------
// TCluster: a control class that automatically adds a TClusterBehavior and can be
// utilized for providing labelled adornments around groups of dialog items.
//----------------------------------------------------------------------------------------
{
	MA_DECLARE_CLASS;
	
public:
	TCluster(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable,
								 ResNumber itsRsrcID = kNoResource,
								 short itsIndex = kEmptySTRIndex);
		// Constructor
		
	TCluster(const TCluster& inCluster);
		// Copy Constructor
		
	virtual ~TCluster();
		// Release the label and call Inherited::Free.

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// Override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

	virtual void Draw(const CViewRect& area);
		// Frame the cluster if there is a label, then draw the label.

	virtual void DrawLabel(const CViewRect& area);
		// Draw the label.
	
	virtual bool IsDrawingBorder() const;
		// Returns whether to draw a border for the cluster.  The default is
		// to draw if there is a label
        
	virtual	void SetEnable(bool state);
		//Set the enable of subviews accordingly
	
	virtual	void DimState(bool state,bool redraw);
		//Set the DimState (or Dim) of subviews accordingly
	
	virtual CStr255_AC GetLabel() const;
		// Returns the text in the clusters label.

	virtual void ReleaseLabel();
		// Releases fDataHandle which is a CPascalStr_AC handle containing the text for the
        // label.

	virtual void SetLabel(const CStr255_AC& theLabel, bool redraw);
		// Sets the clusters label to be theLabel, redrawing if redraw is true.

	virtual void SetLabelWithStrListID(ResNumber theStrListID, short theIndex, bool redraw);
		// Set fDataHandle to the string specified by theStrListID and theIndex and sets fStrListID
		// and fIndex accordingly.

	virtual IDType GetCurrentChoice();
		// Returns the ID of the currently selected button

	virtual void SetCurrentChoice(IDType newChoice);
		// Highlights the specified button, and turns off all others off

#if qGrayscaleAppearance

	// MacApp overrides
	
	virtual void DoPostCreate(TDocument* itsDocument);

	virtual void SetFrame(const CViewRect& newFrame, bool invalidate);

	virtual void Activate(bool entering);
	
	virtual void Dim();

	virtual void DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* event, CPoint_AC hysteresis);

	// AGAControl overrides
	
	CControl_AC* GetAGAControl() const { return fAGAControl; }

	bool IsEmulatingControls() const { return true; } // NB EmulateControls_AC() would cause old-style to be drawn when Appearance Mgr is present
	
	virtual bool IsPrimaryType() const;
		// Returns whether this control is drawn as a primary or secondary cluster.  True for TCluster.
		
protected:

	virtual void CreateAGAControl();
	
	CAutoPtr_AC<CCluster_AC>	fAGAControl;

#endif // qGrayscaleAppearance

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CPascalStrHandle_AC fDataHandle;					// the cluster's label
	TClusterBehavior* fClusterBehavior;			// Manages the currently selected radio button
	ResNumber fStrListID;						// STR# rsrc id of the cluster's label
	short fIndex;								// index into STR#
};

#if qGrayscaleAppearance

class EXPORT_MA TPrimaryCluster : public TCluster
{
	MA_DECLARE_CLASS;

	// TCluster-compatible constructor
	TPrimaryCluster(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable,
								 ResNumber itsRsrcID = kNoResource,
								 short itsIndex = kEmptySTRIndex);

	virtual ~TPrimaryCluster();
	
	virtual bool IsDrawingBorder() const;
		// Returns whether to draw a border for the cluster.  
		// A new-style cluster always returns true.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);
	
	virtual void WriteFields(CStream_AC* aStream) const;

protected:

	virtual void CreateAGAControl();
		// if a title view was specified, use that
	
	IDType	fTitleViewID;	// use to specify title gap. 
};

class EXPORT_MA TSecondaryCluster : public TPrimaryCluster
{
	MA_DECLARE_CLASS;
	
	// TCluster-compatible constructor
	TSecondaryCluster(const CViewPoint& itsLocation = gZeroVPt,
								 const CViewPoint& itsSize = gZeroVPt,
								 SizeDeterminer itsHSizeDet = sizeVariable,
								 SizeDeterminer itsVSizeDet = sizeVariable,
								 ResNumber itsRsrcID = kNoResource,
								 short itsIndex = kEmptySTRIndex);

	virtual ~TSecondaryCluster();

	virtual bool IsPrimaryType() const;
		// Returns false; this is a secondary cluster.

};

#endif // qGrayscaleAppearance

EXPORT_MA  void InitUCluster();

#endif