// UPopup.h
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UPOPUP__
#define __UPOPUP__

//----------------------------------------------------------------------------------------
// Theory of Operation
// ===================
//
// This unit allows Popup menus to be instantiated which are managed by the Popup CDEF,
// included in System 7.0 and in the installer script for the Communications Toolbox.
//	
// To use this Unit and the Popup CDEF with MacApp applications that run under System 6.x,
// the Popup CDEF will need to be included in the application's resource fork.
//	
// This unit actually extends the capabilities provided by the Popup CDEF as follows:
//		allows for non resource-based MENUs
//		allows for hierarchical popups
//	
//	
// Usage
// =====
//
// For Sys 6.0 support: include the Popup CDEF in the resource fork of your application
// with the following lines in your app.r file:
//		include "Popup.rsrc";
//	
// Also, include either the CMNU or MENU resource in your application's resource fork.
//
// To handle the command number associated with the selected item in the popup, include
// code such as the following in the holding view (e.g. a TDialogView subclass) or in the
// document (this code was taken from DemoDialogs):
//
//		void TTestDialogView::DoEvent(EventNumber eventNumber...) // override 
//		
//		{
//			if ((eventNumber == mPopupHit) && (source == fMyPopup))
//			{
//				aCommandNumber = dynamic_cast<TPopup*>(source)->GetCurrentCommand();
//				switch (aCommandNumber)
//				{
//					case cMyCommand:
//					{
//					}
//				}
//			}
//		}
//	
// Limitations
//
// The first item in the popup can't be a hierarchical menu. This is actually a bug in the
// Popup CDEF as it exists in System 7.0b1 and earlier versions (including the Comm
// Toolbox release). A bug report has been filed with SQA.
//----------------------------------------------------------------------------------------

// MacApp

#ifndef __UCONTROL__
#include "UControl.h"
#endif

#if qGrayscaleAppearance
#	ifndef __CPopup_AC__
#	include "CPopup_AC.h"
#	endif
#endif // qGrayscaleAppearance

#ifndef __UMenuMgr__
#include "UMenuMgr.h"
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const	bool		kUseAddResMenu	= true;
const	bool		kDontUseAddResMenu	= !kUseAddResMenu;

const	short		kDynamicMenuID = 0;	// use for menuID; a unique one will get assigned.

//----------------------------------------------------------------------------------------
// TPopup: supports a simple popup with minimal support for hierarchical popups.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPopup : public TCtlMgr					// A typical popup menu.
{
	MA_DECLARE_CLASS;
	
public:
	TPopup(const CViewPoint& itsLocation = gZeroVPt,
								const CViewPoint& itsSize = gZeroVPt,
								SizeDeterminer itsHSizeDet = sizeVariable,
								SizeDeterminer itsVSizeDet = sizeVariable,
								short itsMenuID = kNoResource,				// rsrc ID of 'MENU' or 'CMNU' rsrc (or kDynamicMenuID)
								short itsCurrentItem = 0,
								short itsItemOffset = 0,					// width of the popup title area 
								short itsStrListID = kNoResource,			// popup's title:	STR# rsrc id
								short itsIndex = kNoResource,				//					index into STR#
								/* Style */ short itsStyle = popupTitleNoStyle, // title's style 
								short itsJust= popupTitleLeftJust,			// title's justification 
								bool useAddResMenu = false,					// true if want to AddResMenu (can use
																			// this option iff menuID != kNoResource)
								ResType useAddResMenuResType = kNoIdentifier, // used if useAddResMenu is true
								const CTextStyle_AC& itsTextStyle = gSystemStyle); // popup's text style
		// Initializes the popup.
		// For dynamic menus, pass kDynamicMenuID for the menuID. We'll assign it when we create the menu. 
		
	virtual ~TPopup();

	virtual unsigned short GetPopupTitleStyle();
		// map fTitleStyle and fTitleJust to popupTitleStyle.

	virtual short GetProcID();
		// return the appropriate proc id

	virtual void CreateCMgrControl(const CStr255_AC&	itsTitle,
										  long		itsVal,
										  long		itsMin,
										  long		itsMax,
										  short 	itsProcID); // override 

	//------------------------------------------------------------------------------------
	// Standard signature support.
	//------------------------------------------------------------------------------------

	virtual IDType GetStandardSignature() const;	// override 
		// Returns this class's standard signature.

	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual void ReadFields(CStream_AC* aStream);	// override 
	
	virtual void WriteFields(CStream_AC* aStream) const;	// override 

	//------------------------------------------------------------------------------------
	// Imaging
	//------------------------------------------------------------------------------------

	virtual bool Focus(); // override 

	virtual void Draw(const CViewRect& area); // override 
		// sets a patch to GetResource in certain cases 

	virtual CViewRect CalcMinFrame(); // override
		// Compute the minimum size of this view. 

	//------------------------------------------------------------------------------------
	// tracking
	//------------------------------------------------------------------------------------

	virtual void DoMouseCommand(CViewPoint&			theMouse,
									   TToolboxEvent*	event,
									   CPoint_AC			hysteresis); // override 

	//------------------------------------------------------------------------------------
	// key events
	//------------------------------------------------------------------------------------
	
	virtual void DoKeyEvent(TToolboxEvent* event); // override
		// Handles all keystrokes except those with the command key held down.

	virtual void DoKeySelection(const CStr255_AC& selectionString); // override
        // This method is called by a TKeySelectionBehavior which can
        // be added to a TEventHandler and any of its derived classes.

	//------------------------------------------------------------------------------------
	// accessors
	//------------------------------------------------------------------------------------

	virtual void AttachMenuRef(MenuRef itsMenuRef
#if qGrayscaleAppearance
	, CPopup_AC::DisposalKind disposalKind = CPopup_AC::kDontDispose, 
	  CPopup_AC::RefHandling refHandling = CPopup_AC::kReload
#endif
	);
		// attaches itsMenuRef to this popup 

	virtual MenuRef GetMenuRef();
		// returns the MenuRef by accessing the control handle's private storage 
		
	virtual CStr255_AC GetMenuLabel();
		//  returns the menu label 

	virtual CommandNumber GetCurrentCommand();
		// For the currently selected menu item, returns its CommandNumber. 

	virtual short GetCurrentItem() const;
		// Returns the value of the current item.

	virtual short GetLastItem();
		// Returns the value of the last selected item.
		// For a hierarchical popup menu, this is different than GetCurrentItem().

	inline short GetMenuID() const { return fMenuID; }
		// Returns the menu ID of the popup.
		
	virtual short GetLastMenuID();
		// Returns the menu id of the last selected menu.
		// For a hierarchical popup menu, this is different than GetMenuID().
			
	virtual CStr255_AC GetItemText(short item);
		// Returns in "theText" the text for the specified item in the menu. 
		
	virtual short GetNumberOfItems();
		// Returns the number of items in the menu. 
		
	virtual CStr255_AC GetTitle() const;
		// returns the popup's title 

	virtual void SetCurrentItem(short item, bool redraw);
		// Sets the currently selected item, redrawing if required. 
		
	virtual void SetPopup(MenuRef itsMenuRef,
								 short itsCurrentItem,
								 bool redraw);
		// Sets the fields of the popup to contain theMenu, with currentItem selected,
        // redrawing if required.

	virtual void SetTitle(const CStr255_AC& title);
		// sets the popup's title 

	//------------------------------------------------------------------------------------
	// Dynamic popup methods
	//------------------------------------------------------------------------------------
	
	void CreateMenu();
		// Create the menu using NewMenu. 

	void ReinsertMenu();
		// Insert the menu back in the menu bar if it got cleared out. 

	void FreeMenu();
		// Dispose of fMenuRef. Code split off from AttachMenuRef. 

	//------------------------------------------------------------------------------------
	// Convenience methods
	//------------------------------------------------------------------------------------
	
	void SetMenuItemText(short item, const CStr255_AC& theText);
	
	void AddItem(const CStr255_AC& theText);
	
	inline void AddSeparator() { AddItem("-"); EnableItem(GetNumberOfItems(), false); }
	
	void InsertItem(short item, const CStr255_AC& theText);
	
	void DeleteAll();
	
	void DeleteItem(short item);
	
	void SetItemStyle(short item, short chStyle);
	
	void EnableItem(short item, bool canDo);
	
	void EnableCheckItem(short item, bool canDo, bool checked);

#if qGrayscaleAppearance

	// MacApp overrides
	
	virtual void SetText(const CStr255_AC& title, bool redraw);

	// AGA overrides

	virtual bool IsEmulatingControls() const;
		// Can't emulate if need to list resources (not supported in AGA)
		
	virtual void CreateAGAControl();
		// Subclasses override to call new with an CControl_AC subclass
	
	virtual CControl_AC* GetAGAControl() const { return fAGAControl; }
		// Returns AGA control instance

	inline CPopup_AC* GetAGAPopup() const { return fAGAControl; }
		// Returns AGA checkbox instance

protected:

		CAutoPtr_AC<CPopup_AC>	fAGAControl;
#endif

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	ResType		fUseAddResMenuResType;
	ResNumber	fMenuID;
	ResNumber	fStrListID;						// popup's title:	STR# rsrc id
	short		fItemOffset;
	short		fTitleJust;
	short		fUseAddResMenuNumberInitialItems;
	short		fIndex;							//					index into STR#
	Style		fTitleStyle;
	bool		fUseAddResMenu;
	bool		fMacAppSetDimState;

public:	
	static TMenuTable fgDynamicMenus;
	
	static void AddDynamicMenu(MenuRef itsMenuRef);
	
	static void RemoveDynamicMenu(MenuRef itsMenuRef);
	
	static MenuRef GetDynamicMenuRef(MenuID menuID);
	
	static short UniqueMenuID();
	
protected:
	MenuRef			fMenuRef;		// Our own menu. We kill it, we eat it. 
};
	

EXPORT_MA extern void InitUPopup();

#endif
