// UPrinting.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.

#ifndef __UPRINTING__
#define __UPRINTING__

// ACS

#ifndef __MDependable_AC__
#include "MDependable_AC.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UPRINTHANDLER__
#include "UPrintHandler.h"
#endif

#ifndef __USTREAMABLE__
#include "UStreamable.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#if qGXPrinting && !defined(__GXGRAPHICS__)
#include <GXGraphics.h>
#endif

// ANSI

#ifndef __LIMITS__
#include <limits.h>
#endif


//----------------------------------------------------------------------------------------
//  Constants for the print info resources
//----------------------------------------------------------------------------------------

const ResType	kGXPrintInfoRsrcType = 'FJOB';
const ResNumber	kGXPrintInfoRsrcID = 128;

const ResType	kPrintInfoRsrcType = 'PREC';
const ResNumber	kPrintInfoRsrcID = 128;

const ResType	kAlternatePrintInfoRsrcType = 'PRNT';
const ResNumber	kAlternatePrintInfoRsrcID = 128;

//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TDocument;
class TView;
class TPrintInfo;

//----------------------------------------------------------------------------------------
// Forward and external structures and pointers. 
//----------------------------------------------------------------------------------------
//
// These keep us from having to include any of the printing interfaces in our interface
// Since these are blind pointers they should not ever have to change much
//
#if qGXPrinting
struct gxPrivateJobRecord;
typedef struct gxPrivateJobRecord *gxJob;

struct gxEditMenuRecord;
#endif

struct TPrint;
typedef struct TPrint TPrint, *TPPrint, **THPrint;

struct TPrPort;
typedef struct TPrPort TPrPort, *TPPrPort;

//----------------------------------------------------------------------------------------
// Command constants: Printing-related command numbers which apply only when a view with a
// non-NULL PrintHandler is in the target chain
//----------------------------------------------------------------------------------------

const CommandNumber cChangePrinterStyle = 192;		// Command arising from user's making
                                                	// changes in Page Setup... dialog

const CommandNumber cShowBreaks = 201;				// Show Page Breaks command


//----------------------------------------------------------------------------------------
// Alert rsrc IDs
//----------------------------------------------------------------------------------------

const ResNumber phSpoolPrintDialog = 257;		// the dialog shown when printing a file
                                                // (a misnomer - not just when spooled)

const ResNumber phFinderPrintDialog = 258;		// the dialog shown when doing Finder
                                                // Printing; differs from prev. by
                                                // addition of a 'Cancel All Printing'
                                                // button

const ResNumber phNoPages = 251;				// The "No pages within requested range"
                                                // alert

//----------------------------------------------------------------------------------------
// Dialog rsrc IDs
//----------------------------------------------------------------------------------------

const ResNumber phWhichDoc = 261;				// The "Preparing to print <docname>"
                                                // dialog used during Finder printing


//----------------------------------------------------------------------------------------
// Miscellaneous
//----------------------------------------------------------------------------------------

const short kUsualPages = SHRT_MAX;				// For methods that take a pageNumber
                                                // argument, this value indicates a
                                                // request for a general setting rather
                                                // than a setting customized to one
                                                // particular page

const ResNumber kPrintDriverName = -8192;		// STR resource in System file which
                                                // contains name of the default printer




//----------------------------------------------------------------------------------------
// Common Ancestor for printjob objects from different print architectures (MacPrint, GX, etc)
//----------------------------------------------------------------------------------------

class EXPORT_MA TPrintInfo : public MDependable_AC, public MStreamable
{
	MA_DECLARE_CLASS;
	
public:
	TPrintInfo();
	
	TPrintInfo(const TPrintInfo& inPrintInfo);
		// Copy Constructor

	virtual ~TPrintInfo();
	
	virtual void ReadFrom(CStream_AC* aStream);
		// Read the fHPrint from the stream

	virtual void SetFrom(TPrintInfo* jobToCopy);
		// copy settings from another job

	virtual long GetSize(bool asResource = true) const;
		// return the number of bytes necessary to store the print handle
		// asResource lets you make a distinction in the storage type

	virtual void DoUpdate(ChangeID_AC theChange,
				  MDependable_AC* changedObject,
				  void* changeData,
				  CDependencySpace_AC* dependencySpace);	// Override
		// If the printer has changed then we're invalid

	virtual CViewRect GetPaperRect();
		// returns the paper's rect

	virtual CViewRect GetInkRect();
		// returns the imagable page's rect

	virtual CPoint_AC GetDeviceRes();
		// returns the current device's resolution vertically and horizontally

	virtual void SetToDefault();
		// Set the print Info to the default state

	virtual bool IsSetToDefault();
		// Returns true if this Matches the printer's default

	virtual void GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage);
		// Returns the page range of the job to print

	virtual void SetPrintPages(long fromPage,
							   long toPage,
							   long minFromPage,
							   long maxToPage);
		// Sets the page range of the job to print

	virtual long GetPrintCopies();
		// Returns the number of copies specified in the job

	virtual void SetPrintCopies(long numCopies);
		// Sets the number of copies specified in the job

	virtual void SetTargetPrinter(const CFSSpec_AC& targetPrinter);
		// Sets the target printer for a "drag print"

	virtual void SetSpoolFile(CFSSpec_AC& spoolFile);
		// Sets the location for the spool file

	virtual bool IsSpooling();
		// Returns true if we are spooling and not just drafting straight to the printer

	virtual bool IsMatch(TPrintInfo* thePrintInfo);
		// Returns true if thePrintInfo can be said to match. false if they
		// don't match or it can't be determined without validating them.
		// thePrintInfo doesn't have to be Valid but 'this' must be Valid
		// if it is not waiting for a default call

	virtual void Open(void* byWhom);
		// Attempts to open the printer. Signals failure if an error occurs.

	virtual bool IsOpen();
		// Returns true if the printer is open.

	virtual void Close(void* byWhom);
		// Closes the Printer. DOES NOT signal failure if an error
        // occurs, only reports it to the debug window.

	bool IsDefaultForPrinter(TPrintInfo* thePrintInfo);
		// Returns true if thePrintInfo matches the default for this printer.

	virtual OSErr GetError();
		// Return any current printing error.

	virtual void SetError(OSErr err);
		// Set the current printing error.

	virtual void ClearError();
		// Clear any current printing error.

	virtual bool ChkPrintErr(OSErr& err,
										  bool& ranOutOfSpace);

	virtual void StartJob(const CStr255_AC& docTitle,
						 long numPages);
		// Start a printing job.

	virtual GrafPtr GetJobPort();
		// Returns the port that the job is printing to

	virtual void FinishJob();
		// Completes a printing job.

	virtual void StartPage(long	pageNumber);
		// Start a page.

	virtual void FinishPage();
		// Finish a page.

	virtual void JobMerge(TPrintInfo* mergeFrom);
		// Validate and Merge the job info from the source job onto the
		// destination job and Validate the result

	virtual TPrintInfo* GetDefaultPrintInfo();
		// Returns THE default TPrintInfo. Creating it first, if neccessary
		
	virtual bool CallJobDialog();
		// Calls PrJobDialog.
		
	virtual bool CallStyleDialog();
		// Calls PrStlDialog.
		
	virtual bool CallPrValidate();
		// Validates the printInfo.
		
	virtual void CallPrintDefault();
		// Calls PrintDefault.

	virtual bool CallPrPicFile(OSErr& err);
		// Calls PrPicFile.

	virtual void Invalidate();
		// Mark as invalid and issue a changed message

	virtual void Update();
		// Check if the target printer has changed and inform dependents if necessary
		// Ensures that the print info is current and valid

	void GetMaxSquareResolution(Fixed& hRes, Fixed& vRes);
		// Return the maximum square resolution supported by the printer

	void SetResolution(Fixed hRes, Fixed vRes);
		// Sets the printer to use the supplied resolution

	void MaximizeResolution();
		// Sets the printer to use the maximum square resolution

	virtual short GetScale();
		// returns the current print scaling as a whole percent. 100%=100

	inline bool IsValid()
	{ return fIsValid; }
		// return the fIsValid flag

	inline bool NeedsDefault()
	{ return fNeedsDefault; }
		// return the fNeedsDefault flag

	virtual void PosePrintDialog(const CStr255_AC& docName, CommandNumber aCommandNumber);
		// Put up the 'Printing in Progress' dialog ( if applicable )

	virtual void BanishPrintDialog();
		// Get rid of the print dialog.

	virtual void ShowDocBeingPrinted(const CStr255_AC& docName, bool entering);
		// Put up (or remove) a window whose title is the document's title, because the
        // Print Manager gets the document title from the front window.

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	bool	fIsValid;					// true if the print record is valid
										// because it's been validated OR
										// is the result of a setup dialog or sumthin'.
	bool	fNeedsDefault;				// true if the print record is valid
};


#if qGXPrinting
//----------------------------------------------------------------------------------------
// Encapsulates the the gxJob, etc for GX printing
//----------------------------------------------------------------------------------------

struct MySpoolDataRec
{
	gxRectangle pageArea;					// Page rectangle.
	gxViewPort printViewPort;				// ViewPort we're printing with.
};

class EXPORT_MA TGXJob : public TPrintInfo
{
	MA_DECLARE_CLASS;
	
public:

	TGXJob(gxJob itsgxJob= NULL, bool isValid = false);
		// If itsgxJob is NULL the default job is retrieved from itsPrintMgr.
		// If itsPrintMgr is NULL the gDefaultPrinter will be used.
	
	TGXJob(const TGXJob& inGXJob);
		// Copy Constructor

	virtual ~TGXJob();
	
	virtual void ReadFrom(CStream_AC* aStream);
		// Read the gxJob from the stream

	virtual void ConvertFrom(CStream_AC* aStream);
		// Read the THPrint from the stream

	virtual void WriteTo(CStream_AC* aStream) const;
		// Write the fHPrint to the stream

	virtual void SetFrom(TPrintInfo* jobToCopy);
		// copy settings from another job

	void SetFrom(gxJob jobToCopy, bool isValid = false,
				bool needsDefault = false);
		// copy settings from another gxJob
		// set IsValid true if the handle is known to be valid

	virtual long GetSize(bool asResource = true) const;
		// return the number of bytes necessary to store the print handle
		// asResource lets you make a distinction in the storage type

	static pascal OSErr GetSizeHelper (long size, void *data, void *refCon);
		// Helps the GetSize function by counting the number of bytes in GXFlattenJob

	virtual CViewRect GetPaperRect();
		// returns the paper's rect

	virtual CViewRect GetInkRect();
		// returns the imagable page's rect

	virtual CPoint_AC GetDeviceRes();
		// returns the current device's resolution vertically and horizontally

	virtual void SetToDefault();
		// Set the print Info to the default state

	virtual bool IsSetToDefault();
		// Returns true if this Matches the printer's default

	virtual void GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage);
		// Returns the page range of the job to print

	virtual void SetPrintPages(long fromPage,
							   long toPage,
							   long minFromPage,
							   long maxToPage);
		// Sets the page range of the job to print

	virtual long GetPrintCopies();
		// Returns the number of copies specified in the job

	virtual void SetPrintCopies(long numCopies);
		// Sets the number of copies specified in the job

	virtual void SetTargetPrinter(const CFSSpec_AC& targetPrinter);
		// Sets the target printer for a "drag print"

	bool IsDefaultForPrinter(TPrintInfo* thePrintInfo);
		// Returns true if thePrintInfo matches the default for this printer.

	virtual OSErr GetError();
		// Return any current printing error.

	virtual void SetError(OSErr err);
		// Set the current printing error.

	virtual void StartJob(const CStr255_AC&			 ,
						long					);
		// Start a printing job.

	virtual GrafPtr GetJobPort();
		// Returns the port that the job is printing to

	virtual void FinishJob();
		// Completes a printing job.

	virtual void StartPage(long					);
		// Start a page.

	virtual void FinishPage();
		// Finish a page.

	virtual void JobMerge(TPrintInfo* mergeFrom);
		// Validate and Merge the job info from the source job onto the
		// destination job and Validate the result

	virtual bool CallJobDialog();
		// Calls the job dialog.
		
	virtual bool CallStyleDialog();
		// Calls the setup dialog.
		
	virtual bool CallPrValidate();
		// Validates the printInfo.
		
	virtual void CallPrintDefault();
		// Calls PrintDefault.

	virtual void Update();
		// Check if the target printer has changed and inform dependents if necessary
		// Ensures that the print info is current and valid

	virtual void DoUpdate(ChangeID_AC theChange,
				  MDependable_AC* changedObject,
				  void* changeData,
				  CDependencySpace_AC* dependencySpace);	// Override
		// Check for printer driver changes on mAboutToLoseContro/mRegainControl

	void GetMaxSquareResolution(Fixed& hRes, Fixed& vRes);
		// Return the maximum square resolution supported by the printer

	void SetResolution(Fixed hRes, Fixed vRes);
		// Sets the printer to use the supplied resolution

	virtual short GetScale();
		// returns the current print scaling as a whole percent. 100%=100

protected:
	gxJob GetgxJob();
		// Returns the fgxJob after Validation

	inline gxJob GetRawgxJob() const { return fgxJob; }
		// Returns the fgxJob without Validation

	void AdoptJob(gxJob itsgxJob);
		// call to adopt a job that's new to this info

	virtual void PrepareEditRecord(gxEditMenuRecord* editMenuRecord);
		// fill in an edit record for the setup/job dialogs

	//----------------------------------------------------------------------------------------
	// static member functions
	//----------------------------------------------------------------------------------------
protected:

// These "C++" calling convention routines will be called via UPPs
#if defined(__MWERKS__) && !(__POWERPC__ || __CFM68K__)
#	pragma d0_pointers on
#endif
	static OSErr PrintingEventOverride(EventRecord* anEvent, bool filterEvent);

	static OSErr MyPrintAShape(gxShape currentShape, long refCon);
		// MyPrintAShape - This is a spool proc. for the translator
		// routines. In this routine, we simply draw each shape
		// we're passed (printing it).  You could also capture each
		// shape into one gxPicture shape, and then do one draw of
		// that afterwards.
#if defined(__MWERKS__) && !(__POWERPC__ || __CFM68K__)
#	pragma d0_pointers reset
#endif

	static pascal OSErr MyUnflattenFunction(long size, void *data, void *refCon);
		// MyUnflattenFunction - This proc will get called when unflattening
		// a gxjob.

	static pascal OSErr MyFlattenFunction(long size, void *data, void *refCon);
		// MyFlattenFunction - This proc will get called when flattening
		// a gxjob.


	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	gxJob	fgxJob;						// Default, validated job
	gxViewPort fPrintViewPort;			// ViewPort for printing

	CGrafPort fQDPortRec;				// Printer Port in use during printing
	CGrafPtr fQDPort;					// Printer Port in use during printing
	MySpoolDataRec fSpoolData;			// area to pass stuff to the PrintAShape routine
};
#endif


//----------------------------------------------------------------------------------------
// Encapsulates the THPrint for MacPrint printing
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Manage entrance and exit from MacPrint
//----------------------------------------------------------------------------------------

class EXPORT_MA CInMacPrint
{
public:
	CInMacPrint(TPrintInfo* itsPrintinfo = NULL, void* byWhom = NULL, bool active = true);
	// Puts us in the context of MacPrint.
	// if itsPrintMgr is NULL use the gDefaultPrinter
	// if byWhom is NULL use itsPrintMgr
	// If active is false, nothing happens.

	~CInMacPrint();
	// Removes the MacPrint context

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	TPrintInfo*	fPrintInfo;						// The TPrintInfo used by us
	void*		fByWhom;						// the object responsible for this
	short		fResFile;						// The CurResFile before opening the print shop
	bool		fIsActive;						// True if we are going to activate.
	bool		fPrintShopOpened;				// True if the print shoppe opened successfully
};


class EXPORT_MA TMacPrintJob : public TPrintInfo
{
	MA_DECLARE_CLASS;
	
public:

	TMacPrintJob(THPrint itsHPrint = NULL, bool isValid = false);
		// If itsHPrint is NULL the default HPrint is retrieved from gDefaultPrintMgr
	
	TMacPrintJob(const TMacPrintJob& inMacPrintJob);
		// Copy Constructor

	virtual ~TMacPrintJob();
	
	virtual void ReadFrom(CStream_AC* aStream);
		// Read the fHPrint from the stream

	virtual void WriteTo(CStream_AC* aStream) const;
		// Write the fHPrint to the stream

	virtual void SetFrom(TPrintInfo* jobToCopy);
		// copy settings from another job

	void SetFrom(THPrint hPrintToCopy, bool isValid = false,
				bool needsDefault = false);
		// copy settings from another THPrint
		// set IsValid true if the handle is known to be valid

	virtual long GetSize(bool asResource = true) const;
		// return the number of bytes necessary to store the print handle
		// asResource lets you make a distinction in the storage type

	virtual CViewRect GetPaperRect();
		// returns the paper's rect

	virtual CViewRect GetInkRect();
		// returns the imagable page's rect

	virtual CPoint_AC GetDeviceRes();
		// returns the current device's resolution vertically and horizontally

	virtual void SetToDefault();
		// Set the print Info to the default state

	virtual bool IsSetToDefault();
		// Returns true if this Matches the printer's default

	virtual void GetPrintPages(long& fromPage,
							   long& toPage,
							   long& minFromPage,
							   long& maxToPage);
		// Returns the page range of the job to print

	virtual void SetPrintPages(long fromPage,
							   long toPage,
							   long minFromPage,
							   long maxToPage);
		// Sets the page range of the job to print

	virtual long GetPrintCopies();
		// Returns the number of copies specified in the job

	virtual void SetPrintCopies(long numCopies);
		// Sets the number of copies specified in the job

	virtual void SetSpoolFile(CFSSpec_AC& spoolFile);
		// Sets the location for the spool file

	virtual bool IsSpooling();
		// Returns true if we are spooling and not just drafting straight to the printer

	virtual bool IsMatch(TPrintInfo* thePrintInfo);
		// Returns true if thePrintInfo can be said to match. false if they
		// don't match or it can't be determined without validating them.
		// thePrintInfo doesn't have to be Valid but 'this' must be Valid
		// if it is not waiting for a default call

	virtual void Open(void* byWhom);
		// Attempts to open the printer. Signals failure if an error occurs.

	virtual bool IsOpen();
		// Returns true if the printer is open.

	virtual void Close(void* byWhom);
		// Closes the Printer. DOES NOT signal failure if an error
        // occurs, only reports it to the debug window.

	virtual OSErr GetError();
		// Return any current printing error.

	virtual void SetError(OSErr err);
		// Set the current printing error.

	virtual void StartJob(const CStr255_AC&			 ,
						 long					);
		// Start a printing job.

	virtual GrafPtr GetJobPort();
		// Returns the port that the job is printing to

	virtual void FinishJob();
		// Completes a printing job.

	virtual void StartPage(long					);
		// Start a page.

	virtual void FinishPage();
		// Finish a page.

	virtual void JobMerge(TPrintInfo* mergeFrom);
		// Validate and Merge the job info from the source job onto the
		// destination job and Validate the result

	virtual bool CallJobDialog();
		// Calls the job dialog.
		
	virtual bool CallStyleDialog();
		// Calls the setup dialog.
		
	virtual bool CallPrValidate();
		// Validates the printInfo.
		
	virtual void CallPrintDefault();
		// Calls PrintDefault.

	virtual bool CallPrPicFile(OSErr& err);
		// Calls PrPicFile.

	virtual void Update();
		// Check if the target printer has changed and inform dependents if necessary
		// Ensures that the print info is current and valid

	virtual void DoUpdate(ChangeID_AC theChange,
				  MDependable_AC* changedObject,
				  void* changeData,
				  CDependencySpace_AC* dependencySpace);	// Override
		// Check for printer driver changes on mAboutToLoseContro/mRegainControl

	void GetMaxSquareResolution(Fixed& hRes, Fixed& vRes);
		// Return the maximum square resolution supported by the printer

	void SetResolution(Fixed hRes, Fixed vRes);
		// Sets the printer to use the supplied resolution

	void MaximizeResolution();
		// Sets the printer to use the maximum square resolution

	virtual void DoPrintIdling();
		// Print job idle time handler. Handles events in the 'Print Dialogs' which give
        // the user 'Cancel', and, in the case of printing from the Finder, 'Cancel All
        // Printing.'

	virtual void PosePrintDialog(const CStr255_AC& docName, CommandNumber aCommandNumber);
		// Put up the 'Printing in Progress' dialog ( if applicable )

	virtual void BanishPrintDialog();
		// Get rid of the print dialog.

	virtual void ShowDocBeingPrinted(const CStr255_AC& docName, bool entering);
		// Put up (or remove) a window whose title is the document's title, because the
        // Print Manager gets the document title from the front window.

	virtual short GetScale();
		// returns the current print scaling as a whole percent. 100%=100

protected:
	THPrint GetHPrint();
		// Returns the fHPrint after Validation

	inline THPrint GetRawHPrint() const { return fHPrint; }
		// Returns the fHPrint without Validation

	//----------------------------------------------------------------------------------------
	// static member functions
	//----------------------------------------------------------------------------------------
public:
	static unsigned long CheckPrinterChangeLevel();
		// Determine if the user has selected a new printer type from the chooser.

	static void GetDriverName(CStr255_AC& driverName);
		// Get the name of the current printer driver.

	static pascal void IdleProc();
	// Procedure whose address is passed to the print job for calling as often as
    // possible. Forwards to the DoPrintIdling method of the job's print handler.
	// NOTE: this routine requires the pascal keyword because it is a Toolbox callback

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	CStr255_AC fCurrentDocTitle;				// Name of the current document being printed
	DialogRef fPrintDialog;					// Used for all printing-related dialogs.
                                            // ??? (shouldn't this really be a view
 											// (or TDialogView if using UDialog)

	TemplateTempHandle<THPrint> fHPrint;	// The THPrint for this job
	long fMinFromPage;						// The minimum page that can be printed (for this jobs document)
	long fMaxToPage;						// The maximum page that can be printed (for this jobs document)
	
	unsigned long fPrinterChangeLevel;  	// increases every time the name change is noticed.

	static TPPrPort fQDPort;				// Printer Port in use during printing
	static short fPrintShopOpens;			// counter for balancing Open and Close

	static CStr255_AC fgLastPrinterName;			// printer that we're pointing at
	static unsigned long fgPrinterChangeLevel; // increases every time the name change is noticed.
};


//----------------------------------------------------------------------------------------
// TStdPrintHandler: Actually Implements all the features that were stubbed out in its
// ancestor. Basic flow of control at print time: Print, Validate, For each page(
// FocusOnBorder, DrawPageInterior, FocusOnBorder, AdornPage) are called.
//----------------------------------------------------------------------------------------

class EXPORT_MA TStdPrintHandler : public TPrintHandler, public MDependable_AC
{
	MA_DECLARE_CLASS;
	
public:
	//------------------------------------------------------------------------------------
	// Initialization/ Termination
	//------------------------------------------------------------------------------------

	TStdPrintHandler(TDocument* itsDocument = NULL,
						  TView* itsView = NULL,
						  bool itsHFixedSize = kFixedSize,
						  bool itsVFixedSize = kFixedSize);
		// Initializes the printHandler and installs it in the view; 
        // Uses the default TPrintInfo if itsPrintInfo is NULL
		// By default the pages are considered fixed size.

	TStdPrintHandler(const TStdPrintHandler& inStdPrintHandler);
		// Copy Constructor

	virtual ~TStdPrintHandler();
		// If relevant, frees the PrintInfo record and does other cleanup before freeing
        // this


	//------------------------------------------------------------------------------------
	// Page Strips
	//------------------------------------------------------------------------------------

	virtual CViewPoint CalcPageStrips();
		// Recalculate the number of strips of pages in each dimension.

	virtual CViewPoint CalcViewPerPage();
		// Decide how much view (h and v) normally gets mapped into page interiors.

	virtual ViewCoordinate BreakFollowing(VHSelect vhs,
									   ViewCoordinate previousBreak,
									   bool& automatic);
		// Returns the location of the page break which follows the page break located at
        // 'previousBreak', in direction vhs; returns automatic = true if the page-break is
        // thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note
        // that page-breaks in dimension 'v' are drawn as vertical lines, those in
        // dimension 'h' as horizontal lines. Generic support at present is only provided
        // for 'automatic' page breaks; the 'automatic' parameter in BreakFollowing is a
        // hook to allow clients to support manual page breaks as well

	virtual void GetBreakCoord(VHSelect vhs,
							   long whichBreak,
							   ViewCoordinate& loc);
		// Returns, in 'loc', the coordinate of the 'whichBreak-th' page break in
        // dimension vhs. For example, GetBreakCoord(v, 5, loc) will return, in 'loc', the
        // horizontal coordinate of the vertical line (page-break) which marks the
        // boundary the area of the view which will be mapped into the 4th vertical strip
        // of pages and the area of the view which will be mapped into the 5th vertical
        // strip of pages

	virtual CViewPoint PageToStrip(long pageNumber);
		// Returns the horizontal and vertical strip for the given page.

	virtual long StripToPage(CViewPoint pageStrip);
		// Returns the page number for the given page strip.


	//------------------------------------------------------------------------------------
	// General Utility
	//------------------------------------------------------------------------------------

	virtual CStr255_AC GetDocName();
		// Returns the name of the document being printed, for use in the print dialog.

	virtual long MaxPageNumber();
		// Returns the highest page number which would be printed if the user requested
        // that all pages be printed.

	virtual void AdornPage();
		// Draw things on page # 'aPageNumber' which do not arise from the View but rather
        // from page considerations; examples: Headers and Footers; page numbers; frames
        // drawn around inset text; row and column headers for spreadsheets.

	virtual void ChooseSpoolFile(CFSSpec_AC& theSpoolFile,
								 long& pagesPerSubjob);
		// Select the filename and volume to use for spooling; default chooses empty
        // CPascalStr_AC on vol 0, which results in MacPrint's automatically choosing the local
        // Boot volume

	virtual void DrawPageInterior();
		// Make the QuickDraw calls which will result in the 'Interior' of page #
        // 'aPageNumber' being drawn. The 'Interior' is those things on the page which
        // arise from drawing the View associated with the PrintHandler

	virtual bool Focus();	// override
		// Set up the GrafPort, clipping, etc., to its values prior to printing a view

	virtual void FocusOnBorder();
		// Set up GrafPort, clipping, etc., prior to calling AdornPage

	virtual void FocusOnInterior();
		// Set up the GrafPort, clipping, etc., prior to calling DrawPageInterior

	virtual long GetDefaultSubJobSize();
		// Return default subjob size: SHRT_MAX

	virtual CPoint_AC GetQDOrigin();	// override
		// Overridden to return fQDOrigin.

	virtual CViewPoint GetViewToQDOffset();
		// Overridden to return fViewToQDOffset.

	virtual CViewPoint LocatePageInterior(long pageNumber);
		// Determine where to locate the top-left corner of the Interior of the given page
        // number

	virtual bool OneSubJob(long subjobFirstPage,
								long subjobLastPage,
								long minFromPage,
								long maxToPage,
								bool justSpool,
								bool partialJob,
								bool& ranOutOfSpace,
								long& lastPageTried);
		// Processes one subjob for printing. Where appropriate, a print job is divided
        // automatically into one or more 'subjobs', so that a long document can be
        // printed even if there is very limited spool space available on the disk. This
        // is a generalization of the 'spool-a-page/ print-a-page' technique.
        // TStdPrintHandler will use a subjob size as large as can be handled by current
        // conditions, and will only fall back to more subjobs, each of smaller size, if
        // it experiences problems running with a larger subjob size
		// Returns true to proceed

	virtual bool Print(CommandNumber itsCommandNumber);
		// Carries out printing for the print handler.
		// Returns true to proceed

	virtual void PrintPage(long aPageNumber);
		// Print the indicated page

	virtual void SetPage(long aPageNumber);
		// Forwards to TView::DoSetInterior and TView::DoSetPageOffset

	virtual void SetPageInterior(long pageNumber);
		// Responsible for installing the correct values into fPageAreas.theInterior,
        // representing the 'interior' of the page in View coordinates.

	virtual void SetPageOffset(const CViewPoint& coord);
		// Given the view-coordinate of the top-left-most CPoint_AC of the view which is
        // mapped into the interior of this page, calculate and set the offset for the
        // print code.


	//------------------------------------------------------------------------------------
	// Formatting for printing and other Setup methods
	//------------------------------------------------------------------------------------

	virtual void Invalidate();
		// Mark as invalid and issue a changed message

	virtual void Update();
		// Recompute the projection model and notify views if it changed

	virtual void InstallMargins(const CViewRect& newMargins,
								bool areMinimalMargins);
		// Install the (possibly new) margins specified; if 'areMinimalMargins' is true,
        // then the minimal-margins scheme is used; in this scheme, margins are always
        // maintained at exactly the size necessary to have the page Interior coincide
        // exactly with the printable area of the page

	virtual void MarginsTooBig(CViewRect &margins);
		// called by InstallMargins if the passed-in margins are too big for the current paper.
		// On entry, margins is the bad margins, on exit are (presumably) good margins.
		// The default version of this routine sets the margins so that theInterior == theInk
		
	virtual void PrinterChanged();
		// After printer specifications have changed, absorb the new metrics and reformat
        // the view if necessary

	virtual void RedoPageBreaks();
		// Recalculate the page breaks

	virtual void Reset();
		// Reset print record to default

	virtual TPrintInfo* GetPrintInfo();
		// Returns the printinfo (fPrintInfo)

	virtual void SetDefaultPrintInfo();
		// Create a PrintInfo record and initialize it to default values

	virtual void SetPrintExtent();
		// Sets the part of the print handler's view that is printed by this print
        // handler, by calling the view's GetPrintExtent.

	virtual void SetMargins();
		// Install the desired page margins

	virtual void DoUpdate(ChangeID_AC theChange,
				  MDependable_AC* changedObject,
				  void* changeData,
				  CDependencySpace_AC* dependencySpace);
		// If the printer/printInfo has changed then we're invalid

	//------------------------------------------------------------------------------------
	// Scripting Support
	//------------------------------------------------------------------------------------

	virtual bool DoScriptCommand(CommandNumber aCommandNumber,
									TAppleEvent* message,
									TAppleEvent* reply); 	// override
		// Handle the print apple event

	//------------------------------------------------------------------------------------
	// Menu Commands
	//------------------------------------------------------------------------------------

	virtual void DoMenuCommand(CommandNumber aCommandNumber);
		// Handles 'show page breaks', among others

	virtual bool DoPrintCommand(CommandNumber aCommandNumber);
		// called by DoMenuCommand, handles posting the print command, returns true if
        // handled

	virtual void DoSetupMenus();
		// Sets up menus for 'show page breaks' and company

	virtual void DoSetupPrintMenus();
		// called by DoSetupMenus, sets up the print menus


	//------------------------------------------------------------------------------------
	// Finder Printing
	//------------------------------------------------------------------------------------
	
	virtual bool SetupForFinder(CommandNumber aCommandNumber);
		// Sets up the print record for finder printing. If fFinderPageSetup is true, then
        // PosePageSetupDialog is called. If fFinderJobDialog is true, then PoseJobDialog
        // is called. Otherwise, prJobMerge is called to merge the job record from the

	virtual bool SetupPrintOne(CommandNumber aCommandNumber);
		// Sets up the print record for a "print one" type of operation, one where the
        // entire document or view is printed and no job dialog takes place. This is also
        // used for "drag printing". Returns true if we should proceed.


	//------------------------------------------------------------------------------------
	// Screen Feedback
	//-----------------------------------------------------------------------------------

	virtual void DrawPrintFeedback(const CViewRect& area);
		// Gateway to all screen feedback related to pagination issues

	virtual void DrawPageBreak(VHSelect vhs,
							   long whichBreak,
							   ViewCoordinate loc,
							   bool automatic);
		// Draws a page break on the screen.

	virtual void InvalPageFeedback();
		// Invalidates all page-related screen feedback

	virtual bool ShowsOnScreen();
		// Returns true only if the printHandler has an fView which is in an open window.


	//------------------------------------------------------------------------------------
	// Dialogs relating to printing
	//------------------------------------------------------------------------------------

	virtual bool PoseJobDialog();
		// Pose the 'Print Job' dialog

	virtual bool PosePageSetupDialog(bool isUndoable);
		// Undertake the 'Page Setup' dialog; post a command object to be performed if
        // relevant. Returns TRUE if the user did not cancel.

	virtual bool ChkPrintErr(OSErr& err,
								bool& ranOutOfSpace);
		// Checks for a MacPrint error, specially handles one particular error (sets
		// return parameter 'ranOutOfSpace' if error code indicates disk-space problem
		// while saving a spool file).
		// Returns TRUE to proceed

	virtual bool PrintSpoolFile(OSErr& err);
		// If printing was spooled by the print-driver then this method will be called to
        // print the spooled image.
		// Returns TRUE to proceed

	virtual CViewRect GetPaperRect();
	
	virtual CViewRect GetInkRect();
	
	virtual CViewRect GetMarginsRect();
	
	virtual CViewRect GetInteriorRect();

	virtual CViewPoint GetViewPerPage();

	//----------------------------------------------------------------------------------------
	// member functions that are really stubs (finally fleshed out here)
	//----------------------------------------------------------------------------------------

	virtual void TerminateUPrinting();
	// Call to terminate the printing system

	virtual void PrepareForFinderPrinting(const CFSSpec_AC* targetPrinter);
	// call this to prepare for printing from the finder
	// if targetPrinter is non-NULL then it specifies a "drag print" target

	virtual void DispatcherIsAvailable();
	// MacApp calls this when the dispatcher has been created and initialized.

	virtual long GetSavePrintInfoSize(TPrintInfo* itsPrintInfo, TFile* itsFile, bool useRsrcFork = true);
		// returns the number of bytes necessary to save the print info.

	virtual void SavePrintInfo(TPrintInfo* itsPrintInfo, TFile* itsFile, bool useRsrcFork = true);
		// Saves the print info to the passed in file.

	virtual TPrintInfo* RestorePrintInfo(TFile* itsFile, bool useRsrcFork = true);
		// Initializes the print info from the passed in file.

	virtual void DeleteIfPrintInfo(TPrintInfo* inPrintInfo);
		// delete the printinfo passed in.

	//------------------------------------------------------------------------------------
	// data members
	//------------------------------------------------------------------------------------
public:
	CViewRect fPrintExtent;							// the part of the view's extent that is
                                                // to be printed by this print handler.
                                                // The default is the entire extent, but
                                                // it can be changed with the method
                                                // SetAreaToPrint.

	CViewRect fViewedRect;							// The part of the view shown on the
                                                // "current" page. Set by SetPage.

	CViewPoint fViewToQDOffset;						// the computed translation of coordinates
												// necessary to account for adjustment of large
												// views down to QD space. This is set in
												// FocusOnInterior.
												
	CViewPoint fPageStrips;							// For printing a view which will take
                                                // more than one page to print, the view
                                                // is thought of as being divided up by
                                                // vertical and horizontal lines ('page
                                                // breaks') into a checkerboard of
                                                // 'pages'. A vertical column, one page
                                                // wide, of such pages is called a
                                                // 'vertical page strip', while a
                                                // horizontal row, one page high, of such
                                                // pages is called a 'horizontal page
                                                // strip'. The number of vertical page
                                                // strips is represented by fPageStrips.v,
                                                // and the number of horizontal page
                                                // strips is represented by fPageStrips.h
                                                // In a typical Word Processor
                                                // application, there is only one vertical
                                                // strip of pages, but many horizontal
                                                // 'strips', each one only one page wide.
                                                // In a graphical application, there may
                                                // be many vertical as well as horizontal
                                                // strips of pages. fPageStrips is
                                                // recomputed by method CalcPageStrips.

	CViewPoint fLastStrip;							// The last strip printed.

	CViewPoint fLastBreak;							// The coordinates of the last page break.

	CRect_AC fQDRectToClipTo;						// This the clip rectangle calculated in
												// FocusOnInterior and FocusOnBorder.  We
												// use it in ::Focus when a view or subview
												// being printed needs to focus on its
												// superview.

	CPoint_AC fQDOrigin;							// the computed QD origin for the port to
                                                // account for the area being printed.
												// This is set in FocusOnInterior and
												// FocusOnBorder.
												
	CAutoPtr_AC<TPrintInfo> fPrintInfo;			// The print record encapsulated in an object
                                                // The Print Record may be shared among all
                                                // PrintHandlers in a document or may be
                                                // unique to each PrintHandler, depending on
                                                // the flag TDocument::fSharePrintInfo

	CommandNumber fPrintingCommand;				// The command number that was used when
												// invoking some printing function.  This
												// can be used to determine if we are
												// printing from the finder.

	long fStartPage;							// the page number for the topleftmost
                                                // corner of the view. Normally 1, but can
                                                // be set to any positive value

	VHSelect fPageDirection;					// Determines how page numbers are to
                                                // advance in a large view which is more
                                                // than one page wide and more than one
                                                // page high; v means page 2 is BELOW page
                                                // 1; h means it's to its RIGHT.
                                                // Irrelevant if the view is only one page
                                                // wide or only one page high

	bool fFixedSizePages[2];					// Indicates whether pages are fixed size
                                                // horizontally and vertically.

	bool fFinderSetup;							// whether, during Finder Printing, the
                                                // 'Page Setup' dialog should be presented
                                                // to the user

	bool fFinderJobDialog;						// whether, during Finder Printing, the
                                                // Print Job dialog should be presented to
                                                // the user (making it seem like Print...)
                                                // or not (making it seem like Print One)

	bool fMinimalMargins;						// If true, page margins are maintained
                                                // such that exactly the complete
                                                // printable area of the page is used.

	bool fUseMaxResolution;						// If true the printing device will be set to
												// the maximum resolution available.
protected:
	//
	// Former PageAreas struct: metrics for printing
	//
	CViewRect fPaperRect;							// physical page
	CViewRect fInkRect;								// printable page
	CViewRect fMarginsRect;							// top-left positive; bottom-right negative
	CViewRect fInteriorRect;						// CViewRect into which view-subset is projected
	short fScale;								// print scaling (200%, 100%, 50%, etc. as a whole percent. 100% = 100.


	//
	// Other stuff
	//
	bool fIsValid;								// true if the current printer projection
												// is valid. Set to false when any component
												// of the calculation of the projection changes.
};

//----------------------------------------------------------------------------------------
// TPrintCommand: Returned in response to a printing request. Doesn't actually do the
// printing itself, but forwards to the appropriate handler.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPrintCommand : public TCommand
{
public:
	TPrintCommand(CommandNumber itsCommandNumber = cNoCommand,
					   TStdPrintHandler* itsStdPrintHandler = NULL);
		//Constructor

	virtual ~TPrintCommand();
		// Destructor
		
	virtual void DoIt();
		// Tell the print handler to print

	virtual TAppleEvent* MakeAppleEvent();

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
protected:
	TStdPrintHandler* fStdPrintHandler;			// the associated print handler
};


//----------------------------------------------------------------------------------------
// TPrintStyleChangeCommand: Used to allow changes made by the user in the 'Page Setup...'
// dialog to be undoable.
//----------------------------------------------------------------------------------------

class EXPORT_MA TPrintStyleChangeCommand : public TCommand
{
public:
	//------------------------------------------------------------------------------------
	// Initialization and Termination
	//------------------------------------------------------------------------------------
	TPrintStyleChangeCommand(TStdPrintHandler* itsPrintHandler = NULL, bool isUndoable = true);
		// Constructs the print style change command, makes a copy of the old print
        // handle and creates the new one.
        // Can be undone if isUndoable is true

	virtual ~TPrintStyleChangeCommand();
		// Disposes the old and new print handles.

	//------------------------------------------------------------------------------------
	// Command Execution
	//------------------------------------------------------------------------------------
	virtual void DoIt();
		// Sets the print handler's printinfo from the new printinfo.

	virtual void UndoIt();
		// Sets the print handler's printinfo from the old printinfo.

	virtual void SetupDependencies(); // Override
		// Overridden to make this command a dependent of its print handler's fView. 
		// And a dependent of its print handler's fPrintMgr. 

	virtual void DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject, 
						  void* changeData,
						  CDependencySpace_AC* dependencySpace); // Override
		// If the print handler's fView is closed, commit this command  
		// If the printer driver has changed behind our back, we can no longer be undone.

	//----------------------------------------------------------------------------------------
	// data members
	//----------------------------------------------------------------------------------------
public:
	CAutoPtr_AC<TPrintInfo> fNewPrintInfo;		// The new print record, after the change.

protected:
	CAutoPtr_AC<TPrintInfo> fOldPrintInfo;		// The old print record, before the change.

	TStdPrintHandler* fStdPrintHandler;			// Associated print handler.
};


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

EXPORT_MA extern PenState gBreaksPenState;				// penState to used to draw page breaks

EXPORT_MA extern bool gCancelAllPrinting;				// set to true if during Finder Printing
                                                // user selects Cancel All

EXPORT_MA extern CViewRect gStdPageMargins;					// "Standard" left, top, right, bottom
                                                // page margins, in screen resolution -
                                                // one inch all round by default

EXPORT_MA extern TPrintInfo* gFinderPrintInfo;			// Used for finder printing, to do
                                                // PrJobMerge when printing more than one
                                                // document.

EXPORT_MA extern TStdPrintHandler* gJobPrintHandler;		// The Print Handler in use during
                                                // printing.

EXPORT_MA extern CAutoPtr_AC<TPrintInfo> gDefaultPrintInfo;// The default TPrintInfo used by the
												// application. Used if no TPrintInfo* is
												// supplied to IStdPrintHandler
												// (Usually the case)

EXPORT_MA extern CFSSpec_AC gTargetPrinter;					// The current printer target for a "drag print"
#if qGXPrinting
EXPORT_MA extern bool gUsingGXPrinting;
#endif
	// TRUE if GX Printing has been initialized (when used)

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

EXPORT_MA extern  void InitUPrinting();
	// Call once, at application start-up time. You can interrogate global variable
    // 'gCouldPrint' at any time to determine whether printing code is actually accessible

EXPORT_MA  TPrintInfo* NewTPrintInfo(bool isValid = false);
	// The default print record is retrieved from itsPrintMgr.

#endif
