// UTECommands.h
// Copyright  1986-97 by Apple Computer, Inc. All rights reserved.


#ifndef __UTECOMMANDS__
#define __UTECOMMANDS__

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __TOOLBOX__
#include "Toolbox.h"
#endif

// Toolbox

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations. 
//----------------------------------------------------------------------------------------

class TEditText;
class TTEView;


//----------------------------------------------------------------------------------------
// TTECommand: A command that adds characters to, or delete characters from, a TEView 
//----------------------------------------------------------------------------------------

class EXPORT_MA TTECommand : public TCommand
{
public:
	//------------------------------------------------------------------------------------
	// Initialize and Free 
	//------------------------------------------------------------------------------------
	TTECommand(TTEView* itsTEView = NULL, CommandNumber itsCommandNumber = cNoCommand, bool itsSaveText = false);
		// Initialize the command; if unsuccessful, exit is via Failure mechanism. 
		
	TTECommand(TEditText* itsEditText, CommandNumber itsCommandNumber, bool itsSaveText);
		// Initialize the command; if unsuccessful, exit is via Failure mechanism. 
		
	void CommonInit(TTEView* itsTEView,
							CommandNumber itsCommandNumber,
							bool itsSaveText);
		// shared initialization for both constructors

	virtual ~TTECommand();
		// Free the text handles holding information needed for Undo/ Redo, then Inherited::Free.


	//------------------------------------------------------------------------------------
	// Command execution phase overrides 
	//------------------------------------------------------------------------------------
	virtual void DoIt();
		// Focuses, then calls DoMainFunction 

	virtual void RedoIt();
		// Focuses, calls RestoreSelection to get selection right, then calls
        // DoMainFunction to reinstate the changes done in DoIt which were undone by a
        // preceding UndoIt 

	virtual void UndoIt();
		// Focuses, then dispatches to RemoveAdditions, ReviveDeletions, and
        // RestoreSelection to accomplish the Undo 

	virtual void SetupDependencies(); // Override
		// Overridden to make this command a dependent of its fTEView. 

	virtual void DoUpdate(ChangeID_AC theChange,
								 MDependable_AC* changedObject, 
								 void* changeData,
								 CDependencySpace_AC* dependencySpace); // Override
		// If the fTEView is closed, commit this command  

	//----------------------------------------------------------------------------------------
	// Command execution - Restoration of Selection -- called in both UNDO and REDO phases
	//----------------------------------------------------------------------------------------
	virtual void RestoreSelection();
		// Set the Selection to be what it was just before the DO phase of the command was
        // performed 

	virtual void PrepareForUndoRedo(); // override
		// Make the command context visible for undo/redo. 
	
	virtual void RevealUndoRedo(); // override
		// Make this context visible after undo/redo. 


	//------------------------------------------------------------------------------------
	// Command execution - steps in DO and REDO phases 
	//------------------------------------------------------------------------------------
	void AssureTEView();
		// Guarantee that a TEView exists when the command knows about a TEditText.
		
	void ForgetTEView();
		// Forget references to the TEView when the command knows about a TEditText.
		
	virtual void DoPostProcess();
		// Put TEView/TEdit text fields in post-process state.
		
	virtual void BanishOldText();
		// Remove text that was selected at the outset of the Do phase 

	virtual void InstallNewText();
		// Install the new text 

	virtual void DoMainFunction();
		// Forwards to the relevant methods for Do and Redo phases 


	//------------------------------------------------------------------------------------
	// Command execution - steps in UNDO phase 
	//------------------------------------------------------------------------------------
	virtual void RemoveAdditions();
		// Remove any characters which were added by the DO phase of the command 

	virtual void ReviveDeletions();
		// Bring back the characters which were removed during the DO phase 

public:
	TTEView* fTEView;							// The TEView operated on
	
	TEditText*	fEditText;						// The TEditText operated on. This command
												// class can be used to modify either TTEView
												// or TEditText.

	TEHandle fHTE;								// same as fTEView's fHTE; duplicated for
                                                // code efficiency 

	short fOldStart;							// The beginning position of the selection
                                                // at the moment just before the command
                                                // was done. 

	short fOldEnd;								// The end position of the selection at
                                                // the moment just before the command was
                                                // done. 

	Handle fOldText;							// If fOldStart = fOldEnd, i.e., if old
                                                // selection had been an insertion CPoint_AC,
                                                // this will be NULL. Otherwise, provides a
                                                // temporary home for the characters
                                                // comprising the old selection
                               
	CStr255_AC	fOldStaticText;						// Temporary home for TStaticText contents.
												// Note: TEditText derives from TStaticText
	
	StScrpHandle fOldStyles;

	short fNewStart;							// The beginning location in the Text of
                                                // the new text that is added by the
                                                // command, if any. 

	short fNewEnd;								// The ending location in the Text of the
                                                // new text that is added by the command,
                                                // if any. 

	Handle fNewText;							// A Handle to the characters added by the
                                                // command 

	StScrpHandle fNewStyles;

	Handle fPadding;							// Handle to fill size between new and
                                                // old. This insures that we can always
                                                // undo and redo. 

	short fTextPad;								// Size difference between New and Old
                                                // text 

	long fStylePad;								// Size difference between New and Old
                                                // styles 
};


//----------------------------------------------------------------------------------------
// TTECutCopyCommand: Command for handling Cut & Copy for the TEView.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTECutCopyCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// Creation and Destruction 
	//------------------------------------------------------------------------------------

	TTECutCopyCommand(TTEView* itsTEView = NULL, CommandNumber itsCommandNumber = cNoCommand);
		// constructor
		
	virtual ~TTECutCopyCommand();
		// Free the command 


	//------------------------------------------------------------------------------------
	// Command execution 
	//------------------------------------------------------------------------------------

	virtual void DoIt();
		// Launches a TEView for installation in the Clipboard, then calls DoMainFunction
        // 

	virtual void ReviveDeletions();
		// Bring back the characters which were removed during the DO phase; called during
        // UNDO phase 

public:
	bool fClipCreated;						// Clipboard view created OK 
};


//----------------------------------------------------------------------------------------
// TTEPasteCommand: Command that handles Pasting for the TEView. 
//----------------------------------------------------------------------------------------

class EXPORT_MA TTEPasteCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// Initialization 
	//------------------------------------------------------------------------------------
	TTEPasteCommand(TTEView* itsTEView = NULL);
		// Initialize the command; if unsuccessful, signalled by Failure mechanism 
	
	virtual ~TTEPasteCommand();
		// Destructor
};


//----------------------------------------------------------------------------------------
// TTEStyleCommand: Command for applying a new style to the TEView.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTEStyleCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// Initialization 
	//------------------------------------------------------------------------------------
	TTEStyleCommand(TTEView* itsTEView = NULL,
										const CTextStyle_AC& itsNewStyle = gApplicationStyle,
										CommandNumber itsCommandNumber = cNoCommand,
										short itsMode = 0);
		// Initialize the command; if unsuccessful, signalled by Failure mechanism 
	
	virtual ~TTEStyleCommand();
		// Destructor

	//------------------------------------------------------------------------------------
	// Command execution 
	//------------------------------------------------------------------------------------

	virtual void InstallOneStyle(const CTextStyle_AC& newStyl);
		// Installs one style over entire record - for use with old TextEdit 

	virtual void InstallManyStyles(StScrpHandle newStyls);
		// Installs styles over selection range - for use with styled TextEdit 

	virtual void DoIt();
		// Applies style to selection range 

	virtual void UndoIt();
		// Undoes the action performed by DoIt. 

	virtual void RedoIt();
		// Calls DoIt. 

public:
	//------------------------------------------------------------------------------------
	// These two fields are only used in non-styled TextEdit records 
	//------------------------------------------------------------------------------------

	CTextStyle_AC fOldTextStyle;					// The original text style 
	CTextStyle_AC fNewTextStyle;					// What we're replacing it with 
	short fMode;								// Mode for style change 
};


//----------------------------------------------------------------------------------------
// TTETypingCommand: The command that handles typing in a TEView.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTETypingCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// Creation and destruction 
	//------------------------------------------------------------------------------------
	TTETypingCommand(TTEView* itsTEView = NULL, const CStr2_AC& itsFirstCh = gEmptyString);
		// Initialize the command; if not successful, exit is via Failure mechanism 

	virtual ~TTETypingCommand();
		// Deallocate the command and dependent structures 

	//------------------------------------------------------------------------------------
	// Command processing 
	//------------------------------------------------------------------------------------
	virtual void DoNormalChar(const CStr2_AC& ch);
		// Handle any typed character[s] except a backspace 

	virtual void BkSpcLeft(Handle theText, short curStart);
		// Handle backspace to the left of the original selection 

	virtual void BkSpcRight(Handle theText, short curStart);
		// Handle backspace to the right of the original selection 

	virtual void FwdDelete(Handle theText, short curStart, short curEnd);


	//------------------------------------------------------------------------------------
	// Handle forward delete 
	//------------------------------------------------------------------------------------
	virtual void AddCharacter(const CStr2_AC& ch);
		// Add character[s] to an already-launched TETypingCommand 

	virtual void DoIt();
		// First processing for command; adds first character 

	virtual void RedoIt();
		// Handles undoing forward backspace 

	virtual void UndoIt();
		// Process undo typing 

	virtual void CompleteTyping();
		// Mark as no more typing allowed and fix up style scrap, if any 

	//------------------------------------------------------------------------------------
	// data members 
	//------------------------------------------------------------------------------------
public:
	CStr2_AC fFirstChar;							// First character[s] typed 

	bool fCompleted;							// Indicates whether further keystrokes
                                                // will be extensions to this command
                                                // (false) or whether the command has
                                                // already been completed (true) 
};

#if qDrag

//----------------------------------------------------------------------------------------
// TTEDragDropCommand: Command that handles a drag or drop text operation.
//----------------------------------------------------------------------------------------

class EXPORT_MA TTEDragDropCommand : public TTECommand
{
public:
	TTEDragDropCommand(TTEView	*itsTEView = NULL, CommandNumber itsCommandNumber = cNoCommand);	// overloaded initializer
	
	TTEDragDropCommand(TEditText *itsEditText, CommandNumber itsCommandNumber);
		// Constructor

	virtual ~TTEDragDropCommand();
		// Destructor
	
	void DoMainFunction();
		// Override. Don't banish the old text.
	
	virtual void SetNewText(Handle newText, StScrpHandle newStyles);
		// set text to be installed in the target TTEView
		
	virtual void SetNewStart(short newStart);
		// set the location for the new text
		
	virtual void InstallNewText();
		// OVERRIDE
				
}; // class TTEDragDropCommand

//----------------------------------------------------------------------------------------
// TTEDragMoveCommand: Command that handles moving text within a view as a result of
// a drag
//----------------------------------------------------------------------------------------

class EXPORT_MA TTEDragMoveCommand : public TTECommand
{
public:
	TTEDragMoveCommand(TTEView *itsTEView = NULL, CommandNumber itsCommandNumber = cNoCommand, short newStart = 0);
	
	TTEDragMoveCommand(TEditText *itsEditText, CommandNumber itsCommandNumber, short newStart);
		// Constructor
		
	virtual ~TTEDragMoveCommand();
	
	virtual void InstallNewText();
};

#endif // qDrag
	
//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

EXPORT_MA void DumpTTECommand(TTECommand* theTTECommand);
	// writes TTECommand info out to Debug window 

#endif

