//----------------------------------------------------------------------------------------
// UEditText.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UDIALOG__	
#include "UDialog.h"
#endif

// ACS

#ifndef __TextBox_AC__
#include "TextBox_AC.h"
#endif

// MacApp

#ifndef __UDIALOGTEVIEW__	
#include "UDialogTEView.h"
#endif

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#if defined(qDrag) && !defined(__UDRAGDROP__)
#include "UDragDrop.h"
#endif

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

#ifndef __UMACAPPUTILITIES__	
#include "UMacAppUtilities.h"
#endif

#ifndef __USTATICTEXT__
#include "UStaticText.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UTECOMMANDS__
#include "UTECommands.h"
#endif

//----------------------------------------------------------------------------------------
TFloatingTEManager gFloatingTEManager;			// The floating TEView used with edit text
                                                // fields.


//========================================================================================
// GLOBAL Procedures
//========================================================================================
//----------------------------------------------------------------------------------------
// InitUEditText: 
//----------------------------------------------------------------------------------------
void InitUEditText()
{
	static bool UEditTextInitialized;

	if (!UEditTextInitialized)
	{	
		if (qTemplateViews)
		{
			// So the linker doesn't dead strip these 
			MA_REGISTER_SIGNATURE(TDialogTEView, kStdDialogTEView);
			MA_REGISTER_SIGNATURE(TEditText, kStdEditText);
		}
	
		UEditTextInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// AcquireFloatingTEView: 
//----------------------------------------------------------------------------------------
TDialogTEView* AcquireFloatingTEView()
{
	return gFloatingTEManager.AcquireFloatingTE();
}

//----------------------------------------------------------------------------------------
// ReleaseFloatingTEView: 
//----------------------------------------------------------------------------------------
void ReleaseFloatingTEView(TDialogTEView* aDialogTEView)
{
	gFloatingTEManager.ReleaseFloatingTE(aDialogTEView);
}

//========================================================================================
// CLASS TEditText
//========================================================================================
MA_DEFINE_CLASS(TEditText);

//----------------------------------------------------------------------------------------
// TEditText constructor
//----------------------------------------------------------------------------------------


TEditText::TEditText(const CViewPoint& itsLocation,
						  const CViewPoint& itsSize,
						  short itsMaxChars) :
	TStaticText(itsLocation, itsSize, sizeFixed, sizeFixed, kNoResource, 0),

	fTEView(NULL),
	fMaxChars(itsMaxChars),

	fControlChars(macroAsSetElem(chLeft) | macroAsSetElem(chRight) | 
					macroAsSetElem(chUp) | macroAsSetElem(chDown) | 
					macroAsSetElem(chBackspace))
{
	fTextStyle = gSystemStyle;
	SetPenSize(CPoint_AC(1,1));						// and a thin frame 
	fWantsToBeTarget = true;
	fEventNumber = mEditTextHit;

	fCursorID = iBeamCursor;
	fLetsSubViewsHandleCursor = false;

	SetAdornment(kFrame, kDontRedraw);

	Inset(CViewPoint(2, 2), kDontRedraw);				// Default is a little, teeny inset 

	SetEnable(true);
}

//----------------------------------------------------------------------------------------
// TEditText::Free: 
//----------------------------------------------------------------------------------------


TEditText::~TEditText()
{
	RemoveFloatingTEView();
}

//----------------------------------------------------------------------------------------
// TEditText::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TEditText::GetStandardSignature() const	// Override 
{
	return kStdEditText;
}

//----------------------------------------------------------------------------------------
// TEditText::ReadFields: 
//----------------------------------------------------------------------------------------


void TEditText::ReadFields(CStream_AC* aStream)				// Override 
{
	TStaticText::ReadFields(aStream);
	
	fMaxChars = aStream->ReadInteger();
	fControlChars = aStream->ReadLong();
}

//----------------------------------------------------------------------------------------
// TEditText::WriteFields: 
//----------------------------------------------------------------------------------------


void TEditText::WriteFields(CStream_AC* aStream) const				// Override 
{
	TStaticText::WriteFields(aStream);

	aStream->WriteInteger(fMaxChars);
	aStream->WriteLong(fControlChars);
}

//----------------------------------------------------------------------------------------
// TEditText::WriteSubViews: 
//----------------------------------------------------------------------------------------


void TEditText::WriteSubViews(CStream_AC* aStream) const			// Override 
{
	// remove floating TEView, if any
	
	TStaticText::WriteSubViews(aStream);
	
	// restore floating TEView, if any
}

//----------------------------------------------------------------------------------------
// TEditText::HandleMouseDown: 
//----------------------------------------------------------------------------------------


bool TEditText::HandleMouseDown(const CViewPoint& theMouse,
								   TToolboxEvent* event,
								   CPoint_AC hysteresis,
								   EMouseDownType mouseDownType)	// Override
{
	bool returnVal = false;
	
	if (TStaticText::HandleMouseDown(theMouse,event,hysteresis, mouseDownType))
	{
		HandleEvent(fEventNumber,this,NULL);
		returnVal = true;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditText::DoMouseCommand: 
//----------------------------------------------------------------------------------------


void TEditText::DoMouseCommand(CViewPoint&			theMouse,
								TToolboxEvent*	event,
								CPoint_AC			hysteresis)
{	
	if (fTEView)
		fTEView->DoMouseCommand(theMouse, event, hysteresis);
	else
		TStaticText::DoMouseCommand(theMouse, event, hysteresis);
}

//----------------------------------------------------------------------------------------
// TEditText::BecameWindowTarget: 
//----------------------------------------------------------------------------------------


void TEditText::BecameWindowTarget()		// Override 
{
	if (!fTEView)
		InstallFloatingTEView(false);

	TStaticText::BecameWindowTarget();
}

//----------------------------------------------------------------------------------------
// TEditText::BecameTarget: 
//----------------------------------------------------------------------------------------


void TEditText::BecameTarget()	// Override 
{	
	if (!fTEView)
		InstallFloatingTEView(false);
	SetTarget(fTEView);

	TStaticText::BecameTarget();
}

//----------------------------------------------------------------------------------------
// TEditText::SetTargetSelection: 
//----------------------------------------------------------------------------------------


void TEditText::SetTargetSelection(bool redraw)		// Override
{
	SetSelection(0, SHRT_MAX, redraw);// Select all characters 
	MakeVisible();
}


//----------------------------------------------------------------------------------------
// TEditText::ValidationFailed: 
//----------------------------------------------------------------------------------------


void TEditText::ValidationFailed(long reason)
{
	if (reason != kValidValue)
	{
		if (reason != kErrorHandled)			// go ahead and post an alert 
		{
			::ThrowIfOSErr_AC(MAInteractWithUser());
			if ((reason < 1) || (reason > kNoOfDefaultReasons))
				reason = kInvalidValue;

			CStr255_AC aString(kInvalidValueReasons, (short)reason);//!!! Note casting
			ParamText(aString, gEmptyString, gEmptyString, gEmptyString);
			StdAlert(phInvalidValue);
		}
		if (fDataHandle)
			RestartEdit((**((String255Handle)fDataHandle)));// Restart with previous value 
	}
}

//----------------------------------------------------------------------------------------
// TEditText::ChangeWrap: 
//----------------------------------------------------------------------------------------


void TEditText::ChangeWrap(bool newAutoWrap,
						   bool redraw)	// Override 
{
	TStaticText::ChangeWrap(newAutoWrap, redraw);
	if (fTEView)
		fTEView->ChangeWrap(newAutoWrap, redraw);
}

//----------------------------------------------------------------------------------------
// TEditText::Deselect: 
//----------------------------------------------------------------------------------------


bool TEditText::Deselect()
{
	bool returnVal = true;

	//	// Commit the last command to prevent undo from applying to the wrong edit text, and
	//	// to ensure that all changes are made before validating.
	//	if (fTEView)
	//	{
	//		fTEView->GetContext(cNoCommand)->CommitLastCommand();
	//	}
	// ^^^ Previous code replaced with call to DoneTyping in StopEdit ^^^

	long validateResult = GetValidationError();
	if (validateResult == kValidValue)
		StopEdit();
	else
	{
		ValidationFailed(validateResult);
		returnVal = false;
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditText::DoSubstitution: 
//----------------------------------------------------------------------------------------


void TEditText::DoSubstitution(CStr255_AC& /* theText */)// Override 
{
	// Default action is for editable text items is not to do any substitions 
}

//----------------------------------------------------------------------------------------
// TEditText::GetText: 
//----------------------------------------------------------------------------------------


CStr255_AC TEditText::GetText() const	// Override 
{
	CStr255_AC returnVal;
	if (!fTEView)
		returnVal = TStaticText::GetText();
	else
	{
		Handle theChars = fTEView->ExtractText();
		short numberOfChars = (short)Min_AC(kStr255Len, GetHandleSize(theChars));
		// delete possible trailing half-character

		if (numberOfChars && MACharacterByteType(*theChars, numberOfChars - 1, fTEView->OffsetToScript(numberOfChars - 1)) == smFirstByte)
			--numberOfChars;

		returnVal.CopyFrom((void*)(*theChars), numberOfChars);
	}

	return returnVal;
}

//----------------------------------------------------------------------------------------
// TEditText::ImageText: 
//----------------------------------------------------------------------------------------


void TEditText::ImageText(Ptr text,
						  long Length,
						  const CViewRect& box,
						  short just)	// Override 
{
	if (Length >= 0)
		TextBox_AC(text, Length, ViewToQD(box), just, fAutoWrap, NULL, fEraseFirst,
			kSpaceForCaret, fPreferOutline);
}

//----------------------------------------------------------------------------------------
// TEditText::InstallFloatingTEView: 
//----------------------------------------------------------------------------------------


void TEditText::InstallFloatingTEView(bool selectChars)
{
	if (!fTEView)
	{
		TDialogTEView * aFloatingTEView = GetTEView();
		if (!aFloatingTEView)
		{
#if qDebug
			::ProgramBreak_AC("###the acquired TEView is nil.");
#endif
		}
		aFloatingTEView->InstallEditText(this, selectChars);
		fTEView = aFloatingTEView;
	}

	// since the floating TE view handles dim/hilite drawing - these adorners are removed 
	if (fDimmed)
		DeleteAdorner(&TAdorner::fgDimAdorner, kDontRedraw);
	if (fHilite)
		DeleteAdorner(&TAdorner::fgHiliteAdorner, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TEditText::GetTEView: 
//----------------------------------------------------------------------------------------

	
TDialogTEView* TEditText::GetTEView()
{
	return AcquireFloatingTEView();	// Get it from the server
}

//----------------------------------------------------------------------------------------
// TEditText::ReleaseTEView: 
//----------------------------------------------------------------------------------------


void TEditText::ReleaseTEView()
{
	ReleaseFloatingTEView(fTEView);	// Return it to the server for reuse
	fTEView = NULL;
}

//----------------------------------------------------------------------------------------
// TEditText::MakeVisible: 
//----------------------------------------------------------------------------------------

void TEditText::MakeVisible()
{
	CViewRect myExtent(GetExtent());
	myExtent.Inset(-10, -10);
	
	RevealRect(myExtent, myExtent.GetSize(), kVisible);
}

//----------------------------------------------------------------------------------------
// TEditText::RemoveFloatingTEView: 
//----------------------------------------------------------------------------------------


void TEditText::RemoveFloatingTEView()
{
	if (fTEView)
	{
		fTEView->InstallEditText(NULL, false);
		ReleaseTEView();
	}

	// since the floating TE view handles dim/hilite drawing - these adorners were removed 
	if (fDimmed)
		AddAdorner(&TAdorner::fgDimAdorner, kAdornLast - 10, kDontRedraw);
	if (fHilite)
		AddAdorner(&TAdorner::fgHiliteAdorner, kAdornLast - 5, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TEditText::RestartEdit: 
//----------------------------------------------------------------------------------------


void TEditText::RestartEdit(const CStr255_AC& restartText)
{
	if (fTEView && fTEView->Focus())			// First, attempt to focus the TEView 
	{
		ClipRect(gZeroRect);					// Prevent TE from mucking up the hilite with
												// a stinking insertion point

		fTEView->SetActive(false);				// Deactivate the selection 
		SetText(restartText, kDontRedraw);// Set the text to previous value 
		SetSelection(0, SHRT_MAX, kDontRedraw);// Select all characters 

		fTEView->SetActive(true);				// Activate the selection

		InvalidateFocus();				// Make sure we re-focus 
		fTEView->ForceRedraw();
	}
	else
		SetText(restartText, kDontRedraw);// Just set the text if we can't focus 
}

//----------------------------------------------------------------------------------------
// TEditText::SetJustification: 
//----------------------------------------------------------------------------------------


void TEditText::SetJustification(short theJust,
								 bool redraw)
{
	if (fTEView)
		fTEView->SetJustification(theJust, redraw);
	TStaticText::SetJustification(theJust, redraw);
}

//----------------------------------------------------------------------------------------
// TEditText::SetSelection: 
//----------------------------------------------------------------------------------------


void TEditText::SetSelection(short selStart,
							 short selEnd,
							 bool redraw)
{
	if (fTEView)
	{
		fTEView->SetSelection(selStart, selEnd, redraw && (fTEView->IsDrawable()) );
	}
}

//----------------------------------------------------------------------------------------
// TEditText::SetText: 
//----------------------------------------------------------------------------------------


void TEditText::SetText(const CStr255_AC& theText,
						bool redrawImmediately)	// Override 
{
	CStr255_AC localText(theText);
	if (localText.Length() > fMaxChars)
	{
#if qDebug
		ProgramReport_AC("Truncating text to fMaxChars");
#endif
		localText.Length() = fMaxChars;
	}
	
	if (fTEView)
	{
		if (GetText() != localText)
		{
			// Make sure fDataHandle's text is the same as that in fTEView
			ReleaseText();
			fDataHandle = (CPascalStrHandle_AC)NewString(localText);
			if (MemError() != noErr)
				fDataHandle = NULL;

			fTEView->SetText(localText);
			fTEView->RecalcText();
			fTEView->SynchView(kDontRedraw);
			if (IsDrawable())
			{
				Invalidate(ControlQDArea());
				
				if (redrawImmediately)
					Update();
			}
		}
	}
	else
		TStaticText::SetText(localText, redrawImmediately);
}

//----------------------------------------------------------------------------------------
// TEditText::StartEdit: 
//----------------------------------------------------------------------------------------


void TEditText::StartEdit(bool selectChars)
{
	BecomeTarget();
	if (selectChars)
		SetTargetSelection(kRedraw);
}

//----------------------------------------------------------------------------------------
// TEditText::StopEdit: 
//----------------------------------------------------------------------------------------


void TEditText::StopEdit()
{
	if (fTEView)
	{
		fTEView->DoneTyping();
		
		fTEView->SetActive(false);				// Hides the selection
		CStr255_AC aString(GetText());					// Must get the text before removing
												// the floating TEView 
		RemoveFloatingTEView();
		SetText(aString, kDontRedraw);
	}
}

//----------------------------------------------------------------------------------------
// TEditText::GetValidationError: 
//----------------------------------------------------------------------------------------


long TEditText::GetValidationError()	// Override
{
	long validateResult = TStaticText::GetValidationError();
	
	if ((validateResult == kValidValue) && fTEView && (fTEView->GetNumberOfChars() > fMaxChars))
		validateResult = kTooManyCharacters;

	return validateResult;
}

//----------------------------------------------------------------------------------------
// TEditText::WantsToBeTarget: 
//----------------------------------------------------------------------------------------

bool TEditText::WantsToBeTarget()	// Override 
{
	return fWantsToBeTarget && IsEnabled() && !(fTEView && fTEView->IsTarget());
}

//----------------------------------------------------------------------------------------
// TEditText::GetDefaultCursorRegion: 
//----------------------------------------------------------------------------------------


void TEditText::GetDefaultCursorRegion(const CViewPoint& /* localPoint */ ,
									   RgnHandle cursorRegion)
{
	RectRgn(cursorRegion,&ControlQDArea());
}

#if qDrag

//----------------------------------------------------------------------------------------
// TEditText::DoMakeDragCursorRegion: 
//----------------------------------------------------------------------------------------


RgnHandle TEditText::DoMakeDragCursorRegion()
{
	RgnHandle dragCursorRegion = NULL;
	
	if (fTEView)
		dragCursorRegion = fTEView->DoMakeDragCursorRegion();
	
	return dragCursorRegion;
}

//----------------------------------------------------------------------------------------
// TEditText::WillDrag: 
//----------------------------------------------------------------------------------------


bool TEditText::WillDrag(const CViewPoint& 	theMouse,
							const RgnHandle	dragCursorRegion)
{
	bool willDrag = false;
	
	if (fTEView)
	{
		bool savedDraggable = fTEView->GetDraggable();	// temporarily set the TEView's
		fTEView->SetDraggable(fDraggable);					// is draggable member.
		willDrag = fTEView->WillDrag(theMouse, dragCursorRegion);
		fTEView->SetDraggable(savedDraggable);
	}
		
	return willDrag;
}

//----------------------------------------------------------------------------------------
// TEditText::GetWillDragCursorID: 
//----------------------------------------------------------------------------------------


short TEditText::GetWillDragCursorID()
{
	return kNoResource;
}

//----------------------------------------------------------------------------------------
// TEditText::GetIsDraggingCursorID: 
//----------------------------------------------------------------------------------------


short TEditText::GetIsDraggingCursorID()
{
	return kNoResource;
}

//----------------------------------------------------------------------------------------
// TEditText::DoAddDragContent: 
//----------------------------------------------------------------------------------------


void TEditText::DoAddDragContent()
{
	// will already have a TEView
	fTEView->DoAddDragContent();
}

//----------------------------------------------------------------------------------------
// TEditText::WillAcceptDrop: 
//----------------------------------------------------------------------------------------


bool TEditText::WillAcceptDrop(CDragItemIterator& dragItemIterator)
{
	InstallFloatingTEView(false);

	return fTEView->WillAcceptDrop(dragItemIterator);
}

//----------------------------------------------------------------------------------------
// TEditText::DoDragEnter: 
//----------------------------------------------------------------------------------------


void TEditText::DoDragEnter()
{
	InstallFloatingTEView(false);	 // get a TDialogTEView
	fTEView->Focus();
	fTEView->DoDragEnter();
}

//----------------------------------------------------------------------------------------
// TEditText::DoMakeDropHiliteRegion: 
//----------------------------------------------------------------------------------------


RgnHandle TEditText::DoMakeDropHiliteRegion()
{
	RgnHandle dropRegion = NULL;
	// Will be focused on the TEView.
	// Focus on the EditText to create the correct drop hilite
	Focus();
	dropRegion = TStaticText::DoMakeDropHiliteRegion();
	InsetRgn(dropRegion, 1, 1);		// inset by one pixel to compensate for the border

	return dropRegion;	
}

//----------------------------------------------------------------------------------------
// TEditText::DoDragWithin: 
//----------------------------------------------------------------------------------------


void TEditText::DoDragWithin(const CViewPoint& localMouse)
{
	fTEView->Focus();
	fTEView->DoDragWithin(localMouse);
}

//----------------------------------------------------------------------------------------
// TEditText::DoDragLeave: 
//----------------------------------------------------------------------------------------


void TEditText::DoDragLeave()
{
	fTEView->Focus();
	fTEView->DoDragLeave();
}

//----------------------------------------------------------------------------------------
// TEditText::DoMakeDragDropCommand: 
//----------------------------------------------------------------------------------------


TCommand* TEditText::DoMakeDragDropCommand(	CommandNumber 		itsCommandNumber,
											CDragItemIterator&	dragItemIterator)
{
	TCommand *returnCommand = NULL;
	
	switch (itsCommandNumber)
	{
		case cDrag:
			returnCommand = TH_new TTECommand(this, itsCommandNumber, true);
			break;
		
		case cDrop:
			{
				Handle textHandle = NULL;
				TDragItem *dragItem = dragItemIterator.First();
				
				dragItem->FocusOnFlavor(kTextType_AC);
				textHandle = dragItem->GetDataAsHandle();
				::ThrowIfNULL_AC(textHandle);
				
				TTEDragDropCommand *dropCommand = TH_new TTEDragDropCommand(this, itsCommandNumber);
				dropCommand->SetNewText(textHandle, NULL);
				dropCommand->SetNewStart(fTEView->GetLastDropCaretOffset());
				returnCommand = dropCommand;
			}
			break;
			
		case cDragMove:
			returnCommand = TH_new TTEDragMoveCommand(this, itsCommandNumber, fTEView->GetLastDropCaretOffset());
			break;
			
		default:
			returnCommand = TStaticText::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}

	return returnCommand;
}

//----------------------------------------------------------------------------------------
// TEditText::DoFulfillPromise: 
//----------------------------------------------------------------------------------------


void TEditText::DoFulfillPromise(TDragItem* promisedItem)
{
	fTEView->DoFulfillPromise(promisedItem);
}

#endif // qDrag

