//----------------------------------------------------------------------------------------
// UNumberText.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------


#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UNUMBERTEXT__
#include "UNumberText.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

//----------------------------------------------------------------------------------------
// Function prototype
bool IsValidDecimalString(const CStr255_AC& theString, bool allowDecimalPoint, bool allowSign);

//----------------------------------------------------------------------------------------
// Unit initialization
//----------------------------------------------------------------------------------------
void InitUNumberText()
{
	static bool UNumberTextInitialized;
	if (!UNumberTextInitialized)
	{	
		if (qTemplateViews)
		{
			InitUEditText();
	
			MA_REGISTER_SIGNATURE(TNumberText, kStdNumberText);
		}

		UNumberTextInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// IsValidDecimalString - Utility routine to validate a string of digits.
//		This code was adapted from ODF!
//----------------------------------------------------------------------------------------
bool IsValidDecimalString(const CStr255_AC& theString, bool allowDecimalPoint, bool allowSign)
{
	bool isValid = true;
	const char kZero = '0';
	const char kNine = '9';
	const char kPoint = '.';
	const char kMinus = '-';
	const char kPlus = '+';
	const char* pString = (const char*) theString;
	const char* pLast = pString + theString.Length();
	bool foundDecimal = false;	// make sure there's only one
	short count = 0;
	short maxDigits = 10;
	if (allowDecimalPoint)
		maxDigits = 16;

	if (allowSign)	// check the first character for a sign
	{
		if (*pString == kMinus)
			++pString;
		else if ((*pString == kPlus) && allowDecimalPoint)	// plus-sign only works with reals
			++pString;
	}

	while (pString < pLast)
	{
		if ((*pString >= kZero) && (*pString <= kNine))		// valid decimal digit
		{
			count++;
			if ((count > maxDigits) && !foundDecimal)
			{
				isValid = false;
				break;
			}
		}
		else
		{
			if (allowDecimalPoint && (*pString == kPoint) && !foundDecimal)
				foundDecimal = true;
			else
			{
				isValid = false;
				break;
			}
		}
		++pString;
	}

	return isValid;
}

//========================================================================================
// CLASS TNumberText
//========================================================================================
#undef Inherited
#define Inherited TEditText

MA_DEFINE_CLASS(TNumberText);

//----------------------------------------------------------------------------------------
// TNumberText constructor
//----------------------------------------------------------------------------------------

TNumberText::TNumberText(const CViewPoint& itsLocation,
						 const CViewPoint& itsSize,
						 long itsValue,
						 long itsMinimum,
						 long itsMaximum):
	TEditText(itsLocation, itsSize, 255),

	fMaximum(LONG_MAX),
	fMinimum(0)
{

#if qDebugMsg
	if (itsMinimum > itsMaximum)
		ProgramReport_AC("Minimum value specified is greater than maximum for TNumberText.\n");
#endif

	fMinimum = itsMinimum;
	fMaximum = itsMaximum;

	SetValue(itsValue, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TNumberText destructor
//----------------------------------------------------------------------------------------

TNumberText::~TNumberText()
{
}

//----------------------------------------------------------------------------------------
// TNumberText::GetStandardSignature: 
//----------------------------------------------------------------------------------------

IDType TNumberText::GetStandardSignature() const	// Override 
{
	return kStdNumberText;
}

//----------------------------------------------------------------------------------------
// TNumberText::ReadFields: 
//----------------------------------------------------------------------------------------

void TNumberText::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	long initialValue = aStream->ReadLong();
	fMinimum = aStream->ReadLong();
	fMaximum = aStream->ReadLong();

#if qDebugMsg
	if (fMinimum > fMaximum)
		ProgramReport_AC("Minimum value specified is greater than maximum for TNumberText.\n");
#endif

	SetValue(initialValue, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TNumberText::WriteFields: 
//----------------------------------------------------------------------------------------

void TNumberText::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteLong(GetValue());
	aStream->WriteLong(fMinimum);
	aStream->WriteLong(fMaximum);
}

//----------------------------------------------------------------------------------------
// TNumberText::GetValue: 
//----------------------------------------------------------------------------------------

long TNumberText::GetValue() const
{
	long theValue = 0;
	CStr255_AC theString(GetText());
	
	if (!theString.IsEmpty())
	{
		// Note that we do not check that str2dec works at this point:
		// this method makes an assumption that the text has already
		// been validated
		StringToNum(theString, &theValue);

#if 0
#if qModelCFM
		theValue = atol((char*) theString);
#else
		short index = 0;
		decimal decRec;
		short validPrefix = 0;
		str2dec(theString, &index, &decRec, &validPrefix);
		theValue = (long)dec2num(&decRec);
#endif
#endif
	}

	return theValue;
}

//----------------------------------------------------------------------------------------
// TNumberText::SetValue: 
//----------------------------------------------------------------------------------------

void TNumberText::SetValue(long newValue, bool redraw)
{
	CStr255_AC aString;
	NumToString(MinMax_AC(fMinimum, newValue, fMaximum), aString);
	SetText(aString, redraw);
}

//----------------------------------------------------------------------------------------
// TNumberText::GetValidationError: 
//----------------------------------------------------------------------------------------

long TNumberText::GetValidationError()	// Override 
{
	// Empty CStrings are not validated; if a value is desired, override this method
	
	long theErrorCode = TEditText::GetValidationError();
	if (theErrorCode == kValidValue)
	{ 
		CStr255_AC theString(GetText());

		if (!theString.IsEmpty())
		{
			if (IsValidDecimalString(theString, false, true))	// no decimal point, sign OK
			{
				long value = 0;
				StringToNum(theString, &value);
				if (value < fMinimum)
					theErrorCode = kValueTooSmall;
				else if (value > fMaximum)
					theErrorCode = kValueTooLarge;
			}
			else
				theErrorCode = kNonNumericCharacters;
		}
	}
		
	return theErrorCode;
}
