//----------------------------------------------------------------------------------------
// UPattern.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UPATTERN__
#include "UPattern.h"
#endif

#ifndef __UICON__
#include "UIcon.h"
	// for kPreferColor
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

void InitUPattern()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TPattern, kStdPattern);
	}
}  


//========================================================================================
// CLASS TPattern
//========================================================================================
#undef Inherited
#define Inherited TControl


MA_DEFINE_CLASS(TPattern);

//----------------------------------------------------------------------------------------
// TPattern constructor
//----------------------------------------------------------------------------------------


TPattern::TPattern(const CViewPoint& itsLocation,
						const CViewPoint& itsSize,
						SizeDeterminer itsHSizeDet,
						SizeDeterminer itsVSizeDet,
						ResNumber itsRsrcID,
						bool preferColor) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fDataHandle(NULL),
	fPreferColor(kPreferColor),
	fIsColor(kPreferColor),
	fRsrcID(kNoResource)
{
	fEventNumber = mPatternHit;

	fPreferColor = preferColor;
	fIsColor = preferColor;

	SetPatternRsrcID(itsRsrcID, kDontRedraw);
	SetEnable(false);						// Default is to not enable hit testing 
}

//----------------------------------------------------------------------------------------
// TPattern copy constructor
//----------------------------------------------------------------------------------------


TPattern::TPattern(const TPattern& inPattern) :
	fDataHandle(NULL),
	fPreferColor(inPattern.fPreferColor),
	fIsColor(inPattern.fIsColor),
	fRsrcID(kNoResource)
{
	if (inPattern.fDataHandle)
		SetPatternRsrcID(inPattern.fRsrcID, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TPattern::Free: 
//----------------------------------------------------------------------------------------


TPattern::~TPattern()
{
	ReleasePattern();
}

//----------------------------------------------------------------------------------------
// TPattern::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TPattern::GetStandardSignature() const	// Override 
{
	return kStdPattern;
}

//----------------------------------------------------------------------------------------
// TPattern::ReadFields: 
//----------------------------------------------------------------------------------------


void TPattern::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	fPreferColor = aStream->ReadBoolean();

	fIsColor = fPreferColor;

	ResNumber itsRsrcID = aStream->ReadInteger();
	SetPatternRsrcID(itsRsrcID, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TPattern::WriteFields: 
//----------------------------------------------------------------------------------------


void TPattern::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

	aStream->WriteBoolean(fPreferColor);

#if qDebugMsg
	if (fRsrcID == kNoResource)
		ProgramReport_AC("Tried to write TPattern with no resource ID.\n");
#endif

	aStream->WriteInteger(fRsrcID);
}

//----------------------------------------------------------------------------------------
// TPattern::Draw: 
//----------------------------------------------------------------------------------------


void TPattern::Draw(const CViewRect& area)	// Override 
{
	if (fDataHandle)
	{
		if (IsAResource_AC(fDataHandle) &&!fIsColor)// Pixpat handles != resource handles 
			LoadResource(fDataHandle);
		if (*fDataHandle)				// If there's room for the pattern 
		{
			PenNormal();						// NECESSARY? 
			CRect_AC theQDRect(ControlQDArea());

			CTempHandleLock_AC tempHandleLock(fDataHandle);
			if (fIsColor)
				FillCRect(theQDRect, (PixPatHandle)fDataHandle);
			else
				FillRect(theQDRect, *((PatHandle)fDataHandle));
		}
	}

	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TPattern::ReleasePattern: 
//----------------------------------------------------------------------------------------


void TPattern::ReleasePattern()
{
	fRsrcID = kNoResource;
	if (fDataHandle)
	{
		if (fIsColor)
			DisposePixPat((PixPatHandle)fDataHandle);
		else
			HPurge(fDataHandle);
		fDataHandle = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TPattern::SetPattern: 
//----------------------------------------------------------------------------------------


void TPattern::SetPattern(Handle thePattern,
						  bool redraw)
{
	ReleasePattern();
	fDataHandle = thePattern;

	// get the rsrc id for non-color patterns (color pattern handles aren't resource handles) 
	if (!fIsColor)
	{
		ResNumber theID;
		ResType theType;
		CStr255_AC name;

		GetResInfo(thePattern, &theID, &theType, name);
		if (ResError() == noErr)
			fRsrcID = theID;
	}

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TPattern::SetPatternRsrcID: 
//----------------------------------------------------------------------------------------


void TPattern::SetPatternRsrcID(ResNumber itsRsrcID, bool redraw)
{
	ReleasePattern();

	fRsrcID = itsRsrcID;
	if (fRsrcID != kNoResource)
	{
		if (fPreferColor && (qNeedsColorQD || HasColorQD_AC()))
			fDataHandle = (Handle)GetPixPat(fRsrcID);	// try to get a 'ppat' resource

		if (fDataHandle)
			fIsColor = kPreferColor;
		else								// try to get a 'PAT ' resource
		{
			fDataHandle = (Handle)GetPattern(fRsrcID);
			if (fDataHandle)
				fIsColor =!kPreferColor;	// Either can't or won't 
		}

		::ThrowIfNULLResource_AC(fDataHandle);		// was ::ThrowIfResError_AC();
	}

	if (redraw)
		ForceRedraw();
}
