//----------------------------------------------------------------------------------------
// UPicture.cp 
// Copyright  1988-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UCOREUTILITIES__
#include "UCoreUtilities.h"
#endif

#ifndef __UPICTURE__
#include "UPicture.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UDRAGDROP__
#include "UDragDrop.h"
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

void InitUPicture()
{
	if (qTemplateViews)
	{
		MA_REGISTER_SIGNATURE(TPicture, kStdPicture);
	}
}  


//========================================================================================
// CLASS TPicture
//========================================================================================
#undef Inherited
#define Inherited TControl


MA_DEFINE_CLASS(TPicture);

//----------------------------------------------------------------------------------------
// TPicture constructor
//----------------------------------------------------------------------------------------


TPicture::TPicture(const CViewPoint& itsLocation,
						const CViewPoint& itsSize,
						SizeDeterminer itsHSizeDet,
						SizeDeterminer itsVSizeDet,
						ResNumber itsRsrcID) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet),

	fDataHandle(NULL),
	fRsrcID(kNoResource)
{
	fEventNumber = mPictureHit;

	SetPictureRsrcID(itsRsrcID, kDontRedraw);
	SetEnable(false);						// Default is to not enable hit testing 
}


//----------------------------------------------------------------------------------------
// TPicture copy constructor
//----------------------------------------------------------------------------------------


TPicture::TPicture(const TPicture& inPicture) :
	fDataHandle(NULL),
	fRsrcID(kNoResource)
{
	if (inPicture.fDataHandle)
		SetPictureRsrcID(inPicture.fRsrcID, kDontRedraw);
}


//----------------------------------------------------------------------------------------
// TPicture::Free: 
//----------------------------------------------------------------------------------------


TPicture::~TPicture()
{
	ReleasePicture();
}

//----------------------------------------------------------------------------------------
// TPicture::GetStandardSignature: 
//----------------------------------------------------------------------------------------


IDType TPicture::GetStandardSignature() const	// Override 
{
	return kStdPicture;
}

//----------------------------------------------------------------------------------------
// TPicture::ReadFields: 
//----------------------------------------------------------------------------------------


void TPicture::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	ResNumber itsRsrcID = aStream->ReadInteger();
	SetPictureRsrcID(itsRsrcID, kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TPicture::WriteFields: 
//----------------------------------------------------------------------------------------


void TPicture::WriteFields(CStream_AC* aStream) const				// Override 
{
	Inherited::WriteFields(aStream);

#if qDebugMsg && !qDrag
	if (fRsrcID == kNoResource)
		ProgramReport_AC("Tried to write TPicture with no resource ID.\n");
#endif

	aStream->WriteInteger(fRsrcID);
}

//----------------------------------------------------------------------------------------
// TPicture::Draw: 
//----------------------------------------------------------------------------------------


void TPicture::Draw(const CViewRect& area)	// Override 
{
	if (fDataHandle)
	{
		if (IsAResource_AC((Handle)fDataHandle))
			LoadResource((Handle)fDataHandle);
		if (*fDataHandle)				// If there's room for the picture 
		{
			CTempHandleLock_AC tempHandleLock((Handle)fDataHandle, eDontMoveHi, eDontReleaseIt, eDontLockIt);
			HNoPurge((Handle)fDataHandle);
			PenNormal();
			DrawPicture(fDataHandle, &ControlQDArea());
		}
	}
	Inherited::Draw(area);
}

//----------------------------------------------------------------------------------------
// TPicture::ReleasePicture: 
//----------------------------------------------------------------------------------------


void TPicture::ReleasePicture()
{
	fRsrcID = kNoResource;
	if (fDataHandle)
	{
		HPurge((Handle)fDataHandle);
		fDataHandle = NULL;
	}
}

//----------------------------------------------------------------------------------------
// TPicture::SetPicture: 
//----------------------------------------------------------------------------------------


void TPicture::SetPicture(PicHandle thePicture,
						  bool redraw)
{
	ReleasePicture();
	fDataHandle = thePicture;

	// get the rsrc id 
	ResNumber theID;
	ResType theType;
	CStr255_AC name;

	GetResInfo((Handle)thePicture, &theID, &theType, name);
	if (ResError() == noErr)
		fRsrcID = theID;

	if (redraw)
		ForceRedraw();
}

//----------------------------------------------------------------------------------------
// TPicture::SetPictureRsrcID: 
//----------------------------------------------------------------------------------------


void TPicture::SetPictureRsrcID(ResNumber itsRsrcID, bool redraw)
{
	ReleasePicture();

	fRsrcID = itsRsrcID;

	if (fRsrcID != kNoResource)
	{
		fDataHandle = GetPicture(fRsrcID);
		::ThrowIfNULLResource_AC((Handle)fDataHandle);		// was ::ThrowIfResError_AC();
	}

	if (redraw)
		ForceRedraw();
}

#if qDrag

//----------------------------------------------------------------------------------------
// TPicture::WillDrag
//----------------------------------------------------------------------------------------


bool TPicture::WillDrag(	const CViewPoint& 	localMouse,
							const RgnHandle	dragCursorRegion)
{
	return Inherited::WillDrag(localMouse, dragCursorRegion) ? (fDataHandle || fRsrcID != kNoResource) : false;
}

//----------------------------------------------------------------------------------------
// TPicture::DoAddDragContent
//----------------------------------------------------------------------------------------


void TPicture::DoAddDragContent()
{
	CFlavorFlags	flags;
	
	TDragDropSession::fgDragDropSession->AddDragItem(1)->PromiseFlavor(kPictType_AC, flags);
}

//----------------------------------------------------------------------------------------
// TPicture::WillAcceptDrop
//----------------------------------------------------------------------------------------


bool TPicture::WillAcceptDrop(CDragItemIterator& dragItemIterator)
{
	bool returnVal = false;

	if (dragItemIterator.GetItemCount() == 1)
		returnVal = dragItemIterator.First()->FlavorExists(kPictType_AC);
	
	return returnVal;
}

//----------------------------------------------------------------------------------------
// TPicture::DoMakeDragDropCommand
//----------------------------------------------------------------------------------------


TCommand* TPicture::DoMakeDragDropCommand(CommandNumber 		itsCommandNumber,
										CDragItemIterator&	dragItemIterator)
{
	TCommand  *returnCommand = NULL;
	TCommandHandler *itsContext = GetContext(itsCommandNumber);
	
	switch(itsCommandNumber)
	{
		case cDrag:
			returnCommand = TH_new TPictureCommand(kNoResource, NULL, this, itsCommandNumber, 
				itsContext, kCanUndo, kCausesChange, itsContext);
			break;
			
		case cDrop:
			{
				Handle	 pictureHandle = NULL;
				TDragItem *firstItem = dragItemIterator.First();
				
				firstItem->FocusOnFlavor(kPictType_AC);
				pictureHandle = firstItem->GetDataAsHandle();
					
				returnCommand = TH_new TPictureCommand(kNoResource, (PicHandle)pictureHandle, this, 
					itsCommandNumber, itsContext, kCanUndo, kCausesChange, itsContext);
			}
			break;
		
		case cDragMove:	// doesn't do anything...will cause return value to be NULL
			break;
			
		default:
			returnCommand = Inherited::DoMakeDragDropCommand(itsCommandNumber, dragItemIterator);
			break;
	}
	
	return returnCommand;
}

//----------------------------------------------------------------------------------------
// TPicture::DoFulfillPromise
//----------------------------------------------------------------------------------------


void TPicture::DoFulfillPromise(TDragItem* promisedItem)
{
	switch(promisedItem->GetFlavorType())
	{
		case kPictType_AC:
			{
				if (IsAResource_AC((Handle)fDataHandle))
					LoadResource((Handle)fDataHandle);
				::ThrowIfNULL_AC(fDataHandle);
				::ThrowIfNULL_AC(*fDataHandle);
				
				CTempHandleLock_AC tempHandleLock((Handle)fDataHandle, eDontMoveHi, eDontReleaseIt, eDontLockIt);
				
				HNoPurge((Handle)fDataHandle);
				promisedItem->SetDataFromHandle((Handle)fDataHandle);
			}
			break;
		
		default:
			Inherited::DoFulfillPromise(promisedItem);
			break;
	}
}

#endif // qDrag

//========================================================================================
// CLASS TPictureCommand
//========================================================================================
#undef Inherited
#define Inherited TCommand

//----------------------------------------------------------------------------------------
// TPictureCommand::TPictureCommand
//----------------------------------------------------------------------------------------


TPictureCommand::TPictureCommand(ResNumber				newRsrcID,
										PicHandle			newPicHandle,
										TPicture*			itsPictureView,
										CommandNumber		itsCommandNumber,
										TCommandHandler*	itsContext,
										bool				canUndo,
										bool				causesChange,
										MDependable_AC*			objectToNotify) :
	TCommand(itsCommandNumber, itsContext, canUndo, causesChange, objectToNotify),

	fSavedRsrcID(kNoResource),
	fSavedDataHandle(NULL)
{
	fSavedRsrcID = newRsrcID,
	fSavedDataHandle = newPicHandle;
	fPictureView = itsPictureView;
}

//----------------------------------------------------------------------------------------
// TPictureCommand::Free
//----------------------------------------------------------------------------------------


TPictureCommand::~TPictureCommand()
{
	if (fSavedDataHandle)
	{
		if (IsAResource_AC((Handle)fSavedDataHandle))
		{
			ReleaseResource((Handle)fSavedDataHandle);
			fSavedDataHandle = NULL;
		}
		else
			fSavedDataHandle = (PicHandle)::DisposeIfHandle_AC((Handle)fSavedDataHandle);
	}
}

//----------------------------------------------------------------------------------------
// TPictureCommand::DoIt
//----------------------------------------------------------------------------------------


void TPictureCommand::DoIt()
{
	PicHandle tempDataHandle = NULL;
	
	// prefer saving the resource ID.  If the resource ID is not valid, save the data handle
	ResNumber tempResID = fPictureView->fRsrcID;
	if ( tempResID == kNoResource)
	{
		tempDataHandle = fPictureView->fDataHandle;
		fPictureView->fDataHandle = NULL;
	}
	
	// prefer the resource ID over the data handle
	if (fSavedRsrcID != kNoResource)
		fPictureView->SetPictureRsrcID(fSavedRsrcID, kRedraw);
	else
		fPictureView->SetPicture(fSavedDataHandle, kRedraw);
		
	fSavedRsrcID = tempResID;
	fSavedDataHandle = tempDataHandle;
	
	if (fPictureView->IsDrawable())
	{
		CViewRect area(fPictureView->ControlArea());
		EraseRect(&fPictureView->ViewToQD(area));
		fPictureView->HandleDraw(area);
	}
}

//----------------------------------------------------------------------------------------
// TPictureCommand::UndoIt
//----------------------------------------------------------------------------------------


void TPictureCommand::UndoIt()
{
	DoIt();
}
