//----------------------------------------------------------------------------------------
// USizer.cp
// Copyright  1996-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USizer__
#include "USizer.h"
#endif

// MacApp

#ifndef __UCOREGLOBALS__
#include "UCoreGlobals.h"
#endif

#ifndef __UDISPATCHER__ 
#include "UDispatcher.h"
#endif

#ifndef __USTREAM__
#include "UStream.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

//========================================================================================
// Global Initialization Procedure
//========================================================================================

//----------------------------------------------------------------------------------------
// InitUSizer
//----------------------------------------------------------------------------------------

void InitUSizer()
{
	// So the linker doesn't dead strip class info 
	MA_REGISTER_CLASS(TBaseSizer);
	MA_REGISTER_CLASS(TBaseSpecSizer);
	MA_REGISTER_CLASS(TMultiSizer);
	MA_REGISTER_CLASS(TSizer);
	MA_REGISTER_CLASS(TRightSizer);
	MA_REGISTER_CLASS(TBottomSizer);
	MA_REGISTER_CLASS(TStickToParentSizer);
	MA_REGISTER_CLASS(TStickToBottomOfParent);
	MA_REGISTER_CLASS(TStickToRightOfParent);
	MA_REGISTER_CLASS(TStickToBottomRightOfParent);
}

//========================================================================================
// CLASS TBaseSizer
//========================================================================================


MA_DEFINE_CLASS(TBaseSizer);

//----------------------------------------------------------------------------------------
// TBaseSizer::TBaseSizer 
//----------------------------------------------------------------------------------------


TBaseSizer::TBaseSizer()
	:	fTargetView(NULL),
		fInitialized(false)
{
}

//----------------------------------------------------------------------------------------
// TBaseSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TBaseSizer::InitializeSizer()
{
	// Check to see if we've been initialized and if we haven't then 
	// add a dependency to the target view.
	if (!fInitialized)
	{
		// At this point the InitializeSizer() for the derived class should
		// have properly set fTargetView.
		if (qDebug)
			::ThrowIfNonObject_AC(fTargetView);

		fOldFrame = fTargetView->GetFrame();		// We need to remember the current frame.
		fTargetView->AddDependent(this);		// And register for dependency.

		fInitialized = true;
	}
}

//----------------------------------------------------------------------------------------
// TBaseSizer::DoUpdate 
//----------------------------------------------------------------------------------------


void TBaseSizer::DoUpdate(ChangeID_AC theChange,
						  MDependable_AC* changedObject,
						  void*					/* changeData */ ,
						  CDependencySpace_AC*		/* dependencySpace */ )
{
	// Policy: If the view which has changed is the fTargetView then test that
	// the frame has changed, calculate the delta frame and reset fOldFrame
	// and respond via the normal manner.  Otherwise respond using the
	// alternative RespondToOtherViewChange.  This will allow some flexibility
	// in the dependency mechanism such as with TRightSizer.

	// Initialize ourselves if we haven't been already.
	if (!fInitialized)
		InitializeSizer();

	// Make sure we only respond to the view we're expecting to change.
	if (theChange == mFrameChanged)
	{
		TView * changedView = dynamic_cast_AC(TView*,changedObject);
		// We have a view telling us that it's frame has changed.
		if (changedView == fTargetView)
		{
			// Normal frame change response.
			CViewRect newFrame(changedView->GetFrame());
			if (newFrame != fOldFrame)
			{
				CViewRect deltaFrame = newFrame - fOldFrame;
				this->RespondToFrameAdjustment(newFrame, deltaFrame, changedView);
				fOldFrame = newFrame;
			}
		}
		else if (changedView == fOwner)	// Alternative frame change.
			this->RespondToOtherViewChange(changedView);
	}
}

//========================================================================================
// CLASS TBaseSpecSizer
//========================================================================================


MA_DEFINE_CLASS(TBaseSpecSizer);

//----------------------------------------------------------------------------------------
// TBaseSpecSizer::TBaseSpecSizer 
//----------------------------------------------------------------------------------------


TBaseSpecSizer::TBaseSpecSizer() :
	fChildPeerOrParent(ePeerAssociation),
	fTargetViewID(0)
{
}

//----------------------------------------------------------------------------------------
// TBaseSpecSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TBaseSpecSizer::InitializeSizer()
{
	// Check to see if we've been initialized and if we haven't then find the 
	// target view.
	if (!fInitialized)
	{
		TView * targetView = NULL;

		// Figure out where the view is in which we are interested.
		switch (fChildPeerOrParent)
		{
			case eChildAssociation:
				targetView = (dynamic_cast_AC(TView*,fOwner))->FindSubView(fTargetViewID);
				break;

			case ePeerAssociation:
				targetView = (dynamic_cast_AC(TView*,fOwner))->FindPeerView(fTargetViewID);
				break;

			case eParentAssociation:
				targetView = (dynamic_cast_AC(TView*,fOwner))->FindParentView(fTargetViewID);
				break;
				// Add catch here on qDebug for unrecognized fChildPeerOrParent.
		}
		::ThrowIfNonObject_AC(targetView);
		fTargetView = targetView;
		TBaseSizer::InitializeSizer();		// Complete the initialization.
	}
}

//----------------------------------------------------------------------------------------
// TBaseSpecSizer::ReadFrom 
//----------------------------------------------------------------------------------------


void TBaseSpecSizer::ReadFrom(CStream_AC* aStream)
{
	TBehavior::ReadFrom(aStream);

	fTargetViewID = aStream->ReadIDType();
	fChildPeerOrParent = AssociationMode(aStream->ReadInteger());

	// I know it's skanky but this is where we post a command in order to handle 
	// the initialization of the behavior.
	gDispatcher->PostCommand(TH_new TInitSizerCommand(this));
}

//----------------------------------------------------------------------------------------
// TBaseSpecSizer::WriteTo
//----------------------------------------------------------------------------------------


void TBaseSpecSizer::WriteTo(CStream_AC* aStream) const
{
	TBehavior::WriteTo(aStream);

	aStream->WriteIDType(fTargetViewID);
	aStream->WriteInteger(fChildPeerOrParent);
}

//========================================================================================
// CLASS TMultiSizer
//========================================================================================


MA_DEFINE_CLASS(TMultiSizer);

//----------------------------------------------------------------------------------------
// TMultiSizer::TMultiSizer 
//----------------------------------------------------------------------------------------


TMultiSizer::TMultiSizer() :
	fTopMode(eDontAdjust),
	fLeftMode(eDontAdjust),
	fHeightMode(eDontAdjust),
	fWidthMode(eDontAdjust)
{
}

//----------------------------------------------------------------------------------------
// TMultiSizer::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TMultiSizer::RespondToFrameAdjustment(CViewRect&/* newFrame */  ,
										   CViewRect& deltaFrame,
										   TView*/* changedView */  )
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	CViewRect ourFrame(ownerView->GetFrame());

	switch (fTopMode)
	{
		case eAdjPosByTopOrLeft:
			ourFrame.top += deltaFrame.top;
			ourFrame.bottom += deltaFrame.top;
			break;

		case eAdjNegByTopOrLeft:
			ourFrame.top -= deltaFrame.top;
			ourFrame.bottom -= deltaFrame.top;
			break;

		case eAdjPosByBotOrRight:
			ourFrame.top += deltaFrame.bottom;
			ourFrame.bottom += deltaFrame.bottom;
			break;

		case eAdjNegByBotOrRight:
			ourFrame.top -= deltaFrame.bottom;
			ourFrame.bottom -= deltaFrame.bottom;
			break;
	}

	switch (fLeftMode)
	{
		case eAdjPosByTopOrLeft:
			ourFrame.left += deltaFrame.left;
			ourFrame.right += deltaFrame.left;
			break;

		case eAdjNegByTopOrLeft:
			ourFrame.left -= deltaFrame.left;
			ourFrame.right -= deltaFrame.left;
			break;

		case eAdjPosByBotOrRight:
			ourFrame.left += deltaFrame.right;
			ourFrame.right += deltaFrame.right;
			break;

		case eAdjNegByBotOrRight:
			ourFrame.left -= deltaFrame.right;
			ourFrame.right -= deltaFrame.right;
			break;
	}

	switch (fHeightMode)
	{
		case eAdjPosByTopOrLeft:
			ourFrame.bottom += deltaFrame.top;
			break;

		case eAdjNegByTopOrLeft:
			ourFrame.bottom -= deltaFrame.top;
			break;

		case eAdjPosByBotOrRight:
			ourFrame.bottom += deltaFrame.bottom;
			break;

		case eAdjNegByBotOrRight:
			ourFrame.bottom -= deltaFrame.bottom;
			break;
	}

	switch (fWidthMode)
	{
		case eAdjPosByTopOrLeft:
			ourFrame.right += deltaFrame.left;
			break;

		case eAdjNegByTopOrLeft:
			ourFrame.right -= deltaFrame.left;
			break;

		case eAdjPosByBotOrRight:
			ourFrame.right += deltaFrame.right;
			break;

		case eAdjNegByBotOrRight:
			ourFrame.right -= deltaFrame.right;
			break;
	}

	ownerView->SetFrame(ourFrame, kRedraw);
}

//----------------------------------------------------------------------------------------
// TMultiSizer::ReadFrom 
//----------------------------------------------------------------------------------------


void TMultiSizer::ReadFrom(CStream_AC* aStream)
{
	TBaseSpecSizer::ReadFrom(aStream);

	fTopMode = MultiAdjustMode(aStream->ReadInteger());
	fLeftMode = MultiAdjustMode(aStream->ReadInteger());
	fHeightMode = MultiAdjustMode(aStream->ReadInteger());
	fWidthMode = MultiAdjustMode(aStream->ReadInteger());
}

//----------------------------------------------------------------------------------------
// TMultiSizer::WriteTo
//----------------------------------------------------------------------------------------


void TMultiSizer::WriteTo(CStream_AC* aStream) const
{
	TBaseSpecSizer::WriteTo(aStream);

	aStream->WriteInteger(fTopMode);
	aStream->WriteInteger(fLeftMode);
	aStream->WriteInteger(fHeightMode);
	aStream->WriteInteger(fWidthMode);
}

//========================================================================================
// CLASS TSizer
//========================================================================================


MA_DEFINE_CLASS(TSizer);

//----------------------------------------------------------------------------------------
// TSizer::TSizer 
//----------------------------------------------------------------------------------------


TSizer::TSizer() :
	fChangeTypeMode(eNoMode),
	fAdjustMode(eDontAdjust),
	fProportion(1)
{
}

//----------------------------------------------------------------------------------------
// TSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TSizer::InitializeSizer()
{
	// This could probably be eliminated but to prevent misperceptions I'm
	// leaving it in.
	if (!fInitialized)
	{
		TBaseSpecSizer::InitializeSizer();
		SetGeneralMode(fChangeTypeMode, fAdjustMode);
	}
}

//----------------------------------------------------------------------------------------
// TSizer::ReadFrom 
//----------------------------------------------------------------------------------------


void TSizer::ReadFrom(CStream_AC* aStream)
{
	TBaseSpecSizer::ReadFrom(aStream);

	fChangeTypeMode = ChangeTypeMode(aStream->ReadInteger());
	fAdjustMode = AdjustMode(aStream->ReadInteger());
}

//----------------------------------------------------------------------------------------
// TSizer::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TSizer::RespondToFrameAdjustment(CViewRect&	/* newFrame */ ,
									  CViewRect& deltaFrame,
									  TView*	/* changedView */ )
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	CViewRect ourFrame(ownerView->GetFrame());

	switch (fChangeTypeMode)
	{
		case eTopMode:
			switch (fAdjustMode)
			{
				case eAdjPosByTopOrLeft:
					ourFrame.top += deltaFrame.top;
					ourFrame.bottom += deltaFrame.top;
					break;

				case eAdjNegByTopOrLeft:
					ourFrame.top -= deltaFrame.top;
					ourFrame.bottom -= deltaFrame.top;
					break;

				case eAdjPosByBotOrRight:
					ourFrame.top += deltaFrame.bottom;
					ourFrame.bottom += deltaFrame.bottom;
					break;

				case eAdjNegByBotOrRight:
					ourFrame.top -= deltaFrame.bottom;
					ourFrame.bottom -= deltaFrame.bottom;
					break;

				case eAdjProportional:
					{
						// Adjust based on the height of the parent view.
						ViewCoordinate ourHeight = ourFrame.GetSize()[vSel];
						double aHeight = fOldFrame.GetSize()[vSel] + deltaFrame.GetSize()[vSel];
						ViewCoordinate oldTop = ourFrame.top;
						ourFrame.top = aHeight * fProportion;
						ourFrame.bottom = ourFrame.top + ourHeight;
					}
					break;
			}
			break;

		case eLeftMode:
			switch (fAdjustMode)
			{
				case eAdjPosByTopOrLeft:
					ourFrame.left += deltaFrame.left;
					ourFrame.right += deltaFrame.left;
					break;

				case eAdjNegByTopOrLeft:
					ourFrame.left -= deltaFrame.left;
					ourFrame.right -= deltaFrame.left;
					break;

				case eAdjPosByBotOrRight:
					ourFrame.left += deltaFrame.right;
					ourFrame.right += deltaFrame.right;
					break;

				case eAdjNegByBotOrRight:
					ourFrame.left -= deltaFrame.right;
					ourFrame.right -= deltaFrame.right;
					break;

				case eAdjProportional:		// Adjust proportional to the height/width.
					{
						// Adjust based on the width of the parent view.
						ViewCoordinate ourWidth = ourFrame.GetSize()[hSel];
						double aWidth = fOldFrame.GetSize()[hSel] + deltaFrame.GetSize()[hSel];
						ViewCoordinate oldLeft = ourFrame.left;
						ourFrame.left = aWidth * fProportion;
						ourFrame.right = ourFrame.left + ourWidth;
					}
					break;
			}
			break;

		case eHeightMode:
			switch (fAdjustMode)
			{
				case eAdjPosByTopOrLeft:
					ourFrame.bottom += deltaFrame.top;
					break;

				case eAdjNegByTopOrLeft:
					ourFrame.bottom -= deltaFrame.top;
					break;

				case eAdjPosByBotOrRight:
					ourFrame.bottom += deltaFrame.bottom;
					break;

				case eAdjNegByBotOrRight:
					ourFrame.bottom -= deltaFrame.bottom;
					break;

				case eAdjProportional:
					{
						// Adjust based on the height of the target view.
						double aHeight = fOldFrame.GetSize()[vSel] + deltaFrame.GetSize()[vSel];
						double newHeight = aHeight * fProportion;
						ourFrame.bottom = ourFrame.top + newHeight;
					}
					break;
			}
			break;

		case eWidthMode:
			switch (fAdjustMode)
			{
				case eAdjPosByTopOrLeft:
					ourFrame.right += deltaFrame.left;
					break;

				case eAdjNegByTopOrLeft:
					ourFrame.right -= deltaFrame.left;
					break;

				case eAdjPosByBotOrRight:
					ourFrame.right += deltaFrame.right;
					break;

				case eAdjNegByBotOrRight:
					ourFrame.right -= deltaFrame.right;
					break;

				case eAdjProportional:		// Adjust proportional to the height/width.
					{
						// Adjust based on the width of the target view.
						double aWidth = fOldFrame.GetSize()[hSel] + deltaFrame.GetSize()[hSel];
						double newWidth = aWidth * fProportion;
						ourFrame.right = ourFrame.left + newWidth;
					}
					break;
			}
			break;
	}
	ownerView->SetFrame(ourFrame, kRedraw);
}

//----------------------------------------------------------------------------------------
// TSizer::SetGeneralMode
//----------------------------------------------------------------------------------------


void TSizer::SetGeneralMode(ChangeTypeMode theChangeType,
							AdjustMode theAdjustMode)
{
	// Call this to manually set the change type and adjustment modes.  (This is
	// also called from the initialization commands.)
	fChangeTypeMode = theChangeType;
	fAdjustMode = theAdjustMode;

	// Check to see if we've been initialized and if we haven't then add the adorner to the
	// target view.
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);

	// If this is a proportional relationship then let's figure out the proportion
	// right now rather than having to do so all the time.
	if (fAdjustMode == eAdjProportional)
		switch (fChangeTypeMode)
		{
			case eTopMode:
			case eLeftMode:
				{
					VHSelect aSel = (fChangeTypeMode == eTopMode) ? vSel : hSel;
					double aSize = fOldFrame.GetSize()[aSel];
					CViewPoint thePoint;
					// We have to crawl up to the indicated superview here.
					TView * aView = ownerView;
					while (aView != fTargetView)
					{
						::ThrowIfNonObject_AC(aView);
						thePoint = aView->LocalToSuperView(thePoint);
						aView = aView->GetSuperView();
					}
					double aCoordinate = thePoint[aSel];
					fProportion = aCoordinate / aSize;
				}
				break;

			case eHeightMode:
			case eWidthMode:
				{
					VHSelect aSel = (fChangeTypeMode == eHeightMode) ? vSel : hSel;
					double aSize = fOldFrame.GetSize()[aSel];
					double ourSize = ownerView->GetSize()[aSel];
					fProportion = ourSize / aSize;
				}
				break;
				// Add catch here on qDebug for unrecognized fChangeTypeModes.
		}
}

//----------------------------------------------------------------------------------------
// TSizer::WriteTo
//----------------------------------------------------------------------------------------


void TSizer::WriteTo(CStream_AC* aStream) const
{
	TBaseSpecSizer::WriteTo(aStream);

	aStream->WriteInteger(fChangeTypeMode);
	aStream->WriteInteger(fAdjustMode);
}

//========================================================================================
// CLASS TRightSizer
//========================================================================================


MA_DEFINE_CLASS(TRightSizer);

//----------------------------------------------------------------------------------------
// TRightSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TRightSizer::InitializeSizer()
{
	if (!fInitialized)
	{
		TBaseSpecSizer::InitializeSizer();

		// The right sizer also wants to know about any changes in the frame
		// of the owner view in case it has been affected through any other means.
		// What this means is that you can never call SetFrame for the ownerView
		// and expect to do anything to change the right thereof.
		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		fOldSelfFrame = ownerView->GetFrame();
		ownerView->AddDependent(this);
	}
}

//----------------------------------------------------------------------------------------
// TRightSizer::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TRightSizer::RespondToFrameAdjustment(CViewRect&/* newFrame */  ,
										   CViewRect& deltaFrame,
										   TView*/* changedView */  )
{
	if (!fInProcess)
	{
		fInProcess = true;

		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		CViewRect ourFrame(ownerView->GetFrame());

		// May want to check if (changedView == fTargetView) based on qDebug here.
		ourFrame.right += deltaFrame.right;
		ownerView->SetFrame(ourFrame, kRedraw);

		fInProcess = false;
	}
}

//----------------------------------------------------------------------------------------
// TRightSizer::RespondToOtherViewChange 
//----------------------------------------------------------------------------------------


void TRightSizer::RespondToOtherViewChange(TView* changedView)
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	if (changedView == ownerView && !fInProcess)
	{
		fInProcess = true;

		CViewRect ourFrame(ownerView->GetFrame());

		// Our own view's frame changed.
		ourFrame.right -= (ourFrame.right - fOldSelfFrame.right);

		ownerView->SetFrame(ourFrame, kRedraw);
		fOldSelfFrame = ourFrame;

		fInProcess = false;
	}
	else
	{
		// We have to grab our current frame just in case it has been changed
		// indirectly -- but we don't want to react otherwise.
		CViewRect ourFrame(ownerView->GetFrame());
		fOldSelfFrame = ourFrame;
	}
}

//========================================================================================
// CLASS TBottomSizer
//========================================================================================


MA_DEFINE_CLASS(TBottomSizer);

//----------------------------------------------------------------------------------------
// TBottomSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TBottomSizer::InitializeSizer()
{
	if (!fInitialized)
	{
		TBaseSpecSizer::InitializeSizer();

		// The bottom sizer also wants to know about any changes in the frame
		// of the owner view in case it has been affected through any other means.
		// What this means is that you can never call SetFrame for the ownerView
		// and expect to do anything to change the bottom thereof.
		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		fOldSelfFrame = ownerView->GetFrame();
		ownerView->AddDependent(this);
	}
}

//----------------------------------------------------------------------------------------
// TBottomSizer::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TBottomSizer::RespondToFrameAdjustment(CViewRect&/* newFrame */  ,
											CViewRect& deltaFrame,
											TView*/* changedView */  )
{
	if (!fInProcess)
	{
		fInProcess = true;

		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		CViewRect ourFrame(ownerView->GetFrame());

		// May want to check if (changedView == fTargetView) based on qDebug here.
		ourFrame.bottom += deltaFrame.bottom;
		ownerView->SetFrame(ourFrame, kRedraw);

		fInProcess = false;
	}
}

//----------------------------------------------------------------------------------------
// TBottomSizer::RespondToOtherViewChange 
//----------------------------------------------------------------------------------------


void TBottomSizer::RespondToOtherViewChange(TView* changedView)
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	if (changedView == ownerView && !fInProcess)
	{
		fInProcess = true;

		CViewRect ourFrame(ownerView->GetFrame());

		// Our own view's frame changed.
		ourFrame.bottom -= (ourFrame.bottom - fOldSelfFrame.bottom);

		ownerView->SetFrame(ourFrame, kRedraw);
		fOldSelfFrame = ourFrame;

		fInProcess = false;
	}
	else
		fOldSelfFrame = ownerView->GetFrame();
		// We have to grab our current frame just in case it has been changed
		// indirectly -- but we don't want to react otherwise.
}

//========================================================================================
// CLASS TStickToParentSizer
//========================================================================================


MA_DEFINE_CLASS(TStickToParentSizer);

//----------------------------------------------------------------------------------------
// TStickToParentSizer::InitializeSizer 
//----------------------------------------------------------------------------------------


void TStickToParentSizer::InitializeSizer()
{
	if (!fInitialized)
	{
		// This sizer also wants to know about any changes in the frame
		// of the owner view in case it has been affected through any other means.
		// What this means is that you can never call SetFrame for the ownerView
		// and expect to do anything to change the right and/or bottom thereof.
		
		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		fOldSelfFrame = ownerView->GetFrame();
		ownerView->AddDependent(this);

		fTargetView = ownerView->GetSuperView();

		TBaseSizer::InitializeSizer();
	}
}

//========================================================================================
// CLASS TStickToBottomOfParent
//========================================================================================


MA_DEFINE_CLASS(TStickToBottomOfParent);

//----------------------------------------------------------------------------------------
// TStickToBottomOfParent::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TStickToBottomOfParent::RespondToFrameAdjustment(CViewRect&	/* newFrame */,
													  CViewRect& 	deltaFrame,
													  TView*	/* changedView */)
{
	if (!fInProcess)
	{
		fInProcess = true;

		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		CViewRect ourFrame(ownerView->GetFrame());

		// May want to check if (changedView == fTargetView) based on qDebug here.
		ourFrame.bottom += deltaFrame.bottom;
		ownerView->SetFrame(ourFrame, kRedraw);

		fInProcess = false;
	}
}

//----------------------------------------------------------------------------------------
// TStickToBottomOfParent::RespondToOtherViewChange 
//----------------------------------------------------------------------------------------


void TStickToBottomOfParent::RespondToOtherViewChange(TView* changedView)
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	if (changedView == ownerView && !fInProcess)
	{
		fInProcess = true;

		CViewRect ourFrame(ownerView->GetFrame());

		// Our own view's frame changed.
		ourFrame.bottom -= (ourFrame.bottom - fOldSelfFrame.bottom);

		ownerView->SetFrame(ourFrame, kRedraw);
		fOldSelfFrame = ourFrame;

		fInProcess = false;
	}
	else
	{
		// We have to grab our current frame just in case it has been changed
		// indirectly -- but we don't want to react otherwise.
		CViewRect ourFrame(ownerView->GetFrame());
		fOldSelfFrame = ourFrame;
	}
}

//========================================================================================
// CLASS TStickToRightOfParent
//========================================================================================


MA_DEFINE_CLASS(TStickToRightOfParent);

//----------------------------------------------------------------------------------------
// TStickToRightOfParent::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TStickToRightOfParent::RespondToFrameAdjustment(CViewRect&	/* newFrame */,
													  CViewRect& 	deltaFrame,
													  TView*	/* changedView */)
{
	if (!fInProcess)
	{
		fInProcess = true;

		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		CViewRect ourFrame(ownerView->GetFrame());

		// May want to check if (changedView == fTargetView) based on qDebug here.
		ourFrame.right += deltaFrame.right;
		ownerView->SetFrame(ourFrame, kRedraw);

		fInProcess = false;
	}
}

//----------------------------------------------------------------------------------------
// TStickToRightOfParent::RespondToOtherViewChange 
//----------------------------------------------------------------------------------------


void TStickToRightOfParent::RespondToOtherViewChange(TView* changedView)
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	if (changedView == ownerView && !fInProcess)
	{
		fInProcess = true;

		CViewRect ourFrame(ownerView->GetFrame());

		// Our own view's frame changed.
		ourFrame.right -= (ourFrame.right - fOldSelfFrame.right);

		ownerView->SetFrame(ourFrame, kRedraw);
		fOldSelfFrame = ourFrame;

		fInProcess = false;
	}
	else
	{
		// We have to grab our current frame just in case it has been changed
		// indirectly -- but we don't want to react otherwise.
		CViewRect ourFrame(ownerView->GetFrame());
		fOldSelfFrame = ourFrame;
	}
}

//========================================================================================
// CLASS TStickToBottomRightOfParent
//========================================================================================


MA_DEFINE_CLASS(TStickToBottomRightOfParent);

//----------------------------------------------------------------------------------------
// TStickToBottomRightOfParent::RespondToFrameAdjustment 
//----------------------------------------------------------------------------------------


void TStickToBottomRightOfParent::RespondToFrameAdjustment(CViewRect&	/* newFrame */,
													  CViewRect& 	deltaFrame,
													  TView*	/* changedView */)
{
	if (!fInProcess)
	{
		fInProcess = true;

		TView * ownerView = dynamic_cast_AC(TView*,fOwner);
		CViewRect ourFrame(ownerView->GetFrame());

		// May want to check if (changedView == fTargetView) based on qDebug here.
		ourFrame[botRight] += deltaFrame[botRight];
		ownerView->SetFrame(ourFrame, kRedraw);

		fInProcess = false;
	}
}

//----------------------------------------------------------------------------------------
// TStickToBottomRightOfParent::RespondToOtherViewChange 
//----------------------------------------------------------------------------------------


void TStickToBottomRightOfParent::RespondToOtherViewChange(TView* changedView)
{
	TView * ownerView = dynamic_cast_AC(TView*,fOwner);
	if (changedView == ownerView && !fInProcess)
	{
		fInProcess = true;

		CViewRect ourFrame(ownerView->GetFrame());

		// Our own view's frame changed.
		ourFrame[botRight] -= (ourFrame[botRight] - fOldSelfFrame[botRight]);

		ownerView->SetFrame(ourFrame, kRedraw);
		fOldSelfFrame = ourFrame;

		fInProcess = false;
	}
	else
	{
		// We have to grab our current frame just in case it has been changed
		// indirectly -- but we don't want to react otherwise.
		CViewRect ourFrame(ownerView->GetFrame());
		fOldSelfFrame = ourFrame;
	}
}

//========================================================================================
// CLASS TInitSizerCommand
//========================================================================================
TInitSizerCommand::TInitSizerCommand(TBaseSpecSizer* aBehavior) :
	TCommand(0, gDispatcher, false, false, NULL),

	fBehavior(aBehavior)
{
}

void TInitSizerCommand::DoIt()
{
	fBehavior->InitializeSizer();
}

