//----------------------------------------------------------------------------------------
// USynchScroller.cp 
// Copyright  1989-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __USYNCHSCROLLER__
#include "USynchScroller.h"
#endif

// ACS

#ifndef __CIterator__
#include "CIterator_AC.h"
#endif

#ifndef __CList_AC__
#include "CList_AC.h"
#endif

// MacApp

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UPATCH__
#include "UPatch.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

// Toolbox

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

// Mac OS names are our preference, so remap conflicting ones
#if qMSWindows
#	ifndef __QTMLMAPNAMES__
#		include "QTMLMapNames.h"
#	endif
#endif

// ANSI

#ifndef __STDIO__
#include <stdio.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif

//----------------------------------------------------------------------------------------
// InitUSynchScroller: 
//----------------------------------------------------------------------------------------


void InitUSynchScroller()
{
#if qTemplateViews
	// Suppress Linker dead-stripping of these classes 
	MA_REGISTER_CLASS(TPrimaryScroller);
	MA_REGISTER_CLASS(TSecondaryScroller);
#endif
}


//========================================================================================
// CLASS TPrimaryScroller
//========================================================================================
#undef Inherited
#define Inherited TScroller


MA_DEFINE_CLASS(TPrimaryScroller);

//----------------------------------------------------------------------------------------
// TPrimaryScroller::Free: 
//----------------------------------------------------------------------------------------


TPrimaryScroller::~TPrimaryScroller()
{
	for (TemplateIterator_AC<TSecondaryScroller*> iter(&fSecondaryScrollers); iter; ++iter)
		RemoveSecondaryScroller(iter);
}

//----------------------------------------------------------------------------------------
// TPrimaryScroller::AddSecondaryScroller: 
//----------------------------------------------------------------------------------------


void TPrimaryScroller::AddSecondaryScroller(TSecondaryScroller* itsSecondaryScroller,
												   ViewCoordinate itsHDependency,
												   ViewCoordinate itsVDependency)
{
	if (itsSecondaryScroller)
	{
		itsSecondaryScroller->fPrimaryScroller = this;
		itsSecondaryScroller->fDeltaFactor = CViewPoint(itsHDependency, itsVDependency).ToPoint();

		fSecondaryScrollers.Insert(itsSecondaryScroller);
	}
}

//----------------------------------------------------------------------------------------
// TPrimaryScroller::RemoveSecondaryScroller: 
//----------------------------------------------------------------------------------------


void TPrimaryScroller::RemoveSecondaryScroller(TSecondaryScroller* itsSecondaryScroller)
{
	if (itsSecondaryScroller)
	{
		itsSecondaryScroller->fPrimaryScroller = NULL;
		fSecondaryScrollers.Delete(itsSecondaryScroller);
	}
}

//----------------------------------------------------------------------------------------
// TPrimaryScroller::DoScroll: 
//----------------------------------------------------------------------------------------


void TPrimaryScroller::DoScroll(const CViewPoint& delta,
									   bool redraw)// override 
{
	{
		for (TemplateIterator_AC<TSecondaryScroller*> iter(&fSecondaryScrollers); iter; ++iter)
		{
			TSecondaryScroller* aSecondaryScroller = iter;
			
			CViewPoint itsDelta(delta);
		
			for (VHSelect vhs = vSel; vhs <= hSel; ++vhs)
				if (aSecondaryScroller->fDeltaFactor[vhs])	// kVDependent or kHDependent 
				{
					if (itsDelta[vhs] < 0)
						itsDelta[vhs] = Max_AC(itsDelta[vhs], -aSecondaryScroller->GetTranslation()[vhs]);
					else if (itsDelta[vhs] > 0)
						itsDelta[vhs] = Min_AC(itsDelta[vhs], aSecondaryScroller->fMaxTranslation[vhs] - aSecondaryScroller->GetTranslation()[vhs]);

					CViewPoint itsTranslation(aSecondaryScroller->GetTranslation());
					itsTranslation[vhs] += itsDelta[vhs];
					aSecondaryScroller->SetTranslation(itsTranslation);
				}
		
			if (itsDelta != gZeroVPt)
			{
				aSecondaryScroller->InvalidateFocus();	// You never know who might call 
				aSecondaryScroller->InvalidateCoordinates();
			}
		}
	}

	Inherited::DoScroll(delta, redraw);
}

//----------------------------------------------------------------------------------------
// TPrimaryScroller::ScrollDraw: 
//----------------------------------------------------------------------------------------


void TPrimaryScroller::ScrollDraw(const CViewPoint& delta,
										 bool invalidate)// override 
{
	if (fSuperView->IsDrawable())
	{
		CTempRegion_AC scrollRgn;

		GetExtentRegion(scrollRgn);
		LocalToSuperView(scrollRgn);

		CRect_AC superVisRect((*GetClipRegion(qd.thePort))->rgnBBox); // just test the clip, it's faster

#if qDebug
		if (gIntenseDebugging)
			WriteFocus();
#endif

	    const CViewRect visVRect(superVisRect);
	    if (((visVRect - delta) & visVRect).Empty()) // too far to scrollrect 
		{
			ForceRedraw();

			for (TemplateIterator_AC<TSecondaryScroller*> iter(&fSecondaryScrollers); iter; ++iter)
			{
				TSecondaryScroller* aScroller = iter;
				
				if (aScroller->IsShown ())
				{
					if (delta.v && (aScroller->fDeltaFactor.v == kVDependent))
						aScroller->ForceRedraw();
					else if (delta.h && (aScroller->fDeltaFactor.h == kHDependent))
						aScroller->ForceRedraw();
				}
			}
		}
		else		// Can use ScrollRect
		{
			CTempRegion_AC tempRgn;

#if qDebug
			fSuperView->AssumeFocused();
#endif

			{		// for correct failure handling
				for (TemplateIterator_AC<TSecondaryScroller*> iter(&fSecondaryScrollers); iter; ++iter)
				{
					TSecondaryScroller* aScroller = iter;
					if (aScroller->IsShown ())
					{
						CPoint_AC localDeltaFactor(aScroller->fDeltaFactor);
					
						CRect_AC frameRect(aScroller->GetSuperView()->ViewToQD(aScroller->GetFrame()));
						// main scroller is moving both directions but dependent can only move in v
						if (delta.h && delta.v && (localDeltaFactor.h == kNotHDependent) && (localDeltaFactor.v == kVDependent))
						{
							ScrollRect(frameRect, 0, static_cast<short>(-delta.v), tempRgn);
							aScroller->GetSuperView()->Invalidate(tempRgn);
						}
						// main scroller is moving both directions but dependent can only move in h
						else if (delta.h && delta.v && (localDeltaFactor.h == kHDependent) && (localDeltaFactor.v == kNotVDependent))
						{
							ScrollRect(frameRect, static_cast<short>(-delta.h), 0, tempRgn);
							aScroller->GetSuperView()->Invalidate(tempRgn);
						}
						// main scroller is moving in either direction and dependent can follow
						else if ((delta.h && (localDeltaFactor.h == kHDependent)) || (delta.v && (localDeltaFactor.v == kVDependent)))
						{
							aScroller->GetExtentRegion(tempRgn);
							aScroller->LocalToSuperView(tempRgn);
							UnionRgn(scrollRgn, tempRgn, scrollRgn);
						}
					}
				}
			}

#if qDebug
			fSuperView->AssumeFocused();
#endif

			// Now, we've either scrolled any dependent scrollers that can't scroll with
			// us or we've accumulated their frames in scrollRgn.  By clipping to the
			// accumulated region we can just scroll the entire superview and the right
			// stuff should happen. (Cross fingers)
			SectRgn(scrollRgn, qd.thePort->clipRgn, scrollRgn);

			// Get clip so we can restore it later and keep from having
			// to invalidate the superview's focus
			GetClip(tempRgn);

			SetClip(scrollRgn);
			ScrollRect(superVisRect, static_cast<short>(-delta.h), static_cast<short>(-delta.v), scrollRgn);

			// restore the clip so we don't have to invalidate the focus
			SetClip(tempRgn);

			fSuperView->Invalidate(scrollRgn);
		}

		if (!invalidate)
			fSuperView->Update();
	}
}



//========================================================================================
// CLASS TSecondaryScroller
//========================================================================================
#undef Inherited
#define Inherited TScroller


MA_DEFINE_CLASS(TSecondaryScroller);

//----------------------------------------------------------------------------------------
// TSecondaryScroller constructor
//----------------------------------------------------------------------------------------


TSecondaryScroller::TSecondaryScroller(const CViewPoint& itsLocation,
								  const CViewPoint& itsSize,
								  SizeDeterminer itsHSizeDet,
								  SizeDeterminer itsVSizeDet,
								  const CViewPoint& itsMax,
								  bool wantHorzSBar,
								  bool wantVertSBar) :
	TScroller(itsLocation, itsSize,
					itsHSizeDet, itsVSizeDet, itsMax,wantHorzSBar, wantVertSBar),

	fPrimaryScroller(NULL),
	fDeltaFactor(kNotHDependent, kNotVDependent)
{
}

//----------------------------------------------------------------------------------------
// TSecondaryScroller::Free: 
//----------------------------------------------------------------------------------------


TSecondaryScroller::~TSecondaryScroller()

{
	if (fPrimaryScroller)
		fPrimaryScroller->RemoveSecondaryScroller(this);	// sets fPrimaryScroller to NULL
}

//----------------------------------------------------------------------------------------
// TSecondaryScroller::DoScroll: 
//----------------------------------------------------------------------------------------


void TSecondaryScroller::DoScroll(const CViewPoint& delta,
										 bool redraw)// override 
{
	if (fPrimaryScroller)
		fPrimaryScroller->ScrollBy( CViewPoint( fDeltaFactor.h * delta.h, fDeltaFactor.v * delta.v), redraw);
}

//----------------------------------------------------------------------------------------
// End of USynchScroller.cp


