//----------------------------------------------------------------------------------------
// UTabbedView.cp
// Copyright  1997 by Apple Computer, Inc. All rights reserved. 
// Portions Copyright 1997 by Trygve Isaacson.
//----------------------------------------------------------------------------------------

#ifndef __UTABBEDVIEW__
#include "UTabbedView.h"
#endif

// ACS

#ifndef __ColorRamp_AC__
#include "ColorRamp_AC.h"
#endif

// MacApp

#ifndef __UVIEWSERVER__
#include "UViewServer.h"
#endif

#ifndef __UEDITTEXT__
#include "UEditText.h"
#endif

#ifndef __UMenuMgr__
#include "UMenuMgr.h"
#endif

#ifndef __UAPPLICATION__
#include "UApplication.h"
#endif

//----------------------------------------------------------------------------------------
// InitUTabbedView
//----------------------------------------------------------------------------------------

void InitUTabbedView()
{
	MA_REGISTER_CLASS(TTabbedView);
	MA_REGISTER_CLASS(TSmallTabbedView);
}

//========================================================================================
// CLASS TTabbedView
//========================================================================================

#undef Inherited
#define Inherited TControl

MA_DEFINE_CLASS(TTabbedView);

//----------------------------------------------------------------------------------------
// TTabbedView constructor
//----------------------------------------------------------------------------------------

TTabbedView::TTabbedView(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
							     const CTextStyle_AC& itsTextStyle,
							     ResNumber itsLabelsID) :
	TControl(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle),
	fAGAControl(NULL),
	fPanelContainerView(NULL),
	fActivePanelView(NULL),
	fLabelsID(itsLabelsID)
{
	// caller must call virtual function CreateAGAControl();	
	// caller must call virtual function CreateContainerView();
}

//----------------------------------------------------------------------------------------
// TTabbedView destructor
//----------------------------------------------------------------------------------------

TTabbedView::~TTabbedView()
{
	// We need to make sure that all panel views not currently
	// installed get destructed.

	if (fAGAControl)
	{
		SInt32	numTabs;

		numTabs = fAGAControl->GetNumTabs();
		
		for (SInt32 tabIndex = 0; tabIndex < numTabs; ++tabIndex)
		{
			if (tabIndex != fAGAControl->GetCurrentTab())
			{
				TView*	aPanelView = static_cast<TView*>(fAGAControl->GetTabUserData(tabIndex));
				delete aPanelView;
			}
		}
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::ReadFields: 
//----------------------------------------------------------------------------------------

void TTabbedView::ReadFields(CStream_AC* aStream)				// Override 
{
	Inherited::ReadFields(aStream);

	*aStream >> fLabelsID;
}

//----------------------------------------------------------------------------------------
// TTabbedView::WriteFields: 
//----------------------------------------------------------------------------------------

void TTabbedView::WriteFields(CStream_AC* aStream) const			// Override 
{
	Inherited::WriteFields(aStream);

	*aStream << fLabelsID;
}

//----------------------------------------------------------------------------------------
// TTabbedView::DoPostCreate:
//----------------------------------------------------------------------------------------

void TTabbedView::DoPostCreate(TDocument* itsDocument)
{
	// Create and set up the CControl_AC.

	Inherited::DoPostCreate(itsDocument);
	
	CreateAGAControl();
	
	// Create the container view that we'll load panels in and out of.
	
	CreateContainerView();
}

//----------------------------------------------------------------------------------------
// TTabbedView::CreateAGAControl
//----------------------------------------------------------------------------------------

void TTabbedView::CreateAGAControl()
{
	fTextStyle.Normalize(gSystemStyle.tsFont, gApplicationStyle.tsFont);
	fAGAControl = TH_new CTabbedView_AC(ControlQDArea(), 0, fTextStyle, fLabelsID);

	if (fDimmed || ! fEnabled)
		fAGAControl->SetEnable(CControl_AC::kDisabled, CControl_AC::kDontRedraw);

	// Tabs have to erase when drawing, so make sure that
	// it doesn't assume we're on a gray background.

	(void) Focus();
	fAGAControl->InstallBackgroundColors(CControl_AC::kInstallOnlyIfCustom);
}

//----------------------------------------------------------------------------------------
// TTabbedView::CreateContainerView
//----------------------------------------------------------------------------------------

void TTabbedView::CreateContainerView()
{
	if (fAGAControl->HasFrame())	// if no frame, we just run the tabs, not the panels
	{
		// Create a subview to contain the panels that we load and unload.
		// We locate it 3 pixels inset from our the frame content area.
		
		ViewCoordinate	tabHeight = fAGAControl->GetTabHeight();
		CViewPoint		viewLocation(3, 3 + tabHeight);
		CViewPoint		viewSize(fSize.h - 6, fSize.v - tabHeight - 6);
		
		fPanelContainerView = TH_new TView(viewLocation, viewSize, fSizeDeterminer[hSel], fSizeDeterminer[vSel]);
		AddSubView(fPanelContainerView);
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::Draw
//----------------------------------------------------------------------------------------

void TTabbedView::Draw(const CViewRect& /*area*/)
{
	// Let the CControl_AC draw itself.

	if (fAGAControl)
		fAGAControl->DrawObject();

	// Set the background color so that any non-AGA subviews
	// that erase their background without assuming a color
	// will use our current tab panel color.

	if (IsEnabled())
		kRGBColorRamp[r1].SetBackColor();
	else
		kRGBColorRamp[r2].SetBackColor();
}

//----------------------------------------------------------------------------------------
// TTabbedView::SetFrame
//----------------------------------------------------------------------------------------

void TTabbedView::SetFrame(const CViewRect& newFrame, bool invalidate)
{
	// Let the MacApp and AGA objects update their locations.
	
	(void) Focus();
	CRect_AC	frame(GetQDExtent());
	::InvalRect(frame);
	
	Inherited::SetFrame(newFrame, invalidate);
	
	if (fAGAControl)
		fAGAControl->SetBounds(frame, CControl_AC::kDontRedraw);
}

//----------------------------------------------------------------------------------------
// TTabbedView::DoMouseCommand
//----------------------------------------------------------------------------------------

void TTabbedView::DoMouseCommand(CViewPoint& theMouse, TToolboxEvent* /*event*/, CPoint_AC /*hysteresis*/)
{
	// Let the CControl_AC track the mouse; if tracking succeeds
	// handle our event number.

	if (fAGAControl->TrackMouse(ViewToQD(theMouse)))
		HandleTabSwitch(fAGAControl->GetLastTabClicked());
}

//----------------------------------------------------------------------------------------
// TTabbedView::Activate
//----------------------------------------------------------------------------------------

void TTabbedView::Activate(bool entering)
{
	if (fAGAControl)
		fAGAControl->SetActive(entering, Focus() && IsVisible());
	
	Inherited::Activate(entering);

	if (fPanelContainerView != NULL)
	{
		fPanelContainerView->ForceRedraw();
		fPanelContainerView->Update();
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::IsActive
//----------------------------------------------------------------------------------------

bool TTabbedView::IsActive() const
{
	return fAGAControl->IsActive();
}

//----------------------------------------------------------------------------------------
// TTabbedView::DimState
//----------------------------------------------------------------------------------------

void TTabbedView::DimState(bool state, bool redraw)
{
	// Set the CControl_AC's enable state.

	fDimmed = state;

	if (fAGAControl)
	{
		fAGAControl->SetEnable(!state, redraw && Focus() && IsVisible());
	}

	SetSubviewsEnable(! state);
}

//----------------------------------------------------------------------------------------
// TTabbedView::Dim
//----------------------------------------------------------------------------------------

void TTabbedView::Dim()
{
	// Set the CControl_AC's enable state, with immediate redraw.

	if (fAGAControl)
		fAGAControl->SetEnable(CControl_AC::kDisabled, Focus() && IsVisible());
}

//----------------------------------------------------------------------------------------
// TTabbedView::SetEnable
//----------------------------------------------------------------------------------------

void TTabbedView::SetEnable(bool state)
{
	Inherited::SetEnable(state);

	// Set the CControl_AC's enable state, without redraw.

	if (fAGAControl)
		fAGAControl->SetEnable(state, CControl_AC::kDontRedraw);

	SetSubviewsEnable(state);
}

//----------------------------------------------------------------------------------------
// TTabbedView::SwitchPanels
//----------------------------------------------------------------------------------------

void TTabbedView::SwitchPanels(SInt32 newTabIndex)
{
	// Switch to the specified tab index.
	
	if (fAGAControl)
		fAGAControl->SetCurrentTab(newTabIndex, Focus());

	SwitchPanelIn(newTabIndex, SwitchPanelOut());
}

//----------------------------------------------------------------------------------------
// TTabbedView::HandleTabSwitch
//----------------------------------------------------------------------------------------

void TTabbedView::HandleTabSwitch(SInt32 newTabIndex)
{
	// If it's OK to switch tabs (may need to validate current
	// panel before switching), do the switch.

	SInt32	oldTabIndex = fAGAControl->GetCurrentTab();

	if ((newTabIndex != oldTabIndex) && ValidatePanel(oldTabIndex))
	{
		SwitchPanels(newTabIndex);
		HandleEvent(fEventNumber, this, NULL);
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::AddPanel
//----------------------------------------------------------------------------------------

void TTabbedView::AddPanel(SInt16 panelResourceID, StringPtr tabLabel)
{
	// Create another tab and then install the specified View resource
	// onto that tab.

	if (fAGAControl)
	{
		CStr255_AC	actualTabLabel;
		SInt32	numTabs;

		if (tabLabel != NULL)
			actualTabLabel = tabLabel;
		else
			{
		Handle	viewHandle;
			SInt16	viewID;
			OSType	viewType;
			
			::ThrowIfNULL_AC(viewHandle = ::GetResource('View', panelResourceID));
			
			::GetResInfo(viewHandle, &viewID, &viewType, actualTabLabel);
			
			// Don't release it -- we're about to cause it to be read anyway.
		}

		numTabs = fAGAControl->GetNumTabs();
		
		fAGAControl->SetNumTabs(numTabs + 1, CTabbedView_AC::kDontShrink);
		
		InstallPanel(numTabs, panelResourceID, actualTabLabel);
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::InstallPanel
//----------------------------------------------------------------------------------------

void TTabbedView::InstallPanel(SInt32 tabIndex, SInt16 panelResourceID, StringPtr tabLabel)
{
	// Install the specified View resource onto the specified existing tab.

	if (fAGAControl)
	{
		TView*	newPanelView;
		SInt32	numTabs;

		numTabs = fAGAControl->GetNumTabs();
		
		if (tabIndex < numTabs)
			{
		// Wrap the DoCreateViews with custom color values so that the
			// AGA objects in the panel hierarchy will get the right background
			// colors.

			fAGAControl->TurnOnCustomBackgroundColors(kRGBColorRamp[r1], kRGBColorRamp[r2]);
			::ThrowIfNULL_AC(newPanelView = TViewServer::fgViewServer->DoCreateViews(fDocument, fPanelContainerView, panelResourceID, gZeroVPt));
			fAGAControl->TurnOffCustomBackgroundColors();

			newPanelView->Show(false, kRedraw);

			fAGAControl->SetTabUserData(tabIndex, newPanelView);
			
			if (tabLabel != NULL)
			{
				fAGAControl->SetTabLabel(tabIndex, tabLabel);
				fAGAControl->CalculateTabWidths();
			}

			// If this panel corresponds to the initial value, switch it in.

			if (fAGAControl->GetCurrentTab() == tabIndex)
				SwitchPanels(tabIndex);
		}
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::RemoveTab
//----------------------------------------------------------------------------------------

void TTabbedView::RemoveTab(SInt32 tabIndex, bool redraw)
{
	// Remove the specified tab from the AGA object.

	fAGAControl->RemoveTab(tabIndex);
	
	if (redraw && Focus())
		fAGAControl->DrawObject();
}

//----------------------------------------------------------------------------------------
// TTabbedView::GetPanelView
//----------------------------------------------------------------------------------------

TView* TTabbedView::GetPanelView(SInt32 tabIndex)
{
	// Return the TView* of the specified tab's panel view hierarchy.

	return fAGAControl ? NULL : static_cast<TView*>(fAGAControl->GetTabUserData(tabIndex));
}

//----------------------------------------------------------------------------------------
// TTabbedView::SetSubviewsEnable
//----------------------------------------------------------------------------------------

void TTabbedView::SetSubviewsEnable(bool state)
{
	// Do a recursive walk of our panel subviews, setting the
	// enable state of all of its subviews.

	if (fPanelContainerView != NULL)
	{
		DoSubviewsEnable(fPanelContainerView, state);
		
		fPanelContainerView->ForceRedraw();
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::DoSubviewsEnable
//----------------------------------------------------------------------------------------

void TTabbedView::DoSubviewsEnable(TView* aSuperView, bool state)
{
	// Do a recursive walk of our panel subviews, setting the
	// enable state of all of its subviews.

	CSubViewIterator	iter(aSuperView);
	
	for (TView* aSubView = iter.FirstSubView(); iter.Current(); aSubView = iter.NextSubView())
	{
		aSubView->SetEnable(state);

		if (dynamic_cast<TControl*>(aSubView))
			dynamic_cast<TControl*>(aSubView)->DimState(! state, kDontRedraw);
		
		DoSubviewsEnable(aSubView, state);
	}
}

//----------------------------------------------------------------------------------------
// TTabbedView::ValidatePanel
//----------------------------------------------------------------------------------------

bool TTabbedView::ValidatePanel(SInt32 /*currentTabIndex*/)
{
	// (Override for app-specific validation.)
	//
	// Return true if the current (specified) panel is valid and
	// it's OK to switch to a different panel. Return false otherwise
	// and the tab switch will not occur; it's OK to alert the user
	// of the invalid data before returning.

	return true;
}

//----------------------------------------------------------------------------------------
// TTabbedView::SwitchPanelOut
//----------------------------------------------------------------------------------------

bool TTabbedView::SwitchPanelOut()
{
	// Yank the current panel view hierarchy out of the container view.
	
	TEventHandler*	currentTarget = GetTarget();
	bool	panelContainedTarget = (currentTarget == NULL) ||
						(currentTarget == GetWindow()) ||
						(currentTarget == gApplication);

	if (fActivePanelView != NULL)
	{
		if ((currentTarget != NULL) && 
			dynamic_cast<TView*>(currentTarget) &&
			fActivePanelView->ContainsSubView(dynamic_cast_AC(TView*,currentTarget)))
			{
				panelContainedTarget = true;
				GetWindow()->BecomeTarget();
				GetWindow()->SetWindowTarget(NULL);
			}

		fActivePanelView->Show(false, kRedraw);
		//fPanelContainerView->RemoveSubView(fActivePanelView);
		fActivePanelView = NULL;
	}
	
	return panelContainedTarget;
}

//----------------------------------------------------------------------------------------
// TTabbedView::SwitchPanelIn
//----------------------------------------------------------------------------------------

void TTabbedView::SwitchPanelIn(SInt32 newTabIndex, bool targetInPanel)
{
	// Switch the specified panel view hierarchy in buy placing it
	// inside the container view.

	if (fAGAControl)
	{
		TView*	newPanelView = static_cast<TView*>(fAGAControl->GetTabUserData(newTabIndex));
		
		if (newPanelView != NULL)
		{
			fActivePanelView = newPanelView;

			//
			// Before redrawing, center the new panel inside the container.
			//
			newPanelView->Show(true, kRedraw);
			
			if (targetInPanel)
			{
				TView*	wannabeTarget = newPanelView->FindTarget();
				
				if (wannabeTarget != NULL)
				{
					(void) wannabeTarget->BecomeTarget();
					
					if (dynamic_cast<TEditText*>(wannabeTarget))
						dynamic_cast<TEditText*>(wannabeTarget)->SetTargetSelection(kRedraw);
				}
			}
		}
		
		InvalidateMenus();
	}
}

//========================================================================================
// CLASS TSmallTabbedView
//========================================================================================

#undef Inherited
#define Inherited TTabbedView

MA_DEFINE_CLASS(TSmallTabbedView);

//----------------------------------------------------------------------------------------
// TSmallTabbedView::TSmallTabbedView
//----------------------------------------------------------------------------------------

TSmallTabbedView::TSmallTabbedView(const CViewPoint& itsLocation,
								 const CViewPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
							     const CTextStyle_AC& itsTextStyle) :
	TTabbedView(itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTextStyle)
{
}

//----------------------------------------------------------------------------------------
// TSmallTabbedView::~TSmallTabbedView
//----------------------------------------------------------------------------------------

TSmallTabbedView::~TSmallTabbedView()
{
}

//----------------------------------------------------------------------------------------
// TSmallTabbedView::CreateAGAControl
//----------------------------------------------------------------------------------------

void TSmallTabbedView::CreateAGAControl()
{
	Inherited::CreateAGAControl();
	
	// Set the tabs to use the standard small appearance.

	fAGAControl->SetTabSize(CTabbedView_AC::kSmallTabs);
	fAGAControl->SetLabelsStyle(CTabbedView_AC::kSmallTabPanelStyle);
}
