//----------------------------------------------------------------------------------------
// UWindowMenuBehavior.cp 
// Copyright  1997 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __UWINDOWMENUBEHAVIOR__
#include "UWindowMenuBehavior.h"
#endif

// MacApp

#ifndef __UDISPATCHER__
#include "UDispatcher.h"
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __UMENUMGR__
#include "UMenuMgr.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif

//================================================================================
//	Class TWindowSortedList
//================================================================================

#undef Inherited
#define Inherited CSortedList_AC


MA_DEFINE_CLASS(TWindowSortedList);

//--------------------------------------------------------------------------------
// ~TWindowSortedList
//--------------------------------------------------------------------------------

TWindowSortedList::~TWindowSortedList()
{
}

//--------------------------------------------------------------------------------
// TWindowSortedList
//--------------------------------------------------------------------------------

TWindowSortedList::TWindowSortedList(bool sortByDocument) :
	fSortByDocument(sortByDocument)
{
}



CompareResult_AC TWindowSortedList::Compare(const void* item1,
												const void* item2) const
{
	CompareResult_AC returnVal = kItem1EqualItem2_AC;
	//
	// Sort the windows alphabetically by title
	//
	const TWindow * window1 = static_cast<const TWindow*>(item1);
	const TWindow * window2 = static_cast<const TWindow*>(item2);

	CStr255_AC title1(window1->GetTitle());
	CStr255_AC title2(window2->GetTitle());

	if (fSortByDocument)
	{
		TDocument * doc1 = window1->GetDocument();
		TDocument * doc2 = window2->GetDocument();

		CStr255_AC docTitle1;
		CStr255_AC docTitle2;

		if (doc1)
			docTitle1 = doc1->GetTitle();

		if (doc2)
			docTitle2 = doc2->GetTitle();

		returnVal = ::RelString(docTitle1, docTitle2, false, true);
		if (returnVal == kItem1EqualItem2_AC)
			returnVal = RelString(title1, title2, false, true);
	}
	else
		returnVal = ::RelString(title1, title2, false, true);

	return returnVal;
}

//================================================================================
//	Class TWindowMenuBehavior
//================================================================================

#undef Inherited
#define Inherited TBehavior


MA_DEFINE_CLASS(TWindowMenuBehavior);

//--------------------------------------------------------------------------------
// TWindowMenuBehavior::TWindowMenuBehavior
//--------------------------------------------------------------------------------

TWindowMenuBehavior::TWindowMenuBehavior(IDType itsIdentifier,
												 ResNumber itsMenuNo,
												 EventNumber itsInvalidateEvent,
												 bool sortByDocument,
												 bool documentSeparators,
												 bool includeHidden) :
	TBehavior(itsIdentifier),

	fWindowList(documentSeparators ? static_cast<CSortedList_AC*>(::NewList_AC()) : TH_new TWindowSortedList(sortByDocument)),
	fWindowMenuInvalid(true),
	fMenuNo(itsMenuNo),
	fReserveMenuItems(0),
	fDocumentSeparators(documentSeparators)
{
	fInvalidateEvent = itsInvalidateEvent;
	fIncludeHidden = includeHidden;

	//
	// Automatically reserve any pre-existing menu items so they don't get deleted
	//
	MenuHandle theMenu = MAGetMenu(fMenuNo);
	if (theMenu)
		fReserveMenuItems = CountMItems(theMenu);

}

//--------------------------------------------------------------------------------
// TWindowMenuBehavior::~TWindowMenuBehavior
//--------------------------------------------------------------------------------

TWindowMenuBehavior::~TWindowMenuBehavior()
{
}

//--------------------------------------------------------------------------------
// TWindowMenuBehavior::DoMenuCommand
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::DoMenuCommand(CommandNumber aCommandNumber)
{
	//
	// Handle the Window Menu
	//
	bool tookIt = false;

	if (aCommandNumber < 0)
	{
		//
		// It must be from a non-MacApp menu so most likely the window menu
		//
		short menu;
		short item;
		CommandToMenuItem(aCommandNumber, menu, item);

		if (menu == fMenuNo && GetWindowList())
			if ((item > fReserveMenuItems) && (item <= fReserveMenuItems + GetWindowList()->GetSize()))
			{
				tookIt = true;
				TWindow * aWindow = static_cast<TWindow*>(GetWindowList()->At((ArrayIndex_AC)(item) - fReserveMenuItems));
				aWindow->Show(true, kRedraw);
				aWindow->Select();
			}

	}
	if (!tookIt)
		Inherited::DoMenuCommand(aCommandNumber);
}


//--------------------------------------------------------------------------------
// TWindowMenuBehavior::DoSetupMenus
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::DoSetupMenus()
{
	Inherited::DoSetupMenus();

	//
	// Handle the Window Menu if it needs updating
	//
	if (IsWindowMenuInvalid())
		UpdateWindowMenu();

	//
	// Enable all the appropriate items
	//
	MenuHandle theMenu = MAGetMenu(fMenuNo);

	if (theMenu)
		for (short i = static_cast<short>(GetWindowList()->GetSize() + fReserveMenuItems); i > fReserveMenuItems; --i)
			::EnableItem(theMenu, i);
}


//--------------------------------------------------------------------------------
// TWindowMenuBehavior::InvalidateWindowMenu
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::InvalidateWindowMenu()
{
	//
	// Set the flag to indicate if the window menu needs updating
	//
	fWindowMenuInvalid = true;

	//
	// Make sure the menu title stays in synch
	//
	::InvalidateMenus();
}

//--------------------------------------------------------------------------------
// TWindowMenuBehavior::ShouldBeIncluded
//--------------------------------------------------------------------------------

bool TWindowMenuBehavior::ShouldBeIncluded(TWindow* theWindow) const
{
	bool returnVal = true;

	if ((theWindow->IsShown() || fIncludeHidden) && !theWindow->GetFloats())
	{
		if (theWindow->GetTitle().IsEmpty())					// skip blank titles
			returnVal = false;
	}
	else
		returnVal = false;

	return returnVal;
}

//--------------------------------------------------------------------------------
// TWindowMenuBehavior::BuildWindowList
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::BuildWindowList()
{
	CSortedList_AC * theList = GetWindowList();

	theList->DeleteAll();

	if (fDocumentSeparators)
	{
		for (CDocumentIterator iter(gDispatcher); iter; ++iter)
		{
			if (iter.CurrentIndex() > 1 && iter->GetWindowList()->GetSize() >= 1)
				theList->Insert(NULL);			//  Placeholder for menu list section separator.

			// Add all the windows for this DSP.
			CWindowIterator winIter(iter);
			for (TWindow * theWindow = winIter; winIter.Current(); theWindow = ++winIter)
				if (theWindow && ShouldBeIncluded(theWindow))
					theList->Insert(theWindow);
		}
	}
	else
	{
		CWMgrIterator iter;
		for (WindowPtr aWindowPtr = iter.FirstWMgrWindow(); iter.Current(); aWindowPtr = iter.NextWMgrWindow())
		{
			TWindow * theWindow = TWindow::WMgrToWindow(aWindowPtr);
			if (theWindow && ShouldBeIncluded(theWindow))
				theList->Insert(theWindow);
		}
	}
}

//--------------------------------------------------------------------------------
// TWindowMenuBehavior UpdateWindowMenu
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::UpdateWindowMenu()
{
	MenuHandle theMenu = GetMenuHandle(fMenuNo);
	CSortedList_AC * theList = GetWindowList();

	if (theMenu && theList)
	{
		//
		// reconstruct the window list. It is composed of the window manager's window list,
		// Filter the list while constructing it
		//
		//
		// clear the menu from the list
		//
		for (short loop = static_cast<short>(GetWindowList()->GetSize() + fReserveMenuItems); loop > fReserveMenuItems; --loop)
			DeleteMenuItem(theMenu, loop);

		BuildWindowList();

		//
		// rebuild the menu from the new list
		//
		for (TemplateIterator_AC<TWindow *> iter2(theList); iter2; ++iter2)
			if (iter2)
			{
				// Put a blank string so as to handle special characters
				AppendMenu(theMenu, CStr2_AC(" "));
				
				short itemNumber = static_cast<short>(iter2.CurrentIndex() + fReserveMenuItems);
				
				::CheckItem(theMenu, itemNumber, iter2->IsActive());

				SetMenuItemText(theMenu, itemNumber, iter2->GetTitle());
			}
			else
				AppendMenu(theMenu, CStr2_AC("-"));		//  Add separator between window groups in menu.

		ValidateWindowMenu();					// We just updated it so it's clean
	}
}


//--------------------------------------------------------------------------------
// TWindowMenuBehavior DoToolboxEvent
//--------------------------------------------------------------------------------

bool TWindowMenuBehavior::DoToolboxEvent(TToolboxEvent* event)
{
	if (event->GetWhat() == activateEvt)
		InvalidateWindowMenu();

	return Inherited::DoToolboxEvent(event);
}


//--------------------------------------------------------------------------------
// TWindowMenuBehavior DoEvent
//--------------------------------------------------------------------------------

void TWindowMenuBehavior::DoEvent(EventNumber eventNumber,
											TEventHandler* source,
											TEvent* event)
{
	if (eventNumber == fInvalidateEvent)
		InvalidateWindowMenu();
	else if ((eventNumber == mBecameTarget || eventNumber == mResignedTarget) && source == fOwner)
		InvalidateWindowMenu();
	else
		Inherited::DoEvent(eventNumber, source, event);
}


// -----------------------------------------------------------------------------------------
//	TWindowBehavior::GetMinMaxSize
// -----------------------------------------------------------------------------------------

void TWindowMenuBehavior::GetMinMaxSize(CLongPoint_AC& minSize,
										   CLongPoint_AC& maxSize,
										   CSortedList_AC* windowList)
{
	//
	// return in minSize: the largest minimum window size
	//		  in maxSize: the smallest maximum window size
	//
	maxSize.Set(kMaxLong, kMaxLong);
	minSize.Clear();
	for (TemplateIterator_AC<TWindow *>iter(windowList); iter; ++iter)
		if (ShouldBeIncluded(iter))
		{
			CLongPoint_AC min(iter->GetResizeLimits()[topLeft]);
			if (min > minSize)
				minSize = min;
	
			CLongPoint_AC max(iter->GetResizeLimits()[botRight]);
			if (max < maxSize)
				maxSize = max;
		}
}
